@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -37,7 +37,6 @@ enum rarch_camera_ctl_state
    RARCH_CAMERA_CTL_UNSET_ACTIVE,
    RARCH_CAMERA_CTL_IS_ACTIVE,
    RARCH_CAMERA_CTL_FIND_DRIVER,
-   RARCH_CAMERA_CTL_POLL,
    RARCH_CAMERA_CTL_SET_CB,
    RARCH_CAMERA_CTL_STOP,
    RARCH_CAMERA_CTL_START,
@@ -105,6 +104,8 @@ void driver_camera_stop(void);
 
 bool driver_camera_start(void);
 
+void camera_driver_poll(void);
+
 bool camera_driver_ctl(enum rarch_camera_ctl_state state, void *data);
 
 RETRO_END_DECLS
