@@ -16,7 +16,6 @@
 #include <compat/strl.h>
 
 #include "../menu_driver.h"
-#include "../menu_navigation.h"
 #include "../menu_cbs.h"
 
 #include "../widgets/menu_dialog.h"
@@ -27,29 +26,37 @@
    cbs->action_info_ident = #name;
 #endif
 
+#ifdef HAVE_NETWORKING
+#include "../../network/netplay/netplay_discovery.h"
+#endif
+
 static int action_info_default(unsigned type, const char *label)
 {
-   size_t selection             = 0;
-   menu_displaylist_info_t info = {0};
+   menu_displaylist_info_t info;
    file_list_t *menu_stack      = menu_entries_get_menu_stack_ptr(0);
+   size_t selection             = menu_navigation_get_selection();
 
-   if (!menu_navigation_ctl(MENU_NAVIGATION_CTL_GET_SELECTION, &selection))
-      return 0;
+   menu_displaylist_info_init(&info);
 
-   info.list          = menu_stack;
-   info.directory_ptr = selection;
-   info.enum_idx      = MENU_ENUM_LABEL_INFO_SCREEN;
-   strlcpy(info.label,
-         msg_hash_to_str(MENU_ENUM_LABEL_INFO_SCREEN),
-        sizeof(info.label));
+   info.list                    = menu_stack;
+   info.directory_ptr           = selection;
+   info.enum_idx                = MENU_ENUM_LABEL_INFO_SCREEN;
+   info.label                   = strdup(
+         msg_hash_to_str(MENU_ENUM_LABEL_INFO_SCREEN));
 
    if (!menu_displaylist_ctl(DISPLAYLIST_HELP, &info))
-      return -1;
+      goto error;
 
-   if (!menu_displaylist_ctl(DISPLAYLIST_PROCESS, &info))
-      return -1;
+   if (!menu_displaylist_process(&info))
+      goto error;
+
+   menu_displaylist_info_free(&info);
 
    return 0;
+
+error:
+   menu_displaylist_info_free(&info);
+   return -1;
 }
 
 #ifdef HAVE_CHEEVOS
@@ -64,8 +71,8 @@ static int action_info_cheevos(unsigned type, const char *label)
    menu_dialog_set_current_id(new_id);
 
    return generic_action_ok_help(NULL, label, new_id, 0, 0,
-         MENU_ENUM_LABEL_CHEEVOS_DESCRIPTION,
-         MENU_DIALOG_HELP_CHEEVOS_DESCRIPTION);
+      MENU_ENUM_LABEL_CHEEVOS_DESCRIPTION,
+      MENU_DIALOG_HELP_CHEEVOS_DESCRIPTION);
 }
 #endif
 
@@ -76,7 +83,8 @@ int menu_cbs_init_bind_info(menu_file_list_cbs_t *cbs,
       return -1;
 
 #ifdef HAVE_CHEEVOS
-   if ((type >= MENU_SETTINGS_CHEEVOS_START))
+   if ((type >= MENU_SETTINGS_CHEEVOS_START) &&
+      (type < MENU_SETTINGS_NETPLAY_ROOMS_START))
    {
       BIND_ACTION_INFO(cbs, action_info_cheevos);
       return 0;
