@@ -24,22 +24,22 @@
 
 #include "../menu_content.h"
 #include "../menu_driver.h"
+#include "../menu_entries.h"
 #include "../menu_cbs.h"
 #include "../menu_input.h"
 #include "../menu_setting.h"
 #include "../menu_shader.h"
-#include "../menu_navigation.h"
-
-#include "../widgets/menu_list.h"
 
+#include "../../audio/audio_driver.h"
 #include "../../configuration.h"
 #include "../../core.h"
 #include "../../core_info.h"
 #include "../../managers/cheat_manager.h"
 #include "../../file_path_special.h"
 #include "../../retroarch.h"
-#include "../../runloop.h"
+#include "../../verbosity.h"
 #include "../../ui/ui_companion_driver.h"
+#include "../../network/netplay/netplay.h"
 
 #ifndef BIND_ACTION_RIGHT
 #define BIND_ACTION_RIGHT(cbs, name) \
@@ -49,46 +49,45 @@
    } while(0)
 #endif
 
-#ifdef HAVE_SHADER_MANAGER
+extern struct key_desc key_descriptors[RARCH_MAX_KEYS];
+
 static int generic_shader_action_parameter_right(struct video_shader_parameter *param,
       unsigned type, const char *label, bool wraparound)
 {
    param->current += param->step;
    param->current  = MIN(MAX(param->minimum, param->current), param->maximum);
 
-   if (ui_companion_is_on_foreground())
-      ui_companion_driver_notify_refresh();
    return 0;
 }
 
 int shader_action_parameter_right(unsigned type, const char *label, bool wraparound)
 {
    video_shader_ctx_t shader_info;
-   struct video_shader_parameter *param = NULL;
+   struct video_shader *shader          = menu_shader_get();
+   struct video_shader_parameter *param_menu = NULL;
+   struct video_shader_parameter *param_prev = NULL;
+
+   int ret = 0;
 
    video_shader_driver_get_current_shader(&shader_info);
 
-   param = &shader_info.data->parameters[type - MENU_SETTINGS_SHADER_PARAMETER_0];
-   if (!param)
-      return menu_cbs_exit();
-   return generic_shader_action_parameter_right(param, type, label, wraparound);
-}
+   param_prev = &shader_info.data->parameters[type - MENU_SETTINGS_SHADER_PARAMETER_0];
+   param_menu = shader ? &shader->parameters[type -
+      MENU_SETTINGS_SHADER_PARAMETER_0] : NULL;
 
-int shader_action_parameter_preset_right(unsigned type, const char *label,
-      bool wraparound)
-{
-   struct video_shader_parameter *param = menu_shader_manager_get_parameters(
-         type - MENU_SETTINGS_SHADER_PRESET_PARAMETER_0);
-   if (!param)
+   if (!param_prev || !param_menu)
       return menu_cbs_exit();
-   return generic_shader_action_parameter_right(param, type, label, wraparound);
+   ret = generic_shader_action_parameter_right(param_prev, type, label, wraparound);
+
+   param_menu->current = param_prev->current;
+
+   return ret;
 }
-#endif
 
 int generic_action_cheat_toggle(size_t idx, unsigned type, const char *label,
       bool wraparound)
 {
-   cheat_manager_toggle_index(idx);
+   cheat_manager_toggle_index((unsigned)idx);
 
    return 0;
 }
@@ -101,24 +100,76 @@ int action_right_cheat(unsigned type, const char *label,
          wraparound);
 }
 
-int action_right_input_desc(unsigned type, const char *label,
+int action_right_input_desc_kbd(unsigned type, const char *label,
       bool wraparound)
 {
-   unsigned inp_desc_index_offset = type - MENU_SETTINGS_INPUT_DESC_BEGIN;
-   unsigned inp_desc_user         = inp_desc_index_offset / (RARCH_FIRST_CUSTOM_BIND + 4);
-   unsigned inp_desc_button_index_offset = inp_desc_index_offset - (inp_desc_user * (RARCH_FIRST_CUSTOM_BIND + 4));
+   unsigned key_id, id, offset;
+   unsigned remap_id;
    settings_t *settings = config_get_ptr();
 
-   if (inp_desc_button_index_offset < RARCH_FIRST_CUSTOM_BIND)
+   if (!settings)
+      return 0;
+
+   offset = type / ((MENU_SETTINGS_INPUT_DESC_KBD_END -
+      (MENU_SETTINGS_INPUT_DESC_KBD_END -
+      MENU_SETTINGS_INPUT_DESC_KBD_BEGIN))) - 1;
+
+   id = (type / (offset + 1)) - MENU_SETTINGS_INPUT_DESC_KBD_BEGIN;
+
+   remap_id =
+      settings->uints.input_keymapper_ids[offset][id];
+
+   for (key_id = 0; key_id < RARCH_MAX_KEYS - 1; key_id++)
    {
-      if (settings->input.remap_ids[inp_desc_user][inp_desc_button_index_offset] < RARCH_FIRST_CUSTOM_BIND - 1)
-         settings->input.remap_ids[inp_desc_user][inp_desc_button_index_offset]++;
+      if(remap_id == key_descriptors[key_id].key)
+         break;
    }
+
+   if (key_id < (RARCH_MAX_KEYS - 1) + MENU_SETTINGS_INPUT_DESC_KBD_BEGIN)
+      key_id++;
    else
-   {
-      if (settings->input.remap_ids[inp_desc_user][inp_desc_button_index_offset] < 4 - 1)
-         settings->input.remap_ids[inp_desc_user][inp_desc_button_index_offset]++;
-   }
+      key_id = 0;
+
+   settings->uints.input_keymapper_ids[offset][id] = key_descriptors[key_id].key;
+
+   return 0;
+}
+
+/* fix-me: incomplete, lacks error checking */
+int action_right_input_desc(unsigned type, const char *label,
+   bool wraparound)
+{
+   rarch_system_info_t *system           = runloop_get_system_info();
+   settings_t *settings                  = config_get_ptr();
+   unsigned btn_idx, user_idx, remap_idx;
+
+   if (!settings || !system)
+      return 0;
+
+   user_idx = (type - MENU_SETTINGS_INPUT_DESC_BEGIN) / (RARCH_FIRST_CUSTOM_BIND + 8);
+   btn_idx  = (type - MENU_SETTINGS_INPUT_DESC_BEGIN) - (RARCH_FIRST_CUSTOM_BIND + 8) * user_idx;
+
+   if (settings->uints.input_remap_ids[user_idx][btn_idx] < RARCH_CUSTOM_BIND_LIST_END - 1)
+      settings->uints.input_remap_ids[user_idx][btn_idx]++;
+   else if (settings->uints.input_remap_ids[user_idx][btn_idx] == RARCH_CUSTOM_BIND_LIST_END - 1)
+      settings->uints.input_remap_ids[user_idx][btn_idx] = RARCH_UNMAPPED;
+   else
+      settings->uints.input_remap_ids[user_idx][btn_idx] = 0;
+
+   remap_idx = settings->uints.input_remap_ids[user_idx][btn_idx];
+
+   /* skip the not used buttons (unless they are at the end by calling the right desc function recursively
+      also skip all the axes until analog remapping is implemented */
+   if ((string_is_empty(system->input_desc_btn[user_idx][remap_idx]) && remap_idx < RARCH_CUSTOM_BIND_LIST_END) /*||
+       (remap_idx >= RARCH_FIRST_CUSTOM_BIND && remap_idx < RARCH_CUSTOM_BIND_LIST_END)*/)
+      action_right_input_desc(type, label, wraparound);
+
+#if 0
+   int i = 0;
+   //RARCH_LOG("[remap-debug] new descriptor for %d: %s\n", remap_idx, system->input_desc_btn[user_idx][remap_idx]);
+   for (i = 0; i < RARCH_ANALOG_BIND_LIST_END; i++)
+      RARCH_LOG("[remap-debug]: user %d button %d new id %d\n", user_idx, i, settings->uints.input_remap_ids[user_idx][i]);
+#endif
 
    return 0;
 }
@@ -126,44 +177,75 @@ int action_right_input_desc(unsigned type, const char *label,
 static int action_right_scroll(unsigned type, const char *label,
       bool wraparound)
 {
-   size_t selection;
    size_t scroll_accel   = 0;
    unsigned scroll_speed = 0, fast_scroll_speed = 0;
-   if (!menu_navigation_ctl(MENU_NAVIGATION_CTL_GET_SELECTION, &selection))
-      return false;
-   if (!menu_navigation_ctl(MENU_NAVIGATION_CTL_GET_SCROLL_ACCEL, &scroll_accel))
+   size_t selection      = menu_navigation_get_selection();
+
+   if (!menu_driver_ctl(MENU_NAVIGATION_CTL_GET_SCROLL_ACCEL, &scroll_accel))
       return false;
 
-   scroll_speed      = (MAX(scroll_accel, 2) - 2) / 4 + 1;
+   scroll_speed      = (unsigned)((MAX(scroll_accel, 2) - 2) / 4 + 1);
    fast_scroll_speed = 4 + 4 * scroll_speed;
 
    if (selection  + fast_scroll_speed < (menu_entries_get_size()))
    {
       size_t idx  = selection + fast_scroll_speed;
-      bool scroll = true;
-      menu_navigation_ctl(MENU_NAVIGATION_CTL_SET_SELECTION, &idx);
-      menu_navigation_ctl(MENU_NAVIGATION_CTL_SET, &scroll);
+
+      menu_navigation_set_selection(idx);
+      menu_driver_navigation_set(true);
    }
    else
    {
       if ((menu_entries_get_size() > 0))
-         menu_navigation_ctl(MENU_NAVIGATION_CTL_SET_LAST, NULL);
+         menu_driver_ctl(MENU_NAVIGATION_CTL_SET_LAST, NULL);
    }
 
    return 0;
 }
 
+static int audio_mixer_stream_volume_right(unsigned type, const char *label,
+      bool wraparound)
+{
+   unsigned         offset      = (type - MENU_SETTINGS_AUDIO_MIXER_STREAM_ACTIONS_VOLUME_BEGIN);
+   float orig_volume            = 0.0f;
+   
+   if (offset >= AUDIO_MIXER_MAX_STREAMS)
+      return 0;
+
+   orig_volume                  = audio_driver_mixer_get_stream_volume(offset);
+   orig_volume                  = orig_volume + 1.00f;
+
+   audio_driver_mixer_set_stream_volume(offset, orig_volume);
+
+   return 0;
+}
+
+
+static int action_right_goto_tab(void)
+{
+   menu_ctx_list_t list_info;
+   file_list_t *selection_buf = menu_entries_get_selection_buf_ptr(0);
+   file_list_t *menu_stack    = menu_entries_get_menu_stack_ptr(0);
+   size_t selection           = menu_navigation_get_selection();
+   menu_file_list_cbs_t *cbs  = selection_buf ? (menu_file_list_cbs_t*)
+      file_list_get_actiondata_at_offset(selection_buf, selection) : NULL;
+
+   list_info.type             = MENU_LIST_HORIZONTAL;
+   list_info.action           = MENU_ACTION_RIGHT;
+
+   menu_driver_ctl(RARCH_MENU_CTL_LIST_CACHE, &list_info);
+
+   if (cbs && cbs->action_content_list_switch)
+      return cbs->action_content_list_switch(selection_buf, menu_stack,
+            "", "", 0);
+
+   return 0;
+}
+
 static int action_right_mainmenu(unsigned type, const char *label,
       bool wraparound)
 {
    menu_ctx_list_t list_info;
-   size_t selection          = 0;
-   menu_file_list_cbs_t *cbs = NULL;
-   unsigned        push_list = 0;
-   file_list_t *selection_buf = menu_entries_get_selection_buf_ptr(0);
-   file_list_t *menu_stack   = menu_entries_get_menu_stack_ptr(0);
-   settings_t      *settings = config_get_ptr();
-   unsigned           action = MENU_ACTION_RIGHT;
 
    menu_driver_ctl(RARCH_MENU_CTL_LIST_GET_SELECTION, &list_info);
 
@@ -175,49 +257,20 @@ static int action_right_mainmenu(unsigned type, const char *label,
    {
       menu_ctx_list_t list_horiz_info;
       menu_ctx_list_t list_tabs_info;
+      settings_t      *settings = config_get_ptr();
 
-      list_horiz_info.type = MENU_LIST_HORIZONTAL;
-      list_tabs_info.type  = MENU_LIST_TABS;
+      list_horiz_info.type      = MENU_LIST_HORIZONTAL;
+      list_tabs_info.type       = MENU_LIST_TABS;
 
       menu_driver_ctl(RARCH_MENU_CTL_LIST_GET_SIZE, &list_horiz_info);
       menu_driver_ctl(RARCH_MENU_CTL_LIST_GET_SIZE, &list_tabs_info);
 
-      menu_navigation_ctl(MENU_NAVIGATION_CTL_SET_SELECTION, &selection);
-
       if ((list_info.selection != (list_horiz_info.size + list_tabs_info.size))
-         || settings->menu.navigation.wraparound.enable)
-         push_list = 1;
+         || settings->bools.menu_navigation_wraparound_enable)
+         return action_right_goto_tab();
    }
    else
-      push_list = 2;
-
-   menu_navigation_ctl(MENU_NAVIGATION_CTL_GET_SELECTION, &selection);
-
-   cbs = menu_entries_get_actiondata_at_offset(selection_buf, selection);
-
-   switch (push_list)
-   {
-      case 1:
-         {
-            menu_ctx_list_t list_info;
-
-            list_info.type   = MENU_LIST_HORIZONTAL;
-            list_info.action = action;
-
-            menu_driver_ctl(RARCH_MENU_CTL_LIST_CACHE, &list_info);
-
-            if (cbs && cbs->action_content_list_switch)
-               return cbs->action_content_list_switch(selection_buf, menu_stack,
-                     "", "", 0);
-         }
-         break;
-      case 2:
-         action_right_scroll(0, "", false);
-         break;
-      case 0:
-      default:
-         break;
-   }
+      action_right_scroll(0, "", false);
 
    return 0;
 }
@@ -225,11 +278,11 @@ static int action_right_mainmenu(unsigned type, const char *label,
 static int action_right_shader_scale_pass(unsigned type, const char *label,
       bool wraparound)
 {
-#ifdef HAVE_SHADER_MANAGER
    unsigned current_scale, delta;
-   unsigned pass                         = 
+   unsigned pass                         =
       type - MENU_SETTINGS_SHADER_PASS_SCALE_0;
-   struct video_shader_pass *shader_pass = menu_shader_manager_get_pass(pass);
+   struct video_shader *shader           = menu_shader_get();
+   struct video_shader_pass *shader_pass = shader ? &shader->pass[pass] : NULL;
 
    if (!shader_pass)
       return menu_cbs_exit();
@@ -240,39 +293,33 @@ static int action_right_shader_scale_pass(unsigned type, const char *label,
 
    shader_pass->fbo.valid   = current_scale;
    shader_pass->fbo.scale_x = shader_pass->fbo.scale_y = current_scale;
-#endif
    return 0;
 }
 
 static int action_right_shader_filter_pass(unsigned type, const char *label,
       bool wraparound)
 {
-#ifdef HAVE_SHADER_MANAGER
    unsigned pass                         = type - MENU_SETTINGS_SHADER_PASS_FILTER_0;
    unsigned delta                        = 1;
-   struct video_shader_pass *shader_pass = menu_shader_manager_get_pass(pass);
+   struct video_shader *shader           = menu_shader_get();
+   struct video_shader_pass *shader_pass = shader ? &shader->pass[pass] : NULL;
 
    if (!shader_pass)
       return menu_cbs_exit();
 
    shader_pass->filter = ((shader_pass->filter + delta) % 3);
-#endif
    return 0;
 }
 
 static int action_right_shader_filter_default(unsigned type, const char *label,
       bool wraparound)
 {
-#ifdef HAVE_SHADER_MANAGER
    rarch_setting_t *setting = menu_setting_find_enum(MENU_ENUM_LABEL_VIDEO_SMOOTH);
    if (!setting)
       return menu_cbs_exit();
    return menu_action_handle_setting(setting,
          setting_get_type(setting), MENU_ACTION_RIGHT,
          wraparound);
-#else
-   return 0;
-#endif
 }
 
 static int action_right_cheat_num_passes(unsigned type, const char *label,
@@ -292,21 +339,66 @@ static int action_right_cheat_num_passes(unsigned type, const char *label,
 static int action_right_shader_num_passes(unsigned type, const char *label,
       bool wraparound)
 {
-#ifdef HAVE_SHADER_MANAGER
    bool refresh                = false;
    struct video_shader *shader = menu_shader_get();
+   unsigned pass_count         = shader ? shader->passes : 0;
 
    if (!shader)
       return menu_cbs_exit();
 
-   if ((menu_shader_manager_get_amount_passes() < GFX_MAX_SHADERS))
+   if (pass_count < GFX_MAX_SHADERS)
       menu_shader_manager_increment_amount_passes();
 
    menu_entries_ctl(MENU_ENTRIES_CTL_SET_REFRESH, &refresh);
    menu_driver_ctl(RARCH_MENU_CTL_SET_PREVENT_POPULATE, NULL);
    video_shader_resolve_parameters(NULL, shader);
 
-#endif
+   return 0;
+}
+
+static int action_right_netplay_mitm_server(unsigned type, const char *label,
+      bool wraparound)
+{
+   settings_t *settings = config_get_ptr();
+   unsigned i;
+   bool found = false;
+   unsigned list_len = ARRAY_SIZE(netplay_mitm_server_list);
+
+   for (i = 0; i < list_len; i++)
+   {
+      /* find the currently selected server in the list */
+      if (string_is_equal(settings->arrays.netplay_mitm_server, netplay_mitm_server_list[i].name))
+      {
+         /* move to the next one in the list, wrap around if necessary */
+         if (i + 1 < list_len)
+         {
+            found = true;
+            strlcpy(settings->arrays.netplay_mitm_server, netplay_mitm_server_list[i + 1].name, sizeof(settings->arrays.netplay_mitm_server));
+            break;
+         }
+         else if (wraparound)
+         {
+            found = true;
+            strlcpy(settings->arrays.netplay_mitm_server, netplay_mitm_server_list[0].name, sizeof(settings->arrays.netplay_mitm_server));
+            break;
+         }
+      }
+   }
+
+   if (!found)
+   {
+      /* current entry was invalid, go back to the start */
+      strlcpy(settings->arrays.netplay_mitm_server, netplay_mitm_server_list[0].name, sizeof(settings->arrays.netplay_mitm_server));
+   }
+
+   return 0;
+}
+
+static int action_right_shader_watch_for_changes(unsigned type, const char *label,
+      bool wraparound)
+{
+   settings_t *settings = config_get_ptr();
+   settings->bools.video_shader_watch_files = !settings->bools.video_shader_watch_files;
    return 0;
 }
 
@@ -329,15 +421,15 @@ static int playlist_association_right(unsigned type, const char *label,
    core_info_list_t           *list = NULL;
    settings_t *settings             = config_get_ptr();
    const char *path                 = path_basename(label);
-   
+
    core_info_get_list(&list);
    if (!list)
       return menu_cbs_exit();
 
    core_path[0] = new_playlist_cores[0] = '\0';
 
-   stnames = string_split(settings->playlist_names, ";");
-   stcores = string_split(settings->playlist_cores, ";");
+   stnames = string_split(settings->arrays.playlist_names, ";");
+   stcores = string_split(settings->arrays.playlist_cores, ";");
 
    if (!menu_content_playlist_find_associated_core(path, core_path, sizeof(core_path)))
          strlcpy(core_path,
@@ -354,20 +446,22 @@ static int playlist_association_right(unsigned type, const char *label,
    if (next >= list->count)
    {
       if (wraparound)
-         next = list->count-1;
-      else
          next = 0;
+      else
+         next = list->count-1;
    }
 
    info = core_info_get(list, next);
 
    found = string_list_find_elem(stnames, path);
-   if (found)
-      string_list_set(stcores, found-1, info->path);
+   if (found && info)
+      string_list_set(stcores, (unsigned)(found-1), info->path);
 
-   string_list_join_concat(new_playlist_cores, sizeof(new_playlist_cores), stcores, ";");
+   string_list_join_concat(new_playlist_cores,
+         sizeof(new_playlist_cores), stcores, ";");
 
-   strlcpy(settings->playlist_cores, new_playlist_cores, sizeof(settings->playlist_cores));
+   strlcpy(settings->arrays.playlist_cores,
+         new_playlist_cores, sizeof(settings->arrays.playlist_cores));
 
    string_list_free(stnames);
    string_list_free(stcores);
@@ -379,7 +473,7 @@ int core_setting_right(unsigned type, const char *label,
 {
    unsigned idx     = type - MENU_SETTINGS_CORE_OPTION_START;
 
-   runloop_ctl(RUNLOOP_CTL_CORE_OPTION_NEXT, &idx);
+   rarch_ctl(RARCH_CTL_CORE_OPTION_NEXT, &idx);
 
    return 0;
 }
@@ -406,7 +500,11 @@ static int menu_cbs_init_bind_right_compare_type(menu_file_list_cbs_t *cbs,
    {
       BIND_ACTION_RIGHT(cbs, action_right_cheat);
    }
-#ifdef HAVE_SHADER_MANAGER
+   else if (type >= MENU_SETTINGS_AUDIO_MIXER_STREAM_ACTIONS_VOLUME_BEGIN
+         && type <= MENU_SETTINGS_AUDIO_MIXER_STREAM_ACTIONS_VOLUME_END)
+   {
+      BIND_ACTION_RIGHT(cbs, audio_mixer_stream_volume_right);
+   }
    else if (type >= MENU_SETTINGS_SHADER_PARAMETER_0
          && type <= MENU_SETTINGS_SHADER_PARAMETER_LAST)
    {
@@ -415,14 +513,18 @@ static int menu_cbs_init_bind_right_compare_type(menu_file_list_cbs_t *cbs,
    else if (type >= MENU_SETTINGS_SHADER_PRESET_PARAMETER_0
          && type <= MENU_SETTINGS_SHADER_PRESET_PARAMETER_LAST)
    {
-      BIND_ACTION_RIGHT(cbs, shader_action_parameter_preset_right);
+      BIND_ACTION_RIGHT(cbs, shader_action_parameter_right);
    }
-#endif
    else if (type >= MENU_SETTINGS_INPUT_DESC_BEGIN
          && type <= MENU_SETTINGS_INPUT_DESC_END)
    {
       BIND_ACTION_RIGHT(cbs, action_right_input_desc);
    }
+   else if (type >= MENU_SETTINGS_INPUT_DESC_KBD_BEGIN
+      && type <= MENU_SETTINGS_INPUT_DESC_KBD_END)
+   {
+      BIND_ACTION_RIGHT(cbs, action_right_input_desc_kbd);
+   }
    else if ((type >= MENU_SETTINGS_PLAYLIST_ASSOCIATION_START))
    {
       BIND_ACTION_RIGHT(cbs, playlist_association_right);
@@ -469,9 +571,11 @@ static int menu_cbs_init_bind_right_compare_type(menu_file_list_cbs_t *cbs,
          case FILE_TYPE_DOWNLOAD_THUMBNAIL_CONTENT:
          case FILE_TYPE_DOWNLOAD_URL:
          case FILE_TYPE_SCAN_DIRECTORY:
+         case FILE_TYPE_FONT:
          case MENU_SETTING_GROUP:
          case MENU_SETTINGS_CORE_INFO_NONE:
             if (  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HISTORY_TAB))   ||
+                  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_FAVORITES_TAB)) ||
                   string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_PLAYLISTS_TAB)) ||
                   string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_ADD_TAB)) ||
                   string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_NETPLAY_TAB)) ||
@@ -502,13 +606,12 @@ static int menu_cbs_init_bind_right_compare_type(menu_file_list_cbs_t *cbs,
 static int menu_cbs_init_bind_right_compare_label(menu_file_list_cbs_t *cbs,
       const char *label, uint32_t label_hash, const char *menu_label)
 {
-   unsigned i;
 
    if (cbs->setting)
    {
       const char *parent_group   = cbs->setting->parent_group;
 
-      if (string_is_equal(parent_group, msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU)) 
+      if (string_is_equal(parent_group, msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU))
                && (setting_get_type(cbs->setting) == ST_GROUP))
       {
          BIND_ACTION_RIGHT(cbs, action_right_scroll);
@@ -516,21 +619,25 @@ static int menu_cbs_init_bind_right_compare_label(menu_file_list_cbs_t *cbs,
       }
    }
 
-   for (i = 0; i < MAX_USERS; i++)
+   if (strstr(label, "input_player") && strstr(label, "_joypad_index"))
    {
-      uint32_t label_setting_hash;
-      char label_setting[128];
+      unsigned i;
+      for (i = 0; i < MAX_USERS; i++)
+      {
+         uint32_t label_setting_hash;
+         char label_setting[128];
 
-      label_setting[0] = '\0';
+         label_setting[0] = '\0';
 
-      snprintf(label_setting, sizeof(label_setting), "input_player%d_joypad_index", i + 1);
-      label_setting_hash = msg_hash_calculate(label_setting);
+         snprintf(label_setting, sizeof(label_setting), "input_player%d_joypad_index", i + 1);
+         label_setting_hash = msg_hash_calculate(label_setting);
 
-      if (label_hash != label_setting_hash)
-         continue;
+         if (label_hash != label_setting_hash)
+            continue;
 
-      BIND_ACTION_RIGHT(cbs, bind_right_generic);
-      return 0;
+         BIND_ACTION_RIGHT(cbs, bind_right_generic);
+         return 0;
+      }
    }
 
    if (string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_PLAYLISTS_TAB)))
@@ -561,6 +668,12 @@ static int menu_cbs_init_bind_right_compare_label(menu_file_list_cbs_t *cbs,
             case MENU_ENUM_LABEL_VIDEO_SHADER_DEFAULT_FILTER:
                BIND_ACTION_RIGHT(cbs, action_right_shader_filter_default);
                break;
+            case MENU_ENUM_LABEL_NETPLAY_MITM_SERVER:
+               BIND_ACTION_RIGHT(cbs, action_right_netplay_mitm_server);
+               break;
+            case MENU_ENUM_LABEL_SHADER_WATCH_FOR_CHANGES:
+               BIND_ACTION_RIGHT(cbs, action_right_shader_watch_for_changes);
+               break;
             case MENU_ENUM_LABEL_VIDEO_SHADER_NUM_PASSES:
                BIND_ACTION_RIGHT(cbs, action_right_shader_num_passes);
                break;
@@ -576,8 +689,9 @@ static int menu_cbs_init_bind_right_compare_label(menu_file_list_cbs_t *cbs,
                break;
             case MENU_ENUM_LABEL_NO_ITEMS:
             case MENU_ENUM_LABEL_NO_PLAYLIST_ENTRIES_AVAILABLE:
-               if (  
+               if (
                      string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HISTORY_TAB))   ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_FAVORITES_TAB)) ||
                      string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MAIN_MENU))       ||
                      string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_PLAYLISTS_TAB))   ||
                      string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_MUSIC_TAB)) ||
@@ -596,6 +710,7 @@ static int menu_cbs_init_bind_right_compare_label(menu_file_list_cbs_t *cbs,
             case MENU_ENUM_LABEL_START_VIDEO_PROCESSOR:
             case MENU_ENUM_LABEL_TAKE_SCREENSHOT:
                if (  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HISTORY_TAB))   ||
+                     string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_FAVORITES_TAB)) ||
                      string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_PLAYLISTS_TAB)) ||
                      string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_ADD_TAB)) ||
                      string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_NETPLAY_TAB)) ||
@@ -631,10 +746,11 @@ int menu_cbs_init_bind_right(menu_file_list_cbs_t *cbs,
       return menu_cbs_exit();
 
    BIND_ACTION_RIGHT(cbs, bind_right_generic);
-    
+
    if (type == MENU_SETTING_NO_ITEM)
    {
       if (  string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_HISTORY_TAB))   ||
+            string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_FAVORITES_TAB)) ||
             string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_PLAYLISTS_TAB)) ||
             string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_ADD_TAB)) ||
             string_is_equal(menu_label, msg_hash_to_str(MENU_ENUM_LABEL_NETPLAY_TAB)) ||
