@@ -16,6 +16,7 @@
 #include <compat/strl.h>
 #include <file/file_path.h>
 #include <lists/string_list.h>
+#include <string/stdstring.h>
 
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
@@ -24,38 +25,46 @@
 #include "../menu_driver.h"
 #include "../menu_cbs.h"
 #include "../menu_input.h"
-#include "../menu_display.h"
 #include "../menu_setting.h"
 #include "../menu_shader.h"
 
+#include "../../audio/audio_driver.h"
 #include "../../configuration.h"
 #include "../../core.h"
 #include "../../core_info.h"
 #include "../../managers/core_option_manager.h"
 #include "../../managers/cheat_manager.h"
 #include "../../retroarch.h"
-#include "../../runloop.h"
 #include "../../performance_counters.h"
 
-#include "../../gfx/video_shader_driver.h"
+#include "../../gfx/video_driver.h"
 
+#include "../../input/input_driver.h"
 #include "../../input/input_remapping.h"
 
+#include "../../config.def.h"
+
 #ifndef BIND_ACTION_START
 #define BIND_ACTION_START(cbs, name) \
    cbs->action_start = name; \
    cbs->action_start_ident = #name;
 #endif
 
+static int action_start_audio_mixer_stream_volume(unsigned type, const char *label)
+{
+   unsigned         offset      = (type - MENU_SETTINGS_AUDIO_MIXER_STREAM_ACTIONS_VOLUME_BEGIN);
+
+   if (offset >= AUDIO_MIXER_MAX_STREAMS)
+      return 0;
+
+   audio_driver_mixer_set_stream_volume(offset, 1.0f);
+
+   return 0;
+}
+
 static int action_start_remap_file_load(unsigned type, const char *label)
 {
-   global_t *global = global_get_ptr();
-
-   if (!global)
-      return -1;
-
-   global->name.remapfile[0] = '\0';
-   input_remapping_set_defaults();
+   input_remapping_set_defaults(true);
    return 0;
 }
 
@@ -66,7 +75,7 @@ static int action_start_video_filter_file_load(unsigned type, const char *label)
    if (!settings)
       return -1;
 
-   settings->path.softfilter_plugin[0] = '\0';
+   settings->paths.path_softfilter_plugin[0] = '\0';
    command_event(CMD_EVENT_REINIT, NULL);
    return 0;
 }
@@ -109,20 +118,24 @@ static int action_start_input_desc(unsigned type, const char *label)
    (void)label;
 
    if (inp_desc_button_index_offset < RARCH_FIRST_CUSTOM_BIND)
-      settings->input.remap_ids[inp_desc_user][inp_desc_button_index_offset] =
-         settings->input.binds[inp_desc_user][inp_desc_button_index_offset].id;
+   {
+      const struct retro_keybind *keyptr = &input_config_binds[inp_desc_user]
+            [inp_desc_button_index_offset];
+      settings->uints.input_remap_ids[inp_desc_user][inp_desc_button_index_offset] = keyptr->id;
+   }
    else
-      settings->input.remap_ids[inp_desc_user][inp_desc_button_index_offset] =
+      settings->uints.input_remap_ids[inp_desc_user][inp_desc_button_index_offset] =
          inp_desc_button_index_offset - RARCH_FIRST_CUSTOM_BIND;
 
    return 0;
 }
 
-static int action_start_shader_action_parameter(unsigned type, const char *label)
+static int action_start_shader_action_parameter(
+      unsigned type, const char *label)
 {
-#ifdef HAVE_SHADER_MANAGER
    video_shader_ctx_t shader_info;
    struct video_shader_parameter *param = NULL;
+   unsigned parameter = type - MENU_SETTINGS_SHADER_PARAMETER_0;
 
    video_shader_driver_get_current_shader(&shader_info);
 
@@ -130,26 +143,23 @@ static int action_start_shader_action_parameter(unsigned type, const char *label
       return 0;
 
    param          = &shader_info.data->parameters
-      [type - MENU_SETTINGS_SHADER_PARAMETER_0];
+      [parameter];
    param->current = param->initial;
    param->current = MIN(MAX(param->minimum, param->current), param->maximum);
 
-#endif
-
-   return 0;
-}
-
-static int action_start_shader_action_preset_parameter(unsigned type, const char *label)
-{
-   unsigned parameter = type - MENU_SETTINGS_SHADER_PRESET_PARAMETER_0;
    return menu_shader_manager_clear_parameter(parameter);
 }
 
 static int action_start_shader_pass(unsigned type, const char *label)
 {
-   hack_shader_pass                      = type - MENU_SETTINGS_SHADER_PASS_0;
+   menu_handle_t *menu       = NULL;
 
-   menu_shader_manager_clear_pass_path(hack_shader_pass);
+   if (!menu_driver_ctl(RARCH_MENU_CTL_DRIVER_DATA_GET, &menu))
+      return menu_cbs_exit();
+
+   menu->scratchpad.unsigned_var = type - MENU_SETTINGS_SHADER_PASS_0;
+
+   menu_shader_manager_clear_pass_path(menu->scratchpad.unsigned_var);
 
    return 0;
 }
@@ -170,6 +180,20 @@ static int action_start_shader_filter_pass(unsigned type, const char *label)
    return menu_shader_manager_clear_pass_filter(pass);
 }
 
+static int action_start_netplay_mitm_server(unsigned type, const char *label)
+{
+   settings_t *settings = config_get_ptr();
+   strlcpy(settings->arrays.netplay_mitm_server, netplay_mitm_server, sizeof(settings->arrays.netplay_mitm_server));
+   return 0;
+}
+
+static int action_start_shader_watch_for_changes(unsigned type, const char *label)
+{
+   settings_t *settings = config_get_ptr();
+   settings->bools.video_shader_watch_files = video_shader_watch_files;
+   return 0;
+}
+
 static int action_start_shader_num_passes(unsigned type, const char *label)
 {
    return menu_shader_manager_clear_num_passes();
@@ -192,8 +216,8 @@ static int action_start_core_setting(unsigned type,
 {
    unsigned idx                = type - MENU_SETTINGS_CORE_OPTION_START;
    core_option_manager_t *coreopts = NULL;
-   
-   if (runloop_ctl(RUNLOOP_CTL_CORE_OPTIONS_LIST_GET, &coreopts))
+
+   if (rarch_ctl(RARCH_CTL_CORE_OPTIONS_LIST_GET, &coreopts))
       core_option_manager_set_default(coreopts, idx);
 
    return 0;
@@ -215,8 +239,8 @@ static int action_start_playlist_association(unsigned type, const char *label)
 
    new_playlist_cores[0] = '\0';
 
-   stnames = string_split(settings->playlist_names, ";");
-   stcores = string_split(settings->playlist_cores, ";");
+   stnames = string_split(settings->arrays.playlist_names, ";");
+   stcores = string_split(settings->arrays.playlist_cores, ";");
    found   = string_list_find_elem(stnames, path);
 
    if (found)
@@ -226,8 +250,8 @@ static int action_start_playlist_association(unsigned type, const char *label)
    string_list_join_concat(new_playlist_cores,
          sizeof(new_playlist_cores), stcores, ";");
 
-   strlcpy(settings->playlist_cores,
-         new_playlist_cores, sizeof(settings->playlist_cores));
+   strlcpy(settings->arrays.playlist_cores,
+         new_playlist_cores, sizeof(settings->arrays.playlist_cores));
 
    string_list_free(stcores);
    string_list_free(stnames);
@@ -283,6 +307,9 @@ static int menu_cbs_init_bind_start_compare_label(menu_file_list_cbs_t *cbs)
          case MENU_ENUM_LABEL_VIDEO_SHADER_FILTER_PASS:
             BIND_ACTION_START(cbs, action_start_shader_filter_pass);
             break;
+         case MENU_ENUM_LABEL_SHADER_WATCH_FOR_CHANGES:
+            BIND_ACTION_START(cbs, action_start_shader_watch_for_changes);
+            break;
          case MENU_ENUM_LABEL_VIDEO_SHADER_NUM_PASSES:
             BIND_ACTION_START(cbs, action_start_shader_num_passes);
             break;
@@ -291,6 +318,10 @@ static int menu_cbs_init_bind_start_compare_label(menu_file_list_cbs_t *cbs)
             break;
          case MENU_ENUM_LABEL_SCREEN_RESOLUTION:
             BIND_ACTION_START(cbs, action_start_video_resolution);
+            break;
+         case MENU_ENUM_LABEL_NETPLAY_MITM_SERVER:
+            BIND_ACTION_START(cbs, action_start_netplay_mitm_server);
+            break;
          default:
             return -1;
       }
@@ -310,7 +341,7 @@ static int menu_cbs_init_bind_start_compare_type(menu_file_list_cbs_t *cbs,
    else if (type >= MENU_SETTINGS_SHADER_PRESET_PARAMETER_0
          && type <= MENU_SETTINGS_SHADER_PRESET_PARAMETER_LAST)
    {
-      BIND_ACTION_START(cbs, action_start_shader_action_preset_parameter);
+      BIND_ACTION_START(cbs, action_start_shader_action_parameter);
    }
    else if (type >= MENU_SETTINGS_LIBRETRO_PERF_COUNTERS_BEGIN &&
          type <= MENU_SETTINGS_LIBRETRO_PERF_COUNTERS_END)
@@ -351,8 +382,15 @@ int menu_cbs_init_bind_start(menu_file_list_cbs_t *cbs,
    if (!cbs)
       return -1;
 
+   if (type >= MENU_SETTINGS_AUDIO_MIXER_STREAM_ACTIONS_VOLUME_BEGIN
+         && type <= MENU_SETTINGS_AUDIO_MIXER_STREAM_ACTIONS_VOLUME_END)
+   {
+      BIND_ACTION_START(cbs, action_start_audio_mixer_stream_volume);
+      return 0;
+   }
+
    BIND_ACTION_START(cbs, action_start_lookup_setting);
-   
+
    if (menu_cbs_init_bind_start_compare_label(cbs) == 0)
       return 0;
 
