@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -44,11 +44,11 @@ void cheat_manager_set_code(unsigned index, const char *str);
 
 void cheat_manager_free(void);
 
-void cheat_manager_index_next(cheat_manager_t *handle);
+void cheat_manager_index_next(void);
 
-void cheat_manager_index_prev(cheat_manager_t *handle);
+void cheat_manager_index_prev(void);
 
-void cheat_manager_toggle(cheat_manager_t *handle);
+void cheat_manager_toggle(void);
 
 void cheat_manager_apply_cheats(void);
 
@@ -64,11 +64,6 @@ const char *cheat_manager_get_code(unsigned i);
 
 bool cheat_manager_get_code_state(unsigned i);
 
-void cheat_manager_state_checks(
-      bool cheat_index_plus_pressed,
-      bool cheat_index_minus_pressed,
-      bool cheat_toggle_pressed);
-
 void cheat_manager_state_free(void);
 
 bool cheat_manager_alloc_if_empty(void);
