@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (c) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -168,6 +168,18 @@ void drm_setup(int fd)
       RARCH_WARN("[DRM]: Cannot find original CRTC.\n");
 }
 
+float drm_get_refresh_rate(void *data)
+{
+   float refresh_rate = 0.0f;
+
+   if (g_drm_mode)
+   {
+      refresh_rate = g_drm_mode->clock * 1000.0f / g_drm_mode->htotal / g_drm_mode->vtotal;
+   }
+
+   return refresh_rate;
+}
+
 void drm_free(void)
 {
    if (g_drm_encoder)
