@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (c) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -55,6 +55,8 @@ void drm_free(void);
 
 bool drm_get_connector(int fd, video_frame_info_t *video_info);
 
+float drm_get_refresh_rate(void *data);
+
 static INLINE bool drm_wait_flip(int timeout)
 {
    g_drm_fds.revents = 0;
