@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -14,46 +14,13 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <gfx/math/matrix_4x4.h>
 #include <gfx/gl_capabilities.h>
 
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
 #endif
 
-#include "../drivers/gl_symlinks.h"
-#include "../video_coord_array.h"
-
-void gl_ff_vertex(const struct video_coords *coords)
-{
-#ifndef NO_GL_FF_VERTEX
-   /* Fall back to fixed function-style if needed and possible. */
-   glClientActiveTexture(GL_TEXTURE1);
-   glTexCoordPointer(2, GL_FLOAT, 0, coords->lut_tex_coord);
-   glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-   glClientActiveTexture(GL_TEXTURE0);
-   glVertexPointer(2, GL_FLOAT, 0, coords->vertex);
-   glEnableClientState(GL_VERTEX_ARRAY);
-   glColorPointer(4, GL_FLOAT, 0, coords->color);
-   glEnableClientState(GL_COLOR_ARRAY);
-   glTexCoordPointer(2, GL_FLOAT, 0, coords->tex_coord);
-   glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-#endif
-}
-
-void gl_ff_matrix(const math_matrix_4x4 *mat)
-{
-#ifndef NO_GL_FF_MATRIX
-   math_matrix_4x4 ident;
-
-   /* Fall back to fixed function-style if needed and possible. */
-   glMatrixMode(GL_PROJECTION);
-   glLoadMatrixf(mat->data);
-   glMatrixMode(GL_MODELVIEW);
-   matrix_4x4_identity(&ident);
-   glLoadMatrixf(ident.data);
-#endif
-}
+#include "gl_common.h"
 
 static void gl_size_format(GLint* internalFormat)
 {
