@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -21,23 +21,42 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
+#include <errno.h>
+#include <unistd.h>
+
 #include <X11/Xatom.h>
 
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
 #endif
 
+#ifdef HAVE_XINERAMA
+#include <X11/extensions/Xinerama.h>
+#endif
+
 #include "x11_common.h"
+
+#include <X11/extensions/xf86vmode.h>
+
+#include <encodings/utf.h>
+
+#ifdef HAVE_DBUS
+#include "dbus_common.h"
+#endif
+
 #include "../../frontend/frontend_driver.h"
+#include "../../input/input_driver.h"
+#include "../../input/input_keymaps.h"
 #include "../../input/common/input_x11_common.h"
 #include "../../verbosity.h"
 
-#ifdef HAVE_DBUS
-#include <dbus/dbus.h>
-static DBusConnection* dbus_connection      = NULL;
-static unsigned int dbus_screensaver_cookie = 0;
-#endif
+#define _NET_WM_STATE_ADD                    1
+#define MOVERESIZE_GRAVITY_CENTER            5
+#define MOVERESIZE_X_SHIFT                   8
+#define MOVERESIZE_Y_SHIFT                   9
 
+
+static XF86VidModeModeInfo desktop_mode;
 static bool xdg_screensaver_available       = true;
 bool g_x11_entered                          = false;
 static bool g_x11_has_focus                 = false;
@@ -47,7 +66,7 @@ Display *g_x11_dpy                          = NULL;
 unsigned g_x11_screen                       = 0;
 
 Colormap g_x11_cmap;
-Window   g_x11_win;
+Window   g_x11_win = None;
 
 static Atom XA_NET_WM_STATE;
 static Atom XA_NET_WM_STATE_FULLSCREEN;
@@ -57,139 +76,6 @@ static Atom g_x11_quit_atom;
 static XIM g_x11_xim;
 static XIC g_x11_xic;
 
-#define XA_INIT(x) XA##x = XInternAtom(dpy, #x, False)
-#define _NET_WM_STATE_ADD 1
-#define MOVERESIZE_GRAVITY_CENTER 5
-#define MOVERESIZE_X_SHIFT 8
-#define MOVERESIZE_Y_SHIFT 9
-
-#ifdef HAVE_DBUS
-static void dbus_ensure_connection(void)
-{
-    DBusError err;
-    int ret;
-    
-    dbus_error_init(&err);
-
-    dbus_connection = dbus_bus_get_private(DBUS_BUS_SESSION, &err);
-
-    if (dbus_error_is_set(&err))
-    {
-        RARCH_LOG("[DBus]: Failed to get DBus connection. Screensaver will not be suspended via DBus.\n");
-        dbus_error_free(&err);
-    }
-
-    if (dbus_connection)
-        dbus_connection_set_exit_on_disconnect(dbus_connection, true);
-}
-
-static void dbus_close_connection(void)
-{
-   if (!dbus_connection)
-      return;
-
-   dbus_connection_close(dbus_connection);
-   dbus_connection_unref(dbus_connection);
-   dbus_connection = NULL;
-}
-
-static bool dbus_screensaver_inhibit(void)
-{
-   const char *app    = "RetroArch";
-   const char *reason = "Playing a game";
-   DBusMessage   *msg = NULL;
-   DBusMessage *reply = NULL;
-   bool ret           = false;
-
-   if (!dbus_connection)
-      return false; /* DBus connection was not obtained */
-
-   if (dbus_screensaver_cookie > 0)
-      return true; /* Already inhibited */
-
-   msg = dbus_message_new_method_call("org.freedesktop.ScreenSaver",
-         "/org/freedesktop/ScreenSaver",
-         "org.freedesktop.ScreenSaver",
-         "Inhibit");
-
-   if (!msg)
-      return false;
-
-   if (!dbus_message_append_args(msg,
-            DBUS_TYPE_STRING, &app,
-            DBUS_TYPE_STRING, &reason,
-            DBUS_TYPE_INVALID))
-   {
-      dbus_message_unref(msg);
-      return false;
-   }
-
-   reply = dbus_connection_send_with_reply_and_block(dbus_connection,
-         msg, 300, NULL);
-
-   if (reply != NULL)
-   {
-      if (!dbus_message_get_args(reply, NULL,
-               DBUS_TYPE_UINT32, &dbus_screensaver_cookie,
-               DBUS_TYPE_INVALID))
-         dbus_screensaver_cookie = 0;
-      else
-         ret = true;
-
-      dbus_message_unref(reply);
-   }
-
-   dbus_message_unref(msg);
-
-   if (dbus_screensaver_cookie == 0)
-   {
-      RARCH_ERR("[DBus]: Failed to suspend screensaver via DBus.\n");
-   }
-   else
-   {
-      RARCH_LOG("[DBus]: Suspended screensaver via DBus.\n");
-   }
-
-   return ret;
-}
-
-static void dbus_screensaver_uninhibit(void)
-{
-   DBusMessage *msg = NULL;
-
-   if (!dbus_connection)
-      return;
-
-   if (dbus_screensaver_cookie == 0)
-      return;
-
-   msg = dbus_message_new_method_call("org.freedesktop.ScreenSaver",
-         "/org/freedesktop/ScreenSaver",
-         "org.freedesktop.ScreenSaver",
-         "UnInhibit");
-   if (!msg)
-       return;
-
-   dbus_message_append_args(msg,
-         DBUS_TYPE_UINT32, &dbus_screensaver_cookie,
-         DBUS_TYPE_INVALID);
-
-   if (dbus_connection_send(dbus_connection, msg, NULL))
-      dbus_connection_flush(dbus_connection);
-   dbus_message_unref(msg);
-
-   dbus_screensaver_cookie = 0;
-}
-
-/* Returns false when fallback should be attempted */
-bool x11_suspend_screensaver_dbus(bool enable)
-{
-   if (enable) return dbus_screensaver_inhibit();
-   dbus_screensaver_uninhibit();
-   return false;
-}
-#endif
-
 static void x11_hide_mouse(Display *dpy, Window win)
 {
    static char bm_no_data[] = {0, 0, 0, 0, 0, 0, 0, 0};
@@ -221,20 +107,20 @@ void x11_show_mouse(Display *dpy, Window win, bool state)
       x11_hide_mouse(dpy, win);
 }
 
-void x11_windowed_fullscreen(Display *dpy, Window win)
+void x11_set_net_wm_fullscreen(Display *dpy, Window win)
 {
-   XEvent xev = {0};
+   XEvent xev                 = {0};
 
-   XA_INIT(_NET_WM_STATE);
-   XA_INIT(_NET_WM_STATE_FULLSCREEN);
+   XA_NET_WM_STATE            = XInternAtom(dpy, "_NET_WM_STATE", False);
+   XA_NET_WM_STATE_FULLSCREEN = XInternAtom(dpy, "_NET_WM_STATE_FULLSCREEN", False);
 
-   xev.xclient.type         = ClientMessage;
-   xev.xclient.send_event   = True;
-   xev.xclient.message_type = XA_NET_WM_STATE;
-   xev.xclient.window       = win;
-   xev.xclient.format       = 32;
-   xev.xclient.data.l[0]    = _NET_WM_STATE_ADD;
-   xev.xclient.data.l[1]    = XA_NET_WM_STATE_FULLSCREEN;
+   xev.xclient.type           = ClientMessage;
+   xev.xclient.send_event     = True;
+   xev.xclient.message_type   = XA_NET_WM_STATE;
+   xev.xclient.window         = win;
+   xev.xclient.format         = 32;
+   xev.xclient.data.l[0]      = _NET_WM_STATE_ADD;
+   xev.xclient.data.l[1]      = XA_NET_WM_STATE_FULLSCREEN;
 
    XSendEvent(dpy, DefaultRootWindow(dpy), False,
          SubstructureRedirectMask | SubstructureNotifyMask,
@@ -246,16 +132,16 @@ void x11_windowed_fullscreen(Display *dpy, Window win)
 void x11_move_window(Display *dpy, Window win, int x, int y,
       unsigned width, unsigned height)
 {
-   XEvent xev = {0};
+   XEvent xev               = {0};
 
-   XA_INIT(_NET_MOVERESIZE_WINDOW);
+   XA_NET_MOVERESIZE_WINDOW = XInternAtom(dpy, "_NET_MOVERESIZE_WINDOW", False);
 
    xev.xclient.type         = ClientMessage;
    xev.xclient.send_event   = True;
    xev.xclient.message_type = XA_NET_MOVERESIZE_WINDOW;
    xev.xclient.window       = win;
    xev.xclient.format       = 32;
-   xev.xclient.data.l[0]    = (1 << MOVERESIZE_X_SHIFT) 
+   xev.xclient.data.l[0]    = (1 << MOVERESIZE_X_SHIFT)
       | (1 << MOVERESIZE_Y_SHIFT);
    xev.xclient.data.l[1]    = x;
    xev.xclient.data.l[2]    = y;
@@ -274,9 +160,35 @@ static void x11_set_window_class(Display *dpy, Window win)
    XSetClassHint(dpy, win, &hint);
 }
 
+static void x11_set_window_pid(Display *dpy, Window win)
+{
+    long scret     = 0;
+    char *hostname = NULL;
+    pid_t pid      = getpid();
+
+    XChangeProperty(dpy, win, XInternAtom(dpy, "_NET_WM_PID", False),
+        XA_CARDINAL, 32, PropModeReplace, (unsigned char *)&pid, 1);
+
+    errno = 0;
+    if((scret = sysconf(_SC_HOST_NAME_MAX)) == -1 && errno)
+        return;
+    if((hostname = (char*)malloc(scret + 1)) == NULL)
+        return;
+
+    if(gethostname(hostname, scret + 1) == -1)
+        RARCH_WARN("Failed to get hostname.\n");
+    else
+    {
+        XChangeProperty(dpy, win, XA_WM_CLIENT_MACHINE, XA_STRING, 8,
+            PropModeReplace, (unsigned char *)hostname, strlen(hostname));
+    }
+    free(hostname);
+}
+
 void x11_set_window_attr(Display *dpy, Window win)
 {
    x11_set_window_class(dpy, win);
+   x11_set_window_pid(dpy, win);
 }
 
 static void xdg_screensaver_inhibit(Window wnd)
@@ -286,7 +198,7 @@ static void xdg_screensaver_inhibit(Window wnd)
 
    cmd[0] = '\0';
 
-   RARCH_LOG("Suspending screensaver (X11, xdg-screensaver).\n");
+   RARCH_LOG("[X11]: Suspending screensaver (X11, xdg-screensaver).\n");
 
    snprintf(cmd, sizeof(cmd), "xdg-screensaver suspend 0x%x", (int)wnd);
 
@@ -316,19 +228,41 @@ void x11_suspend_screensaver_xdg_screensaver(Window wnd, bool enable)
 void x11_suspend_screensaver(Window wnd, bool enable)
 {
 #ifdef HAVE_DBUS
-    if (x11_suspend_screensaver_dbus(enable))
+    if (dbus_suspend_screensaver(enable))
        return;
 #endif
     x11_suspend_screensaver_xdg_screensaver(wnd, enable);
 }
 
+float x11_get_refresh_rate(void *data)
+{
+   XWindowAttributes attr;
+   XF86VidModeModeLine modeline;
+   Screen *screen;
+   int screenid;
+   int dotclock;
+
+   if (!g_x11_dpy || g_x11_win == None)
+      return 0.0f;
+
+   if (!XGetWindowAttributes(g_x11_dpy, g_x11_win, &attr))
+      return 0.0f;
+
+   screen = attr.screen;
+   screenid = XScreenNumberOfScreen(screen);
+
+   XF86VidModeGetModeLine(g_x11_dpy, screenid, &dotclock, &modeline);
+
+   return (float) dotclock * 1000.0f / modeline.htotal / modeline.vtotal;
+}
+
 static bool get_video_mode(video_frame_info_t *video_info,
       Display *dpy, unsigned width, unsigned height,
       XF86VidModeModeInfo *mode, XF86VidModeModeInfo *desktop_mode)
 {
-   float refresh_mod;
    int i, num_modes            = 0;
    bool ret                    = false;
+   float refresh_mod           = 0.0f;
    float minimum_fps_diff      = 0.0f;
    XF86VidModeModeInfo **modes = NULL;
 
@@ -342,7 +276,7 @@ static bool get_video_mode(video_frame_info_t *video_info,
 
    *desktop_mode = *modes[0];
 
-   /* If we use black frame insertion, we fake a 60 Hz monitor 
+   /* If we use black frame insertion, we fake a 60 Hz monitor
     * for 120 Hz one, etc, so try to match that. */
    refresh_mod = video_info->black_frame_insertion ? 0.5f : 1.0f;
 
@@ -376,11 +310,11 @@ static bool get_video_mode(video_frame_info_t *video_info,
 
 bool x11_enter_fullscreen(video_frame_info_t *video_info,
       Display *dpy, unsigned width,
-      unsigned height, XF86VidModeModeInfo *desktop_mode)
+      unsigned height)
 {
    XF86VidModeModeInfo mode;
 
-   if (!get_video_mode(video_info, dpy, width, height, &mode, desktop_mode))
+   if (!get_video_mode(video_info, dpy, width, height, &mode, &desktop_mode))
       return false;
 
    if (!XF86VidModeSwitchToMode(dpy, DefaultScreen(dpy), &mode))
@@ -390,94 +324,12 @@ bool x11_enter_fullscreen(video_frame_info_t *video_info,
    return true;
 }
 
-void x11_exit_fullscreen(Display *dpy, XF86VidModeModeInfo *desktop_mode)
+void x11_exit_fullscreen(Display *dpy)
 {
-   XF86VidModeSwitchToMode(dpy, DefaultScreen(dpy), desktop_mode);
+   XF86VidModeSwitchToMode(dpy, DefaultScreen(dpy), &desktop_mode);
    XF86VidModeSetViewPort(dpy, DefaultScreen(dpy), 0, 0);
 }
 
-#ifdef HAVE_XINERAMA
-static XineramaScreenInfo *x11_query_screens(Display *dpy, int *num_screens)
-{
-   int major, minor;
-
-   if (!XineramaQueryExtension(dpy, &major, &minor))
-      return NULL;
-
-   XineramaQueryVersion(dpy, &major, &minor);
-   RARCH_LOG("[X11]: Xinerama version: %d.%d.\n", major, minor);
-
-   if (!XineramaIsActive(dpy))
-      return NULL;
-
-   return XineramaQueryScreens(dpy, num_screens);
-}
-
-bool x11_get_xinerama_coord(Display *dpy, int screen,
-      int *x, int *y, unsigned *w, unsigned *h)
-{
-   int i, num_screens       = 0;
-   bool                 ret = false;
-   XineramaScreenInfo *info = x11_query_screens(dpy, &num_screens);
-
-   RARCH_LOG("[X11]: Xinerama screens: %d.\n", num_screens);
-
-   for (i = 0; i < num_screens; i++)
-   {
-      if (info[i].screen_number != screen)
-         continue;
-
-      *x = info[i].x_org;
-      *y = info[i].y_org;
-      *w = info[i].width;
-      *h = info[i].height;
-      ret = true;
-      break;
-   }
-
-   XFree(info);
-   return ret;
-}
-
-unsigned x11_get_xinerama_monitor(Display *dpy, int x, int y,
-      int w, int h)
-{
-   int       i, num_screens = 0;
-   unsigned       monitor   = 0;
-   int       largest_area   = 0;
-   XineramaScreenInfo *info = x11_query_screens(dpy, &num_screens);
-
-   RARCH_LOG("[X11]: Xinerama screens: %d.\n", num_screens);
-
-   for (i = 0; i < num_screens; i++)
-   {
-      int area;
-      int max_lx = MAX(x, info[i].x_org);
-      int min_rx = MIN(x + w, info[i].x_org + info[i].width);
-      int max_ty = MAX(y, info[i].y_org);
-      int min_by = MIN(y + h, info[i].y_org + info[i].height);
-
-      int len_x  = min_rx - max_lx;
-      int len_y  = min_by - max_ty;
-
-      /* The whole window is outside the screen. */
-      if (len_x < 0 || len_y < 0)
-         continue;
-
-      area = len_x * len_y;
-
-      if (area > largest_area)
-      {
-         monitor = i;
-         largest_area = area;
-      }
-   }
-
-   XFree(info);
-   return monitor;
-}
-#endif
-
 bool x11_create_input_context(Display *dpy, Window win, XIM *xim, XIC *xic)
 {
    x11_destroy_input_context(xim, xic);
@@ -553,6 +405,78 @@ bool x11_get_metrics(void *data,
    return true;
 }
 
+static void x11_handle_key_event(XEvent *event, XIC ic, bool filter)
+{
+   int i;
+   Status status;
+   uint32_t chars[32];
+   unsigned key   = 0;
+   uint16_t mod   = 0;
+   unsigned state = event->xkey.state;
+   bool down      = event->type == KeyPress;
+   int num        = 0;
+   KeySym keysym  = 0;
+
+   chars[0]       = '\0';
+
+   if (!filter)
+   {
+      if (down)
+      {
+         char keybuf[32];
+
+         keybuf[0] = '\0';
+#ifdef X_HAVE_UTF8_STRING
+         status = 0;
+
+         /* XwcLookupString doesn't seem to work. */
+         num = Xutf8LookupString(ic, &event->xkey, keybuf,
+               ARRAY_SIZE(keybuf), &keysym, &status);
+
+         /* libc functions need UTF-8 locale to work properly,
+          * which makes mbrtowc a bit impractical.
+          *
+          * Use custom UTF8 -> UTF-32 conversion. */
+         num = utf8_conv_utf32(chars, ARRAY_SIZE(chars), keybuf, num);
+#else
+         (void)ic;
+         num = XLookupString(&event->xkey, keybuf,
+               sizeof(keybuf), &keysym, NULL); /* ASCII only. */
+         for (i = 0; i < num; i++)
+            chars[i] = keybuf[i] & 0x7f;
+#endif
+      }
+      else
+         keysym = XLookupKeysym(&event->xkey, (state & ShiftMask) || (state & LockMask));
+   }
+
+   /* We can't feed uppercase letters to the keycode translator. Seems like a bad idea
+    * to feed it keysyms anyway, so here is a little hack... */
+   if (keysym >= XK_A && keysym <= XK_Z)
+       keysym += XK_z - XK_Z;
+
+   key   = input_keymaps_translate_keysym_to_rk(keysym);
+
+   if (state & ShiftMask)
+      mod |= RETROKMOD_SHIFT;
+   if (state & LockMask)
+      mod |= RETROKMOD_CAPSLOCK;
+   if (state & ControlMask)
+      mod |= RETROKMOD_CTRL;
+   if (state & Mod1Mask)
+      mod |= RETROKMOD_ALT;
+   if (state & Mod4Mask)
+      mod |= RETROKMOD_META;
+   if (IsKeypadKey(keysym))
+      mod |= RETROKMOD_NUMLOCK;
+
+   input_keyboard_event(down, key, chars[0], mod, RETRO_DEVICE_KEYBOARD);
+
+   for (i = 1; i < num; i++)
+      input_keyboard_event(down, RETROK_UNKNOWN,
+            chars[i], mod, RETRO_DEVICE_KEYBOARD);
+}
+
 bool x11_alive(void *data)
 {
    while (XPending(g_x11_dpy))
@@ -567,7 +491,7 @@ bool x11_alive(void *data)
       switch (event.type)
       {
          case ClientMessage:
-            if (event.xclient.window == g_x11_win && 
+            if (event.xclient.window == g_x11_win &&
                   (Atom)event.xclient.data.l[0] == g_x11_quit_atom)
                frontend_driver_set_signal_handler_state(1);
             break;
@@ -711,11 +635,10 @@ bool x11_connect(void)
    dbus_ensure_connection();
 #endif
 
-
    return true;
 }
 
-void x11_update_title(void *data, video_frame_info_t *video_info)
+void x11_update_title(void *data, void *data2)
 {
    char title[128];
 
@@ -786,21 +709,104 @@ void x11_event_queue_check(XEvent *event)
    XIfEvent(g_x11_dpy, event, x11_wait_notify, NULL);
 }
 
-void x11_save_last_used_monitor(Window win)
+static bool x11_check_atom_supported(Display *dpy, Atom atom)
 {
-#ifdef HAVE_XINERAMA
-   XWindowAttributes target;
-   Window child;
-   int x = 0, y = 0;
+   Atom XA_NET_SUPPORTED = XInternAtom(dpy, "_NET_SUPPORTED", True);
+   Atom type;
+   int format;
+   unsigned long nitems;
+   unsigned long bytes_after;
+   Atom *prop;
+   int i;
 
-   XGetWindowAttributes(g_x11_dpy, g_x11_win, &target);
-   XTranslateCoordinates(g_x11_dpy, g_x11_win,
-         DefaultRootWindow(g_x11_dpy),
-         target.x, target.y, &x, &y, &child);
+   if (XA_NET_SUPPORTED == None)
+      return false;
 
-   g_x11_screen = x11_get_xinerama_monitor(g_x11_dpy, x, y,
-         target.width, target.height);
+   XGetWindowProperty(dpy, DefaultRootWindow(dpy), XA_NET_SUPPORTED,
+         0, UINT_MAX, False, XA_ATOM, &type, &format,&nitems,
+         &bytes_after, (unsigned char **) &prop);
 
-   RARCH_LOG("[X11]: Saved monitor #%u.\n", g_x11_screen);
-#endif
+   if (!prop || type != XA_ATOM)
+      return false;
+
+   for (i = 0; i < nitems; i++)
+   {
+      if (prop[i] == atom)
+      {
+         XFree(prop);
+         return true;
+      }
+   }
+
+   XFree(prop);
+
+   return false;
 }
+
+bool x11_has_net_wm_fullscreen(Display *dpy)
+{
+   XA_NET_WM_STATE_FULLSCREEN = XInternAtom(dpy, "_NET_WM_STATE_FULLSCREEN", False);
+
+   return x11_check_atom_supported(dpy, XA_NET_WM_STATE_FULLSCREEN);
+}
+
+char *x11_get_wm_name(Display *dpy)
+{
+   Atom XA_NET_SUPPORTING_WM_CHECK = XInternAtom(g_x11_dpy, "_NET_SUPPORTING_WM_CHECK", False);
+   Atom XA_NET_WM_NAME             = XInternAtom(g_x11_dpy, "_NET_WM_NAME", False);
+   Atom XA_UTF8_STRING             = XInternAtom(g_x11_dpy, "UTF8_STRING", False);
+   int status;
+   Atom type;
+   int  format;
+   unsigned long nitems;
+   unsigned long bytes_after;
+   unsigned char *propdata;
+   char *title;
+   Window window;
+
+   if (!XA_NET_SUPPORTING_WM_CHECK || !XA_NET_WM_NAME)
+      return NULL;
+
+   status = XGetWindowProperty(dpy,
+                               DefaultRootWindow(dpy),
+                               XA_NET_SUPPORTING_WM_CHECK,
+                               0,
+                               1,
+                               False,
+                               XA_WINDOW,
+                               &type,
+                               &format,
+                               &nitems,
+                               &bytes_after,
+                               &propdata);
+
+   if (status == Success && propdata)
+      window = ((Window *) propdata)[0];
+   else
+      return NULL;
+
+   XFree(propdata);
+
+   status = XGetWindowProperty(dpy,
+                               window,
+                               XA_NET_WM_NAME,
+                               0,
+                               8192,
+                               False,
+                               XA_UTF8_STRING,
+                               &type,
+                               &format,
+                               &nitems,
+                               &bytes_after,
+                               &propdata);
+
+   if (status == Success && propdata)
+      title = strdup((char *) propdata);
+   else
+      return NULL;
+
+   XFree(propdata);
+
+   return title;
+}
+
