@@ -1,4 +1,4 @@
-/* Copyright  (C) 2010-2017 The RetroArch team
+/* Copyright  (C) 2010-2018 The RetroArch team
  *
  * ---------------------------------------------------------------------------------------
  * The following license statement only applies to this file (compat_snprintf.c).
@@ -21,12 +21,32 @@
  */
 
 /* THIS FILE HAS NOT BEEN VALIDATED ON PLATFORMS BESIDES MSVC */
+#ifdef _MSC_VER
 
 #include <retro_common.h>
-
-#include <stdio.h>
+#if _MSC_VER >= 1800
+#include <stdio.h> /* added for _vsnprintf_s and _vscprintf on VS2015 and VS2017 */
+#endif
 #include <stdarg.h>
 
+#if _MSC_VER < 1800
+#define va_copy(dst, src) ((dst) = (src))
+#endif
+
+#if _MSC_VER < 1300
+#define _vscprintf c89_vscprintf_retro__
+
+static int c89_vscprintf_retro__(const char *format, va_list pargs)
+{
+   int retval;
+   va_list argcopy;
+   va_copy(argcopy, pargs);
+   retval = vsnprintf(NULL, 0, format, argcopy);
+   va_end(argcopy);
+   return retval;
+}
+#endif
+
 /* http://stackoverflow.com/questions/2915672/snprintf-and-visual-studio-2010 */
 
 int c99_vsnprintf_retro__(char *outBuf, size_t size, const char *format, va_list ap)
@@ -35,13 +55,19 @@ int c99_vsnprintf_retro__(char *outBuf, size_t size, const char *format, va_list
 
    if (size != 0)
 #if (_MSC_VER <= 1310)
-       count = _vsnprintf(outBuf, size, format, ap);
+       count = _vsnprintf(outBuf, size - 1, format, ap);
 #else
-       count = _vsnprintf_s(outBuf, size, _TRUNCATE, format, ap);
+       count = _vsnprintf_s(outBuf, size, size - 1, format, ap);
 #endif
    if (count == -1)
        count = _vscprintf(format, ap);
 
+   if (count == size)
+   {
+      /* there was no room for a NULL, so truncate the last character */
+      outBuf[size - 1] = '\0';
+   }
+
    return count;
 }
 
@@ -56,3 +82,4 @@ int c99_snprintf_retro__(char *outBuf, size_t size, const char *format, ...)
 
    return count;
 }
+#endif
