@@ -1,3 +1,9 @@
+HAVE_LIBRETRO=             # Libretro library used
+HAVE_ASSETS_DIR=           # Assets install directory
+HAVE_BIN_DIR=              # Binary install directory (Deprecated)
+HAVE_MAN_DIR=              # Manpage install directory (Deprecated)
+HAVE_OPENGLES_LIBS=        # Link flags for custom GLES library
+HAVE_OPENGLES_CFLAGS=      # C-flags for custom GLES library
 HAVE_CACA=no               # Libcaca support
 HAVE_LIBRETRODB=yes        # Libretrodb support
 HAVE_RGUI=yes              # RGUI menu
@@ -5,6 +11,8 @@ HAVE_MATERIALUI=auto       # MaterialUI menu
 HAVE_XMB=auto              # XMB menu
 HAVE_ZARCH=no              # Zarch menu
 HAVE_NUKLEAR=no            # Nuklear menu
+HAVE_RUNAHEAD=yes          # Runahead support
+HAVE_OVERLAY=yes           # Overlay support
 HAVE_DYNAMIC=yes           # Dynamic loading of libretro library
 HAVE_SDL=auto              # SDL support
 C89_SDL=no
@@ -12,14 +20,9 @@ HAVE_SDL2=auto             # SDL2 support (disables SDL 1.x)
 C89_SDL2=no
 HAVE_LIBUSB=auto           # Libusb HID support
 C89_LIBUSB=no
-HAVE_DBUS=auto             # dbus support
+HAVE_DBUS=no               # dbus support
+HAVE_SYSTEMD=auto          # Systemd support
 HAVE_UDEV=auto             # Udev/Evdev gamepad support
-HAVE_LIBRETRO=             # Libretro library used
-HAVE_ASSETS_DIR=           # Assets install directory
-HAVE_BIN_DIR=              # Binary install directory
-HAVE_MAN_DIR=              # Manpage install directory
-HAVE_OPENGLES_LIBS=        # Link flags for custom GLES library
-HAVE_OPENGLES_CFLAGS=      # C-flags for custom GLES library
 HAVE_THREADS=auto          # Threading support
 HAVE_THREAD_STORAGE=auto   # Thread Local Storage support
 HAVE_FFMPEG=auto           # FFmpeg recording support
@@ -30,7 +33,18 @@ HAVE_NETWORKING=auto       # Networking features (recommended)
 HAVE_NETWORKGAMEPAD=auto   # Networked game pad (plus baked-in core)
 C89_NETWORKGAMEPAD=no
 HAVE_MINIUPNPC=auto        # Mini UPnP client library (for NAT traversal)
+HAVE_BUILTINMINIUPNPC=yes  # Bake in Mini UPnP client library (for NAT traversal)
+C89_BUILTINMINIUPNPC=no
+HAVE_D3D8=no               # Direct3D 8 support
 HAVE_D3D9=yes              # Direct3D 9 support
+C89_D3D9=no
+HAVE_D3D10=yes             # Direct3D 10 support
+C89_D3D10=no
+HAVE_D3D11=yes             # Direct3D 11 support
+C89_D3D11=no
+HAVE_D3D12=yes             # Direct3D 12 support
+C89_D3D12=no
+HAVE_D3DX=yes              # Direct3DX support
 HAVE_OPENGL=auto           # OpenGL support
 HAVE_MALI_FBDEV=no         # Mali fbdev context support
 HAVE_VIVANTE_FBDEV=no      # Vivante fbdev context support
@@ -50,12 +64,13 @@ C89_WAYLAND=no
 HAVE_EGL=auto              # EGL context support
 HAVE_VG=auto               # OpenVG support
 HAVE_CG=auto               # Cg shader support
+HAVE_HLSL=no               # HLSL9 shader support (for Direct3D9)
 HAVE_LIBXML2=auto          # libxml2 support
 HAVE_BUILTINZLIB=no        # Bake in zlib
 HAVE_ZLIB=auto             # zlib support (ZIP extract, PNG decoding/encoding)
-HAVE_FBO=auto              # render-to-texture (FBO) support
 HAVE_ALSA=auto             # ALSA support
 C89_ALSA=no
+HAVE_TINYALSA=auto         # TinyALSA support
 HAVE_OSS=auto              # OSS support
 HAVE_RSOUND=auto           # RSound support
 HAVE_ROAR=auto             # RoarAudio support
@@ -67,6 +82,8 @@ C89_PULSE=no
 HAVE_FREETYPE=auto         # FreeType support
 HAVE_STB_FONT=yes          # stb_truetype font support
 HAVE_STB_IMAGE=yes         # stb image loading support
+HAVE_STB_VORBIS=yes        # stb vorbis support
+HAVE_IBXM=yes        	   # IBXM support
 HAVE_XVIDEO=auto           # XVideo support
 HAVE_PYTHON=no             # Python 3 support for shaders
 C89_PYTHON=no
@@ -75,14 +92,17 @@ HAVE_NEON=no               # ARM NEON optimizations
 HAVE_SSE=no                # x86 SSE optimizations (SSE, SSE2)
 HAVE_FLOATHARD=no          # Force hard float ABI (for ARM)
 HAVE_FLOATSOFTFP=no        # Force soft float ABI (for ARM)
+HAVE_CHD=yes               # Compile in chd support
 HAVE_7ZIP=yes              # Compile in 7z support
+HAVE_FLAC=auto             # Compile in flac support
+HAVE_BUILTINFLAC=yes       # Bake in flac support
 HAVE_UPDATE_ASSETS=yes     # Disable downloading assets with online updater
 HAVE_PRESERVE_DYLIB=no     # Enable dlclose() for Valgrind support
 HAVE_PARPORT=auto          # Parallel port joypad support
 HAVE_IMAGEVIEWER=yes       # Built-in image viewer support.
 HAVE_MMAP=auto             # MMAP support
-HAVE_QT=no                 # Qt companion support
-HAVE_QT_WRAPPER=no
+HAVE_QT=auto               # Qt companion support
+C89_QT=no
 HAVE_XSHM=no               # XShm video driver support
 HAVE_CHEEVOS=yes           # Retro Achievements
 HAVE_SHADERPIPELINE=yes    # Additional shader-based pipelines
@@ -98,3 +118,4 @@ HAVE_HID=yes               # Low-level HID (Human Interface Device) support
 HAVE_LANGEXTRA=yes         # Multi-language support
 HAVE_OSMESA=no             # Off-screen Mesa rendering
 HAVE_VIDEOPROCESSOR=auto   # Enable video processor core
+HAVE_VIDEOCORE=auto        # Broadcom Videocore 4 support
