@@ -20,18 +20,27 @@
 
 #include <boolean.h>
 #include "joypad_connection.h"
+#include "../input_defines.h"
+#include "../common/hid/hid_device_driver.h"
 
 struct hidpad_ps3_data
 {
    struct pad_connection* connection;
-   send_control_t send_control;
+   hid_driver_t *driver;
    uint8_t data[512];
    uint32_t slot;
-   uint64_t buttons;
+   uint32_t buttons;
    bool have_led;
    uint16_t motors[2];
 };
 
+/*
+ * TODO: give these more meaningful names.
+ */
+
+#define DS3_ACTIVATION_REPORT_ID 0xf4
+#define DS3_RUMBLE_REPORT_ID     0x01
+
 static void hidpad_ps3_send_control(struct hidpad_ps3_data* device)
 {
    /* TODO: Can this be modified to turn off motion tracking? */
@@ -48,20 +57,28 @@ static void hidpad_ps3_send_control(struct hidpad_ps3_data* device)
       0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
    };
 
+   /* Turn on the appropriate LED */
    report_buffer[11] = 1 << ((device->slot % 4) + 1);
+   /* Set rumble state */
    report_buffer[4]  = device->motors[1] >> 8;
    report_buffer[6]  = device->motors[0] >> 8;
 #ifdef HAVE_WIIUSB_HID
    report_buffer[1]  = 0x03; /* send control message type */
-   device->send_control(device->connection, &report_buffer[1], sizeof(report_buffer)-1);
+   device->driver->send_control(device->connection, &report_buffer[1], sizeof(report_buffer)-1);
+#elif defined(WIIU)
+   device->driver->set_report(device->connection,
+                              HID_REPORT_OUTPUT,
+                              DS3_RUMBLE_REPORT_ID,
+                              report_buffer+2,
+                              sizeof(report_buffer) - (2*sizeof(uint8_t)));
 #else
-   device->send_control(device->connection, report_buffer, sizeof(report_buffer));
+   device->driver->send_control(device->connection, report_buffer, sizeof(report_buffer));
 #endif
 }
 
-static void* hidpad_ps3_init(void *data, uint32_t slot, send_control_t ptr)
+static void* hidpad_ps3_init(void *data, uint32_t slot, hid_driver_t *driver)
 {
-#ifdef HAVE_WIIUSB_HID
+#if defined(HAVE_WIIUSB_HID) || defined(WIIU)
    /* Special command to enable Sixaxis, first byte defines the message type */
    static uint8_t magic_data[]       = {0x02, 0x42, 0x0c, 0x00, 0x00};
 #elif defined(IOS)
@@ -81,12 +98,22 @@ static void* hidpad_ps3_init(void *data, uint32_t slot, send_control_t ptr)
       return NULL;
    }
 
-   device->connection   = connection;
-   device->slot         = slot;
-   device->send_control = ptr;
+   device->connection = connection;
+   device->slot       = slot;
+   device->driver     = driver;
 
 #if defined(IOS) || defined(HAVE_WIIUSB_HID)
-   device->send_control(device->connection, magic_data, sizeof(magic_data));
+   device->driver->send_control(device->connection, magic_data, sizeof(magic_data));
+#endif
+
+#ifdef WIIU
+   device->driver->set_protocol(device->connection, 1);
+   hidpad_ps3_send_control(device);
+   device->driver->set_report(device->connection,
+                              HID_REPORT_FEATURE,
+                              DS3_ACTIVATION_REPORT_ID,
+                              magic_data+1,
+                              (sizeof(magic_data) - sizeof(uint8_t)));
 #endif
 
 #ifndef HAVE_WIIUSB_HID
@@ -104,12 +131,20 @@ static void hidpad_ps3_deinit(void *data)
       free(device);
 }
 
-static uint64_t hidpad_ps3_get_buttons(void *data)
+static void hidpad_ps3_get_buttons(void *data, input_bits_t *state)
 {
-   struct hidpad_ps3_data *device = (struct hidpad_ps3_data*)data;
-   if (!device)
-      return 0;
-   return device->buttons;
+	struct hidpad_ps3_data *device = (struct hidpad_ps3_data*)data;
+	if ( device )
+	{
+		/*copy first 16 bits - standard RetroPad controls*/
+		BITS_COPY16_PTR(state, device->buttons);
+
+		/*PS button?*/
+		if (device->buttons & 0x10000)
+			BIT256_SET_PTR(state, RARCH_MENU_TOGGLE);
+	}
+	else
+      BIT256_CLEAR_ALL_PTR(state);
 }
 
 static int16_t hidpad_ps3_get_axis(void *data, unsigned axis)
