@@ -28,10 +28,10 @@ struct hidpad_snesusb_data
    struct pad_connection* connection;
    uint8_t data[64];
    uint32_t slot;
-   uint64_t buttons;
+   uint32_t buttons;
 };
 
-static void* hidpad_snesusb_init(void *data, uint32_t slot, send_control_t ptr)
+static void* hidpad_snesusb_init(void *data, uint32_t slot, hid_driver_t *driver)
 {
    struct pad_connection* connection = (struct pad_connection*)data;
    struct hidpad_snesusb_data* device    = (struct hidpad_snesusb_data*)
@@ -60,12 +60,15 @@ static void hidpad_snesusb_deinit(void *data)
       free(device);
 }
 
-static uint64_t hidpad_snesusb_get_buttons(void *data)
+static void hidpad_snesusb_get_buttons(void *data, input_bits_t *state)
 {
-   struct hidpad_snesusb_data *device = (struct hidpad_snesusb_data*)data;
-   if (!device)
-      return 0;
-   return device->buttons;
+	struct hidpad_snesusb_data *device = (struct hidpad_snesusb_data*)data;
+	if (device)
+   {
+		BITS_COPY16_PTR(state, device->buttons);
+	}
+   else
+		BIT256_CLEAR_ALL_PTR(state);
 }
 
 static int16_t hidpad_snesusb_get_axis(void *data, unsigned axis)
@@ -85,7 +88,7 @@ static int16_t hidpad_snesusb_get_axis(void *data, unsigned axis)
 static void hidpad_snesusb_packet_handler(void *data, uint8_t *packet, uint16_t size)
 {
    uint32_t i, pressed_keys;
-   static const uint32_t button_mapping[17] =
+   static const uint32_t button_mapping[16] =
    {
       RETRO_DEVICE_ID_JOYPAD_L,
       RETRO_DEVICE_ID_JOYPAD_R,
@@ -102,8 +105,7 @@ static void hidpad_snesusb_packet_handler(void *data, uint8_t *packet, uint16_t
       RETRO_DEVICE_ID_JOYPAD_X,
       RETRO_DEVICE_ID_JOYPAD_A,
       RETRO_DEVICE_ID_JOYPAD_B,
-      RETRO_DEVICE_ID_JOYPAD_Y,
-      16, /* HOME BUTTON when pressing SELECT+START */
+      RETRO_DEVICE_ID_JOYPAD_Y
    };
    struct hidpad_snesusb_data *device = (struct hidpad_snesusb_data*)data;
 
@@ -114,12 +116,11 @@ static void hidpad_snesusb_packet_handler(void *data, uint8_t *packet, uint16_t
 
    device->buttons = 0;
 
-   pressed_keys  = device->data[7] | (device->data[6] << 8) |
-                (((device->data[7] & 0x30) == 0x30) ? (1 << 16) : 0);  /* SELECT+START = MENU TOGGLE */
+   pressed_keys  = device->data[7] | (device->data[6] << 8);
 
-   for (i = 0; i < 17; i ++)
+   for (i = 0; i < 16; i ++)
       if (button_mapping[i] != NO_BTN)
-         device->buttons |= (pressed_keys & (1 << i)) ? (UINT64_C(1) << button_mapping[i]) : 0;
+         device->buttons |= (pressed_keys & (1 << i)) ? (1 << button_mapping[i]) : 0;
 }
 
 static void hidpad_snesusb_set_rumble(void *data,
