@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -27,6 +27,8 @@
 
 #include "core_option_manager.h"
 
+#include "../verbosity.h"
+
 static bool core_option_manager_parse_variable(
       core_option_manager_t *opt, size_t idx,
       const struct retro_variable *var)
@@ -42,7 +44,8 @@ static bool core_option_manager_parse_variable(
    if (!string_is_empty(var->value))
       value                   = strdup(var->value);
 
-   desc_end    = strstr(value, "; ");
+   if (!string_is_empty(value))
+      desc_end                = strstr(value, "; ");
 
    if (!desc_end)
       goto error;
@@ -238,6 +241,7 @@ bool core_option_manager_flush(core_option_manager_t *opt)
                core_option_manager_get_val(opt, i));
    }
 
+   RARCH_LOG("Saved core options file to \"%s\"\n", opt->conf_path);
    return config_file_write(opt->conf, opt->conf_path);
 }
 
@@ -323,7 +327,7 @@ void core_option_manager_set_val(core_option_manager_t *opt,
 
    if (!opt)
       return;
-   
+
    option        = (struct core_option*)&opt->opts[idx];
    option->index = val_idx % option->vals->size;
 
@@ -344,7 +348,7 @@ void core_option_manager_next(core_option_manager_t *opt, size_t idx)
 
    if (!opt)
       return;
-   
+
    option        = (struct core_option*)&opt->opts[idx];
 
    option->index = (option->index + 1) % option->vals->size;
@@ -366,7 +370,7 @@ void core_option_manager_prev(core_option_manager_t *opt, size_t idx)
 
    if (!opt)
       return;
-   
+
    option        = (struct core_option*)&opt->opts[idx];
    option->index = (option->index + option->vals->size - 1) %
       option->vals->size;
