@@ -1,8 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- *  Copyright (C) 2013-2015 - Jason Fetters
- *  Copyright (C) 2016 - Brad Parker
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *  Copyright (C) 2016-2017 - Brad Parker
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -24,6 +23,7 @@
 
 #include <file/archive_file.h>
 #include <retro_common_api.h>
+#include <queues/task_queue.h>
 
 RETRO_BEGIN_DECLS
 
@@ -81,6 +81,18 @@ typedef struct
 
 typedef struct
 {
+   int analog_supported;
+   int rumble_supported;
+   int coop_supported;
+   uint32_t crc32;
+   unsigned size;
+   unsigned famitsu_magazine_rating;
+   unsigned edge_magazine_rating;
+   unsigned edge_magazine_issue;
+   unsigned max_users;
+   unsigned releasemonth;
+   unsigned releaseyear;
+   unsigned tgdb_rating;
    char *name;
    char *rom_name;
    char *serial;
@@ -97,27 +109,15 @@ typedef struct
    char *pegi_rating;
    char *cero_rating;
    char *enhancement_hw;
-   uint32_t crc32;
    char *sha1;
    char *md5;
-   unsigned size;
-   unsigned famitsu_magazine_rating;
-   unsigned edge_magazine_rating;
-   unsigned edge_magazine_issue;
-   unsigned max_users;
-   unsigned releasemonth;
-   unsigned releaseyear;
-   unsigned tgdb_rating;
-   int analog_supported;
-   int rumble_supported;
-   int coop_supported;
    void *userdata;
 } database_info_t;
 
 typedef struct
 {
-   database_info_t *list;
    size_t count;
+   database_info_t *list;
 } database_info_list_t;
 
 database_info_list_t *database_info_list_new(const char *rdb_path,
@@ -126,18 +126,14 @@ database_info_list_t *database_info_list_new(const char *rdb_path,
 void database_info_list_free(database_info_list_t *list);
 
 database_info_handle_t *database_info_dir_init(const char *dir,
-      enum database_type type);
+      enum database_type type, retro_task_t *task,
+      bool show_hidden_files);
 
 database_info_handle_t *database_info_file_init(const char *path,
-      enum database_type type);
+      enum database_type type, retro_task_t *task);
 
 void database_info_free(database_info_handle_t *handle);
 
-#if 0
-int database_info_build_query(
-      char *query, size_t len, const char *label, const char *path);
-#endif
-
 int database_info_build_query_enum(
       char *query, size_t len, enum database_query_type type, const char *path);
 
