@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- * 
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -29,6 +29,7 @@
 #endif
 
 #include "command.h"
+#include "dynamic.h"
 #include "msg_hash.h"
 
 #include "audio/audio_driver.h"
@@ -36,24 +37,14 @@
 #include "record/record_driver.h"
 #include "location/location_driver.h"
 #include "wifi/wifi_driver.h"
+#include "led/led_driver.h"
 #include "configuration.h"
 #include "core.h"
 #include "core_info.h"
 #include "driver.h"
-#include "runloop.h"
+#include "retroarch.h"
 #include "verbosity.h"
 
-#define HASH_LOCATION_DRIVER           0x09189689U
-#define HASH_CAMERA_DRIVER             0xf25db959U
-#define HASH_MENU_DRIVER               0xd607fb05U
-#define HASH_INPUT_DRIVER              0x4c087840U
-#define HASH_INPUT_JOYPAD_DRIVER       0xab124146U
-#define HASH_VIDEO_DRIVER              0x1805a5e7U
-#define HASH_AUDIO_DRIVER              0x26594002U
-#define HASH_AUDIO_RESAMPLER_DRIVER    0xedcba9ecU
-#define HASH_RECORD_DRIVER             0x144cd2cfU
-#define HASH_WIFI_DRIVER               0x64d7d17fU
-
 /**
  * find_driver_nonempty:
  * @label              : string of driver type to be found.
@@ -71,62 +62,68 @@ static const void *find_driver_nonempty(const char *label, int i,
       char *s, size_t len)
 {
    const void *drv = NULL;
-   uint32_t hash   = msg_hash_calculate(label);
 
-   switch (hash)
+   if (string_is_equal(label, "camera_driver"))
    {
-      case HASH_CAMERA_DRIVER:
-         drv = camera_driver_find_handle(i);
-         if (drv)
-            strlcpy(s, camera_driver_find_ident(i), len);
-         break;
-      case HASH_LOCATION_DRIVER:
-         drv = location_driver_find_handle(i);
-         if (drv)
-            strlcpy(s, location_driver_find_ident(i), len);
-         break;
-      case HASH_MENU_DRIVER:
+      drv = camera_driver_find_handle(i);
+      if (drv)
+         strlcpy(s, camera_driver_find_ident(i), len);
+   }
+   else if (string_is_equal(label, "location_driver"))
+   {
+      drv = location_driver_find_handle(i);
+      if (drv)
+         strlcpy(s, location_driver_find_ident(i), len);
+   }
 #ifdef HAVE_MENU
-         drv = menu_driver_find_handle(i);
-         if (drv)
-            strlcpy(s, menu_driver_find_ident(i), len);
+   else if (string_is_equal(label, "menu_driver"))
+   {
+      drv = menu_driver_find_handle(i);
+      if (drv)
+         strlcpy(s, menu_driver_find_ident(i), len);
+   }
 #endif
-         break;
-      case HASH_INPUT_DRIVER:
-         drv = input_driver_find_handle(i);
-         if (drv)
-            strlcpy(s, input_driver_find_ident(i), len);
-         break;
-      case HASH_INPUT_JOYPAD_DRIVER:
-         drv = joypad_driver_find_handle(i);
-         if (drv)
-            strlcpy(s, joypad_driver_find_ident(i), len);
-         break;
-      case HASH_VIDEO_DRIVER:
-         drv = video_driver_find_handle(i);
-         if (drv)
-            strlcpy(s, video_driver_find_ident(i), len);
-         break;
-      case HASH_AUDIO_DRIVER:
-         drv = audio_driver_find_handle(i);
-         if (drv)
-            strlcpy(s, audio_driver_find_ident(i), len);
-         break;
-      case HASH_RECORD_DRIVER:
-         drv = record_driver_find_handle(i);
-         if (drv)
-            strlcpy(s, record_driver_find_ident(i), len);
-         break;
-      case HASH_AUDIO_RESAMPLER_DRIVER:
-         drv = audio_resampler_driver_find_handle(i);
-         if (drv)
-            strlcpy(s, audio_resampler_driver_find_ident(i), len);
-         break;
-      case HASH_WIFI_DRIVER:
-         drv = wifi_driver_find_handle(i);
-         if (drv)
-            strlcpy(s, wifi_driver_find_ident(i), len);
-         break;
+   else if (string_is_equal(label, "input_driver"))
+   {
+      drv = input_driver_find_handle(i);
+      if (drv)
+         strlcpy(s, input_driver_find_ident(i), len);
+   }
+   else if (string_is_equal(label, "input_joypad_driver"))
+   {
+      drv = joypad_driver_find_handle(i);
+      if (drv)
+         strlcpy(s, joypad_driver_find_ident(i), len);
+   }
+   else if (string_is_equal(label, "video_driver"))
+   {
+      drv = video_driver_find_handle(i);
+      if (drv)
+         strlcpy(s, video_driver_find_ident(i), len);
+   }
+   else if (string_is_equal(label, "audio_driver"))
+   {
+      drv = audio_driver_find_handle(i);
+      if (drv)
+         strlcpy(s, audio_driver_find_ident(i), len);
+   }
+   else if (string_is_equal(label, "record_driver"))
+   {
+      drv = record_driver_find_handle(i);
+      if (drv)
+         strlcpy(s, record_driver_find_ident(i), len);
+   }
+   else if (string_is_equal(label, "audio_resampler_driver"))
+   {
+      drv = audio_resampler_driver_find_handle(i);
+      if (drv)
+         strlcpy(s, audio_resampler_driver_find_ident(i), len);
+   }
+   else if (string_is_equal(label, "wifi_driver"))
+   {
+      drv = wifi_driver_find_handle(i);
+      if (drv)
+         strlcpy(s, wifi_driver_find_ident(i), len);
    }
 
    return drv;
@@ -149,7 +146,7 @@ static int driver_find_index(const char * label, const char *drv)
 
    str[0] = '\0';
 
-   for (i = 0; 
+   for (i = 0;
          find_driver_nonempty(label, i, str, sizeof(str)) != NULL; i++)
    {
       if (string_is_empty(str))
@@ -167,6 +164,30 @@ static bool driver_find_first(const char *label, char *s, size_t len)
    return true;
 }
 
+/**
+ * driver_find_last:
+ * @label              : string of driver type to be found.
+ * @s                  : identifier of driver to be found.
+ * @len                : size of @s.
+ *
+ * Find last driver in driver array.
+ **/
+static bool driver_find_last(const char *label, char *s, size_t len)
+{
+   unsigned i;
+
+   for (i = 0;
+         find_driver_nonempty(label, i, s, len) != NULL; i++)
+   {}
+
+   if (i)
+      find_driver_nonempty(label, i-1, s, len);
+   else
+      driver_find_first(label, s, len);
+
+   return true;
+}
+
 /**
  * driver_find_prev:
  * @label              : string of driver type to be found.
@@ -202,7 +223,7 @@ bool driver_find_next(const char *label, char *s, size_t len)
 {
    int i = driver_find_index(label, s);
 
-   if (i >= 0 && !string_is_equal(s, "null"))
+   if (i >= 0 && string_is_not_equal(s, "null"))
    {
       find_driver_nonempty(label, i + 1, s, len);
       return true;
@@ -222,7 +243,7 @@ static void driver_adjust_system_rates(void)
    if (!video_driver_get_ptr(false))
       return;
 
-   if (runloop_ctl(RUNLOOP_CTL_IS_NONBLOCK_FORCED, NULL))
+   if (rarch_ctl(RARCH_CTL_IS_NONBLOCK_FORCED, NULL))
       command_event(CMD_EVENT_VIDEO_SET_NONBLOCKING_STATE, NULL);
    else
       driver_set_nonblock_state();
@@ -233,7 +254,7 @@ static void driver_adjust_system_rates(void)
  *
  * Sets audio and video drivers to nonblock state (if enabled).
  *
- * If nonblock state is false, sets 
+ * If nonblock state is false, sets
  * blocking state for both audio and video drivers instead.
  **/
 void driver_set_nonblock_state(void)
@@ -246,8 +267,8 @@ void driver_set_nonblock_state(void)
       settings_t *settings = config_get_ptr();
       bool video_nonblock  = enable;
 
-      if (     !settings->video.vsync 
-            || runloop_ctl(RUNLOOP_CTL_IS_NONBLOCK_FORCED, NULL))
+      if (     !settings->bools.video_vsync
+            || rarch_ctl(RARCH_CTL_IS_NONBLOCK_FORCED, NULL))
          video_nonblock = true;
       video_driver_set_nonblock_state(video_nonblock);
    }
@@ -259,7 +280,7 @@ void driver_set_nonblock_state(void)
  * driver_update_system_av_info:
  * @data               : pointer to new A/V info
  *
- * Update the system Audio/Video information. 
+ * Update the system Audio/Video information.
  * Will reinitialize audio/video drivers.
  * Used by RETRO_ENVIRONMENT_SET_SYSTEM_AV_INFO.
  *
@@ -268,6 +289,7 @@ void driver_set_nonblock_state(void)
 static bool driver_update_system_av_info(const struct retro_system_av_info *info)
 {
    struct retro_system_av_info *av_info    = video_viewport_get_system_av_info();
+   settings_t *settings = config_get_ptr();
 
    memcpy(av_info, info, sizeof(*av_info));
    command_event(CMD_EVENT_REINIT, NULL);
@@ -283,6 +305,11 @@ static bool driver_update_system_av_info(const struct retro_system_av_info *info
       command_event(CMD_EVENT_RECORD_INIT, NULL);
    }
 
+   /* Hide mouse cursor in fullscreen after 
+    * a RETRO_ENVIRONMENT_SET_SYSTEM_AV_INFO call. */
+   if (settings->bools.video_fullscreen)
+      video_driver_hide_mouse();
+
    return true;
 }
 
@@ -295,6 +322,8 @@ static bool driver_update_system_av_info(const struct retro_system_av_info *info
  **/
 void drivers_init(int flags)
 {
+   bool video_is_threaded = false;
+
    if (flags & DRIVER_VIDEO_MASK)
       video_driver_unset_own_driver();
    if (flags & DRIVER_AUDIO_MASK)
@@ -322,14 +351,14 @@ void drivers_init(int flags)
          video_driver_get_hw_context();
 
       video_driver_monitor_reset();
-      video_driver_init();
+      video_driver_init(&video_is_threaded);
 
       if (!video_driver_is_video_cache_context_ack()
             && hwr->context_reset)
          hwr->context_reset();
       video_driver_unset_video_cache_context_ack();
 
-      runloop_ctl(RUNLOOP_CTL_SET_FRAME_TIME_LAST, NULL);
+      rarch_ctl(RARCH_CTL_SET_FRAME_TIME_LAST, NULL);
    }
 
    if (flags & DRIVER_AUDIO_MASK)
@@ -349,10 +378,10 @@ void drivers_init(int flags)
    core_info_init_current_core();
 
 #ifdef HAVE_MENU
-   if (flags & DRIVER_MENU_MASK)
+   if (flags & DRIVER_VIDEO_MASK)
    {
-      menu_driver_ctl(RARCH_MENU_CTL_INIT, NULL);
-      menu_driver_ctl(RARCH_MENU_CTL_CONTEXT_RESET, NULL);
+      if (flags & DRIVER_MENU_MASK)
+         menu_driver_init(video_is_threaded);
    }
 #endif
 
@@ -362,6 +391,11 @@ void drivers_init(int flags)
       if (input_driver_is_nonblock_state())
          driver_set_nonblock_state();
    }
+
+   if (flags & DRIVER_LED_MASK)
+   {
+      led_driver_init();
+   }
 }
 
 
@@ -376,20 +410,20 @@ void drivers_init(int flags)
  **/
 
 /**
- * Driver ownership - set this to true if the platform in question needs to 'own' 
- * the respective handle and therefore skip regular RetroArch 
+ * Driver ownership - set this to true if the platform in question needs to 'own'
+ * the respective handle and therefore skip regular RetroArch
  * driver teardown/reiniting procedure.
  *
- * If  to true, the 'free' function will get skipped. It is 
- * then up to the driver implementation to properly handle 
- * 'reiniting' inside the 'init' function and make sure it 
- * returns the existing handle instead of allocating and 
+ * If  to true, the 'free' function will get skipped. It is
+ * then up to the driver implementation to properly handle
+ * 'reiniting' inside the 'init' function and make sure it
+ * returns the existing handle instead of allocating and
  * returning a pointer to a new handle.
  *
- * Typically, if a driver intends to make use of this, it should 
+ * Typically, if a driver intends to make use of this, it should
  * set this to true at the end of its 'init' function.
  **/
-static void uninit_drivers(int flags)
+void driver_uninit(int flags)
 {
    core_info_deinit_list();
    core_info_free_current_core();
@@ -405,21 +439,24 @@ static void uninit_drivers(int flags)
    if ((flags & DRIVER_CAMERA_MASK) && !camera_driver_ctl(RARCH_CAMERA_CTL_OWNS_DRIVER, NULL))
       camera_driver_ctl(RARCH_CAMERA_CTL_DEINIT, NULL);
 
-   if (flags & DRIVER_AUDIO_MASK)
-      audio_driver_deinit();
-
    if ((flags & DRIVER_WIFI_MASK) && !wifi_driver_ctl(RARCH_WIFI_CTL_OWNS_DRIVER, NULL))
       wifi_driver_ctl(RARCH_WIFI_CTL_DEINIT, NULL);
 
+   if (flags & DRIVER_LED)
+      led_driver_free();
+   
    if (flags & DRIVERS_VIDEO_INPUT)
       video_driver_free();
 
+   if (flags & DRIVER_AUDIO_MASK)
+      audio_driver_deinit();
+
    if ((flags & DRIVER_VIDEO_MASK) && !video_driver_owns_driver())
       video_driver_destroy_data();
 
    if ((flags & DRIVER_INPUT_MASK) && !input_driver_owns_driver())
       input_driver_destroy_data();
-
+   
    if ((flags & DRIVER_AUDIO_MASK) && !audio_driver_owns_driver())
       audio_driver_destroy_data();
 }
@@ -433,26 +470,13 @@ bool driver_ctl(enum driver_ctl_state state, void *data)
          audio_driver_destroy();
          input_driver_destroy();
 #ifdef HAVE_MENU
-         menu_driver_ctl(RARCH_MENU_CTL_DESTROY, NULL);
+         menu_driver_destroy();
 #endif
          location_driver_ctl(RARCH_LOCATION_CTL_DESTROY, NULL);
          camera_driver_ctl(RARCH_CAMERA_CTL_DESTROY, NULL);
          wifi_driver_ctl(RARCH_WIFI_CTL_DESTROY, NULL);
          core_uninit_libretro_callbacks();
          break;
-      case RARCH_DRIVER_CTL_UNINIT:
-         {
-            int *flags = (int*)data;
-            if (!flags)
-               return false;
-            uninit_drivers(*flags);
-         }
-         break;
-      case RARCH_DRIVER_CTL_UNINIT_ALL:
-         {
-            int flags = DRIVERS_CMD_ALL;
-            return driver_ctl(RARCH_DRIVER_CTL_UNINIT, &flags);
-         }
       case RARCH_DRIVER_CTL_INIT_PRE:
          audio_driver_find_driver();
          video_driver_find_driver();
@@ -486,6 +510,13 @@ bool driver_ctl(enum driver_ctl_state state, void *data)
                return false;
             return driver_find_first(drv->label, drv->s, drv->len);
          }
+      case RARCH_DRIVER_CTL_FIND_LAST:
+         {
+            driver_ctx_info_t *drv = (driver_ctx_info_t*)data;
+            if (!drv)
+               return false;
+            return driver_find_last(drv->label, drv->s, drv->len);
+         }
       case RARCH_DRIVER_CTL_FIND_PREV:
          {
             driver_ctx_info_t *drv = (driver_ctx_info_t*)data;
