@@ -23,9 +23,9 @@
 #include "cocoa_common.h"
 #include "../../ui_companion_driver.h"
 
-static bool ui_application_cocoa_initialize(void)
+static void* ui_application_cocoa_initialize(void)
 {
-   return true;
+   return NULL;
 }
 
 static bool ui_application_cocoa_pending_events(void)
@@ -49,9 +49,16 @@ static void ui_application_cocoa_process_events(void)
     }
 }
 
-const ui_application_t ui_application_cocoa = {
+static void ui_application_cocoa_run(void *args)
+{
+   (void)args;
+}
+
+ui_application_t ui_application_cocoa = {
    ui_application_cocoa_initialize,
    ui_application_cocoa_pending_events,
    ui_application_cocoa_process_events,
+   ui_application_cocoa_run,
+   NULL,
    "cocoa"
 };
