@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2011-2017 - Daniel De Matteis
  *  Copyright (C) 2013-2014 - Jason Fetters
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -17,16 +17,17 @@
 #include <stdint.h>
 #include <unistd.h>
 
+#include <retro_miscellaneous.h>
+
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
 #endif
 
-#include "../input_config.h"
 #include "../input_driver.h"
-
-#include "../input_joypad_driver.h"
 #include "../input_keymaps.h"
+
 #include "cocoa_input.h"
+
 #include "../../gfx/video_driver.h"
 #include "../../driver.h"
 
@@ -38,13 +39,13 @@ float get_backing_scale_factor(void);
 int32_t cocoa_input_find_any_key(void)
 {
    cocoa_input_data_t *apple = (cocoa_input_data_t*)input_driver_get_data();
-    
+
    if (!apple)
       return 0;
-    
+
    if (apple->joypad)
        apple->joypad->poll();
-    
+
     if (apple->sec_joypad)
         apple->sec_joypad->poll();
 
@@ -52,13 +53,13 @@ int32_t cocoa_input_find_any_key(void)
 }
 
 static int cocoa_input_find_any_button_ret(cocoa_input_data_t *apple,
-   unsigned buttons, unsigned port)
+   input_bits_t * state, unsigned port)
 {
    unsigned i;
 
-   if (buttons)
-      for (i = 0; i < 32; i++)
-         if (buttons & (1 << i))
+   if (state)
+      for (i = 0; i < 256; i++)
+         if (BIT256_GET_PTR(state,i))
             return i;
    return -1;
 }
@@ -70,26 +71,32 @@ int32_t cocoa_input_find_any_button(uint32_t port)
 
    if (!apple)
       return -1;
-    
+
    if (apple->joypad)
    {
        apple->joypad->poll();
 
        if (apple->joypad->get_buttons)
-          ret = cocoa_input_find_any_button_ret(apple, apple->joypad->get_buttons(port), port);
+       {
+          input_bits_t state;
+          apple->joypad->get_buttons(port,&state);
+          ret = cocoa_input_find_any_button_ret(apple, &state, port);
+       }
    }
 
    if (ret != -1)
       return ret;
-    
+
    if (apple->sec_joypad)
    {
        apple->sec_joypad->poll();
 
        if (apple->sec_joypad->get_buttons)
        {
+          input_bits_t state;
           apple->sec_joypad->poll();
-          ret = cocoa_input_find_any_button_ret(apple, apple->sec_joypad->get_buttons(port), port);
+          apple->sec_joypad->get_buttons(port,&state);
+          ret = cocoa_input_find_any_button_ret(apple, &state, port);
        }
    }
 
@@ -103,17 +110,17 @@ int32_t cocoa_input_find_any_axis(uint32_t port)
 {
    int i;
    cocoa_input_data_t *apple = (cocoa_input_data_t*)input_driver_get_data();
-    
+
    if (apple && apple->joypad)
        apple->joypad->poll();
-    
+
    if (apple && apple->sec_joypad)
        apple->sec_joypad->poll();
 
    for (i = 0; i < 6; i++)
    {
       int16_t value = apple->joypad ? apple->joypad->axis(port, i) : 0;
-      
+
       if (abs(value) > 0x4000)
          return (value < 0) ? -(i + 1) : i + 1;
 
@@ -132,15 +139,15 @@ static void *cocoa_input_init(const char *joypad_driver)
    cocoa_input_data_t *apple = (cocoa_input_data_t*)calloc(1, sizeof(*apple));
    if (!apple)
       return NULL;
-    
+
    input_keymaps_init_keyboard_lut(rarch_key_map_apple_hid);
 
    apple->joypad = input_joypad_init_driver(joypad_driver, apple);
-    
+
 #ifdef HAVE_MFI
    apple->sec_joypad = input_joypad_init_driver("mfi", apple);
 #endif
-    
+
    return apple;
 }
 
@@ -151,13 +158,21 @@ static void cocoa_input_poll(void *data)
 #ifndef IOS
    float   backing_scale_factor = get_backing_scale_factor();
 #endif
-    
+
    if (!apple)
       return;
 
    for (i = 0; i < apple->touch_count; i++)
    {
-      struct video_viewport vp = {0};
+      struct video_viewport vp;
+
+      vp.x                        = 0;
+      vp.y                        = 0;
+      vp.width                    = 0;
+      vp.height                   = 0;
+      vp.full_width               = 0;
+      vp.full_height              = 0;
+
 #ifndef IOS
       apple->touches[i].screen_x *= backing_scale_factor;
       apple->touches[i].screen_y *= backing_scale_factor;
@@ -243,13 +258,13 @@ static int16_t cocoa_pointer_state(cocoa_input_data_t *apple,
       int16_t x, y;
       const cocoa_touch_data_t *touch = (const cocoa_touch_data_t *)
          &apple->touches[idx];
-       
+
        if (!touch)
            return 0;
-       
+
       x = touch->fixed_x;
       y = touch->fixed_y;
-      
+
       if (want_full)
       {
          x = touch->full_x;
@@ -278,22 +293,21 @@ static int16_t cocoa_input_state(void *data,
    int16_t ret               = 0;
    cocoa_input_data_t *apple = (cocoa_input_data_t*)data;
 
-   if (!apple || !apple->joypad)
-      return 0;
-
    switch (device)
    {
       case RETRO_DEVICE_JOYPAD:
-         return apple_input_is_pressed(port, binds[port], id) ||
-            input_joypad_pressed(apple->joypad, joypad_info, port, binds[port], id)
+         if (id < RARCH_BIND_LIST_END)
+            ret = apple_key_state[rarch_keysym_lut[binds[port][id].key]];
+         if (!ret)
+            ret = input_joypad_pressed(apple->joypad, joypad_info, port, binds[port], id);
 #ifdef HAVE_MFI
-            || input_joypad_pressed(apple->sec_joypad, joypad_info, port, binds[port], id)
+         if (!ret)
+            ret = input_joypad_pressed(apple->sec_joypad, joypad_info, port, binds[port], id);
 #endif
-            ;
+         return ret;
       case RETRO_DEVICE_ANALOG:
 #ifdef HAVE_MFI
-         if (binds[port])
-            ret = input_joypad_analog(apple->sec_joypad, joypad_info, port,
+         ret = input_joypad_analog(apple->sec_joypad, joypad_info, port,
                idx, id, binds[port]);
 #endif
          if (!ret && binds[port])
@@ -301,11 +315,11 @@ static int16_t cocoa_input_state(void *data,
                   idx, id, binds[port]);
          return ret;
       case RETRO_DEVICE_KEYBOARD:
-         return apple_keyboard_state(id);
+         return (id < RETROK_LAST) && apple_key_state[rarch_keysym_lut[(enum retro_key)id]];
       case RETRO_DEVICE_MOUSE:
          return cocoa_mouse_state(apple, id);
-       case RARCH_DEVICE_MOUSE_SCREEN:
-           return cocoa_mouse_state_screen(apple, id);
+      case RARCH_DEVICE_MOUSE_SCREEN:
+         return cocoa_mouse_state_screen(apple, id);
       case RETRO_DEVICE_POINTER:
       case RARCH_DEVICE_POINTER_SCREEN:
          return cocoa_pointer_state(apple, device, idx, id);
@@ -314,25 +328,23 @@ static int16_t cocoa_input_state(void *data,
    return 0;
 }
 
-static bool cocoa_input_meta_key_pressed(void *data, int key)
-{
-   return false;
-}
-
 static void cocoa_input_free(void *data)
 {
+   unsigned i;
    cocoa_input_data_t *apple = (cocoa_input_data_t*)data;
-    
+
    if (!apple || !data)
       return;
-    
+
    if (apple->joypad)
       apple->joypad->destroy();
-    
+
    if (apple->sec_joypad)
        apple->sec_joypad->destroy();
-    
-   apple_keyboard_free();
+
+   for (i = 0; i < MAX_KEYS; i++)
+      apple_key_state[i] = 0;
+
    free(apple);
 }
 
@@ -340,7 +352,7 @@ static bool cocoa_input_set_rumble(void *data,
    unsigned port, enum retro_rumble_effect effect, uint16_t strength)
 {
    cocoa_input_data_t *apple = (cocoa_input_data_t*)data;
-    
+
    if (apple && apple->joypad)
       return input_joypad_set_rumble(apple->joypad,
             port, effect, strength);
@@ -356,7 +368,7 @@ static uint64_t cocoa_input_get_capabilities(void *data)
 {
    (void)data;
 
-   return 
+   return
       (1 << RETRO_DEVICE_JOYPAD)   |
       (1 << RETRO_DEVICE_MOUSE)    |
       (1 << RETRO_DEVICE_KEYBOARD) |
@@ -374,7 +386,7 @@ static void cocoa_input_grab_mouse(void *data, bool state)
 static const input_device_driver_t *cocoa_input_get_sec_joypad_driver(void *data)
 {
    cocoa_input_data_t *apple = (cocoa_input_data_t*)data;
-    
+
    if (apple && apple->sec_joypad)
       return apple->sec_joypad;
    return NULL;
@@ -383,7 +395,7 @@ static const input_device_driver_t *cocoa_input_get_sec_joypad_driver(void *data
 static const input_device_driver_t *cocoa_input_get_joypad_driver(void *data)
 {
    cocoa_input_data_t *apple = (cocoa_input_data_t*)data;
-    
+
    if (apple && apple->joypad)
       return apple->joypad;
    return NULL;
@@ -409,7 +421,6 @@ input_driver_t input_cocoa = {
    cocoa_input_init,
    cocoa_input_poll,
    cocoa_input_state,
-   cocoa_input_meta_key_pressed,
    cocoa_input_free,
    NULL,
    NULL,
