@@ -37,8 +37,9 @@
 #endif
 
 #include "../font_driver.h"
-#include "../video_context_driver.h"
 #include "../../retroarch.h"
+#include "../../verbosity.h"
+#include "../common/drm_common.h"
 
 #include "drm_pixformats.h"
 
@@ -68,18 +69,18 @@ struct drm_rect
 
 struct drm_page
 {
-   struct modeset_buf buf;	
+   struct modeset_buf buf;
    bool used;
 
    /* Each page has it's own mutex for
     * isolating it's used flag access. */
    slock_t *page_used_mutex;
 
-   /* This field will allow us to access the 
+   /* This field will allow us to access the
     * main _dispvars struct from the vsync CB function */
-   struct drm_video *drmvars;	
+   struct drm_video *drmvars;
 
-   /* This field will allow us to access the 
+   /* This field will allow us to access the
     * surface the page belongs to. */
    struct drm_surface *surface;
 };
@@ -93,18 +94,18 @@ struct drm_surface
 
    /* the page that's currently on screen */
    struct drm_page *current_page;
-   unsigned int bpp;   
+   unsigned int bpp;
    uint32_t pixformat;
-   
+
    /* The internal buffers size. */
    int src_width;
    int src_height;
 
-   /* Surfaces with a higher layer will be on top of 
+   /* Surfaces with a higher layer will be on top of
     * the ones with lower. Default is 0. */
    int layer;
 
-   /* We need to keep this value for the blitting on 
+   /* We need to keep this value for the blitting on
     * the surface_update function. */
    int pitch;
    int total_pitch;
@@ -123,7 +124,7 @@ struct drm_struct
 
    drmModeCrtcPtr orig_crtc;
 
-   uint32_t plane_id;	
+   uint32_t plane_id;
    uint32_t plane_fb_prop_id;
 
    drmModeEncoder *encoder;
@@ -136,13 +137,13 @@ struct drm_video
    struct drm_surface *main_surface;
    struct drm_surface *menu_surface;
 
-   /* Total dispmanx video dimensions. 
+   /* Total dispmanx video dimensions.
     * Not counting overscan settings. */
    unsigned int kms_width;
    unsigned int kms_height;
 
    /* For threading */
-   scond_t *vsync_condition;	
+   scond_t *vsync_condition;
    slock_t *vsync_cond_mutex;
    slock_t *pending_mutex;
 
@@ -151,13 +152,13 @@ struct drm_video
 
    bool rgb32;
 
-   /* We use this to keep track of internal resolution changes 
+   /* We use this to keep track of internal resolution changes
     * done by cores in the main surface or in the menu.
     * We need these outside the surface because we free surfaces
     * and then we want to test if these values have changed before
     * recreating them. */
-   int core_width; 
-   int core_height; 
+   int core_width;
+   int core_height;
    int core_pitch;
    /* Both main and menu surfaces are going to have the same aspect,
     * so we keep it here for future reference. */
@@ -172,15 +173,15 @@ static int modeset_create_dumbfb(int fd,
 
 static void deinit_drm(void)
 {
-   /* Restore the original videomode/connector/scanoutbuffer(fb) 
-    * combination (the original CRTC, that is). */ 
+   /* Restore the original videomode/connector/scanoutbuffer(fb)
+    * combination (the original CRTC, that is). */
    drmModeSetCrtc(drm.fd, drm.orig_crtc->crtc_id,
          drm.orig_crtc->buffer_id,
          drm.orig_crtc->x, drm.orig_crtc->y,
-         &drm.connector_id, 1, &drm.orig_crtc->mode);	
+         &drm.connector_id, 1, &drm.orig_crtc->mode);
 
 #if 0
-   /* TODO: Free surfaces here along 
+   /* TODO: Free surfaces here along
     * with their pages (framebuffers)! */
 
    if (bufs[0].fb_id)
@@ -193,12 +194,12 @@ static void deinit_drm(void)
 
 static void drm_surface_free(void *data, struct drm_surface **sp)
 {
-   int i;	
+   int i;
    struct drm_video *_drmvars = data;
    struct drm_surface *surface = *sp;
 
    for (i = 0; i < surface->numpages; i++)
-      surface->pages[i].used = false;   
+      surface->pages[i].used = false;
 
    free(surface->pages);
 
@@ -212,9 +213,9 @@ static void drm_surface_set_aspect(struct drm_surface *surface, float aspect)
 	surface->aspect = aspect;
 }
 
-static void drm_surface_setup(void *data,  int src_width, int src_height, 
+static void drm_surface_setup(void *data,  int src_width, int src_height,
       int pitch, int bpp, uint32_t pixformat,
-      int alpha, float aspect, int numpages, int layer, 
+      int alpha, float aspect, int numpages, int layer,
       struct drm_surface **sp)
 {
    struct drm_video *_drmvars = data;
@@ -223,23 +224,23 @@ static void drm_surface_setup(void *data,  int src_width, int src_height,
 
    *sp = calloc (1, sizeof(struct drm_surface));
 
-   surface = *sp;   
+   surface = *sp;
 
    /* Setup surface parameters */
    surface->numpages = numpages;
-   /* We receive the total pitch, including things that are 
-    * between scanlines and we calculate the visible pitch 
-    * from the visible width. 
+   /* We receive the total pitch, including things that are
+    * between scanlines and we calculate the visible pitch
+    * from the visible width.
     *
     * These will be used to increase the offsets for blitting. */
    surface->total_pitch = pitch;
-   surface->pitch       = src_width * bpp; 
+   surface->pitch       = src_width * bpp;
    surface->bpp         = bpp;
    surface->pixformat   = pixformat;
    surface->src_width   = src_width;
    surface->src_height  = src_height;
    surface->aspect      = aspect;
-   
+
    /* Allocate memory for all the pages in each surface
     * and initialize variables inside each page's struct. */
    surface->pages = (struct drm_page*)
@@ -247,10 +248,10 @@ static void drm_surface_setup(void *data,  int src_width, int src_height,
 
    for (i = 0; i < surface->numpages; i++)
    {
-      surface->pages[i].used            = false;   
-      surface->pages[i].surface         = surface;   
-      surface->pages[i].drmvars         = _drmvars;   
-      surface->pages[i].page_used_mutex = slock_new(); 
+      surface->pages[i].used            = false;
+      surface->pages[i].surface         = surface;
+      surface->pages[i].drmvars         = _drmvars;
+      surface->pages[i].page_used_mutex = slock_new();
    }
 
    /* Create the framebuffer for each one of the pages of the surface. */
@@ -259,7 +260,7 @@ static void drm_surface_setup(void *data,  int src_width, int src_height,
       surface->pages[i].buf.width  = src_width;
       surface->pages[i].buf.height = src_height;
       int ret                      = modeset_create_dumbfb(
-            drm.fd, &surface->pages[i].buf, bpp, pixformat);	
+            drm.fd, &surface->pages[i].buf, bpp, pixformat);
 
       if (ret)
       {
@@ -272,20 +273,20 @@ static void drm_surface_setup(void *data,  int src_width, int src_height,
 
 static void drm_page_flip(struct drm_surface *surface)
 {
-   /* We alredy have the id of the FB_ID property of 
+   /* We alredy have the id of the FB_ID property of
     * the plane on which we are going to do a pageflip:
     * we got it back in drm_plane_setup()  */
    int ret;
    static drmModeAtomicReqPtr req = NULL;
 
-   req = drmModeAtomicAlloc();	
+   req = drmModeAtomicAlloc();
 
-   /* We add the buffer to the plane properties we want to 
-    * set on an atomically, in a single step. 
+   /* We add the buffer to the plane properties we want to
+    * set on an atomically, in a single step.
     * We pass the plane id, the property id and the new fb id. */
-   ret = drmModeAtomicAddProperty(req, 
-         drm.plane_id, 
-         drm.plane_fb_prop_id, 
+   ret = drmModeAtomicAddProperty(req,
+         drm.plane_id,
+         drm.plane_fb_prop_id,
          surface->pages[surface->flip_page].buf.fb_id);
 
    if (ret < 0)
@@ -294,9 +295,9 @@ static void drm_page_flip(struct drm_surface *surface)
    }
    /*... now we just need to do the commit */
 
-   /* REMEMBER!!! The DRM_MODE_PAGE_FLIP_EVENT flag asks the kernel 
-    * to send you an event to the drm.fd once the 
-    * pageflip is complete. If you don't want -12 errors 
+   /* REMEMBER!!! The DRM_MODE_PAGE_FLIP_EVENT flag asks the kernel
+    * to send you an event to the drm.fd once the
+    * pageflip is complete. If you don't want -12 errors
     * (ENOMEM), namely "Cannot allocate memory", then
     * you must drain the event queue of that fd. */
    ret = drmModeAtomicCommit(drm.fd, req, 0, NULL);
@@ -324,7 +325,7 @@ static void drm_surface_update(void *data, const void *frame,
    for (line = 0; line < surface->src_height; line++)
    {
       memcpy (
-            surface->pages[surface->flip_page].buf.map + dst_offset, 
+            surface->pages[surface->flip_page].buf.map + dst_offset,
             (uint8_t*)frame + src_offset,
             surface->pitch);
       src_offset += surface->total_pitch;
@@ -342,7 +343,7 @@ static uint32_t get_plane_prop_id(uint32_t obj_id, const char *name)
    int i,j;
    drmModePlaneRes *plane_resources;
    drmModePlane *plane;
-   drmModeObjectProperties *props;	
+   drmModeObjectProperties *props;
    drmModePropertyRes **props_info;
 
    char format_str[5];
@@ -354,16 +355,16 @@ static uint32_t get_plane_prop_id(uint32_t obj_id, const char *name)
       if (plane->plane_id != obj_id)
          continue;
 
-      /* TODO: Improvement. We get all the properties of the 
+      /* TODO: Improvement. We get all the properties of the
        * plane and info about the properties.
-       * We should have done this already... 
+       * We should have done this already...
        * This implementation must be improved. */
       props      = drmModeObjectGetProperties(drm.fd,
             plane->plane_id, DRM_MODE_OBJECT_PLANE);
       props_info = malloc(props->count_props * sizeof *props_info);
 
       for (j = 0; j < props->count_props; ++j)
-         props_info[j] =	drmModeGetProperty(drm.fd, props->props[j]);	
+         props_info[j] =	drmModeGetProperty(drm.fd, props->props[j]);
 
       /* We look for the prop_id we need */
       for (j = 0; j < props->count_props; j++)
@@ -405,30 +406,35 @@ static bool format_support(const drmModePlanePtr ovr, uint32_t fmt)
 static uint64_t drm_plane_type(drmModePlane *plane)
 {
    int i,j;
-   /* The property values and their names are stored in different arrays, so we
-    * access them simultaneously here.
-    * We are interested in OVERLAY planes only, that's type 0 or DRM_PLANE_TYPE_OVERLAY
+
+   /* The property values and their names are stored in different arrays, 
+    * so we access them simultaneously here.
+    * We are interested in OVERLAY planes only, that's 
+    * type 0 or DRM_PLANE_TYPE_OVERLAY
     * (see /usr/xf86drmMode.h for definition). */
-   drmModeObjectPropertiesPtr props = 
-      drmModeObjectGetProperties(drm.fd, plane->plane_id, DRM_MODE_OBJECT_PLANE);
+   drmModeObjectPropertiesPtr props =
+      drmModeObjectGetProperties(drm.fd, plane->plane_id,
+            DRM_MODE_OBJECT_PLANE);
 
    for (j = 0; j < props->count_props; j++)
    {
       /* found the type property */
-      if (string_is_equal(drmModeGetProperty(drm.fd, props->props[j])->name, "type"))
+      if (string_is_equal(
+               drmModeGetProperty(drm.fd, props->props[j])->name, "type"))
          return (props->prop_values[j]);
    }
+
    return (0);
 }
 
-/* This configures our only overlay plane to render the given surface. */ 
+/* This configures our only overlay plane to render the given surface. */
 static void drm_plane_setup(struct drm_surface *surface)
 {
    int i,j;
    char fmt_name[5];
 
    /* Get plane resources */
-   drmModePlane *plane;	
+   drmModePlane *plane;
    drmModePlaneRes *plane_resources;
    plane_resources = drmModeGetPlaneResources(drm.fd);
    if (!plane_resources)
@@ -437,14 +443,14 @@ static void drm_plane_setup(struct drm_surface *surface)
    }
 
    RARCH_LOG ("DRM: Number of planes on FD %d is %d\n",
-         drm.fd, plane_resources->count_planes);	
+         drm.fd, plane_resources->count_planes);
 
-   /* dump_planes(drm.fd); */	
+   /* dump_planes(drm.fd); */
 
-   /* Look for a plane/overlay we can use with the configured CRTC	
+   /* Look for a plane/overlay we can use with the configured CRTC
     * Find a  plane which can be connected to our CRTC. Find the
     * CRTC index first, then iterate over available planes.
-    * Yes, strangely we need the in-use CRTC index to mask possible_crtc 
+    * Yes, strangely we need the in-use CRTC index to mask possible_crtc
     * during the planes iteration... */
    unsigned int crtc_index = 0;
    for (i = 0; i < (unsigned int)drm.resources->count_crtcs; i++)
@@ -457,8 +463,8 @@ static void drm_plane_setup(struct drm_surface *surface)
       }
    }
 
-   /* Programmer!! Save your sanity!! Primary planes have to 
-    * cover the entire CRTC, and if you don't do that, you 
+   /* Programmer!! Save your sanity!! Primary planes have to
+    * cover the entire CRTC, and if you don't do that, you
     * will get dmesg error "Plane must cover entire CRTC".
     *
     * Look at linux/source/drivers/gpu/drm/drm_plane_helper.c comments for more info.
@@ -473,20 +479,20 @@ static void drm_plane_setup(struct drm_surface *surface)
          continue;
       }
 
-      /* We are only interested in overlay planes. No overlay, no fun. 
+      /* We are only interested in overlay planes. No overlay, no fun.
        * (no scaling, must cover crtc..etc) so we skip primary planes */
       if (drm_plane_type(plane) != DRM_PLANE_TYPE_OVERLAY)
       {
          RARCH_LOG ("DRM: plane with ID %d is not an overlay. May be primary or cursor. Not usable.\n",
                plane->plane_id);
          continue;
-      }	
+      }
 
       if (!format_support(plane, surface->pixformat))
       {
          RARCH_LOG ("DRM: plane with ID %d does not support framebuffer format\n", plane->plane_id);
          continue;
-      }         
+      }
 
       drm.plane_id = plane->plane_id;
       drmModeFreePlane(plane);
@@ -504,11 +510,11 @@ static void drm_plane_setup(struct drm_surface *surface)
    }
 
    /* We are going to be changing the framebuffer ID property of the chosen overlay every time
-    * we do a pageflip, so we get the property ID here to have it handy on the PageFlip function. */	
+    * we do a pageflip, so we get the property ID here to have it handy on the PageFlip function. */
    drm.plane_fb_prop_id = get_plane_prop_id(drm.plane_id, "FB_ID");
    if (!drm.plane_fb_prop_id)
    {
-      RARCH_LOG("DRM: Can't get the FB property ID for plane(%u)\n", drm.plane_id);
+      RARCH_LOG("[DRM]: Can't get the FB property ID for plane(%u)\n", drm.plane_id);
    }
 
    /* Note src coords (last 4 args) are in Q16 format
@@ -531,22 +537,22 @@ static void drm_plane_setup(struct drm_surface *surface)
    uint32_t src_x = 0;
    uint32_t src_y = 0;
 
-   /* We have to set a buffer for the plane, whatever buffer we want, 
+   /* We have to set a buffer for the plane, whatever buffer we want,
     * but we must set a buffer so the plane starts reading from it now. */
-   if (drmModeSetPlane(drm.fd, drm.plane_id, drm.crtc_id, 
+   if (drmModeSetPlane(drm.fd, drm.plane_id, drm.crtc_id,
             surface->pages[surface->flip_page].buf.fb_id,
             plane_flags, plane_x, plane_y, plane_w, plane_h,
             src_x<<16, src_y<<16, src_w<<16, src_h<<16))
    {
-      RARCH_ERR("DRM: failed to enable plane: %s\n", strerror(errno));	
+      RARCH_ERR("[DRM]: failed to enable plane: %s\n", strerror(errno));
    }
 
-   RARCH_LOG("DRM: src_w %d, src_h %d, plane_w %d, plane_h %d\n",
+   RARCH_LOG("[DRM]: src_w %d, src_h %d, plane_w %d, plane_h %d\n",
          src_w, src_h, plane_w, plane_h);
 
    /* Report what plane (of overlay type) we're using. */
    drm_format_name(surface->pixformat, fmt_name);
-   RARCH_LOG("DRM: Using plane with ID %d on CRTC ID %d format %s\n",
+   RARCH_LOG("[DRM]: Using plane with ID %d on CRTC ID %d format %s\n",
          drm.plane_id, drm.crtc_id, fmt_name);
 }
 
@@ -615,7 +621,7 @@ static bool init_drm(void)
     * VERY important or we won't get all the available planes on drmGetPlaneResources()!
     * We also need to enable the ATOMIC cap to see the atomic properties in objects!! */
    ret = drmSetClientCap(drm.fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 1);
-   if (ret) 
+   if (ret)
       RARCH_ERR ("DRM: can't set UNIVERSAL PLANES cap.\n");
    else
       RARCH_LOG ("DRM: UNIVERSAL PLANES cap set\n");
@@ -623,8 +629,8 @@ static bool init_drm(void)
 
    ret = drmSetClientCap(drm.fd, DRM_CLIENT_CAP_ATOMIC, 1);
    if (ret)
-   { 
-      /*If this happens, check kernel support and kernel parameters 
+   {
+      /*If this happens, check kernel support and kernel parameters
        * (add i915.nuclear_pageflip=y to the kernel boot line for example) */
       RARCH_ERR ("DRM: can't set ATOMIC caps: %s\n", strerror(errno));
    }
@@ -674,16 +680,17 @@ static bool init_drm(void)
    drm.crtc_id = drm.encoder->crtc_id;
    drm.connector_id = connector->connector_id;
 
-   /* Backup original crtc and it's mode, so we can restore the original video mode 
+   /* Backup original crtc and it's mode, so we can restore the original video mode
     * on exit in case we change it. */
    drm.orig_crtc = drmModeGetCrtc(drm.fd, drm.encoder->crtc_id);
    drm.current_mode = &(drm.orig_crtc->mode);
+   g_drm_mode = drm.current_mode;
 
    /* Set mode physical video mode. Not really needed, but clears TTY console. */
    struct modeset_buf buf;
    buf.width = drm.current_mode->hdisplay;
    buf.height = drm.current_mode->vdisplay;
-   ret = modeset_create_dumbfb(drm.fd, &buf, 4, DRM_FORMAT_XRGB8888);	
+   ret = modeset_create_dumbfb(drm.fd, &buf, 4, DRM_FORMAT_XRGB8888);
    if (ret)
    {
       RARCH_ERR ("DRM: can't create dumb fb\n");
@@ -692,7 +699,7 @@ static bool init_drm(void)
    if (drmModeSetCrtc(drm.fd, drm.crtc_id, buf.fb_id, 0, 0,
             &drm.connector_id, 1, drm.current_mode))
    {
-      RARCH_ERR ("DRM: failed to set mode\n");	
+      RARCH_ERR ("DRM: failed to set mode\n");
       return false;
    }
 
@@ -710,11 +717,11 @@ static void *drm_gfx_init(const video_info_t *video,
 
    /* Setup surface parameters */
    _drmvars->menu_active      = false;
-   _drmvars->rgb32            = video->rgb32; 
+   _drmvars->rgb32            = video->rgb32;
 
-   /* It's very important that we set aspect here because the 
+   /* It's very important that we set aspect here because the
     * call seq when a core is loaded is gfx_init()->set_aspect()->gfx_frame()
-    * and we don't want the main surface to be setup in set_aspect() 
+    * and we don't want the main surface to be setup in set_aspect()
     * before we get to gfx_frame(). */
    _drmvars->current_aspect = video_driver_get_aspect_ratio();
 
@@ -743,9 +750,9 @@ static void *drm_gfx_init(const video_info_t *video,
       RARCH_LOG ("DRM: Init successful.\n");
    }
 
-   _drmvars->kms_width  = drm.current_mode->hdisplay;  
-   _drmvars->kms_height = drm.current_mode->vdisplay;  
- 
+   _drmvars->kms_width  = drm.current_mode->hdisplay;
+   _drmvars->kms_height = drm.current_mode->vdisplay;
+
    return _drmvars;
 }
 
@@ -755,7 +762,7 @@ static bool drm_gfx_frame(void *data, const void *frame, unsigned width,
 {
    struct drm_video *_drmvars = data;
 
-   if (  ( width != _drmvars->core_width) || 
+   if (  ( width != _drmvars->core_width) ||
          (height != _drmvars->core_height))
    {
       /* Sanity check. */
@@ -766,22 +773,22 @@ static bool drm_gfx_frame(void *data, const void *frame, unsigned width,
       _drmvars->core_height = height;
       _drmvars->core_pitch  = pitch;
 
-      if (_drmvars->main_surface != NULL) 
+      if (_drmvars->main_surface != NULL)
          drm_surface_free(_drmvars, &_drmvars->main_surface);
 
       /* We need to recreate the main surface and it's pages (buffers). */
-      drm_surface_setup(_drmvars, 
-            width, 
-            height, 
-            pitch, 
+      drm_surface_setup(_drmvars,
+            width,
+            height,
+            pitch,
             _drmvars->rgb32 ? 4 : 2,
             _drmvars->rgb32 ? DRM_FORMAT_XRGB8888 : DRM_FORMAT_RGB565,
 	    255,
-            _drmvars->current_aspect, 
+            _drmvars->current_aspect,
             3,
             0,
             &_drmvars->main_surface);
-    
+
       /* We need to change the plane to read from the main surface */
       drm_plane_setup(_drmvars->main_surface);
    }
@@ -820,34 +827,34 @@ static void drm_set_texture_frame(void *data, const void *frame, bool rgb32,
    if (!_drmvars->menu_active)
       return;
 
-   /* If menu is active in this frame but the 
+   /* If menu is active in this frame but the
     * menu surface is NULL, we allocate a new one.*/
    if (!_drmvars->menu_surface)
    {
-      drm_surface_setup(_drmvars, 
-            width, 
-            height, 
-            width * 4, 
+      drm_surface_setup(_drmvars,
+            width,
+            height,
+            width * 4,
             4,
             DRM_FORMAT_XRGB8888,
             210,
-            _drmvars->current_aspect, 
+            _drmvars->current_aspect,
             2,
             0,
             &_drmvars->menu_surface);
 
-      /* We need to re-setup the ONLY plane as the setup 
+      /* We need to re-setup the ONLY plane as the setup
        * depends on input buffers dimensions. */
       drm_plane_setup(_drmvars->menu_surface);
    }
 
-   /* We have to go on a pixel format conversion adventure 
-    * for now, until we can convince RGUI to output 
+   /* We have to go on a pixel format conversion adventure
+    * for now, until we can convince RGUI to output
     * in an 8888 format. */
-   unsigned int src_pitch        = width * 2; 
+   unsigned int src_pitch        = width * 2;
    unsigned int dst_pitch        = width * 4;
    unsigned int dst_width        = width;
-   uint32_t line[dst_width]; 
+   uint32_t line[dst_width];
 
    /* The output pixel array with the converted pixels. */
    char *frame_output = (char *) malloc (dst_pitch * height);
@@ -859,12 +866,12 @@ static void drm_set_texture_frame(void *data, const void *frame, bool rgb32,
    {
       for (j = 0; j < src_pitch / 2; j++)
       {
-         uint16_t src_pix = *((uint16_t*)frame + (src_pitch / 2 * i) + j); 
+         uint16_t src_pix = *((uint16_t*)frame + (src_pitch / 2 * i) + j);
          /* The hex AND is for keeping only the part we need for each component. */
          uint32_t R = (src_pix << 8) & 0x00FF0000;
          uint32_t G = (src_pix << 4) & 0x0000FF00;
          uint32_t B = (src_pix << 0) & 0x000000FF;
-         line[j] = (0 | R | G | B); 
+         line[j] = (0 | R | G | B);
       }
       memcpy(dst_base_addr + (dst_pitch * i), (char*)line, dst_pitch);
    }
@@ -916,13 +923,6 @@ static bool drm_gfx_suppress_screensaver(void *data, bool enable)
    return false;
 }
 
-static bool drm_gfx_has_windowed(void *data)
-{
-   (void)data;
-
-   return false;
-}
-
 static bool drm_gfx_set_shader(void *data,
       enum rarch_shader_type type, const char *path)
 {
@@ -930,7 +930,7 @@ static bool drm_gfx_set_shader(void *data,
    (void)type;
    (void)path;
 
-   return false; 
+   return false;
 }
 
 static void drm_gfx_set_rotation(void *data, unsigned rotation)
@@ -947,28 +947,32 @@ static bool drm_gfx_read_viewport(void *data, uint8_t *buffer, bool is_idle)
    return true;
 }
 
-static void drm_set_aspect_ratio (void *data, unsigned aspect_ratio_idx) 
+static void drm_set_aspect_ratio (void *data, unsigned aspect_ratio_idx)
 {
    struct drm_video *_drmvars = data;
    /* Here we obtain the new aspect ratio. */
    float new_aspect = aspectratio_lut[aspect_ratio_idx].value;
 
    if (_drmvars->current_aspect != new_aspect)
-   { 
+   {
       _drmvars->current_aspect = new_aspect;
       drm_surface_set_aspect(_drmvars->main_surface, new_aspect);
       if (_drmvars->menu_active)
       {
          drm_surface_set_aspect(_drmvars->menu_surface, new_aspect);
-         drm_plane_setup(_drmvars->menu_surface); 
+         drm_plane_setup(_drmvars->menu_surface);
       }
    }
 }
 
 static const video_poke_interface_t drm_poke_interface = {
+   NULL, /* get_flags */
+   NULL, /* set_coords */
+   NULL, /* set_mvp    */
    NULL,
    NULL,
    NULL, /* set_video_mode */
+   drm_get_refresh_rate,
    NULL, /* set_filtering */
    NULL, /* get_video_output_size */
    NULL, /* get_video_output_prev */
@@ -977,12 +981,14 @@ static const video_poke_interface_t drm_poke_interface = {
    NULL, /* get_proc_address */
    drm_set_aspect_ratio,
    NULL, /* drm_apply_state_changes */
-#ifdef HAVE_MENU
    drm_set_texture_frame,
    drm_set_texture_enable,
-#endif
-   NULL, /* drm_set_osd_msg */
-   NULL  /* drm_show_mouse */
+   NULL,                         /* drm_set_osd_msg */
+   NULL,                         /* drm_show_mouse */
+   NULL,                         /* grab_mouse_toggle */
+   NULL,                         /* get_current_shader */
+   NULL,                         /* get_current_software_framebuffer */
+   NULL                          /* get_hw_render_interface */
 };
 
 static void drm_gfx_get_poke_interface(void *data,
@@ -1001,13 +1007,15 @@ static void drm_gfx_free(void *data)
 
    drm_surface_free(_drmvars, &_drmvars->main_surface);
 
-   if (_drmvars->menu_surface) 
+   if (_drmvars->menu_surface)
       drm_surface_free(_drmvars, &_drmvars->menu_surface);
 
    /* Destroy mutexes and conditions. */
    slock_free(_drmvars->pending_mutex);
    slock_free(_drmvars->vsync_cond_mutex);
-   scond_free(_drmvars->vsync_condition);		
+   scond_free(_drmvars->vsync_condition);
+
+   g_drm_mode = NULL;
 
    free(_drmvars);
 }
@@ -1019,7 +1027,7 @@ video_driver_t video_drm = {
    drm_gfx_alive,
    drm_gfx_focus,
    drm_gfx_suppress_screensaver,
-   drm_gfx_has_windowed,
+   NULL, /* has_windowed */
    drm_gfx_set_shader,
    drm_gfx_free,
    "drm",
