@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -21,8 +21,8 @@
 
 #ifndef _XBOX
 #include <windows.h>
-#include <mmreg.h>
 #include <mmsystem.h>
+#include <mmreg.h>
 #endif
 
 #include <dsound.h>
@@ -31,6 +31,7 @@
 
 #include <retro_inline.h>
 #include <retro_miscellaneous.h>
+#include <retro_timers.h>
 #include <rthreads/rthreads.h>
 #include <queues/fifo_queue.h>
 
@@ -92,7 +93,7 @@ static INLINE bool grab_region(dsound_t *ds, uint32_t write_ptr,
       struct audio_lock *region)
 {
    const char *err = NULL;
-   HRESULT     res = IDirectSoundBuffer_Lock(ds->dsb, write_ptr, CHUNK_SIZE, 
+   HRESULT     res = IDirectSoundBuffer_Lock(ds->dsb, write_ptr, CHUNK_SIZE,
          &region->chunk1, &region->size1, &region->chunk2, &region->size2, 0);
 
    if (res == DSERR_BUFFERLOST)
@@ -151,7 +152,7 @@ static void dsound_thread(void *data)
       struct audio_lock region;
       DWORD read_ptr, avail, fifo_avail;
       get_positions(ds, &read_ptr, NULL);
-      
+
       avail = write_avail(read_ptr, write_ptr, ds->buffer_size);
 
       EnterCriticalSection(&ds->crit);
@@ -160,12 +161,12 @@ static void dsound_thread(void *data)
 
       if (avail < CHUNK_SIZE || ((fifo_avail < CHUNK_SIZE) && (avail < ds->buffer_size / 2)))
       {
-         /* No space to write, or we don't have data in our fifo, 
+         /* No space to write, or we don't have data in our fifo,
           * but we can wait some time before it underruns ... */
 
 
          /* We could opt for using the notification interface,
-          * but it is not guaranteed to work, so use high 
+          * but it is not guaranteed to work, so use high
           * priority sleeping patterns.
           */
          retro_sleep(1);
@@ -181,7 +182,7 @@ static void dsound_thread(void *data)
 
       if (fifo_avail < CHUNK_SIZE)
       {
-         /* Got space to write, but nothing in FIFO (underrun), 
+         /* Got space to write, but nothing in FIFO (underrun),
           * fill block with silence. */
 
          memset(region.chunk1, 0, region.size1);
@@ -190,7 +191,7 @@ static void dsound_thread(void *data)
          release_region(ds, &region);
          write_ptr = (write_ptr + region.size1 + region.size2) % ds->buffer_size;
       }
-      else 
+      else
       {
          /* All is good. Pull from it and notify FIFO. */
 
@@ -321,7 +322,11 @@ static void *dsound_init(const char *device, unsigned rate, unsigned latency,
 
    RARCH_LOG("DirectSound devices:\n");
 #ifndef _XBOX
-   DirectSoundEnumerate(enumerate_cb, &dev);
+#ifdef UNICODE
+   DirectSoundEnumerate((LPDSENUMCALLBACKW)enumerate_cb, &dev);
+#else
+   DirectSoundEnumerate((LPDSENUMCALLBACKA)enumerate_cb, &dev);
+#endif
 #endif
 
    if (DirectSoundCreate(dev.guid, &ds->ds, NULL) != DS_OK)
@@ -426,8 +431,7 @@ static void dsound_set_nonblock_state(void *data, bool state)
       ds->nonblock = state;
 }
 
-static ssize_t dsound_write(void *data, const void *buf_, size_t size,
-      bool is_perfcnt_enable)
+static ssize_t dsound_write(void *data, const void *buf_, size_t size)
 {
    size_t     written = 0;
    dsound_t       *ds = (dsound_t*)data;
