@@ -1,6 +1,6 @@
 #include "shaders_common.h"
 
-static const char *stock_vertex_xmb_legacy = GLSL(
+static const char *stock_vertex_xmb_ribbon_legacy = GLSL(
    attribute vec3 VertexCoord;
    uniform float time;
    varying vec3 fragVertexEc;
@@ -24,7 +24,7 @@ static const char *stock_vertex_xmb_legacy = GLSL(
 
    float xmb_noise2( vec3 x )
    {
-     return cos((x.z*1.0)*2.0);
+     return cos(x.z*4.0)*cos(x.z+time/10.0+x.x);
    }
 
    void main()
@@ -33,16 +33,16 @@ static const char *stock_vertex_xmb_legacy = GLSL(
      vec3 v2 = v;
      vec3 v3 = v;
 
-     v.y = xmb_noise2(v2)/6.0;
+     v.y = xmb_noise2(v2)/8.0;
 
      v3.x = v3.x + time/5.0;
-     v3.x = v3.x / 2.0;
+     v3.x = v3.x / 4.0;
 
      v3.z = v3.z + time/10.0;
      v3.y = v3.y + time/100.0;
 
      v.z = v.z + noise(v3*7.0)/15.0;
-     v.y = v.y + noise(v3*7.0)/15.0 + cos(v.x*2.0-time/5.0)/5.0 - 0.3;
+     v.y = v.y + noise(v3*7.0)/15.0 + cos(v.x*2.0-time/2.0)/5.0 - 0.3;
 
      gl_Position = vec4(v, 1.0);
      fragVertexEc = gl_Position.xyz;
