@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -83,12 +83,13 @@ static void *gx_audio_init(const char *device,
    AIInit(NULL);
    AIRegisterDMACallback(dma_callback);
 
-   if (rate < 33000)
+   //ranges 0-32000 (default low) and 40000-47999 (in settings going down from 48000) -> set to 32000 hz
+   if (rate <= 32000 || (rate >= 40000 && rate < 48000))
    {
       AISetDSPSampleRate(AI_SAMPLERATE_32KHZ);
       *new_rate = 32000;
    }
-   else
+   else //ranges 32001-39999 (in settings going up from 32000) and 48000-max (default high) -> set to 48000 hz
    {
       AISetDSPSampleRate(AI_SAMPLERATE_48KHZ);
       *new_rate = 48000;
@@ -116,8 +117,7 @@ static INLINE void copy_swapped(uint32_t * restrict dst,
    }while(--size);
 }
 
-static ssize_t gx_audio_write(void *data, const void *buf_, size_t size,
-      bool is_perfcnt_enable)
+static ssize_t gx_audio_write(void *data, const void *buf_, size_t size)
 {
    size_t       frames = size >> 2;
    const uint32_t *buf = buf_;
@@ -130,9 +130,9 @@ static ssize_t gx_audio_write(void *data, const void *buf_, size_t size,
       if (frames < to_write)
          to_write = frames;
 
-      /* FIXME: Nonblocking audio should break out of loop 
+      /* FIXME: Nonblocking audio should break out of loop
        * when it has nothing to write. */
-      while ((wa->dma_write == wa->dma_next || 
+      while ((wa->dma_write == wa->dma_next ||
                wa->dma_write == wa->dma_busy) && !wa->nonblock)
          OSSleepThread(wa->cond);
 
