@@ -15,22 +15,19 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/*  This file is intended for helper functions, custom controls, etc. */
+/*  This file is intended for menu functions, custom controls, etc. */
 
 #ifndef _NK_MENU_H
 #define _NK_MENU_H
 
 #include "nk_common.h"
-#include "../../menu_display.h"
-#include "../../menu_input.h"
 
+#include "../../menu_driver.h"
+#include "../../menu_input.h"
 
 enum
 {
-   NK_WND_MAIN = 0,
-   NK_WND_SETTINGS,
-   NK_WND_FILE_PICKER,
-   NK_WND_SHADER_PARAMETERS,
+   NK_WND_DEBUG = 0,
    NK_WND_LAST
 };
 
@@ -80,14 +77,13 @@ typedef struct nk_menu_handle
    video_font_raster_block_t list_block;
 } nk_menu_handle_t;
 
-void nk_wnd_shader_parameters(nk_menu_handle_t *nk);
-void nk_wnd_main(nk_menu_handle_t *nk, const char* title);
-bool nk_wnd_file_picker(nk_menu_handle_t *nk, char* title, char* in, char* out, char* filter);
-void nk_wnd_settings(nk_menu_handle_t *nk);
+struct nk_color nk_colors[NK_COLOR_COUNT];
+
+void nk_wnd_debug(nk_menu_handle_t *nk);
 void nk_wnd_set_state(nk_menu_handle_t *nk, const int id,
    struct nk_vec2 pos, struct nk_vec2 size);
 void nk_wnd_get_state(nk_menu_handle_t *nk, const int id,
    struct nk_vec2 *pos, struct nk_vec2 *size);
-void nk_common_set_style(struct nk_context *ctx, enum theme theme);
+void nk_common_set_style(struct nk_context *ctx);
 
 #endif
