@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -116,7 +116,7 @@ static void *sl_init(const char *device, unsigned rate, unsigned latency,
    if (!sl)
       goto error;
 
-   RARCH_LOG("[SLES]: Requested audio latency: %u ms.", latency);
+   RARCH_LOG("[OpenSL]: Requested audio latency: %u ms.", latency);
 
    GOTO_IF_FAIL(slCreateEngine(&sl->engine_object, 0, NULL, 0, NULL, NULL));
    GOTO_IF_FAIL(SLObjectItf_Realize(sl->engine_object, SL_BOOLEAN_FALSE));
@@ -133,6 +133,9 @@ static void *sl_init(const char *device, unsigned rate, unsigned latency,
    sl->buf_count    = (latency * 4 * rate + 500) / 1000;
    sl->buf_count    = (sl->buf_count + sl->buf_size / 2) / sl->buf_size;
 
+   if (sl->buf_count < 2)
+      sl->buf_count = 2;
+
    sl->buffer       = (uint8_t**)calloc(sizeof(uint8_t*), sl->buf_count);
    if (!sl->buffer)
       goto error;
@@ -144,7 +147,7 @@ static void *sl_init(const char *device, unsigned rate, unsigned latency,
    for (i = 0; i < sl->buf_count; i++)
       sl->buffer[i] = sl->buffer_chunk + i * sl->buf_size;
 
-   RARCH_LOG("[SLES]: Setting audio latency: Block size = %u, Blocks = %u, Total = %u ...\n",
+   RARCH_LOG("[OpenSL]: Setting audio latency: Block size = %u, Blocks = %u, Total = %u ...\n",
          sl->buf_size, sl->buf_count, sl->buf_size * sl->buf_count);
 
    fmt_pcm.formatType     = SL_DATAFORMAT_PCM;
@@ -192,7 +195,7 @@ static void *sl_init(const char *device, unsigned rate, unsigned latency,
    return sl;
 
 error:
-   RARCH_ERR("Couldn't initialize OpenSL ES driver, error code: [%d].\n", (int)res);
+   RARCH_ERR("[OpenSL]: Couldn't initialize OpenSL ES driver, error code: [%d].\n", (int)res);
    sl_free(sl);
    return NULL;
 }
@@ -200,7 +203,7 @@ error:
 static bool sl_stop(void *data)
 {
    sl_t      *sl = (sl_t*)data;
-   sl->is_paused = (SLPlayItf_SetPlayState(sl->player, SL_PLAYSTATE_STOPPED) 
+   sl->is_paused = (SLPlayItf_SetPlayState(sl->player, SL_PLAYSTATE_STOPPED)
          == SL_RESULT_SUCCESS) ? true : false;
 
    return sl->is_paused ? true : false;
@@ -230,8 +233,7 @@ static bool sl_start(void *data, bool is_shutdown)
 }
 
 
-static ssize_t sl_write(void *data, const void *buf_, size_t size,
-      bool is_perfcnt_enable)
+static ssize_t sl_write(void *data, const void *buf_, size_t size)
 {
    sl_t           *sl = (sl_t*)data;
    size_t     written = 0;
