@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -33,6 +33,7 @@
 
 #include <compat/strl.h>
 #include <string/stdstring.h>
+#include <streams/file_stream.h>
 #include <file/file_path.h>
 #ifndef IS_SALAMANDER
 #include <lists/file_list.h>
@@ -56,7 +57,7 @@ SYS_PROCESS_PARAM(1001, 0x100000)
 #else
 SYS_PROCESS_PARAM(1001, 0x200000)
 #endif
-   
+
 #ifdef HAVE_MULTIMAN
 #define MULTIMAN_SELF_FILE "/dev_hdd0/game/BLES80608/USRDIR/RELOAD.SELF"
 static bool multiman_detected  = false;
@@ -91,7 +92,7 @@ static void callback_sysutil_exit(uint64_t status,
             if (frontend)
                frontend->shutdown = frontend_ps3_shutdown;
 
-            runloop_ctl(RUNLOOP_CTL_SET_SHUTDOWN, NULL);
+            rarch_ctl(RARCH_CTL_SET_SHUTDOWN, NULL);
          }
          break;
    }
@@ -125,7 +126,7 @@ static void frontend_ps3_get_environment_settings(int *argc, char *argv[],
 #ifdef HAVE_MULTIMAN
    /* not launched from external launcher, set default path */
    // second param is multiMAN SELF file
-   if(path_file_exists(argv[2]) && *argc > 1
+   if (     filestream_exists(argv[2]) && *argc > 1
          && (string_is_equal(argv[2], EMULATOR_CONTENT_DIR)))
    {
       multiman_detected = true;
@@ -189,19 +190,19 @@ static void frontend_ps3_get_environment_settings(int *argc, char *argv[],
             break;
       }
 
-      if((get_attributes & CELL_GAME_ATTRIBUTE_APP_HOME) 
+      if((get_attributes & CELL_GAME_ATTRIBUTE_APP_HOME)
             == CELL_GAME_ATTRIBUTE_APP_HOME)
          RARCH_LOG("RetroArch was launched from host machine (APP_HOME).\n");
 
-      ret = cellGameContentPermit(content_info_path, g_defaults.dir.port);
+      ret = cellGameContentPermit(content_info_path, g_defaults.dirs[DEFAULT_DIR_PORT]);
 
 #ifdef HAVE_MULTIMAN
       if (multiman_detected)
       {
          fill_pathname_join(content_info_path, "/dev_hdd0/game/",
                EMULATOR_CONTENT_DIR, sizeof(content_info_path));
-         fill_pathname_join(g_defaults.dir.port, content_info_path,
-               "USRDIR", sizeof(g_defaults.dir.port));
+         fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_PORT], content_info_path,
+               "USRDIR", sizeof(g_defaults.dirs[DEFAULT_DIR_PORT]));
       }
 #endif
 
@@ -211,35 +212,57 @@ static void frontend_ps3_get_environment_settings(int *argc, char *argv[],
       {
          RARCH_LOG("cellGameContentPermit() OK.\n");
          RARCH_LOG("content_info_path : [%s].\n", content_info_path);
-         RARCH_LOG("usrDirPath : [%s].\n", g_defaults.dir.port);
+         RARCH_LOG("usrDirPath : [%s].\n", g_defaults.dirs[DEFAULT_DIR_PORT]);
       }
 
-      strlcpy(g_defaults.dir.content_history,
-            g_defaults.dir.port, sizeof(g_defaults.dir.content_history));
-      fill_pathname_join(g_defaults.dir.core, g_defaults.dir.port,
-            "cores", sizeof(g_defaults.dir.core));
-      fill_pathname_join(g_defaults.dir.core_info, g_defaults.dir.core,
-            "info", sizeof(g_defaults.dir.core_info));
-      fill_pathname_join(g_defaults.dir.savestate, g_defaults.dir.core,
-            "savestates", sizeof(g_defaults.dir.savestate));
-      fill_pathname_join(g_defaults.dir.sram, g_defaults.dir.core,
-            "savefiles", sizeof(g_defaults.dir.sram));
-      fill_pathname_join(g_defaults.dir.system, g_defaults.dir.core,
-            "system", sizeof(g_defaults.dir.system));
-      fill_pathname_join(g_defaults.dir.shader,  g_defaults.dir.core,
-            "shaders_cg", sizeof(g_defaults.dir.shader));
-      fill_pathname_join(g_defaults.path.config, g_defaults.dir.port,
+      strlcpy(g_defaults.dirs[DEFAULT_DIR_CONTENT_HISTORY],
+            g_defaults.dirs[DEFAULT_DIR_PORT],
+            sizeof(g_defaults.dirs[DEFAULT_DIR_CONTENT_HISTORY]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE],
+            g_defaults.dirs[DEFAULT_DIR_PORT],
+            "cores", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE_INFO],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "info",
+            sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_INFO]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SAVESTATE],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "savestates", sizeof(g_defaults.dirs[DEFAULT_DIR_SAVESTATE]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SRAM],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "savefiles", sizeof(g_defaults.dirs[DEFAULT_DIR_SRAM]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SYSTEM],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "system", sizeof(g_defaults.dirs[DEFAULT_DIR_SYSTEM]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_SHADER],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "shaders_cg", sizeof(g_defaults.dirs[DEFAULT_DIR_SHADER]));
+      fill_pathname_join(g_defaults.path.config, g_defaults.dirs[DEFAULT_DIR_PORT],
             file_path_str(FILE_PATH_MAIN_CONFIG),  sizeof(g_defaults.path.config));
-      fill_pathname_join(g_defaults.dir.overlay, g_defaults.dir.core,
-            "overlays", sizeof(g_defaults.dir.overlay));
-      fill_pathname_join(g_defaults.dir.assets,   g_defaults.dir.core,
-            "assets", sizeof(g_defaults.dir.assets));
-      fill_pathname_join(g_defaults.dir.cursor,   g_defaults.dir.core,
-            "database/cursors", sizeof(g_defaults.dir.cursor));
-      fill_pathname_join(g_defaults.dir.database,   g_defaults.dir.core,
-            "database/rdb", sizeof(g_defaults.dir.database));
-      fill_pathname_join(g_defaults.dir.playlist,   g_defaults.dir.core,
-            "playlists", sizeof(g_defaults.dir.playlist));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_OVERLAY],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "overlays", sizeof(g_defaults.dirs[DEFAULT_DIR_OVERLAY]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_ASSETS],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "assets", sizeof(g_defaults.dirs[DEFAULT_DIR_ASSETS]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CURSOR],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "database/cursors", sizeof(g_defaults.dirs[DEFAULT_DIR_CURSOR]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_DATABASE],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "database/rdb", sizeof(g_defaults.dirs[DEFAULT_DIR_DATABASE]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_PLAYLIST],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "playlists", sizeof(g_defaults.dirs[DEFAULT_DIR_PLAYLIST]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CORE_ASSETS],
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "downloads", sizeof(g_defaults.dirs[DEFAULT_DIR_CORE_ASSETS]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_CHEATS],
+            g_defaults.dirs[DEFAULT_DIR_CORE], "cheats",
+            sizeof(g_defaults.dirs[DEFAULT_DIR_CHEATS]));
+      fill_pathname_join(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG], 
+            g_defaults.dirs[DEFAULT_DIR_CORE],
+            "autoconfig", sizeof(g_defaults.dirs[DEFAULT_DIR_AUTOCONFIG]));
    }
 
 #ifndef IS_SALAMANDER
@@ -491,71 +514,74 @@ enum frontend_architecture frontend_ps3_get_architecture(void)
    return FRONTEND_ARCH_PPC;
 }
 
-static int frontend_ps3_parse_drive_list(void *data)
+static int frontend_ps3_parse_drive_list(void *data, bool load_content)
 {
 #ifndef IS_SALAMANDER
    file_list_t *list = (file_list_t*)data;
+   enum msg_hash_enums enum_idx = load_content ?
+      MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR :
+      MSG_UNKNOWN;
 
    menu_entries_append_enum(list,
          "/app_home/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/dev_hdd0/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/dev_hdd1/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/dev_bdvd/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/host_root/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/dev_usb000/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/dev_usb001/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/dev_usb002/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/dev_usb003/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/dev_usb004/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/dev_usb005/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
    menu_entries_append_enum(list,
          "/dev_usb006/",
          msg_hash_to_str(MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR),
-         MENU_ENUM_LABEL_FILE_DETECT_CORE_LIST_PUSH_DIR,
-         MENU_SETTING_ACTION, 0, 0);
+         enum_idx,
+         FILE_TYPE_DIRECTORY, 0, 0);
 #endif
 
    return 0;
@@ -572,8 +598,8 @@ static void frontend_ps3_process_args(int *argc, char *argv[])
    {
       char path[PATH_MAX_LENGTH] = {0};
       strlcpy(path, argv[0], sizeof(path));
-      if (path_file_exists(path))
-         runloop_ctl(RUNLOOP_CTL_SET_LIBRETRO_PATH, path);
+      if (filestream_exists(path))
+         rarch_ctl(RARCH_CTL_SET_LIBRETRO_PATH, path);
    }
 #endif
 }
@@ -606,5 +632,7 @@ frontend_ctx_driver_t frontend_ctx_ps3 = {
    NULL,                         /* destroy_sighandler_state */
    NULL,                         /* attach_console */
    NULL,                         /* detach_console */
+   NULL,                         /* watch_path_for_changes */
+   NULL,                         /* check_for_path_changes */
    "ps3",
 };
