@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -110,11 +110,11 @@ static void *ps3_audio_init(const char *device,
       return NULL;
    }
 
-   data->buffer = fifo_new(CELL_AUDIO_BLOCK_SAMPLES * 
+   data->buffer = fifo_new(CELL_AUDIO_BLOCK_SAMPLES *
          AUDIO_CHANNELS * AUDIO_BLOCKS * sizeof(float));
 
 #ifdef __PSL1GHT__
-   sys_lwmutex_attr_t lock_attr = 
+   sys_lwmutex_attr_t lock_attr =
    {SYS_LWMUTEX_ATTR_PROTOCOL, SYS_LWMUTEX_ATTR_RECURSIVE, "\0"};
    sys_lwmutex_attr_t cond_lock_attr =
    {SYS_LWMUTEX_ATTR_PROTOCOL, SYS_LWMUTEX_ATTR_RECURSIVE, "\0"};
@@ -146,8 +146,7 @@ static void *ps3_audio_init(const char *device,
    return data;
 }
 
-static ssize_t ps3_audio_write(void *data, const void *buf, size_t size,
-      bool is_perfcnt_enable)
+static ssize_t ps3_audio_write(void *data, const void *buf, size_t size)
 {
    ps3_audio_t *aud = data;
 
