@@ -41,9 +41,7 @@
 
 #include "../../defines/psp_defines.h"
 
-#include "../input_config.h"
 #include "../input_driver.h"
-#include "../input_joypad_driver.h"
 
 typedef struct psp_input
 {
@@ -51,13 +49,11 @@ typedef struct psp_input
    const input_device_driver_t *joypad;
 } psp_input_t;
 
-uint64_t lifecycle_state;
-
 static void psp_input_poll(void *data)
 {
    psp_input_t *psp = (psp_input_t*)data;
 
-   if (psp->joypad)
+   if (psp && psp->joypad)
       psp->joypad->poll();
 }
 
@@ -108,11 +104,6 @@ static void* psp_input_initialize(const char *joypad_driver)
    return psp;
 }
 
-static bool psp_input_meta_key_pressed(void *data, int key)
-{
-   return (BIT64_GET(lifecycle_state, key));
-}
-
 static uint64_t psp_input_get_capabilities(void *data)
 {
    (void)data;
@@ -165,7 +156,6 @@ input_driver_t input_psp = {
    psp_input_initialize,
    psp_input_poll,
    psp_input_state,
-   psp_input_meta_key_pressed,
    psp_input_free_input,
    NULL,
    NULL,
