@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -102,8 +102,7 @@ error:
    return NULL;
 }
 
-static ssize_t rs_write(void *data, const void *buf, size_t size,
-      bool is_perfcnt_enable)
+static ssize_t rs_write(void *data, const void *buf, size_t size)
 {
    rsd_t *rsd = (rsd_t*)data;
 
