@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2015 - Michael Lelli
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -51,8 +51,7 @@ static void *rwebaudio_init(const char *device, unsigned rate, unsigned latency,
    return data;
 }
 
-static ssize_t rwebaudio_write(void *data, const void *buf, size_t size,
-      bool is_perfcnt_enable)
+static ssize_t rwebaudio_write(void *data, const void *buf, size_t size)
 {
    (void)data;
    return RWebAudioWrite(buf, size);
