@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -14,6 +14,15 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <boolean.h>
+#include <rthreads/rthreads.h>
+#include <queues/fifo_queue.h>
+#include <retro_inline.h>
+#include <retro_math.h>
+
+#include "../audio_driver.h"
+#include "../../verbosity.h"
+
 #include <stdint.h>
 #include <stddef.h>
 #include <stdlib.h>
@@ -22,14 +31,6 @@
 #include "SDL.h"
 #include "SDL_audio.h"
 
-#include <boolean.h>
-#include <rthreads/rthreads.h>
-#include <queues/fifo_queue.h>
-#include <retro_inline.h>
-
-#include "../audio_driver.h"
-#include "../../verbosity.h"
-
 typedef struct sdl_audio
 {
    bool nonblock;
@@ -92,11 +93,11 @@ static void *sdl_audio_init(const char *device,
    if (!sdl)
       return NULL;
 
-   /* We have to buffer up some data ourselves, so we let SDL 
+   /* We have to buffer up some data ourselves, so we let SDL
     * carry approximately half of the latency.
     *
     * SDL double buffers audio and we do as well. */
-   frames = find_num_frames(rate, latency / 4);
+   frames        = find_num_frames(rate, latency / 4);
 
    spec.freq     = rate;
    spec.format   = AUDIO_S16SYS;
@@ -107,9 +108,8 @@ static void *sdl_audio_init(const char *device,
 
    if (SDL_OpenAudio(&spec, &out) < 0)
    {
-      RARCH_ERR("Failed to open SDL audio: %s\n", SDL_GetError());
-      free(sdl);
-      return 0;
+      RARCH_ERR("[SDL audio]: Failed to open SDL audio: %s\n", SDL_GetError());
+      goto error;
    }
 
    *new_rate                = out.freq;
@@ -119,7 +119,7 @@ static void *sdl_audio_init(const char *device,
    sdl->cond                = scond_new();
 #endif
 
-   RARCH_LOG("SDL audio: Requested %u ms latency, got %d ms\n", 
+   RARCH_LOG("[SDL audio]: Requested %u ms latency, got %d ms\n",
          latency, (int)(out.samples * 4 * 1000 / (*new_rate)));
 
    /* Create a buffer twice as big as needed and prefill the buffer. */
@@ -135,12 +135,15 @@ static void *sdl_audio_init(const char *device,
 
    SDL_PauseAudio(0);
    return sdl;
+
+error:
+   free(sdl);
+   return NULL;
 }
 
-static ssize_t sdl_audio_write(void *data, const void *buf, size_t size,
-      bool is_perfcnt_enable)
+static ssize_t sdl_audio_write(void *data, const void *buf, size_t size)
 {
-   ssize_t ret = 0;
+   ssize_t ret      = 0;
    sdl_audio_t *sdl = (sdl_audio_t*)data;
 
    if (sdl->nonblock)
