@@ -43,9 +43,6 @@
 #include "../video_driver.h"
 
 #include "../../configuration.h"
-#include "../../performance_counters.h"
-
-#include "../video_context_driver.h"
 
 typedef struct sdl_menu_frame
 {
@@ -96,20 +93,20 @@ static void sdl_init_font(sdl_video_t *vid, const char *font_path, unsigned font
    int r, g, b;
    settings_t *settings = config_get_ptr();
 
-   if (!settings->video.font_enable)
+   if (!settings->bools.video_font_enable)
       return;
 
    if (!font_renderer_create_default((const void**)&vid->font_driver, &vid->font,
-            *settings->path.font ? settings->path.font : NULL,
-            settings->video.font_size))
+            *settings->paths.path_font ? settings->paths.path_font : NULL,
+            settings->floats.video_font_size))
    {
       RARCH_LOG("[SDL]: Could not initialize fonts.\n");
       return;
    }
 
-   r = settings->video.msg_color_r * 255;
-   g = settings->video.msg_color_g * 255;
-   b = settings->video.msg_color_b * 255;
+   r = settings->floats.video_msg_color_r * 255;
+   g = settings->floats.video_msg_color_g * 255;
+   b = settings->floats.video_msg_color_b * 255;
 
    r = (r < 0) ? 0 : (r > 255 ? 255 : r);
    g = (g < 0) ? 0 : (g > 255 ? 255 : g);
@@ -133,8 +130,8 @@ static void sdl_render_msg(sdl_video_t *vid, SDL_Surface *buffer,
 
    atlas = vid->font_driver->get_atlas(vid->font);
 
-   msg_base_x = settings->video.msg_pos_x * width;
-   msg_base_y = (1.0f - settings->video.msg_pos_y) * height;
+   msg_base_x = settings->floats.video_msg_pos_x * width;
+   msg_base_y = (1.0f - settings->floats.video_msg_pos_y) * height;
 
    rshift = fmt->Rshift;
    gshift = fmt->Gshift;
@@ -155,7 +152,7 @@ static void sdl_render_msg(sdl_video_t *vid, SDL_Surface *buffer,
 
       base_x = msg_base_x + glyph->draw_offset_x;
       base_y = msg_base_y + glyph->draw_offset_y;
-      src    = atlas->buffer + glyph->atlas_offset_x 
+      src    = atlas->buffer + glyph->atlas_offset_x
          + glyph->atlas_offset_y * atlas->width;
 
       if (base_x < 0)
@@ -270,7 +267,7 @@ static void *sdl_gfx_init(const video_info_t *video, const input_driver_t **inpu
 
    if (!vid->screen)
    {
-      RARCH_ERR("[SDL]: Failed to init SDL surface: %s\n", SDL_GetError());
+      RARCH_ERR("[SDL1]: Failed to init SDL surface: %s\n", SDL_GetError());
       goto error;
    }
 
@@ -281,7 +278,7 @@ static void *sdl_gfx_init(const video_info_t *video, const input_driver_t **inpu
 
    if (input && input_data)
    {
-      void *sdl_input = input_sdl.init(settings->input.joypad_driver);
+      void *sdl_input = input_sdl.init(settings->arrays.input_joypad_driver);
 
       if (sdl_input)
       {
@@ -295,7 +292,7 @@ static void *sdl_gfx_init(const video_info_t *video, const input_driver_t **inpu
       }
    }
 
-   sdl_init_font(vid, settings->path.font, settings->video.font_size);
+   sdl_init_font(vid, settings->paths.path_font, settings->floats.video_font_size);
 
    vid->scaler.scaler_type      = video->smooth ? SCALER_TYPE_BILINEAR : SCALER_TYPE_POINT;
    vid->scaler.in_fmt           = video->rgb32 ? SCALER_FMT_ARGB8888 : SCALER_FMT_RGB565;
@@ -309,7 +306,7 @@ static void *sdl_gfx_init(const video_info_t *video, const input_driver_t **inpu
 
    if (!vid->menu.frame)
    {
-      RARCH_ERR("[SDL]: Failed to init menu surface: %s\n", SDL_GetError());
+      RARCH_ERR("[SDL1]: Failed to init menu surface: %s\n", SDL_GetError());
       goto error;
    }
 
@@ -339,7 +336,6 @@ static bool sdl_gfx_frame(void *data, const void *frame, unsigned width,
       unsigned height, uint64_t frame_count,
       unsigned pitch, const char *msg, video_frame_info_t *video_info)
 {
-   static struct retro_perf_counter sdl_scale = {0};
    sdl_video_t                    *vid = (sdl_video_t*)data;
    char title[128];
 
@@ -353,9 +349,6 @@ static bool sdl_gfx_frame(void *data, const void *frame, unsigned width,
    if (SDL_MUSTLOCK(vid->screen))
       SDL_LockSurface(vid->screen);
 
-   performance_counter_init(sdl_scale, "sdl_scale");
-   performance_counter_start_plus(video_info->is_perfcnt_enable, sdl_scale);
-
    video_frame_scale(
          &vid->scaler,
          vid->screen->pixels,
@@ -367,7 +360,6 @@ static bool sdl_gfx_frame(void *data, const void *frame, unsigned width,
          width,
          height,
          pitch);
-   performance_counter_stop_plus(video_info->is_perfcnt_enable, sdl_scale);
 
 #ifdef HAVE_MENU
    menu_driver_frame(video_info);
@@ -377,7 +369,8 @@ static bool sdl_gfx_frame(void *data, const void *frame, unsigned width,
       SDL_BlitSurface(vid->menu.frame, NULL, vid->screen, NULL);
 
    if (msg)
-      sdl_render_msg(vid, vid->screen, msg, vid->screen->w, vid->screen->h, vid->screen->format);
+      sdl_render_msg(vid, vid->screen,
+            msg, vid->screen->w, vid->screen->h, vid->screen->format);
 
    if (SDL_MUSTLOCK(vid->screen))
       SDL_UnlockSurface(vid->screen);
@@ -474,11 +467,10 @@ static void sdl_apply_state_changes(void *data)
    (void)data;
 }
 
-#ifdef HAVE_MENU
 static void sdl_set_texture_frame(void *data, const void *frame, bool rgb32,
       unsigned width, unsigned height, float alpha)
 {
-   enum scaler_pix_fmt format = rgb32 
+   enum scaler_pix_fmt format = rgb32
       ? SCALER_FMT_ARGB8888 : SCALER_FMT_RGBA4444;
    sdl_video_t           *vid = (sdl_video_t*)data;
 
@@ -507,13 +499,13 @@ static void sdl_set_texture_enable(void *data, bool state, bool full_screen)
    vid->menu.active = state;
 }
 
+
 static void sdl_show_mouse(void *data, bool state)
 {
    (void)data;
 
    SDL_ShowCursor(state);
 }
-#endif
 
 static void sdl_grab_mouse_toggle(void *data)
 {
@@ -525,9 +517,13 @@ static void sdl_grab_mouse_toggle(void *data)
 }
 
 static const video_poke_interface_t sdl_poke_interface = {
+   NULL, /* get_flags */
+   NULL,                /* set_coords */
+   NULL,                /* set_mvp */
    NULL,
    NULL,
    NULL,
+   NULL, /* get_refresh_rate */
    sdl_set_filtering,
    NULL, /* get_video_output_size */
    NULL, /* get_video_output_prev */
@@ -536,20 +532,14 @@ static const video_poke_interface_t sdl_poke_interface = {
    NULL, /* get_proc_address */
    sdl_set_aspect_ratio,
    sdl_apply_state_changes,
-#ifdef HAVE_MENU
    sdl_set_texture_frame,
-#endif
-#ifdef HAVE_MENU
    sdl_set_texture_enable,
    NULL,
    sdl_show_mouse,
-#else
-   NULL,
-   NULL,
-   NULL,
-#endif
    sdl_grab_mouse_toggle,
-   NULL
+   NULL,                         /* get_current_shader */
+   NULL,                         /* get_current_software_framebuffer */
+   NULL                          /* get_hw_render_interface */
 };
 
 static void sdl_get_poke_interface(void *data, const video_poke_interface_t **iface)
@@ -566,7 +556,7 @@ static bool sdl_gfx_set_shader(void *data,
    (void)type;
    (void)path;
 
-   return false; 
+   return false;
 }
 
 static void sdl_gfx_set_rotation(void *data, unsigned rotation)
