@@ -24,6 +24,7 @@
 #include <file/file_path.h>
 #include <string/stdstring.h>
 #include <queues/task_queue.h>
+#include <retro_timers.h>
 
 #include "cocoa/cocoa_common.h"
 #include "../ui_companion_driver.h"
@@ -34,7 +35,6 @@
 #include "../../paths.h"
 #include "../../core.h"
 #include "../../retroarch.h"
-#include "../../runloop.h"
 #include "../../tasks/tasks_internal.h"
 
 id apple_platform;
@@ -197,13 +197,13 @@ static char** waiting_argv;
    SEL selector     = NSSelectorFromString(BOXSTRING("setCollectionBehavior:"));
    SEL fsselector   = NSSelectorFromString(BOXSTRING("toggleFullScreen:"));
    apple_platform   = self;
-    
+
    if ([self.window respondsToSelector:selector])
    {
        if ([self.window respondsToSelector:fsselector])
           [self.window setCollectionBehavior:NS_WINDOW_COLLECTION_BEHAVIOR_FULLSCREEN_PRIMARY];
    }
-   
+
    [self.window setAcceptsMouseMovedEvents: YES];
 
    [[CocoaView get] setFrame: [[self.window contentView] bounds]];
@@ -240,12 +240,12 @@ static char** waiting_argv;
        ret = runloop_iterate(&sleep_ms);
        if (ret == 1 && sleep_ms > 0)
           retro_sleep(sleep_ms);
-       task_queue_ctl(TASK_QUEUE_CTL_CHECK, NULL);
+       task_queue_check();
        while(CFRunLoopRunInMode(kCFRunLoopDefaultMode, 0.002, FALSE) == kCFRunLoopRunHandledSource);
        if (ret == -1)
           break;
     }while(1);
-    
+
     main_exit(NULL);
 }
 
@@ -279,24 +279,21 @@ static char** waiting_argv;
 {
    if (filenames.count == 1 && [filenames objectAtIndex:0])
    {
-      struct retro_system_info         *system = NULL;
-      NSString *__core = [filenames objectAtIndex:0];
-      const char *core_name = NULL;
+      rarch_system_info_t *info        = runloop_get_system_info();
+      struct retro_system_info *system = &info->info;
+      NSString *__core                 = [filenames objectAtIndex:0];
+      const char *core_name            = NULL;
 
-      menu_driver_ctl(RARCH_MENU_CTL_SYSTEM_INFO_GET, &system);
-      
       if (system)
          core_name = system->library_name;
-		
+
       if (core_name)
       {
          content_ctx_info_t content_info = {0};
-         task_push_content_load_default(
-               NULL,
+         task_push_load_content_with_current_core_from_companion_ui(
                __core.UTF8String,
                &content_info,
                CORE_TYPE_PLAIN,
-               CONTENT_MODE_LOAD_CONTENT_WITH_CURRENT_CORE_FROM_COMPANION_UI,
                NULL, NULL);
       }
       else
@@ -323,6 +320,7 @@ static char** waiting_argv;
 
 static void open_core_handler(ui_browser_window_state_t *state, bool result)
 {
+   rarch_system_info_t *info      = runloop_get_system_info();
     if (!state)
         return;
     if (string_is_empty(state->result))
@@ -331,20 +329,19 @@ static void open_core_handler(ui_browser_window_state_t *state, bool result)
         return;
 
     settings_t *settings = config_get_ptr();
-                
-    runloop_ctl(RUNLOOP_CTL_SET_LIBRETRO_PATH, (void*)state->result);
+
+    rarch_ctl(RARCH_CTL_SET_LIBRETRO_PATH, (void*)state->result);
     ui_companion_event_command(CMD_EVENT_LOAD_CORE);
-                
-    if (menu_driver_ctl(RARCH_MENU_CTL_HAS_LOAD_NO_CONTENT, NULL)
-                      && settings->set_supports_no_game_enable)
+
+    if (info && info->load_no_content
+          && settings->bools.set_supports_no_game_enable)
     {
         content_ctx_info_t content_info = {0};
         path_clear(RARCH_PATH_CONTENT);
-        task_push_content_load_default(
-                NULL, NULL,
+        task_push_load_content_with_current_core_from_companion_ui(
+                NULL,
                 &content_info,
                 CORE_TYPE_PLAIN,
-                CONTENT_MODE_LOAD_CONTENT_WITH_CURRENT_CORE_FROM_COMPANION_UI,
                 NULL, NULL);
     }
 }
@@ -357,45 +354,43 @@ static void open_document_handler(ui_browser_window_state_t *state, bool result)
         return;
     if (!result)
         return;
-    
-    struct retro_system_info *system = NULL;
+
+    rarch_system_info_t *info        = runloop_get_system_info();
+    struct retro_system_info *system = &info->info;
     const char            *core_name = NULL;
-                
-    menu_driver_ctl(RARCH_MENU_CTL_SYSTEM_INFO_GET, &system);
-                
+
     if (system)
         core_name = system->library_name;
-                
+
     path_set(RARCH_PATH_CONTENT, state->result);
-                
+
     if (core_name)
     {
         content_ctx_info_t content_info = {0};
-        task_push_content_load_default(
-                NULL, NULL,
+        task_push_load_content_with_current_core_from_companion_ui(
+                NULL,
                 &content_info,
                 CORE_TYPE_PLAIN,
-                CONTENT_MODE_LOAD_CONTENT_WITH_CURRENT_CORE_FROM_COMPANION_UI,
                 NULL, NULL);
     }
 }
 
 - (IBAction)openCore:(id)sender {
     const ui_browser_window_t *browser = ui_companion_driver_get_browser_window_ptr();
-    
+
     if (browser)
     {
         ui_browser_window_state_t browser_state;
         settings_t *settings        = config_get_ptr();
-        
+
         browser_state.filters       = strdup("dylib");
         browser_state.filters_title = strdup("Core");
         browser_state.title         = strdup("Load Core");
-        browser_state.startdir      = strdup(settings->directory.libretro);
-        
+        browser_state.startdir      = strdup(settings->paths.directory_libretro);
+
         bool result = browser->open(&browser_state);
         open_core_handler(&browser_state, result);
-        
+
         free(browser_state.filters);
         free(browser_state.filters_title);
         free(browser_state.title);
@@ -406,22 +401,22 @@ static void open_document_handler(ui_browser_window_state_t *state, bool result)
 - (void)openDocument:(id)sender
 {
    const ui_browser_window_t *browser = ui_companion_driver_get_browser_window_ptr();
-    
+
     if (browser)
     {
         ui_browser_window_state_t browser_state = {{0}};
         settings_t *settings  = config_get_ptr();
-        NSString *startdir    = BOXSTRING(settings->directory.menu_content);
-        
+        NSString *startdir    = BOXSTRING(settings->paths.directory_menu_content);
+
         if (!startdir.length)
             startdir           = BOXSTRING("/");
-        
+
         browser_state.title = strdup("Load Content");
         browser_state.startdir = strdup([startdir UTF8String]);
-        
+
         bool result = browser->open(&browser_state);
         open_document_handler(&browser_state, result);
-        
+
         free(browser_state.startdir);
         free(browser_state.title);
     }
@@ -434,7 +429,7 @@ static void open_document_handler(ui_browser_window_state_t *state, bool result)
 - (IBAction)showCoresDirectory:(id)sender
 {
    settings_t *settings = config_get_ptr();
-   [[NSWorkspace sharedWorkspace] openFile:BOXSTRING(settings->directory.libretro)];
+   [[NSWorkspace sharedWorkspace] openFile:BOXSTRING(settings->paths.directory_libretro)];
 }
 
 - (IBAction)showPreferences:(id)sender
@@ -445,7 +440,7 @@ static void open_document_handler(ui_browser_window_state_t *state, bool result)
 {
    enum event_command cmd;
    unsigned sender_tag = (unsigned)[sender tag];
-   
+
    switch (sender_tag)
    {
       case 1:
@@ -482,11 +477,11 @@ static void open_document_handler(ui_browser_window_state_t *state, bool result)
          cmd = CMD_EVENT_NONE;
          break;
    }
-   
+
    if (sender_tag >= 10 && sender_tag <= 19)
    {
       unsigned idx = (sender_tag - (10-1));
-      runloop_ctl(RUNLOOP_CTL_SET_WINDOWED_SCALE, &idx);
+      rarch_ctl(RARCH_CTL_SET_WINDOWED_SCALE, &idx);
       cmd = CMD_EVENT_RESIZE_WINDOWED_SCALE;
    }
 
@@ -508,7 +503,7 @@ int main(int argc, char *argv[])
            if (!strncmp(argv[1], "-psn", 4))
                argc = 1;
    }
-    
+
    waiting_argc = argc;
    waiting_argv = argv;
 
@@ -525,9 +520,10 @@ static void ui_companion_cocoa_notify_content_loaded(void *data)
     (void)data;
 }
 
-static void ui_companion_cocoa_toggle(void *data)
+static void ui_companion_cocoa_toggle(void *data, bool force)
 {
    (void)data;
+   (void)force;
 }
 
 static int ui_companion_cocoa_iterate(void *data, unsigned action)
@@ -576,7 +572,7 @@ static void *ui_companion_cocoa_get_main_window(void *data)
     return ((RetroArch_OSX*)[[NSApplication sharedApplication] delegate]).window;
 }
 
-const ui_companion_driver_t ui_companion_cocoa = {
+ui_companion_driver_t ui_companion_cocoa = {
    ui_companion_cocoa_init,
    ui_companion_cocoa_deinit,
    ui_companion_cocoa_iterate,
@@ -588,6 +584,7 @@ const ui_companion_driver_t ui_companion_cocoa = {
    NULL,
    NULL,
    ui_companion_cocoa_get_main_window,
+   NULL,
    &ui_browser_window_cocoa,
    &ui_msg_window_cocoa,
    &ui_window_cocoa,
