@@ -58,9 +58,10 @@ static void ui_companion_null_notify_content_loaded(void *data)
    (void)data;
 }
 
-static void ui_companion_null_toggle(void *data)
+static void ui_companion_null_toggle(void *data, bool force)
 {
    (void)data;
+   (void)force;
 }
 
 static void ui_companion_null_event_command(void *data, enum event_command cmd)
@@ -77,7 +78,7 @@ static void ui_companion_null_notify_list_pushed(void *data,
     (void)menu_list;
 }
 
-const ui_companion_driver_t ui_companion_null = {
+ui_companion_driver_t ui_companion_null = {
    ui_companion_null_init,
    ui_companion_null_deinit,
    ui_companion_null_iterate,
@@ -89,6 +90,7 @@ const ui_companion_driver_t ui_companion_null = {
    NULL,
    NULL,
    NULL,
+   NULL,
    &ui_browser_window_null,
    &ui_msg_window_null,
    &ui_window_null,
