@@ -49,12 +49,10 @@
 #include "../../configuration.h"
 #include "../../driver.h"
 #include "../../paths.h"
-#include "../../runloop.h"
-#include "../../gfx/video_context_driver.h"
-#include "../../gfx/video_shader_driver.h"
+#include "../../retroarch.h"
+#include "../../gfx/video_driver.h"
 #include "../../tasks/tasks_internal.h"
 
-#include "../../gfx/common/gl_common.h"
 #include "ui_win32.h"
 
 #define SHADER_DLG_WIDTH                  220
@@ -76,6 +74,7 @@ typedef struct ui_companion_win32
    void *empty;
 } ui_companion_win32_t;
 
+#ifdef HAVE_SHADERPIPELINE
 enum shader_param_ctrl_type
 {
    SHADER_PARAM_CTRL_NONE = 0,
@@ -102,7 +101,7 @@ typedef struct
          HWND label_title;
          HWND label_val;
       } trackbar;
-   };
+   } elems;
 } shader_param_ctrl_t;
 
 typedef struct
@@ -116,12 +115,12 @@ typedef struct
 
 static shader_dlg_t g_shader_dlg = {{0}};
 
-static bool shader_dlg_refresh_trackbar_label(int index, 
+static bool shader_dlg_refresh_trackbar_label(int index,
       video_shader_ctx_t *shader_info)
 {
    char val_buffer[32]         = {0};
 
-   if (floorf(shader_info->data->parameters[index].current) 
+   if (floorf(shader_info->data->parameters[index].current)
          == shader_info->data->parameters[index].current)
       snprintf(val_buffer, sizeof(val_buffer), "%.0f",
             shader_info->data->parameters[index].current);
@@ -129,7 +128,7 @@ static bool shader_dlg_refresh_trackbar_label(int index,
       snprintf(val_buffer, sizeof(val_buffer), "%.2f",
             shader_info->data->parameters[index].current);
 
-   SendMessage(g_shader_dlg.controls[index].trackbar.label_val,
+   SendMessage(g_shader_dlg.controls[index].elems.trackbar.label_val,
          WM_SETTEXT, 0, (LPARAM)val_buffer);
 
    return true;
@@ -150,13 +149,15 @@ static void shader_dlg_params_refresh(void)
       {
          case SHADER_PARAM_CTRL_CHECKBOX:
             {
+               bool checked;
+
                video_shader_ctx_t shader_info;
                video_shader_driver_get_current_shader(&shader_info);
 
-               bool checked = shader_info.data ?
-                  (shader_info.data->parameters[i].current == 
+               checked = shader_info.data ?
+                  (shader_info.data->parameters[i].current ==
                    shader_info.data->parameters[i].maximum) : false;
-               SendMessage(control->checkbox.hwnd, BM_SETCHECK, checked, 0);
+               SendMessage(control->elems.checkbox.hwnd, BM_SETCHECK, checked, 0);
             }
             break;
          case SHADER_PARAM_CTRL_TRACKBAR:
@@ -168,16 +169,16 @@ static void shader_dlg_params_refresh(void)
 
                if (shader_info.data)
                {
-                  SendMessage(control->trackbar.hwnd,
+                  SendMessage(control->elems.trackbar.hwnd,
                         TBM_SETRANGEMIN, (WPARAM)TRUE, (LPARAM)0);
-                  SendMessage(control->trackbar.hwnd,
+                  SendMessage(control->elems.trackbar.hwnd,
                         TBM_SETRANGEMAX, (WPARAM)TRUE,
-                        (LPARAM)((shader_info.data->parameters[i].maximum - 
-                              shader_info.data->parameters[i].minimum) 
+                        (LPARAM)((shader_info.data->parameters[i].maximum -
+                              shader_info.data->parameters[i].minimum)
                            / shader_info.data->parameters[i].step));
-                  SendMessage(control->trackbar.hwnd, TBM_SETPOS, (WPARAM)TRUE,
-                        (LPARAM)((shader_info.data->parameters[i].current - 
-                              shader_info.data->parameters[i].minimum) / 
+                  SendMessage(control->elems.trackbar.hwnd, TBM_SETPOS, (WPARAM)TRUE,
+                        (LPARAM)((shader_info.data->parameters[i].current -
+                              shader_info.data->parameters[i].minimum) /
                            shader_info.data->parameters[i].step));
                }
             }
@@ -208,19 +209,20 @@ static void shader_dlg_params_clear(void)
             {
                const ui_window_t *window = ui_companion_driver_get_window_ptr();
                if (window)
-                  window->destroy(&control->checkbox);
+                  window->destroy(&control->elems.checkbox);
             }
             break;
          case SHADER_PARAM_CTRL_TRACKBAR:
-            DestroyWindow(control->trackbar.label_title);
-            DestroyWindow(control->trackbar.label_val);
-            DestroyWindow(control->trackbar.hwnd);
+            DestroyWindow(control->elems.trackbar.label_title);
+            DestroyWindow(control->elems.trackbar.label_val);
+            DestroyWindow(control->elems.trackbar.hwnd);
             break;
       }
 
       control->type = SHADER_PARAM_CTRL_NONE;
    }
 }
+#endif
 
 void shader_dlg_params_reload(void)
 {
@@ -230,7 +232,7 @@ void shader_dlg_params_reload(void)
    int i, pos_x, pos_y;
    video_shader_ctx_t shader_info;
    const ui_window_t *window = NULL;
-   
+
    shader_dlg_params_clear();
 
    video_shader_driver_get_current_shader(&shader_info);
@@ -248,12 +250,12 @@ void shader_dlg_params_reload(void)
       shader_param_ctrl_t*control = &g_shader_dlg.controls[i];
 
       if ((shader_info.data->parameters[i].minimum == 0.0)
-            && (shader_info.data->parameters[i].maximum 
-               == (shader_info.data->parameters[i].minimum 
+            && (shader_info.data->parameters[i].maximum
+               == (shader_info.data->parameters[i].minimum
                   + shader_info.data->parameters[i].step)))
       {
-         if ((pos_y + SHADER_DLG_CHECKBOX_HEIGHT 
-                    + SHADER_DLG_CTRL_MARGIN + 20) 
+         if ((pos_y + SHADER_DLG_CHECKBOX_HEIGHT
+                    + SHADER_DLG_CTRL_MARGIN + 20)
                > SHADER_DLG_MAX_HEIGHT)
          {
             pos_y  = g_shader_dlg.parameters_start_y;
@@ -261,12 +263,12 @@ void shader_dlg_params_reload(void)
          }
 
          control->type          = SHADER_PARAM_CTRL_CHECKBOX;
-         control->checkbox.hwnd = CreateWindowEx(0, "BUTTON",
+         control->elems.checkbox.hwnd = CreateWindowEx(0, "BUTTON",
                shader_info.data->parameters[i].desc,
                WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, pos_x, pos_y,
                SHADER_DLG_CTRL_WIDTH, SHADER_DLG_CHECKBOX_HEIGHT,
                g_shader_dlg.window.hwnd, (HMENU)(size_t)i, NULL, NULL);
-         SendMessage(control->checkbox.hwnd, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
+         SendMessage(control->elems.checkbox.hwnd, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
          pos_y += SHADER_DLG_CHECKBOX_HEIGHT + SHADER_DLG_CTRL_MARGIN;
       }
       else
@@ -278,29 +280,29 @@ void shader_dlg_params_reload(void)
             pos_x += SHADER_DLG_WIDTH;
          }
 
-         control->type                 = SHADER_PARAM_CTRL_TRACKBAR;
-         control->trackbar.label_title = CreateWindowEx(0, "STATIC",
+         control->type                       = SHADER_PARAM_CTRL_TRACKBAR;
+         control->elems.trackbar.label_title = CreateWindowEx(0, "STATIC",
                shader_info.data->parameters[i].desc,
                WS_CHILD | WS_VISIBLE | SS_LEFT, pos_x, pos_y,
                SHADER_DLG_CTRL_WIDTH, SHADER_DLG_LABEL_HEIGHT, g_shader_dlg.window.hwnd,
                (HMENU)(size_t)i, NULL, NULL);
-         SendMessage(control->trackbar.label_title, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
+         SendMessage(control->elems.trackbar.label_title, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
 
          pos_y += SHADER_DLG_LABEL_HEIGHT;
-         control->trackbar.hwnd = CreateWindowEx(0, TRACKBAR_CLASS, "",
+         control->elems.trackbar.hwnd = CreateWindowEx(0, (LPCSTR)TRACKBAR_CLASS, "",
                WS_CHILD | WS_VISIBLE | TBS_HORZ | TBS_NOTICKS,
                pos_x + SHADER_DLG_TRACKBAR_LABEL_WIDTH, pos_y,
                SHADER_DLG_TRACKBAR_WIDTH, SHADER_DLG_TRACKBAR_HEIGHT,
                g_shader_dlg.window.hwnd, (HMENU)(size_t)i, NULL, NULL);
 
-         control->trackbar.label_val = CreateWindowEx(0, "STATIC", "",
+         control->elems.trackbar.label_val = CreateWindowEx(0, "STATIC", "",
                WS_CHILD | WS_VISIBLE | SS_LEFT, pos_x,
                pos_y, SHADER_DLG_TRACKBAR_LABEL_WIDTH, SHADER_DLG_LABEL_HEIGHT,
                g_shader_dlg.window.hwnd, (HMENU)(size_t)i, NULL, NULL);
-         SendMessage(control->trackbar.label_val, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
+         SendMessage(control->elems.trackbar.label_val, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
 
-         SendMessage(control->trackbar.hwnd, TBM_SETBUDDY, (WPARAM)TRUE,
-               (LPARAM)control->trackbar.label_val);
+         SendMessage(control->elems.trackbar.hwnd, TBM_SETBUDDY, (WPARAM)TRUE,
+               (LPARAM)control->elems.trackbar.label_val);
 
          pos_y += SHADER_DLG_TRACKBAR_HEIGHT + SHADER_DLG_CTRL_MARGIN;
 
@@ -329,17 +331,18 @@ void shader_dlg_params_reload(void)
 #endif
 }
 
+#ifdef HAVE_SHADERPIPELINE
 static void shader_dlg_update_on_top_state(void)
 {
    bool on_top = SendMessage(g_shader_dlg.on_top_checkbox.hwnd,
          BM_GETCHECK, 0, 0) == BST_CHECKED;
 
-   SetWindowPos(g_shader_dlg.window.hwnd, on_top 
+   SetWindowPos(g_shader_dlg.window.hwnd, on_top
          ? HWND_TOPMOST : HWND_NOTOPMOST , 0, 0, 0, 0,
          SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
 }
 
-void shader_dlg_show(HWND parent_hwnd)
+static void shader_dlg_show(HWND parent_hwnd)
 {
    const ui_window_t *window = ui_companion_driver_get_window_ptr();
 
@@ -364,8 +367,9 @@ void shader_dlg_show(HWND parent_hwnd)
 
    window->set_focused(&g_shader_dlg.window);
 }
+#endif
 
-#if defined(HAVE_OPENGL) || defined(HAVE_VULKAN)
+#if 0
 static LRESULT CALLBACK ShaderDlgWndProc(HWND hwnd, UINT message,
       WPARAM wparam, LPARAM lparam)
 {
@@ -403,12 +407,12 @@ static LRESULT CALLBACK ShaderDlgWndProc(HWND hwnd, UINT message,
             video_shader_ctx_t shader_info;
             video_shader_driver_get_current_shader(&shader_info);
 
-            if (SendMessage(g_shader_dlg.controls[i].checkbox.hwnd,
+            if (SendMessage(g_shader_dlg.controls[i].elems.checkbox.hwnd,
                      BM_GETCHECK, 0, 0) == BST_CHECKED)
-               shader_info.data->parameters[i].current = 
+               shader_info.data->parameters[i].current =
                   shader_info.data->parameters[i].maximum;
             else
-               shader_info.data->parameters[i].current = 
+               shader_info.data->parameters[i].current =
                   shader_info.data->parameters[i].minimum;
          }
          break;
@@ -425,11 +429,11 @@ static LRESULT CALLBACK ShaderDlgWndProc(HWND hwnd, UINT message,
             if (g_shader_dlg.controls[i].type != SHADER_PARAM_CTRL_TRACKBAR)
                break;
 
-            pos = (int)SendMessage(g_shader_dlg.controls[i].trackbar.hwnd, TBM_GETPOS, 0, 0);
+            pos = (int)SendMessage(g_shader_dlg.controls[i].elems.trackbar.hwnd, TBM_GETPOS, 0, 0);
 
             {
 
-               shader_info.data->parameters[i].current = 
+               shader_info.data->parameters[i].current =
                   shader_info.data->parameters[i].minimum + pos * shader_info.data->parameters[i].step;
             }
 
@@ -442,43 +446,9 @@ static LRESULT CALLBACK ShaderDlgWndProc(HWND hwnd, UINT message,
 
    return DefWindowProc(hwnd, message, wparam, lparam);
 }
-#endif
 
-bool win32_window_init(WNDCLASSEX *wndclass,
-      bool fullscreen, const char *class_name)
+static bool win32_shader_dlg_init(void)
 {
-   wndclass->cbSize        = sizeof(WNDCLASSEX);
-   wndclass->style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
-   wndclass->hInstance     = GetModuleHandle(NULL);
-   wndclass->hCursor       = LoadCursor(NULL, IDC_ARROW);
-   wndclass->lpszClassName = (class_name != NULL) ? class_name : "RetroArch";
-   wndclass->hIcon         = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICON));
-   wndclass->hIconSm       = (HICON)LoadImage(GetModuleHandle(NULL),
-         MAKEINTRESOURCE(IDI_ICON), IMAGE_ICON, 16, 16, 0);
-   if (!fullscreen)
-      wndclass->hbrBackground = (HBRUSH)COLOR_WINDOW;
-
-   if (class_name != NULL)
-      wndclass->style         |= CS_CLASSDC;
-
-   if (!RegisterClassEx(wndclass))
-      return false;
-
-   /* This is non-NULL when we want a window for shader dialogs, 
-    * therefore early return here */
-   /* TODO/FIXME - this is ugly. Find a better way */
-   if (class_name != NULL) 
-      return true;
-
-   /* Shader dialog is disabled for now, until video_threaded issues are fixed.
-   if (!win32_shader_dlg_init())
-      RARCH_ERR("[WGL]: wgl_shader_dlg_init() failed.\n");*/
-   return true;
-}
-
-bool win32_shader_dlg_init(void)
-{
-#if defined(HAVE_OPENGL) || defined(HAVE_VULKAN)
    static bool inited = false;
    int pos_y;
    HFONT hFont;
@@ -526,29 +496,64 @@ bool win32_shader_dlg_init(void)
    pos_y +=  SHADER_DLG_SEPARATOR_HEIGHT + SHADER_DLG_CTRL_MARGIN;
 
    g_shader_dlg.parameters_start_y = pos_y;
-#endif
    return true;
 }
+#endif
+
+bool win32_window_init(WNDCLASSEX *wndclass,
+      bool fullscreen, const char *class_name)
+{
+   wndclass->cbSize        = sizeof(WNDCLASSEX);
+   wndclass->style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
+   wndclass->hInstance     = GetModuleHandle(NULL);
+   wndclass->hCursor       = LoadCursor(NULL, IDC_ARROW);
+   wndclass->lpszClassName = (class_name != NULL) ? class_name : "RetroArch";
+   wndclass->hIcon         = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICON));
+   wndclass->hIconSm       = (HICON)LoadImage(GetModuleHandle(NULL),
+         MAKEINTRESOURCE(IDI_ICON), IMAGE_ICON, 16, 16, 0);
+   if (!fullscreen)
+      wndclass->hbrBackground = (HBRUSH)COLOR_WINDOW;
+
+   if (class_name != NULL)
+      wndclass->style         |= CS_CLASSDC;
+
+   if (!RegisterClassEx(wndclass))
+      return false;
+
+   /* This is non-NULL when we want a window for shader dialogs,
+    * therefore early return here */
+   /* TODO/FIXME - this is ugly. Find a better way */
+   if (class_name != NULL)
+      return true;
+
+   /* Shader dialog is disabled for now, until
+    * video_threaded issues are fixed.
+   if (!win32_shader_dlg_init())
+      RARCH_ERR("[WGL]: wgl_shader_dlg_init() failed.\n");*/
+   return true;
+}
+
 
 static bool win32_browser(
       HWND owner,
       char *filename,
+      size_t filename_size,
       const char *extensions,
       const char *title,
       const char *initial_dir)
 {
    bool result = false;
-   const ui_browser_window_t *browser = 
+   const ui_browser_window_t *browser =
       ui_companion_driver_get_browser_window_ptr();
 
    if (browser)
    {
       ui_browser_window_state_t browser_state;
-      /* OPENFILENAME.lpstrFilter requires a null separated list of name/ext pairs terminated by a second null at the end. */
-      char *all_files[] = {"All Files (*.*)", "*.*", ""};
 
-      /* These need to be big enough to hold the path/name of any file the user may select.
-       * FIXME: We should really handle the error case when this isn't big enough. */
+      /* These need to be big enough to hold the
+       * path/name of any file the user may select.
+       * FIXME: We should really handle the
+       * error case when this isn't big enough. */
       char new_title[TITLE_MAX];
       char new_file[FULLPATH_MAX];
 
@@ -561,7 +566,9 @@ static bool win32_browser(
       if (filename && *filename)
          strlcpy(new_file, filename, sizeof(new_file));
 
-      browser_state.filters  = all_files[0];
+      /* OPENFILENAME.lpstrFilters is actually const,
+       * so this cast should be safe */
+      browser_state.filters  = (char*)extensions;
       browser_state.title    = new_title;
       browser_state.startdir = strdup("");
       browser_state.path     = new_file;
@@ -569,6 +576,9 @@ static bool win32_browser(
 
       result = browser->open(&browser_state);
 
+      if (filename && browser_state.path)
+         strlcpy(filename, browser_state.path, filename_size);
+
       free(browser_state.startdir);
    }
 
@@ -597,33 +607,41 @@ LRESULT win32_menu_loop(HWND owner, WPARAM wparam)
 
             switch (mode)
             {
+               /* OPENFILENAME.lpstrFilter requires
+                * a NULL-separated list of name/ext
+                * pairs terminated by a second null at the end. */
                case ID_M_LOAD_CORE:
-                  extensions  = "Libretro core (.dll)\0*.dll\0All Files\0*.*\0";
+                  extensions  = "Libretro core (.dll)\0*.dll\0All Files\0*.*\0\0";
                   title       = msg_hash_to_str(MENU_ENUM_LABEL_VALUE_CORE_LIST);
-                  initial_dir = settings->directory.libretro;
+                  initial_dir = settings->paths.directory_libretro;
                   break;
                case ID_M_LOAD_CONTENT:
-                  extensions  = "All Files (*.*)\0*.*\0";
+                  extensions  = "All Files (*.*)\0*.*\0\0";
                   title       = msg_hash_to_str(
                         MENU_ENUM_LABEL_VALUE_LOAD_CONTENT_LIST);
-                  initial_dir = settings->directory.menu_content;
+                  initial_dir = settings->paths.directory_menu_content;
                   break;
             }
 
-            /* Convert UTF8 to UTF16, then back to the local code page.
-             * This is needed for proper multi-byte string display until Unicode is fully supported.
+            /* Convert UTF8 to UTF16, then back to the
+             * local code page.
+             * This is needed for proper multi-byte
+             * string display until Unicode is
+             * fully supported.
              */
-            MultiByteToWideChar(CP_UTF8, 0, title, -1, title_wide, sizeof(title_wide) / sizeof(title_wide[0]));
+            MultiByteToWideChar(CP_UTF8, 0, title, -1,
+                  title_wide,
+                  sizeof(title_wide) / sizeof(title_wide[0]));
             wcstombs(title_cp, title_wide, sizeof(title_cp) - 1);
 
-            if (!win32_browser(owner, win32_file,
+            if (!win32_browser(owner, win32_file, sizeof(win32_file),
                      extensions, title_cp, initial_dir))
                break;
 
             switch (mode)
             {
                case ID_M_LOAD_CORE:
-                  runloop_ctl(RUNLOOP_CTL_SET_LIBRETRO_PATH, win32_file);
+                  rarch_ctl(RARCH_CTL_SET_LIBRETRO_PATH, win32_file);
                   cmd         = CMD_EVENT_LOAD_CORE;
                   break;
                case ID_M_LOAD_CONTENT:
@@ -633,11 +651,10 @@ LRESULT win32_menu_loop(HWND owner, WPARAM wparam)
                      path_set(RARCH_PATH_CONTENT, win32_file);
 
                      do_wm_close = true;
-                     task_push_content_load_default(
-                           NULL, NULL,
+                     task_push_load_content_with_current_core_from_companion_ui(
+                           NULL,
                            &content_info,
                            CORE_TYPE_PLAIN,
-                           CONTENT_MODE_LOAD_CONTENT_WITH_CURRENT_CORE_FROM_COMPANION_UI,
                            NULL, NULL);
                   }
                   break;
@@ -674,11 +691,11 @@ LRESULT win32_menu_loop(HWND owner, WPARAM wparam)
       case ID_M_FULL_SCREEN:
          cmd = CMD_EVENT_FULLSCREEN_TOGGLE;
          break;
-#ifndef _XBOX
       case ID_M_SHADER_PARAMETERS:
+#if !defined(_XBOX) && defined(HAVE_SHADERPIPELINE)
          shader_dlg_show(owner);
-         break;
 #endif
+         break;
       case ID_M_MOUSE_GRAB:
          cmd = CMD_EVENT_GRAB_MOUSE_TOGGLE;
          break;
@@ -692,19 +709,21 @@ LRESULT win32_menu_loop(HWND owner, WPARAM wparam)
          if (mode >= ID_M_WINDOW_SCALE_1X && mode <= ID_M_WINDOW_SCALE_10X)
          {
             unsigned idx = (mode - (ID_M_WINDOW_SCALE_1X-1));
-            runloop_ctl(RUNLOOP_CTL_SET_WINDOWED_SCALE, &idx);
+            rarch_ctl(RARCH_CTL_SET_WINDOWED_SCALE, &idx);
             cmd = CMD_EVENT_RESIZE_WINDOWED_SCALE;
          }
          else if (mode == ID_M_STATE_INDEX_AUTO)
          {
             signed idx = -1;
-            settings->state_slot = idx;
+            configuration_set_int(
+                  settings, settings->ints.state_slot, idx);
          }
-         else if (mode >= (ID_M_STATE_INDEX_AUTO+1) 
+         else if (mode >= (ID_M_STATE_INDEX_AUTO+1)
                && mode <= (ID_M_STATE_INDEX_AUTO+10))
          {
             signed idx = (mode - (ID_M_STATE_INDEX_AUTO+1));
-            settings->state_slot = idx;
+            configuration_set_int(
+                  settings, settings->ints.state_slot, idx);
          }
          break;
    }
@@ -714,7 +733,7 @@ LRESULT win32_menu_loop(HWND owner, WPARAM wparam)
 
    if (do_wm_close)
       PostMessage(owner, WM_CLOSE, 0, 0);
-   
+
    return 0L;
 }
 
@@ -750,9 +769,10 @@ static void ui_companion_win32_notify_content_loaded(void *data)
    (void)data;
 }
 
-static void ui_companion_win32_toggle(void *data)
+static void ui_companion_win32_toggle(void *data, bool force)
 {
    (void)data;
+   (void)force;
 }
 
 static void ui_companion_win32_event_command(
@@ -770,7 +790,7 @@ static void ui_companion_win32_notify_list_pushed(void *data,
     (void)menu_list;
 }
 
-const ui_companion_driver_t ui_companion_win32 = {
+ui_companion_driver_t ui_companion_win32 = {
    ui_companion_win32_init,
    ui_companion_win32_deinit,
    ui_companion_win32_iterate,
@@ -782,6 +802,7 @@ const ui_companion_driver_t ui_companion_win32 = {
    NULL,
    NULL,
    NULL,
+   NULL,
    &ui_browser_window_win32,
    &ui_msg_window_win32,
    &ui_window_win32,
