@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2015 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -22,14 +22,15 @@
 
 #include <boolean.h>
 #include <compat/strl.h>
+#include <retro_inline.h>
 
-#include "../input_config.h"
-#include "../input_joypad_driver.h"
+#include "../input_driver.h"
 #include "../input_keymaps.h"
 
 #include "../../gfx/video_driver.h"
 #include "../common/input_x11_common.h"
 
+#include "../../configuration.h"
 #include "../../verbosity.h"
 
 typedef struct x11_input
@@ -48,7 +49,6 @@ typedef struct x11_input
    bool grab_mouse;
 } x11_input_t;
 
-
 static void *x_input_init(const char *joypad_driver)
 {
    x11_input_t *x11;
@@ -73,29 +73,73 @@ static void *x_input_init(const char *joypad_driver)
    return x11;
 }
 
-static bool x_key_pressed(x11_input_t *x11, int key)
+static bool x_keyboard_pressed(x11_input_t *x11, unsigned key)
 {
-   unsigned sym;
-   int keycode;
-   bool ret;
+   int keycode = XKeysymToKeycode(x11->display, rarch_keysym_lut[(enum retro_key)key]);
+   return x11->state[keycode >> 3] & (1 << (keycode & 7));
+}
 
-   if (key >= RETROK_LAST)
+static bool x_mbutton_pressed(x11_input_t *x11, unsigned port, unsigned key)
+{
+   bool result;
+   settings_t *settings = config_get_ptr();
+
+   if (port >= MAX_USERS)
       return false;
 
-   sym     = input_keymaps_translate_rk_to_keysym((enum retro_key)key);
-   keycode = XKeysymToKeycode(x11->display, sym);
-   ret     = x11->state[keycode >> 3] & (1 << (keycode & 7));
+   /* the driver only supports one mouse */
+   if ( settings->uints.input_mouse_index[ port ] != 0 )
+      return false;
 
-   return ret;
+   switch ( key )
+   {
+
+   case RETRO_DEVICE_ID_MOUSE_LEFT:
+      return x11->mouse_l;
+   case RETRO_DEVICE_ID_MOUSE_RIGHT:
+      return x11->mouse_r;
+   case RETRO_DEVICE_ID_MOUSE_MIDDLE:
+      return x11->mouse_m;
+/*   case RETRO_DEVICE_ID_MOUSE_BUTTON_4:
+      return x11->mouse_b4;*/
+/*   case RETRO_DEVICE_ID_MOUSE_BUTTON_5:
+      return x11->mouse_b5;*/
+
+   case RETRO_DEVICE_ID_MOUSE_WHEELUP:
+   case RETRO_DEVICE_ID_MOUSE_WHEELDOWN:
+      return x_mouse_state_wheel( key );
+
+/*   case RETRO_DEVICE_ID_MOUSE_HORIZ_WHEELUP:
+      result = x11->mouse_hwu;
+      x11->mouse_hwu = false;
+      return result;
+
+   case RETRO_DEVICE_ID_MOUSE_HORIZ_WHEELDOWN:
+      result = x11->mouse_hwd;
+      x11->mouse_hwd = false;
+      return result;
+*/
+   }
+
+   return false;
 }
 
 static bool x_is_pressed(x11_input_t *x11,
-      const struct retro_keybind *binds, unsigned id)
+      rarch_joypad_info_t joypad_info,
+      const struct retro_keybind *binds,
+      unsigned port, unsigned id)
 {
-   if (id < RARCH_BIND_LIST_END)
+   const struct retro_keybind *bind = &binds[id];
+
+   if ( (bind->key < RETROK_LAST) && x_keyboard_pressed(x11, bind->key) )
+      return true;
+
+   if (binds && binds[id].valid)
    {
-      const struct retro_keybind *bind = &binds[id];
-      return x_key_pressed(x11, binds[id].key);
+      if (x_mbutton_pressed(x11, port, bind->mbutton))
+         return true;
+      if (input_joypad_pressed(x11->joypad, joypad_info, port, binds, id))
+         return true;
    }
 
    return false;
@@ -104,23 +148,73 @@ static bool x_is_pressed(x11_input_t *x11,
 static int16_t x_pressed_analog(x11_input_t *x11,
       const struct retro_keybind *binds, unsigned idx, unsigned id)
 {
-   int16_t pressed_minus = 0, pressed_plus = 0;
-   unsigned id_minus = 0;
-   unsigned id_plus  = 0;
+   int16_t pressed_minus = 0;
+   int16_t pressed_plus  = 0;
+   unsigned id_minus     = 0;
+   unsigned id_plus      = 0;
+   int id_minus_key      = 0;
+   int id_plus_key       = 0;
+   unsigned sym          = 0;
+   int keycode           = 0;
 
    input_conv_analog_id_to_bind_id(idx, id, &id_minus, &id_plus);
 
-   if (binds && binds[id_minus].valid && x_is_pressed(x11, binds, id_minus))
+   id_minus_key          = binds[id_minus].key;
+   id_plus_key           = binds[id_plus].key;
+
+   sym                   = rarch_keysym_lut[(enum retro_key)id_minus_key];
+   keycode               = XKeysymToKeycode(x11->display, sym);
+   if (      binds[id_minus].valid
+         && (id_minus_key < RETROK_LAST)
+         && (x11->state[keycode >> 3] & (1 << (keycode & 7))))
       pressed_minus = -0x7fff;
-   if (binds && binds[id_plus].valid && x_is_pressed(x11, binds, id_plus))
+
+   sym                   = rarch_keysym_lut[(enum retro_key)id_plus_key];
+   keycode               = XKeysymToKeycode(x11->display, sym);
+   if (      binds[id_plus].valid
+         && (id_plus_key < RETROK_LAST)
+         && (x11->state[keycode >> 3] & (1 << (keycode & 7))))
       pressed_plus  =  0x7fff;
 
    return pressed_plus + pressed_minus;
 }
 
-static bool x_input_meta_key_pressed(void *data, int key)
+static int16_t x_lightgun_aiming_state( x11_input_t *x11, unsigned idx, unsigned id )
 {
-   return false;
+   const int edge_detect = 32700;
+   struct video_viewport vp;
+   bool inside                 = false;
+   int16_t res_x               = 0;
+   int16_t res_y               = 0;
+   int16_t res_screen_x        = 0;
+   int16_t res_screen_y        = 0;
+
+   vp.x                        = 0;
+   vp.y                        = 0;
+   vp.width                    = 0;
+   vp.height                   = 0;
+   vp.full_width               = 0;
+   vp.full_height              = 0;
+
+   if (!(video_driver_translate_coord_viewport_wrap(&vp, x11->mouse_x, x11->mouse_y,
+         &res_x, &res_y, &res_screen_x, &res_screen_y)))
+      return 0;
+
+   inside = (res_x >= -edge_detect) && (res_y >= -edge_detect) && (res_x <= edge_detect) && (res_y <= edge_detect);
+
+   switch ( id )
+   {
+   case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_X:
+      return inside ? res_x : 0;
+   case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_Y:
+      return inside ? res_y : 0;
+   case RETRO_DEVICE_ID_LIGHTGUN_IS_OFFSCREEN:
+      return !inside;
+   default:
+      break;
+   }
+
+   return 0;
 }
 
 static int16_t x_mouse_state(x11_input_t *x11, unsigned id)
@@ -163,9 +257,19 @@ static int16_t x_mouse_state_screen(x11_input_t *x11, unsigned id)
 static int16_t x_pointer_state(x11_input_t *x11,
       unsigned idx, unsigned id, bool screen)
 {
-   bool inside;
-   struct video_viewport vp = {0};
-   int16_t res_x = 0, res_y = 0, res_screen_x = 0, res_screen_y = 0;
+   struct video_viewport vp;
+   bool inside                 = false;
+   int16_t res_x               = 0;
+   int16_t res_y               = 0;
+   int16_t res_screen_x        = 0;
+   int16_t res_screen_y        = 0;
+
+   vp.x                        = 0;
+   vp.y                        = 0;
+   vp.width                    = 0;
+   vp.height                   = 0;
+   vp.full_width               = 0;
+   vp.full_height              = 0;
 
    if (!(video_driver_translate_coord_viewport_wrap(&vp, x11->mouse_x, x11->mouse_y,
          &res_x, &res_y, &res_screen_x, &res_screen_y)))
@@ -195,29 +299,6 @@ static int16_t x_pointer_state(x11_input_t *x11,
    return 0;
 }
 
-static int16_t x_lightgun_state(x11_input_t *x11, unsigned id)
-{
-   switch (id)
-   {
-      case RETRO_DEVICE_ID_LIGHTGUN_X:
-         return x11->mouse_x - x11->mouse_last_x;
-      case RETRO_DEVICE_ID_LIGHTGUN_Y:
-         return x11->mouse_y - x11->mouse_last_y;
-      case RETRO_DEVICE_ID_LIGHTGUN_TRIGGER:
-         return x11->mouse_l;
-      case RETRO_DEVICE_ID_LIGHTGUN_CURSOR:
-         return x11->mouse_m;
-      case RETRO_DEVICE_ID_LIGHTGUN_TURBO:
-         return x11->mouse_r;
-      case RETRO_DEVICE_ID_LIGHTGUN_START:
-         return x11->mouse_m && x11->mouse_r; 
-      case RETRO_DEVICE_ID_LIGHTGUN_PAUSE:
-         return x11->mouse_m && x11->mouse_l; 
-   }
-
-   return 0;
-}
-
 static int16_t x_input_state(void *data,
       rarch_joypad_info_t joypad_info,
       const struct retro_keybind **binds, unsigned port,
@@ -229,19 +310,18 @@ static int16_t x_input_state(void *data,
    switch (device)
    {
       case RETRO_DEVICE_JOYPAD:
-         return x_is_pressed(x11, binds[port], id) ||
-            input_joypad_pressed(x11->joypad, joypad_info, port, binds[port], id);
+         if (id < RARCH_BIND_LIST_END)
+            return x_is_pressed(x11, joypad_info, binds[port], port, id);
+         break;
       case RETRO_DEVICE_KEYBOARD:
-         return x_key_pressed(x11, id);
+         return (id < RETROK_LAST) && x_keyboard_pressed(x11, id);
       case RETRO_DEVICE_ANALOG:
-         if (binds[port])
-            ret = x_pressed_analog(x11, binds[port], idx, id);
+         ret = x_pressed_analog(x11, binds[port], idx, id);
          if (!ret && binds[port])
             ret = input_joypad_analog(x11->joypad, joypad_info,
                   port, idx,
                   id, binds[port]);
          return ret;
-
       case RETRO_DEVICE_MOUSE:
          return x_mouse_state(x11, id);
       case RARCH_DEVICE_MOUSE_SCREEN:
@@ -254,7 +334,48 @@ static int16_t x_input_state(void *data,
                   device == RARCH_DEVICE_POINTER_SCREEN);
          break;
       case RETRO_DEVICE_LIGHTGUN:
-         return x_lightgun_state(x11, id);
+         switch ( id )
+         {
+            /*aiming*/
+            case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_X:
+            case RETRO_DEVICE_ID_LIGHTGUN_SCREEN_Y:
+            case RETRO_DEVICE_ID_LIGHTGUN_IS_OFFSCREEN:
+               return x_lightgun_aiming_state( x11, idx, id );
+
+            /*buttons*/
+            case RETRO_DEVICE_ID_LIGHTGUN_TRIGGER:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_TRIGGER);
+            case RETRO_DEVICE_ID_LIGHTGUN_RELOAD:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_RELOAD);
+            case RETRO_DEVICE_ID_LIGHTGUN_AUX_A:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_AUX_A);
+            case RETRO_DEVICE_ID_LIGHTGUN_AUX_B:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_AUX_B);
+            case RETRO_DEVICE_ID_LIGHTGUN_AUX_C:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_AUX_C);
+            case RETRO_DEVICE_ID_LIGHTGUN_START:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_START);
+            case RETRO_DEVICE_ID_LIGHTGUN_SELECT:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_SELECT);
+            case RETRO_DEVICE_ID_LIGHTGUN_DPAD_UP:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_DPAD_UP);
+            case RETRO_DEVICE_ID_LIGHTGUN_DPAD_DOWN:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_DPAD_DOWN);
+            case RETRO_DEVICE_ID_LIGHTGUN_DPAD_LEFT:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_DPAD_LEFT);
+            case RETRO_DEVICE_ID_LIGHTGUN_DPAD_RIGHT:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_DPAD_RIGHT);
+
+            /*deprecated*/
+            case RETRO_DEVICE_ID_LIGHTGUN_X:
+               return x11->mouse_x - x11->mouse_last_x;
+            case RETRO_DEVICE_ID_LIGHTGUN_Y:
+               return x11->mouse_y - x11->mouse_last_y;
+            case RETRO_DEVICE_ID_LIGHTGUN_PAUSE:
+               return x_is_pressed(x11, joypad_info, binds[port], port, RARCH_LIGHTGUN_START);
+
+         }
+         break;
    }
 
    return 0;
@@ -295,15 +416,22 @@ static void x_input_poll_mouse(x11_input_t *x11)
    {
       x11->mouse_x  = win_x;
       x11->mouse_y  = win_y;
-      x11->mouse_l  = mask & Button1Mask; 
-      x11->mouse_m  = mask & Button2Mask; 
-      x11->mouse_r  = mask & Button3Mask; 
+      x11->mouse_l  = mask & Button1Mask;
+      x11->mouse_m  = mask & Button2Mask;
+      x11->mouse_r  = mask & Button3Mask;
 
       /* Somewhat hacky, but seem to do the job. */
-      if (x11->grab_mouse && video_driver_is_focused())
+      if (x11->grab_mouse && video_driver_cb_has_focus())
       {
          int mid_w, mid_h;
-         struct video_viewport vp = {0};
+         struct video_viewport vp;
+
+         vp.x                        = 0;
+         vp.y                        = 0;
+         vp.width                    = 0;
+         vp.height                   = 0;
+         vp.full_width               = 0;
+         vp.full_height              = 0;
 
          video_driver_get_viewport_info(&vp);
 
@@ -328,7 +456,7 @@ static void x_input_poll(void *data)
 {
    x11_input_t *x11 = (x11_input_t*)data;
 
-   if (video_driver_is_focused())
+   if (video_driver_cb_has_focus())
       XQueryKeymap(x11->display, x11->state);
    else
       memset(x11->state, 0, sizeof(x11->state));
@@ -398,7 +526,6 @@ input_driver_t input_x = {
    x_input_init,
    x_input_poll,
    x_input_state,
-   x_input_meta_key_pressed,
    x_input_free,
    NULL,
    NULL,
