@@ -15,6 +15,8 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define CINTERFACE
+
 #include <stdint.h>
 #include <stddef.h>
 #include <stdlib.h>
@@ -26,6 +28,7 @@
 
 #include <file/file_path.h>
 #include <lists/string_list.h>
+#include <string/stdstring.h>
 #include <queues/message_queue.h>
 
 #include "menu_generic.h"
@@ -35,18 +38,18 @@
 #include "../widgets/menu_entry.h"
 #include "../menu_entries.h"
 #include "../menu_input.h"
-#include "../menu_navigation.h"
 #include "../menu_setting.h"
-#include "../menu_display.h"
 #include "../widgets/menu_input_dialog.h"
 
 #include "../../gfx/video_driver.h"
-#include "../../gfx/video_context_driver.h"
 
 #include "../../configuration.h"
-#include "../../runloop.h"
+#include "../../retroarch.h"
+#include "../../verbosity.h"
 
 #include "../../gfx/drivers/d3d.h"
+#include "../../gfx/common/d3d_common.h"
+#include "../../gfx/common/d3d9_common.h"
 
 #define XUI_CONTROL_NAVIGATE_OK (XUI_CONTROL_NAVIGATE_RIGHT + 1)
 
@@ -67,6 +70,7 @@ HXUIOBJ m_back;
 HXUIOBJ root_menu;
 HXUIOBJ current_menu;
 static msg_queue_t *xui_msg_queue = NULL;
+static uint64_t xui_frame_count   = 0;
 
 class CRetroArch : public CXuiModule
 {
@@ -165,7 +169,8 @@ HRESULT CRetroArchMain::OnInit(XUIMessageInit * pInitData, BOOL& bHandled)
    {
       char str[PATH_MAX_LENGTH] = {0};
 
-      if (menu_entries_get_core_title(str, sizeof(str)) == 0)
+      if (
+            menu_entries_get_core_title(str, sizeof(str)) == 0)
       {
          mbstowcs(strw_buffer, str, sizeof(strw_buffer) / sizeof(wchar_t));
          XuiTextElementSetText(m_menutitlebottom, strw_buffer);
@@ -234,7 +239,7 @@ HRESULT XuiTextureLoader(IXuiDevice *pDevice, LPCWSTR szFileName,
          D3DX_DEFAULT_NONPOW2,
          1,
          D3DUSAGE_CPU_CACHED_MEMORY,
-         D3DFMT_LIN_A8R8G8B8,
+         (D3DFORMAT)d3d9_get_argb8888_format(),
          D3DPOOL_DEFAULT,
          D3DX_FILTER_NONE,
          D3DX_FILTER_NONE,
@@ -269,10 +274,12 @@ cleanup:
    return hr;
 }
 
-static void* xui_init(void **userdata)
+void d3d9_make_d3dpp(void *data, const video_info_t *info, void *_d3dpp);
+
+static void* xui_init(void **userdata, bool video_is_threaded)
 {
    HRESULT hr;
-   d3d_video_t *d3d            = NULL;
+   d3d9_video_t *d3d           = NULL;
    D3DPRESENT_PARAMETERS d3dpp = {0};
    video_info_t video_info     = {0};
    TypefaceDescriptor typeface = {0};
@@ -282,21 +289,21 @@ static void* xui_init(void **userdata)
    if (!menu)
       return NULL;
 
-   d3d = (d3d_video_t*)video_driver_get_ptr(false);
+   d3d = (d3d9_video_t*)video_driver_get_ptr(false);
 
    if (d3d->resolution_hd_enable)
       RARCH_LOG("HD menus enabled.\n");
 
-   video_info.vsync        = settings->video.vsync;
+   video_info.vsync        = settings->bools.video_vsync;
    video_info.force_aspect = false;
-   video_info.smooth       = settings->video.smooth;
+   video_info.smooth       = settings->bools.video_smooth;
    video_info.input_scale  = 2;
    video_info.fullscreen   = true;
    video_info.rgb32        = false;
 
-   d3d_make_d3dpp(d3d, &video_info, &d3dpp);
+   d3d9_make_d3dpp(d3d, &video_info, &d3dpp);
 
-   hr = app.InitShared(d3d->dev, &d3dpp,
+   hr = app.InitShared((D3DDevice*)d3d->dev, &d3dpp,
          (PFN_XUITEXTURELOADER)XuiTextureLoader);
 
    if (FAILED(hr))
@@ -360,6 +367,8 @@ static void xui_free(void *data)
    (void)data;
    app.Uninit();
 
+   xui_frame_count = 0;
+
    if (xui_msg_queue)
       msg_queue_free(xui_msg_queue);
 }
@@ -370,7 +379,7 @@ static void xui_render_message(const char *msg)
    size_t i                      = 0;
    size_t j                      = 0;
    struct string_list *list      = NULL;
-   d3d_video_t              *d3d = (d3d_video_t*)video_driver_get_ptr(false);
+   d3d9_video_t             *d3d = (d3d9_video_t*)video_driver_get_ptr(false);
 
    if (!d3d)
       return;
@@ -407,18 +416,14 @@ static void xui_frame(void *data, video_frame_info_t *video_info)
    XUIMessage msg;
    XUIMessageRender msgRender;
    D3DXMATRIX matOrigView;
-   LPDIRECT3DDEVICE d3dr;
    const char *message   = NULL;
-   D3DVIEWPORT vp_full   = {0};
-   d3d_video_t *d3d      = (d3d_video_t*)video_driver_get_ptr(false);
+   D3DVIEWPORT9 vp_full  = {0};
+   d3d9_video_t *d3d     = (d3d9_video_t*)video_driver_get_ptr(false);
 
    if (!d3d)
       return;
 
-   d3dr = (LPDIRECT3DDEVICE)d3d->dev;
-
-   if (!d3dr)
-      return;
+   xui_frame_count++;
 
    menu_display_set_viewport(video_info->width, video_info->height);
 
@@ -434,17 +439,20 @@ static void xui_frame(void *data, video_frame_info_t *video_info)
 
    XuiRenderSetViewTransform( app.GetDC(), &matOrigView );
 
-   runloop_ctl(RUNLOOP_CTL_MSG_QUEUE_PULL, &message);
+#if 0
+   /* TODO/FIXME - update this code */
+   rarch_ctl(RARCH_CTL_MSG_QUEUE_PULL, &message);
 
    if (message)
       xui_render_message(message);
    else
    {
-      runloop_ctl(RUNLOOP_CTL_MSG_QUEUE_PULL, &message);
+      rarch_ctl(RARCH_CTL_MSG_QUEUE_PULL, &message);
 
       if (message)
          xui_render_message(message);
    }
+#endif
 
    XuiRenderEnd( app.GetDC() );
 
@@ -457,9 +465,12 @@ static void blit_line(int x, int y, const char *message, bool green)
 
 static void xui_render_background(void)
 {
+#if 0
+   /* TODO/FIXME - refactor this */
    if (menu_display_libretro_running())
       XuiElementSetShow(m_background, FALSE);
    else
+#endif
       XuiElementSetShow(m_background, TRUE);
 }
 
@@ -526,7 +537,7 @@ static void xui_set_list_text(int index, const wchar_t* leftText,
    }
 }
 
-static void xui_render(void *data)
+static void xui_render(void *data, bool is_idle)
 {
    size_t end, i, selection, fb_pitch;
    unsigned fb_width, fb_height;
@@ -534,7 +545,7 @@ static void xui_render(void *data)
    const char *dir             = NULL;
    const char *label           = NULL;
    unsigned menu_type          = 0;
-   uint64_t *frame_count       = video_driver_get_frame_count_ptr();
+   uint64_t frame_count        = xui_frame_count;
    bool              msg_force = menu_display_get_msg_force();
 
    menu_display_get_fb_size(&fb_width, &fb_height,
@@ -554,40 +565,55 @@ static void xui_render(void *data)
 
    if (XuiHandleIsValid(m_menutitle))
    {
+      menu_animation_ctx_ticker_t ticker;
       menu_entries_get_title(title, sizeof(title));
       mbstowcs(strw_buffer, title, sizeof(strw_buffer) / sizeof(wchar_t));
       XuiTextElementSetText(m_menutitle, strw_buffer);
-      menu_animation_ticker_str(title, RXUI_TERM_WIDTH(fb_width) - 3,
-            (unsigned int)*frame_count / 15, title, true);
+
+	  ticker.s        = title;
+	  ticker.len      = RXUI_TERM_WIDTH(fb_width) - 3;
+	  ticker.idx      = (unsigned int)frame_count / 15;
+	  ticker.str      = title;
+	  ticker.selected = true;
+
+      menu_animation_ticker(&ticker);
    }
 
    if (XuiHandleIsValid(m_menutitle))
    {
-      if (menu_entries_get_core_title(title, sizeof(title)) == 0)
+      if (
+            menu_entries_get_core_title(title, sizeof(title)) == 0)
       {
          mbstowcs(strw_buffer, title, sizeof(strw_buffer) / sizeof(wchar_t));
          XuiTextElementSetText(m_menutitlebottom, strw_buffer);
       }
    }
 
-   end = menu_entries_get_end();
+   end = menu_entries_get_size();
    for (i = 0; i < end; i++)
    {
-      char entry_path[PATH_MAX_LENGTH]     = {0};
+      menu_entry_t entry;
+      char *entry_path                     = NULL;
       char entry_value[PATH_MAX_LENGTH]    = {0};
       wchar_t msg_right[PATH_MAX_LENGTH]   = {0};
       wchar_t msg_left[PATH_MAX_LENGTH]    = {0};
 
-      menu_entry_get_value(i, NULL, entry_value, sizeof(entry_value));
-      menu_entry_get_path(i, entry_path, sizeof(entry_path));
+      menu_entry_init(&entry);
+      menu_entry_get(&entry, 0, i, NULL, true);
+
+      menu_entry_get_value(&entry, entry_value, sizeof(entry_value));
+      entry_path = menu_entry_get_path(&entry);
 
       mbstowcs(msg_left,  entry_path,  sizeof(msg_left)  / sizeof(wchar_t));
       mbstowcs(msg_right, entry_value, sizeof(msg_right) / sizeof(wchar_t));
       xui_set_list_text(i, msg_left, msg_right);
+
+      menu_entry_free(&entry);
+      if (!string_is_empty(entry_path))
+         free(entry_path);
    }
 
-   if (!menu_navigation_ctl(MENU_NAVIGATION_CTL_GET_SELECTION, &selection))
-      return;
+   selection = menu_navigation_get_selection();
 
    XuiListSetCurSelVisible(m_menulist, selection);
 
@@ -606,31 +632,19 @@ static void xui_populate_entries(void *data,
       const char *path,
       const char *label, unsigned i)
 {
-   size_t selection;
-   if (!menu_navigation_ctl(MENU_NAVIGATION_CTL_GET_SELECTION, &selection))
-      return;
-
-   (void)label;
-   (void)path;
-
+   size_t selection = menu_navigation_get_selection();
    XuiListSetCurSelVisible(m_menulist, selection);
 }
 
 static void xui_navigation_clear(void *data, bool pending_push)
 {
-   size_t selection;
-   if (!menu_navigation_ctl(MENU_NAVIGATION_CTL_GET_SELECTION, &selection))
-      return;
-
+   size_t selection = menu_navigation_get_selection();
    XuiListSetCurSelVisible(m_menulist, selection);
 }
 
 static void xui_navigation_set_visible(void *data)
 {
-   size_t selection;
-   if (!menu_navigation_ctl(MENU_NAVIGATION_CTL_GET_SELECTION, &selection))
-      return;
-
+   size_t selection = menu_navigation_get_selection();
    XuiListSetCurSelVisible(m_menulist, selection);
 }
 
@@ -643,7 +657,9 @@ static void xui_list_insert(void *data,
       file_list_t *list,
       const char *path,
       const char *fullpath,
-      const char *, size_t list_size)
+      const char *unused,
+	  size_t list_size,
+	  unsigned entry_type)
 {
    wchar_t buf[PATH_MAX_LENGTH] = {0};
 
@@ -660,9 +676,9 @@ static void xui_list_free(
 
    (void)idx;
 
-   if( list_size > x)
+   if (list_size > (unsigned)x)
       list_size = x;
-   if( list_size > 0)
+   if (list_size > 0)
       XuiListDeleteItems(m_menulist, 0, list_size);
 }
 
@@ -728,6 +744,8 @@ menu_ctx_driver_t menu_ctx_xui = {
    NULL,          /* pointer_tap */
    NULL,          /* update_thumbnail_path */
    NULL,          /* update_thumbnail_image */
+   NULL,          /* set_thumbnail_system */
+   NULL,          /* set_thumbnail_content */
    NULL,          /* osk_ptr_at_pos */
    NULL,          /* update_savestate_thumbnail_path */
    NULL           /* update_savestate_thumbnail_image */
