@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -36,7 +36,7 @@
 #include "../common/vulkan_common.h"
 #endif
 
-#include "../../frontend/drivers/platform_linux.h"
+#include "../../frontend/drivers/platform_unix.h"
 
 static enum gfx_ctx_api android_api           = GFX_CTX_NONE;
 
@@ -107,13 +107,6 @@ static void *android_gfx_ctx_init(video_frame_info_t *video_info, void *video_dr
    struct retro_hw_render_callback *hwr = video_driver_get_hw_context();
    bool debug = hwr->debug_context;
 #endif
-   EGLint context_attributes[] = {
-      EGL_CONTEXT_CLIENT_VERSION, g_es3 ? 3 : 2,
-#if 0
-      EGL_CONTEXT_FLAGS_KHR, debug ? EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR : 0,
-#endif
-      EGL_NONE
-   };
    EGLint attribs[] = {
       EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
       EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
@@ -143,7 +136,7 @@ static void *android_gfx_ctx_init(video_frame_info_t *video_info, void *video_dr
 #ifdef HAVE_EGL
          RARCH_LOG("Android EGL: GLES version = %d.\n", g_es3 ? 3 : 2);
 
-         if (!egl_init_context(&and->egl, EGL_DEFAULT_DISPLAY,
+         if (!egl_init_context(&and->egl, EGL_NONE, EGL_DEFAULT_DISPLAY,
                   &major, &minor, &n, attribs))
          {
             egl_report_error();
@@ -175,16 +168,6 @@ static void *android_gfx_ctx_init(video_frame_info_t *video_info, void *video_dr
       case GFX_CTX_OPENGL_ES_API:
          ANativeWindow_setBuffersGeometry(android_app->window, 0, 0, format);
 
-#ifdef HAVE_EGL
-         if (!egl_create_context(&and->egl, context_attributes))
-         {
-            egl_report_error();
-            goto unlock_error;
-         }
-
-         if (!egl_create_surface(&and->egl, android_app->window))
-            goto unlock_error;
-#endif
          break;
       case GFX_CTX_NONE:
       default:
@@ -247,7 +230,7 @@ static void android_gfx_ctx_check_window(void *data, bool *quit,
          break;
       case GFX_CTX_VULKAN_API:
 #ifdef HAVE_VULKAN
-         /* Swapchains are recreated in set_resize as a 
+         /* Swapchains are recreated in set_resize as a
           * central place, so use that to trigger swapchain reinit. */
          *resize    = and->vk.need_new_swapchain;
          new_width  = and->width;
@@ -298,6 +281,7 @@ static bool android_gfx_ctx_set_resize(void *data,
             return false;
          }
 
+         vulkan_acquire_next_image(&and->vk);
          and->vk.context.invalid_swapchain = true;
          and->vk.need_new_swapchain        = false;
 #endif
@@ -320,6 +304,15 @@ static bool android_gfx_ctx_set_video_mode(void *data,
    struct android_app *android_app = (struct android_app*)g_android;
    android_ctx_data_t *and = (android_ctx_data_t*)data;
 #endif
+#if defined(HAVE_OPENGLES) && defined(HAVE_EGL)
+   EGLint context_attributes[] = {
+      EGL_CONTEXT_CLIENT_VERSION, g_es3 ? 3 : 2,
+#if 0
+      EGL_CONTEXT_FLAGS_KHR, debug ? EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR : 0,
+#endif
+      EGL_NONE
+   };
+#endif
 
    (void)width;
    (void)height;
@@ -327,6 +320,20 @@ static bool android_gfx_ctx_set_video_mode(void *data,
 
    switch (android_api)
    {
+      case GFX_CTX_OPENGL_API:
+         break;
+      case GFX_CTX_OPENGL_ES_API:
+#if defined(HAVE_OPENGLES) && defined(HAVE_EGL)
+         if (!egl_create_context(&and->egl, context_attributes))
+         {
+            egl_report_error();
+            return false;
+         }
+
+         if (!egl_create_surface(&and->egl, android_app->window))
+            return false;
+#endif
+         break;
       case GFX_CTX_VULKAN_API:
 #ifdef HAVE_VULKAN
          and->width  = ANativeWindow_getWidth(android_app->window);
@@ -359,11 +366,18 @@ static void android_gfx_ctx_input_driver(void *data,
    *input_data          = androidinput;
 }
 
+static enum gfx_ctx_api android_gfx_ctx_get_api(void *data)
+{
+   return android_api;
+}
+
 static bool android_gfx_ctx_bind_api(void *data,
       enum gfx_ctx_api api, unsigned major, unsigned minor)
 {
    unsigned version;
-   
+
+   android_api = api;
+
    switch (api)
    {
       case GFX_CTX_OPENGL_API:
@@ -378,15 +392,11 @@ static bool android_gfx_ctx_bind_api(void *data,
             g_es3 = true;
 
          if (api == GFX_CTX_OPENGL_ES_API)
-         {
-            android_api = api;
             return true;
-         }
 #endif
          break;
       case GFX_CTX_VULKAN_API:
 #ifdef HAVE_VULKAN
-         android_api = api;
          return true;
 #else
          break;
@@ -471,7 +481,7 @@ dpi_fallback:
    return true;
 }
 
-static void android_gfx_ctx_swap_buffers(void *data, video_frame_info_t *video_info)
+static void android_gfx_ctx_swap_buffers(void *data, void *data2)
 {
    android_ctx_data_t *and  = (android_ctx_data_t*)data;
 
@@ -575,7 +585,7 @@ static uint32_t android_gfx_ctx_get_flags(void *data)
 {
    uint32_t flags = 0;
    BIT32_SET(flags, GFX_CTX_FLAGS_NONE);
-   
+
    return flags;
 }
 
@@ -587,10 +597,12 @@ static void android_gfx_ctx_set_flags(void *data, uint32_t flags)
 const gfx_ctx_driver_t gfx_ctx_android = {
    android_gfx_ctx_init,
    android_gfx_ctx_destroy,
+   android_gfx_ctx_get_api,
    android_gfx_ctx_bind_api,
    android_gfx_ctx_set_swap_interval,
    android_gfx_ctx_set_video_mode,
    android_gfx_ctx_get_video_size,
+   NULL, /* get_refresh_rate */
    NULL, /* get_video_output_size */
    NULL, /* get_video_output_prev */
    NULL, /* get_video_output_next */
