@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -29,7 +29,7 @@
 #include <OpenGL/OpenGL.h>
 #include <OpenGL/gl.h>
 
-#include "../video_context_driver.h"
+#include "../video_driver.h"
 
 typedef int CGSConnectionID;
 typedef int CGSWindowID;
@@ -54,6 +54,8 @@ extern CGLError CGLSetSurface(CGLContextObj gl, CGSConnectionID cid, CGSWindowID
 }
 #endif
 
+static enum gfx_ctx_api cgl_api = GFX_CTX_NONE;
+
 typedef struct gfx_ctx_cgl_data
 {
    CGLContextObj glCtx;
@@ -93,7 +95,7 @@ static void gfx_ctx_cgl_check_window(void *data, bool *quit,
    }
 }
 
-static void gfx_ctx_cgl_swap_buffers(void *data, video_frame_info_t *video_info)
+static void gfx_ctx_cgl_swap_buffers(void *data, void *data2)
 {
    gfx_ctx_cgl_data_t *cgl = (gfx_ctx_cgl_data_t*)data;
 
@@ -116,7 +118,7 @@ static bool gfx_ctx_cgl_set_video_mode(void *data,
 static void gfx_ctx_cgl_destroy(void *data)
 {
    gfx_ctx_cgl_data_t *cgl = (gfx_ctx_cgl_data_t*)data;
-   
+
    if (cgl->glCtx)
    {
       CGLSetCurrentContext(NULL);
@@ -171,6 +173,11 @@ static bool gfx_ctx_cgl_suppress_screensaver(void *data, bool enable)
    return false;
 }
 
+static enum gfx_ctx_api gfx_ctx_cgl_get_api(void *data)
+{
+   return cgl_api;
+}
+
 static bool gfx_ctx_cgl_bind_api(void *data, enum gfx_ctx_api api,
    unsigned major, unsigned minor)
 {
@@ -179,7 +186,13 @@ static bool gfx_ctx_cgl_bind_api(void *data, enum gfx_ctx_api api,
    (void)major;
    (void)minor;
 
-   return api == GFX_CTX_OPENGL_API;
+   if (api == GFX_CTX_OPENGL_API)
+   {
+      cgl_api = api;
+      return true;
+   }
+
+   return false;
 }
 
 static void gfx_ctx_cgl_show_mouse(void *data, bool state)
@@ -230,9 +243,9 @@ static CGSSurfaceID attach_gl_context_to_window(CGLContextObj glCtx,
     CGSConnectionID cid = CGSMainConnectionID();
 
     printf("cid:%d wid:%d\n", cid, wid);
- 
+
     /* determine window size */
-    /* FIXME/TODO - CGWindowListCopyWindowInfo was introduced on OSX 10.5, 
+    /* FIXME/TODO - CGWindowListCopyWindowInfo was introduced on OSX 10.5,
      * find alternative for lower versions. */
     wins = CGWindowListCopyWindowInfo(kCGWindowListOptionIncludingWindow, wid); /* expect one result only */
     win = (CFDictionaryRef)CFArrayGetValueAtIndex(wins, 0);
@@ -242,33 +255,33 @@ static CGSSurfaceID attach_gl_context_to_window(CGLContextObj glCtx,
     CFNumberGetValue((CFNumberRef)CFDictionaryGetValue(bnd, CFSTR("Height")),
        kCFNumberFloat64Type, &h);
     CFRelease(wins);
- 
+
     /* create a surface. */
     if(CGSAddSurface(cid, wid, &sid) != kCGErrorSuccess)
     {
        printf("ERR: no surface\n");
     }
     printf("sid:%d\n", sid);
- 
+
     /* set surface size, and order it frontmost */
     if(CGSSetSurfaceBounds(cid, wid, sid, CGRectMake(0, 0, w, h)) != kCGErrorSuccess)
        printf("ERR: cant set bounds\n");
     if(CGSOrderSurface(cid, wid, sid, 1, 0) != kCGErrorSuccess)
        printf("ERR: cant order front\n");
- 
+
     /* attach context to the surface */
     if(CGLSetSurface(glCtx, cid, wid, sid) != kCGErrorSuccess)
     {
        printf("ERR: cant set surface\n");
     }
- 
+
     /* check drawable */
     CGLGetParameter(glCtx, kCGLCPHasDrawable, &params);
     if(params != 1)
     {
        printf("ERR: no drawable\n");
     }
- 
+
     *width  = (int)w;
     *height = (int)h;
 
@@ -325,10 +338,12 @@ static void gfx_ctx_cgl_set_flags(void *data, uint32_t flags)
 const gfx_ctx_driver_t gfx_ctx_cgl = {
    gfx_ctx_cgl_init,
    gfx_ctx_cgl_destroy,
+   gfx_ctx_cgl_get_api,
    gfx_ctx_cgl_bind_api,
    gfx_ctx_cgl_swap_interval,
    gfx_ctx_cgl_set_video_mode,
    gfx_ctx_cgl_get_video_size,
+   NULL, /* get_refresh_rate */
    NULL, /* get_video_output_size */
    NULL, /* get_video_output_prev */
    NULL, /* get_video_output_next */
