@@ -39,10 +39,16 @@
 #include <compat/apple_compat.h>
 
 #import "../../ui/drivers/cocoa/cocoa_common.h"
-#include "../video_context_driver.h"
+#include "../video_driver.h"
 #include "../../configuration.h"
 #include "../../verbosity.h"
 
+#if __has_feature(objc_arc)
+#define BRIDGE __bridge
+#else
+#define BRIDGE
+#endif
+
 #if defined(HAVE_COCOATOUCH)
 #define GLContextClass EAGLContext
 #define GLFrameworkID CFSTR("com.apple.opengles")
@@ -79,6 +85,13 @@
 #define RAScreen NSScreen
 #endif
 
+static enum gfx_ctx_api cocoagl_api = GFX_CTX_NONE;
+
+typedef struct cocoa_ctx_data
+{
+   bool core_hw_context_enable; 
+} cocoa_ctx_data_t;
+
 #if defined(HAVE_COCOATOUCH)
 
 static GLKView *g_view;
@@ -90,7 +103,7 @@ static GLContextClass* g_context;
 
 static int g_fast_forward_skips;
 static bool g_is_syncing = true;
-static bool g_use_hw_ctx;
+static bool g_use_hw_ctx = false;
 
 #if defined(HAVE_COCOA)
 #include "../../ui/drivers/ui_cocoa.h"
@@ -127,7 +140,7 @@ void *glkitview_init(void)
    g_view.enableSetNeedsDisplay = NO;
    [g_view addSubview:g_pause_indicator_view];
     
-   return (__bridge void *)((GLKView*)g_view);
+   return (BRIDGE void *)((GLKView*)g_view);
 #else
     return nsview_get_ptr();
 #endif
@@ -155,6 +168,10 @@ static float get_from_selector(Class obj_class, id obj_id, SEL selector, CGFloat
     [invocation setTarget:obj_id];
     [invocation invoke];
     [invocation getReturnValue:ret];
+#if __has_feature(objc_arc)
+#else
+    [invocation release];
+#endif
     return *ret;
 }
 
@@ -165,48 +182,55 @@ void *get_chosen_screen(void)
    if (!screens || !settings)
       return NULL;
 
-   if (settings->video.monitor_index >= screens.count)
+   if (settings->uints.video_monitor_index >= screens.count)
    {
       RARCH_WARN("video_monitor_index is greater than the number of connected monitors; using main screen instead.");
-#if __has_feature(objc_arc)
-      return (__bridge void*)screens;
-#else
-      return (void*)screens;
-#endif
+      return (BRIDGE void*)screens;
    }
 	
-#if __has_feature(objc_arc)
-   return ((__bridge void*)[screens objectAtIndex:settings->video.monitor_index]);
-#else
-   return ((void*)[screens objectAtIndex:settings->video.monitor_index]);
-#endif
+   return ((BRIDGE void*)[screens objectAtIndex:settings->uints.video_monitor_index]);
 }
 
+
 float get_backing_scale_factor(void)
 {
-#if __has_feature(objc_arc)
-   RAScreen *screen = (__bridge RAScreen*)get_chosen_screen();
-#else
-   RAScreen *screen = (RAScreen*)get_chosen_screen();
-#endif
-   if (!screen)
-      return 0.0;
+   static float 
+      backing_scale_def = 0.0f;
+   RAScreen *screen     = NULL;
+
+   (void)screen;
+
+   if (backing_scale_def != 0.0f)
+      return backing_scale_def;
+
+   backing_scale_def = 1.0f;
 #ifdef HAVE_COCOA
-   CGFloat ret;
-    CocoaView *g_view = (CocoaView*)nsview_get_ptr();
-   SEL selector     = NSSelectorFromString(BOXSTRING("backingScaleFactor"));
-   if ([screen respondsToSelector:selector])
-        return (float)get_from_selector([[g_view window] class], [g_view window], selector, &ret);
+   screen = (BRIDGE RAScreen*)get_chosen_screen();
+
+   if (screen)
+   {
+      SEL selector = NSSelectorFromString(BOXSTRING("backingScaleFactor"));
+      if ([screen respondsToSelector:selector])
+      {
+         CGFloat ret;
+         CocoaView *g_view     = (CocoaView*)nsview_get_ptr();
+         backing_scale_def     = (float)get_from_selector
+            ([[g_view window] class], [g_view window], selector, &ret);
+      }
+   }
 #endif
-   return 1.0f;
+
+   return backing_scale_def;
 }
 
 void cocoagl_gfx_ctx_update(void)
 {
 #if defined(HAVE_COCOA)
 #if MAC_OS_X_VERSION_10_7
+   CGLUpdateContext(g_hw_ctx.CGLContextObj);
    CGLUpdateContext(g_context.CGLContextObj);
 #else
+    [g_hw_ctx update];
 	[g_context update];
 #endif
 #endif
@@ -214,58 +238,21 @@ void cocoagl_gfx_ctx_update(void)
 
 static void *cocoagl_gfx_ctx_init(video_frame_info_t *video_info, void *video_driver)
 {
-   (void)video_driver;
-    
-#if defined(HAVE_COCOA)
-    CocoaView *g_view = (CocoaView*)nsview_get_ptr();
-    if ([g_view respondsToSelector: @selector(setWantsBestResolutionOpenGLSurface:)])
-        [g_view setWantsBestResolutionOpenGLSurface:YES];
-    
-    NSOpenGLPixelFormatAttribute attributes [] = {
-        NSOpenGLPFAColorSize, 24,
-        NSOpenGLPFADoubleBuffer,
-        NSOpenGLPFAAllowOfflineRenderers,
-        NSOpenGLPFADepthSize,
-        (NSOpenGLPixelFormatAttribute)16, // 16 bit depth buffer
-#ifdef MAC_OS_X_VERSION_10_7
-        (g_major || g_minor) ? NSOpenGLPFAOpenGLProfile : 0,
-        (g_major << 12) | (g_minor << 8),
-#endif
-        (NSOpenGLPixelFormatAttribute)0
-    };
-    
-    g_format = [[NSOpenGLPixelFormat alloc] initWithAttributes:attributes];
-    
-#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5
-    if (g_format == nil)
-    {
-        /* NSOpenGLFPAAllowOfflineRenderers is 
-         not supported on this OS version. */
-        attributes[3] = (NSOpenGLPixelFormatAttribute)0;
-        g_format = [[NSOpenGLPixelFormat alloc] initWithAttributes:attributes];
-    }
-#endif
-    
-    if (g_use_hw_ctx)
-    {
-        //g_hw_ctx  = [[NSOpenGLContext alloc] initWithFormat:g_format shareContext:nil];
-    }
-    g_context = [[NSOpenGLContext alloc] initWithFormat:g_format shareContext:(g_use_hw_ctx) ? g_hw_ctx : nil];
-    [g_context setView:g_view];
-#else
-    g_context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
-    g_view.context = g_context;
-#endif
-    
-    [g_context makeCurrentContext];
-   // Make sure the view was created
-   [CocoaView get];
-   return (void*)"cocoa";
+   cocoa_ctx_data_t *cocoa_ctx = (cocoa_ctx_data_t*)
+      calloc(1, sizeof(cocoa_ctx_data_t));
+
+   if (!cocoa_ctx)
+      return NULL;
+
+   return cocoa_ctx;
 }
 
 static void cocoagl_gfx_ctx_destroy(void *data)
 {
-   (void)data;
+   cocoa_ctx_data_t *cocoa_ctx = (cocoa_ctx_data_t*)data;
+
+   if (!cocoa_ctx)
+      return;
 
    [GLContextClass clearCurrentContext];
 
@@ -278,11 +265,21 @@ static void cocoagl_gfx_ctx_destroy(void *data)
         [g_format release];
     g_format = nil;
     if (g_hw_ctx)
+    {
+        [g_hw_ctx clearDrawable];
         [g_hw_ctx release];
+    }
     g_hw_ctx = nil;
 #endif
    [GLContextClass clearCurrentContext];
    g_context = nil;
+
+   free(cocoa_ctx);
+}
+
+static enum gfx_ctx_api cocoagl_gfx_ctx_get_api(void *data)
+{
+   return cocoagl_api;
 }
 
 static bool cocoagl_gfx_ctx_bind_api(void *data, enum gfx_ctx_api api, unsigned major, unsigned minor)
@@ -296,8 +293,9 @@ static bool cocoagl_gfx_ctx_bind_api(void *data, enum gfx_ctx_api api, unsigned
       return false;
 #endif
     
-   g_minor = minor;
-   g_major = major;
+   cocoagl_api = api;
+   g_minor     = minor;
+   g_major     = major;
   
    return true;
 }
@@ -332,9 +330,66 @@ static bool cocoagl_gfx_ctx_set_video_mode(void *data,
       video_frame_info_t *video_info,
       unsigned width, unsigned height, bool fullscreen)
 {
+#if defined(HAVE_COCOA)
+    CocoaView *g_view = (CocoaView*)nsview_get_ptr();
+    if ([g_view respondsToSelector: @selector(setWantsBestResolutionOpenGLSurface:)])
+        [g_view setWantsBestResolutionOpenGLSurface:YES];
+    
+    NSOpenGLPixelFormatAttribute attributes [] = {
+        NSOpenGLPFAColorSize,
+        24,
+        NSOpenGLPFADoubleBuffer,
+        NSOpenGLPFAAllowOfflineRenderers,
+        NSOpenGLPFADepthSize,
+        (NSOpenGLPixelFormatAttribute)16, // 16 bit depth buffer
+        0,                                /* profile */
+        0,                                /* profile enum */
+        (NSOpenGLPixelFormatAttribute)0
+    };
+
+#if MAC_OS_X_VERSION_10_7
+    if (g_major == 3 && (g_minor >= 1 && g_minor <= 3))
+    {
+       attributes[6] = NSOpenGLPFAOpenGLProfile;
+       attributes[7] = NSOpenGLProfileVersion3_2Core;
+    }
+#endif
+
+#if MAC_OS_X_VERSION_10_10
+    if (g_major == 4 && g_minor == 1)
+    {
+       attributes[6] = NSOpenGLPFAOpenGLProfile;
+       attributes[7] = NSOpenGLProfileVersion4_1Core;
+    }
+#endif
+    
+    g_format = [[NSOpenGLPixelFormat alloc] initWithAttributes:attributes];
+    
+#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5
+    if (g_format == nil)
+    {
+        /* NSOpenGLFPAAllowOfflineRenderers is
+         not supported on this OS version. */
+        attributes[3] = (NSOpenGLPixelFormatAttribute)0;
+        g_format = [[NSOpenGLPixelFormat alloc] initWithAttributes:attributes];
+    }
+#endif
+    
+    if (g_use_hw_ctx)
+        g_hw_ctx  = [[NSOpenGLContext alloc] initWithFormat:g_format shareContext:nil];
+    g_context = [[NSOpenGLContext alloc] initWithFormat:g_format shareContext:(g_use_hw_ctx) ? g_hw_ctx : nil];
+    [g_context setView:g_view];
+#else
+    if (g_use_hw_ctx)
+        g_hw_ctx = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
+    g_context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
+    g_view.context = g_context;
+#endif
+    
+    [g_context makeCurrentContext];
+    
 #if defined(HAVE_COCOA)
    static bool has_went_fullscreen = false;
-   CocoaView *g_view = (CocoaView*)nsview_get_ptr();
    /* TODO: Screen mode support. */
 
    if (fullscreen)
@@ -369,22 +424,18 @@ static bool cocoagl_gfx_ctx_set_video_mode(void *data,
 
 float cocoagl_gfx_ctx_get_native_scale(void)
 {
-    static CGFloat ret = 0.0f;
-    SEL selector     = NSSelectorFromString(BOXSTRING("nativeScale"));
-#if __has_feature(objc_arc)
-    RAScreen *screen = (__bridge RAScreen*)get_chosen_screen();
-#else
-    RAScreen *screen = (RAScreen*)get_chosen_screen();
-#endif
-    
-    if (ret != 0.0f)
-       return ret;
-    if (!screen)
-       return 0.0f;
-    
+   static CGFloat ret = 0.0f;
+   SEL selector     = NSSelectorFromString(BOXSTRING("nativeScale"));
+   RAScreen *screen = (BRIDGE RAScreen*)get_chosen_screen();
+
+   if (ret != 0.0f)
+      return ret;
+   if (!screen)
+      return 0.0f;
+
    if ([screen respondsToSelector:selector])
-       return (float)get_from_selector([screen class], screen, selector, &ret);
-    
+      return (float)get_from_selector([screen class], screen, selector, &ret);
+
    ret          = 1.0f;
    selector     = NSSelectorFromString(BOXSTRING("scale"));
    if ([screen respondsToSelector:selector])
@@ -416,10 +467,10 @@ static void cocoagl_gfx_ctx_get_video_size(void *data, unsigned* width, unsigned
 }
 
 #if defined(HAVE_COCOA)
-static void cocoagl_gfx_ctx_update_title(void *data, video_frame_info_t *video_info)
+static void cocoagl_gfx_ctx_update_title(void *data, void *data2)
 {
    ui_window_cocoa_t view;
-   const ui_window_t *window  = ui_companion_driver_get_window_ptr();
+   const ui_window_t *window      = ui_companion_driver_get_window_ptr();
 
    view.data = (CocoaView*)nsview_get_ptr();
 
@@ -440,11 +491,7 @@ static void cocoagl_gfx_ctx_update_title(void *data, video_frame_info_t *video_i
 static bool cocoagl_gfx_ctx_get_metrics(void *data, enum display_metric_types type,
             float *value)
 {
-#if __has_feature(objc_arc)
-    RAScreen *screen              = (__bridge RAScreen*)get_chosen_screen();
-#else
-    RAScreen *screen              = (RAScreen*)get_chosen_screen();
-#endif
+    RAScreen *screen              = (BRIDGE RAScreen*)get_chosen_screen();
 #if defined(HAVE_COCOA)
     NSDictionary *description     = [screen deviceDescription];
     NSSize  display_pixel_size    = [[description objectForKey:NSDeviceSize] sizeValue];
@@ -531,13 +578,14 @@ static bool cocoagl_gfx_ctx_has_windowed(void *data)
 }
 #endif
 
-static void cocoagl_gfx_ctx_swap_buffers(void *data, video_frame_info_t *video_info)
+static void cocoagl_gfx_ctx_swap_buffers(void *data, void *data2)
 {
    if (!(--g_fast_forward_skips < 0))
       return;
     
 #if defined(HAVE_COCOA)
     [g_context flushBuffer];
+    [g_hw_ctx flushBuffer];
 #elif defined(HAVE_COCOATOUCH)
     if (g_view)
         [g_view display];
@@ -549,11 +597,7 @@ static void cocoagl_gfx_ctx_swap_buffers(void *data, video_frame_info_t *video_i
 static gfx_ctx_proc_t cocoagl_gfx_ctx_get_proc_address(const char *symbol_name)
 {
    return (gfx_ctx_proc_t)CFBundleGetFunctionPointerForName(CFBundleGetBundleWithIdentifier(GLFrameworkID),
-   (
-#if __has_feature(objc_arc)
-         __bridge
-#endif
-CFStringRef)BOXSTRING(symbol_name)
+   (BRIDGE CFStringRef)BOXSTRING(symbol_name)
          );
 }
 
@@ -594,23 +638,35 @@ static void cocoagl_gfx_ctx_bind_hw_render(void *data, bool enable)
 
 static uint32_t cocoagl_gfx_ctx_get_flags(void *data)
 {
-   uint32_t flags = 0;
+   uint32_t flags                 = 0;
+   cocoa_ctx_data_t    *cocoa_ctx = (cocoa_ctx_data_t*)data;
+
    BIT32_SET(flags, GFX_CTX_FLAGS_NONE);
+
+   if (cocoa_ctx->core_hw_context_enable)
+      BIT32_SET(flags, GFX_CTX_FLAGS_GL_CORE_CONTEXT);
+
    return flags;
 }
 
 static void cocoagl_gfx_ctx_set_flags(void *data, uint32_t flags)
 {
    (void)flags;
+   cocoa_ctx_data_t *cocoa_ctx = (cocoa_ctx_data_t*)data;
+
+   if (BIT32_GET(flags, GFX_CTX_FLAGS_GL_CORE_CONTEXT))
+      cocoa_ctx->core_hw_context_enable = true;
 }
 
 const gfx_ctx_driver_t gfx_ctx_cocoagl = {
    cocoagl_gfx_ctx_init,
    cocoagl_gfx_ctx_destroy,
+   cocoagl_gfx_ctx_get_api,
    cocoagl_gfx_ctx_bind_api,
    cocoagl_gfx_ctx_swap_interval,
    cocoagl_gfx_ctx_set_video_mode,
    cocoagl_gfx_ctx_get_video_size,
+   NULL, /* get_refresh_rate */
    NULL, /* get_video_output_size */
    NULL, /* get_video_output_prev */
    NULL, /* get_video_output_next */
