@@ -20,12 +20,13 @@
 #include <unistd.h>
 
 #include <emscripten/emscripten.h>
+#include <emscripten/html5.h>
 
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
 #endif
 
-#include "../video_context_driver.h"
+#include "../video_driver.h"
 
 #ifdef HAVE_EGL
 #include "../common/egl_common.h"
@@ -44,40 +45,104 @@ typedef struct
    unsigned fb_height;
 } emscripten_ctx_data_t;
 
+static int emscripten_initial_width;
+static int emscripten_initial_height;
+static enum gfx_ctx_api emscripten_api = GFX_CTX_NONE;
+
 static void gfx_ctx_emscripten_swap_interval(void *data, unsigned interval)
 {
    (void)data;
-   /* no way to control VSync in WebGL. */
-   (void)interval;
+
+   if (interval == 0)
+      emscripten_set_main_loop_timing(EM_TIMING_SETIMMEDIATE, 0);
+   else
+      emscripten_set_main_loop_timing(EM_TIMING_RAF, (int)interval);
+}
+
+static void gfx_ctx_emscripten_get_canvas_size(int *width, int *height)
+{
+   EmscriptenFullscreenChangeEvent fullscreen_status;
+   bool  is_fullscreen = false;
+   EMSCRIPTEN_RESULT r = emscripten_get_fullscreen_status(&fullscreen_status);
+
+   if (r == EMSCRIPTEN_RESULT_SUCCESS)
+   {
+      if (fullscreen_status.isFullscreen)
+      {
+         is_fullscreen = true;
+         *width = fullscreen_status.screenWidth;
+         *height = fullscreen_status.screenHeight;
+      }
+   }
+
+   if (!is_fullscreen)
+   {
+      r = emscripten_get_canvas_element_size("#canvas", width, height);
+
+      if (r != EMSCRIPTEN_RESULT_SUCCESS)
+      {
+         RARCH_ERR("[EMSCRIPTEN/EGL]: Could not get screen dimensions: %d\n",
+            r);
+         *width = 800;
+         *height = 600;
+      }
+   }
 }
 
 static void gfx_ctx_emscripten_check_window(void *data, bool *quit,
       bool *resize, unsigned *width, unsigned *height, bool is_shutdown)
 {
+   EMSCRIPTEN_RESULT r;
    int input_width;
    int input_height;
-   int is_fullscreen;
    emscripten_ctx_data_t *emscripten = (emscripten_ctx_data_t*)data;
 
-   (void)data;
+   gfx_ctx_emscripten_get_canvas_size(&input_width, &input_height);
+
+   if (input_width == 0 || input_height == 0)
+   {
+      input_width = emscripten_initial_width;
+      input_height = emscripten_initial_height;
+      emscripten->fb_width = emscripten->fb_height = 0;
+   }
 
-   emscripten_get_canvas_size(&input_width, &input_height, &is_fullscreen);
    *width      = (unsigned)input_width;
    *height     = (unsigned)input_height;
    *resize     = false;
 
-   if (*width != emscripten->fb_width || *height != emscripten->fb_height)
+   if (input_width != emscripten->fb_width ||
+      input_height != emscripten->fb_height)
+   {
+      r = emscripten_set_canvas_element_size("#canvas",
+         input_width, input_height);
+
+      if (r != EMSCRIPTEN_RESULT_SUCCESS)
+         RARCH_ERR("[EMSCRIPTEN/EGL]: error resizing canvas: %d\n", r);
+
+      /* fix Module.requestFullscreen messing with the canvas size */
+      r = emscripten_set_element_css_size("#canvas",
+         (double)input_width, (double)input_height);
+
+      if (r != EMSCRIPTEN_RESULT_SUCCESS)
+         RARCH_ERR("[EMSCRIPTEN/EGL]: error resizing canvas css: %d\n", r);
+
       *resize  = true;
+   }
 
    emscripten->fb_width  = (unsigned)input_width;
    emscripten->fb_height = (unsigned)input_height;
    *quit       = false;
 }
 
-static void gfx_ctx_emscripten_swap_buffers(void *data, video_frame_info_t *video_info)
+static void gfx_ctx_emscripten_swap_buffers(void *data, void *data2)
 {
-   (void)data;
-   /* no-op in emscripten, no way to force swap/wait for VSync in browsers */
+   emscripten_ctx_data_t *emscripten = (emscripten_ctx_data_t*)data;
+
+   /* doesn't really do anything in WebGL, but it might if we use WebGL workers
+    * in the future */
+#ifdef HAVE_EGL
+   egl_swap_buffers(&emscripten->egl);
+#endif
 }
 
 static void gfx_ctx_emscripten_get_video_size(void *data,
@@ -85,6 +150,9 @@ static void gfx_ctx_emscripten_get_video_size(void *data,
 {
    emscripten_ctx_data_t *emscripten = (emscripten_ctx_data_t*)data;
 
+   if (!emscripten)
+      return;
+
    *width  = emscripten->fb_width;
    *height = emscripten->fb_height;
 }
@@ -93,12 +161,16 @@ static void gfx_ctx_emscripten_destroy(void *data)
 {
    emscripten_ctx_data_t *emscripten = (emscripten_ctx_data_t*)data;
 
+   if (!emscripten)
+      return;
+
    egl_destroy(&emscripten->egl);
 
    free(data);
 }
 
-static void *gfx_ctx_emscripten_init(video_frame_info_t *video_info, void *video_driver)
+static void *gfx_ctx_emscripten_init(video_frame_info_t *video_info,
+   void *video_driver)
 {
 #ifdef HAVE_EGL
    unsigned width, height;
@@ -128,6 +200,10 @@ static void *gfx_ctx_emscripten_init(video_frame_info_t *video_info, void *video
 
    (void)video_driver;
 
+   if (emscripten_initial_width == 0 || emscripten_initial_height == 0)
+      emscripten_get_canvas_element_size("#canvas",
+         &emscripten_initial_width, &emscripten_initial_height);
+
 #ifdef HAVE_EGL
    if (g_egl_inited)
    {
@@ -135,8 +211,8 @@ static void *gfx_ctx_emscripten_init(video_frame_info_t *video_info, void *video
       return (void*)"emscripten";
    }
 
-   if (!egl_init_context(&emscripten->egl, EGL_DEFAULT_DISPLAY,
-            &major, &minor, &n, attribute_list))
+   if (!egl_init_context(&emscripten->egl, EGL_NONE,
+      (void *)EGL_DEFAULT_DISPLAY, &major, &minor, &n, attribute_list))
    {
       egl_report_error();
       goto error;
@@ -179,6 +255,11 @@ static bool gfx_ctx_emscripten_set_video_mode(void *data,
    return true;
 }
 
+static enum gfx_ctx_api gfx_ctx_emscripten_get_api(void *data)
+{
+   return emscripten_api;
+}
+
 static bool gfx_ctx_emscripten_bind_api(void *data,
       enum gfx_ctx_api api, unsigned major, unsigned minor)
 {
@@ -186,6 +267,8 @@ static bool gfx_ctx_emscripten_bind_api(void *data,
    (void)major;
    (void)minor;
 
+   emscripten_api = api;
+
    switch (api)
    {
       case GFX_CTX_OPENGL_ES_API:
@@ -202,20 +285,10 @@ static void gfx_ctx_emscripten_input_driver(void *data,
       const char *name,
       const input_driver_t **input, void **input_data)
 {
-   void *rwebinput = NULL;
+   void *rwebinput = input_rwebinput.init(name);
 
-   *input          = NULL;
-   *input_data     = NULL;
-
-#ifndef HAVE_SDL2
-   rwebinput = input_rwebinput.init();
-
-   if (!rwebinput)
-      return;
-
-   *input      = &input_rwebinput;
+   *input      = rwebinput ? &input_rwebinput : NULL;
    *input_data = rwebinput;
-#endif
 }
 
 static bool gfx_ctx_emscripten_has_focus(void *data)
@@ -298,10 +371,12 @@ static void gfx_ctx_emscripten_set_flags(void *data, uint32_t flags)
 const gfx_ctx_driver_t gfx_ctx_emscripten = {
    gfx_ctx_emscripten_init,
    gfx_ctx_emscripten_destroy,
+   gfx_ctx_emscripten_get_api,
    gfx_ctx_emscripten_bind_api,
    gfx_ctx_emscripten_swap_interval,
    gfx_ctx_emscripten_set_video_mode,
    gfx_ctx_emscripten_get_video_size,
+   NULL, /* get_refresh_rate */
    NULL, /* get_video_output_size */
    NULL, /* get_video_output_prev */
    NULL, /* get_video_output_next */
