@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2014 2015 - Jean-Andre Santoni
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -40,6 +40,8 @@ typedef struct
    unsigned width, height;
 } opendingux_ctx_data_t;
 
+static enum gfx_ctx_api opendingux_api = GFX_CTX_NONE;
+
 static void gfx_ctx_opendingux_destroy(void *data)
 {
    opendingux_ctx_data_t *viv = (opendingux_ctx_data_t*)data;
@@ -79,11 +81,11 @@ static void *gfx_ctx_opendingux_init(video_frame_info_t *video_info, void *video
 
    if (!viv)
       return NULL;
-   
+
 #ifdef HAVE_EGL
    frontend_driver_install_signal_handler();
 
-   if (!egl_init_context(&viv->egl, EGL_DEFAULT_DISPLAY,
+   if (!egl_init_context(&viv->egl, EGL_NONE, EGL_DEFAULT_DISPLAY,
             &major, &minor,
             &n, attribs))
    {
@@ -183,11 +185,21 @@ static void gfx_ctx_opendingux_input_driver(void *data,
    *input_data = NULL;
 }
 
+static enum gfx_ctx_api gfx_ctx_opendingux_get_api(void *data)
+{
+   return opendingux_api;
+}
+
 static bool gfx_ctx_opendingux_bind_api(void *data,
       enum gfx_ctx_api api, unsigned major, unsigned minor)
 {
    (void)data;
-   return api == GFX_CTX_OPENGL_ES_API;
+
+   opendingux_api = api;
+
+   if (api == GFX_CTX_OPENGL_ES_API)
+      return true;
+   return false;
 }
 
 static bool gfx_ctx_opendingux_has_focus(void *data)
@@ -203,7 +215,7 @@ static bool gfx_ctx_opendingux_suppress_screensaver(void *data, bool enable)
    return false;
 }
 
-static void gfx_ctx_opendingux_swap_buffers(void *data, video_frame_info_t *video_info)
+static void gfx_ctx_opendingux_swap_buffers(void *data, void *data2)
 {
    opendingux_ctx_data_t *viv = (opendingux_ctx_data_t*)data;
 
@@ -254,10 +266,12 @@ static void gfx_ctx_opendingux_set_flags(void *data, uint32_t flags)
 const gfx_ctx_driver_t gfx_ctx_opendingux_fbdev = {
    gfx_ctx_opendingux_init,
    gfx_ctx_opendingux_destroy,
+   gfx_ctx_opendingux_get_api,
    gfx_ctx_opendingux_bind_api,
    gfx_ctx_opendingux_set_swap_interval,
    gfx_ctx_opendingux_set_video_mode,
    gfx_ctx_opendingux_get_video_size,
+   NULL, /* get_refresh_rate */
    NULL, /* get_video_output_size */
    NULL, /* get_video_output_prev */
    NULL, /* get_video_output_next */
