@@ -14,8 +14,6 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "SDL.h"
-
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
 #endif
@@ -27,8 +25,10 @@
 #include "../../configuration.h"
 #include "../common/gl_common.h"
 
+#include "SDL.h"
+
 static enum gfx_ctx_api sdl_api = GFX_CTX_OPENGL_API;
-static unsigned       g_major = 2;
+static unsigned       g_major   = 2;
 static unsigned       g_minor = 1;
 
 typedef struct gfx_ctx_sdl_data
@@ -114,13 +114,19 @@ static void sdl_ctx_destroy(void *data)
 
    if (!sdl)
       return;
-   
+
    sdl_ctx_destroy_resources(sdl);
    free(sdl);
 }
 
-static bool sdl_ctx_bind_api(void *data, enum gfx_ctx_api api, unsigned major,
-                             unsigned minor)
+static enum gfx_ctx_api sdl_ctx_get_api(void *data)
+{
+   return sdl_api;
+}
+
+static bool sdl_ctx_bind_api(void *data,
+      enum gfx_ctx_api api, unsigned major,
+      unsigned minor)
 {
 #ifdef HAVE_SDL2
    unsigned profile;
@@ -245,7 +251,7 @@ static void sdl_ctx_get_video_size(void *data,
    {
 #ifdef HAVE_SDL2
       SDL_DisplayMode mode = {0};
-      int i = settings->video.monitor_index;
+      int i = settings->uints.video_monitor_index;
 
       if (SDL_GetCurrentDisplayMode(i, &mode) < 0)
          RARCH_WARN("[SDL_GL]: Failed to get display #%i mode: %s\n", i,
@@ -266,7 +272,7 @@ static void sdl_ctx_get_video_size(void *data,
    }
 }
 
-static void sdl_ctx_update_title(void *data, video_frame_info_t *video_info)
+static void sdl_ctx_update_title(void *data, void *data2)
 {
    char title[128];
 
@@ -365,7 +371,7 @@ static bool sdl_ctx_has_windowed(void *data)
    return true;
 }
 
-static void sdl_ctx_swap_buffers(void *data, video_frame_info_t *video_info)
+static void sdl_ctx_swap_buffers(void *data, void *data2)
 {
 #ifdef HAVE_SDL2
    gfx_ctx_sdl_data_t *sdl = (gfx_ctx_sdl_data_t*)data;
@@ -374,7 +380,6 @@ static void sdl_ctx_swap_buffers(void *data, video_frame_info_t *video_info)
 #else
    SDL_GL_SwapBuffers();
 #endif
-   (void)data;
 }
 
 static void sdl_ctx_input_driver(void *data,
@@ -412,10 +417,12 @@ const gfx_ctx_driver_t gfx_ctx_sdl_gl =
 {
    sdl_ctx_init,
    sdl_ctx_destroy,
+   sdl_ctx_get_api,
    sdl_ctx_bind_api,
    sdl_ctx_swap_interval,
    sdl_ctx_set_video_mode,
    sdl_ctx_get_video_size,
+   NULL, /* get_refresh_rate */
    NULL, /* get_video_output_size */
    NULL, /* get_video_output_prev */
    NULL, /* get_video_output_next */
