@@ -2,7 +2,7 @@
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011 2017 - Daniel De Matteis
  *  Copyright (C) 2014 2015 - Jean-Andre Santoni
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -42,6 +42,8 @@ typedef struct
    unsigned width, height;
 } vivante_ctx_data_t;
 
+static enum gfx_ctx_api viv_api = GFX_CTX_NONE;
+
 static void gfx_ctx_vivante_destroy(void *data)
 {
    vivante_ctx_data_t *viv = (vivante_ctx_data_t*)data;
@@ -90,7 +92,7 @@ static void *gfx_ctx_vivante_init(video_frame_info_t *video_info, void *video_dr
    system("setterm -cursor off");
 
 #ifdef HAVE_EGL
-   if (!egl_init_context(&viv->egl, EGL_DEFAULT_DISPLAY, &major, &minor,
+   if (!egl_init_context(&viv->egl, EGL_NONE, EGL_DEFAULT_DISPLAY, &major, &minor,
             &n, attribs))
    {
       egl_report_error();
@@ -190,11 +192,20 @@ static void gfx_ctx_vivante_input_driver(void *data,
    *input_data = NULL;
 }
 
+static enum gfx_ctx_api gfx_ctx_vivante_get_api(void *data)
+{
+   return viv_api;
+}
+
 static bool gfx_ctx_vivante_bind_api(void *data,
       enum gfx_ctx_api api, unsigned major, unsigned minor)
 {
-   (void)data;
-   return api == GFX_CTX_OPENGL_ES_API;
+
+   viv_api = api;
+
+   if (api == GFX_CTX_OPENGL_ES_API)
+      return true;
+   return false;
 }
 
 static bool gfx_ctx_vivante_has_focus(void *data)
@@ -219,7 +230,7 @@ static void gfx_ctx_vivante_set_swap_interval(void *data, unsigned swap_interval
 #endif
 }
 
-static void gfx_ctx_vivante_swap_buffers(void *data, video_frame_info_t *video_info)
+static void gfx_ctx_vivante_swap_buffers(void *data, void *data2)
 {
    vivante_ctx_data_t *viv = (vivante_ctx_data_t*)data;
 
@@ -261,10 +272,12 @@ static void gfx_ctx_vivante_set_flags(void *data, uint32_t flags)
 const gfx_ctx_driver_t gfx_ctx_vivante_fbdev = {
    gfx_ctx_vivante_init,
    gfx_ctx_vivante_destroy,
+   gfx_ctx_vivante_get_api,
    gfx_ctx_vivante_bind_api,
    gfx_ctx_vivante_set_swap_interval,
    gfx_ctx_vivante_set_video_mode,
    gfx_ctx_vivante_get_video_size,
+   NULL, /* get_refresh_rate */
    NULL, /* get_video_output_size */
    NULL, /* get_video_output_prev */
    NULL, /* get_video_output_next */
