@@ -21,11 +21,10 @@
 
 #include "../../retroarch.h"
 #include "../../gfx/font_driver.h"
-#include "../../gfx/video_context_driver.h"
-#include "../../gfx/video_shader_driver.h"
+#include "../../gfx/video_driver.h"
 #include "../../gfx/common/gl_common.h"
 
-#include "../menu_display.h"
+#include "../menu_driver.h"
 
 static const GLfloat gl_vertexes[] = {
    0, 0,
@@ -51,9 +50,9 @@ static const float *menu_display_gl_get_default_tex_coords(void)
    return &gl_tex_coords[0];
 }
 
-static void *menu_display_gl_get_default_mvp(void)
+static void *menu_display_gl_get_default_mvp(video_frame_info_t *video_info)
 {
-   gl_t *gl = (gl_t*)video_driver_get_ptr(false);
+   gl_t *gl = video_info ? (gl_t*)video_info->userdata : NULL;
 
    if (!gl)
       return NULL;
@@ -78,7 +77,7 @@ static GLenum menu_display_prim_to_gl_enum(
    return 0;
 }
 
-static void menu_display_gl_blend_begin(void)
+static void menu_display_gl_blend_begin(video_frame_info_t *video_info)
 {
    video_shader_ctx_info_t shader_info;
 
@@ -89,41 +88,39 @@ static void menu_display_gl_blend_begin(void)
    shader_info.idx        = VIDEO_SHADER_STOCK_BLEND;
    shader_info.set_active = true;
 
-   video_shader_driver_use(shader_info);
+   video_shader_driver_use(&shader_info);
 }
 
-static void menu_display_gl_blend_end(void)
+static void menu_display_gl_blend_end(video_frame_info_t *video_info)
 {
    glDisable(GL_BLEND);
 }
 
-static void menu_display_gl_viewport(void *data)
+static void menu_display_gl_viewport(void *data, video_frame_info_t *video_info)
 {
-   gl_t             *gl          = (gl_t*)video_driver_get_ptr(false);
    menu_display_ctx_draw_t *draw = (menu_display_ctx_draw_t*)data;
-   
-   if (!gl || !draw)
-      return;
-   glViewport(draw->x, draw->y, draw->width, draw->height);
+
+   if (draw)
+      glViewport(draw->x, draw->y, draw->width, draw->height);
 }
 
 static void menu_display_gl_bind_texture(void *data)
 {
    menu_display_ctx_draw_t *draw = (menu_display_ctx_draw_t*)data;
-   
+
    if (!draw)
       return;
 
    glBindTexture(GL_TEXTURE_2D, (GLuint)draw->texture);
 }
 
-static void menu_display_gl_draw(void *data)
+static void menu_display_gl_draw(void *data, video_frame_info_t *video_info)
 {
    video_shader_ctx_mvp_t mvp;
    video_shader_ctx_coords_t coords;
-   gl_t             *gl          = (gl_t*)video_driver_get_ptr(false);
+   gl_t             *gl          = video_info ? (gl_t*)video_info->userdata : NULL;
    menu_display_ctx_draw_t *draw = (menu_display_ctx_draw_t*)data;
-   
+
    if (!gl || !draw)
       return;
 
@@ -134,19 +131,19 @@ static void menu_display_gl_draw(void *data)
    if (!draw->coords->lut_tex_coord)
       draw->coords->lut_tex_coord = menu_display_gl_get_default_tex_coords();
 
-   menu_display_gl_viewport(draw);
+   menu_display_gl_viewport(draw, video_info);
    menu_display_gl_bind_texture(draw);
 
    coords.handle_data = gl;
    coords.data        = draw->coords;
-   
-   video_shader_driver_set_coords(coords);
+
+   video_driver_set_coords(&coords);
 
    mvp.data   = gl;
-   mvp.matrix = draw->matrix_data ? (math_matrix_4x4*)draw->matrix_data 
-      : (math_matrix_4x4*)menu_display_gl_get_default_mvp();
+   mvp.matrix = draw->matrix_data ? (math_matrix_4x4*)draw->matrix_data
+      : (math_matrix_4x4*)menu_display_gl_get_default_mvp(video_info);
 
-   video_shader_driver_set_mvp(mvp);
+   video_driver_set_mvp(&mvp);
 
    glDrawArrays(menu_display_prim_to_gl_enum(
             draw->prim_type), 0, draw->coords->vertices);
@@ -154,7 +151,8 @@ static void menu_display_gl_draw(void *data)
    gl->coords.color     = gl->white_color_ptr;
 }
 
-static void menu_display_gl_draw_pipeline(void *data)
+static void menu_display_gl_draw_pipeline(void *data,
+      video_frame_info_t *video_info)
 {
 #ifdef HAVE_SHADERPIPELINE
    video_shader_ctx_info_t shader_info;
@@ -163,10 +161,21 @@ static void menu_display_gl_draw_pipeline(void *data)
    menu_display_ctx_draw_t *draw    = (menu_display_ctx_draw_t*)data;
    video_coord_array_t *ca          = menu_display_get_coords_array();
 
-   draw->x           = 0;
-   draw->y           = 0;
-   draw->coords      = (struct video_coords*)(&ca->coords);
-   draw->matrix_data = NULL;
+   draw->x                          = 0;
+   draw->y                          = 0;
+   draw->coords                     = (struct video_coords*)(&ca->coords);
+   draw->matrix_data                = NULL;
+
+   switch (draw->pipeline.id)
+   {
+      case VIDEO_SHADER_MENU:
+      case VIDEO_SHADER_MENU_2:
+         glBlendFunc(GL_ONE, GL_ONE);
+         break;
+      default:
+         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+         break;
+   }
 
    switch (draw->pipeline.id)
    {
@@ -175,11 +184,12 @@ static void menu_display_gl_draw_pipeline(void *data)
       case VIDEO_SHADER_MENU_3:
       case VIDEO_SHADER_MENU_4:
       case VIDEO_SHADER_MENU_5:
+      case VIDEO_SHADER_MENU_6:
          shader_info.data       = NULL;
          shader_info.idx        = draw->pipeline.id;
          shader_info.set_active = true;
 
-         video_shader_driver_use(shader_info);
+         video_shader_driver_use(&shader_info);
 
          t += 0.01;
 
@@ -196,7 +206,7 @@ static void menu_display_gl_draw_pipeline(void *data)
 
          uniform_param.result.f.v0       = t;
 
-         video_shader_driver_set_parameter(uniform_param);            
+         video_shader_driver_set_parameter(&uniform_param);
          break;
    }
 
@@ -205,13 +215,14 @@ static void menu_display_gl_draw_pipeline(void *data)
       case VIDEO_SHADER_MENU_3:
       case VIDEO_SHADER_MENU_4:
       case VIDEO_SHADER_MENU_5:
+      case VIDEO_SHADER_MENU_6:
 #ifndef HAVE_PSGL
          uniform_param.type              = UNIFORM_2F;
          uniform_param.lookup.ident      = "OutputSize";
          uniform_param.result.f.v0       = draw->width;
          uniform_param.result.f.v1       = draw->height;
 
-         video_shader_driver_set_parameter(uniform_param);
+         video_shader_driver_set_parameter(&uniform_param);
 #endif
          break;
    }
@@ -223,7 +234,9 @@ static void menu_display_gl_restore_clear_color(void)
    glClearColor(0.0f, 0.0f, 0.0f, 0.00f);
 }
 
-static void menu_display_gl_clear_color(menu_display_ctx_clearcolor_t *clearcolor)
+static void menu_display_gl_clear_color(
+      menu_display_ctx_clearcolor_t *clearcolor,
+      video_frame_info_t *video_info)
 {
    if (!clearcolor)
       return;
@@ -235,13 +248,16 @@ static void menu_display_gl_clear_color(menu_display_ctx_clearcolor_t *clearcolo
 
 static bool menu_display_gl_font_init_first(
       void **font_handle, void *video_data,
-      const char *font_path, float font_size)
+      const char *font_path, float font_size,
+      bool is_threaded)
 {
    font_data_t **handle = (font_data_t**)font_handle;
-   *handle = font_driver_init_first(video_data,
-         font_path, font_size, true, FONT_DRIVER_RENDER_OPENGL_API);
-
-   return *handle;
+   if (!(*handle = font_driver_init_first(video_data,
+         font_path, font_size, true,
+         is_threaded,
+         FONT_DRIVER_RENDER_OPENGL_API)))
+		 return false;
+   return true;
 }
 
 menu_display_ctx_driver_t menu_display_ctx_gl = {
@@ -258,4 +274,5 @@ menu_display_ctx_driver_t menu_display_ctx_gl = {
    menu_display_gl_font_init_first,
    MENU_VIDEO_DRIVER_OPENGL,
    "menu_display_gl",
+   false
 };
