@@ -19,34 +19,35 @@
 #include <retro_miscellaneous.h>
 
 #include "../../gfx/font_driver.h"
-#include "../../gfx/video_context_driver.h"
+#include "../../gfx/video_driver.h"
 
-#include "../menu_display.h"
+#include "../menu_driver.h"
 
-static void *menu_display_null_get_default_mvp(void)
+static void *menu_display_null_get_default_mvp(video_frame_info_t *video_info)
 {
    return NULL;
 }
 
-static void menu_display_null_blend_begin(void)
+static void menu_display_null_blend_begin(video_frame_info_t *video_info)
 {
 }
 
-static void menu_display_null_blend_end(void)
+static void menu_display_null_blend_end(video_frame_info_t *video_info)
 {
 }
 
-static void menu_display_null_draw(void *data)
+static void menu_display_null_draw(void *data, video_frame_info_t *video_info)
 {
    (void)data;
 }
 
-static void menu_display_null_draw_pipeline(void *data)
+static void menu_display_null_draw_pipeline(
+      void *data, video_frame_info_t *video_info)
 {
    (void)data;
 }
 
-static void menu_display_null_viewport(void *data)
+static void menu_display_null_viewport(void *data, video_frame_info_t *video_info)
 {
    (void)data;
 }
@@ -55,14 +56,17 @@ static void menu_display_null_restore_clear_color(void)
 {
 }
 
-static void menu_display_null_clear_color(menu_display_ctx_clearcolor_t *clearcolor)
+static void menu_display_null_clear_color(
+      menu_display_ctx_clearcolor_t *clearcolor,
+      video_frame_info_t *video_info)
 {
    (void)clearcolor;
 }
 
 static bool menu_display_null_font_init_first(
       void **font_handle, void *video_data,
-      const char *font_path, float font_size)
+      const char *font_path, float font_size,
+      bool is_threaded)
 {
    return true;
 }
@@ -93,4 +97,5 @@ menu_display_ctx_driver_t menu_display_ctx_null = {
    menu_display_null_font_init_first,
    MENU_VIDEO_DRIVER_GENERIC,
    "menu_display_null",
+   false
 };
