@@ -1,4 +1,4 @@
-/*  RetroArch - A frontend for libretro.
+﻿/*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2016-2017 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
@@ -20,12 +20,11 @@
 #include "../../config.h"
 #endif
 
-#include "../menu_display.h"
+#include "../menu_driver.h"
 
 #include "../../gfx/font_driver.h"
-#include "../../gfx/video_context_driver.h"
+#include "../../gfx/video_driver.h"
 #include "../../gfx/common/vulkan_common.h"
-#include "../../gfx/video_shader_driver.h"
 
 /* Will do Y-flip later, but try to make it similar to GL. */
 static const float vk_vertexes[] = {
@@ -42,9 +41,16 @@ static const float vk_tex_coords[] = {
    1, 0
 };
 
-static void *menu_display_vk_get_default_mvp(void)
+static const float vk_colors[] = {
+   1.0f, 1.0f, 1.0f, 1.0f,
+   1.0f, 1.0f, 1.0f, 1.0f,
+   1.0f, 1.0f, 1.0f, 1.0f,
+   1.0f, 1.0f, 1.0f, 1.0f,
+};
+
+static void *menu_display_vk_get_default_mvp(video_frame_info_t *video_info)
 {
-   vk_t *vk = (vk_t*)video_driver_get_ptr(false);
+   vk_t *vk = video_info ? (vk_t*)video_info->userdata : NULL;
    if (!vk)
       return NULL;
    return &vk->mvp_no_rot;
@@ -55,6 +61,11 @@ static const float *menu_display_vk_get_default_vertices(void)
    return &vk_vertexes[0];
 }
 
+static const float *menu_display_vk_get_default_color(void)
+{
+   return &vk_colors[0];
+}
+
 static const float *menu_display_vk_get_default_tex_coords(void)
 {
    return &vk_tex_coords[0];
@@ -76,16 +87,24 @@ static unsigned to_menu_pipeline(
          return 4 + (type == MENU_DISPLAY_PRIM_TRIANGLESTRIP);
       case VIDEO_SHADER_MENU_2:
          return 6 + (type == MENU_DISPLAY_PRIM_TRIANGLESTRIP);
+      case VIDEO_SHADER_MENU_3:
+         return 8 + (type == MENU_DISPLAY_PRIM_TRIANGLESTRIP);
+      case VIDEO_SHADER_MENU_4:
+         return 10 + (type == MENU_DISPLAY_PRIM_TRIANGLESTRIP);
+      case VIDEO_SHADER_MENU_5:
+         return 12 + (type == MENU_DISPLAY_PRIM_TRIANGLESTRIP);
       default:
          return 0;
    }
 }
 #endif
 
-static void menu_display_vk_viewport(void *data)
+static void menu_display_vk_viewport(void *data,
+      video_frame_info_t *video_info)
 {
    menu_display_ctx_draw_t *draw = (menu_display_ctx_draw_t*)data;
-   vk_t *vk                      = (vk_t*)video_driver_get_ptr(false);
+   vk_t *vk                      = video_info ? (vk_t*)video_info->userdata 
+      : NULL;
 
    if (!vk || !draw)
       return;
@@ -98,29 +117,69 @@ static void menu_display_vk_viewport(void *data)
    vk->vk_vp.maxDepth = 1.0f;
 }
 
-static void menu_display_vk_draw_pipeline(void *data)
+static void menu_display_vk_draw_pipeline(void *data, video_frame_info_t *video_info)
 {
 #ifdef HAVE_SHADERPIPELINE
+   float output_size[2];
    menu_display_ctx_draw_t *draw = (menu_display_ctx_draw_t*)data;
-   vk_t *vk                      = (vk_t*)video_driver_get_ptr(false);
-   video_coord_array_t *ca         = NULL;
+   video_coord_array_t *ca       = NULL;
+   vk_t *vk                      = video_info ? (vk_t*)video_info->userdata : NULL;
+
+   static uint8_t ubo_scratch_data[768];
    static float t                = 0.0f;
+   static struct video_coords blank_coords;
 
    if (!vk || !draw)
       return;
 
-   ca = menu_display_get_coords_array();
-   draw->x                     = 0;
-   draw->y                     = 0;
-   draw->coords                = (struct video_coords*)&ca->coords;
-   draw->matrix_data           = NULL;
-   draw->pipeline.backend_data = &t;
+   draw->x                          = 0;
+   draw->y                          = 0;
+   draw->matrix_data                = NULL;
+
+   output_size[0]                   = (float)vk->context->swapchain_width;
+   output_size[1]                   = (float)vk->context->swapchain_height;
+
+   switch (draw->pipeline.id)
+   {
+      /* Ribbon */
+      default:
+      case VIDEO_SHADER_MENU:
+      case VIDEO_SHADER_MENU_2:
+         ca = menu_display_get_coords_array();
+         draw->coords                     = (struct video_coords*)&ca->coords;
+         draw->pipeline.backend_data      = ubo_scratch_data;
+         draw->pipeline.backend_data_size = sizeof(float);
+
+         /* Match UBO layout in shader. */
+         memcpy(ubo_scratch_data, &t, sizeof(t));
+         break;
+
+      /* Snow simple */
+      case VIDEO_SHADER_MENU_3:
+      case VIDEO_SHADER_MENU_4:
+      case VIDEO_SHADER_MENU_5:
+         draw->pipeline.backend_data      = ubo_scratch_data;
+         draw->pipeline.backend_data_size = sizeof(math_matrix_4x4) + 3 * sizeof(float);
+
+         /* Match UBO layout in shader. */
+         memcpy(ubo_scratch_data,
+               menu_display_vk_get_default_mvp(video_info),
+               sizeof(math_matrix_4x4));
+         memcpy(ubo_scratch_data + sizeof(math_matrix_4x4),
+               output_size,
+               sizeof(output_size));
+         memcpy(ubo_scratch_data + sizeof(math_matrix_4x4) + 2 * sizeof(float), &t, sizeof(t));
+         draw->coords = &blank_coords;
+         blank_coords.vertices = 4;
+         draw->prim_type = MENU_DISPLAY_PRIM_TRIANGLESTRIP;
+         break;
+   }
 
    t += 0.01;
 #endif
 }
 
-static void menu_display_vk_draw(void *data)
+static void menu_display_vk_draw(void *data, video_frame_info_t *video_info)
 {
    unsigned i;
    struct vk_buffer_range range;
@@ -130,7 +189,7 @@ static void menu_display_vk_draw(void *data)
    const float *color            = NULL;
    struct vk_vertex *pv          = NULL;
    menu_display_ctx_draw_t *draw = (menu_display_ctx_draw_t*)data;
-   vk_t *vk                      = (vk_t*)video_driver_get_ptr(false);
+   vk_t *vk                      = video_info ? (vk_t*)video_info->userdata : NULL;
 
    if (!vk || !draw)
       return;
@@ -148,8 +207,11 @@ static void menu_display_vk_draw(void *data)
       draw->coords->lut_tex_coord = menu_display_vk_get_default_tex_coords();
    if (!texture)
       texture         = &vk->display.blank_texture;
+   if (!color)
+      color           = menu_display_vk_get_default_color();
+
+   menu_display_vk_viewport(draw, video_info);
 
-   menu_display_vk_viewport(draw);
    vk->tracker.dirty |= VULKAN_DIRTY_DYNAMIC_BIT;
 
    /* Bake interleaved VBO. Kinda ugly, we should probably try to move to
@@ -162,7 +224,8 @@ static void menu_display_vk_draw(void *data)
    for (i = 0; i < draw->coords->vertices; i++, pv++)
    {
       pv->x       = *vertex++;
-      pv->y       = 1.0f - (*vertex++); /* Y-flip. Vulkan is top-left clip space */
+      /* Y-flip. Vulkan is top-left clip space */
+      pv->y       = 1.0f - (*vertex++);
       pv->tex_x   = *tex_coord++;
       pv->tex_y   = *tex_coord++;
       pv->color.r = *color++;
@@ -180,36 +243,41 @@ static void menu_display_vk_draw(void *data)
       case VIDEO_SHADER_MENU_4:
       case VIDEO_SHADER_MENU_5:
       {
-         const struct vk_draw_triangles call = {
-            vk->display.pipelines[
-               to_menu_pipeline(draw->prim_type, draw->pipeline.id)],
-               NULL,
-               VK_NULL_HANDLE,
-               draw->pipeline.backend_data,
-               sizeof(float),
-               &range,
-               draw->coords->vertices,
-         };
+         struct vk_draw_triangles call;
+
+         call.pipeline     = vk->display.pipelines[
+               to_menu_pipeline(draw->prim_type, draw->pipeline.id)];
+         call.texture      = NULL;
+         call.sampler      = VK_NULL_HANDLE;
+         call.uniform      = draw->pipeline.backend_data;
+         call.uniform_size = draw->pipeline.backend_data_size;
+         call.vbo          = &range;
+         call.vertices     = draw->coords->vertices;
+
          vulkan_draw_triangles(vk, &call);
          break;
       }
+
+         break;
 #endif
 
       default:
       {
-         const struct vk_draw_triangles call = {
-            vk->display.pipelines[
-               to_display_pipeline(draw->prim_type, vk->display.blend)],
-               texture,
-               texture->mipmap ?
-                  vk->samplers.mipmap_linear :
-                  (texture->default_smooth ? vk->samplers.linear : vk->samplers.nearest),
-               draw->matrix_data
-                  ? draw->matrix_data : menu_display_vk_get_default_mvp(),
-               sizeof(math_matrix_4x4),
-               &range,
-               draw->coords->vertices,
-         };
+         struct vk_draw_triangles call;
+
+         call.pipeline     = vk->display.pipelines[
+               to_display_pipeline(draw->prim_type, vk->display.blend)];
+         call.texture      = texture;
+         call.sampler      = texture->mipmap ?
+            vk->samplers.mipmap_linear :
+            (texture->default_smooth ? vk->samplers.linear
+             : vk->samplers.nearest);
+         call.uniform      = draw->matrix_data
+            ? draw->matrix_data : menu_display_vk_get_default_mvp(video_info);
+         call.uniform_size = sizeof(math_matrix_4x4);
+         call.vbo          = &range;
+         call.vertices     = draw->coords->vertices;
+
          vulkan_draw_triangles(vk, &call);
          break;
       }
@@ -220,22 +288,25 @@ static void menu_display_vk_restore_clear_color(void)
 {
 }
 
-static void menu_display_vk_clear_color(menu_display_ctx_clearcolor_t *clearcolor)
+static void menu_display_vk_clear_color(
+      menu_display_ctx_clearcolor_t *clearcolor,
+      video_frame_info_t *video_info)
 {
    VkClearRect rect;
    VkClearAttachment attachment;
-   vk_t *vk                      = (vk_t*)video_driver_get_ptr(false);
+   vk_t *vk = video_info ? (vk_t*)video_info->userdata : NULL;
    if (!vk || !clearcolor)
       return;
 
    memset(&attachment, 0, sizeof(attachment));
+   memset(&rect, 0, sizeof(rect));
+
    attachment.aspectMask                  = VK_IMAGE_ASPECT_COLOR_BIT;
    attachment.clearValue.color.float32[0] = clearcolor->r;
    attachment.clearValue.color.float32[1] = clearcolor->g;
    attachment.clearValue.color.float32[2] = clearcolor->b;
    attachment.clearValue.color.float32[3] = clearcolor->a;
 
-   memset(&rect, 0, sizeof(rect));
    rect.rect.extent.width  = vk->context->swapchain_width;
    rect.rect.extent.height = vk->context->swapchain_height;
    rect.layerCount         = 1;
@@ -243,27 +314,36 @@ static void menu_display_vk_clear_color(menu_display_ctx_clearcolor_t *clearcolo
    vkCmdClearAttachments(vk->cmd, 1, &attachment, 1, &rect);
 }
 
-static void menu_display_vk_blend_begin(void)
+static void menu_display_vk_blend_begin(video_frame_info_t *video_info)
 {
-   vk_t *vk = (vk_t*)video_driver_get_ptr(false);
-   vk->display.blend = true;
+   vk_t *vk = video_info ? (vk_t*)video_info->userdata : NULL;
+
+   if (vk)
+      vk->display.blend = true;
 }
 
-static void menu_display_vk_blend_end(void)
+static void menu_display_vk_blend_end(video_frame_info_t *video_info)
 {
-   vk_t *vk = (vk_t*)video_driver_get_ptr(false);
-   vk->display.blend = false;
+   vk_t *vk = video_info ? (vk_t*)video_info->userdata : NULL;
+
+   if (vk)
+      vk->display.blend = false;
 }
 
 static bool menu_display_vk_font_init_first(
       void **font_handle, void *video_data, const char *font_path,
-      float font_size)
+      float font_size, bool is_threaded)
 {
    font_data_t **handle = (font_data_t**)font_handle;
    *handle = font_driver_init_first(video_data,
-         font_path, font_size, true, FONT_DRIVER_RENDER_VULKAN_API);
+         font_path, font_size, true,
+         is_threaded,
+         FONT_DRIVER_RENDER_VULKAN_API);
 
-   return *handle;
+   if (*handle)
+      return true;
+
+   return false;
 }
 
 menu_display_ctx_driver_t menu_display_ctx_vulkan = {
@@ -280,4 +360,5 @@ menu_display_ctx_driver_t menu_display_ctx_vulkan = {
    menu_display_vk_font_init_first,
    MENU_VIDEO_DRIVER_VULKAN,
    "menu_display_vulkan",
+   false
 };
