@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -67,20 +67,18 @@ static void libdbg_font_free_font(void *data, bool is_threaded)
 static void libdbg_font_render_msg(
       video_frame_info_t *video_info,
       void *data, const char *msg,
-      const void *userdata)
+      const struct font_params *params)
 {
    float x, y, scale;
    unsigned color;
-   const struct font_params *params = (const struct font_params*)userdata;
-
-   (void)data;
 
    if (params)
    {
-      x     = params->x;
-      y     = params->y;
-      scale = params->scale;
-      color = params->color;
+      x                    = params->x;
+      y                    = params->y;
+      scale                = params->scale;
+
+      color                = params->color;
    }
    else
    {
@@ -96,7 +94,7 @@ static void libdbg_font_render_msg(
       DbgFontPrint(x, y, scale - 0.01f, WHITE, msg);
 
 #ifdef SN_TARGET_PSP2
-   /* FIXME - if we ever get around to this port, 
+   /* FIXME - if we ever get around to this port,
     * move this out to some better place */
    sceDbgFontFlush();
 #endif
