@@ -124,7 +124,7 @@ static bool coretext_font_renderer_create_atlas(CTFontRef face, ct_font_renderer
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
          kCTFontOrientationDefault,
 #else
-         kCTFontDefaultOrientation, 
+         kCTFontDefaultOrientation,
 #endif
          glyphs, advances, CT_ATLAS_SIZE);
 
@@ -177,8 +177,8 @@ static bool coretext_font_renderer_create_atlas(CTFontRef face, ct_font_renderer
 
    CGContextSetTextMatrix(offscreen, CGAffineTransformIdentity);
 
-   attr = CFDictionaryCreate(NULL, (const void **)&keys, (const void **)&values, 
-         sizeof(keys) / sizeof(keys[0]), &kCFTypeDictionaryKeyCallBacks, 
+   attr = CFDictionaryCreate(NULL, (const void **)&keys, (const void **)&values,
+         sizeof(keys) / sizeof(keys[0]), &kCFTypeDictionaryKeyCallBacks,
          &kCFTypeDictionaryValueCallBacks);
 
    for (i = 0; i < CT_ATLAS_SIZE; i++)
@@ -208,7 +208,7 @@ static bool coretext_font_renderer_create_atlas(CTFontRef face, ct_font_renderer
       glyph_cstr[1] = 0;
       glyph_cfstr   = CFStringCreateWithCString(
             NULL, glyph_cstr, kCFStringEncodingASCII );
-      attrString = 
+      attrString =
          CFAttributedStringCreate(NULL, glyph_cfstr, attr);
       CFRelease(glyph_cfstr);
       glyph_cfstr = NULL;
@@ -231,8 +231,8 @@ static bool coretext_font_renderer_create_atlas(CTFontRef face, ct_font_renderer
       {
          for (c = 0; c < max_width; c++)
          {
-            unsigned src_idx = r * bytesPerRow + c;
-            unsigned dest_idx = 
+            unsigned src_idx = (unsigned)(r * bytesPerRow + c);
+            unsigned dest_idx =
                (r + offset_y) * (CT_ATLAS_COLS * max_width) + (c + offset_x);
             uint8_t v = src[src_idx];
 
@@ -253,9 +253,12 @@ static bool coretext_font_renderer_create_atlas(CTFontRef face, ct_font_renderer
 
 static void *font_renderer_ct_init(const char *font_path, float font_size)
 {
-   char err                   = 0;
-   CFStringRef cf_font_path   = NULL;
-   CTFontRef face             = NULL;
+   char err                       = 0;
+   CFStringRef cf_font_path       = NULL;
+   CTFontRef face                 = NULL;
+   CFURLRef url                   = NULL;
+   CGDataProviderRef dataProvider = NULL;
+   CGFontRef theCGFont            = NULL;
    ct_font_renderer_t *handle = (ct_font_renderer_t*)
       calloc(1, sizeof(*handle));
 
@@ -273,7 +276,10 @@ static void *font_renderer_ct_init(const char *font_path, float font_size)
       goto error;
    }
 
-   face = CTFontCreateWithName(cf_font_path, font_size, NULL);
+   url = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, cf_font_path, kCFURLPOSIXPathStyle, false);
+   dataProvider = CGDataProviderCreateWithURL(url);
+   theCGFont = CGFontCreateWithDataProvider(dataProvider);
+   face = CTFontCreateWithGraphicsFont(theCGFont, font_size, NULL, NULL);
 
    if (!face)
    {
@@ -304,6 +310,21 @@ error:
       CFRelease(face);
       face = NULL;
    }
+   if (url)
+   {
+      CFRelease(url);
+      url = NULL;
+   }
+   if (dataProvider)
+   {
+      CFRelease(dataProvider);
+      dataProvider = NULL;
+   }
+   if (theCGFont)
+   {
+      CFRelease(theCGFont);
+      theCGFont = NULL;
+   }
 
    return handle;
 }
