@@ -19,9 +19,8 @@
 #include <gccore.h>
 #include <rthreads/rthreads.h>
 
-#include "../input_config.h"
 #include "../input_defines.h"
-#include "../input_hid_driver.h"
+#include "../input_driver.h"
 
 #include "../connect/joypad_connection.h"
 
@@ -44,7 +43,7 @@ typedef struct wiiusb_hid
    /* helps on knowing if a new device has been inserted */
    bool device_detected;
    /* helps on detecting that a device has just been removed */
-   bool removal_cb;      
+   bool removal_cb;
 
    bool manual_removal;
 } wiiusb_hid_t;
@@ -65,7 +64,7 @@ struct wiiusb_adapter
    int32_t slot;
    uint8_t *data;
 
-   uint8_t	send_control_type;
+   uint8_t  send_control_type;
    uint8_t *send_control_buffer;
    uint32_t send_control_size;
 };
@@ -114,10 +113,10 @@ static int32_t wiiusb_hid_read_cb(int32_t size, void *data)
       pad_connection_packet(&hid->connections[adapter->slot],
             adapter->slot, adapter->data-1, size+1);
 
-	if (adapter)
+  if (adapter)
       adapter->busy = false;
 
-	return size;
+  return size;
 }
 
 static void wiiusb_hid_device_send_control(void *data,
@@ -323,7 +322,7 @@ static int wiiusb_hid_add_adapter(void *data, usb_device_entry *dev)
     * control name until we get its interface */
    adapter->slot = pad_connection_pad_init(hid->connections,
          "hid", desc.idVendor, desc.idProduct,
-         adapter, &wiiusb_hid_device_send_control);
+         adapter, &wiiusb_hid);
 
    if (adapter->slot == -1)
       goto error;
@@ -399,7 +398,7 @@ static void wiiusb_hid_scan_for_devices(wiiusb_hid_t *hid)
 
    for (i = 0; i < count; i++)
    {
-		/* first check the device is not already in our list */
+    /* first check the device is not already in our list */
       if (!wiiusb_hid_new_device(hid, dev_entries[i].device_id))
          continue;
 
@@ -462,11 +461,11 @@ static int wiiusb_hid_change_cb(int result, void *usrdata)
       return -1;
 
    /* As it's not coming from the removal callback
-      then we	detected a new device being inserted */
-	if (!hid->removal_cb)
-		hid->device_detected = true;
-	else
-		hid->removal_cb      = false;
+      then we detected a new device being inserted */
+  if (!hid->removal_cb)
+    hid->device_detected = true;
+  else
+    hid->removal_cb      = false;
 
    /* Re-submit the change alert */
    USB_DeviceChangeNotifyAsync(USB_CLASS_HID, wiiusb_hid_change_cb, usrdata);
@@ -479,33 +478,40 @@ static bool wiiusb_hid_joypad_query(void *data, unsigned pad)
    return pad < MAX_USERS;
 }
 
-static uint64_t wiiusb_hid_joypad_get_buttons(void *data, unsigned port)
+static void wiiusb_hid_joypad_get_buttons(void *data,
+      unsigned port, input_bits_t *state)
 {
-   wiiusb_hid_t *hid = (wiiusb_hid_t*)data;
-   if (hid)
-      return pad_connection_get_buttons(&hid->connections[port], port);
-   return 0;
+  wiiusb_hid_t *hid = (wiiusb_hid_t*)data;
+  if (hid)
+  {
+    pad_connection_get_buttons(&hid->connections[port], port, state);
+    return;
+  }
+  BIT256_CLEAR_ALL_PTR(state);
 }
 
-static bool wiiusb_hid_joypad_button(void *data, unsigned port, uint16_t joykey)
+static bool wiiusb_hid_joypad_button(void *data,
+      unsigned port, uint16_t joykey)
 {
-   uint64_t buttons = wiiusb_hid_joypad_get_buttons(data, port);
+  input_bits_t buttons;
 
-   /* Check hat. */
-   if (GET_HAT_DIR(joykey))
-      return false;
+  wiiusb_hid_joypad_get_buttons(data, port, &buttons);
 
-   /* Check the button. */
-   if ((port < MAX_USERS) && (joykey < 32))
-      return ((buttons & (1 << joykey)) != 0);
+  /* Check hat. */
+  if (GET_HAT_DIR(joykey))
+    return false;
 
-   return false;
+  /* Check the button. */
+  if ((port < MAX_USERS) && (joykey < 32))
+    return (BIT256_GET(buttons, joykey) != 0);
+
+  return false;
 }
 
 static bool wiiusb_hid_joypad_rumble(void *data, unsigned pad,
       enum retro_rumble_effect effect, uint16_t strength)
 {
-   wiiusb_hid_t *hid	= (wiiusb_hid_t*)data;
+   wiiusb_hid_t *hid  = (wiiusb_hid_t*)data;
 
    if (!hid)
       return false;
@@ -542,7 +548,7 @@ static int16_t wiiusb_hid_joypad_axis(void *data,
    return val;
 }
 
-static void wiiusb_hid_free(void *data)
+static void wiiusb_hid_free(const void *data)
 {
    struct wiiusb_adapter      *adapter = NULL;
    struct wiiusb_adapter *next_adapter = NULL;
@@ -626,4 +632,5 @@ hid_driver_t wiiusb_hid = {
    wiiusb_hid_joypad_rumble,
    wiiusb_hid_joypad_name,
    "wiiusb",
+   wiiusb_hid_device_send_control,
 };
