@@ -138,13 +138,13 @@ static void apple_gamecontroller_joypad_connect(GCController *controller)
        ? desired_index : 0;
     
     /* prevent same controller getting set twice */
-    if (mfi_controllers[desired_index] != controller.hash)
+    if (mfi_controllers[desired_index] != (uint32_t)controller.hash)
     {
         /* desired slot is unused, take it */
         if (!mfi_controllers[desired_index])
         {
             controller.playerIndex = desired_index;
-            mfi_controllers[desired_index] = controller.hash;
+            mfi_controllers[desired_index] = (uint32_t)controller.hash;
         }
         else
         {
@@ -156,7 +156,7 @@ static void apple_gamecontroller_joypad_connect(GCController *controller)
               if (mfi_controllers[i])
                  continue;
 
-              mfi_controllers[i] = controller.hash;
+              mfi_controllers[i] = (uint32_t)controller.hash;
               controller.playerIndex = i;
               break;
            }
@@ -221,9 +221,10 @@ static bool apple_gamecontroller_joypad_button(unsigned port, uint16_t joykey)
    return false;
 }
 
-static uint64_t apple_gamecontroller_joypad_get_buttons(unsigned port)
+static void apple_gamecontroller_joypad_get_buttons(unsigned port,
+      input_bits_t *state)
 {
-   return mfi_buttons[port];
+	BITS_COPY16_PTR(state, mfi_buttons[port]);
 }
 
 static int16_t apple_gamecontroller_joypad_axis(unsigned port, uint32_t joyaxis)
