@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -68,9 +68,14 @@ static bool ps3_joypad_button(unsigned port_num, uint16_t joykey)
    return pad_state[port_num] & (UINT64_C(1) << joykey);
 }
 
-static uint64_t ps3_joypad_get_buttons(unsigned port_num)
+static void ps3_joypad_get_buttons(unsigned port_num, input_bits_t *state)
 {
-   return pad_state[port_num];
+	if (port_num < MAX_PADS)
+   {
+		BITS_COPY16_PTR( state, pad_state[port_num] );
+	}
+   else
+		BIT256_CLEAR_ALL_PTR(state);
 }
 
 static int16_t ps3_joypad_axis(unsigned port_num, uint32_t joyaxis)
@@ -142,7 +147,7 @@ static void ps3_joypad_poll(void)
             ps3_joypad_autodetect_add(port);
          }
       }
-      
+
       if (pads_connected[port] == 0)
          continue;
 
