@@ -17,7 +17,7 @@
 #include <stdint.h>
 #include <string.h>
 
-#include "../input_config.h"
+#include "../input_driver.h"
 
 #include "../../tasks/tasks_internal.h"
 
@@ -81,19 +81,6 @@ static const char *psp_joypad_name(unsigned pad)
 #endif
 }
 
-static void psp_joypad_autodetect_add(unsigned autoconf_pad)
-{
-   if (!input_autoconfigure_connect(
-         psp_joypad_name(autoconf_pad),
-         NULL,
-         psp_joypad.ident,
-         autoconf_pad,
-         0,
-         0
-         ))
-      input_config_set_device_name(autoconf_pad, psp_joypad_name(autoconf_pad));
-}
-
 static bool psp_joypad_init(void *data)
 {
    unsigned i;
@@ -114,7 +101,17 @@ static bool psp_joypad_init(void *data)
 #endif
 
    for (i = 0; i < players_count; i++)
-      psp_joypad_autodetect_add(i);
+   {
+      if (!input_autoconfigure_connect(
+               psp_joypad_name(i),
+               NULL,
+               psp_joypad.ident,
+               i,
+               0,
+               0
+               ))
+         input_config_set_device_name(i, psp_joypad_name(i));
+   }
 
    return true;
 }
@@ -127,9 +124,14 @@ static bool psp_joypad_button(unsigned port_num, uint16_t key)
    return (pad_state[port_num] & (UINT64_C(1) << key));
 }
 
-static uint64_t psp_joypad_get_buttons(unsigned port_num)
+static void psp_joypad_get_buttons(unsigned port_num, input_bits_t *state)
 {
-   return pad_state[port_num];
+	if (port_num < PSP_MAX_PADS)
+   {
+		BITS_COPY16_PTR( state, pad_state[port_num] );
+	}
+   else
+      BIT256_CLEAR_ALL_PTR(state);
 }
 
 static int16_t psp_joypad_axis(unsigned port_num, uint32_t joyaxis)
@@ -209,7 +211,17 @@ static void psp_joypad_poll(void)
 
          if (old_ctrl_info.port[player + 1] == SCE_CTRL_TYPE_UNPAIRED &&
                curr_ctrl_info.port[player + 1] != SCE_CTRL_TYPE_UNPAIRED)
-            psp_joypad_autodetect_add(player);
+         {
+            if (!input_autoconfigure_connect(
+                     psp_joypad_name(player),
+                     NULL,
+                     psp_joypad.ident,
+                     player,
+                     0,
+                     0
+                     ))
+               input_config_set_device_name(player, psp_joypad_name(player));
+         }
       }
       memcpy(&old_ctrl_info, &curr_ctrl_info, sizeof(SceCtrlPortInfo));
    }
@@ -229,10 +241,10 @@ static void psp_joypad_poll(void)
       if (curr_ctrl_info.port[p] == SCE_CTRL_TYPE_UNPAIRED)
          continue;
 #elif defined(SN_TARGET_PSP2)
-      /* Dumb hack, but here's the explanation - 
+      /* Dumb hack, but here's the explanation -
        * sceCtrlPeekBufferPositive's port parameter
        * can be 0 or 1 to read the first controller on
-       * a PSTV, but HAS to be 0 for a real VITA and 2 
+       * a PSTV, but HAS to be 0 for a real VITA and 2
        * for the 2nd controller on a PSTV */
       unsigned p  = (player > 0) ? player+1 : player;
 #else
@@ -249,13 +261,13 @@ static void psp_joypad_poll(void)
          continue;
 #endif
 #if defined(VITA)
-      if (psp2_model == SCE_KERNEL_MODEL_VITA 
-         && !menu_driver_is_alive()
-         && settings->input.backtouch_enable)
+      if (psp2_model == SCE_KERNEL_MODEL_VITA
+         && settings->bools.input_backtouch_enable)
       {
          unsigned i;
          SceTouchData touch_surface = {0};
-         sceTouchPeek(settings->input.backtouch_toggle ? SCE_TOUCH_PORT_FRONT : SCE_TOUCH_PORT_BACK, &touch_surface, 1);
+         sceTouchPeek(settings->bools.input_backtouch_toggle
+               ? SCE_TOUCH_PORT_FRONT : SCE_TOUCH_PORT_BACK, &touch_surface, 1);
 
          for (i = 0; i < touch_surface.reportNum; i++)
          {
