@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2013-2015 - pinumbernumber
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- *  
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -36,10 +36,14 @@
 #endif
 
 #include "../../tasks/tasks_internal.h"
-#include "../input_config.h"
+#include "../input_driver.h"
 
 #include "../../verbosity.h"
 
+#ifdef HAVE_DINPUT
+#include "dinput_joypad.h"
+#endif
+
 /* Check if the definitions do not already exist.
  * Official and mingw xinput headers have different include guards.
  */
@@ -92,12 +96,8 @@ typedef struct
 #define ERROR_DEVICE_NOT_CONNECTED 1167
 #endif
 
-#ifndef HAVE_DINPUT
-#error Cannot compile xinput without dinput.
-#endif
-
-/* Due to 360 pads showing up under both XInput and DirectInput, 
- * and since we are going to have to pass through unhandled 
+/* Due to 360 pads showing up under both XInput and DirectInput,
+ * and since we are going to have to pass through unhandled
  * joypad numbers to DirectInput, a slightly ugly
  * hack is required here. dinput_joypad_init will fill this.
  *
@@ -107,8 +107,10 @@ typedef struct
 extern int g_xinput_pad_indexes[MAX_USERS];
 extern bool g_xinput_block_pads;
 
+#ifdef HAVE_DYNAMIC
 /* For xinput1_n.dll */
 static dylib_t g_xinput_dll;
+#endif
 
 /* Function pointer, to be assigned with dylib_proc */
 typedef uint32_t (__stdcall *XInputGetStateEx_t)(uint32_t, XINPUT_STATE*);
@@ -157,6 +159,8 @@ static const char* const XBOX_ONE_CONTROLLER_NAMES[4] =
 const char *xinput_joypad_name(unsigned pad)
 {
    int xuser = pad_index_to_xuser_index(pad);
+
+#ifdef HAVE_DINPUT
    /* Use the real controller name for XBOX One controllers since
       they are slightly different  */
    if (xuser < 0)
@@ -164,6 +168,7 @@ const char *xinput_joypad_name(unsigned pad)
 
    if (strstr(dinput_joypad.name(pad), "Xbox One For Windows"))
       return XBOX_ONE_CONTROLLER_NAMES[xuser];
+#endif
 
    return XBOX_CONTROLLER_NAMES[xuser];
 }
@@ -175,7 +180,7 @@ static bool xinput_joypad_init(void *data)
    const char *version = "1.4";
 
    (void)data;
-
+#ifdef HAVE_DYNAMIC
    g_xinput_dll = NULL;
 
    /* Find the correct path to load the DLL from.
@@ -188,7 +193,7 @@ static bool xinput_joypad_init(void *data)
     * success anyway.
     */
 
-   g_xinput_dll = dylib_load("xinput1_4.dll"); 
+   g_xinput_dll = dylib_load("xinput1_4.dll");
    if (!g_xinput_dll)
    {
       g_xinput_dll = dylib_load("xinput1_3.dll");
@@ -197,16 +202,19 @@ static bool xinput_joypad_init(void *data)
 
    if (!g_xinput_dll)
    {
-      RARCH_ERR("Failed to load XInput, ensure DirectX and controller drivers are up to date.\n");
+      RARCH_ERR("[XInput]: Failed to load XInput, ensure DirectX and controller drivers are up to date.\n");
       return false;
    }
 
-   RARCH_LOG("Found XInput v%s.\n", version);
+   RARCH_LOG("[XInput]: Found XInput v%s.\n", version);
 
    /* If we get here then an xinput DLL is correctly loaded.
     * First try to load ordinal 100 (XInputGetStateEx).
     */
    g_XInputGetStateEx = (XInputGetStateEx_t)dylib_proc(g_xinput_dll, (const char*)100);
+#else
+   g_XInputGetStateEx = (XInputGetStateEx_t)XInputGetStateEx;
+#endif
    g_xinput_guide_button_supported = true;
 
    if (!g_XInputGetStateEx)
@@ -215,22 +223,34 @@ static bool xinput_joypad_init(void *data)
        * XInputGetState, at the cost of losing guide button support.
        */
       g_xinput_guide_button_supported = false;
+#ifdef HAVE_DYNAMIC
       g_XInputGetStateEx = (XInputGetStateEx_t)dylib_proc(g_xinput_dll, "XInputGetState");
+#else
+	  g_XInputGetStateEx = (XInputGetStateEx_t)XInputGetState;
+#endif
 
       if (!g_XInputGetStateEx)
       {
-         RARCH_ERR("Failed to init XInput: DLL is invalid or corrupt.\n");
+         RARCH_ERR("[XInput]: Failed to init: DLL is invalid or corrupt.\n");
+#ifdef HAVE_DYNAMIC
          dylib_close(g_xinput_dll);
+#endif
          return false; /* DLL was loaded but did not contain the correct function. */
       }
-      RARCH_WARN("XInput: No guide button support.\n");
+      RARCH_WARN("[XInput]: No guide button support.\n");
    }
 
+#ifdef HAVE_DYNAMIC
    g_XInputSetState = (XInputSetState_t)dylib_proc(g_xinput_dll, "XInputSetState");
+#else
+   g_XInputSetState = (XInputSetState_t)XInputSetState;
+#endif
    if (!g_XInputSetState)
    {
-      RARCH_ERR("Failed to init XInput: DLL is invalid or corrupt.\n");
+      RARCH_ERR("[XInput]: Failed to init: DLL is invalid or corrupt.\n");
+#ifdef HAVE_DYNAMIC
       dylib_close(g_xinput_dll);
+#endif
       return false; /* DLL was loaded but did not contain the correct function. */
    }
 
@@ -243,7 +263,7 @@ static bool xinput_joypad_init(void *data)
    {
       g_xinput_states[i].connected = !(g_XInputGetStateEx(i, &dummy_state) == ERROR_DEVICE_NOT_CONNECTED);
       if (g_xinput_states[i].connected)
-         RARCH_LOG("Found XInput controller, user #%u\n", i);
+         RARCH_LOG("[XInput]: Found controller, user #%u\n", i);
    }
 
    if ((!g_xinput_states[0].connected) &&
@@ -252,8 +272,11 @@ static bool xinput_joypad_init(void *data)
          (!g_xinput_states[3].connected))
       return false;
 
+   RARCH_LOG("[XInput]: Pads connected: %d\n", g_xinput_states[0].connected +
+         g_xinput_states[1].connected + g_xinput_states[2].connected + g_xinput_states[3].connected);
    g_xinput_block_pads = true;
 
+#ifdef HAVE_DINPUT
    /* We're going to have to be buddies with dinput if we want to be able
     * to use XInput and non-XInput controllers together. */
    if (!dinput_joypad.init(data))
@@ -261,18 +284,36 @@ static bool xinput_joypad_init(void *data)
       g_xinput_block_pads = false;
       return false;
    }
+#endif
 
    for (j = 0; j < MAX_USERS; j++)
    {
+      if (xinput_joypad_name(j))
+         RARCH_LOG("[XInput]: Attempting autoconf for \"%s\", user #%u\n", xinput_joypad_name(j), j);
+      else
+         RARCH_LOG("[XInput]: Attempting autoconf for user #%u\n", j);
+
       if (pad_index_to_xuser_index(j) > -1)
       {
+         int32_t vid          = 0;
+         int32_t pid          = 0;
+#ifdef HAVE_DINPUT
+         int32_t dinput_index = 0;
+         bool success     = dinput_joypad_get_vidpid_from_xinput_index((int32_t)pad_index_to_xuser_index(j), (int32_t*)&vid, (int32_t*)&pid,
+			 (int32_t*)&dinput_index);
+
+         if (success)
+            RARCH_LOG("[XInput]: Found VID/PID (%04X/%04X) from DINPUT index %d for \"%s\", user #%u\n",
+                  vid, pid, dinput_index, xinput_joypad_name(j), j);
+#endif
+
          if (!input_autoconfigure_connect(
                xinput_joypad_name(j),
                NULL,
                xinput_joypad.ident,
                j,
-               0,
-               0))
+               vid,
+               pid))
             input_config_set_device_name(j, xinput_joypad_name(j));
       }
    }
@@ -285,7 +326,11 @@ static bool xinput_joypad_query_pad(unsigned pad)
    int xuser = pad_index_to_xuser_index(pad);
    if (xuser > -1)
       return g_xinput_states[xuser].connected;
+#ifdef HAVE_DINPUT
    return dinput_joypad.query_pad(pad);
+#else
+   return false;
+#endif
 }
 
 static void xinput_joypad_destroy(void)
@@ -295,13 +340,17 @@ static void xinput_joypad_destroy(void)
    for (i = 0; i < 4; ++i)
       memset(&g_xinput_states[i], 0, sizeof(xinput_joypad_state));
 
+#ifdef HAVE_DYNAMIC
    dylib_close(g_xinput_dll);
 
    g_xinput_dll        = NULL;
+#endif
    g_XInputGetStateEx  = NULL;
    g_XInputSetState    = NULL;
 
+#ifdef HAVE_DINPUT
    dinput_joypad.destroy();
+#endif
 
    g_xinput_block_pads = false;
 }
@@ -320,7 +369,7 @@ static const uint16_t button_index_to_bitmap_code[] =  {
    XINPUT_GAMEPAD_BACK,
    XINPUT_GAMEPAD_LEFT_THUMB,
    XINPUT_GAMEPAD_RIGHT_THUMB,
-   XINPUT_GAMEPAD_GUIDE      
+   XINPUT_GAMEPAD_GUIDE
 };
 
 static bool xinput_joypad_button(unsigned port_num, uint16_t joykey)
@@ -330,8 +379,10 @@ static bool xinput_joypad_button(unsigned port_num, uint16_t joykey)
    unsigned hat_dir     = 0;
    int xuser            = pad_index_to_xuser_index(port_num);
 
+#ifdef HAVE_DINPUT
    if (xuser == -1)
       return dinput_joypad.button(port_num, joykey);
+#endif
 
    if (!(g_xinput_states[xuser].connected))
       return false;
@@ -368,25 +419,27 @@ static bool xinput_joypad_button(unsigned port_num, uint16_t joykey)
 static int16_t xinput_joypad_axis (unsigned port_num, uint32_t joyaxis)
 {
    int xuser;
-   int16_t val  = 0;
-   int     axis = -1;
-
-   bool is_neg  = false;
-   bool is_pos  = false;
+   int16_t val         = 0;
+   int     axis        = -1;
+   bool is_neg         = false;
+   bool is_pos         = false;
+   XINPUT_GAMEPAD* pad = NULL;
 
    if (joyaxis == AXIS_NONE)
       return 0;
 
    xuser = pad_index_to_xuser_index(port_num);
 
+#ifdef HAVE_DINPUT
    if (xuser == -1)
       return dinput_joypad.axis(port_num, joyaxis);
+#endif
 
    if (!(g_xinput_states[xuser].connected))
       return 0;
 
    /* triggers (axes 4,5) cannot be negative */
-   if (AXIS_NEG_GET(joyaxis) <= 3) 
+   if (AXIS_NEG_GET(joyaxis) <= 3)
    {
       axis = AXIS_NEG_GET(joyaxis);
       is_neg = true;
@@ -397,7 +450,7 @@ static int16_t xinput_joypad_axis (unsigned port_num, uint32_t joyaxis)
       is_pos = true;
    }
 
-   XINPUT_GAMEPAD* pad = &(g_xinput_states[xuser].xstate.Gamepad);
+   pad = &(g_xinput_states[xuser].xstate.Gamepad);
 
    switch (axis)
    {
@@ -441,14 +494,16 @@ static void xinput_joypad_poll(void)
    {
       if (g_xinput_states[i].connected)
       {
-         if (g_XInputGetStateEx(i, 
-                  &(g_xinput_states[i].xstate)) 
+         if (g_XInputGetStateEx && g_XInputGetStateEx(i,
+                  &(g_xinput_states[i].xstate))
                == ERROR_DEVICE_NOT_CONNECTED)
             g_xinput_states[i].connected = false;
       }
    }
 
+#ifdef HAVE_DINPUT
    dinput_joypad.poll();
+#endif
 }
 
 static bool xinput_joypad_rumble(unsigned pad,
@@ -458,8 +513,10 @@ static bool xinput_joypad_rumble(unsigned pad,
 
    if (xuser == -1)
    {
+#ifdef HAVE_DINPUT
       if (dinput_joypad.set_rumble)
          return dinput_joypad.set_rumble(pad, effect, strength);
+#endif
       return false;
    }
 
@@ -469,7 +526,10 @@ static bool xinput_joypad_rumble(unsigned pad,
    else if (effect == RETRO_RUMBLE_WEAK)
       g_xinput_rumble_states[xuser].wRightMotorSpeed = strength;
 
-   return (g_XInputSetState(xuser, &g_xinput_rumble_states[xuser]) 
+   if (!g_XInputSetState)
+      return false;
+
+   return (g_XInputSetState(xuser, &g_xinput_rumble_states[xuser])
       == 0);
 }
 
