@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -19,7 +19,7 @@
 
 #define AKEYCODE_ASSIST 219
 
-#define LAST_KEYCODE AKEYCODE_ASSIST 
+#define LAST_KEYCODE AKEYCODE_ASSIST
 
 #define MAX_KEYS ((LAST_KEYCODE + 7) / 8)
 
@@ -31,7 +31,7 @@ bool android_keyboard_port_input_pressed(const struct retro_keybind *binds, unsi
    if (id < RARCH_BIND_LIST_END)
    {
       const struct retro_keybind *bind = &binds[id];
-      unsigned bit = input_keymaps_translate_rk_to_keysym(bind->key);
+      unsigned bit                     = rarch_keysym_lut[bind->key];
       return BIT_GET(android_key_state[ANDROID_KEYBOARD_PORT], bit);
    }
    return false;
