@@ -19,21 +19,21 @@
 
 #include <xkbcommon/xkbcommon.h>
 
+#include <sys/mman.h>
+
 #include <lists/string_list.h>
 
-#include "../input_joypad_driver.h"
+#include "../input_driver.h"
 #include "../input_keymaps.h"
-#include "../input_keyboard.h"
 #include "../../configuration.h"
-#include <sys/mman.h>
 
 #define MOD_MAP_SIZE 5
 
-static struct xkb_context *xkb_ctx;
-static struct xkb_keymap  *xkb_map;
-static struct xkb_state   *xkb_state;
-static xkb_mod_index_t    *mod_map_idx;
-static uint16_t           *mod_map_bit;
+static struct xkb_context *xkb_ctx     = NULL;
+static struct xkb_keymap  *xkb_map     = NULL;
+static struct xkb_state   *xkb_state   = NULL;
+static xkb_mod_index_t    *mod_map_idx = NULL;
+static uint16_t           *mod_map_bit = 0;
 
 void free_xkb(void)
 {
@@ -57,18 +57,21 @@ void free_xkb(void)
 
 int init_xkb(int fd, size_t size)
 {
-   char *map_str;
-   mod_map_idx          = (xkb_mod_index_t *)calloc(MOD_MAP_SIZE, sizeof(xkb_mod_index_t));
+   char *map_str        = NULL;
+   mod_map_idx          = (xkb_mod_index_t *)calloc(
+         MOD_MAP_SIZE, sizeof(xkb_mod_index_t));
 
    if (!mod_map_idx)
       goto error;
 
-   mod_map_bit = (uint16_t*)calloc(MOD_MAP_SIZE, sizeof(uint16_t));
+   mod_map_bit          = (uint16_t*)
+      calloc(MOD_MAP_SIZE, sizeof(uint16_t));
 
    if (!mod_map_bit)
       goto error;
 
-   xkb_ctx = xkb_context_new(XKB_CONTEXT_NO_FLAGS);
+   xkb_ctx              = xkb_context_new(XKB_CONTEXT_NO_FLAGS);
+
    if (xkb_ctx)
    {
       if (fd >= 0)
@@ -77,7 +80,8 @@ int init_xkb(int fd, size_t size)
          if (map_str == MAP_FAILED)
             goto error;
 
-         xkb_map = xkb_keymap_new_from_string(xkb_ctx, map_str, XKB_KEYMAP_FORMAT_TEXT_V1, XKB_KEYMAP_COMPILE_NO_FLAGS);
+         xkb_map = xkb_keymap_new_from_string(xkb_ctx, map_str,
+               XKB_KEYMAP_FORMAT_TEXT_V1, XKB_KEYMAP_COMPILE_NO_FLAGS);
          munmap(map_str, size);
       }
       else
@@ -88,20 +92,23 @@ int init_xkb(int fd, size_t size)
 
          rule.rules = "evdev";
 
-         if (*settings->input.keyboard_layout)
+         if (*settings->arrays.input_keyboard_layout)
          {
-            list = string_split(settings->input.keyboard_layout, ":");
+            list = string_split(settings->arrays.input_keyboard_layout, ":");
             if (list && list->size >= 2)
                rule.variant = list->elems[1].data;
             if (list && list->size >= 1)
                rule.layout = list->elems[0].data;
          }
 
-         xkb_map = xkb_keymap_new_from_names(xkb_ctx, &rule, XKB_MAP_COMPILE_NO_FLAGS);
+         xkb_map = xkb_keymap_new_from_names(xkb_ctx,
+               &rule, XKB_MAP_COMPILE_NO_FLAGS);
+
          if (list)
             string_list_free(list);
       }
    }
+
    if (xkb_map)
    {
       xkb_mod_index_t *map_idx = (xkb_mod_index_t*)&mod_map_idx[0];
@@ -137,14 +144,15 @@ error:
    return -1;
 }
 
-void handle_xkb_state_mask(uint32_t depressed, uint32_t latched, uint32_t locked, uint32_t group)
+void handle_xkb_state_mask(uint32_t depressed,
+      uint32_t latched, uint32_t locked, uint32_t group)
 {
    if (!xkb_state)
       return;
    xkb_state_update_mask(xkb_state, depressed, latched, locked, 0, 0, group);
 }
 
-/* FIXME: Don't handle composed and dead-keys properly. 
+/* FIXME: Don't handle composed and dead-keys properly.
  * Waiting for support in libxkbcommon ... */
 int handle_xkb(int code, int value)
 {
@@ -164,14 +172,14 @@ int handle_xkb(int code, int value)
    if (value)
       num_syms = xkb_state_key_get_syms(xkb_state, xk_code, &syms);
 
-   if (!syms)
-      return -1;
-
    if (value > 0)
       xkb_state_update_key(xkb_state, xk_code, XKB_KEY_DOWN);
    else
       xkb_state_update_key(xkb_state, xk_code, XKB_KEY_UP);
 
+   if (!syms)
+      return -1;
+
    /* Build mod state. */
    for (i = 0; i < MOD_MAP_SIZE; i++)
    {
@@ -181,7 +189,8 @@ int handle_xkb(int code, int value)
       if (*map_idx != XKB_MOD_INVALID)
          mod |= xkb_state_mod_index_is_active(
                xkb_state, *map_idx,
-               (enum xkb_state_component)((XKB_STATE_MODS_EFFECTIVE) > 0)) ? *map_bit : 0;
+               (enum xkb_state_component)
+               ((XKB_STATE_MODS_EFFECTIVE) > 0)) ? *map_bit : 0;
    }
 
    input_keyboard_event(value, input_keymaps_translate_keysym_to_rk(code),
