@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -17,10 +17,9 @@
 #ifndef __RARCH_HLSL_H
 #define __RARCH_HLSL_H
 
-#include "../video_shader_driver.h"
 #include <stdint.h>
 
-void hlsl_set_proj_matrix(void *data, XMMATRIX rotation_value);
+#include "../video_driver.h"
 
 #define RARCH_HLSL_MAX_SHADERS 16
 
