@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -25,6 +25,9 @@
 #include <dynamic/dylib.h>
 #include <string/stdstring.h>
 #include <retro_assert.h>
+#include <libretro.h>
+#define VFS_FRONTEND
+#include <vfs/vfs_implementation.h>
 
 #include <features/features_cpu.h>
 
@@ -33,7 +36,7 @@
 #endif
 
 #ifdef HAVE_CHEEVOS
-#include "cheevos.h"
+#include "cheevos/cheevos.h"
 #endif
 
 #ifdef HAVE_NETWORKING
@@ -50,7 +53,8 @@
 #include "core.h"
 #include "driver.h"
 #include "performance_counters.h"
-#include "gfx/video_context_driver.h"
+#include "gfx/video_driver.h"
+#include "led/led_driver.h"
 
 #include "cores/internal_cores.h"
 #include "frontend/frontend_driver.h"
@@ -58,14 +62,18 @@
 #include "dirs.h"
 #include "paths.h"
 #include "retroarch.h"
-#include "runloop.h"
 #include "configuration.h"
 #include "msg_hash.h"
 #include "verbosity.h"
 
+#ifdef HAVE_RUNAHEAD
+#include "runahead/secondary_core.h"
+#include "runahead/run_ahead.h"
+#endif
+
 #ifdef HAVE_DYNAMIC
 #define SYMBOL(x) do { \
-   function_t func = dylib_proc(lib_handle, #x); \
+   function_t func = dylib_proc(lib_handle_local, #x); \
    memcpy(&current_core->x, &func, sizeof(func)); \
    if (current_core->x == NULL) { RARCH_ERR("Failed to load symbol: \"%s\"\n", #x); retroarch_fail(1, "init_libretro_sym()"); } \
 } while (0)
@@ -93,8 +101,9 @@ static dylib_t lib_handle;
 #define SYMBOL_VIDEOPROCESSOR(x) current_core->x = libretro_videoprocessor_##x
 #endif
 
-static bool ignore_environment_cb = false;
-static bool *load_no_content_hook = NULL;
+static bool ignore_environment_cb   = false;
+static bool core_set_shared_context = false;
+static bool *load_no_content_hook   = NULL;
 
 const struct retro_subsystem_info *libretro_find_subsystem_info(
       const struct retro_subsystem_info *info, unsigned num_info,
@@ -191,7 +200,7 @@ static bool environ_cb_get_system_info(unsigned cmd, void *data)
  *
  * For statically linked cores, pass retro_set_environment as argument.
  */
-void libretro_get_environment_info(void (*func)(retro_environment_t),
+static void libretro_get_environment_info(void (*func)(retro_environment_t),
       bool *load_no_content)
 {
    load_no_content_hook = load_no_content;
@@ -199,11 +208,11 @@ void libretro_get_environment_info(void (*func)(retro_environment_t),
    /* load_no_content gets set in this callback. */
    func(environ_cb_get_system_info);
 
-   /* It's possible that we just set get_system_info callback 
+   /* It's possible that we just set get_system_info callback
     * to the currently running core.
     *
     * Make sure we reset it to the actual environment callback.
-    * Ignore any environment callbacks here in case we're running 
+    * Ignore any environment callbacks here in case we're running
     * on the non-current core. */
    ignore_environment_cb = true;
    func(rarch_environment_cb);
@@ -252,7 +261,6 @@ static bool load_dynamic_core(void)
    RARCH_ERR("Error(s): %s\n", dylib_error());
 
    runloop_msg_queue_push(msg_hash_to_str(MSG_FAILED_TO_OPEN_LIBRETRO_CORE), 1, 180, true);
-   retroarch_fail(1, "load_dynamic_core()");
 
    return false;
 }
@@ -298,42 +306,6 @@ static dylib_t libretro_get_system_info_lib(const char *path,
 
    return lib;
 }
-#else
-static bool libretro_get_system_info_static(struct retro_system_info *info,
-      bool *load_no_content)
-{
-   struct retro_system_info dummy_info = {0};
-
-   if (load_no_content)
-   {
-      load_no_content_hook = load_no_content;
-
-      /* load_no_content gets set in this callback. */
-      retro_set_environment(environ_cb_get_system_info);
-
-      /* It's possible that we just set get_system_info callback 
-       * to the currently running core.
-       *
-       * Make sure we reset it to the actual environment callback.
-       * Ignore any environment callbacks here in case we're running 
-       * on the non-current core. */
-      ignore_environment_cb = true;
-      retro_set_environment(rarch_environment_cb);
-      ignore_environment_cb = false;
-   }
-
-   retro_get_system_info(&dummy_info);
-   memcpy(info, &dummy_info, sizeof(*info));
-
-   if (!string_is_empty(dummy_info.library_name))
-      info->library_name    = strdup(dummy_info.library_name);
-   if (!string_is_empty(dummy_info.library_version))
-      info->library_version    = strdup(dummy_info.library_version);
-
-   if (dummy_info.valid_extensions)
-      info->valid_extensions = strdup(dummy_info.valid_extensions);
-   return true;
-}
 #endif
 
 /**
@@ -351,32 +323,62 @@ static bool libretro_get_system_info_static(struct retro_system_info *info,
 bool libretro_get_system_info(const char *path,
       struct retro_system_info *info, bool *load_no_content)
 {
+   struct retro_system_info dummy_info;
 #ifdef HAVE_DYNAMIC
-   struct retro_system_info dummy_info = {0};
-   dylib_t lib                         = libretro_get_system_info_lib(
+   dylib_t lib;
+#endif
+
+   dummy_info.library_name     = NULL;
+   dummy_info.library_version  = NULL;
+   dummy_info.valid_extensions = NULL;
+   dummy_info.need_fullpath    = false;
+   dummy_info.block_extract    = false;
+
+#ifdef HAVE_DYNAMIC
+   lib                         = libretro_get_system_info_lib(
          path, &dummy_info, load_no_content);
 
    if (!lib)
       return false;
+#else
+   if (load_no_content)
+   {
+      load_no_content_hook = load_no_content;
+
+      /* load_no_content gets set in this callback. */
+      retro_set_environment(environ_cb_get_system_info);
+
+      /* It's possible that we just set get_system_info callback
+       * to the currently running core.
+       *
+       * Make sure we reset it to the actual environment callback.
+       * Ignore any environment callbacks here in case we're running
+       * on the non-current core. */
+      ignore_environment_cb = true;
+      retro_set_environment(rarch_environment_cb);
+      ignore_environment_cb = false;
+   }
+
+   retro_get_system_info(&dummy_info);
+#endif
 
    memcpy(info, &dummy_info, sizeof(*info));
+
    if (!string_is_empty(dummy_info.library_name))
-      info->library_name        = strdup(dummy_info.library_name);
+      info->library_name    = strdup(dummy_info.library_name);
    if (!string_is_empty(dummy_info.library_version))
-      info->library_version     = strdup(dummy_info.library_version);
+      info->library_version    = strdup(dummy_info.library_version);
 
    if (dummy_info.valid_extensions)
-      info->valid_extensions    = strdup(dummy_info.valid_extensions);
+      info->valid_extensions = strdup(dummy_info.valid_extensions);
 
+#ifdef HAVE_DYNAMIC
    dylib_close(lib);
-#else
-   if (!libretro_get_system_info_static(info, load_no_content))
-      return false;
 #endif
+
    return true;
 }
 
-
 /**
  * load_symbols:
  * @type                        : Type of core to be loaded.
@@ -386,14 +388,36 @@ bool libretro_get_system_info(const char *path,
  * Setup libretro callback symbols. Returns true on success,
  * or false if symbols could not be loaded.
  **/
-static bool load_symbols(enum rarch_core_type type, struct retro_core_t *current_core)
+bool init_libretro_sym_custom(enum rarch_core_type type, struct retro_core_t *current_core, const char *lib_path, dylib_t *lib_handle_p)
 {
+   /* the library handle for use with the SYMBOL macro */
+   dylib_t lib_handle_local;
    switch (type)
    {
       case CORE_TYPE_PLAIN:
 #ifdef HAVE_DYNAMIC
-         if (!load_dynamic_core())
-            return false;
+#ifdef HAVE_RUNAHEAD
+         if (!lib_path || !lib_handle_p)
+#endif
+         {
+            if (!load_dynamic_core())
+               return false;
+            lib_handle_local = lib_handle;
+         }
+#ifdef HAVE_RUNAHEAD
+         else
+         {
+            /* for a secondary core, we already have a 
+             * primary library loaded, so we can skip 
+             * some checks and just load the library */
+            retro_assert(lib_path != NULL && lib_handle_p != NULL);
+            lib_handle_local = dylib_load(lib_path);
+
+            if (!lib_handle_local)
+               return false;
+            *lib_handle_p = lib_handle_local;
+         }
+#endif
 #endif
 
          SYMBOL(retro_init);
@@ -618,44 +642,9 @@ static bool load_symbols(enum rarch_core_type type, struct retro_core_t *current
    return true;
 }
 
-/**
- * libretro_get_current_core_pathname:
- * @name                         : Sanitized name of libretro core.
- * @size                         : Size of @name
- *
- * Transforms a library id to a name suitable as a pathname.
- **/
-void libretro_get_current_core_pathname(char *name, size_t size)
+static bool load_symbols(enum rarch_core_type type, struct retro_core_t *current_core)
 {
-   size_t i;
-   struct retro_system_info info;
-   const char                *id = msg_hash_to_str(MSG_UNKNOWN);
-
-   if (size == 0)
-      return;
-
-   core_get_system_info(&info);
-
-   if (info.library_name)
-      id = info.library_name;
-
-   if (!id || strlen(id) >= size)
-   {
-      name[0] = '\0';
-      return;
-   }
-
-   name[strlen(id)] = '\0';
-
-   for (i = 0; id[i] != '\0'; i++)
-   {
-      char c = id[i];
-
-      if (isspace((int)c) || isblank((int)c))
-         name[i] = '_';
-      else
-         name[i] = tolower((int)c);
-   }
+   return init_libretro_sym_custom(type, current_core, NULL, NULL);
 }
 
 /**
@@ -677,9 +666,19 @@ bool init_libretro_sym(enum rarch_core_type type, struct retro_core_t *current_c
    if (!load_symbols(type, current_core))
       return false;
 
+#ifdef HAVE_RUNAHEAD
+   /* remember last core type created, so creating a 
+    * secondary core will know what core type to use. */
+   set_last_core_type(type);
+#endif
    return true;
 }
 
+bool libretro_get_shared_context(void)
+{
+   return core_set_shared_context;
+}
+
 /**
  * uninit_libretro_sym:
  *
@@ -699,9 +698,11 @@ void uninit_libretro_sym(struct retro_core_t *current_core)
 
    memset(current_core, 0, sizeof(struct retro_core_t));
 
-   runloop_ctl(RUNLOOP_CTL_CORE_OPTIONS_DEINIT, NULL);
-   runloop_ctl(RUNLOOP_CTL_SYSTEM_INFO_FREE, NULL);
-   runloop_ctl(RUNLOOP_CTL_FRAME_TIME_FREE, NULL);
+   core_set_shared_context = false;
+
+   rarch_ctl(RARCH_CTL_CORE_OPTIONS_DEINIT, NULL);
+   rarch_ctl(RARCH_CTL_SYSTEM_INFO_FREE, NULL);
+   rarch_ctl(RARCH_CTL_FRAME_TIME_FREE, NULL);
    camera_driver_ctl(RARCH_CAMERA_CTL_UNSET_ACTIVE, NULL);
    location_driver_ctl(RARCH_LOCATION_CTL_UNSET_ACTIVE, NULL);
 
@@ -715,7 +716,10 @@ static void rarch_log_libretro(enum retro_log_level level,
    va_list vp;
    settings_t *settings = config_get_ptr();
 
-   if ((unsigned)level < settings->libretro_log_level)
+   if ((unsigned)level < settings->uints.libretro_log_level)
+      return;
+
+   if (!verbosity_is_enabled())
       return;
 
    va_start(vp, fmt);
@@ -752,11 +756,11 @@ static size_t mmap_add_bits_down(size_t n)
    n |= n >>  4;
    n |= n >>  8;
    n |= n >> 16;
-   
+
    /* double shift to avoid warnings on 32bit (it's dead code, but compilers suck) */
    if (sizeof(size_t) > 4)
       n |= n >> 16 >> 16;
-   
+
    return n;
 }
 
@@ -770,7 +774,7 @@ static size_t mmap_inflate(size_t addr, size_t mask)
       addr       = ((addr & ~tmp) << 1) | (addr & tmp);
       mask       = mask & (mask - 1);
    }
-   
+
    return addr;
 }
 
@@ -782,7 +786,7 @@ static size_t mmap_reduce(size_t addr, size_t mask)
       addr       = (addr & tmp) | ((addr >> 1) & ~tmp);
       mask       = (mask & (mask - 1)) >> 1;
    }
-   
+
    return addr;
 }
 
@@ -797,7 +801,7 @@ static bool mmap_preprocess_descriptors(rarch_memory_descriptor_t *first, unsign
    size_t                      top_addr = 1;
    rarch_memory_descriptor_t *desc      = NULL;
    const rarch_memory_descriptor_t *end = first + count;
-   
+
    for (desc = first; desc < end; desc++)
    {
       if (desc->core.select != 0)
@@ -805,43 +809,43 @@ static bool mmap_preprocess_descriptors(rarch_memory_descriptor_t *first, unsign
       else
          top_addr |= desc->core.start + desc->core.len - 1;
    }
-   
+
    top_addr = mmap_add_bits_down(top_addr);
-   
+
    for (desc = first; desc < end; desc++)
    {
       if (desc->core.select == 0)
       {
          if (desc->core.len == 0)
             return false;
-         
+
          if ((desc->core.len & (desc->core.len - 1)) != 0)
             return false;
-         
+
          desc->core.select = top_addr & ~mmap_inflate(mmap_add_bits_down(desc->core.len - 1),
                desc->core.disconnect);
       }
-      
+
       if (desc->core.len == 0)
          desc->core.len = mmap_add_bits_down(mmap_reduce(top_addr & ~desc->core.select,
                   desc->core.disconnect)) + 1;
-      
+
       if (desc->core.start & ~desc->core.select)
          return false;
-       
+
       while (mmap_reduce(top_addr & ~desc->core.select, desc->core.disconnect) >> 1 > desc->core.len - 1)
          desc->core.disconnect |= mmap_highest_bit(top_addr & ~desc->core.select & ~desc->core.disconnect);
-      
+
       desc->disconnect_mask = mmap_add_bits_down(desc->core.len - 1);
       desc->core.disconnect &= desc->disconnect_mask;
-      
+
       while ((~desc->disconnect_mask) >> 1 & desc->core.disconnect)
       {
          desc->disconnect_mask >>= 1;
          desc->core.disconnect &= desc->disconnect_mask;
       }
    }
-   
+
    return true;
 }
 
@@ -918,6 +922,27 @@ static bool dynamic_request_hw_context(enum retro_hw_context_type type,
          break;
 #endif
 
+#if defined(HAVE_D3D9) || defined(HAVE_D3D11)
+      case RETRO_HW_CONTEXT_DIRECT3D:
+         switch (major)
+         {
+#ifdef HAVE_D3D9
+            case 9:
+               RARCH_LOG("Requesting D3D9 context.\n");
+               break;
+#endif
+#ifdef HAVE_D3D11
+            case 11:
+               RARCH_LOG("Requesting D3D11 context.\n");
+               break;
+#endif
+            default:
+               RARCH_LOG("Requesting unknown context.\n");
+               return false;
+         }
+         break;
+#endif
+
       default:
          RARCH_LOG("Requesting unknown context.\n");
          return false;
@@ -945,6 +970,10 @@ static bool dynamic_verify_hw_context(enum retro_hw_context_type type,
          if (!string_is_equal(video_ident, "gl"))
             return false;
          break;
+		case RETRO_HW_CONTEXT_DIRECT3D:
+			if (!(string_is_equal(video_ident, "d3d11") && major == 11))
+				return false;
+		break;
       default:
          break;
    }
@@ -954,7 +983,7 @@ static bool dynamic_verify_hw_context(enum retro_hw_context_type type,
 
 static void core_performance_counter_start(struct retro_perf_counter *perf)
 {
-   if (runloop_ctl(RUNLOOP_CTL_IS_PERFCNT_ENABLE, NULL))
+   if (rarch_ctl(RARCH_CTL_IS_PERFCNT_ENABLE, NULL))
    {
       perf->call_cnt++;
       perf->start      = cpu_features_get_perf_counter();
@@ -963,7 +992,7 @@ static void core_performance_counter_start(struct retro_perf_counter *perf)
 
 static void core_performance_counter_stop(struct retro_perf_counter *perf)
 {
-   if (runloop_ctl(RUNLOOP_CTL_IS_PERFCNT_ENABLE, NULL))
+   if (rarch_ctl(RARCH_CTL_IS_PERFCNT_ENABLE, NULL))
       perf->total += cpu_features_get_perf_counter() - perf->start;
 }
 
@@ -981,9 +1010,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
 {
    unsigned p;
    settings_t         *settings = config_get_ptr();
-   rarch_system_info_t *system  = NULL;
-   
-   runloop_ctl(RUNLOOP_CTL_SYSTEM_INFO_GET, &system);
+   rarch_system_info_t *system  = runloop_get_system_info();
 
    if (ignore_environment_cb)
       return false;
@@ -991,9 +1018,9 @@ bool rarch_environment_cb(unsigned cmd, void *data)
    switch (cmd)
    {
       case RETRO_ENVIRONMENT_GET_OVERSCAN:
-         *(bool*)data = !settings->video.crop_overscan;
+         *(bool*)data = !settings->bools.video_crop_overscan;
          RARCH_LOG("Environ GET_OVERSCAN: %u\n",
-               (unsigned)!settings->video.crop_overscan);
+               (unsigned)!settings->bools.video_crop_overscan);
          break;
 
       case RETRO_ENVIRONMENT_GET_CAN_DUPE:
@@ -1002,7 +1029,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          break;
 
       case RETRO_ENVIRONMENT_GET_VARIABLE:
-         if (!runloop_ctl(RUNLOOP_CTL_CORE_OPTIONS_GET, data))
+         if (!rarch_ctl(RARCH_CTL_CORE_OPTIONS_GET, data))
          {
             struct retro_variable *var = (struct retro_variable*)data;
 
@@ -1016,14 +1043,14 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          break;
 
       case RETRO_ENVIRONMENT_GET_VARIABLE_UPDATE:
-         *(bool*)data = runloop_ctl(RUNLOOP_CTL_IS_CORE_OPTION_UPDATED, NULL);
+         *(bool*)data = rarch_ctl(RARCH_CTL_IS_CORE_OPTION_UPDATED, NULL);
          break;
 
       case RETRO_ENVIRONMENT_SET_VARIABLES:
          RARCH_LOG("Environ SET_VARIABLES.\n");
 
-         runloop_ctl(RUNLOOP_CTL_CORE_OPTIONS_DEINIT, NULL);
-         runloop_ctl(RUNLOOP_CTL_CORE_OPTIONS_INIT,   data);
+         rarch_ctl(RARCH_CTL_CORE_OPTIONS_DEINIT, NULL);
+         rarch_ctl(RARCH_CTL_CORE_OPTIONS_INIT,   data);
 
          break;
 
@@ -1039,7 +1066,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
       {
          unsigned rotation = *(const unsigned*)data;
          RARCH_LOG("Environ SET_ROTATION: %u\n", rotation);
-         if (!settings->video.allow_rotate)
+         if (!settings->bools.video_allow_rotate)
             break;
 
          if (system)
@@ -1052,8 +1079,8 @@ bool rarch_environment_cb(unsigned cmd, void *data)
 
       case RETRO_ENVIRONMENT_SHUTDOWN:
          RARCH_LOG("Environ SHUTDOWN.\n");
-         runloop_ctl(RUNLOOP_CTL_SET_SHUTDOWN,      NULL);
-         runloop_ctl(RUNLOOP_CTL_SET_CORE_SHUTDOWN, NULL);
+         rarch_ctl(RARCH_CTL_SET_SHUTDOWN,      NULL);
+         rarch_ctl(RARCH_CTL_SET_CORE_SHUTDOWN, NULL);
          break;
 
       case RETRO_ENVIRONMENT_SET_PERFORMANCE_LEVEL:
@@ -1066,19 +1093,22 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          break;
 
       case RETRO_ENVIRONMENT_GET_SYSTEM_DIRECTORY:
-         if (string_is_empty(settings->directory.system))
+         if (string_is_empty(settings->paths.directory_system) || settings->bools.systemfiles_in_content_dir)
          {
             const char *fullpath = path_get(RARCH_PATH_CONTENT);
             if (!string_is_empty(fullpath))
             {
-               char temp_path[PATH_MAX_LENGTH];
+               size_t path_size = PATH_MAX_LENGTH * sizeof(char);
+               char *temp_path  = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
 
                temp_path[0] = '\0';
 
-               RARCH_WARN("SYSTEM DIR is empty, assume CONTENT DIR %s\n",
-                     fullpath);
-               fill_pathname_basedir(temp_path, fullpath, sizeof(temp_path));
+               if (string_is_empty(settings->paths.directory_system))
+                  RARCH_WARN("SYSTEM DIR is empty, assume CONTENT DIR %s\n",
+                        fullpath);
+               fill_pathname_basedir(temp_path, fullpath, path_size);
                dir_set(RARCH_DIR_SYSTEM, temp_path);
+               free(temp_path);
             }
 
             *(const char**)data = dir_get_ptr(RARCH_DIR_SYSTEM);
@@ -1087,9 +1117,9 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          }
          else
          {
-            *(const char**)data = settings->directory.system;
+            *(const char**)data = settings->paths.directory_system;
             RARCH_LOG("Environ SYSTEM_DIRECTORY: \"%s\".\n",
-               settings->directory.system);
+               settings->paths.directory_system);
          }
 
          break;
@@ -1099,17 +1129,19 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          break;
 
       case RETRO_ENVIRONMENT_GET_USERNAME:
-         *(const char**)data = *settings->username ?
-            settings->username : NULL;
+         *(const char**)data = *settings->paths.username ?
+            settings->paths.username : NULL;
          RARCH_LOG("Environ GET_USERNAME: \"%s\".\n",
-               settings->username);
+               settings->paths.username);
          break;
 
       case RETRO_ENVIRONMENT_GET_LANGUAGE:
 #ifdef HAVE_LANGEXTRA
-         *(unsigned *)data = settings->user_language;
-         RARCH_LOG("Environ GET_LANGUAGE: \"%u\".\n",
-               settings->user_language);
+         {
+            unsigned user_lang = *msg_hash_get_uint(MSG_HASH_USER_LANGUAGE);
+            *(unsigned *)data  = user_lang;
+            RARCH_LOG("Environ GET_LANGUAGE: \"%u\".\n", user_lang);
+         }
 #endif
          break;
 
@@ -1219,17 +1251,22 @@ bool rarch_environment_cb(unsigned cmd, void *data)
             }
 
             RARCH_LOG("Environ SET_INPUT_DESCRIPTORS:\n");
-            for (p = 0; p < settings->input.max_users; p++)
+
             {
-               for (retro_id = 0; retro_id < RARCH_FIRST_CUSTOM_BIND; retro_id++)
+               unsigned max_users = *(input_driver_get_uint(INPUT_ACTION_MAX_USERS));
+
+               for (p = 0; p < max_users; p++)
                {
-                  const char *description = system->input_desc_btn[p][retro_id];
+                  for (retro_id = 0; retro_id < RARCH_FIRST_CUSTOM_BIND; retro_id++)
+                  {
+                     const char *description = system->input_desc_btn[p][retro_id];
 
-                  if (!description)
-                     continue;
+                     if (!description)
+                        continue;
 
-                  RARCH_LOG("\tRetroPad, User %u, Button \"%s\" => \"%s\"\n",
-                        p + 1, libretro_btn_desc[retro_id], description);
+                     RARCH_LOG("\tRetroPad, User %u, Button \"%s\" => \"%s\"\n",
+                           p + 1, libretro_btn_desc[retro_id], description);
+                  }
                }
             }
 
@@ -1246,8 +1283,8 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          const struct retro_keyboard_callback *info =
             (const struct retro_keyboard_callback*)data;
 
-         runloop_ctl(RUNLOOP_CTL_FRONTEND_KEY_EVENT_GET, &frontend_key_event);
-         runloop_ctl(RUNLOOP_CTL_KEY_EVENT_GET, &key_event);
+         rarch_ctl(RARCH_CTL_FRONTEND_KEY_EVENT_GET, &frontend_key_event);
+         rarch_ctl(RARCH_CTL_KEY_EVENT_GET, &key_event);
 
          RARCH_LOG("Environ SET_KEYBOARD_CALLBACK.\n");
          if (key_event)
@@ -1270,7 +1307,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
       {
          struct retro_hw_render_callback *cb =
             (struct retro_hw_render_callback*)data;
-         struct retro_hw_render_callback *hwr = 
+         struct retro_hw_render_callback *hwr =
             video_driver_get_hw_context();
 
          RARCH_LOG("Environ SET_HW_RENDER.\n");
@@ -1285,7 +1322,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          cb->get_proc_address        = video_driver_get_proc_address;
 
          /* Old ABI. Don't copy garbage. */
-         if (cmd & RETRO_ENVIRONMENT_EXPERIMENTAL) 
+         if (cmd & RETRO_ENVIRONMENT_EXPERIMENTAL)
             memcpy(hwr,
                   cb, offsetof(struct retro_hw_render_callback, stencil));
          else
@@ -1334,7 +1371,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
       case RETRO_ENVIRONMENT_SET_FRAME_TIME_CALLBACK:
       {
          RARCH_LOG("Environ SET_FRAME_TIME_CALLBACK.\n");
-         runloop_ctl(RUNLOOP_CTL_SET_FRAME_TIME, data);
+         rarch_ctl(RARCH_CTL_SET_FRAME_TIME, data);
          break;
       }
 
@@ -1424,7 +1461,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          cb->get_cpu_features = cpu_features_get;
          cb->get_perf_counter = cpu_features_get_perf_counter;
 
-         cb->perf_register    = performance_counter_register; 
+         cb->perf_register    = performance_counter_register;
          cb->perf_start       = core_performance_counter_start;
          cb->perf_stop        = core_performance_counter_stop;
          cb->perf_log         = retro_perf_log;
@@ -1435,10 +1472,10 @@ bool rarch_environment_cb(unsigned cmd, void *data)
       {
          const char **dir = (const char**)data;
 
-         *dir = *settings->directory.core_assets ?
-            settings->directory.core_assets : NULL;
+         *dir = *settings->paths.directory_core_assets ?
+            settings->paths.directory_core_assets : NULL;
          RARCH_LOG("Environ CORE_ASSETS_DIRECTORY: \"%s\".\n",
-               settings->directory.core_assets);
+               settings->paths.directory_core_assets);
          break;
       }
 
@@ -1526,7 +1563,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          }
          break;
       }
-      
+
       case RETRO_ENVIRONMENT_SET_MEMORY_MAPS:
       {
          if (system)
@@ -1607,9 +1644,9 @@ bool rarch_environment_cb(unsigned cmd, void *data)
       {
          const struct retro_game_geometry *in_geom = NULL;
          struct retro_game_geometry *geom = NULL;
-         struct retro_system_av_info *av_info = 
+         struct retro_system_av_info *av_info =
             video_viewport_get_system_av_info();
-         
+
          if (av_info)
             geom = (struct retro_game_geometry*)&av_info->geometry;
 
@@ -1622,9 +1659,9 @@ bool rarch_environment_cb(unsigned cmd, void *data)
 
          /* Can potentially be called every frame,
           * don't do anything unless required. */
-         if (geom->base_width != in_geom->base_width ||
-               geom->base_height != in_geom->base_height ||
-               geom->aspect_ratio != in_geom->aspect_ratio)
+         if (  (geom->base_width   != in_geom->base_width)  ||
+               (geom->base_height  != in_geom->base_height) ||
+               (geom->aspect_ratio != in_geom->aspect_ratio))
          {
             geom->base_width   = in_geom->base_width;
             geom->base_height  = in_geom->base_height;
@@ -1649,7 +1686,7 @@ bool rarch_environment_cb(unsigned cmd, void *data)
       case RETRO_ENVIRONMENT_GET_HW_RENDER_INTERFACE:
          return video_driver_get_hw_render_interface(
                (const struct retro_hw_render_interface**)data);
-      
+
       case RETRO_ENVIRONMENT_SET_SUPPORT_ACHIEVEMENTS:
 #ifdef HAVE_CHEEVOS
          {
@@ -1676,7 +1713,69 @@ bool rarch_environment_cb(unsigned cmd, void *data)
          break;
       }
 
-      /* Default */
+      case RETRO_ENVIRONMENT_SET_HW_SHARED_CONTEXT:
+      {
+         core_set_shared_context = true;
+         break;
+      }
+ 
+      case RETRO_ENVIRONMENT_GET_VFS_INTERFACE:
+      {
+         const uint32_t supported_vfs_version = 1;
+         static struct retro_vfs_interface vfs_iface =
+         {
+            retro_vfs_file_get_path_impl,
+            retro_vfs_file_open_impl,
+            retro_vfs_file_close_impl,
+            retro_vfs_file_size_impl,
+            retro_vfs_file_tell_impl,
+            retro_vfs_file_seek_impl,
+            retro_vfs_file_read_impl,
+            retro_vfs_file_write_impl,
+            retro_vfs_file_flush_impl,
+            retro_vfs_file_remove_impl
+         };
+
+         struct retro_vfs_interface_info *vfs_iface_info = (struct retro_vfs_interface_info *) data;
+         if (vfs_iface_info->required_interface_version <= supported_vfs_version)
+         {
+            vfs_iface_info->required_interface_version = supported_vfs_version;
+            vfs_iface_info->iface                      = &vfs_iface;
+         }
+
+         break;
+      }
+
+      case RETRO_ENVIRONMENT_GET_LED_INTERFACE:
+      {
+         struct retro_led_interface *ledintf =
+            (struct retro_led_interface *)data;
+         if (ledintf)
+            ledintf->set_led_state = led_driver_set_led;
+      }
+      break;
+
+      case RETRO_ENVIRONMENT_GET_AUDIO_VIDEO_ENABLE:
+      {
+         int result = 0;
+         if (!audio_driver_is_suspended() && audio_driver_is_active())
+            result |= 2;
+         if (video_driver_is_active() && !video_driver_is_stub_frame())
+            result |= 1;
+#ifdef HAVE_RUNAHEAD
+         if (want_fast_savestate())
+            result |= 4;
+         if (get_hard_disable_audio())
+            result |= 8;
+#endif
+         if (data != NULL)
+         {
+            int* result_p = (int*)data;
+            *result_p = result;
+         }
+      }
+      break;
+      
       default:
          RARCH_LOG("Environ UNSUPPORTED (#%u).\n", cmd);
          return false;
