@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- * 
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -20,31 +20,12 @@
 #include <boolean.h>
 #include <retro_common_api.h>
 #include <libretro.h>
+#include <dynamic/dylib.h>
 
 #include "core_type.h"
 
 RETRO_BEGIN_DECLS
 
-/**
- * libretro_get_environment_info:
- * @func                         : Function pointer for get_environment_info.
- * @load_no_content              : If true, core should be able to auto-start
- *                                 without any content loaded.
- *
- * Sets environment callback in order to get statically known 
- * information from it.
- *
- * Fetched via environment callbacks instead of
- * retro_get_system_info(), as this info is part of extensions.
- *
- * Should only be called once right after core load to 
- * avoid overwriting the "real" environ callback.
- *
- * For statically linked cores, pass retro_set_environment as argument.
- */
-void libretro_get_environment_info(void (*)(retro_environment_t),
-      bool *load_no_content);
-
 /**
  * libretro_get_system_info:
  * @path                         : Path to libretro library.
@@ -68,15 +49,6 @@ bool libretro_get_system_info(const char *path,
  **/
 void libretro_free_system_info(struct retro_system_info *info);
 
-/**
- * libretro_get_current_core_pathname:
- * @name                         : Sanitized name of libretro core.
- * @size                         : Size of @name
- *
- * Transforms a library id to a name suitable as a pathname.
- **/
-void libretro_get_current_core_pathname(char *name, size_t size);
-
 const struct retro_subsystem_info *libretro_find_subsystem_info(
       const struct retro_subsystem_info *info,
       unsigned num_info, const char *ident);
@@ -122,7 +94,7 @@ struct retro_core_t
    void (*retro_set_input_poll)(retro_input_poll_t);
    void (*retro_set_input_state)(retro_input_state_t);
    void (*retro_set_controller_port_device)(unsigned, unsigned);
-   void (*retro_reset)(void); 
+   void (*retro_reset)(void);
    void (*retro_run)(void);
    size_t (*retro_serialize_size)(void);
    bool (*retro_serialize)(void*, size_t);
@@ -136,12 +108,22 @@ struct retro_core_t
    unsigned (*retro_get_region)(void);
    void *(*retro_get_memory_data)(unsigned);
    size_t (*retro_get_memory_size)(unsigned);
+
+   unsigned poll_type;
+   bool inited;
+   bool symbols_inited;
+   bool game_loaded;
+   bool input_polled;
+   bool has_set_input_descriptors;
+   uint64_t serialization_quirks_v;
 };
 
+bool libretro_get_shared_context(void);
+
 /**
  * init_libretro_sym:
  * @type                        : Type of core to be loaded.
- *                                If CORE_TYPE_DUMMY, will 
+ *                                If CORE_TYPE_DUMMY, will
  *                                load dummy symbols.
  *
  * Initializes libretro symbols and
@@ -151,6 +133,8 @@ struct retro_core_t
 bool init_libretro_sym(enum rarch_core_type type,
       struct retro_core_t *core);
 
+bool init_libretro_sym_custom(enum rarch_core_type type, struct retro_core_t *current_core, const char *lib_path, dylib_t *lib_handle_p);
+
 /**
  * uninit_libretro_sym:
  *
