@@ -1,4 +1,4 @@
-/* Copyright  (C) 2010-2016 The RetroArch team
+/* Copyright  (C) 2010-2018 The RetroArch team
  *
  * ---------------------------------------------------------------------------------------
  * The following license statement only applies to this file (features_cpu.c).
@@ -33,10 +33,10 @@
 #include <streams/file_stream.h>
 #include <libretro.h>
 #include <features/features_cpu.h>
+#include <retro_timers.h>
 
 #if defined(_WIN32) && !defined(_XBOX)
 #include <windows.h>
-#include <intrin.h>
 #endif
 
 #if defined(__CELLOS_LV2__)
@@ -45,7 +45,7 @@
 #endif
 #elif defined(_XBOX360)
 #include <PPCIntrinsics.h>
-#elif defined(_POSIX_MONOTONIC_CLOCK) || defined(ANDROID) || defined(__QNX__)
+#elif defined(_POSIX_MONOTONIC_CLOCK) || defined(ANDROID) || defined(__QNX__) || defined(DJGPP)
 /* POSIX_MONOTONIC_CLOCK is not being defined in Android headers despite support being present. */
 #include <time.h>
 #endif
@@ -55,6 +55,7 @@
 #endif
 
 #if defined(PSP)
+#include <pspkernel.h>
 #include <sys/time.h>
 #include <psprtc.h>
 #endif
@@ -74,6 +75,21 @@
 #include <ogc/lwp_watchdog.h>
 #endif
 
+#ifdef WIIU
+#include <wiiu/os/time.h>
+#endif
+
+#ifdef SWITCH
+#include <libtransistor/types.h>
+#include <libtransistor/svc.h>
+#endif
+
+#if defined(_3DS)
+#include <3ds/svc.h>
+#include <3ds/os.h>
+#include <3ds/services/cfgu.h>
+#endif
+
 /* iOS/OSX specific. Lacks clock_gettime(), so implement it. */
 #ifdef __MACH__
 #include <sys/time.h>
@@ -86,9 +102,8 @@
 #define CLOCK_REALTIME 0
 #endif
 
-#if __IPHONE_OS_VERSION_MAX_ALLOWED < 100000
-// this function is part of iOS 10 now
-static int clock_gettime(int clk_ik, struct timespec *t)
+/* this function is part of iOS 10 now */
+static int ra_clock_gettime(int clk_ik, struct timespec *t)
 {
    struct timeval now;
    int rv = gettimeofday(&now, NULL);
@@ -99,8 +114,13 @@ static int clock_gettime(int clk_ik, struct timespec *t)
    return 0;
 }
 #endif
+
+#if defined(__MACH__) && __IPHONE_OS_VERSION_MIN_REQUIRED < 100000
+#else
+#define ra_clock_gettime clock_gettime
 #endif
 
+
 #ifdef EMSCRIPTEN
 #include <emscripten.h>
 #endif
@@ -123,7 +143,11 @@ retro_perf_tick_t cpu_features_get_perf_counter(void)
    retro_perf_tick_t time_ticks = 0;
 #if defined(_WIN32)
    long tv_sec, tv_usec;
+#if defined(_MSC_VER) && _MSC_VER <= 1200
+   static const unsigned __int64 epoch = 11644473600000000;
+#else
    static const unsigned __int64 epoch = 11644473600000000ULL;
+#endif
    FILETIME file_time;
    SYSTEMTIME system_time;
    ULARGE_INTEGER ularge;
@@ -137,8 +161,8 @@ retro_perf_tick_t cpu_features_get_perf_counter(void)
    tv_usec    = (long)(system_time.wMilliseconds * 1000);
    time_ticks = (1000000 * tv_sec + tv_usec);
 #elif defined(__linux__) || defined(__QNX__) || defined(__MACH__)
-   struct timespec tv;
-   if (clock_gettime(CLOCK_MONOTONIC, &tv) == 0)
+   struct timespec tv = {0};
+   if (ra_clock_gettime(CLOCK_MONOTONIC, &tv) == 0)
       time_ticks = (retro_perf_tick_t)tv.tv_sec * 1000000000 +
          (retro_perf_tick_t)tv.tv_nsec;
 
@@ -154,10 +178,14 @@ retro_perf_tick_t cpu_features_get_perf_counter(void)
    time_ticks = __mftb();
 #elif defined(GEKKO)
    time_ticks = gettime();
-#elif defined(PSP) || defined(VITA)
-   sceRtcGetCurrentTick(&time_ticks);
+#elif defined(PSP)
+   sceRtcGetCurrentTick((uint64_t*)&time_ticks);
+#elif defined(VITA)
+   sceRtcGetCurrentTick((SceRtcTick*)&time_ticks);
 #elif defined(_3DS)
    time_ticks = svcGetSystemTick();
+#elif defined(WIIU)
+   time_ticks = OSGetSystemTime();
 #elif defined(__mips__)
    struct timeval tv;
    gettimeofday(&tv,NULL);
@@ -191,14 +219,16 @@ retro_time_t cpu_features_get_time_usec(void)
    return sys_time_get_system_time();
 #elif defined(GEKKO)
    return ticks_to_microsecs(gettime());
+#elif defined(SWITCH)
+   return (svcGetSystemTick() * 10) / 192;
 #elif defined(_POSIX_MONOTONIC_CLOCK) || defined(__QNX__) || defined(ANDROID) || defined(__MACH__)
    struct timespec tv = {0};
-   if (clock_gettime(CLOCK_MONOTONIC, &tv) < 0)
+   if (ra_clock_gettime(CLOCK_MONOTONIC, &tv) < 0)
       return 0;
    return tv.tv_sec * INT64_C(1000000) + (tv.tv_nsec + 500) / 1000;
 #elif defined(EMSCRIPTEN)
    return emscripten_get_now() * 1000;
-#elif defined(__mips__)
+#elif defined(__mips__) || defined(DJGPP)
    struct timeval tv;
    gettimeofday(&tv,NULL);
    return (1000000 * tv.tv_sec + tv.tv_usec);
@@ -206,18 +236,22 @@ retro_time_t cpu_features_get_time_usec(void)
    return osGetTime() * 1000;
 #elif defined(VITA)
    return sceKernelGetProcessTimeWide();
+#elif defined(WIIU)
+   return ticks_to_us(OSGetSystemTime());
 #else
 #error "Your platform does not have a timer function implemented in cpu_features_get_time_usec(). Cannot continue."
 #endif
 }
 
-#if defined(__x86_64__) || defined(__i386__) || defined(__i486__) || defined(__i686__)
+#if defined(__x86_64__) || defined(__i386__) || defined(__i486__) || defined(__i686__) || (defined(_M_X64) && _MSC_VER > 1310) || (defined(_M_IX86)  && _MSC_VER > 1310)
 #define CPU_X86
 #endif
 
 #if defined(_MSC_VER) && !defined(_XBOX)
+#if (_MSC_VER > 1310)
 #include <intrin.h>
 #endif
+#endif
 
 #if defined(CPU_X86) && !defined(__MACH__)
 void x86_cpuid(int func, int flags[4])
@@ -270,7 +304,7 @@ static uint64_t xgetbv_x86(uint32_t idx)
 }
 #endif
 
-#if defined(__ARM_NEON__)  
+#if defined(__ARM_NEON__)
 static void arm_enable_runfast_mode(void)
 {
    /* RunFast mode. Enables flush-to-zero and some
@@ -294,7 +328,9 @@ static unsigned char check_arm_cpu_feature(const char* feature)
 {
    char line[1024];
    unsigned char status = 0;
-   RFILE *fp = filestream_open("/proc/cpuinfo", RFILE_MODE_READ_TEXT, -1);
+   RFILE *fp = filestream_open("/proc/cpuinfo",
+         RETRO_VFS_FILE_ACCESS_READ,
+         RETRO_VFS_FILE_ACCESS_HINT_NONE);
 
    if (!fp)
       return 0;
@@ -446,13 +482,35 @@ unsigned cpu_features_get_core_amount(void)
 #elif defined(VITA)
    return 4;
 #elif defined(_3DS)
+   u8 device_model = 0xFF;
+   CFGU_GetSystemModel(&device_model);/*(0 = O3DS, 1 = O3DSXL, 2 = N3DS, 3 = 2DS, 4 = N3DSXL, 5 = N2DSXL)*/
+   switch (device_model)
+   {
+		case 0:
+		case 1:
+		case 3:
+			/*Old 3/2DS*/
+			return 2;
+	   
+		case 2:
+		case 4:
+		case 5:
+			/*New 3/2DS*/
+			return 4;
+	   
+		default:
+			/*Unknown Device Or Check Failed*/
+			break;
+   }
    return 1;
+#elif defined(WIIU)
+   return 3;
 #elif defined(_SC_NPROCESSORS_ONLN)
    /* Linux, most UNIX-likes. */
    long ret = sysconf(_SC_NPROCESSORS_ONLN);
    if (ret <= 0)
       return (unsigned)1;
-   return ret;
+   return (unsigned)ret;
 #elif defined(BSD) || defined(__APPLE__)
    /* BSD */
    /* Copypasta from stackoverflow, dunno if it works. */
@@ -512,7 +570,7 @@ uint64_t cpu_features_get(void)
 {
    int flags[4];
    int vendor_shuffle[3];
-   char vendor[13]     = {0};
+   char vendor[13];
    size_t len          = 0;
    uint64_t cpu_flags  = 0;
    uint64_t cpu        = 0;
@@ -541,6 +599,12 @@ uint64_t cpu_features_get(void)
       cpu |= RETRO_SIMD_MMXEXT;
    }
 
+   len            = sizeof(size_t);
+   if (sysctlbyname("hw.optional.floatingpoint", NULL, &len, NULL, 0) == 0)
+   {
+      cpu |= RETRO_SIMD_CMOV;
+   }
+
    len            = sizeof(size_t);
    if (sysctlbyname("hw.optional.sse", NULL, &len, NULL, 0) == 0)
       cpu |= RETRO_SIMD_SSE;
@@ -585,6 +649,10 @@ uint64_t cpu_features_get(void)
    if (sysctlbyname("hw.optional.neon", NULL, &len, NULL, 0) == 0)
       cpu |= RETRO_SIMD_NEON;
 
+#elif defined(_XBOX1)
+   cpu |= RETRO_SIMD_MMX;
+   cpu |= RETRO_SIMD_SSE;
+   cpu |= RETRO_SIMD_MMXEXT;
 #elif defined(CPU_X86)
    (void)avx_flags;
 
@@ -592,6 +660,8 @@ uint64_t cpu_features_get(void)
    vendor_shuffle[0] = flags[1];
    vendor_shuffle[1] = flags[3];
    vendor_shuffle[2] = flags[2];
+
+   vendor[0]         = '\0';
    memcpy(vendor, vendor_shuffle, sizeof(vendor_shuffle));
 
    /* printf("[CPUID]: Vendor: %s\n", vendor); */
@@ -607,6 +677,9 @@ uint64_t cpu_features_get(void)
 
    x86_cpuid(1, flags);
 
+   if (flags[3] & (1 << 15))
+      cpu |= RETRO_SIMD_CMOV;
+
    if (flags[3] & (1 << 23))
       cpu |= RETRO_SIMD_MMX;
 
@@ -681,6 +754,15 @@ uint64_t cpu_features_get(void)
    if (check_arm_cpu_feature("vfpv4"))
       cpu |= RETRO_SIMD_VFPV4;
 
+   if (check_arm_cpu_feature("asimd"))
+   {
+      cpu |= RETRO_SIMD_ASIMD;
+#ifdef __ARM_NEON__
+      cpu |= RETRO_SIMD_NEON;
+      arm_enable_runfast_mode();
+#endif
+   }
+
 #if 0
     check_arm_cpu_feature("swp");
     check_arm_cpu_feature("half");
@@ -725,6 +807,7 @@ uint64_t cpu_features_get(void)
    if (cpu & RETRO_SIMD_VMX128) strlcat(buf, " VMX128", sizeof(buf));
    if (cpu & RETRO_SIMD_VFPU)   strlcat(buf, " VFPU", sizeof(buf));
    if (cpu & RETRO_SIMD_PS)     strlcat(buf, " PS", sizeof(buf));
+   if (cpu & RETRO_SIMD_ASIMD)  strlcat(buf, " ASIMD", sizeof(buf));
 
    return cpu;
 }
