@@ -1,4 +1,4 @@
-/* Copyright  (C) 2010-2017 The RetroArch team
+/* Copyright  (C) 2010-2018 The RetroArch team
  *
  * ---------------------------------------------------------------------------------------
  * The following license statement only applies to this file (config_file.c).
@@ -38,13 +38,13 @@
 #include <retro_miscellaneous.h>
 #include <compat/strl.h>
 #include <compat/posix_string.h>
+#include <compat/fopen_utf8.h>
 #include <compat/msvc.h>
 #include <file/config_file.h>
 #include <file/file_path.h>
-#include <retro_stat.h>
 #include <lists/string_list.h>
 #include <string/stdstring.h>
-#include <rhash.h>
+#include <streams/file_stream.h>
 
 #define MAX_INCLUDE_DEPTH 16
 
@@ -53,10 +53,9 @@ struct config_entry_list
    /* If we got this from an #include,
     * do not allow overwrite. */
    bool readonly;
+
    char *key;
    char *value;
-   uint32_t key_hash;
-
    struct config_entry_list *next;
 };
 
@@ -66,87 +65,42 @@ struct config_include_list
    struct config_include_list *next;
 };
 
-struct config_file
-{
-   char *path;
-   struct config_entry_list *entries;
-   struct config_entry_list *tail;
-   unsigned include_depth;
-
-   struct config_include_list *includes;
-};
-
 static config_file_t *config_file_new_internal(
       const char *path, unsigned depth);
 
-static char *getaline(FILE *file)
-{
-   char* newline     = (char*)malloc(9);
-   char* newline_tmp = NULL;
-   size_t cur_size   = 8;
-   size_t idx        = 0;
-   int in            = getc(file);
-
-   if (!newline)
-      return NULL;
-
-   while (in != EOF && in != '\n')
-   {
-      if (idx == cur_size)
-      {
-         cur_size *= 2;
-         newline_tmp = (char*)realloc(newline, cur_size + 1);
-
-         if (!newline_tmp)
-         {
-            free(newline);
-            return NULL;
-         }
-
-         newline = newline_tmp;
-      }
-
-      newline[idx++] = in;
-      in = getc(file);
-   }
-   newline[idx] = '\0';
-   return newline; 
-}
-
 static char *strip_comment(char *str)
 {
    /* Remove everything after comment.
     * Keep #s inside string literals. */
-   char *string_end = str + strlen(str);
-   bool cut_comment = true;
+   char *string_end  = str + strlen(str);
+   bool cut_comment  = true;
 
    while (!string_is_empty(str))
    {
-      char *comment = NULL;
-      char *literal = strchr(str, '\"');
+      char *comment  = NULL;
+      char *literal  = strchr(str, '\"');
       if (!literal)
-         literal = string_end;
-      comment = (char*)strchr(str, '#');
+         literal     = string_end;
+      comment        = (char*)strchr(str, '#');
+
       if (!comment)
-         comment = string_end;
+         comment     = string_end;
 
       if (cut_comment && literal < comment)
       {
          cut_comment = false;
-         str = literal + 1;
+         str         = literal + 1;
       }
       else if (!cut_comment && literal)
       {
          cut_comment = true;
-         str = literal + 1;
-      }
-      else if (comment)
-      {
-         *comment = '\0';
-         str = comment;
+         str         = literal + 1;
       }
       else
-         str = string_end;
+      {
+         *comment    = '\0';
+         str         = comment;
+      }
    }
 
    return str;
@@ -177,67 +131,46 @@ static char *extract_value(char *line, bool is_value)
    if (*line == '"')
    {
       line++;
-      tok = strtok_r(line, "\"", &save);
-      if (!tok)
+      if (*line == '"')
          return NULL;
-      return strdup(tok);
+      tok = strtok_r(line, "\"", &save);
    }
-   else if (*line == '\0') /* Nothing */
-      return NULL;
-
    /* We don't have that. Read until next space. */
-   tok = strtok_r(line, " \n\t\f\r\v", &save);
-   if (tok)
+   else if (*line != '\0') /* Nothing */
+      tok = strtok_r(line, " \n\t\f\r\v", &save);
+
+   if (tok && *tok)
       return strdup(tok);
    return NULL;
 }
 
-static void add_include_list(config_file_t *conf, const char *path)
-{
-   struct config_include_list *head = conf->includes;
-   struct config_include_list *node = (struct config_include_list*)calloc(1, sizeof(*node));
-
-   if (!node)
-      return;
-
-   node->path = strdup(path);
-
-   if (head)
-   {
-      while (head->next)
-         head = head->next;
-
-      head->next = node;
-   }
-   else
-      conf->includes = node;
-}
-
-static void set_list_readonly(struct config_entry_list *list)
-{
-   while (list)
-   {
-      list->readonly = true;
-      list = list->next;
-   }
-}
-
 /* Move semantics? */
 static void add_child_list(config_file_t *parent, config_file_t *child)
 {
+   struct config_entry_list *list = child->entries;
    if (parent->entries)
    {
       struct config_entry_list *head = parent->entries;
       while (head->next)
          head = head->next;
 
-      set_list_readonly(child->entries);
-      head->next = child->entries;
+      /* set list readonly */
+      while (list)
+      {
+         list->readonly = true;
+         list           = list->next;
+      }
+      head->next        = child->entries;
    }
    else
    {
-      set_list_readonly(child->entries);
-      parent->entries = child->entries;
+      /* set list readonly */
+      while (list)
+      {
+         list->readonly = true;
+         list           = list->next;
+      }
+      parent->entries   = child->entries;
    }
 
    child->entries = NULL;
@@ -245,7 +178,7 @@ static void add_child_list(config_file_t *parent, config_file_t *child)
    /* Rebase tail. */
    if (parent->entries)
    {
-      struct config_entry_list *head = 
+      struct config_entry_list *head =
          (struct config_entry_list*)parent->entries;
 
       while (head->next)
@@ -256,22 +189,37 @@ static void add_child_list(config_file_t *parent, config_file_t *child)
       parent->tail = NULL;
 }
 
-static void add_sub_conf(config_file_t *conf, char *line)
+static void add_sub_conf(config_file_t *conf, char *path)
 {
    char real_path[PATH_MAX_LENGTH];
-   config_file_t         *sub_conf = NULL;
-   char                      *path = extract_value(line, false);
+   config_file_t         *sub_conf  = NULL;
+   struct config_include_list *head = conf->includes;
+   struct config_include_list *node = (struct config_include_list*)
+      malloc(sizeof(*node));
 
-   if (!path)
-      return;
+   if (node)
+   {
+      node->next = NULL;
+      /* Add include list */
+      node->path = strdup(path);
 
-   add_include_list(conf, path);
+      if (head)
+      {
+         while (head->next)
+            head = head->next;
+
+         head->next = node;
+      }
+      else
+         conf->includes = node;
+   }
 
    real_path[0] = '\0';
 
 #ifdef _WIN32
-   fill_pathname_resolve_relative(real_path, conf->path,
-         path, sizeof(real_path));
+   if (!string_is_empty(conf->path))
+      fill_pathname_resolve_relative(real_path, conf->path,
+            path, sizeof(real_path));
 #else
 #ifndef __CELLOS_LV2__
    if (*path == '~')
@@ -282,22 +230,19 @@ static void add_sub_conf(config_file_t *conf, char *line)
    }
    else
 #endif
-      fill_pathname_resolve_relative(real_path, conf->path,
-            path, sizeof(real_path));
+      if (!string_is_empty(conf->path))
+         fill_pathname_resolve_relative(real_path, conf->path,
+               path, sizeof(real_path));
 #endif
 
    sub_conf = (config_file_t*)
       config_file_new_internal(real_path, conf->include_depth + 1);
    if (!sub_conf)
-   {
-      free(path);
       return;
-   }
 
    /* Pilfer internal list. */
    add_child_list(conf, sub_conf);
    config_file_free(sub_conf);
-   free(path);
 }
 
 static bool parse_line(config_file_t *conf,
@@ -312,29 +257,28 @@ static bool parse_line(config_file_t *conf,
    if (!key)
       return false;
 
-   if (!line || !*line)
-   {
-      free(key);
-      return false;
-   }
-
    comment = strip_comment(line);
 
-   /* Starting line with # and include includes config files. */
-   if ((comment == line) && (conf->include_depth < MAX_INCLUDE_DEPTH))
+   /* Starting line with #include includes config files. */
+   if (comment == line)
    {
       comment++;
       if (strstr(comment, "include ") == comment)
       {
-         add_sub_conf(conf, comment + strlen("include "));
-         free(key);
-         return false;
+         char *line = comment + strlen("include ");
+         char *path = extract_value(line, false);
+
+         if (path)
+         {
+            if (conf->include_depth >= MAX_INCLUDE_DEPTH)
+               fprintf(stderr, "!!! #include depth exceeded for config. Might be a cycle.\n");
+            else
+               add_sub_conf(conf, path);
+            free(path);
+         }
+         goto error;
       }
    }
-   else if (conf->include_depth >= MAX_INCLUDE_DEPTH)
-   {
-      fprintf(stderr, "!!! #include depth exceeded for config. Might be a cycle.\n");
-   }
 
    /* Skips to first character. */
    while (isspace((int)*line))
@@ -348,73 +292,84 @@ static bool parse_line(config_file_t *conf,
          key_tmp = (char*)realloc(key, cur_size + 1);
 
          if (!key_tmp)
-         {
-            free(key);
-            return false;
-         }
+            goto error;
 
          key = key_tmp;
       }
 
       key[idx++] = *line++;
    }
-   key[idx] = '\0';
-   list->key = key;
-   list->key_hash = djb2_calculate(key);
+   key[idx]      = '\0';
+   list->key     = key;
+
+   list->value   = extract_value(line, true);
 
-   list->value = extract_value(line, true);
    if (!list->value)
    {
       list->key = NULL;
-      free(key);
-      return false;
+      goto error;
    }
 
    return true;
+
+error:
+   free(key);
+   return false;
 }
 
 static config_file_t *config_file_new_internal(
       const char *path, unsigned depth)
 {
-   FILE *file = NULL;
-   struct config_file *conf = (struct config_file*)calloc(1, sizeof(*conf));
+   RFILE              *file = NULL;
+   struct config_file *conf = (struct config_file*)malloc(sizeof(*conf));
    if (!conf)
       return NULL;
 
+   conf->path          = NULL;
+   conf->entries       = NULL;
+   conf->tail          = NULL;
+   conf->includes      = NULL;
+   conf->include_depth = 0;
+
    if (!path || !*path)
       return conf;
 
    if (path_is_directory(path))
       goto error;
 
-   conf->path = strdup(path);
+   conf->path          = strdup(path);
    if (!conf->path)
       goto error;
 
    conf->include_depth = depth;
-   file = fopen(path, "r");
+   file                = filestream_open(path,
+         RETRO_VFS_FILE_ACCESS_READ,
+         RETRO_VFS_FILE_ACCESS_HINT_NONE);
 
    if (!file)
    {
       free(conf->path);
       goto error;
    }
-   setvbuf(file, NULL, _IOFBF, 0x4000);
 
-   while (!feof(file))
+   while (!filestream_eof(file))
    {
-      struct config_entry_list *list = (struct config_entry_list*)
-         calloc(1, sizeof(*list));
-      char *line = NULL;
+      char *line                     = NULL;
+      struct config_entry_list *list = (struct config_entry_list*)malloc(sizeof(*list));
 
       if (!list)
       {
          config_file_free(conf);
-         fclose(file);
+         filestream_close(file);
          return NULL;
       }
 
-      line = getaline(file);
+      list->readonly  = false;
+      list->key       = NULL;
+      list->value     = NULL;
+      list->next      = NULL;
+
+      line            = filestream_getline(file);
 
       if (!line)
       {
@@ -422,7 +377,7 @@ static config_file_t *config_file_new_internal(
          continue;
       }
 
-      if (parse_line(conf, list, line))
+      if (*line && parse_line(conf, list, line))
       {
          if (conf->entries)
             conf->tail->next = list;
@@ -438,7 +393,7 @@ static config_file_t *config_file_new_internal(
          free(list);
    }
 
-   fclose(file);
+   filestream_close(file);
 
    return conf;
 
@@ -479,7 +434,7 @@ void config_file_free(config_file_t *conf)
    {
       struct config_include_list *hold = NULL;
       free(inc_tmp->path);
-      hold = (struct config_include_list*)inc_tmp;
+      hold    = (struct config_include_list*)inc_tmp;
       inc_tmp = inc_tmp->next;
       free(hold);
    }
@@ -511,25 +466,27 @@ config_file_t *config_file_new_from_string(const char *from_string)
 {
    size_t i;
    struct string_list *lines = NULL;
-   struct config_file *conf = (struct config_file*)calloc(1, sizeof(*conf));
+   struct config_file *conf = (struct config_file*)malloc(sizeof(*conf));
    if (!conf)
       return NULL;
 
    if (!from_string)
       return conf;
 
-   conf->path = NULL;
+   conf->path          = NULL;
+   conf->entries       = NULL;
+   conf->tail          = NULL;
+   conf->includes      = NULL;
    conf->include_depth = 0;
-   
+
    lines = string_split(from_string, "\n");
    if (!lines)
       return conf;
 
    for (i = 0; i < lines->size; i++)
    {
-      struct config_entry_list *list = (struct config_entry_list*)
-         calloc(1, sizeof(*list));
-      char* line = lines->elems[i].data;
+      struct config_entry_list *list = (struct config_entry_list*)malloc(sizeof(*list));
+      char                    *line  = lines->elems[i].data;
 
       if (!list)
       {
@@ -538,9 +495,14 @@ config_file_t *config_file_new_from_string(const char *from_string)
          return NULL;
       }
 
+      list->readonly  = false;
+      list->key       = NULL;
+      list->value     = NULL;
+      list->next      = NULL;
+
       if (line && conf)
       {
-         if (parse_line(conf, list, line))
+         if (*line && parse_line(conf, list, line))
          {
             if (conf->entries)
                conf->tail->next = list;
@@ -565,21 +527,15 @@ config_file_t *config_file_new(const char *path)
    return config_file_new_internal(path, 0);
 }
 
-
 static struct config_entry_list *config_get_entry(const config_file_t *conf,
       const char *key, struct config_entry_list **prev)
 {
-   struct config_entry_list *entry;
-   struct config_entry_list *previous = NULL;
-
-   uint32_t hash = djb2_calculate(key);
-
-   if (prev)
-      previous = *prev;
+   struct config_entry_list *entry    = NULL;
+   struct config_entry_list *previous = prev ? *prev : NULL;
 
    for (entry = conf->entries; entry; entry = entry->next)
    {
-      if (hash == entry->key_hash && !strcmp(key, entry->key))
+      if (string_is_equal(key, entry->key))
          return entry;
 
       previous = entry;
@@ -596,9 +552,12 @@ bool config_get_double(config_file_t *conf, const char *key, double *in)
    const struct config_entry_list *entry = config_get_entry(conf, key, NULL);
 
    if (entry)
+   {
       *in = strtod(entry->value, NULL);
+      return true;
+   }
 
-   return entry != NULL;
+   return false;
 }
 
 bool config_get_float(config_file_t *conf, const char *key, float *in)
@@ -609,9 +568,9 @@ bool config_get_float(config_file_t *conf, const char *key, float *in)
    {
       /* strtof() is C99/POSIX. Just use the more portable kind. */
       *in = (float)strtod(entry->value, NULL);
+      return true;
    }
-
-   return entry != NULL;
+   return false;
 }
 
 bool config_get_int(config_file_t *conf, const char *key, int *in)
@@ -621,13 +580,16 @@ bool config_get_int(config_file_t *conf, const char *key, int *in)
 
    if (entry)
    {
-      int val = strtol(entry->value, NULL, 0);
+      int val = (int)strtol(entry->value, NULL, 0);
 
       if (errno == 0)
+      {
          *in = val;
+         return true;
+      }
    }
 
-   return entry != NULL && errno == 0;
+   return false;
 }
 
 #if defined(__STDC_VERSION__) && __STDC_VERSION__>=199901L
@@ -641,10 +603,12 @@ bool config_get_uint64(config_file_t *conf, const char *key, uint64_t *in)
       uint64_t val = strtoull(entry->value, NULL, 0);
 
       if (errno == 0)
+      {
          *in = val;
+         return true;
+      }
    }
-
-   return entry != NULL && errno == 0;
+   return false;
 }
 #endif
 
@@ -655,13 +619,16 @@ bool config_get_uint(config_file_t *conf, const char *key, unsigned *in)
 
    if (entry)
    {
-      unsigned val = strtoul(entry->value, NULL, 0);
+      unsigned val = (unsigned)strtoul(entry->value, NULL, 0);
 
       if (errno == 0)
+      {
          *in = val;
+         return true;
+      }
    }
 
-   return entry != NULL && errno == 0;
+   return false;
 }
 
 bool config_get_hex(config_file_t *conf, const char *key, unsigned *in)
@@ -671,13 +638,16 @@ bool config_get_hex(config_file_t *conf, const char *key, unsigned *in)
 
    if (entry)
    {
-      unsigned val = strtoul(entry->value, NULL, 16);
+      unsigned val = (unsigned)strtoul(entry->value, NULL, 16);
 
       if (errno == 0)
+      {
          *in = val;
+         return true;
+      }
    }
 
-   return entry != NULL && errno == 0;
+   return false;
 }
 
 bool config_get_char(config_file_t *conf, const char *key, char *in)
@@ -690,9 +660,10 @@ bool config_get_char(config_file_t *conf, const char *key, char *in)
          return false;
 
       *in = *entry->value;
+      return true;
    }
 
-   return entry != NULL;
+   return false;
 }
 
 bool config_get_string(config_file_t *conf, const char *key, char **str)
@@ -700,9 +671,11 @@ bool config_get_string(config_file_t *conf, const char *key, char **str)
    const struct config_entry_list *entry = config_get_entry(conf, key, NULL);
 
    if (entry)
+   {
       *str = strdup(entry->value);
-
-   return entry != NULL;
+      return true;
+   }
+   return false;
 }
 
 bool config_get_config_path(config_file_t *conf, char *s, size_t len)
@@ -720,23 +693,25 @@ bool config_get_array(config_file_t *conf, const char *key,
 
    if (entry)
       return strlcpy(buf, entry->value, size) < size;
-
-   return entry != NULL;
+   return false;
 }
 
 bool config_get_path(config_file_t *conf, const char *key,
       char *buf, size_t size)
 {
-#if defined(RARCH_CONSOLE)
-   return config_get_array(conf, key, buf, size);
+#if defined(RARCH_CONSOLE) || !defined(RARCH_INTERNAL)
+   if (config_get_array(conf, key, buf, size))
+      return true;
 #else
    const struct config_entry_list *entry = config_get_entry(conf, key, NULL);
 
    if (entry)
+   {
       fill_pathname_expand_special(buf, entry->value, size);
-
-   return entry != NULL;
+      return true;
+   }
 #endif
+   return false;
 }
 
 bool config_get_bool(config_file_t *conf, const char *key, bool *in)
@@ -745,13 +720,13 @@ bool config_get_bool(config_file_t *conf, const char *key, bool *in)
 
    if (entry)
    {
-      if (strcasecmp(entry->value, "true") == 0)
+      if (string_is_equal(entry->value, "true"))
          *in = true;
-      else if (strcasecmp(entry->value, "1") == 0)
+      else if (string_is_equal(entry->value, "1"))
          *in = true;
-      else if (strcasecmp(entry->value, "false") == 0)
+      else if (string_is_equal(entry->value, "false"))
          *in = false;
-      else if (strcasecmp(entry->value, "0") == 0)
+      else if (string_is_equal(entry->value, "0"))
          *in = false;
       else
          return false;
@@ -772,16 +747,20 @@ void config_set_string(config_file_t *conf, const char *key, const char *val)
       return;
    }
 
-   if (!val) return;
+   if (!val)
+      return;
 
-   entry = (struct config_entry_list*)calloc(1, sizeof(*entry));
-   if (!entry) return;
+   entry = (struct config_entry_list*)malloc(sizeof(*entry));
+   if (!entry)
+      return;
 
-   entry->key   = strdup(key);
-   entry->value = strdup(val);
+   entry->readonly  = false;
+   entry->key       = strdup(key);
+   entry->value     = strdup(val);
+   entry->next      = NULL;
 
    if (last)
-      last->next = entry;
+      last->next    = entry;
    else
       conf->entries = entry;
 }
@@ -802,7 +781,7 @@ void config_unset(config_file_t *conf, const char *key)
 
 void config_set_path(config_file_t *conf, const char *entry, const char *val)
 {
-#if defined(RARCH_CONSOLE)
+#if defined(RARCH_CONSOLE) || !defined(RARCH_INTERNAL)
    config_set_string(conf, entry, val);
 #else
    char buf[PATH_MAX_LENGTH];
@@ -860,11 +839,7 @@ void config_set_uint64(config_file_t *conf, const char *key, uint64_t val)
    char buf[128];
 
    buf[0] = '\0';
-#ifdef _WIN32
-   snprintf(buf, sizeof(buf), "%I64u", val);
-#else
-   snprintf(buf, sizeof(buf), "%llu", (long long unsigned)val);
-#endif
+   snprintf(buf, sizeof(buf), "%" PRIu64, val);
    config_set_string(conf, key, buf);
 }
 
@@ -884,22 +859,25 @@ void config_set_bool(config_file_t *conf, const char *key, bool val)
 
 bool config_file_write(config_file_t *conf, const char *path)
 {
-   FILE *file;
-
-   if (path)
+   if (!string_is_empty(path))
    {
-      file = fopen(path, "w");
+      void* buf  = NULL;
+      FILE *file = (FILE*)fopen_utf8(path, "wb");
       if (!file)
          return false;
-      setvbuf(file, NULL, _IOFBF, 0x4000);
+
+      /* TODO: this is only useful for a few platforms, find which and add ifdef */
+      buf = calloc(1, 0x4000);
+      setvbuf(file, (char*)buf, _IOFBF, 0x4000);
+
+      config_file_dump(conf, file);
+
+      if (file != stdout)
+         fclose(file);
+      free(buf);
    }
    else
-      file = stdout;
-
-   config_file_dump(conf, file);
-
-   if (path)
-      fclose(file);
+      config_file_dump(conf, stdout);
 
    return true;
 }
@@ -931,7 +909,7 @@ bool config_entry_exists(config_file_t *conf, const char *entry)
 
    while (list)
    {
-      if (!strcmp(entry, list->key))
+      if (string_is_equal(entry, list->key))
          return true;
       list = list->next;
    }
@@ -975,3 +953,63 @@ bool config_file_exists(const char *path)
    config_file_free(config);
    return true;
 }
+
+#if 0
+static void test_config_file_parse_contains(
+      const char * cfgtext,
+      const char *key, const char *val)
+{
+   config_file_t *cfg = config_file_new_from_string(cfgtext);
+   char          *out = NULL;
+   bool            ok = false;
+
+   if (!cfg)
+      abort();
+
+   ok = config_get_string(cfg, key, &out);
+   if (ok != (bool)val)
+      abort();
+   if (!val)
+      return;
+
+   if (out == NULL)
+      out = strdup("");
+   if (strcmp(out, val) != 0)
+      abort();
+   free(out);
+}
+
+static void test_config_file(void)
+{
+   test_config_file_parse_contains("foo = \"bar\"\n",   "foo", "bar");
+   test_config_file_parse_contains("foo = \"bar\"",     "foo", "bar");
+   test_config_file_parse_contains("foo = \"bar\"\r\n", "foo", "bar");
+   test_config_file_parse_contains("foo = \"bar\"",     "foo", "bar");
+
+#if 0
+   /* turns out it treats empty as nonexistent - 
+    * should probably be fixed */
+   test_config_file_parse_contains("foo = \"\"\n",   "foo", "");
+   test_config_file_parse_contains("foo = \"\"",     "foo", "");
+   test_config_file_parse_contains("foo = \"\"\r\n", "foo", "");
+   test_config_file_parse_contains("foo = \"\"",     "foo", "");
+#endif
+
+   test_config_file_parse_contains("foo = \"\"\n",   "bar", NULL);
+   test_config_file_parse_contains("foo = \"\"",     "bar", NULL);
+   test_config_file_parse_contains("foo = \"\"\r\n", "bar", NULL);
+   test_config_file_parse_contains("foo = \"\"",     "bar", NULL);
+}
+
+/* compile with:
+ gcc config_file.c -g -I ../include/ \
+ ../streams/file_stream.c ../vfs/vfs_implementation.c ../lists/string_list.c \
+ ../compat/compat_strl.c file_path.c ../compat/compat_strcasestr.c \
+ && ./a.out
+*/
+
+int main(void)
+{
+	test_config_file();
+}
+#endif
