@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- * 
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -13,6 +13,14 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/* Assume W-functions do not work below Win2K and Xbox platforms */
+#if defined(_WIN32_WINNT) && _WIN32_WINNT < 0x0500 || defined(_XBOX)
+
+#ifndef LEGACY_WIN32
+#define LEGACY_WIN32
+#endif
+
+#endif
 
 #ifdef _WIN32
 #include <direct.h>
@@ -20,14 +28,14 @@
 #include <unistd.h>
 #endif
 
-#ifdef __APPLE__
-#include <CoreFoundation/CoreFoundation.h>
-#endif
-
 #ifdef __QNX__
 #include <libgen.h>
 #endif
 
+#ifdef __HAIKU__
+#include <kernel/image.h>
+#endif
+
 #include <stdlib.h>
 #include <boolean.h>
 #include <string.h>
@@ -41,6 +49,7 @@
 #include <compat/posix_string.h>
 #include <retro_assert.h>
 #include <retro_miscellaneous.h>
+#include <encodings/utf.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -52,112 +61,10 @@
 #include "paths.h"
 #include "verbosity.h"
 
-void fill_pathname_expand_special(char *out_path,
-      const char *in_path, size_t size)
-{
-#if !defined(RARCH_CONSOLE)
-   if (*in_path == '~')
-   {
-      const char *home = getenv("HOME");
-      if (home)
-      {
-         size_t src_size = strlcpy(out_path, home, size);
-         retro_assert(src_size < size);
-
-         out_path  += src_size;
-         size      -= src_size;
-         in_path++;
-      }
-   }
-   else if ((in_path[0] == ':') &&
-         (
-         (in_path[1] == '/')
-#ifdef _WIN32
-         || (in_path[1] == '\\')
-#endif
-         )
-            )
-   {
-      size_t src_size;
-      char application_dir[PATH_MAX_LENGTH];
-
-      application_dir[0] = '\0';
-
-      fill_pathname_application_path(application_dir, sizeof(application_dir));
-      path_basedir_wrapper(application_dir);
-
-      src_size   = strlcpy(out_path, application_dir, size);
-      retro_assert(src_size < size);
-
-      out_path  += src_size;
-      size      -= src_size;
-      in_path   += 2;
-   }
-#endif
-
-   retro_assert(strlcpy(out_path, in_path, size) < size);
-}
-
-
-void fill_pathname_abbreviate_special(char *out_path,
-      const char *in_path, size_t size)
-{
-#if !defined(RARCH_CONSOLE)
-   unsigned i;
-   const char *candidates[3];
-   const char *notations[3];
-   char application_dir[PATH_MAX_LENGTH];
-   const char                      *home = getenv("HOME");
-
-   application_dir[0] = '\0';
-
-   /* application_dir could be zero-string. Safeguard against this.
-    *
-    * Keep application dir in front of home, moving app dir to a
-    * new location inside home would break otherwise. */
-
-   /* ugly hack - use application_dir pointer before filling it in. C89 reasons */
-   candidates[0] = application_dir;
-   candidates[1] = home;
-   candidates[2] = NULL;
-
-   notations [0] = ":";
-   notations [1] = "~";
-   notations [2] = NULL;
-
-   fill_pathname_application_path(application_dir, sizeof(application_dir));
-   path_basedir_wrapper(application_dir);
-   
-   for (i = 0; candidates[i]; i++)
-   {
-      if (!string_is_empty(candidates[i]) && strstr(in_path, candidates[i]) == in_path)
-      {
-         size_t src_size  = strlcpy(out_path, notations[i], size);
-
-         retro_assert(src_size < size);
-      
-         out_path        += src_size;
-         size            -= src_size;
-         in_path         += strlen(candidates[i]);
-      
-         if (!path_char_is_slash(*in_path))
-         {
-            retro_assert(strlcpy(out_path, path_default_slash(), size) < size);
-            out_path++;
-            size--;
-         }
-
-         break; /* Don't allow more abbrevs to take place. */
-      }
-   }
-#endif
-
-   retro_assert(strlcpy(out_path, in_path, size) < size);
-}
-
 bool fill_pathname_application_data(char *s, size_t len)
 {
 #if defined(_WIN32) && !defined(_XBOX)
+#ifdef LEGACY_WIN32
    const char *appdata = getenv("APPDATA");
 
    if (appdata)
@@ -165,6 +72,21 @@ bool fill_pathname_application_data(char *s, size_t len)
       strlcpy(s, appdata, len);
       return true;
    }
+#else
+   const wchar_t *appdataW = _wgetenv(L"APPDATA");
+
+   if (appdataW)
+   {
+      char *appdata = utf16_to_utf8_string_alloc(appdataW);
+
+      if (appdata)
+      {
+         strlcpy(s, appdata, len);
+         free(appdata);
+         return true;
+      }
+   }
+#endif
 
 #elif defined(OSX)
    const char *appdata = getenv("HOME");
@@ -179,16 +101,18 @@ bool fill_pathname_application_data(char *s, size_t len)
    const char *xdg     = getenv("XDG_CONFIG_HOME");
    const char *appdata = getenv("HOME");
 
-   /* XDG_CONFIG_HOME falls back to $HOME/.config. */
+   /* XDG_CONFIG_HOME falls back to $HOME/.config with most Unix systems */
+   /* On Haiku, it is set by default to /home/user/config/settings */
    if (xdg)
    {
-      fill_pathname_join(s, xdg, "retroarch", len);
+      fill_pathname_join(s, xdg, "retroarch/", len);
       return true;
    }
 
    if (appdata)
    {
 #ifdef __HAIKU__
+      /* in theory never used as Haiku has XDG_CONFIG_HOME set by default */
       fill_pathname_join(s, appdata,
             "config/settings/retroarch/", len);
 #else
@@ -202,93 +126,12 @@ bool fill_pathname_application_data(char *s, size_t len)
    return false;
 }
 
-#if !defined(RARCH_CONSOLE)
-void fill_pathname_application_path(char *s, size_t len)
-{
-   size_t i;
-#ifdef __APPLE__
-  CFBundleRef bundle = CFBundleGetMainBundle();
-#endif
-#ifdef _WIN32
-   DWORD ret;
-   wchar_t ws[PATH_MAX_LENGTH] = {0};
-#endif
-#ifdef __HAIKU__
-   image_info info;
-   int32_t cookie = 0;
-#endif
-   (void)i;
-
-   if (!len)
-      return;
-
-#ifdef _WIN32
-   ret    = GetModuleFileName(GetModuleHandle(NULL), s, len - 1);
-   s[ret] = '\0';
-#elif defined(__APPLE__)
-   if (bundle)
-   {
-      CFURLRef bundle_url = CFBundleCopyBundleURL(bundle);
-      CFStringRef bundle_path = CFURLCopyPath(bundle_url);
-      CFStringGetCString(bundle_path, s, len, kCFStringEncodingUTF8);
-      CFRelease(bundle_path);
-      CFRelease(bundle_url);
-      
-      retro_assert(strlcat(s, "nobin", len) < len);
-      return;
-   }
-#elif defined(__HAIKU__)
-   while (get_next_image_info(0, &cookie, &info) == B_OK)
-   {
-      if (info.type == B_APP_IMAGE)
-      {
-         strlcpy(s, info.name, len);
-         return;
-      }
-   }
-#elif defined(__QNX__)
-   char *buff = malloc(len);
-
-   if(_cmdname(buff))
-      strlcpy(s, buff, len);
-
-   free(buff);
-#else
-   {
-      pid_t pid;
-      static const char *exts[] = { "exe", "file", "path/a.out" };
-      char link_path[255];
-
-      link_path[0] = *s = '\0';
-      pid       = getpid(); 
-
-      /* Linux, BSD and Solaris paths. Not standardized. */
-      for (i = 0; i < ARRAY_SIZE(exts); i++)
-      {
-         ssize_t ret;
-
-         snprintf(link_path, sizeof(link_path), "/proc/%u/%s",
-               (unsigned)pid, exts[i]);
-         ret = readlink(link_path, s, len - 1);
-
-         if (ret >= 0)
-         {
-            s[ret] = '\0';
-            return;
-         }
-      }
-   }
-   
-   RARCH_ERR("Cannot resolve application path! This should not happen.\n");
-#endif
-}
-#endif
-
 #ifdef HAVE_XMB
 const char* xmb_theme_ident(void);
 #endif
 
-void fill_pathname_application_special(char *s, size_t len, enum application_special_type type)
+void fill_pathname_application_special(char *s,
+      size_t len, enum application_special_type type)
 {
    switch (type)
    {
@@ -296,8 +139,8 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
          {
             settings_t *settings     = config_get_ptr();
             fill_pathname_join(s,
-                  settings->directory.autoconfig,
-                  settings->input.joypad_driver,
+                  settings->paths.directory_autoconfig,
+                  settings->arrays.input_joypad_driver,
                   len);
          }
          break;
@@ -307,8 +150,8 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
 
             /* Try config directory setting first,
              * fallback to the location of the current configuration file. */
-            if (!string_is_empty(settings->directory.menu_config))
-               strlcpy(s, settings->directory.menu_config, len);
+            if (!string_is_empty(settings->paths.directory_menu_config))
+               strlcpy(s, settings->paths.directory_menu_config, len);
             else if (!path_is_empty(RARCH_PATH_CONFIG))
                fill_pathname_basedir(s, path_get(RARCH_PATH_CONFIG), len);
          }
@@ -322,13 +165,16 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
       case APPLICATION_SPECIAL_DIRECTORY_ASSETS_ZARCH_FONT:
 #ifdef HAVE_ZARCH
          {
-            char s1[PATH_MAX_LENGTH];
-            s1[0] = '\0';
+            char *s1 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
+            s1[0]    = '\0';
 
-            fill_pathname_application_special(s1, sizeof(s1),
+            fill_pathname_application_special(s1,
+                  PATH_MAX_LENGTH * sizeof(char),
                   APPLICATION_SPECIAL_DIRECTORY_ASSETS_ZARCH);
             fill_pathname_join(s,
                   s1, "Roboto-Condensed.ttf", len);
+
+            free(s1);
          }
 #endif
          break;
@@ -336,8 +182,8 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
 #ifdef HAVE_ZARCH
          {
             settings_t *settings     = config_get_ptr();
-            fill_pathname_join(s, 
-                  settings->directory.assets,
+            fill_pathname_join(s,
+                  settings->paths.directory_assets,
                   "zarch",
                   len);
          }
@@ -346,17 +192,23 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
       case APPLICATION_SPECIAL_DIRECTORY_ASSETS_XMB_ICONS:
 #ifdef HAVE_XMB
          {
-            char s1[PATH_MAX_LENGTH];
-            char s2[PATH_MAX_LENGTH];
+            char *s1 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
+            char *s2 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
 
             s1[0] = s2[0] = '\0';
 
-            fill_pathname_application_special(s1, sizeof(s1),
+            fill_pathname_application_special(s1,
+                  PATH_MAX_LENGTH * sizeof(char),
                   APPLICATION_SPECIAL_DIRECTORY_ASSETS_XMB);
             fill_pathname_join(s2, s1, "png",
-                  sizeof(s2));
-            fill_pathname_slash(s2, sizeof(s2));
+                  PATH_MAX_LENGTH * sizeof(char)
+                  );
+            fill_pathname_slash(s2,
+                  PATH_MAX_LENGTH * sizeof(char)
+                  );
             strlcpy(s, s2, len);
+            free(s1);
+            free(s2);
          }
 #endif
          break;
@@ -365,19 +217,21 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
          {
             settings_t *settings     = config_get_ptr();
 
-            if (!string_is_empty(settings->path.menu_wallpaper))
-               strlcpy(s, settings->path.menu_wallpaper, len);
+            if (!string_is_empty(settings->paths.path_menu_wallpaper))
+               strlcpy(s, settings->paths.path_menu_wallpaper, len);
             else
             {
-               char s1[PATH_MAX_LENGTH];
+               char *s1 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
 
                s1[0] = '\0';
 
-               fill_pathname_application_special(s1, sizeof(s1),
+               fill_pathname_application_special(s1,
+                     PATH_MAX_LENGTH * sizeof(char),
                      APPLICATION_SPECIAL_DIRECTORY_ASSETS_XMB_ICONS);
                fill_pathname_join(s, s1,
                      file_path_str(FILE_PATH_BACKGROUND_IMAGE),
                      len);
+               free(s1);
             }
          }
 #endif
@@ -385,20 +239,25 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
       case APPLICATION_SPECIAL_DIRECTORY_ASSETS_XMB:
 #ifdef HAVE_XMB
          {
-            char s1[PATH_MAX_LENGTH];
-            char s2[PATH_MAX_LENGTH];
+            char *s1 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
+            char *s2 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
             settings_t *settings     = config_get_ptr();
 
             s1[0] = s2[0] = '\0';
 
             fill_pathname_join(
                   s1,
-                  settings->directory.assets,
+                  settings->paths.directory_assets,
                   "xmb",
-                  sizeof(s1));
+                  PATH_MAX_LENGTH * sizeof(char)
+                  );
             fill_pathname_join(s2,
-                  s1, xmb_theme_ident(), sizeof(s2));
+                  s1, xmb_theme_ident(),
+                  PATH_MAX_LENGTH * sizeof(char)
+                  );
             strlcpy(s, s2, len);
+            free(s1);
+            free(s2);
          }
 #endif
          break;
@@ -409,7 +268,7 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
 
             fill_pathname_join(
                   s,
-                  settings->directory.assets,
+                  settings->paths.directory_assets,
                   "glui",
                   len);
          }
@@ -418,27 +277,35 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
       case APPLICATION_SPECIAL_DIRECTORY_ASSETS_MATERIALUI_ICONS:
 #ifdef HAVE_MATERIALUI
          {
-            char s1[PATH_MAX_LENGTH];
+            char *s1 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
 
             s1[0] = '\0';
 
             fill_pathname_application_special(s1,
-                  sizeof(s1), APPLICATION_SPECIAL_DIRECTORY_ASSETS_MATERIALUI);
-            fill_pathname_slash(s1, sizeof(s1));
+                  PATH_MAX_LENGTH * sizeof(char),
+                  APPLICATION_SPECIAL_DIRECTORY_ASSETS_MATERIALUI);
+            fill_pathname_slash(s1,
+                  PATH_MAX_LENGTH * sizeof(char)
+                  );
             strlcpy(s, s1, len);
+
+            free(s1);
          }
 #endif
          break;
       case APPLICATION_SPECIAL_DIRECTORY_ASSETS_MATERIALUI_FONT:
 #ifdef HAVE_MATERIALUI
          {
-            char s1[PATH_MAX_LENGTH];
+            char *s1 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
 
             s1[0] = '\0';
 
-            fill_pathname_application_special(s1, sizeof(s1),
+            fill_pathname_application_special(s1,
+                  PATH_MAX_LENGTH * sizeof(char),
                   APPLICATION_SPECIAL_DIRECTORY_ASSETS_MATERIALUI);
             fill_pathname_join(s, s1, "font.ttf", len);
+
+            free(s1);
          }
 #endif
          break;
@@ -447,23 +314,51 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
          {
             settings_t *settings = config_get_ptr();
 
-            if (!string_is_empty(settings->menu.xmb.font))
-               strlcpy(s, settings->menu.xmb.font, len);
+            if (!string_is_empty(settings->paths.path_menu_xmb_font))
+               strlcpy(s, settings->paths.path_menu_xmb_font, len);
             else
             {
-               char s1[PATH_MAX_LENGTH];
+               char *s1 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
 
                s1[0] = '\0';
 
-               fill_pathname_application_special(s1, sizeof(s1),
+               fill_pathname_application_special(s1,
+                     PATH_MAX_LENGTH * sizeof(char),
                      APPLICATION_SPECIAL_DIRECTORY_ASSETS_XMB);
                fill_pathname_join(s, s1,
                      file_path_str(FILE_PATH_TTF_FONT),
                      len);
+
+               free(s1);
             }
          }
 #endif
          break;
+      case APPLICATION_SPECIAL_DIRECTORY_THUMBNAILS_CHEEVOS_BADGES:
+      {
+        char *s1 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
+        char *s2 = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
+        settings_t *settings     = config_get_ptr();
+
+        s1[0] = s2[0] = '\0';
+
+        fill_pathname_join(s1,
+              settings->paths.directory_thumbnails,
+              "cheevos",
+              len);
+        fill_pathname_join(s2,
+              s1, "badges",
+              PATH_MAX_LENGTH * sizeof(char)
+              );
+        fill_pathname_slash(s2,
+              PATH_MAX_LENGTH * sizeof(char)
+              );
+        strlcpy(s, s2, len);
+        free(s1);
+        free(s2);
+      }
+      break;
+
       case APPLICATION_SPECIAL_NONE:
       default:
          break;
@@ -488,18 +383,18 @@ void fill_pathname_application_special(char *s, size_t len, enum application_spe
 void fill_short_pathname_representation_wrapper(char* out_rep,
       const char *in_path, size_t size)
 {
-   char path_short[PATH_MAX_LENGTH];
 #ifdef HAVE_COMPRESSION
-   char *last_slash                  = NULL;
-#endif
+   char *path_short = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
+   char *last_slash = NULL;
 
-   path_short[0] = '\0';
+   path_short[0]    = '\0';
 
    fill_pathname(path_short, path_basename(in_path), "",
-            sizeof(path_short));
+         PATH_MAX_LENGTH * sizeof(char)
+         );
 
-#ifdef HAVE_COMPRESSION
    last_slash  = find_last_slash(path_short);
+
    if (last_slash != NULL)
    {
       /* We handle paths like:
@@ -511,37 +406,12 @@ void fill_short_pathname_representation_wrapper(char* out_rep,
        */
       retro_assert(strlen(last_slash) > 1);
       strlcpy(out_rep, last_slash + 1, size);
+      free(path_short);
       return;
    }
+
+   free(path_short);
 #endif
 
    fill_short_pathname_representation(out_rep, in_path, size);
 }
-
-/**
- * path_basedir:
- * @path               : path
- *
- * Extracts base directory by mutating path.
- * Keeps trailing '/'.
- **/
-void path_basedir_wrapper(char *path)
-{
-   char *last = NULL;
-   if (strlen(path) < 2)
-      return;
-
-#ifdef HAVE_COMPRESSION
-   /* We want to find the directory with the archive in basedir. */
-   last = (char*)path_get_archive_delim(path);
-   if (last)
-      *last = '\0';
-#endif
-
-   last = find_last_slash(path);
-
-   if (last)
-      last[1] = '\0';
-   else
-      snprintf(path, 3, ".%s", path_default_slash());
-}
