@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2016 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -21,6 +21,10 @@
 #include <stddef.h>
 
 #include <boolean.h>
+#include <retro_common_api.h>
+#include <retro_environment.h>
+
+RETRO_BEGIN_DECLS
 
 enum file_path_enum
 {
@@ -34,6 +38,7 @@ enum file_path_enum
    FILE_PATH_LOG_ERROR,
    FILE_PATH_LOG_INFO,
    FILE_PATH_CONTENT_HISTORY,
+   FILE_PATH_CONTENT_FAVORITES,
    FILE_PATH_CONTENT_MUSIC_HISTORY,
    FILE_PATH_CONTENT_VIDEO_HISTORY,
    FILE_PATH_CONTENT_IMAGE_HISTORY,
@@ -80,6 +85,13 @@ enum file_path_enum
    FILE_PATH_AUTO_EXTENSION,
    FILE_PATH_ZIP_EXTENSION,
    FILE_PATH_7Z_EXTENSION,
+   FILE_PATH_OGG_EXTENSION,
+   FILE_PATH_MP3_EXTENSION,
+   FILE_PATH_FLAC_EXTENSION,
+   FILE_PATH_WAV_EXTENSION,
+   FILE_PATH_MOD_EXTENSION,
+   FILE_PATH_S3M_EXTENSION,
+   FILE_PATH_XM_EXTENSION,
    FILE_PATH_CONFIG_EXTENSION,
    FILE_PATH_CORE_INFO_EXTENSION
 };
@@ -98,7 +110,8 @@ enum application_special_type
    APPLICATION_SPECIAL_DIRECTORY_ASSETS_XMB_FONT,
    APPLICATION_SPECIAL_DIRECTORY_ASSETS_ZARCH,
    APPLICATION_SPECIAL_DIRECTORY_ASSETS_ZARCH_FONT,
-   APPLICATION_SPECIAL_DIRECTORY_ASSETS_ZARCH_ICONS
+   APPLICATION_SPECIAL_DIRECTORY_ASSETS_ZARCH_ICONS,
+   APPLICATION_SPECIAL_DIRECTORY_THUMBNAILS_CHEEVOS_BADGES
 };
 
 /**
@@ -119,19 +132,12 @@ enum application_special_type
 void fill_short_pathname_representation_wrapper(char* out_rep,
       const char *in_path, size_t size);
 
-/**
- * path_basedir:
- * @path               : path
- *
- * Extracts base directory by mutating path.
- * Keeps trailing '/'.
- **/
-void path_basedir_wrapper(char *path);
-
 const char *file_path_str(enum file_path_enum enum_idx);
 
 bool fill_pathname_application_data(char *s, size_t len);
 
 void fill_pathname_application_special(char *s, size_t len, enum application_special_type type);
 
+RETRO_END_DECLS
+
 #endif
