@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -26,32 +26,12 @@
 
 RETRO_BEGIN_DECLS
 
-enum font_driver_render_api
-{
-   FONT_DRIVER_RENDER_DONT_CARE,
-   FONT_DRIVER_RENDER_OPENGL_API,
-   FONT_DRIVER_RENDER_DIRECT3D_API,
-   FONT_DRIVER_RENDER_VITA2D,
-   FONT_DRIVER_RENDER_CTR,
-   FONT_DRIVER_RENDER_VULKAN_API,
-   FONT_DRIVER_RENDER_CACA,
-   FONT_DRIVER_RENDER_GDI,
-   FONT_DRIVER_RENDER_VGA
-};
-
-enum text_alignment
-{
-   TEXT_ALIGN_LEFT = 0,
-   TEXT_ALIGN_RIGHT,
-   TEXT_ALIGN_CENTER
-};
-
 /* All coordinates and offsets are top-left oriented.
  *
- * This is a texture-atlas approach which allows text to 
+ * This is a texture-atlas approach which allows text to
  * be drawn in a single draw call.
  *
- * It is up to the code using this interface to actually 
+ * It is up to the code using this interface to actually
  * generate proper vertex buffers and upload the atlas texture to GPU. */
 
 struct font_glyph
@@ -63,7 +43,7 @@ struct font_glyph
    unsigned atlas_offset_x;
    unsigned atlas_offset_y;
 
-   /* When drawing this glyph, apply an offset to 
+   /* When drawing this glyph, apply an offset to
     * current X/Y draw coordinate. */
    int draw_offset_x;
    int draw_offset_y;
@@ -107,13 +87,14 @@ typedef struct font_renderer
    void (*render_msg)(
          video_frame_info_t *video_info,
          void *data, const char *msg,
-         const void *params);
+         const struct font_params *params);
    const char *ident;
 
    const struct font_glyph *(*get_glyph)(void *data, uint32_t code);
    void (*bind_block)(void *data, void *block);
-   void (*flush)(unsigned width, unsigned height, void *data);
-   
+   void (*flush)(unsigned width, unsigned height, void *data,
+         video_frame_info_t *video_info);
+
    int (*get_message_width)(void *data, const char *msg, unsigned msg_len_full, float scale);
 } font_renderer_t;
 
@@ -131,7 +112,7 @@ typedef struct font_renderer_driver
    const char *(*get_default_font)(void);
 
    const char *ident;
-   
+
    int (*get_line_height)(void* data);
 } font_renderer_driver_t;
 
@@ -145,22 +126,32 @@ typedef struct
 /* font_path can be NULL for default font. */
 int font_renderer_create_default(const void **driver,
       void **handle, const char *font_path, unsigned font_size);
-      
+
 void font_driver_render_msg(video_frame_info_t *video_info,
-      void *font_data, const char *msg, const void *params);
+      void *font_data, const char *msg, const struct font_params *params);
 
 void font_driver_bind_block(void *font_data, void *block);
 
 int font_driver_get_message_width(void *font_data, const char *msg, unsigned len, float scale);
 
-void font_driver_flush(unsigned width, unsigned height, void *font_data);
+void font_driver_flush(unsigned width, unsigned height, void *font_data,
+      video_frame_info_t *video_info);
 
 void font_driver_free(void *font_data);
 
-font_data_t *font_driver_init_first(void *video_data, const char *font_path,
-      float font_size, bool threading_hint, enum font_driver_render_api api);
+font_data_t *font_driver_init_first(
+      void *video_data,
+      const char *font_path,
+      float font_size,
+      bool threading_hint,
+      bool is_threaded,
+      enum font_driver_render_api api);
 
-void font_driver_init_osd(void *video_data, bool threading_hint, enum font_driver_render_api api);
+void font_driver_init_osd(
+      void *video_data,
+      bool threading_hint,
+      bool is_threaded,
+      enum font_driver_render_api api);
 void font_driver_free_osd(void);
 
 extern font_renderer_t gl_raster_font;
@@ -170,7 +161,11 @@ extern font_renderer_t d3d_xdk1_font;
 extern font_renderer_t d3d_win32_font;
 extern font_renderer_t vita2d_vita_font;
 extern font_renderer_t ctr_font;
+extern font_renderer_t wiiu_font;
 extern font_renderer_t vulkan_raster_font;
+extern font_renderer_t d3d10_font;
+extern font_renderer_t d3d11_font;
+extern font_renderer_t d3d12_font;
 extern font_renderer_t caca_font;
 extern font_renderer_t gdi_font;
 extern font_renderer_t vga_font;
