@@ -1,4 +1,4 @@
-/* Copyright  (C) 2010-2017 The RetroArch team
+/* Copyright  (C) 2010-2018 The RetroArch team
  *
  * ---------------------------------------------------------------------------------------
  * The following license statement only applies to this file (rpng_encode.c).
@@ -57,7 +57,7 @@ static bool png_write_crc(RFILE *file, const uint8_t *data, size_t size)
 static bool png_write_ihdr(RFILE *file, const struct png_ihdr *ihdr)
 {
    uint8_t ihdr_raw[21];
-   
+
    ihdr_raw[0]  = '0';                 /* Size */
    ihdr_raw[1]  = '0';
    ihdr_raw[2]  = '0';
@@ -227,7 +227,9 @@ static bool rpng_save_image(const char *path,
    void *stream            = NULL;
    uint32_t total_in       = 0;
    uint32_t total_out      = 0;
-   RFILE *file             = filestream_open(path, RFILE_MODE_WRITE, -1);
+   RFILE *file             = filestream_open(path,
+         RETRO_VFS_FILE_ACCESS_WRITE,
+         RETRO_VFS_FILE_ACCESS_HINT_NONE);
    if (!file)
       GOTO_END_ERROR();
 
@@ -272,7 +274,7 @@ static bool rpng_save_image(const char *path,
       /* Try every filtering method, and choose the method
        * which has most entries as zero.
        *
-       * This is probably not very optimal, but it's very 
+       * This is probably not very optimal, but it's very
        * simple to implement.
        */
       {
@@ -311,7 +313,6 @@ static bool rpng_save_image(const char *path,
          {
             filter = 4;
             chosen_filtered = paeth_filtered;
-            min_sad = paeth_score;
          }
 
          *encode_target++ = filter;
@@ -333,11 +334,11 @@ static bool rpng_save_image(const char *path,
    stream_backend->set_in(
          stream,
          encode_buf,
-         encode_buf_size);
+         (unsigned)encode_buf_size);
    stream_backend->set_out(
          stream,
          deflate_buf + 8,
-         encode_buf_size * 2);
+         (unsigned)(encode_buf_size * 2));
 
    if (!stream_backend->trans(stream, true, &total_in, &total_out, NULL))
    {
@@ -353,7 +354,8 @@ static bool rpng_save_image(const char *path,
       GOTO_END_ERROR();
 
 end:
-   filestream_close(file);
+   if (file)
+      filestream_close(file);
    free(encode_buf);
    free(deflate_buf);
    free(rgba_line);
