@@ -12,7 +12,7 @@ int UmountFS(void *pClient, void *pCmd, const char *mountPath);
 
 int LoadFileToMem(const char *filepath, u8 **inbuffer, u32 *size);
 
-//! todo: C++ class
+/* TODO/FIXME: C++ class */
 int CreateSubfolder(const char * fullpath);
 int CheckFile(const char * filepath);
 
@@ -20,4 +20,4 @@ int CheckFile(const char * filepath);
 }
 #endif
 
-#endif // __FS_UTILS_H_
+#endif /* __FS_UTILS_H_ */
