@@ -1,4 +1,4 @@
-/* Copyright  (C) 2010-2017 The RetroArch team
+/* Copyright  (C) 2010-2018 The RetroArch team
  *
  * ---------------------------------------------------------------------------------------
  * The following license statement only applies to this file (file_path.h).
@@ -31,7 +31,6 @@
 #include <retro_common_api.h>
 
 #include <boolean.h>
-#include <retro_inline.h>
 
 RETRO_BEGIN_DECLS
 
@@ -74,16 +73,6 @@ bool path_is_compressed_file(const char *path);
  **/
 #define path_contains_compressed_file(path) (path_get_archive_delim((path)) != NULL)
 
-/**
- * path_file_exists:
- * @path               : path
- *
- * Checks if a file already exists at the specified path (@path).
- *
- * Returns: true (1) if file already exists, otherwise false (0).
- */
-bool path_file_exists(const char *path);
-
 /**
  * path_get_archive_delim:
  * @path               : path
@@ -100,7 +89,7 @@ const char *path_get_archive_delim(const char *path);
  * path_get_extension:
  * @path               : path
  *
- * Gets extension of file. Only '.'s 
+ * Gets extension of file. Only '.'s
  * after the last slash are considered.
  *
  * Returns: extension part from the path.
@@ -131,7 +120,7 @@ const char *path_basename(const char *path);
 
 /**
  * path_basedir:
- * @path               : path           
+ * @path               : path
  *
  * Extracts base directory by mutating path.
  * Keeps trailing '/'.
@@ -171,7 +160,7 @@ bool path_is_absolute(const char *path);
  * fill_pathname:
  * @out_path           : output path
  * @in_path            : input  path
- * @replace            : what to replace 
+ * @replace            : what to replace
  * @size               : buffer size of output path
  *
  * FIXME: Verify
@@ -183,10 +172,10 @@ bool path_is_absolute(const char *path);
  * Only '.'s after the last slash are considered as extensions.
  * If no '.' is present, in_path and replace will simply be concatenated.
  * 'size' is buffer size of 'out_path'.
- * E.g.: in_path = "/foo/bar/baz/boo.c", replace = ".asm" => 
- * out_path = "/foo/bar/baz/boo.asm" 
+ * E.g.: in_path = "/foo/bar/baz/boo.c", replace = ".asm" =>
+ * out_path = "/foo/bar/baz/boo.asm"
  * E.g.: in_path = "/foo/bar/baz/boo.c", replace = ""     =>
- * out_path = "/foo/bar/baz/boo" 
+ * out_path = "/foo/bar/baz/boo"
  */
 void fill_pathname(char *out_path, const char *in_path,
       const char *replace, size_t size);
@@ -197,10 +186,10 @@ void fill_pathname(char *out_path, const char *in_path,
  * @ext                : extension of output filename
  * @size               : buffer size of output filename
  *
- * Creates a 'dated' filename prefixed by 'RetroArch', and 
+ * Creates a 'dated' filename prefixed by 'RetroArch', and
  * concatenates extension (@ext) to it.
  *
- * E.g.: 
+ * E.g.:
  * out_filename = "RetroArch-{month}{day}-{Hours}{Minutes}.{@ext}"
  **/
 void fill_dated_filename(char *out_filename,
@@ -226,7 +215,7 @@ void fill_str_dated_filename(char *out_filename,
  * fill_pathname_noext:
  * @out_path           : output path
  * @in_path            : input  path
- * @replace            : what to replace 
+ * @replace            : what to replace
  * @size               : buffer size of output path
  *
  * Appends a filename extension 'replace' to 'in_path', and outputs
@@ -271,7 +260,7 @@ void fill_pathname_dir(char *in_dir, const char *in_basename,
 
 /**
  * fill_pathname_base:
- * @out                : output path         
+ * @out                : output path
  * @in_path            : input path
  * @size               : size of output path
  *
@@ -288,7 +277,7 @@ void fill_pathname_base_ext(char *out,
 
 /**
  * fill_pathname_basedir:
- * @out_dir            : output directory        
+ * @out_dir            : output directory
  * @in_path            : input path
  * @size               : size of output directory
  *
@@ -301,9 +290,22 @@ void fill_pathname_basedir(char *out_path, const char *in_path, size_t size);
 void fill_pathname_basedir_noext(char *out_dir,
       const char *in_path, size_t size);
 
+/**
+ * fill_pathname_parent_dir_name:
+ * @out_dir            : output directory
+ * @in_dir             : input directory
+ * @size               : size of output directory
+ *
+ * Copies only the parent directory name of @in_dir into @out_dir.
+ * The two buffers must not overlap. Removes trailing '/'.
+ * Returns true on success, false if a slash was not found in the path.
+ **/
+bool fill_pathname_parent_dir_name(char *out_dir,
+      const char *in_dir, size_t size);
+
 /**
  * fill_pathname_parent_dir:
- * @out_dir            : output directory        
+ * @out_dir            : output directory
  * @in_dir             : input directory
  * @size               : size of output directory
  *
@@ -331,12 +333,12 @@ void fill_pathname_resolve_relative(char *out_path, const char *in_refpath,
 /**
  * fill_pathname_join:
  * @out_path           : output path
- * @dir                : directory   
+ * @dir                : directory
  * @path               : path
  * @size               : size of output path
  *
  * Joins a directory (@dir) and path (@path) together.
- * Makes sure not to get  two consecutive slashes 
+ * Makes sure not to get  two consecutive slashes
  * between directory and path.
  **/
 void fill_pathname_join(char *out_path, const char *dir,
@@ -347,8 +349,14 @@ void fill_pathname_join_special_ext(char *out_path,
       const char *last, const char *ext,
       size_t size);
 
+void fill_pathname_join_concat_noext(
+      char *out_path,
+      const char *dir, const char *path,
+      const char *concat,
+      size_t size);
+
 void fill_pathname_join_concat(char *out_path,
-      const char *dir, const char *path, 
+      const char *dir, const char *path,
       const char *concat,
       size_t size);
 
@@ -358,12 +366,12 @@ void fill_pathname_join_noext(char *out_path,
 /**
  * fill_pathname_join_delim:
  * @out_path           : output path
- * @dir                : directory   
+ * @dir                : directory
  * @path               : path
- * @delim              : delimiter 
+ * @delim              : delimiter
  * @size               : size of output path
  *
- * Joins a directory (@dir) and path (@path) together 
+ * Joins a directory (@dir) and path (@path) together
  * using the given delimiter (@delim).
  **/
 void fill_pathname_join_delim(char *out_path, const char *dir,
@@ -400,6 +408,15 @@ void fill_pathname_expand_special(char *out_path,
 void fill_pathname_abbreviate_special(char *out_path,
       const char *in_path, size_t size);
 
+/**
+ * path_basedir:
+ * @path               : path
+ *
+ * Extracts base directory by mutating path.
+ * Keeps trailing '/'.
+ **/
+void path_basedir_wrapper(char *path);
+
 /**
  * path_char_is_slash:
  * @c                  : character
@@ -408,32 +425,28 @@ void fill_pathname_abbreviate_special(char *out_path,
  *
  * Returns: true (1) if character is a slash, otherwise false (0).
  */
-static INLINE bool path_char_is_slash(char c)
-{
 #ifdef _WIN32
-   return (c == '/') || (c == '\\');
+#define path_char_is_slash(c) (((c) == '/') || ((c) == '\\'))
 #else
-   return (c == '/');
+#define path_char_is_slash(c) ((c) == '/')
 #endif
-}
 
 /**
- * path_default_slash:
+ * path_default_slash and path_default_slash_c:
  *
  * Gets the default slash separator.
  *
  * Returns: default slash separator.
  */
-static INLINE const char *path_default_slash(void)
-{
 #ifdef _WIN32
-   return "\\";
+#define path_default_slash() "\\"
+#define path_default_slash_c() '\\'
 #else
-   return "/";
+#define path_default_slash() "/"
+#define path_default_slash_c() '/'
 #endif
-}
 
-/** 
+/**
  * fill_pathname_slash:
  * @path               : path
  * @size               : size of path
@@ -443,7 +456,7 @@ static INLINE const char *path_default_slash(void)
  **/
 void fill_pathname_slash(char *path, size_t size);
 
-#ifndef RARCH_CONSOLE
+#if !defined(RARCH_CONSOLE) && defined(RARCH_INTERNAL)
 void fill_pathname_application_path(char *buf, size_t size);
 #endif
 
@@ -457,6 +470,22 @@ void fill_pathname_application_path(char *buf, size_t size);
  **/
 bool path_mkdir(const char *dir);
 
+/**
+ * path_is_directory:
+ * @path               : path
+ *
+ * Checks if path is a directory.
+ *
+ * Returns: true (1) if path is a directory, otherwise false (0).
+ */
+bool path_is_directory(const char *path);
+
+bool path_is_character_special(const char *path);
+
+bool path_is_valid(const char *path);
+
+int32_t path_get_size(const char *path);
+
 RETRO_END_DECLS
 
 #endif
