@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -18,6 +18,7 @@
 #define __INPUT_DEFINES__H
 
 #include <stdint.h>
+#include <string.h>
 
 #include <retro_common_api.h>
 
@@ -25,9 +26,17 @@ RETRO_BEGIN_DECLS
 
 #define MAX_USERS                      16
 
+#define MAX_INPUT_DEVICES              16
+
+#define RARCH_MAX_KEYS 136
+
 #define RARCH_FIRST_CUSTOM_BIND        16
+#define RARCH_FIRST_LIGHTGUN_BIND      RARCH_ANALOG_BIND_LIST_END
+#define RARCH_FIRST_MISC_CUSTOM_BIND   RARCH_LIGHTGUN_BIND_LIST_END
 #define RARCH_FIRST_META_KEY           RARCH_CUSTOM_BIND_LIST_END
 
+#define RARCH_UNMAPPED                 1024
+
 /* RetroArch specific bind IDs. */
 enum
 {
@@ -42,9 +51,24 @@ enum
    RARCH_ANALOG_RIGHT_X_MINUS,
    RARCH_ANALOG_RIGHT_Y_PLUS,
    RARCH_ANALOG_RIGHT_Y_MINUS,
+   RARCH_ANALOG_BIND_LIST_END,
+
+   /* Lightgun */
+   RARCH_LIGHTGUN_TRIGGER = RARCH_FIRST_LIGHTGUN_BIND,
+   RARCH_LIGHTGUN_RELOAD,
+   RARCH_LIGHTGUN_AUX_A,
+   RARCH_LIGHTGUN_AUX_B,
+   RARCH_LIGHTGUN_AUX_C,
+   RARCH_LIGHTGUN_START,
+   RARCH_LIGHTGUN_SELECT,
+   RARCH_LIGHTGUN_DPAD_UP,
+   RARCH_LIGHTGUN_DPAD_DOWN,
+   RARCH_LIGHTGUN_DPAD_LEFT,
+   RARCH_LIGHTGUN_DPAD_RIGHT,
+   RARCH_LIGHTGUN_BIND_LIST_END,
 
    /* Turbo */
-   RARCH_TURBO_ENABLE,
+   RARCH_TURBO_ENABLE = RARCH_FIRST_MISC_CUSTOM_BIND,
 
    RARCH_CUSTOM_BIND_LIST_END,
 
@@ -52,6 +76,8 @@ enum
     * only usable for port 0. */
    RARCH_FAST_FORWARD_KEY = RARCH_FIRST_META_KEY,
    RARCH_FAST_FORWARD_HOLD_KEY,
+   RARCH_SLOWMOTION_KEY,
+   RARCH_SLOWMOTION_HOLD_KEY,
    RARCH_LOAD_STATE_KEY,
    RARCH_SAVE_STATE_KEY,
    RARCH_FULLSCREEN_TOGGLE_KEY,
@@ -71,9 +97,7 @@ enum
    RARCH_SCREENSHOT,
    RARCH_MUTE,
    RARCH_OSK,
-   RARCH_NETPLAY_FLIP,
    RARCH_NETPLAY_GAME_WATCH,
-   RARCH_SLOWMOTION,
    RARCH_ENABLE_HOTKEY,
    RARCH_VOLUME_UP,
    RARCH_VOLUME_DOWN,
@@ -83,6 +107,7 @@ enum
    RARCH_DISK_PREV,
    RARCH_GRAB_MOUSE_TOGGLE,
    RARCH_GAME_FOCUS_TOGGLE,
+   RARCH_UI_COMPANION_TOGGLE,
 
    RARCH_MENU_TOGGLE,
 
@@ -116,15 +141,15 @@ enum analog_dpad_mode
  * internally in RetroArch for simplicity, so they are mapped into [16, 23].
  */
 
-#define AXIS_NEG(x)        (((uint32_t)(x) << 16) | UINT16_C(0xFFFF))
-#define AXIS_POS(x)        ((uint32_t)(x) | UINT32_C(0xFFFF0000))
-#define AXIS_NONE          UINT32_C(0xFFFFFFFF)
-#define AXIS_DIR_NONE      UINT16_C(0xFFFF)
+#define AXIS_NEG(x)        (((uint32_t)(x) << 16) | 0xFFFFU)
+#define AXIS_POS(x)        ((uint32_t)(x) | 0xFFFF0000UL)
+#define AXIS_NONE          0xFFFFFFFFUL
+#define AXIS_DIR_NONE      0xFFFFU
 
-#define AXIS_NEG_GET(x)    (((uint32_t)(x) >> 16) & UINT16_C(0xFFFF))
-#define AXIS_POS_GET(x)    ((uint32_t)(x) & UINT16_C(0xFFFF))
+#define AXIS_NEG_GET(x)    (((uint32_t)(x) >> 16) & 0xFFFFU)
+#define AXIS_POS_GET(x)    ((uint32_t)(x) & 0xFFFFU)
 
-#define NO_BTN             UINT16_C(0xFFFF)
+#define NO_BTN             0xFFFFU
 
 #define HAT_UP_SHIFT       15
 #define HAT_DOWN_SHIFT     14
