@@ -42,21 +42,29 @@ struct apple_key_name_map_entry
    const char* const keyname;
    const uint32_t hid_id;
 };
-    
+
 extern const struct apple_key_name_map_entry apple_key_name_map[];
 #endif
 
+#define RARCH_KEY_MAP_RWEBINPUT_SIZE 111
+
 extern const struct input_key_map input_config_key_map[];
 
 extern const struct rarch_key_map rarch_key_map_x11[];
 extern const struct rarch_key_map rarch_key_map_sdl[];
 extern const struct rarch_key_map rarch_key_map_sdl2[];
 extern const struct rarch_key_map rarch_key_map_dinput[];
-extern const struct rarch_key_map rarch_key_map_rwebinput[];
+
+ /* is generated at runtime so can't be const */
+extern struct rarch_key_map rarch_key_map_rwebinput[RARCH_KEY_MAP_RWEBINPUT_SIZE];
+
 extern const struct rarch_key_map rarch_key_map_linux[];
 extern const struct rarch_key_map rarch_key_map_apple_hid[];
 extern const struct rarch_key_map rarch_key_map_android[];
 extern const struct rarch_key_map rarch_key_map_qnx[];
+extern const struct rarch_key_map rarch_key_map_dos[];
+extern const struct rarch_key_map rarch_key_map_wiiu[];
+extern const struct rarch_key_map rarch_key_map_winraw[];
 
 /**
  * input_keymaps_init_keyboard_lut:
@@ -77,29 +85,19 @@ void input_keymaps_init_keyboard_lut(const struct rarch_key_map *map);
  **/
 enum retro_key input_keymaps_translate_keysym_to_rk(unsigned sym);
 
-/**
- * input_keymaps_translate_rk_to_keysym:
- * @key                   : Retro key identifier
- *
- * Translates a retro key identifier to a key symbol
- * from the keyboard layout table.
- *
- * Returns: key symbol from the keyboard layout table.
- **/
-unsigned input_keymaps_translate_rk_to_keysym(enum retro_key key);
-
 /**
  * input_keymaps_translate_rk_to_str:
  * @key                   : Retro key identifier.
  * @buf                   : Buffer.
  * @size                  : Size of @buf.
  *
- * Translates a retro key identifier to a human-readable 
+ * Translates a retro key identifier to a human-readable
  * identifier string.
  **/
 void input_keymaps_translate_rk_to_str(enum retro_key key, char *buf, size_t size);
 
+extern enum retro_key rarch_keysym_lut[RETROK_LAST];
+
 RETRO_END_DECLS
 
 #endif
-
