@@ -1,5 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *  Copyright (C) 2016-2017 - Andrés Suárez
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -44,7 +45,9 @@ bool input_remapping_load_file(void *data, const char *path);
  **/
 bool input_remapping_save_file(const char *path);
 
-void input_remapping_set_defaults(void);
+bool input_remapping_remove_file(const char *path);
+
+void input_remapping_set_defaults(bool deinit);
 
 RETRO_END_DECLS
 
