@@ -29,6 +29,7 @@
 #include <compat/posix_string.h>
 #include <file/file_path.h>
 #include <retro_miscellaneous.h>
+#include <libretro.h>
 #include <net/net_compat.h>
 #include <net/net_socket.h>
 
@@ -47,30 +48,27 @@
 
 struct remote_message
 {
+   uint16_t state;
    int port;
    int device;
    int index;
    int id;
-   uint16_t state;
 };
 
 struct input_remote
 {
-
+   bool state[RARCH_BIND_LIST_END];
 #if defined(HAVE_NETWORKING) && defined(HAVE_NETWORKGAMEPAD)
    int net_fd[MAX_USERS];
 #endif
-
-   bool state[RARCH_BIND_LIST_END];
 };
 
 typedef struct input_remote_state
 {
+   /* Left X, Left Y, Right X, Right Y */
+   int16_t analog[4][MAX_USERS];
    /* This is a bitmask of (1 << key_bind_id). */
    uint64_t buttons[MAX_USERS];
-   /* Left X, Left Y, Right X, Right Y */
-   int16_t analog[4][MAX_USERS]; 
-
 } input_remote_state_t;
 
 static input_remote_state_t remote_st_ptr;
@@ -138,7 +136,7 @@ input_remote_t *input_remote_new(uint16_t port, unsigned max_users)
    for(user = 0; user < max_users; user ++)
    {
       handle->net_fd[user] = -1;
-      if(settings->network_remote_enable_user[user])
+      if(settings->bools.network_remote_enable_user[user])
          if (!input_remote_init_network(handle, port, user))
             goto error;
    }
@@ -168,18 +166,18 @@ void input_remote_free(input_remote_t *handle, unsigned max_users)
 #if defined(HAVE_NETWORKING) && defined(HAVE_NETWORKGAMEPAD)
 static void input_remote_parse_packet(struct remote_message *msg, unsigned user)
 {
-   input_remote_state_t *ol_state  = input_remote_get_state_ptr();
+   input_remote_state_t *input_state  = input_remote_get_state_ptr();
 
    /* Parse message */
    switch (msg->device)
    {
       case RETRO_DEVICE_JOYPAD:
-         ol_state->buttons[user] &= ~(1 << msg->id);
+         input_state->buttons[user] &= ~(1 << msg->id);
          if (msg->state)
-            ol_state->buttons[user] |= 1 << msg->id;
+            input_state->buttons[user] |= 1 << msg->id;
          break;
       case RETRO_DEVICE_ANALOG:
-         ol_state->analog[msg->index * 2 + msg->id][user] = msg->state;
+         input_state->analog[msg->index * 2 + msg->id][user] = msg->state;
          break;
    }
 }
@@ -192,11 +190,6 @@ void input_remote_state(
       unsigned idx,
       unsigned id)
 {
-   input_remote_state_t *ol_state  = input_remote_get_state_ptr();
-
-   if (!ol_state)
-      return;
-
    switch (device)
    {
       case RETRO_DEVICE_JOYPAD:
@@ -206,13 +199,17 @@ void input_remote_state(
       case RETRO_DEVICE_ANALOG:
          {
             unsigned base = 0;
+            input_remote_state_t *input_state  = input_remote_get_state_ptr();
+
+            if (!input_state)
+               return;
 
             if (idx == RETRO_DEVICE_INDEX_ANALOG_RIGHT)
                base = 2;
             if (id == RETRO_DEVICE_ID_ANALOG_Y)
                base += 1;
-            if (ol_state && ol_state->analog[base][port])
-               *ret = ol_state->analog[base][port];
+            if (input_state && input_state->analog[base][port])
+               *ret = input_state->analog[base][port];
          }
          break;
    }
@@ -220,23 +217,23 @@ void input_remote_state(
 
 bool input_remote_key_pressed(int key, unsigned port)
 {
-   input_remote_state_t *ol_state  = input_remote_get_state_ptr();
+   input_remote_state_t *input_state  = input_remote_get_state_ptr();
 
-   if (!ol_state)
+   if (!input_state)
       return false;
 
-   return (ol_state->buttons[port] & (UINT64_C(1) << key));
+   return (input_state->buttons[port] & (UINT64_C(1) << key));
 }
 
 void input_remote_poll(input_remote_t *handle, unsigned max_users)
 {
    unsigned user;
    settings_t *settings            = config_get_ptr();
-   input_remote_state_t *ol_state  = input_remote_get_state_ptr();
-   
+   input_remote_state_t *input_state  = input_remote_get_state_ptr();
+
    for(user = 0; user < max_users; user++)
    {
-      if (settings->network_remote_enable_user[user])
+      if (settings->bools.network_remote_enable_user[user])
       {
 #if defined(HAVE_NETWORKING) && defined(HAVE_NETWORKGAMEPAD)
          struct remote_message msg;
@@ -257,11 +254,11 @@ void input_remote_poll(input_remote_t *handle, unsigned max_users)
          else if ((ret != -1) || ((errno != EAGAIN) && (errno != ENOENT)))
 #endif
          {
-            ol_state->buttons[user] = 0;
-            ol_state->analog[0][user] = 0;
-            ol_state->analog[1][user] = 0;
-            ol_state->analog[2][user] = 0;
-            ol_state->analog[3][user] = 0;
+            input_state->buttons[user]   = 0;
+            input_state->analog[0][user] = 0;
+            input_state->analog[1][user] = 0;
+            input_state->analog[2][user] = 0;
+            input_state->analog[3][user] = 0;
          }
       }
    }
