@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 - Brad Parker */
+#include <retro_common_api.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -36,7 +36,7 @@ extern "C" {
 #endif
 
 #ifdef HAVE_GL_FFT
-#include "fft/fft.h"
+#include "ffmpeg_fft.h"
 #endif
 
 #if defined(HAVE_OPENGL) || defined(HAVE_OPENGLES)
@@ -45,6 +45,7 @@ extern "C" {
 
 #include <rthreads/rthreads.h>
 #include <queues/fifo_queue.h>
+#include <string/stdstring.h>
 
 #include <libretro.h>
 #ifdef RARCH_INTERNAL
@@ -115,7 +116,7 @@ static struct attachment *attachments;
 static size_t attachments_size;
 
 #ifdef HAVE_GL_FFT
-static glfft_t *fft;
+static fft_t *fft;
 unsigned fft_width;
 unsigned fft_height;
 unsigned fft_multisample;
@@ -214,7 +215,7 @@ void CORE_PREFIX(retro_init)(void)
 
    av_register_all();
 #if 0
-   /* FIXME: Occasionally crashes inside libavdevice 
+   /* FIXME: Occasionally crashes inside libavdevice
     * for some odd reason on reentrancy. Likely a libavdevice bug. */
    avdevice_register_all();
 #endif
@@ -272,10 +273,10 @@ void CORE_PREFIX(retro_set_environment)(retro_environment_t cb)
 {
    static const struct retro_variable vars[] = {
 #if defined(HAVE_OPENGL) || defined(HAVE_OPENGLES)
-      { "ffmpeg_temporal_interp", "Temporal Interpolation; enabled|disabled" },
+      { "ffmpeg_temporal_interp", "Temporal Interpolation; disabled|enabled" },
 #ifdef HAVE_GL_FFT
-      { "ffmpeg_fft_resolution", "GLFFT Resolution; 1280x720|1920x1080|640x360|320x180" },
-      { "ffmpeg_fft_multisample", "GLFFT Multisample; 1x|2x|4x" },
+      { "ffmpeg_fft_resolution", "FFT Resolution; 1280x720|1920x1080|2560x1440|3840x2160|640x360|320x180" },
+      { "ffmpeg_fft_multisample", "FFT Multisample; 1x|2x|4x" },
 #endif
 #endif
       { "ffmpeg_color_space", "Colorspace; auto|BT.709|BT.601|FCC|SMPTE240M" },
@@ -339,9 +340,9 @@ static void check_variables(void)
 
    if (CORE_PREFIX(environ_cb)(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
    {
-      if (!strcmp(var.value, "enabled"))
+      if (memcmp(var.value, "enabled", 7) == 0)
          temporal_interpolation = true;
-      else if (!strcmp(var.value, "disabled"))
+      else if (memcmp(var.value, "disabled", 8) == 0)
          temporal_interpolation = false;
    }
 
@@ -373,13 +374,13 @@ static void check_variables(void)
    if (CORE_PREFIX(environ_cb)(RETRO_ENVIRONMENT_GET_VARIABLE, &color_var) && color_var.value)
    {
       slock_lock(decode_thread_lock);
-      if (!strcmp(color_var.value, "BT.709"))
+      if (string_is_equal(color_var.value, "BT.709"))
          colorspace = AVCOL_SPC_BT709;
-      else if (!strcmp(color_var.value, "BT.601"))
+      else if (string_is_equal(color_var.value, "BT.601"))
          colorspace = AVCOL_SPC_BT470BG;
-      else if (!strcmp(color_var.value, "FCC"))
+      else if (memcmp(color_var.value, "FCC", 3) == 0)
          colorspace = AVCOL_SPC_FCC;
-      else if (!strcmp(color_var.value, "SMPTE240M"))
+      else if (string_is_equal(color_var.value, "SMPTE240M"))
          colorspace = AVCOL_SPC_SMPTE240M;
       else
          colorspace = AVCOL_SPC_UNSPECIFIED;
@@ -462,7 +463,7 @@ void CORE_PREFIX(retro_run)(void)
    }
 
    if (fft && (old_fft_multisample != fft_multisample))
-      glfft_init_multisample(fft, fft_width, fft_height, fft_multisample);
+      fft_init_multisample(fft, fft_width, fft_height, fft_multisample);
 #endif
 
    CORE_PREFIX(input_poll_cb)();
@@ -548,7 +549,7 @@ void CORE_PREFIX(retro_run)(void)
 
    frame_cnt++;
 
-   /* Have to decode audio before video 
+   /* Have to decode audio before video
     * incase there are PTS fuckups due
     * to seeking. */
    if (audio_streams_num > 0)
@@ -679,20 +680,20 @@ void CORE_PREFIX(retro_run)(void)
 
          if (!temporal_interpolation)
             mix_factor = 1.0f;
-         
+
          glBindFramebuffer(GL_FRAMEBUFFER, hw_render.get_current_framebuffer());
          glClearColor(0, 0, 0, 1);
          glClear(GL_COLOR_BUFFER_BIT);
          glViewport(0, 0, media.width, media.height);
          glUseProgram(prog);
-         
+
          glUniform1f(mix_loc, mix_factor);
          glActiveTexture(GL_TEXTURE1);
          glBindTexture(GL_TEXTURE_2D, frames[1].tex);
          glActiveTexture(GL_TEXTURE0);
          glBindTexture(GL_TEXTURE_2D, frames[0].tex);
-         
-         
+
+
          glBindBuffer(GL_ARRAY_BUFFER, vbo);
          glVertexAttribPointer(vertex_loc, 2, GL_FLOAT, GL_FALSE,
                4 * sizeof(GLfloat), (const GLvoid*)(0 * sizeof(GLfloat)));
@@ -701,17 +702,17 @@ void CORE_PREFIX(retro_run)(void)
          glEnableVertexAttribArray(vertex_loc);
          glEnableVertexAttribArray(tex_loc);
          glBindBuffer(GL_ARRAY_BUFFER, 0);
-         
+
          glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
          glDisableVertexAttribArray(vertex_loc);
          glDisableVertexAttribArray(tex_loc);
-         
+
          glUseProgram(0);
          glActiveTexture(GL_TEXTURE1);
          glBindTexture(GL_TEXTURE_2D, 0);
          glActiveTexture(GL_TEXTURE0);
          glBindTexture(GL_TEXTURE_2D, 0);
-         
+
          CORE_PREFIX(video_cb)(RETRO_HW_FRAME_BUFFER_VALID,
                media.width, media.height, media.width * sizeof(uint32_t));
       }
@@ -731,17 +732,17 @@ void CORE_PREFIX(retro_run)(void)
       while (frames)
       {
          unsigned to_read = frames;
-         
+
          /* FFT size we use (1 << 11). Really shouldn't happen,
           * unless we use a crazy high sample rate. */
-         if (to_read > (1 << 11)) 
+         if (to_read > (1 << 11))
             to_read = 1 << 11;
 
-         glfft_step_fft(fft, buffer, to_read);
+         fft_step_fft(fft, buffer, to_read);
          buffer += to_read * 2;
          frames -= to_read;
       }
-      glfft_render(fft, hw_render.get_current_framebuffer(), fft_width, fft_height);
+      fft_render(fft, hw_render.get_current_framebuffer(), fft_width, fft_height);
       CORE_PREFIX(video_cb)(RETRO_HW_FRAME_BUFFER_VALID,
             fft_width, fft_height, fft_width * sizeof(uint32_t));
    }
@@ -861,7 +862,7 @@ static bool open_codecs(void)
             break;
 
          case AVMEDIA_TYPE_VIDEO:
-            if (     !vctx 
+            if (     !vctx
                   && !codec_is_image(fctx->streams[i]->codec->codec_id))
             {
                if (!open_codec(&vctx, i))
@@ -872,7 +873,7 @@ static bool open_codecs(void)
 
          case AVMEDIA_TYPE_SUBTITLE:
 #ifdef HAVE_SSA
-            if (     subtitle_streams_num < MAX_STREAMS 
+            if (     subtitle_streams_num < MAX_STREAMS
                   && codec_id_is_ass(fctx->streams[i]->codec->codec_id))
             {
                int size;
@@ -923,7 +924,7 @@ static bool init_media_info(void)
    {
       media.width  = vctx->width;
       media.height = vctx->height;
-      media.aspect = (float)vctx->width * 
+      media.aspect = (float)vctx->width *
          av_q2d(vctx->sample_aspect_ratio) / vctx->height;
    }
 
@@ -1149,7 +1150,7 @@ static void render_ass_img(AVFrame *conv_frame, ASS_Image *img)
             dst_g = (g * src_alpha + dst_g * dst_alpha) >> 8;
             dst_b = (b * src_alpha + dst_b * dst_alpha) >> 8;
 
-            dst[x] = (0xffu << 24) | (dst_r << 16) | 
+            dst[x] = (0xffu << 24) | (dst_r << 16) |
                (dst_g << 8) | (dst_b << 0);
          }
       }
@@ -1171,7 +1172,7 @@ static void decode_thread(void *data)
    struct SwsContext *sws  = NULL;
 
    (void)data;
-   
+
    if (video_stream >= 0)
       sws = sws_getCachedContext(NULL,
             media.width, media.height, vctx->pix_fmt,
@@ -1295,7 +1296,7 @@ static void decode_thread(void *data)
                int stride;
                unsigned y;
                const uint8_t *src = NULL;
-               
+
                fifo_write(video_decode_fifo, &pts, sizeof(pts));
                src    = conv_frame->data[0];
                stride = conv_frame->linesize[0];
@@ -1369,7 +1370,7 @@ static void context_destroy(void)
 #ifdef HAVE_GL_FFT
    if (fft)
    {
-      glfft_free(fft);
+      fft_free(fft);
       fft = NULL;
    }
 #endif
@@ -1377,8 +1378,8 @@ static void context_destroy(void)
 
 #include "gl_shaders/ffmpeg.glsl.vert.h"
 
-/* OpenGL ES note about main() -  Get format as GL_RGBA/GL_UNSIGNED_BYTE. 
- * Assume little endian, so we get ARGB -> BGRA byte order, and 
+/* OpenGL ES note about main() -  Get format as GL_RGBA/GL_UNSIGNED_BYTE.
+ * Assume little endian, so we get ARGB -> BGRA byte order, and
  * we have to swizzle to .BGR. */
 #ifdef HAVE_OPENGLES
 #include "gl_shaders/ffmpeg_es.glsl.frag.h"
@@ -1400,9 +1401,9 @@ static void context_reset(void)
 #ifdef HAVE_GL_FFT
    if (audio_streams_num > 0 && video_stream < 0)
    {
-      fft = glfft_new(11, hw_render.get_proc_address);
+      fft = fft_new(11, hw_render.get_proc_address);
       if (fft)
-         glfft_init_multisample(fft, fft_width, fft_height, fft_multisample);
+         fft_init_multisample(fft, fft_width, fft_height, fft_multisample);
    }
 
    /* Already inits symbols. */
@@ -1445,7 +1446,7 @@ static void context_reset(void)
 #if !defined(HAVE_OPENGLES)
       glGenBuffers(1, &frames[i].pbo);
       glBindBuffer(GL_PIXEL_UNPACK_BUFFER, frames[i].pbo);
-      glBufferData(GL_PIXEL_UNPACK_BUFFER, media.width 
+      glBufferData(GL_PIXEL_UNPACK_BUFFER, media.width
             * media.height * sizeof(uint32_t), NULL, GL_STREAM_DRAW);
       glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);
 #endif
@@ -1555,7 +1556,7 @@ void CORE_PREFIX(retro_unload_game)(void)
 
 bool CORE_PREFIX(retro_load_game)(const struct retro_game_info *info)
 {
-   bool is_glfft = false;
+   bool is_fft = false;
    enum retro_pixel_format fmt = RETRO_PIXEL_FORMAT_XRGB8888;
    struct retro_input_descriptor desc[] = {
       { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_LEFT,  "Seek -10 seconds" },
@@ -1606,21 +1607,21 @@ bool CORE_PREFIX(retro_load_game)(const struct retro_game_info *info)
    }
 
 #ifdef HAVE_GL_FFT
-   is_glfft = video_stream < 0 && audio_streams_num > 0;
+   is_fft = video_stream < 0 && audio_streams_num > 0;
 #endif
 
-   if (video_stream >= 0 || is_glfft)
+   if (video_stream >= 0 || is_fft)
    {
-      video_decode_fifo = fifo_new(media.width 
+      video_decode_fifo = fifo_new(media.width
             * media.height * sizeof(uint32_t) * 32);
 
 #if defined(HAVE_OPENGL) || defined(HAVE_OPENGLES)
       use_gl = true;
-      hw_render.context_reset = context_reset;
-      hw_render.context_destroy = context_destroy;
-      hw_render.bottom_left_origin = is_glfft;
-      hw_render.depth = is_glfft;
-      hw_render.stencil = is_glfft;
+      hw_render.context_reset      = context_reset;
+      hw_render.context_destroy    = context_destroy;
+      hw_render.bottom_left_origin = is_fft;
+      hw_render.depth              = is_fft;
+      hw_render.stencil            = is_fft;
 #if defined(HAVE_OPENGLES)
       hw_render.context_type = RETRO_HW_CONTEXT_OPENGLES2;
 #else
