@@ -1,8 +1,8 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- *  Copyright (C) 2016 - Brad Parker
- * 
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *  Copyright (C) 2016-2017 - Brad Parker
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -48,8 +48,6 @@
 #include "core_info.h"
 #include "gfx/video_driver.h"
 #include "input/input_driver.h"
-#include "input/input_hid_driver.h"
-#include "input/input_joypad_driver.h"
 #include "audio/audio_driver.h"
 #include "record/record_driver.h"
 #include "configuration.h"
@@ -59,37 +57,32 @@ struct string_list *dir_list_new_special(const char *input_dir,
 {
    char ext_shaders[255];
    char ext_name[255];
-   const char *dir                   = NULL;
    const char *exts                  = NULL;
-   bool include_dirs                 = false;
    bool recursive                    = false;
    settings_t *settings              = config_get_ptr();
 
-   ext_shaders[0] = ext_name[0] = '\0';
-
-   (void)input_dir;
-
    switch (type)
    {
       case DIR_LIST_AUTOCONFIG:
-         dir  = input_dir;
          exts = filter;
          break;
       case DIR_LIST_CORES:
-         dir  = input_dir;
+         {
+            ext_name[0]         = '\0';
 
-         if (!frontend_driver_get_core_extension(ext_name, sizeof(ext_name)))
-            return NULL;
+            if (!frontend_driver_get_core_extension(ext_name, sizeof(ext_name)))
+               return NULL;
 
-         exts = ext_name;
+            exts = ext_name;
+         }
          break;
       case DIR_LIST_CORE_INFO:
          {
             core_info_list_t *list = NULL;
             core_info_get_list(&list);
 
-            dir  = input_dir;
-            exts = list->all_ext;
+            if (list)
+               exts = list->all_ext;
          }
          break;
       case DIR_LIST_RECURSIVE:
@@ -97,49 +90,54 @@ struct string_list *dir_list_new_special(const char *input_dir,
           core_info_list_t *list = NULL;
           core_info_get_list(&list);
 
-          dir  = input_dir;
-          exts = list->all_ext;
+          if (list)
+             exts = list->all_ext;
           recursive = true;
        }
        break;
       case DIR_LIST_SHADERS:
          {
-            union string_list_elem_attr attr = {0};
+            union string_list_elem_attr attr;
+            bool is_preset                   = false;
             struct string_list *str_list     = string_list_new();
 
             if (!str_list)
                return NULL;
+            
+            ext_shaders[0]                   = '\0';
 
-            (void)attr;
+            attr.i = 0;
 
-            dir  = input_dir;
-#ifdef HAVE_CG
-            string_list_append(str_list, "cg", attr);
-            string_list_append(str_list, "cgp", attr);
-#endif
-#ifdef HAVE_GLSL
-            string_list_append(str_list, "glsl", attr);
-            string_list_append(str_list, "glslp", attr);
-#endif
-#ifdef HAVE_VULKAN
-            string_list_append(str_list, "slang", attr);
-            string_list_append(str_list, "slangp", attr);
-#endif
+            if (video_shader_is_supported(RARCH_SHADER_CG) &&
+                  video_shader_get_type_from_ext("cgp", &is_preset))
+               string_list_append(str_list, "cgp", attr);
+            if (video_shader_is_supported(RARCH_SHADER_CG) &&
+                  video_shader_get_type_from_ext("cg", &is_preset))
+               string_list_append(str_list, "cg", attr);
+            if (video_shader_is_supported(RARCH_SHADER_GLSL) &&
+                  video_shader_get_type_from_ext("glsl", &is_preset))
+               string_list_append(str_list, "glsl", attr);
+            if (video_shader_is_supported(RARCH_SHADER_GLSL) &&
+                  video_shader_get_type_from_ext("glslp", &is_preset))
+               string_list_append(str_list, "glslp", attr);
+            if (video_shader_is_supported(RARCH_SHADER_SLANG) &&
+                  video_shader_get_type_from_ext("slang", &is_preset))
+               string_list_append(str_list, "slang", attr);
+            if (video_shader_is_supported(RARCH_SHADER_SLANG) &&
+                  video_shader_get_type_from_ext("slangp", &is_preset))
+               string_list_append(str_list, "slangp", attr);
             string_list_join_concat(ext_shaders, sizeof(ext_shaders), str_list, "|");
             string_list_free(str_list);
             exts = ext_shaders;
          }
          break;
       case DIR_LIST_COLLECTIONS:
-         dir  = input_dir;
          exts = "lpl";
          break;
       case DIR_LIST_DATABASES:
-         dir  = input_dir;
          exts = "rdb";
          break;
       case DIR_LIST_PLAIN:
-         dir  = input_dir;
          exts = filter;
          break;
       case DIR_LIST_NONE:
@@ -147,7 +145,8 @@ struct string_list *dir_list_new_special(const char *input_dir,
          return NULL;
    }
 
-   return dir_list_new(dir, exts, include_dirs, settings->show_hidden_files, type == DIR_LIST_CORE_INFO, recursive);
+   return dir_list_new(input_dir, exts, false, settings->bools.show_hidden_files,
+         type == DIR_LIST_CORE_INFO, recursive);
 }
 
 struct string_list *string_list_new_special(enum string_list_type type,
