@@ -1,8 +1,8 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- *  Copyright (C) 2016 - Brad Parker
- * 
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *  Copyright (C) 2016-2017 - Brad Parker
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -22,6 +22,9 @@
 #include <stddef.h>
 
 #include <lists/string_list.h>
+#include <retro_environment.h>
+
+RETRO_BEGIN_DECLS
 
 enum dir_list_type
 {
@@ -62,4 +65,6 @@ struct string_list *string_list_new_special(enum string_list_type type,
 
 const char *char_list_new_special(enum string_list_type type, void *data);
 
+RETRO_END_DECLS
+
 #endif
