@@ -19,6 +19,7 @@
 
 #include <compat/strl.h>
 #include <encodings/utf.h>
+#include <retro_math.h>
 #include <retro_miscellaneous.h>
 #include <features/features_cpu.h>
 
@@ -36,7 +37,7 @@ struct tween
    float       initial_value;
    float       target_value;
    float       *subject;
-   int         tag;
+   uintptr_t   tag;
    easing_cb   easing;
    tween_cb    cb;
 };
@@ -44,6 +45,7 @@ struct tween
 struct menu_animation
 {
    struct tween *list;
+   bool need_defrag;
 
    size_t capacity;
    size_t size;
@@ -289,15 +291,15 @@ static float easing_out_in_bounce(float t, float b, float c, float d)
 static void menu_animation_ticker_generic(uint64_t idx,
       size_t max_width, size_t *offset, size_t *width)
 {
-   int ticker_period     = 2 * (*width - max_width) + 4;
+   int ticker_period     = (int)(2 * (*width - max_width) + 4);
    int phase             = idx % ticker_period;
 
    int phase_left_stop   = 2;
-   int phase_left_moving = phase_left_stop + (*width - max_width);
+   int phase_left_moving = (int)(phase_left_stop + (*width - max_width));
    int phase_right_stop  = phase_left_moving + 2;
 
    int left_offset       = phase - phase_left_stop;
-   int right_offset      = (*width - max_width) - (phase - phase_right_stop);
+   int right_offset      = (int)((*width - max_width) - (phase - phase_right_stop));
 
    if (phase < phase_left_stop)
       *offset = 0;
@@ -459,9 +461,38 @@ bool menu_animation_push(menu_animation_ctx_entry_t *entry)
 
    *target = t;
 
+   anim.need_defrag = true;
+
    return true;
 }
 
+static int menu_animation_defrag_cmp(const void *a, const void *b)
+{
+   const struct tween *ta = (const struct tween *)a;
+   const struct tween *tb = (const struct tween *)b;
+
+   return tb->alive - ta->alive;
+}
+
+/* defragments and shrinks the tween list when possible */
+static void menu_animation_defrag()
+{
+   size_t i;
+
+   qsort(anim.list, anim.size, sizeof(anim.list[0]), menu_animation_defrag_cmp);
+
+   for (i = anim.size-1; i > 0; i--)
+   {
+      if (anim.list[i].alive)
+         break;
+
+      anim.size--;
+   }
+
+   anim.first_dead = anim.size;
+   anim.need_defrag = false;
+}
+
 bool menu_animation_update(float delta_time)
 {
    unsigned i;
@@ -485,9 +516,7 @@ bool menu_animation_update(float delta_time)
       {
          *tween->subject = tween->target_value;
          tween->alive    = false;
-
-         if (i < anim.first_dead)
-            anim.first_dead = i;
+         anim.need_defrag = true;
 
          if (tween->cb)
             tween->cb();
@@ -497,13 +526,18 @@ bool menu_animation_update(float delta_time)
          active_tweens += 1;
    }
 
+   if (anim.need_defrag)
+      menu_animation_defrag();
+
    if (!active_tweens)
    {
       anim.size           = 0;
       anim.first_dead     = 0;
+      anim.need_defrag    = false;
       return false;
    }
 
+
    animation_is_active = true;
 
    return true;
@@ -558,7 +592,7 @@ bool menu_animation_get_ideal_delta_time(menu_animation_ctx_delta_t *delta)
 
 void menu_animation_update_time(bool timedate_enable)
 {
-   static retro_time_t 
+   static retro_time_t
       last_clock_update     = 0;
 
    cur_time                 = cpu_features_get_time_usec();
@@ -570,7 +604,7 @@ void menu_animation_update_time(bool timedate_enable)
       delta_time            = IDEAL_DELTA_TIME / 4;
    old_time                 = cur_time;
 
-   if (((cur_time - last_clock_update) > 1000000) 
+   if (((cur_time - last_clock_update) > 1000000)
          && timedate_enable)
    {
       animation_is_active   = true;
@@ -622,14 +656,14 @@ bool menu_animation_ctl(enum menu_animation_ctl_state state, void *data)
       case MENU_ANIMATION_CTL_KILL_BY_TAG:
          {
             unsigned i;
-            menu_animation_ctx_tag_t *tag = (menu_animation_ctx_tag_t*)data;
+            menu_animation_ctx_tag *tag = (menu_animation_ctx_tag*)data;
 
-            if (!tag || tag->id == -1)
+            if (!tag || *tag == (uintptr_t)-1)
                return false;
 
             for (i = 0; i < anim.size; ++i)
             {
-               if (anim.list[i].tag != tag->id)
+               if (anim.list[i].tag != *tag)
                   continue;
 
                anim.list[i].alive   = false;
@@ -637,17 +671,19 @@ bool menu_animation_ctl(enum menu_animation_ctl_state state, void *data)
 
                if (i < anim.first_dead)
                   anim.first_dead = i;
+
+               anim.need_defrag = true;
             }
          }
          break;
       case MENU_ANIMATION_CTL_KILL_BY_SUBJECT:
          {
             unsigned i, j,  killed = 0;
-            menu_animation_ctx_subject_t *subject = 
+            menu_animation_ctx_subject_t *subject =
                (menu_animation_ctx_subject_t*)data;
             float            **sub = (float**)subject->data;
 
-            for (i = 0; i < anim.size; ++i)
+            for (i = 0; i < anim.size && killed < subject->count; ++i)
             {
                if (!anim.list[i].alive)
                   continue;
@@ -664,6 +700,7 @@ bool menu_animation_ctl(enum menu_animation_ctl_state state, void *data)
                      anim.first_dead = i;
 
                   killed++;
+                  anim.need_defrag = true;
                   break;
                }
             }
