@@ -1,6 +1,6 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -22,7 +22,7 @@
 #include <boolean.h>
 #include <retro_common_api.h>
 
-#include "widgets/menu_list.h"
+#include <lists/file_list.h>
 
 #include "menu_setting.h"
 #include "menu_displaylist.h"
@@ -52,92 +52,98 @@ enum menu_entries_ctl_state
    MENU_ENTRIES_CTL_SHOW_BACK
 };
 
+enum menu_list_type
+{
+   MENU_LIST_PLAIN = 0,
+   MENU_LIST_HORIZONTAL,
+   MENU_LIST_TABS
+};
+
+typedef struct menu_list menu_list_t;
+
+typedef struct menu_ctx_list
+{
+   enum menu_list_type type;
+   const char *path;
+   char       *fullpath;
+   const char *label;
+   unsigned entry_type;
+   unsigned action;
+   size_t idx;
+   size_t selection;
+   size_t size;
+   size_t list_size;
+   void *entry;
+   file_list_t *list;
+} menu_ctx_list_t;
+
 typedef struct menu_file_list_cbs
 {
-   rarch_setting_t *setting;
    enum msg_hash_enums enum_idx;
+   const char *action_iterate_ident;
+   const char *action_deferred_push_ident;
+   const char *action_select_ident;
+   const char *action_get_title_ident;
+   const char *action_ok_ident;
+   const char *action_cancel_ident;
+   const char *action_scan_ident;
+   const char *action_right_ident;
+   const char *action_start_ident;
+   const char *action_info_ident;
+   const char *action_content_list_switch_ident;
+   const char *action_left_ident;
+   const char *action_refresh_ident;
+   const char *action_up_ident;
+   const char *action_label_ident;
+   const char *action_sublabel_ident;
+   const char *action_down_ident;
+   const char *action_get_value_ident;
+
+   rarch_setting_t *setting;
 
    int (*action_iterate)(const char *label, unsigned action);
-   const char *action_iterate_ident;
-
    int (*action_deferred_push)(menu_displaylist_info_t *info);
-   const char *action_deferred_push_ident;
-
    int (*action_select)(const char *path, const char *label, unsigned type,
          size_t idx);
-   const char *action_select_ident;
-
    int (*action_get_title)(const char *path, const char *label,
          unsigned type, char *s, size_t len);
-   const char *action_get_title_ident;
-
    int (*action_ok)(const char *path, const char *label, unsigned type,
          size_t idx, size_t entry_idx);
-   const char *action_ok_ident;
-
    int (*action_cancel)(const char *path, const char *label, unsigned type,
          size_t idx);
-   const char *action_cancel_ident;
-
    int (*action_scan)(const char *path, const char *label, unsigned type,
          size_t idx);
-   const char *action_scan_ident;
-
    int (*action_start)(unsigned type,  const char *label);
-   const char *action_start_ident;
-
    int (*action_info)(unsigned type,  const char *label);
-   const char *action_info_ident;
-
    int (*action_content_list_switch)(void *data, void *userdata, const char
          *path, const char *label, unsigned type);
-   const char *action_content_list_switch_ident;
-
    int (*action_left)(unsigned type, const char *label, bool wraparound);
-   const char *action_left_ident;
-
    int (*action_right)(unsigned type, const char *label, bool wraparound);
-   const char *action_right_ident;
-
    int (*action_refresh)(file_list_t *list, file_list_t *menu_list);
-   const char *action_refresh_ident;
-
    int (*action_up)(unsigned type, const char *label);
-   const char *action_up_ident;
-
    int (*action_label)(file_list_t *list,
          unsigned type, unsigned i,
          const char *label, const char *path,
          char *s, size_t len);
-   const char *action_label_ident;
-
    int (*action_sublabel)(file_list_t *list,
          unsigned type, unsigned i,
          const char *label, const char *path,
          char *s, size_t len);
-   const char *action_sublabel_ident;
-
    int (*action_down)(unsigned type, const char *label);
-   const char *action_down_ident;
-
    void (*action_get_value)(file_list_t* list,
          unsigned *w, unsigned type, unsigned i,
          const char *label, char *s, size_t len,
          const char *entry_label,
          const char *path,
          char *path_buf, size_t path_buf_size);
-   const char *action_get_value_ident;
-
 } menu_file_list_cbs_t;
 
-size_t menu_entries_get_end(void);
-
-void menu_entries_get(size_t i, void *data_entry);
-
 int menu_entries_get_title(char *title, size_t title_len);
 
 int menu_entries_get_core_title(char *title_msg, size_t title_msg_len);
 
+int menu_entries_get_core_name(char *s, size_t len);
+
 file_list_t *menu_entries_get_selection_buf_ptr(size_t idx);
 
 file_list_t *menu_entries_get_menu_stack_ptr(size_t idx);
@@ -162,19 +168,6 @@ void menu_entries_get_at_offset(const file_list_t *list, size_t idx,
       const char **path, const char **label, unsigned *file_type,
       size_t *entry_idx, const char **alt);
 
-void *menu_entries_get_userdata_at_offset(
-      const file_list_t *list, size_t idx);
-
-menu_file_list_cbs_t *menu_entries_get_actiondata_at_offset(
-      const file_list_t *list, size_t idx);
-
-void menu_entries_get_last(const file_list_t *list,
-      const char **path, const char **label,
-      unsigned *file_type, size_t *entry_idx);
-
-void menu_entries_set_alt_at_offset(file_list_t *list, size_t idx,
-      const char *alt);
-
 rarch_setting_t *menu_entries_get_setting(uint32_t i);
 
 void menu_entries_prepend(file_list_t *list, const char *path, const char *label,
