@@ -25,17 +25,36 @@
 
 RETRO_BEGIN_DECLS
 
-/* Send input code to menu for one frame.
+/*
+ * This function gets called in order to process all input events
+ * for the current frame.
  *
- * TODO/FIXME - needs to be overhauled so we can send multiple
+ * Sends input code to menu for one frame.
+ *
+ * It uses as input the local variables' input' and 'trigger_input'.
+ *
+ * Mouse and touch input events get processed inside this function.
+ *
+ * NOTE: 'input' and 'trigger_input' is sourced from the keyboard and/or
+ * the gamepad. It does not contain input state derived from the mouse
+ * and/or touch - this gets dealt with separately within this function.
+ *
+ * TODO/FIXME - maybe needs to be overhauled so we can send multiple
  * events per frame if we want to, and we shouldn't send the
  * entire button state either but do a separate event per button
  * state.
  */
-unsigned menu_event(uint64_t input, uint64_t trigger_state);
+unsigned menu_event(input_bits_t *p_input, input_bits_t *p_trigger_state);
 
+/* Set a specific keyboard key.
+ *
+ * 'down' sets the latch (true would
+ * mean the key is being pressed down, while 'false' would mean that
+ * the key has been released).
+ **/
 void menu_event_kb_set(bool down, enum retro_key key);
 
+/* Check if a specific keyboard key has been pressed. */
 unsigned char menu_event_kb_is_set(enum retro_key key);
 
 RETRO_END_DECLS
