@@ -18,11 +18,9 @@
 #ifndef _MENU_INPUT_H
 #define _MENU_INPUT_H
 
+#include <stdint.h>
 #include <retro_common_api.h>
 
-#include "../input/input_driver.h"
-#include "../input/input_keyboard.h"
-
 RETRO_BEGIN_DECLS
 
 enum menu_action
@@ -90,14 +88,15 @@ typedef struct menu_input
 
    struct
    {
+      bool back;
+      bool pressed[2];
       int16_t x;
       int16_t y;
       int16_t dx;
       int16_t dy;
-      float accel;
-      bool pressed[2];
-      bool back;
       unsigned ptr;
+      unsigned counter;
+      float accel;
    } pointer;
 } menu_input_t;
 
