@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- * 
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -14,20 +14,20 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <libretro.h>
 #include <rhash.h>
 #include <compat/strl.h>
 #include <retro_endianness.h>
+#include <streams/interface_stream.h>
 
 #include "configuration.h"
 #include "movie.h"
 #include "core.h"
 #include "content.h"
 #include "retroarch.h"
-#include "runloop.h"
 #include "msg_hash.h"
 #include "verbosity.h"
 
@@ -36,7 +36,7 @@
 
 struct bsv_movie
 {
-   FILE *file;
+   intfstream_t *file;
 
    /* A ring buffer keeping track of positions
     * in the file for each frame. */
@@ -56,42 +56,40 @@ struct bsv_movie
 
 struct bsv_state
 {
-   /* Movie playback/recording support. */
-   bsv_movie_t *movie;
-   char movie_path[PATH_MAX_LENGTH];
-   bool movie_playback;
-   bool eof_exit;
-
-   /* Immediate playback/recording. */
-   char movie_start_path[PATH_MAX_LENGTH];
    bool movie_start_recording;
    bool movie_start_playback;
+   bool movie_playback;
+   bool eof_exit;
    bool movie_end;
+
+   /* Movie playback/recording support. */
+   char movie_path[PATH_MAX_LENGTH];
+   /* Immediate playback/recording. */
+   char movie_start_path[PATH_MAX_LENGTH];
 };
 
-struct bsv_state bsv_movie_state;
+static bsv_movie_t     *bsv_movie_state_handle = NULL;
+static struct bsv_state bsv_movie_state;
 
-static bool init_playback(bsv_movie_t *handle, const char *path)
+static bool bsv_movie_init_playback(bsv_movie_t *handle, const char *path)
 {
-   uint32_t state_size;
-   uint32_t *content_crc_ptr = NULL;
+   uint32_t state_size       = 0;
+   uint32_t content_crc      = 0;
    uint32_t header[4]        = {0};
+   intfstream_t *file        = intfstream_open_file(path,
+         RETRO_VFS_FILE_ACCESS_READ,
+         RETRO_VFS_FILE_ACCESS_HINT_NONE);
 
-   handle->playback          = true;
-   handle->file              = fopen(path, "rb");
-
-   if (!handle->file)
+   if (!file)
    {
       RARCH_ERR("Could not open BSV file for playback, path : \"%s\".\n", path);
       return false;
    }
 
-   if (fread(header, sizeof(uint32_t), 4, handle->file) != 4)
-   {
-      RARCH_ERR("%s\n", msg_hash_to_str(MSG_COULD_NOT_READ_MOVIE_HEADER));
-      return false;
-   }
+   handle->file              = file;
+   handle->playback          = true;
 
+   intfstream_read(handle->file, header, sizeof(uint32_t) * 4);
    /* Compatibility with old implementation that
     * used incorrect documentation. */
    if (swap_if_little32(header[MAGIC_INDEX]) != BSV_MAGIC
@@ -101,13 +99,21 @@ static bool init_playback(bsv_movie_t *handle, const char *path)
       return false;
    }
 
-   content_get_crc(&content_crc_ptr);
+   content_crc               = content_get_crc();
 
-   if (swap_if_big32(header[CRC_INDEX]) != *content_crc_ptr)
-      RARCH_WARN("%s.\n", msg_hash_to_str(MSG_CRC32_CHECKSUM_MISMATCH));
+   if (content_crc != 0)
+      if (swap_if_big32(header[CRC_INDEX]) != content_crc)
+         RARCH_WARN("%s.\n", msg_hash_to_str(MSG_CRC32_CHECKSUM_MISMATCH));
 
    state_size = swap_if_big32(header[STATE_SIZE_INDEX]);
 
+#if 0
+   RARCH_ERR("----- debug %u -----\n", header[0]);
+   RARCH_ERR("----- debug %u -----\n", header[1]);
+   RARCH_ERR("----- debug %u -----\n", header[2]);
+   RARCH_ERR("----- debug %u -----\n", header[3]);
+#endif
+
    if (state_size)
    {
       retro_ctx_size_info_t info;
@@ -119,8 +125,8 @@ static bool init_playback(bsv_movie_t *handle, const char *path)
 
       handle->state      = buf;
       handle->state_size = state_size;
-
-      if (fread(handle->state, 1, state_size, handle->file) != state_size)
+      if (intfstream_read(handle->file,
+               handle->state, state_size) != state_size)
       {
          RARCH_ERR("%s\n", msg_hash_to_str(MSG_COULD_NOT_READ_STATE_FROM_MOVIE));
          return false;
@@ -144,34 +150,44 @@ static bool init_playback(bsv_movie_t *handle, const char *path)
    return true;
 }
 
-static bool init_record(bsv_movie_t *handle, const char *path)
+static bool bsv_movie_init_record(bsv_movie_t *handle, const char *path)
 {
    retro_ctx_size_info_t info;
-   uint32_t state_size;
+   uint32_t state_size       = 0;
+   uint32_t content_crc      = 0;
    uint32_t header[4]        = {0};
-   uint32_t *content_crc_ptr = NULL;
+   intfstream_t *file        = intfstream_open_file(path,
+         RETRO_VFS_FILE_ACCESS_WRITE,
+         RETRO_VFS_FILE_ACCESS_HINT_NONE);
 
-   handle->file       = fopen(path, "wb");
-   if (!handle->file)
+   if (!file)
    {
       RARCH_ERR("Could not open BSV file for recording, path : \"%s\".\n", path);
       return false;
    }
 
-   content_get_crc(&content_crc_ptr);
+   handle->file             = file;
+
+   content_crc              = content_get_crc();
 
    /* This value is supposed to show up as
     * BSV1 in a HEX editor, big-endian. */
    header[MAGIC_INDEX]      = swap_if_little32(BSV_MAGIC);
-   header[CRC_INDEX]        = swap_if_big32(*content_crc_ptr);
+   header[CRC_INDEX]        = swap_if_big32(content_crc);
 
    core_serialize_size(&info);
 
-   state_size               = info.size;
+   state_size               = (unsigned)info.size;
 
    header[STATE_SIZE_INDEX] = swap_if_big32(state_size);
+#if 0
+   RARCH_ERR("----- debug %u -----\n", header[0]);
+   RARCH_ERR("----- debug %u -----\n", header[1]);
+   RARCH_ERR("----- debug %u -----\n", header[2]);
+   RARCH_ERR("----- debug %u -----\n", header[3]);
+#endif
 
-   fwrite(header, 4, sizeof(uint32_t), handle->file);
+   intfstream_write(handle->file, header, 4 * sizeof(uint32_t));
 
    handle->min_file_pos     = sizeof(header) + state_size;
    handle->state_size       = state_size;
@@ -189,7 +205,8 @@ static bool init_record(bsv_movie_t *handle, const char *path)
 
       core_serialize(&serial_info);
 
-      fwrite(handle->state, 1, state_size, handle->file);
+      intfstream_write(handle->file,
+            handle->state, state_size);
    }
 
    return true;
@@ -200,33 +217,35 @@ static void bsv_movie_free(bsv_movie_t *handle)
    if (!handle)
       return;
 
-   if (handle->file)
-      fclose(handle->file);
+   intfstream_close(handle->file);
+   free(handle->file);
+
    free(handle->state);
    free(handle->frame_pos);
    free(handle);
 }
 
-static bsv_movie_t *bsv_movie_init(const char *path,
+static bsv_movie_t *bsv_movie_init_internal(const char *path,
       enum rarch_movie_type type)
 {
    size_t *frame_pos   = NULL;
    bsv_movie_t *handle = (bsv_movie_t*)calloc(1, sizeof(*handle));
+
    if (!handle)
       return NULL;
 
    if (type == RARCH_MOVIE_PLAYBACK)
    {
-      if (!init_playback(handle, path))
+      if (!bsv_movie_init_playback(handle, path))
          goto error;
    }
-   else if (!init_record(handle, path))
+   else if (!bsv_movie_init_record(handle, path))
       goto error;
 
-   /* Just pick something really large 
+   /* Just pick something really large
     * ~1 million frames rewind should do the trick. */
    if (!(frame_pos = (size_t*)calloc((1 << 20), sizeof(size_t))))
-      goto error; 
+      goto error;
 
    handle->frame_pos       = frame_pos;
 
@@ -243,33 +262,35 @@ error:
 /* Used for rewinding while playback/record. */
 void bsv_movie_set_frame_start(void)
 {
-   bsv_movie_t *handle = bsv_movie_state.movie;
-   if (handle)
-      handle->frame_pos[handle->frame_ptr] = ftell(handle->file);
+   if (bsv_movie_state_handle)
+      bsv_movie_state_handle->frame_pos[bsv_movie_state_handle->frame_ptr]
+         = intfstream_tell(bsv_movie_state_handle->file);
 }
 
 void bsv_movie_set_frame_end(void)
 {
-   bsv_movie_t *handle = bsv_movie_state.movie;
-   if (!handle)
+   if (!bsv_movie_state_handle)
       return;
 
-   handle->frame_ptr    = (handle->frame_ptr + 1) & handle->frame_mask;
+   bsv_movie_state_handle->frame_ptr    =
+      (bsv_movie_state_handle->frame_ptr + 1)
+      & bsv_movie_state_handle->frame_mask;
 
-   handle->first_rewind = !handle->did_rewind;
-   handle->did_rewind   = false;
+   bsv_movie_state_handle->first_rewind =
+      !bsv_movie_state_handle->did_rewind;
+   bsv_movie_state_handle->did_rewind   = false;
 }
 
 static void bsv_movie_frame_rewind(bsv_movie_t *handle)
 {
    handle->did_rewind = true;
 
-   if (     (handle->frame_ptr <= 1) 
+   if (     (handle->frame_ptr <= 1)
          && (handle->frame_pos[0] == handle->min_file_pos))
    {
       /* If we're at the beginning... */
       handle->frame_ptr = 0;
-      fseek(handle->file, handle->min_file_pos, SEEK_SET);
+      intfstream_seek(handle->file, (int)handle->min_file_pos, SEEK_SET);
    }
    else
    {
@@ -281,10 +302,11 @@ static void bsv_movie_frame_rewind(bsv_movie_t *handle)
        * plus another. */
       handle->frame_ptr = (handle->frame_ptr -
             (handle->first_rewind ? 1 : 2)) & handle->frame_mask;
-      fseek(handle->file, handle->frame_pos[handle->frame_ptr], SEEK_SET);
+      intfstream_seek(handle->file,
+            (int)handle->frame_pos[handle->frame_ptr], SEEK_SET);
    }
 
-   if (ftell(handle->file) <= (long)handle->min_file_pos)
+   if (intfstream_tell(handle->file) <= (long)handle->min_file_pos)
    {
       /* We rewound past the beginning. */
 
@@ -295,28 +317,30 @@ static void bsv_movie_frame_rewind(bsv_movie_t *handle)
          /* If recording, we simply reset
           * the starting point. Nice and easy. */
 
-         fseek(handle->file, 4 * sizeof(uint32_t), SEEK_SET);
+         intfstream_seek(handle->file, 4 * sizeof(uint32_t), SEEK_SET);
 
          serial_info.data = handle->state;
          serial_info.size = handle->state_size;
 
          core_serialize(&serial_info);
 
-         fwrite(handle->state, 1, handle->state_size, handle->file);
+         intfstream_write(handle->file, handle->state, handle->state_size);
       }
       else
-         fseek(handle->file, handle->min_file_pos, SEEK_SET);
+         intfstream_seek(handle->file, (int)handle->min_file_pos, SEEK_SET);
    }
 }
 
-static void bsv_movie_init_state(void)
+bool bsv_movie_init(void)
 {
-   settings_t *settings = config_get_ptr();
+   bool set_granularity = false;
+   bool ret             = true;
 
-   if (bsv_movie_ctl(BSV_MOVIE_CTL_START_PLAYBACK, NULL))
+   if (bsv_movie_state.movie_start_playback)
    {
-      if (!(bsv_movie_init_handle(bsv_movie_state.movie_start_path,
-                  RARCH_MOVIE_PLAYBACK)))
+      ret = bsv_movie_init_handle(bsv_movie_state.movie_start_path,
+                  RARCH_MOVIE_PLAYBACK);
+      if (!ret)
       {
          RARCH_ERR("%s: \"%s\".\n",
                msg_hash_to_str(MSG_FAILED_TO_LOAD_MOVIE_FILE),
@@ -328,9 +352,10 @@ static void bsv_movie_init_state(void)
       runloop_msg_queue_push(msg_hash_to_str(MSG_STARTING_MOVIE_PLAYBACK),
             2, 180, false);
       RARCH_LOG("%s.\n", msg_hash_to_str(MSG_STARTING_MOVIE_PLAYBACK));
-      settings->rewind_granularity = 1;
+
+      set_granularity = true;
    }
-   else if (bsv_movie_ctl(BSV_MOVIE_CTL_START_RECORDING, NULL))
+   else if (bsv_movie_state.movie_start_recording)
    {
       char msg[256];
       snprintf(msg, sizeof(msg),
@@ -338,28 +363,39 @@ static void bsv_movie_init_state(void)
             msg_hash_to_str(MSG_STARTING_MOVIE_RECORD_TO),
             bsv_movie_state.movie_start_path);
 
-      if (!(bsv_movie_init_handle(bsv_movie_state.movie_start_path,
-                  RARCH_MOVIE_RECORD)))
+      ret = bsv_movie_init_handle(bsv_movie_state.movie_start_path,
+                  RARCH_MOVIE_RECORD);
+      if (!ret)
       {
          runloop_msg_queue_push(
                msg_hash_to_str(MSG_FAILED_TO_START_MOVIE_RECORD),
                1, 180, true);
-         RARCH_ERR("%s.\n", msg_hash_to_str(MSG_FAILED_TO_START_MOVIE_RECORD));
-         return;
+         RARCH_ERR("%s.\n",
+               msg_hash_to_str(MSG_FAILED_TO_START_MOVIE_RECORD));
+         return ret;
       }
 
       runloop_msg_queue_push(msg, 1, 180, true);
       RARCH_LOG("%s \"%s\".\n",
             msg_hash_to_str(MSG_STARTING_MOVIE_RECORD_TO),
             bsv_movie_state.movie_start_path);
-      settings->rewind_granularity = 1;
+
+      set_granularity = true;
    }
+
+   if (set_granularity)
+   {
+      settings_t *settings = config_get_ptr();
+      configuration_set_uint(settings,
+            settings->uints.rewind_granularity, 1);
+   }
+
+   return ret;
 }
 
 bool bsv_movie_get_input(int16_t *bsv_data)
 {
-   bsv_movie_t *handle = bsv_movie_state.movie;
-   if (fread(bsv_data, sizeof(int16_t), 1, handle->file) != 1)
+   if (intfstream_read(bsv_movie_state_handle->file, bsv_data, 1) != 1)
       return false;
 
    *bsv_data = swap_if_big16(*bsv_data);
@@ -367,26 +403,33 @@ bool bsv_movie_get_input(int16_t *bsv_data)
    return true;
 }
 
+bool bsv_movie_is_playback_on(void)
+{
+   return bsv_movie_state_handle && bsv_movie_state.movie_playback;
+}
+
+bool bsv_movie_is_playback_off(void)
+{
+   return bsv_movie_state_handle && !bsv_movie_state.movie_playback;
+}
+
+bool bsv_movie_is_end_of_file(void)
+{
+   return bsv_movie_state.movie_end && bsv_movie_state.eof_exit;
+}
+
 bool bsv_movie_ctl(enum bsv_ctl_state state, void *data)
 {
    switch (state)
    {
       case BSV_MOVIE_CTL_IS_INITED:
-         return bsv_movie_state.movie;
-      case BSV_MOVIE_CTL_PLAYBACK_ON:
-         return bsv_movie_state.movie && bsv_movie_state.movie_playback;
-      case BSV_MOVIE_CTL_PLAYBACK_OFF:
-         return bsv_movie_state.movie && !bsv_movie_state.movie_playback;
-      case BSV_MOVIE_CTL_START_RECORDING:
-         return bsv_movie_state.movie_start_recording;
+         return (bsv_movie_state_handle != NULL);
       case BSV_MOVIE_CTL_SET_START_RECORDING:
          bsv_movie_state.movie_start_recording = true;
          break;
       case BSV_MOVIE_CTL_UNSET_START_RECORDING:
          bsv_movie_state.movie_start_recording = false;
          break;
-      case BSV_MOVIE_CTL_START_PLAYBACK:
-         return bsv_movie_state.movie_start_playback;
       case BSV_MOVIE_CTL_SET_START_PLAYBACK:
          bsv_movie_state.movie_start_playback = true;
          break;
@@ -396,8 +439,6 @@ bool bsv_movie_ctl(enum bsv_ctl_state state, void *data)
       case BSV_MOVIE_CTL_SET_END_EOF:
          bsv_movie_state.eof_exit = true;
          break;
-      case BSV_MOVIE_CTL_END_EOF:
-         return bsv_movie_state.movie_end && bsv_movie_state.eof_exit;
       case BSV_MOVIE_CTL_SET_END:
          bsv_movie_state.movie_end = true;
          break;
@@ -407,24 +448,15 @@ bool bsv_movie_ctl(enum bsv_ctl_state state, void *data)
       case BSV_MOVIE_CTL_UNSET_PLAYBACK:
          bsv_movie_state.movie_playback = false;
          break;
-      case BSV_MOVIE_CTL_DEINIT:
-         if (bsv_movie_state.movie)
-            bsv_movie_free(bsv_movie_state.movie);
-         bsv_movie_state.movie = NULL;
-         break;
-      case BSV_MOVIE_CTL_INIT:
-         bsv_movie_init_state();
-         break;
       case BSV_MOVIE_CTL_FRAME_REWIND:
-         bsv_movie_frame_rewind(bsv_movie_state.movie);
+         bsv_movie_frame_rewind(bsv_movie_state_handle);
          break;
       case BSV_MOVIE_CTL_SET_INPUT:
          {
             int16_t *bsv_data = (int16_t*)data;
-            bsv_movie_t *handle = bsv_movie_state.movie;
 
             *bsv_data = swap_if_big16(*bsv_data);
-            fwrite(bsv_data, sizeof(int16_t), 1, handle->file);
+            intfstream_write(bsv_movie_state_handle->file, bsv_data, 1);
          }
          break;
       case BSV_MOVIE_CTL_NONE:
@@ -435,11 +467,6 @@ bool bsv_movie_ctl(enum bsv_ctl_state state, void *data)
    return true;
 }
 
-const char *bsv_movie_get_path(void)
-{
-   return bsv_movie_state.movie_path;
-}
-
 void bsv_movie_set_path(const char *path)
 {
    strlcpy(bsv_movie_state.movie_path,
@@ -455,26 +482,34 @@ void bsv_movie_set_start_path(const char *path)
 bool bsv_movie_init_handle(const char *path,
       enum rarch_movie_type type)
 {
-   bsv_movie_state.movie = bsv_movie_init(path, type);
-   if (!bsv_movie_state.movie)
+   bsv_movie_t *state     = bsv_movie_init_internal(path, type);
+   if (!state)
       return false;
+
+   bsv_movie_state_handle = state;
    return true;
 }
 
-/* Checks if movie is being played back. */
-static bool runloop_check_movie_playback(void)
+void bsv_movie_deinit(void)
 {
-   if (!bsv_movie_state.movie_end)
-      return false;
+   if (!bsv_movie_state_handle)
+      return;
 
+   bsv_movie_free(bsv_movie_state_handle);
+   bsv_movie_state_handle = NULL;
+}
+
+/* Checks if movie is being played back. */
+static bool bsv_movie_check_movie_playback(void)
+{
    runloop_msg_queue_push(
          msg_hash_to_str(MSG_MOVIE_PLAYBACK_ENDED), 2, 180, false);
    RARCH_LOG("%s\n", msg_hash_to_str(MSG_MOVIE_PLAYBACK_ENDED));
 
    command_event(CMD_EVENT_BSV_MOVIE_DEINIT, NULL);
 
-   bsv_movie_ctl(BSV_MOVIE_CTL_UNSET_END, NULL);
-   bsv_movie_ctl(BSV_MOVIE_CTL_UNSET_PLAYBACK, NULL);
+   bsv_movie_state.movie_end      = false;
+   bsv_movie_state.movie_playback = false;
 
    return true;
 }
@@ -482,7 +517,7 @@ static bool runloop_check_movie_playback(void)
 /* Checks if movie is being recorded. */
 static bool runloop_check_movie_record(void)
 {
-   if (!bsv_movie_ctl(BSV_MOVIE_CTL_IS_INITED, NULL))
+   if (!bsv_movie_state_handle)
       return false;
 
    runloop_msg_queue_push(
@@ -496,20 +531,19 @@ static bool runloop_check_movie_record(void)
 
 static bool runloop_check_movie_init(void)
 {
-   char msg[128]              = {0};
-   char path[PATH_MAX_LENGTH] = {0};
+   char msg[128], path[PATH_MAX_LENGTH];
    settings_t *settings       = config_get_ptr();
 
-   if (bsv_movie_ctl(BSV_MOVIE_CTL_IS_INITED, NULL))
-      return false;
+   msg[0] = path[0]           = '\0';
 
-   settings->rewind_granularity = 1;
+   configuration_set_uint(settings, settings->uints.rewind_granularity, 1);
 
-   if (settings->state_slot > 0)
+   if (settings->ints.state_slot > 0)
       snprintf(path, sizeof(path), "%s%d",
-            bsv_movie_get_path(), settings->state_slot);
+            bsv_movie_state.movie_path,
+            settings->ints.state_slot);
    else
-      strlcpy(path, bsv_movie_get_path(), sizeof(path));
+      strlcpy(path, bsv_movie_state.movie_path, sizeof(path));
 
    strlcat(path,
          file_path_str(FILE_PATH_BSV_EXTENSION),
@@ -521,33 +555,35 @@ static bool runloop_check_movie_init(void)
 
    bsv_movie_init_handle(path, RARCH_MOVIE_RECORD);
 
-   if (!bsv_movie_ctl(BSV_MOVIE_CTL_IS_INITED, NULL))
-      return false;
-
-   if (bsv_movie_ctl(BSV_MOVIE_CTL_IS_INITED, NULL))
-   {
-      runloop_msg_queue_push(msg, 2, 180, true);
-      RARCH_LOG("%s \"%s\".\n",
-            msg_hash_to_str(MSG_STARTING_MOVIE_RECORD_TO),
-            path);
-   }
-   else
+   if (!bsv_movie_state_handle)
    {
       runloop_msg_queue_push(
             msg_hash_to_str(MSG_FAILED_TO_START_MOVIE_RECORD),
             2, 180, true);
       RARCH_ERR("%s\n",
             msg_hash_to_str(MSG_FAILED_TO_START_MOVIE_RECORD));
+      return false;
    }
 
+   runloop_msg_queue_push(msg, 2, 180, true);
+   RARCH_LOG("%s \"%s\".\n",
+         msg_hash_to_str(MSG_STARTING_MOVIE_RECORD_TO),
+         path);
+
    return true;
 }
 
 bool bsv_movie_check(void)
 {
-   if (bsv_movie_ctl(BSV_MOVIE_CTL_PLAYBACK_ON, NULL))
-      return runloop_check_movie_playback();
-   if (!bsv_movie_ctl(BSV_MOVIE_CTL_IS_INITED, NULL))
+   if (!bsv_movie_state_handle)
       return runloop_check_movie_init();
+
+   if (bsv_movie_state.movie_playback)
+   {
+      if (!bsv_movie_state.movie_end)
+         return false;
+      return bsv_movie_check_movie_playback();
+   }
+
    return runloop_check_movie_record();
 }
