@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- * 
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -44,26 +44,25 @@ enum bsv_ctl_state
 {
    BSV_MOVIE_CTL_NONE = 0,
    BSV_MOVIE_CTL_IS_INITED,
-   BSV_MOVIE_CTL_PLAYBACK_ON,
-   BSV_MOVIE_CTL_PLAYBACK_OFF,
    BSV_MOVIE_CTL_SET_INPUT,
    BSV_MOVIE_CTL_SET_START_RECORDING,
    BSV_MOVIE_CTL_UNSET_START_RECORDING,
-   BSV_MOVIE_CTL_START_RECORDING,
    BSV_MOVIE_CTL_SET_START_PLAYBACK,
    BSV_MOVIE_CTL_UNSET_START_PLAYBACK,
-   BSV_MOVIE_CTL_START_PLAYBACK,
    BSV_MOVIE_CTL_UNSET_PLAYBACK,
    BSV_MOVIE_CTL_FRAME_REWIND,
-   BSV_MOVIE_CTL_DEINIT,
-   BSV_MOVIE_CTL_INIT,
-   BSV_MOVIE_CTL_END_EOF,
    BSV_MOVIE_CTL_SET_END_EOF,
    BSV_MOVIE_CTL_SET_END,
    BSV_MOVIE_CTL_UNSET_END
 };
 
-const char *bsv_movie_get_path(void);
+void bsv_movie_deinit(void);
+
+bool bsv_movie_init(void);
+
+bool bsv_movie_is_playback_on(void);
+
+bool bsv_movie_is_playback_off(void);
 
 void bsv_movie_set_path(const char *path);
 
@@ -75,6 +74,8 @@ void bsv_movie_set_frame_end(void);
 
 bool bsv_movie_get_input(int16_t *bsv_data);
 
+bool bsv_movie_is_end_of_file(void);
+
 bool bsv_movie_ctl(enum bsv_ctl_state state, void *data);
 
 bool bsv_movie_check(void);
