@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- *  Copyright (C) 2016 - Brad Parker
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *  Copyright (C) 2016-2017 - Brad Parker
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -29,6 +29,12 @@
 
 RETRO_BEGIN_DECLS
 
+enum msg_hash_action
+{
+   MSG_HASH_NONE = 0,
+   MSG_HASH_USER_LANGUAGE
+};
+
 enum msg_file_type
 {
    FILE_TYPE_NONE = 0,
@@ -121,9 +127,16 @@ enum msg_file_type
    FILE_TYPE_TGA,
    FILE_TYPE_BMP,
 
+   FILE_TYPE_MOD,
+   FILE_TYPE_S3M,
+   FILE_TYPE_XM,
+
    FILE_TYPE_CUE,
+   FILE_TYPE_GDI,
    FILE_TYPE_ISO,
    FILE_TYPE_LUTRO,
+   FILE_TYPE_CHD,
+   FILE_TYPE_WBFS,
 
    FILE_TYPE_DIRECT_LOAD,
 
@@ -141,7 +154,6 @@ enum msg_hash_enums
    MSG_SETTING_DISK_IN_TRAY,
    MSG_FAILED_TO_SET_DISK,
    MSG_NETPLAY_FAILED,
-   MSG_NETPLAY_USERS_HAS_FLIPPED,
    MSG_UNKNOWN_NETPLAY_COMMAND_RECEIVED,
    MSG_CONNECTING_TO_NETPLAY_HOST,
    MSG_NETPLAY_LAN_SCAN_COMPLETE,
@@ -149,7 +161,15 @@ enum msg_hash_enums
    MSG_WAITING_FOR_CLIENT,
    MSG_NETPLAY_YOU_HAVE_LEFT_THE_GAME,
    MSG_NETPLAY_YOU_HAVE_JOINED_AS_PLAYER_N,
-   MSG_NETPLAY_IMPLEMENTATIONS_DIFFER,
+   MSG_NETPLAY_YOU_HAVE_JOINED_WITH_INPUT_DEVICES_S,
+   MSG_NETPLAY_PLAYER_S_LEFT,
+   MSG_NETPLAY_S_HAS_JOINED_AS_PLAYER_N,
+   MSG_NETPLAY_S_HAS_JOINED_WITH_INPUT_DEVICES_S,
+   MSG_NETPLAY_NOT_RETROARCH,
+   MSG_NETPLAY_OUT_OF_DATE,
+   MSG_NETPLAY_DIFFERENT_VERSIONS,
+   MSG_NETPLAY_DIFFERENT_CORES,
+   MSG_NETPLAY_DIFFERENT_CORE_VERSIONS,
    MSG_NETPLAY_ENDIAN_DEPENDENT,
    MSG_NETPLAY_PLATFORM_DEPENDENT,
    MSG_NETPLAY_ENTER_PASSWORD,
@@ -159,15 +179,21 @@ enum msg_hash_enums
    MSG_NETPLAY_CLIENT_HANGUP,
    MSG_NETPLAY_CANNOT_PLAY_UNPRIVILEGED,
    MSG_NETPLAY_CANNOT_PLAY_NO_SLOTS,
+   MSG_NETPLAY_CANNOT_PLAY_NOT_AVAILABLE,
    MSG_NETPLAY_CANNOT_PLAY,
    MSG_NETPLAY_PEER_PAUSED,
    MSG_NETPLAY_CHANGED_NICK,
+   MSG_ADDED_TO_FAVORITES,
+   MSG_RESET_CORE_ASSOCIATION,
+   MSG_CORE_ASSOCIATION_RESET,
    MSG_AUTODETECT,
    MSG_AUDIO_VOLUME,
+   MSG_AUDIO_MIXER_VOLUME,
    MSG_LIBRETRO_FRONTEND,
    MSG_CAPABILITIES,
    MSG_DEVICE_CONFIGURED_IN_PORT,
    MSG_DEVICE_NOT_CONFIGURED,
+   MSG_DEVICE_NOT_CONFIGURED_FALLBACK,
    MSG_DEVICE_DISCONNECTED_FROM_PORT,
    MSG_NO_ARGUMENTS_SUPPLIED_AND_NO_MENU_BUILTIN,
    MSG_COMPILER,
@@ -274,9 +300,18 @@ enum msg_hash_enums
    MSG_INPUT_CHEAT,
    MSG_INPUT_PRESET_FILENAME,
    MSG_INPUT_CHEAT_FILENAME,
+   MSG_INPUT_RENAME_ENTRY,
+   MSG_INPUT_ENABLE_SETTINGS_PASSWORD,
+   MSG_INPUT_ENABLE_SETTINGS_PASSWORD_OK,
+   MSG_INPUT_ENABLE_SETTINGS_PASSWORD_NOK,
+   MSG_INPUT_KIOSK_MODE_PASSWORD,
+   MSG_INPUT_KIOSK_MODE_PASSWORD_OK,
+   MSG_INPUT_KIOSK_MODE_PASSWORD_NOK,
    MSG_REMAP_FILE_SAVED_SUCCESSFULLY,
+   MSG_REMAP_FILE_REMOVED_SUCCESSFULLY,
    MSG_SHADER_PRESET_SAVED_SUCCESSFULLY,
    MSG_ERROR_SAVING_REMAP_FILE,
+   MSG_ERROR_REMOVING_REMAP_FILE,
    MSG_ERROR_SAVING_SHADER_PRESET,
    MSG_FAILED_TO_CREATE_THE_DIRECTORY,
    MSG_ERROR_SAVING_CORE_OPTIONS_FILE,
@@ -318,6 +353,7 @@ enum msg_hash_enums
    MSG_REWINDING,
    MSG_SLOW_MOTION_REWIND,
    MSG_SLOW_MOTION,
+   MSG_FAST_FORWARD,
    MSG_REWIND_REACHED_END,
    MSG_FAILED_TO_START_MOVIE_RECORD,
    MSG_CHEEVOS_HARDCORE_MODE_ENABLE,
@@ -329,8 +365,10 @@ enum msg_hash_enums
    MSG_REDIRECTING_SAVESTATE_TO,
    MSG_REDIRECTING_SAVEFILE_TO,
    MSG_REDIRECTING_CHEATFILE_TO,
+   MSG_PREPARING_FOR_CONTENT_SCAN,
    MSG_SCANNING,
    MSG_SCANNING_OF_DIRECTORY_FINISHED,
+   MSG_SCANNING_OF_FILE_FINISHED,
    MSG_LOADED_STATE_FROM_SLOT,
    MSG_LOADED_STATE_FROM_SLOT_AUTO,
    MSG_REMOVING_TEMPORARY_CONTENT_FILE,
@@ -364,29 +402,51 @@ enum msg_hash_enums
    MSG_REMOVED_DISK_FROM_TRAY,
    MSG_FAILED_TO_REMOVE_DISK_FROM_TRAY,
    MSG_GOT_INVALID_DISK_INDEX,
+   MSG_INDEX_FILE,
    MSG_DOWNLOADING,
    MSG_EXTRACTING,
    MSG_EXTRACTING_FILE,
    MSG_NO_CONTENT_STARTING_DUMMY_CORE,
+   MSG_CONFIG_OVERRIDE_LOADED,
+   MSG_GAME_REMAP_FILE_LOADED,
+   MSG_CORE_REMAP_FILE_LOADED,
 
+   MENU_LABEL(ADD_TO_MIXER),
+   MENU_LABEL(ADD_TO_MIXER_AND_PLAY),
+   MENU_LABEL(ADD_TO_MIXER_AND_COLLECTION),
+   MENU_LABEL(ADD_TO_MIXER_AND_COLLECTION_AND_PLAY),
    MENU_ENUM_LABEL_MENU_TOGGLE,
 
+   MENU_LABEL(FILTER_BY_CURRENT_CORE),
+
    MENU_LABEL(NO_HISTORY_AVAILABLE),
 
+   MENU_ENUM_LABEL_VALUE_MIXER_ACTION_PLAY,
+   MENU_ENUM_LABEL_VALUE_MIXER_ACTION_PLAY_LOOPED,
+   MENU_ENUM_LABEL_VALUE_MIXER_ACTION_PLAY_SEQUENTIAL,
+   MENU_ENUM_LABEL_VALUE_MIXER_ACTION_STOP,
+   MENU_ENUM_LABEL_VALUE_MIXER_ACTION_REMOVE,
+   MENU_ENUM_LABEL_VALUE_MIXER_ACTION_VOLUME,
+
    MENU_ENUM_LABEL_VALUE_REMAP_FILE,
    MENU_ENUM_LABEL_VALUE_CHEAT_FILE,
    MENU_ENUM_LABEL_INPUT_HOTKEY_BIND_BEGIN,
    MENU_ENUM_LABEL_INPUT_HOTKEY_BIND_END   = MENU_ENUM_LABEL_INPUT_HOTKEY_BIND_BEGIN + RARCH_BIND_LIST_END,
 
    MENU_LABEL(MENU_WALLPAPER_OPACITY),
+   MENU_LABEL(MENU_FRAMEBUFFER_OPACITY),
 
    MENU_ENUM_LABEL_VALUE_CONFIG,
    MENU_ENUM_LABEL_VALUE_OVERLAY,
    MENU_ENUM_LABEL_VALUE_AUTO,
 
+   MENU_LABEL(MATERIALUI_ICONS_ENABLE),
+
    MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_MONOCHROME,
+   MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_MONOCHROME_INVERTED,
    MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_FLATUI,
    MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_RETROACTIVE,
+   MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_RETROSYSTEM,
    MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_PIXEL,
    MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_NEOACTIVE,
    MENU_ENUM_LABEL_VALUE_XMB_ICON_THEME_SYSTEMATIC,
@@ -402,13 +462,16 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_XMB_MENU_COLOR_THEME_UNDERSEA,
    MENU_ENUM_LABEL_VALUE_XMB_MENU_COLOR_THEME_VOLCANIC_RED,
    MENU_ENUM_LABEL_VALUE_XMB_MENU_COLOR_THEME_DARK,
+   MENU_ENUM_LABEL_VALUE_XMB_MENU_COLOR_THEME_LIGHT,
    MENU_ENUM_LABEL_VALUE_XMB_MENU_COLOR_THEME_PLAIN,
+   MENU_ENUM_LABEL_VALUE_XMB_MENU_COLOR_THEME_MORNING_BLUE,
 
    MENU_LABEL(SHADER_PIPELINE_RIBBON_SIMPLIFIED),
    MENU_LABEL(SHADER_PIPELINE_RIBBON),
    MENU_LABEL(SHADER_PIPELINE_SIMPLE_SNOW),
    MENU_LABEL(SHADER_PIPELINE_SNOW),
    MENU_LABEL(SHADER_PIPELINE_BOKEH),
+   MENU_LABEL(SHADER_PIPELINE_SNOWFLAKE),
 
    MENU_LABEL(MATERIALUI_MENU_HEADER_OPACITY),
    MENU_LABEL(MATERIALUI_MENU_FOOTER_OPACITY),
@@ -426,6 +489,7 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_PLAYLIST_COLLECTION_ENTRY,
 
    MENU_LABEL(CHEEVOS_UNLOCKED_ENTRY),
+   MENU_LABEL(CHEEVOS_UNLOCKED_ENTRY_HARDCORE),
    MENU_LABEL(CHEEVOS_LOCKED_ENTRY),
 
    MENU_ENUM_LABEL_SHADER_PARAMETERS_ENTRY,
@@ -509,6 +573,15 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_INPUT_JOYPAD_L3,
    MENU_ENUM_LABEL_VALUE_INPUT_JOYPAD_R3,
    MENU_ENUM_LABEL_VALUE_INPUT_KEY,
+   MENU_ENUM_LABEL_VALUE_INPUT_MOUSE_LEFT,
+   MENU_ENUM_LABEL_VALUE_INPUT_MOUSE_RIGHT,
+   MENU_ENUM_LABEL_VALUE_INPUT_MOUSE_MIDDLE,
+   MENU_ENUM_LABEL_VALUE_INPUT_MOUSE_BUTTON4,
+   MENU_ENUM_LABEL_VALUE_INPUT_MOUSE_BUTTON5,
+   MENU_ENUM_LABEL_VALUE_INPUT_MOUSE_WHEEL_UP,
+   MENU_ENUM_LABEL_VALUE_INPUT_MOUSE_WHEEL_DOWN,
+   MENU_ENUM_LABEL_VALUE_INPUT_MOUSE_HORIZ_WHEEL_UP,
+   MENU_ENUM_LABEL_VALUE_INPUT_MOUSE_HORIZ_WHEEL_DOWN,
    MENU_ENUM_LABEL_VALUE_INPUT_ANALOG_LEFT_X,
    MENU_ENUM_LABEL_VALUE_INPUT_ANALOG_LEFT_Y,
    MENU_ENUM_LABEL_VALUE_INPUT_ANALOG_RIGHT_X,
@@ -521,10 +594,23 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_INPUT_ANALOG_RIGHT_X_MINUS,
    MENU_ENUM_LABEL_VALUE_INPUT_ANALOG_RIGHT_Y_PLUS,
    MENU_ENUM_LABEL_VALUE_INPUT_ANALOG_RIGHT_Y_MINUS,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_TRIGGER,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_RELOAD,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_AUX_A,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_AUX_B,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_AUX_C,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_START,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_SELECT,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_DPAD_UP,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_DPAD_DOWN,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_DPAD_LEFT,
+   MENU_ENUM_LABEL_VALUE_INPUT_LIGHTGUN_DPAD_RIGHT,
    MENU_ENUM_LABEL_VALUE_INPUT_TURBO_ENABLE,
 
    MENU_ENUM_LABEL_VALUE_INPUT_META_FAST_FORWARD_KEY,
    MENU_ENUM_LABEL_VALUE_INPUT_META_FAST_FORWARD_HOLD_KEY,
+   MENU_ENUM_LABEL_VALUE_INPUT_META_SLOWMOTION_KEY,
+   MENU_ENUM_LABEL_VALUE_INPUT_META_SLOWMOTION_HOLD_KEY,
    MENU_ENUM_LABEL_VALUE_INPUT_META_LOAD_STATE_KEY,
    MENU_ENUM_LABEL_VALUE_INPUT_META_SAVE_STATE_KEY,
    MENU_ENUM_LABEL_VALUE_INPUT_META_FULLSCREEN_TOGGLE_KEY,
@@ -543,9 +629,7 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_INPUT_META_SCREENSHOT,
    MENU_ENUM_LABEL_VALUE_INPUT_META_MUTE,
    MENU_ENUM_LABEL_VALUE_INPUT_META_OSK,
-   MENU_ENUM_LABEL_VALUE_INPUT_META_NETPLAY_FLIP,
    MENU_ENUM_LABEL_VALUE_INPUT_META_NETPLAY_GAME_WATCH,
-   MENU_ENUM_LABEL_VALUE_INPUT_META_SLOWMOTION,
    MENU_ENUM_LABEL_VALUE_INPUT_META_ENABLE_HOTKEY,
    MENU_ENUM_LABEL_VALUE_INPUT_META_VOLUME_UP,
    MENU_ENUM_LABEL_VALUE_INPUT_META_VOLUME_DOWN,
@@ -555,6 +639,7 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_INPUT_META_DISK_PREV,
    MENU_ENUM_LABEL_VALUE_INPUT_META_GRAB_MOUSE_TOGGLE,
    MENU_ENUM_LABEL_VALUE_INPUT_META_GAME_FOCUS_TOGGLE,
+   MENU_ENUM_LABEL_VALUE_INPUT_META_UI_COMPANION_TOGGLE,
    MENU_ENUM_LABEL_VALUE_INPUT_META_MENU_TOGGLE,
 
    MENU_ENUM_LABEL_VALUE_INPUT_DEVICE_INDEX,
@@ -563,6 +648,7 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_INPUT_BIND_ALL,
    MENU_ENUM_LABEL_VALUE_INPUT_BIND_DEFAULT_ALL,
    MENU_ENUM_LABEL_VALUE_INPUT_SAVE_AUTOCONFIG,
+   MENU_ENUM_LABEL_VALUE_INPUT_MOUSE_INDEX,
 
    MENU_LABEL(INPUT_MAX_USERS),
    MENU_LABEL(INPUT_USER_BINDS),
@@ -578,6 +664,8 @@ enum msg_hash_enums
    MENU_LABEL(INPUT_OSK_OVERLAY_ENABLE),
    MENU_LABEL(INPUT_MENU_ENUM_TOGGLE_GAMEPAD_COMBO),
    MENU_LABEL(INPUT_OVERLAY_HIDE_IN_MENU),
+   MENU_LABEL(INPUT_OVERLAY_SHOW_PHYSICAL_INPUTS),
+   MENU_LABEL(INPUT_OVERLAY_SHOW_PHYSICAL_INPUTS_PORT),
    MENU_LABEL(INPUT_KEYBOARD_GAMEPAD_MAPPING_TYPE),
    MENU_LABEL(INPUT_SMALL_KEYBOARD_ENABLE),
    MENU_LABEL(INPUT_TOUCH_ENABLE),
@@ -588,11 +676,21 @@ enum msg_hash_enums
    MENU_LABEL(INPUT_UNIFIED_MENU_CONTROLS),
 
    /* Video */
+   MENU_LABEL(CRT_SWITCH_RESOLUTION),
+   MENU_LABEL(CRT_SWITCH_RESOLUTION_SUPER),
    MENU_LABEL(VIDEO_FONT_ENABLE),
    MENU_LABEL(VIDEO_FONT_PATH),
    MENU_LABEL(VIDEO_FONT_SIZE),
    MENU_LABEL(VIDEO_MESSAGE_POS_X),
    MENU_LABEL(VIDEO_MESSAGE_POS_Y),
+   MENU_LABEL(VIDEO_MESSAGE_COLOR_RED),
+   MENU_LABEL(VIDEO_MESSAGE_COLOR_GREEN),
+   MENU_LABEL(VIDEO_MESSAGE_COLOR_BLUE),
+   MENU_LABEL(VIDEO_MESSAGE_BGCOLOR_ENABLE),
+   MENU_LABEL(VIDEO_MESSAGE_BGCOLOR_RED),
+   MENU_LABEL(VIDEO_MESSAGE_BGCOLOR_GREEN),
+   MENU_LABEL(VIDEO_MESSAGE_BGCOLOR_BLUE),
+   MENU_LABEL(VIDEO_MESSAGE_BGCOLOR_OPACITY),
    MENU_LABEL(VIDEO_FILTER_FLICKER),
    MENU_LABEL(VIDEO_SOFT_FILTER),
    MENU_LABEL(VIDEO_MAX_SWAPCHAIN_IMAGES),
@@ -605,6 +703,9 @@ enum msg_hash_enums
    MENU_LABEL(VIDEO_WINDOWED_FULLSCREEN),
    MENU_LABEL(VIDEO_WINDOW_WIDTH),
    MENU_LABEL(VIDEO_WINDOW_HEIGHT),
+   MENU_LABEL(VIDEO_WINDOW_OPACITY),
+   MENU_LABEL(VIDEO_FULLSCREEN_X),
+   MENU_LABEL(VIDEO_FULLSCREEN_Y),
    MENU_LABEL(VIDEO_FORCE_SRGB_DISABLE),
    MENU_LABEL(VIDEO_ROTATION),
    MENU_LABEL(VIDEO_SCALE),
@@ -629,6 +730,7 @@ enum msg_hash_enums
    MENU_LABEL(VIDEO_WINDOW_SCALE),
    MENU_LABEL(VIDEO_REFRESH_RATE),
    MENU_LABEL(VIDEO_REFRESH_RATE_AUTO),
+   MENU_LABEL(VIDEO_REFRESH_RATE_POLLED),
 
    MENU_ENUM_LABEL_VALUE_DOWNLOAD_CORE,
 
@@ -638,13 +740,30 @@ enum msg_hash_enums
 
 
    /* Menu settings */
+   MENU_LABEL(MENU_SHOW_LOAD_CORE),
+   MENU_LABEL(MENU_SHOW_LOAD_CONTENT),
+   MENU_LABEL(MENU_SHOW_INFORMATION),
+   MENU_LABEL(MENU_SHOW_CONFIGURATIONS),
+   MENU_LABEL(MENU_SHOW_HELP),
+   MENU_LABEL(MENU_SHOW_QUIT_RETROARCH),
+   MENU_LABEL(MENU_SHOW_REBOOT),
+   MENU_LABEL(MENU_SHOW_ONLINE_UPDATER),
+   MENU_LABEL(MENU_SHOW_CORE_UPDATER),
+   MENU_LABEL(MENU_ENABLE_KIOSK_MODE),
+   MENU_LABEL(MENU_DISABLE_KIOSK_MODE),
+   MENU_LABEL(MENU_KIOSK_MODE_PASSWORD),
+   MENU_LABEL(RUN_MUSIC),
    MENU_LABEL(RUN),
    MENU_LABEL(MENU_WALLPAPER),
    MENU_LABEL(DYNAMIC_WALLPAPER),
    MENU_LABEL(PAUSE_NONACTIVE),
    MENU_LABEL(MOUSE_ENABLE),
    MENU_LABEL(POINTER_ENABLE),
+   MENU_LABEL(MENU_RGUI_BORDER_FILLER_ENABLE),
+   MENU_LABEL(MENU_RGUI_BACKGROUND_FILLER_THICKNESS_ENABLE),
+   MENU_LABEL(MENU_RGUI_BORDER_FILLER_THICKNESS_ENABLE),
    MENU_LABEL(MENU_LINEAR_FILTER),
+   MENU_LABEL(MENU_HORIZONTAL_ANIMATION),
    MENU_LABEL(NAVIGATION_WRAPAROUND),
    MENU_LABEL(SHOW_ADVANCED_SETTINGS),
    MENU_LABEL(THREADED_DATA_RUNLOOP_ENABLE),
@@ -652,26 +771,54 @@ enum msg_hash_enums
    MENU_LABEL(ENTRY_HOVER_COLOR),
    MENU_LABEL(XMB_ALPHA_FACTOR),
    MENU_LABEL(XMB_SCALE_FACTOR),
+   MENU_LABEL(MENU_FONT_COLOR_RED),
+   MENU_LABEL(MENU_FONT_COLOR_GREEN),
+   MENU_LABEL(MENU_FONT_COLOR_BLUE),
    MENU_LABEL(XMB_FONT),
+   MENU_LABEL(XMB_LAYOUT),
    MENU_LABEL(XMB_THEME),
+   MENU_LABEL(XMB_MAIN_MENU_ENABLE_SETTINGS),
    MENU_LABEL(XMB_MENU_COLOR_THEME),
    MENU_LABEL(XMB_SHADOWS_ENABLE),
-   MENU_LABEL(XMB_SHOW_SETTINGS),
-   MENU_LABEL(XMB_SHOW_IMAGES),
-   MENU_LABEL(XMB_SHOW_MUSIC),
-   MENU_LABEL(XMB_SHOW_VIDEO),
-   MENU_LABEL(XMB_SHOW_HISTORY),
-   MENU_LABEL(XMB_SHOW_ADD),
+   MENU_LABEL(CONTENT_SHOW_REWIND),
+   MENU_LABEL(CONTENT_SHOW_LATENCY),
+   MENU_LABEL(CONTENT_SHOW_OVERLAYS),
+   MENU_LABEL(CONTENT_SHOW_SETTINGS),
+   MENU_LABEL(CONTENT_SHOW_SETTINGS_PASSWORD),
+   MENU_LABEL(CONTENT_SHOW_FAVORITES),
+   MENU_LABEL(CONTENT_SHOW_IMAGES),
+   MENU_LABEL(CONTENT_SHOW_MUSIC),
+   MENU_LABEL(CONTENT_SHOW_VIDEO),
+   MENU_LABEL(CONTENT_SHOW_NETPLAY),
+   MENU_LABEL(CONTENT_SHOW_HISTORY),
+   MENU_LABEL(CONTENT_SHOW_ADD),
+   MENU_LABEL(CONTENT_SHOW_PLAYLISTS),
    MENU_LABEL(XMB_RIBBON_ENABLE),
    MENU_LABEL(THUMBNAILS),
+   MENU_LABEL(LEFT_THUMBNAILS),
+   MENU_LABEL(XMB_VERTICAL_THUMBNAILS),
    MENU_LABEL(TIMEDATE_ENABLE),
    MENU_LABEL(BATTERY_LEVEL_ENABLE),
    MENU_LABEL(MATERIALUI_MENU_COLOR_THEME),
+   MENU_LABEL(QUICK_MENU_OVERRIDE_OPTIONS),
+   MENU_LABEL(QUICK_MENU_SHOW_TAKE_SCREENSHOT),
+   MENU_LABEL(QUICK_MENU_SHOW_SAVE_LOAD_STATE),
+   MENU_LABEL(QUICK_MENU_SHOW_UNDO_SAVE_LOAD_STATE),
+   MENU_LABEL(QUICK_MENU_SHOW_ADD_TO_FAVORITES),
+   MENU_LABEL(QUICK_MENU_SHOW_OPTIONS),
+   MENU_LABEL(QUICK_MENU_SHOW_CONTROLS),
+   MENU_LABEL(QUICK_MENU_SHOW_CHEATS),
+   MENU_LABEL(QUICK_MENU_SHOW_SHADERS),
+   MENU_LABEL(QUICK_MENU_SHOW_SAVE_CORE_OVERRIDES),
+   MENU_LABEL(QUICK_MENU_SHOW_SAVE_GAME_OVERRIDES),
+   MENU_LABEL(QUICK_MENU_SHOW_INFORMATION),
 
    /* UI settings */
    MENU_LABEL(VIDEO_DISABLE_COMPOSITION),
    MENU_LABEL(UI_COMPANION_ENABLE),
    MENU_LABEL(UI_COMPANION_START_ON_BOOT),
+   MENU_LABEL(UI_COMPANION_TOGGLE),
+   MENU_LABEL(DESKTOP_MENU_ENABLE),
    MENU_LABEL(UI_MENUBAR_ENABLE),
 
    MENU_ENUM_LABEL_FILE_CONFIG,
@@ -704,6 +851,14 @@ enum msg_hash_enums
    MENU_LABEL(HISTORY_LIST_ENABLE),
    MENU_LABEL(CONTENT_HISTORY_SIZE),
    MENU_LABEL(PLAYLIST_ENTRY_REMOVE),
+   MENU_LABEL(PLAYLIST_ENTRY_RENAME),
+   MENU_LABEL(GOTO_FAVORITES),
+   MENU_LABEL(GOTO_MUSIC),
+   MENU_LABEL(GOTO_IMAGES),
+   MENU_LABEL(GOTO_VIDEO),
+   MENU_LABEL(ADD_TO_FAVORITES),
+   MENU_LABEL(ADD_TO_FAVORITES_PLAYLIST),
+   MENU_LABEL(RESET_CORE_ASSOCIATION),
    MENU_LABEL(MENU_THROTTLE_FRAMERATE),
    MENU_LABEL(NO_ACHIEVEMENTS_TO_DISPLAY),
    MENU_LABEL(NO_ENTRIES_TO_DISPLAY),
@@ -730,6 +885,8 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_BASIC_MENU_ENUM_CONTROLS_QUIT,
    MENU_ENUM_LABEL_VALUE_BASIC_MENU_ENUM_CONTROLS_TOGGLE_KEYBOARD,
 
+   MENU_LABEL(MENU_VIEWS_SETTINGS),
+   MENU_LABEL(QUICK_MENU_VIEWS_SETTINGS),
    MENU_LABEL(MENU_SETTINGS),
    MENU_LABEL(THUMBNAILS_UPDATER_LIST),
    MENU_LABEL(USER_INTERFACE_SETTINGS),
@@ -764,7 +921,11 @@ enum msg_hash_enums
 
    MENU_LABEL(ACCOUNTS_CHEEVOS_USERNAME),
    MENU_LABEL(CHEEVOS_HARDCORE_MODE_ENABLE),
+   MENU_LABEL(CHEEVOS_LEADERBOARDS_ENABLE),
+   MENU_LABEL(CHEEVOS_BADGES_ENABLE),
    MENU_LABEL(CHEEVOS_TEST_UNOFFICIAL),
+   MENU_LABEL(CHEEVOS_VERBOSE_ENABLE),
+   MENU_LABEL(CHEEVOS_AUTO_SCREENSHOT),
    MENU_LABEL(CHEEVOS_ENABLE),
    MENU_LABEL(CHEEVOS_DESCRIPTION),
    MENU_LABEL(CHEEVOS_UNLOCKED_ACHIEVEMENTS),
@@ -781,6 +942,7 @@ enum msg_hash_enums
    MENU_LABEL(HELP_LOADING_CONTENT),
    MENU_LABEL(HELP_LIST),
    MENU_LABEL(HELP_CONTROLS),
+   MENU_LABEL(HELP_CHEEVOS_DESCRIPTION),
 
    MENU_ENUM_LABEL_VALUE_EXTRACTING_PLEASE_WAIT,
 
@@ -789,6 +951,15 @@ enum msg_hash_enums
    MENU_LABEL(BROWSE_URL),
    MENU_LABEL(BROWSE_START),
    /* Deferred */
+   MENU_ENUM_LABEL_DEFERRED_MIXER_STREAM_SETTINGS_LIST,
+   MENU_ENUM_LABEL_DEFERRED_CONFIGURATIONS_LIST,
+   MENU_ENUM_LABEL_DEFERRED_FAVORITES_LIST,
+   MENU_ENUM_LABEL_DEFERRED_PLAYLIST_LIST,
+   MENU_ENUM_LABEL_DEFERRED_IMAGES_LIST,
+   MENU_ENUM_LABEL_DEFERRED_MUSIC_LIST,
+   MENU_ENUM_LABEL_DEFERRED_VIDEO_LIST,
+   MENU_ENUM_LABEL_DEFERRED_NETPLAY,
+   MENU_ENUM_LABEL_DEFERRED_MUSIC,
    MENU_ENUM_LABEL_DEFERRED_BROWSE_URL_START,
    MENU_ENUM_LABEL_DEFERRED_BROWSE_URL_LIST,
    MENU_ENUM_LABEL_DEFERRED_ARCHIVE_ACTION_DETECT_CORE,
@@ -828,6 +999,7 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_DEFERRED_RECORDING_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_PLAYLIST_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_INPUT_SETTINGS_LIST,
+   MENU_ENUM_LABEL_DEFERRED_LATENCY_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_DRIVER_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_VIDEO_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_CONFIGURATION_SETTINGS_LIST,
@@ -837,6 +1009,9 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_DEFERRED_ONSCREEN_DISPLAY_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_ONSCREEN_OVERLAY_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_ONSCREEN_NOTIFICATIONS_SETTINGS_LIST,
+   MENU_ENUM_LABEL_DEFERRED_MENU_VIEWS_SETTINGS_LIST,
+   MENU_ENUM_LABEL_DEFERRED_QUICK_MENU_VIEWS_SETTINGS_LIST,
+   MENU_ENUM_LABEL_DEFERRED_QUICK_MENU_OVERRIDE_OPTIONS,
    MENU_ENUM_LABEL_DEFERRED_MENU_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_USER_INTERFACE_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_MENU_FILE_BROWSER_SETTINGS_LIST,
@@ -851,6 +1026,7 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_DEFERRED_PRIVACY_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_LOGGING_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_AUDIO_SETTINGS_LIST,
+   MENU_ENUM_LABEL_DEFERRED_AUDIO_MIXER_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_CORE_SETTINGS_LIST,
    MENU_ENUM_LABEL_DEFERRED_USER_BINDS_LIST,
    MENU_ENUM_LABEL_DEFERRED_ACCOUNTS_CHEEVOS_LIST,
@@ -868,6 +1044,7 @@ enum msg_hash_enums
    MENU_LABEL(SCAN_FILE),
    MENU_LABEL(NETPLAY_REFRESH_ROOMS),
    MENU_LABEL(NETPLAY_ROOM_NICKNAME),
+   MENU_LABEL(NETPLAY_ROOM_NICKNAME_LAN),
    MENU_LABEL(ADD_CONTENT_LIST),
    MENU_LABEL(CONFIGURATIONS_LIST),
 
@@ -876,6 +1053,7 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_NEAREST,
    MENU_ENUM_LABEL_VALUE_UNKNOWN,
    MENU_ENUM_LABEL_VALUE_USER,
+   MENU_ENUM_LABEL_VALUE_KEYBOARD,
    MENU_ENUM_LABEL_VALUE_CHEAT,
    MENU_ENUM_LABEL_VALUE_SHADER,
    MENU_ENUM_LABEL_VALUE_DIRECTORY_CONTENT,
@@ -900,8 +1078,11 @@ enum msg_hash_enums
    MENU_LABEL(INFORMATION),
    MENU_LABEL(INFORMATION_LIST),
    MENU_LABEL(USE_BUILTIN_PLAYER),
+   MENU_LABEL(AUTOMATICALLY_ADD_CONTENT_TO_PLAYLIST),
+   MENU_LABEL(NETPLAY_SETTINGS),
    MENU_LABEL(CONTENT_SETTINGS),
    MENU_LABEL(LOAD_CONTENT_LIST),
+   MENU_LABEL(LOAD_CONTENT_SPECIAL),
    MENU_LABEL(NO_SETTINGS_FOUND),
    MENU_LABEL(NO_NETWORKS_FOUND),
    MENU_LABEL(NO_PERFORMANCE_COUNTERS),
@@ -911,14 +1092,14 @@ enum msg_hash_enums
 
    /* Netplay */
    MENU_LABEL(NETPLAY_ENABLE_HOST),
+   MENU_LABEL(NETPLAY_DISABLE_HOST),
    MENU_LABEL(NETPLAY_ENABLE_CLIENT),
    MENU_LABEL(NETPLAY_DISCONNECT),
-   MENU_LABEL(NETPLAY_SETTINGS),
    MENU_LABEL(NETPLAY_CONNECT_TO),
    MENU_LABEL(NO_NETPLAY_HOSTS_FOUND),
 
    MENU_ENUM_LABEL_RESET,
-   MENU_ENUM_LABEL_SLOWMOTION,
+   MENU_ENUM_LABEL_SLOWMOTION_HOLD,
    MENU_ENUM_LABEL_HOLD_FAST_FORWARD,
    MENU_ENUM_LABEL_CHEAT_TOGGLE,
    MENU_ENUM_LABEL_PAUSE_TOGGLE,
@@ -938,7 +1119,6 @@ enum msg_hash_enums
    MSG_APPLICATION_DIR,
    MSG_EXTERNAL_APPLICATION_DIR,
 
-   MENU_LABEL(ARCHIVE_MODE),
    MENU_LABEL(SHADER_OPTIONS),
    MENU_LABEL(USE_THIS_DIRECTORY),
    MENU_LABEL(CORE_OPTIONS),
@@ -948,22 +1128,29 @@ enum msg_hash_enums
 
    /* Audio */
    MENU_LABEL(AUDIO_ENABLE),
+   MENU_LABEL(AUDIO_ENABLE_MENU),
    MENU_LABEL(AUDIO_MAX_TIMING_SKEW),
    MENU_LABEL(AUDIO_OUTPUT_RATE),
    MENU_LABEL(AUDIO_DEVICE),
    MENU_LABEL(AUDIO_BLOCK_FRAMES),
    MENU_LABEL(AUDIO_DSP_PLUGIN),
    MENU_LABEL(AUDIO_MUTE),
+   MENU_LABEL(AUDIO_MIXER_MUTE),
    MENU_LABEL(AUDIO_SYNC),
    MENU_LABEL(AUDIO_VOLUME),
+   MENU_LABEL(AUDIO_MIXER_VOLUME),
    MENU_LABEL(AUDIO_RATE_CONTROL_DELTA),
    MENU_LABEL(AUDIO_LATENCY),
+   MENU_LABEL(AUDIO_RESAMPLER_QUALITY),
+   MENU_LABEL(AUDIO_WASAPI_EXCLUSIVE_MODE),
+   MENU_LABEL(AUDIO_WASAPI_FLOAT_FORMAT),
+   MENU_LABEL(AUDIO_WASAPI_SH_BUFFER_LENGTH),
+
    MENU_LABEL(SAVE_STATE),
    MENU_LABEL(LOAD_STATE),
    MENU_LABEL(UNDO_LOAD_STATE),
    MENU_LABEL(UNDO_SAVE_STATE),
 
-   MENU_LABEL(NETPLAY_FLIP_PLAYERS),
    MENU_LABEL(NETPLAY_GAME_WATCH),
    MENU_LABEL(CHEAT_INDEX_MINUS),
    MENU_LABEL(CHEAT_INDEX_PLUS),
@@ -971,6 +1158,8 @@ enum msg_hash_enums
    MENU_LABEL(SHADER_PREV),
    MENU_LABEL(FRAME_ADVANCE),
    MENU_LABEL(FPS_SHOW),
+   MENU_LABEL(STATISTICS_SHOW),
+   MENU_LABEL(FRAMECOUNT_SHOW),
    MENU_LABEL(MOVIE_RECORD_TOGGLE),
    MENU_ENUM_LABEL_L_X_PLUS,
    MENU_ENUM_LABEL_L_X_MINUS,
@@ -1015,8 +1204,11 @@ enum msg_hash_enums
    MENU_LABEL(SSH_ENABLE),
    MENU_LABEL(SAMBA_ENABLE),
    MENU_LABEL(BLUETOOTH_ENABLE),
-   MENU_LABEL(NETPLAY_CLIENT_SWAP_INPUT),
    MENU_LABEL(NETPLAY_DELAY_FRAMES),
+   MENU_LABEL(NETPLAY_PUBLIC_ANNOUNCE),
+   MENU_LABEL(NETPLAY_START_AS_SPECTATOR),
+   MENU_LABEL(NETPLAY_ALLOW_SLAVES),
+   MENU_LABEL(NETPLAY_REQUIRE_SLAVES),
    MENU_LABEL(NETPLAY_STATELESS_MODE),
    MENU_LABEL(NETPLAY_CHECK_FRAMES),
    MENU_LABEL(NETPLAY_INPUT_LATENCY_FRAMES_MIN),
@@ -1024,8 +1216,25 @@ enum msg_hash_enums
    MENU_LABEL(NETPLAY_SPECTATOR_MODE_ENABLE),
    MENU_LABEL(NETPLAY_TCP_UDP_PORT),
    MENU_LABEL(NETPLAY_NAT_TRAVERSAL),
+   MENU_LABEL(NETPLAY_REQUEST_DEVICE_I),
+   MENU_ENUM_LABEL_NETPLAY_REQUEST_DEVICE_1,
+   MENU_ENUM_LABEL_NETPLAY_REQUEST_DEVICE_LAST = MENU_ENUM_LABEL_NETPLAY_REQUEST_DEVICE_1 + MAX_USERS,
+   MENU_ENUM_LABEL_VALUE_NETPLAY_SHARE_NONE,
+   MENU_ENUM_LABEL_VALUE_NETPLAY_SHARE_NO_PREFERENCE,
+   MENU_LABEL(NETPLAY_SHARE_DIGITAL),
+   MENU_ENUM_LABEL_VALUE_NETPLAY_SHARE_DIGITAL_OR,
+   MENU_ENUM_LABEL_VALUE_NETPLAY_SHARE_DIGITAL_XOR,
+   MENU_ENUM_LABEL_VALUE_NETPLAY_SHARE_DIGITAL_VOTE,
+   MENU_LABEL(NETPLAY_SHARE_ANALOG),
+   MENU_ENUM_LABEL_VALUE_NETPLAY_SHARE_ANALOG_MAX,
+   MENU_ENUM_LABEL_VALUE_NETPLAY_SHARE_ANALOG_AVERAGE,
+
    MENU_LABEL(SORT_SAVEFILES_ENABLE),
    MENU_LABEL(SORT_SAVESTATES_ENABLE),
+   MENU_LABEL(SAVEFILES_IN_CONTENT_DIR_ENABLE),
+   MENU_LABEL(SAVESTATES_IN_CONTENT_DIR_ENABLE),
+   MENU_LABEL(SYSTEMFILES_IN_CONTENT_DIR_ENABLE),
+   MENU_LABEL(SCREENSHOTS_IN_CONTENT_DIR_ENABLE),
    MENU_LABEL(NETPLAY_IP_ADDRESS),
    MENU_LABEL(NETPLAY_PASSWORD),
    MENU_LABEL(NETPLAY_SPECTATE_PASSWORD),
@@ -1085,6 +1294,9 @@ enum msg_hash_enums
    MENU_LABEL(THUMBNAILS_DIRECTORY),
 
    MENU_LABEL(SLOWMOTION_RATIO),
+   MENU_LABEL(RUN_AHEAD_ENABLED),
+   MENU_LABEL(RUN_AHEAD_SECONDARY_INSTANCE),
+   MENU_LABEL(RUN_AHEAD_FRAMES),
    MENU_LABEL(TURBO),
 
    /* Privacy settings */
@@ -1146,6 +1358,7 @@ enum msg_hash_enums
    MENU_LABEL(HORIZONTAL_MENU),
    MENU_LABEL(SETTINGS_TAB),
    MENU_LABEL(HISTORY_TAB),
+   MENU_LABEL(FAVORITES_TAB),
    MENU_LABEL(ADD_TAB),
    MENU_LABEL(NETPLAY_TAB),
    MENU_LABEL(PLAYLISTS_TAB),
@@ -1215,6 +1428,8 @@ enum msg_hash_enums
    MENU_LABEL(SHUTDOWN),
    MENU_LABEL(REBOOT),
    MENU_LABEL(DISK_IMAGE_APPEND),
+   MENU_LABEL(SUBSYSTEM_ADD),
+   MENU_LABEL(SUBSYSTEM_LOAD),
    MENU_LABEL(CORE_LIST),
    MENU_LABEL(MANAGEMENT),
    MENU_LABEL(ONLINE),
@@ -1229,6 +1444,7 @@ enum msg_hash_enums
    MENU_LABEL(ACHIEVEMENT_LIST),
    MENU_LABEL(ACHIEVEMENT_LIST_HARDCORE),
    MENU_LABEL(CORE_INFORMATION),
+   MENU_LABEL(CORE_DELETE),
 
    MENU_LABEL(VIDEO_SHADER_PARAMETERS),
    MENU_LABEL(VIDEO_SHADER_PRESET_PARAMETERS),
@@ -1288,6 +1504,8 @@ enum msg_hash_enums
    MENU_LABEL(DRIVER_SETTINGS),
    MENU_LABEL(VIDEO_SETTINGS),
    MENU_LABEL(AUDIO_SETTINGS),
+   MENU_LABEL(AUDIO_MIXER_SETTINGS),
+   MENU_LABEL(LATENCY_SETTINGS),
    MENU_LABEL(CORE_SETTINGS),
    MENU_LABEL(CONFIGURATION_SETTINGS),
    MENU_LABEL(LOGGING_SETTINGS),
@@ -1303,6 +1521,7 @@ enum msg_hash_enums
    MENU_LABEL(CORE_UPDATER_SETTINGS),
    MENU_LABEL(LAKKA_SERVICES),
    MENU_LABEL(SHADER_APPLY_CHANGES),
+   MENU_LABEL(SHADER_WATCH_FOR_CHANGES),
    MENU_LABEL(SAVE_NEW_CONFIG),
    MENU_LABEL(ONSCREEN_DISPLAY_SETTINGS),
    MENU_LABEL(ONSCREEN_OVERLAY_SETTINGS),
@@ -1324,12 +1543,15 @@ enum msg_hash_enums
 
    MENU_LABEL(REMAP_FILE_SAVE_CORE),
    MENU_LABEL(REMAP_FILE_SAVE_GAME),
+   MENU_LABEL(REMAP_FILE_REMOVE_CORE),
+   MENU_LABEL(REMAP_FILE_REMOVE_GAME),
    MENU_LABEL(RESTART_CONTENT),
    MENU_LABEL(RESUME),
    MENU_LABEL(RESUME_CONTENT),
 
    MENU_LABEL(CHEAT_FILE_SAVE_AS),
    MENU_LABEL(DELETE_ENTRY),
+   MENU_LABEL(RENAME_ENTRY),
    MENU_LABEL(RESTART_RETROARCH),
 
    MENU_LABEL(TAKE_SCREENSHOT),
@@ -1339,9 +1561,11 @@ enum msg_hash_enums
    MENU_LABEL(VIDEO_SHADER_PRESET_SAVE_AS),
    MENU_LABEL(VIDEO_SHADER_PRESET_SAVE_CORE),
    MENU_LABEL(VIDEO_SHADER_PRESET_SAVE_GAME),
+   MENU_LABEL(VIDEO_SHADER_PRESET_SAVE_PARENT),
    MENU_LABEL(USER_LANGUAGE),
    MENU_LABEL(NETPLAY_NICKNAME),
    MENU_LABEL(VIDEO_VI_WIDTH),
+   MENU_LABEL(VIDEO_ASPECT_RATIO),
    MENU_LABEL(VIDEO_FORCE_ASPECT),
    MENU_LABEL(VIDEO_ASPECT_RATIO_AUTO),
    MENU_LABEL(VIDEO_ASPECT_RATIO_INDEX),
@@ -1369,6 +1593,7 @@ enum msg_hash_enums
 
    /* System information */
 
+   MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_LAKKA_VERSION,
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_DYNAMIC_SUPPORT,
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_BUILD_DATE,
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_GIT_VERSION,
@@ -1418,6 +1643,7 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_JACK_SUPPORT,
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_PULSEAUDIO_SUPPORT,
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_DSOUND_SUPPORT,
+   MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_WASAPI_SUPPORT,
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_XAUDIO2_SUPPORT,
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_ZLIB_SUPPORT,
    MENU_ENUM_LABEL_VALUE_SYSTEM_INFO_7ZIP_SUPPORT,
@@ -1473,7 +1699,8 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_LANG_GERMAN,
    MENU_ENUM_LABEL_VALUE_LANG_ITALIAN,
    MENU_ENUM_LABEL_VALUE_LANG_DUTCH,
-   MENU_ENUM_LABEL_VALUE_LANG_PORTUGUESE,
+   MENU_ENUM_LABEL_VALUE_LANG_PORTUGUESE_BRAZIL,
+   MENU_ENUM_LABEL_VALUE_LANG_PORTUGUESE_PORTUGAL,
    MENU_ENUM_LABEL_VALUE_LANG_RUSSIAN,
    MENU_ENUM_LABEL_VALUE_LANG_KOREAN,
    MENU_ENUM_LABEL_VALUE_LANG_CHINESE_TRADITIONAL,
@@ -1481,6 +1708,7 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_LANG_ESPERANTO,
    MENU_ENUM_LABEL_VALUE_LANG_POLISH,
    MENU_ENUM_LABEL_VALUE_LANG_VIETNAMESE,
+   MENU_ENUM_LABEL_VALUE_LANG_ARABIC,
 
    MENU_ENUM_LABEL_VALUE_NONE,
    MENU_ENUM_LABEL_VALUE_NO_INFORMATION_AVAILABLE,
@@ -1524,6 +1752,7 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_CB_LAKKA_DOWNLOAD,
    MENU_ENUM_LABEL_CB_LAKKA_LIST,
    MENU_ENUM_LABEL_CB_MENU_THUMBNAIL,
+   MENU_ENUM_LABEL_CB_MENU_LEFT_THUMBNAIL,
    MENU_ENUM_LABEL_CB_MENU_SAVESTATE_THUMBNAIL,
    MENU_ENUM_LABEL_CB_MENU_WALLPAPER,
    MENU_ENUM_LABEL_CB_THUMBNAILS_UPDATER_DOWNLOAD,
@@ -1539,6 +1768,12 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_CB_UPDATE_SHADERS_SLANG,
 
    /* Sublabels */
+   MENU_ENUM_SUBLABEL_MIXER_ACTION_PLAY,
+   MENU_ENUM_SUBLABEL_MIXER_ACTION_PLAY_LOOPED,
+   MENU_ENUM_SUBLABEL_MIXER_ACTION_PLAY_SEQUENTIAL,
+   MENU_ENUM_SUBLABEL_MIXER_ACTION_STOP,
+   MENU_ENUM_SUBLABEL_MIXER_ACTION_REMOVE,
+   MENU_ENUM_SUBLABEL_MIXER_ACTION_VOLUME,
    MENU_ENUM_SUBLABEL_INFORMATION_LIST_LIST,
    MENU_ENUM_SUBLABEL_SERVICES_SETTINGS,
 
@@ -1551,10 +1786,70 @@ enum msg_hash_enums
    MENU_ENUM_LABEL_VALUE_NETPLAY_LOAD_CONTENT_MANUALLY,
    MENU_ENUM_LABEL_VALUE_NETPLAY_COMPAT_CONTENT_FOUND,
 
+   MENU_LABEL(NETPLAY_USE_MITM_SERVER),
+   MENU_LABEL(NETPLAY_MITM_SERVER),
+   MENU_LABEL(VIDEO_WINDOW_SHOW_DECORATIONS),
+
+   MENU_ENUM_LABEL_VALUE_QT_INFO,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_FILE,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_FILE_LOAD_CORE,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_FILE_UNLOAD_CORE,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_FILE_EXIT,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_EDIT,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_EDIT_SEARCH,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_CLOSED_DOCKS,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS_SAVE_DOCK_POSITIONS,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS_SAVE_GEOMETRY,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS_SAVE_LAST_TAB,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS_THEME,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS_THEME_SYSTEM_DEFAULT,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS_THEME_DARK,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS_THEME_CUSTOM,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS_TITLE,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS_HIGHLIGHT_COLOR,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_VIEW_OPTIONS_SUGGEST_LOADED_CORE_FIRST,
+   MENU_ENUM_LABEL_VALUE_QT_MENU_DOCK_CONTENT_BROWSER,
+   MENU_ENUM_LABEL_VALUE_QT_THUMBNAIL_BOXART,
+   MENU_ENUM_LABEL_VALUE_QT_THUMBNAIL_SCREENSHOT,
+   MENU_ENUM_LABEL_VALUE_QT_THUMBNAIL_TITLE_SCREEN,
+   MENU_ENUM_LABEL_VALUE_QT_LOAD_CORE,
+   MENU_ENUM_LABEL_VALUE_QT_LOAD_CUSTOM_CORE,
+   MENU_ENUM_LABEL_VALUE_QT_LOADING_CORE,
+   MENU_ENUM_LABEL_VALUE_QT_NAME,
+   MENU_ENUM_LABEL_VALUE_QT_CORE_VERSION,
+   MENU_ENUM_LABEL_VALUE_QT_TAB_PLAYLISTS,
+   MENU_ENUM_LABEL_VALUE_QT_TAB_FILE_BROWSER,
+   MENU_ENUM_LABEL_VALUE_QT_TAB_FILE_BROWSER_TOP,
+   MENU_ENUM_LABEL_VALUE_QT_TAB_FILE_BROWSER_UP,
+   MENU_ENUM_LABEL_VALUE_QT_ALL_PLAYLISTS,
+   MENU_ENUM_LABEL_VALUE_QT_CORE,
+   MENU_ENUM_LABEL_VALUE_QT_CORE_INFO,
+   MENU_ENUM_LABEL_VALUE_QT_CORE_SELECTION_ASK,
+   MENU_ENUM_LABEL_VALUE_QT_INFORMATION,
+   MENU_ENUM_LABEL_VALUE_QT_WARNING,
+   MENU_ENUM_LABEL_VALUE_QT_ERROR,
+   MENU_ENUM_LABEL_VALUE_QT_RESTART_TO_TAKE_EFFECT,
+   MENU_ENUM_LABEL_VALUE_QT_LOG,
+   MENU_ENUM_LABEL_VALUE_QT_SCAN_FINISHED,
+   MENU_ENUM_LABEL_VALUE_QT_DONT_SHOW_AGAIN,
+   MENU_ENUM_LABEL_VALUE_QT_STOP,
+   MENU_ENUM_LABEL_VALUE_QT_ASSOCIATE_CORE,
+   MENU_ENUM_LABEL_VALUE_QT_HIDDEN_PLAYLISTS,
+   MENU_ENUM_LABEL_VALUE_QT_HIDE,
+   MENU_ENUM_LABEL_VALUE_QT_CHOOSE,
+   MENU_ENUM_LABEL_VALUE_QT_SELECT_COLOR,
+   MENU_ENUM_LABEL_VALUE_QT_SELECT_THEME,
+   MENU_ENUM_LABEL_VALUE_QT_CUSTOM_THEME,
+   MENU_ENUM_LABEL_VALUE_QT_FILE_PATH_IS_BLANK,
+   MENU_ENUM_LABEL_VALUE_QT_FILE_IS_EMPTY,
+   MENU_ENUM_LABEL_VALUE_QT_FILE_READ_OPEN_FAILED,
+   MENU_ENUM_LABEL_VALUE_QT_FILE_DOES_NOT_EXIST,
+
    MSG_LAST
 };
 
-
 /* Callback strings */
 
 #define CB_CORE_UPDATER_DOWNLOAD                                               0x7412da7dU
@@ -1562,7 +1857,6 @@ enum msg_hash_enums
 
 /* Deferred */
 
-#define MENU_LABEL_DEFERRED_BROWSE_URL_LIST                                    0x6358c2c4U
 #define MENU_LABEL_DEFERRED_THUMBNAILS_UPDATER_LIST                            0x364dfa2bU
 #define MENU_LABEL_DEFERRED_VIDEO_FILTER                                       0x966ad201U
 #define MENU_LABEL_DEFERRED_CORE_LIST_SET                                      0xa6d5fdb4U
@@ -1611,25 +1905,13 @@ enum msg_hash_enums
 
 #define MENU_LABEL_CHEEVOS_DESCRIPTION                                         0x7e00e0f5U
 
-/* Playlist settings */
-
-#define MENU_LABEL_PLAYLIST_SETTINGS_BEGIN                                     0x80a8d2cbU
-
 /* Accounts settings */
 
 #define MENU_LABEL_ACCOUNTS_RETRO_ACHIEVEMENTS                                 0xe6b7c16cU
 #define MENU_LABEL_ACCOUNTS_LIST                                               0x774c15a0U
 
-#define MENU_LABEL_VALUE_ACCOUNTS_LIST_END                                     0x3d559522U
-
-
 #define MENU_LABEL_DOWNLOADED_FILE_DETECT_CORE_LIST                            0xb4f82700U
 
-/* Scan values */
-#define MENU_LABEL_SCAN_THIS_DIRECTORY                                         0x6921b775U
-#define MENU_LABEL_SCAN_DIRECTORY                                              0x57de303eU
-#define MENU_LABEL_SCAN_FILE                                                   0xd5d1eee9U
-
 /* Online updater settings */
 
 #define MENU_LABEL_UPDATE_LAKKA                                                0x19b51eebU
@@ -1637,8 +1919,6 @@ enum msg_hash_enums
 /* Information settings */
 
 #define MENU_LABEL_INFORMATION_LIST                                            0x225e7606U
-#define MENU_LABEL_SYSTEM_INFORMATION                                          0x206ebf0fU
-#define MENU_LABEL_NETWORK_INFORMATION                                         0x73ae3cb4U
 
 #define MENU_LABEL_CONTENT_SETTINGS                                            0xe789f7f6U
 
@@ -1651,9 +1931,6 @@ enum msg_hash_enums
 
 /* Video settings */
 #define MENU_LABEL_VIDEO_FONT_PATH                                             0xd0de729eU
-#define MENU_LABEL_VIDEO_VI_WIDTH                                              0x6e4a6d3aU
-#define MENU_LABEL_VIDEO_VFILTER                                               0x664f8397U
-#define MENU_LABEL_VIDEO_GAMMA                                                 0x08a951beU
 
 #define MENU_LABEL_VIDEO_SHADER_PRESET_SAVE_AS                                 0x3d6e5ce5U
 #define MENU_LABEL_VIDEO_SHADER_DEFAULT_FILTER                                 0x4468cb1bU
@@ -1683,6 +1960,9 @@ enum msg_hash_enums
 #define MENU_LABEL_DISK_OPTIONS                                                0xc61ab5fbU
 #define MENU_LABEL_DISK_IMAGE_APPEND                                           0x5af7d709U
 
+/* Subsystem stuff */
+#define MENU_LABEL_SUBSYSTEM_ADD                                               0x576da67cU
+
 /* Menu settings */
 
 #define MENU_LABEL_MENU_WALLPAPER                                              0x3b84de01U
@@ -1745,7 +2025,6 @@ enum msg_hash_enums
 #define MENU_LABEL_FRONTEND_COUNTERS                                           0xe5696877U
 #define MENU_LABEL_CORE_COUNTERS                                               0x64cc83e0U
 #define MENU_LABEL_ACHIEVEMENT_LIST                                            0x7b90fc49U
-#define MENU_LABEL_ACHIEVEMENT_LIST_HARDCORE                                   0x7c632930U
 #define MENU_LABEL_CORE_INFORMATION                                            0xb638e0d3U
 #define MENU_LABEL_CORE_OPTIONS                                                0xf65e60f9U
 #define MENU_LABEL_SHADER_OPTIONS                                              0x1f7d2fc7U
@@ -1803,9 +2082,6 @@ enum msg_hash_enums
 #define MENU_LABEL_SETTINGS                                                    0x1304dc16U
 #define MENU_LABEL_HELP                                                        0x7c97d2eeU
 #define MENU_VALUE_HORIZONTAL_MENU                                             0x35761704U
-#define MENU_LABEL_QUIT_RETROARCH                                              0x84b0bc71U
-
-#define MENU_LABEL_BROWSE_URL_LIST                                             0xa7d1a004U
 
 const char *msg_hash_to_str(enum msg_hash_enums msg);
 
@@ -1829,8 +2105,14 @@ int menu_hash_get_help_it_enum(enum msg_hash_enums msg, char *s, size_t len);
 const char *msg_hash_to_str_jp(enum msg_hash_enums msg);
 int menu_hash_get_help_jp_enum(enum msg_hash_enums msg, char *s, size_t len);
 
-const char *msg_hash_to_str_pt(enum msg_hash_enums msg);
-int menu_hash_get_help_pt_enum(enum msg_hash_enums msg, char *s, size_t len);
+const char *msg_hash_to_str_ko(enum msg_hash_enums msg);
+int menu_hash_get_help_ko_enum(enum msg_hash_enums msg, char *s, size_t len);
+
+const char *msg_hash_to_str_pt_br(enum msg_hash_enums msg);
+int menu_hash_get_help_pt_br_enum(enum msg_hash_enums msg, char *s, size_t len);
+
+const char *msg_hash_to_str_pt_pt(enum msg_hash_enums msg);
+int menu_hash_get_help_pt_pt_enum(enum msg_hash_enums msg, char *s, size_t len);
 
 const char *msg_hash_to_str_pl(enum msg_hash_enums msg);
 int menu_hash_get_help_pl_enum(enum msg_hash_enums msg, char *s, size_t len);
@@ -1844,13 +2126,23 @@ int menu_hash_get_help_vn_enum(enum msg_hash_enums msg, char *s, size_t len);
 const char *msg_hash_to_str_chs(enum msg_hash_enums msg);
 int menu_hash_get_help_chs_enum(enum msg_hash_enums msg, char *s, size_t len);
 
+const char *msg_hash_to_str_cht(enum msg_hash_enums msg);
+int menu_hash_get_help_cht_enum(enum msg_hash_enums msg, char *s, size_t len);
+
 const char *msg_hash_to_str_us(enum msg_hash_enums msg);
 int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len);
 
+const char *msg_hash_to_str_ar(enum msg_hash_enums msg);
+int menu_hash_get_help_ar_enum(enum msg_hash_enums msg, char *s, size_t len);
+
 int menu_hash_get_help_enum(enum msg_hash_enums msg, char *s, size_t len);
 
 enum msg_file_type msg_hash_to_file_type(uint32_t hash);
 
+unsigned *msg_hash_get_uint(enum msg_hash_action type);
+
+void msg_hash_set_uint(enum msg_hash_action type, unsigned val);
+
 uint32_t msg_hash_calculate(const char *s);
 
 RETRO_END_DECLS
