@@ -16,8 +16,6 @@ MSG_HASH(MENU_ENUM_LABEL_ADD_TAB,
       "add_tab")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_TAB,
       "netplay_tab")
-MSG_HASH(MENU_ENUM_LABEL_ARCHIVE_MODE,
-      "archive_mode")
 MSG_HASH(MENU_ENUM_LABEL_ASSETS_DIRECTORY,
       "assets_directory")
 MSG_HASH(MENU_ENUM_LABEL_AUDIO_BLOCK_FRAMES,
@@ -52,6 +50,12 @@ MSG_HASH(MENU_ENUM_LABEL_AUDIO_SYNC,
       "audio_sync")
 MSG_HASH(MENU_ENUM_LABEL_AUDIO_VOLUME,
       "audio_volume")
+MSG_HASH(MENU_ENUM_LABEL_AUDIO_WASAPI_EXCLUSIVE_MODE,
+      "audio_wasapi_exclusive_mode")
+MSG_HASH(MENU_ENUM_LABEL_AUDIO_WASAPI_FLOAT_FORMAT,
+      "audio_wasapi_float_format")
+MSG_HASH(MENU_ENUM_LABEL_AUDIO_WASAPI_SH_BUFFER_LENGTH,
+      "audio_wasapi_sh_buffer_length")
 MSG_HASH(MENU_ENUM_LABEL_AUTOSAVE_INTERVAL,
       "autosave_interval")
 MSG_HASH(MENU_ENUM_LABEL_AUTO_OVERRIDES_ENABLE,
@@ -130,6 +134,10 @@ MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_ENABLE,
       "cheevos_enable")
 MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_HARDCORE_MODE_ENABLE,
       "cheevos_hardcore_mode_enable")
+MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_LEADERBOARDS_ENABLE,
+      "cheevos_leaderboards_enable")
+MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_BADGES_ENABLE,
+      "cheevos_badges_enable")
 MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_LOCKED_ACHIEVEMENTS,
       "cheevos_locked_achievements")
 MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_LOCKED_ENTRY,
@@ -142,8 +150,14 @@ MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_UNLOCKED_ACHIEVEMENTS,
       "cheevos_unlocked_achievements")
 MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_UNLOCKED_ENTRY,
       "cheevos_unlocked_entry")
+MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_UNLOCKED_ENTRY_HARDCORE,
+      "cheevos_unlocked_entry_hardcore")
 MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_USERNAME,
       "cheevos_username")
+MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_VERBOSE_ENABLE,
+      "cheevos_verbose_enable")
+MSG_HASH(MENU_ENUM_LABEL_CHEEVOS_AUTO_SCREENSHOT,
+      "cheevos_auto_screenshot")
 MSG_HASH(MENU_ENUM_LABEL_CLOSE_CONTENT,
       "unload_core")
 MSG_HASH(MENU_ENUM_LABEL_COLLECTION,
@@ -223,6 +237,22 @@ MSG_HASH(MENU_ENUM_LABEL_DATABASE_MANAGER,
       "database_manager")
 MSG_HASH(MENU_ENUM_LABEL_DATABASE_MANAGER_LIST,
       "database_manager_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CONFIGURATIONS_LIST,
+      "deferred_configurations_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_PLAYLIST_LIST,
+      "deferred_playlist_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_FAVORITES_LIST,
+      "deferred_favorites_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_IMAGES_LIST,
+      "deferred_images_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_MUSIC_LIST,
+      "deferred_music_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_VIDEO_LIST,
+      "deferred_video_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_NETPLAY,
+      "deferred_netplay")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_MUSIC,
+      "deferred_music")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_BROWSE_URL_START,
       "deferred_browse_url_start")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_BROWSE_URL_LIST,
@@ -241,6 +271,8 @@ MSG_HASH(MENU_ENUM_LABEL_DEFERRED_ARCHIVE_OPEN_DETECT_CORE,
       "deferred_archive_open_detect_core")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_AUDIO_SETTINGS_LIST,
       "deferred_audio_settings_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_AUDIO_MIXER_SETTINGS_LIST,
+      "deferred_audio_mixer_settings_list")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CONFIGURATION_SETTINGS_LIST,
       "deferred_configuration_settings_list")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CORE_CONTENT_DIRS_LIST,
@@ -271,6 +303,8 @@ MSG_HASH(MENU_ENUM_LABEL_DEFERRED_INPUT_HOTKEY_BINDS_LIST,
       "deferred_input_hotkey_binds")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_INPUT_SETTINGS_LIST,
       "deferred_input_settings_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_LATENCY_SETTINGS_LIST,
+      "deferred_latency_settings_list")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_LAKKA_LIST,
       "deferred_lakka_list")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_LAKKA_SERVICES_LIST,
@@ -281,6 +315,10 @@ MSG_HASH(MENU_ENUM_LABEL_DEFERRED_MENU_FILE_BROWSER_SETTINGS_LIST,
       "deferred_menu_file_browser_settings_list")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_MENU_SETTINGS_LIST,
       "deferred_menu_settings_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_MENU_VIEWS_SETTINGS_LIST,
+      "deferred_menu_views_settings_list")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_QUICK_MENU_VIEWS_SETTINGS_LIST,
+      "deferred_quick_menu_views_settings_list")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_NETWORK_SETTINGS_LIST,
       "deferred_network_settings_list")
 MSG_HASH(MENU_ENUM_LABEL_DEFERRED_ONSCREEN_DISPLAY_SETTINGS_LIST,
@@ -329,6 +367,10 @@ MSG_HASH(MENU_ENUM_LABEL_DISK_CYCLE_TRAY_STATUS,
       "disk_cycle_tray_status")
 MSG_HASH(MENU_ENUM_LABEL_DISK_IMAGE_APPEND,
       "disk_image_append")
+MSG_HASH(MENU_ENUM_LABEL_SUBSYSTEM_ADD,
+      "subsystem_add")
+MSG_HASH(MENU_ENUM_LABEL_SUBSYSTEM_LOAD,
+      "subsystem_load")
 MSG_HASH(MENU_ENUM_LABEL_DISK_OPTIONS,
       "core_disk_options")
 MSG_HASH(MENU_ENUM_LABEL_DOWNLOADED_FILE_DETECT_CORE_LIST,
@@ -385,8 +427,12 @@ MSG_HASH(MENU_ENUM_LABEL_FILE_BROWSER_SHADER,
       "file_browser_shader")
 MSG_HASH(MENU_ENUM_LABEL_FILE_BROWSER_SHADER_PRESET,
       "file_browser_shader_preset")
+MSG_HASH(MENU_ENUM_LABEL_FAVORITES_TAB,
+      "favorites_tab")
 MSG_HASH(MENU_ENUM_LABEL_FPS_SHOW,
       "fps_show")
+MSG_HASH(MENU_ENUM_LABEL_STATISTICS_SHOW,
+      "statistics_show")
 MSG_HASH(MENU_ENUM_LABEL_FRAME_THROTTLE_ENABLE,
       "fastforward_ratio_throttle_enable")
 MSG_HASH(MENU_ENUM_LABEL_FRAME_THROTTLE_SETTINGS,
@@ -401,6 +447,8 @@ MSG_HASH(MENU_ENUM_LABEL_GAME_SPECIFIC_OPTIONS_IN_USE,
       "game_specific_options_in_use")
 MSG_HASH(MENU_ENUM_LABEL_HELP,
       "help")
+MSG_HASH(MENU_ENUM_LABEL_HELP_CHEEVOS_DESCRIPTION,
+      "help_cheevos_description")
 MSG_HASH(MENU_ENUM_LABEL_HELP_AUDIO_VIDEO_TROUBLESHOOTING,
       "help_audio_video_troubleshooting")
 MSG_HASH(MENU_ENUM_LABEL_HELP_CHANGE_VIRTUAL_GAMEPAD,
@@ -467,6 +515,10 @@ MSG_HASH(MENU_ENUM_LABEL_INPUT_OVERLAY_ENABLE,
       "input_overlay_enable")
 MSG_HASH(MENU_ENUM_LABEL_INPUT_OVERLAY_HIDE_IN_MENU,
       "overlay_hide_in_menu")
+MSG_HASH(MENU_ENUM_LABEL_INPUT_OVERLAY_SHOW_PHYSICAL_INPUTS,
+      "overlay_show_physical_inputs")
+MSG_HASH(MENU_ENUM_LABEL_INPUT_OVERLAY_SHOW_PHYSICAL_INPUTS_PORT,
+      "overlay_show_physical_inputs_port")
 MSG_HASH(MENU_ENUM_LABEL_INPUT_PLAYER_ANALOG_DPAD_MODE,
       "input_player%u_analog_dpad_mode")
 MSG_HASH(MENU_ENUM_LABEL_INPUT_POLL_TYPE_BEHAVIOR,
@@ -481,6 +533,8 @@ MSG_HASH(MENU_ENUM_LABEL_INPUT_SETTINGS,
       "input_settings")
 MSG_HASH(MENU_ENUM_LABEL_INPUT_SETTINGS_BEGIN,
       "input_settings_begin")
+MSG_HASH(MENU_ENUM_LABEL_LATENCY_SETTINGS,
+      "latency_settings")
 MSG_HASH(MENU_ENUM_LABEL_INPUT_SMALL_KEYBOARD_ENABLE,
       "input_small_keyboard_enable")
 MSG_HASH(MENU_ENUM_LABEL_INPUT_TOUCH_ENABLE,
@@ -537,6 +591,8 @@ MSG_HASH(MENU_ENUM_LABEL_LOAD_ARCHIVE_DETECT_CORE,
       "load_archive_detect_core")
 MSG_HASH(MENU_ENUM_LABEL_LOAD_CONTENT_HISTORY,
       "load_recent")
+MSG_HASH(MENU_ENUM_LABEL_LOAD_CONTENT_SPECIAL,
+      "load_special")
 MSG_HASH(MENU_ENUM_LABEL_LOAD_CONTENT_LIST,
       "load_content")
 MSG_HASH(MENU_ENUM_LABEL_LOAD_STATE,
@@ -567,6 +623,8 @@ MSG_HASH(MENU_ENUM_LABEL_MENU_FILE_BROWSER_SETTINGS,
       "menu_file_browser_settings")
 MSG_HASH(MENU_ENUM_LABEL_MENU_LINEAR_FILTER,
       "menu_linear_filter")
+MSG_HASH(MENU_ENUM_LABEL_MENU_HORIZONTAL_ANIMATION,
+      "menu_horizontal_animation")
 MSG_HASH(MENU_ENUM_LABEL_MENU_SETTINGS,
       "menu_settings")
 MSG_HASH(MENU_ENUM_LABEL_MENU_WALLPAPER,
@@ -585,8 +643,12 @@ MSG_HASH(MENU_ENUM_LABEL_NETPLAY,
       "netplay")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_CHECK_FRAMES,
       "netplay_check_frames")
-MSG_HASH(MENU_ENUM_LABEL_NETPLAY_CLIENT_SWAP_INPUT,
-      "netplay_client_swap_input")
+MSG_HASH(MENU_ENUM_LABEL_NETPLAY_REQUEST_DEVICE_I,
+      "netplay_request_device_%u")
+MSG_HASH(MENU_ENUM_LABEL_NETPLAY_SHARE_ANALOG,
+      "netplay_share_analog")
+MSG_HASH(MENU_ENUM_LABEL_NETPLAY_SHARE_DIGITAL,
+      "netplay_share_digital")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_INPUT_LATENCY_FRAMES_MIN,
       "netplay_input_latency_frames_min")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_INPUT_LATENCY_FRAMES_RANGE,
@@ -599,6 +661,8 @@ MSG_HASH(MENU_ENUM_LABEL_NETPLAY_ENABLE_CLIENT,
       "menu_netplay_enable_client")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_ENABLE_HOST,
       "menu_netplay_enable_host")
+MSG_HASH(MENU_ENUM_LABEL_NETPLAY_ALLOW_SLAVES,
+      "netplay_allow_slaves")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_IP_ADDRESS,
       "netplay_ip_address")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_MODE,
@@ -611,10 +675,16 @@ MSG_HASH(MENU_ENUM_LABEL_NETPLAY_PASSWORD,
       "netplay_password")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_SETTINGS,
       "menu_netplay_settings")
+MSG_HASH(MENU_ENUM_LABEL_NETPLAY_PUBLIC_ANNOUNCE,
+      "netplay_public_announce")
+MSG_HASH(MENU_ENUM_LABEL_NETPLAY_REQUIRE_SLAVES,
+      "netplay_require_slaves")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_SPECTATE_PASSWORD,
       "netplay_spectate_password")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_SPECTATOR_MODE_ENABLE,
       "netplay_spectator_mode_enable")
+MSG_HASH(MENU_ENUM_LABEL_NETPLAY_START_AS_SPECTATOR,
+      "netplay_start_as_spectator")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_STATELESS_MODE,
       "netplay_stateless_mode")
 MSG_HASH(MENU_ENUM_LABEL_NETPLAY_TCP_UDP_PORT,
@@ -731,6 +801,36 @@ MSG_HASH(MENU_ENUM_LABEL_RDB_ENTRY_CRC32,
       "rdb_entry_crc32")
 MSG_HASH(MENU_ENUM_LABEL_RDB_ENTRY_DESCRIPTION,
       "rdb_entry_description")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_DEVELOPER,
+      "deferred_cursor_manager_list_rdb_entry_developer")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_PUBLISHER,
+      "deferred_cursor_manager_list_rdb_entry_publisher")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ORIGIN,
+      "deferred_cursor_manager_list_rdb_entry_origin")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_FRANCHISE,
+      "deferred_cursor_manager_list_rdb_entry_franchise")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_EDGE_MAGAZINE_RATING,
+      "deferred_cursor_manager_list_rdb_entry_edge_magazine_rating")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_EDGE_MAGAZINE_ISSUE,
+      "deferred_cursor_manager_list_rdb_entry_edge_magazine_issue")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_FAMITSU_MAGAZINE_RATING,
+      "deferred_cursor_manager_list_rdb_entry_famitsu_magazine_rating")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ENHANCEMENT_HW,
+      "deferred_cursor_manager_list_rdb_entry_enhancement_hw")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_RELEASEMONTH,
+      "deferred_cursor_manager_list_rdb_entry_releasemonth")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_RELEASEYEAR,
+      "deferred_cursor_manager_list_rdb_entry_releaseyear")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_ESRB_RATING,
+      "deferred_cursor_manager_list_rdb_entry_esrb_rating")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_PEGI_RATING,
+      "deferred_cursor_manager_list_rdb_entry_pegi_rating")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_CERO_RATING,
+      "deferred_cursor_manager_list_rdb_entry_cero_rating")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_BBFC_RATING,
+      "deferred_cursor_manager_list_rdb_entry_bbfc_rating")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_CURSOR_MANAGER_LIST_RDB_ENTRY_MAX_USERS,
+      "deferred_cursor_manager_list_rdb_entry_max_users")
 MSG_HASH(MENU_ENUM_LABEL_RDB_ENTRY_DEVELOPER,
       "rdb_entry_developer")
 MSG_HASH(MENU_ENUM_LABEL_RDB_ENTRY_EDGE_MAGAZINE_ISSUE,
@@ -799,6 +899,10 @@ MSG_HASH(MENU_ENUM_LABEL_REMAP_FILE_SAVE_CORE,
       "remap_file_save_core")
 MSG_HASH(MENU_ENUM_LABEL_REMAP_FILE_SAVE_GAME,
       "remap_file_save_game")
+MSG_HASH(MENU_ENUM_LABEL_REMAP_FILE_REMOVE_CORE,
+      "remap_file_remove_core")
+MSG_HASH(MENU_ENUM_LABEL_REMAP_FILE_REMOVE_GAME,
+      "remap_file_remove_game")
 MSG_HASH(MENU_ENUM_LABEL_RESTART_CONTENT,
       "restart_content")
 MSG_HASH(MENU_ENUM_LABEL_RESTART_RETROARCH,
@@ -823,8 +927,16 @@ MSG_HASH(MENU_ENUM_LABEL_RGUI_CONFIG_DIRECTORY,
       "rgui_config_directory")
 MSG_HASH(MENU_ENUM_LABEL_RGUI_SHOW_START_SCREEN,
       "rgui_show_start_screen")
+MSG_HASH(MENU_ENUM_LABEL_ADD_TO_FAVORITES,
+      "favorites_add")
+MSG_HASH(MENU_ENUM_LABEL_ADD_TO_FAVORITES_PLAYLIST,
+      "favorites_add_playlist")
+MSG_HASH(MENU_ENUM_LABEL_RESET_CORE_ASSOCIATION,
+      "reset_core_association")
 MSG_HASH(MENU_ENUM_LABEL_RUN,
       "collection")
+MSG_HASH(MENU_ENUM_LABEL_RUN_MUSIC,
+      "collection_music")
 MSG_HASH(MENU_ENUM_LABEL_SAMBA_ENABLE,
       "samba_enable")
 MSG_HASH(MENU_ENUM_LABEL_SAVEFILE_DIRECTORY,
@@ -833,6 +945,8 @@ MSG_HASH(MENU_ENUM_LABEL_SAVESTATE_AUTO_INDEX,
       "savestate_auto_index")
 MSG_HASH(MENU_ENUM_LABEL_SAVESTATE_AUTO_LOAD,
       "savestate_auto_load")
+MSG_HASH(MENU_ENUM_LABEL_SAVESTATE_THUMBNAIL_ENABLE,
+      "savestate_thumbnails")
 MSG_HASH(MENU_ENUM_LABEL_SAVESTATE_AUTO_SAVE,
       "savestate_auto_save")
 MSG_HASH(MENU_ENUM_LABEL_SAVESTATE_DIRECTORY,
@@ -867,6 +981,8 @@ MSG_HASH(MENU_ENUM_LABEL_SETTINGS_TAB,
       "settings_tab")
 MSG_HASH(MENU_ENUM_LABEL_SHADER_APPLY_CHANGES,
       "shader_apply_changes")
+MSG_HASH(MENU_ENUM_LABEL_SHADER_WATCH_FOR_CHANGES,
+      "video_shader_watch_files")
 MSG_HASH(MENU_ENUM_LABEL_SHADER_OPTIONS,
       "shader_options")
 MSG_HASH(MENU_ENUM_LABEL_SHADER_PARAMETERS_ENTRY,
@@ -879,10 +995,24 @@ MSG_HASH(MENU_ENUM_LABEL_SHUTDOWN,
       "shutdown")
 MSG_HASH(MENU_ENUM_LABEL_SLOWMOTION_RATIO,
       "slowmotion_ratio")
+MSG_HASH(MENU_ENUM_LABEL_RUN_AHEAD_ENABLED,
+      "run_ahead_enabled")
+MSG_HASH(MENU_ENUM_LABEL_RUN_AHEAD_SECONDARY_INSTANCE,
+      "run_ahead_secondary_instance")
+MSG_HASH(MENU_ENUM_LABEL_RUN_AHEAD_FRAMES,
+      "run_ahead_frames")
 MSG_HASH(MENU_ENUM_LABEL_SORT_SAVEFILES_ENABLE,
       "sort_savefiles_enable")
 MSG_HASH(MENU_ENUM_LABEL_SORT_SAVESTATES_ENABLE,
       "sort_savestates_enable")
+MSG_HASH(MENU_ENUM_LABEL_SAVEFILES_IN_CONTENT_DIR_ENABLE,
+      "savefiles_in_content_dir_enable")
+MSG_HASH(MENU_ENUM_LABEL_SAVESTATES_IN_CONTENT_DIR_ENABLE,
+      "savestates_in_content_dir_enable")
+MSG_HASH(MENU_ENUM_LABEL_SYSTEMFILES_IN_CONTENT_DIR_ENABLE,
+      "systemfiles_in_content_dir_enable")
+MSG_HASH(MENU_ENUM_LABEL_SCREENSHOTS_IN_CONTENT_DIR_ENABLE,
+      "screenshots_in_content_dir_enable")
 MSG_HASH(MENU_ENUM_LABEL_SSH_ENABLE,
       "ssh_enable")
 MSG_HASH(MENU_ENUM_LABEL_START_CORE,
@@ -911,6 +1041,10 @@ MSG_HASH(MENU_ENUM_LABEL_THREADED_DATA_RUNLOOP_ENABLE,
       "threaded_data_runloop_enable")
 MSG_HASH(MENU_ENUM_LABEL_THUMBNAILS,
       "thumbnails")
+MSG_HASH(MENU_ENUM_LABEL_LEFT_THUMBNAILS,
+      "left thumbnails")
+MSG_HASH(MENU_ENUM_LABEL_XMB_VERTICAL_THUMBNAILS,
+      "xmb_vertical_thumbnails")
 MSG_HASH(MENU_ENUM_LABEL_THUMBNAILS_DIRECTORY,
       "thumbnails_directory")
 MSG_HASH(MENU_ENUM_LABEL_THUMBNAILS_UPDATER_LIST,
@@ -923,6 +1057,10 @@ MSG_HASH(MENU_ENUM_LABEL_UI_COMPANION_ENABLE,
       "ui_companion_enable")
 MSG_HASH(MENU_ENUM_LABEL_UI_COMPANION_START_ON_BOOT,
       "ui_companion_start_on_boot")
+MSG_HASH(MENU_ENUM_LABEL_UI_COMPANION_TOGGLE,
+      "ui_companion_toggle")
+MSG_HASH(MENU_ENUM_LABEL_DESKTOP_MENU_ENABLE,
+      "desktop_menu_enable")
 MSG_HASH(MENU_ENUM_LABEL_UI_MENUBAR_ENABLE,
       "ui_menubar_enable")
 MSG_HASH(MENU_ENUM_LABEL_UNDO_LOAD_STATE,
@@ -967,6 +1105,12 @@ MSG_HASH(MENU_ENUM_LABEL_USE_THIS_DIRECTORY,
       "use_this_directory")
 MSG_HASH(MENU_ENUM_LABEL_VIDEO_ALLOW_ROTATE,
       "video_allow_rotate")
+MSG_HASH(MENU_ENUM_LABEL_CRT_SWITCH_RESOLUTION, 
+	  "crt_switch_resolution") 
+MSG_HASH(MENU_ENUM_LABEL_CRT_SWITCH_RESOLUTION_SUPER, 
+	  "crt_switch_resolution_super") 
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_ASPECT_RATIO,
+      "video_aspect_ratio")
 MSG_HASH(MENU_ENUM_LABEL_VIDEO_ASPECT_RATIO_AUTO,
       "video_aspect_ratio_auto")
 MSG_HASH(MENU_ENUM_LABEL_VIDEO_ASPECT_RATIO_INDEX,
@@ -1023,6 +1167,8 @@ MSG_HASH(MENU_ENUM_LABEL_VIDEO_REFRESH_RATE,
       "video_refresh_rate")
 MSG_HASH(MENU_ENUM_LABEL_VIDEO_REFRESH_RATE_AUTO,
       "video_refresh_rate_auto")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_REFRESH_RATE_POLLED,
+      "video_refresh_rate_polled")
 MSG_HASH(MENU_ENUM_LABEL_VIDEO_ROTATION,
       "video_rotation")
 MSG_HASH(MENU_ENUM_LABEL_VIDEO_SCALE,
@@ -1081,14 +1227,26 @@ MSG_HASH(MENU_ENUM_LABEL_VIDEO_WINDOW_WIDTH,
       "video_window_width")
 MSG_HASH(MENU_ENUM_LABEL_VIDEO_WINDOW_HEIGHT,
       "video_window_height")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_FULLSCREEN_X,
+      "video_fullscreen_x")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_FULLSCREEN_Y,
+      "video_fullscreen_y")
 MSG_HASH(MENU_ENUM_LABEL_WIFI_DRIVER,
       "wifi_driver")
 MSG_HASH(MENU_ENUM_LABEL_WIFI_SETTINGS,
       "wifi_settings")
 MSG_HASH(MENU_ENUM_LABEL_XMB_ALPHA_FACTOR,
       "xmb_alpha_factor")
+MSG_HASH(MENU_ENUM_LABEL_MENU_FONT_COLOR_RED,
+      "menu_font_color_red")
+MSG_HASH(MENU_ENUM_LABEL_MENU_FONT_COLOR_GREEN,
+      "menu_font_color_green")
+MSG_HASH(MENU_ENUM_LABEL_MENU_FONT_COLOR_BLUE,
+      "menu_font_color_blue")
 MSG_HASH(MENU_ENUM_LABEL_XMB_FONT,
       "xmb_font")
+MSG_HASH(MENU_ENUM_LABEL_XMB_MAIN_MENU_ENABLE_SETTINGS,
+      "xmb_main_menu_enable_settings")
 MSG_HASH(MENU_ENUM_LABEL_XMB_MENU_COLOR_THEME,
       "xmb_menu_color_theme")
 MSG_HASH(MENU_ENUM_LABEL_XMB_RIBBON_ENABLE,
@@ -1097,18 +1255,28 @@ MSG_HASH(MENU_ENUM_LABEL_XMB_SCALE_FACTOR,
       "xmb_scale_factor")
 MSG_HASH(MENU_ENUM_LABEL_XMB_SHADOWS_ENABLE,
       "xmb_shadows_enable")
-MSG_HASH(MENU_ENUM_LABEL_XMB_SHOW_HISTORY,
-      "xmb_show_history")
-MSG_HASH(MENU_ENUM_LABEL_XMB_SHOW_ADD,
-      "xmb_show_add")
-MSG_HASH(MENU_ENUM_LABEL_XMB_SHOW_IMAGES,
-      "xmb_show_images")
-MSG_HASH(MENU_ENUM_LABEL_XMB_SHOW_MUSIC,
-      "xmb_show_music")
-MSG_HASH(MENU_ENUM_LABEL_XMB_SHOW_SETTINGS,
-      "xmb_show_settings")
-MSG_HASH(MENU_ENUM_LABEL_XMB_SHOW_VIDEO,
-      "xmb_show_video")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_HISTORY,
+      "content_show_history")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_ADD,
+      "content_show_add")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_PLAYLISTS,
+      "content_show_playlists")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_FAVORITES,
+      "content_show_favorites")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_IMAGES,
+      "content_show_images")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_MUSIC,
+      "content_show_music")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_SETTINGS,
+      "content_show_settings")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_SETTINGS_PASSWORD,
+      "content_show_settings_password")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_VIDEO,
+      "content_show_video")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_NETPLAY,
+      "content_show_netplay")
+MSG_HASH(MENU_ENUM_LABEL_XMB_LAYOUT,
+      "xmb_layout")
 MSG_HASH(MENU_ENUM_LABEL_XMB_THEME,
       "xmb_theme")
 MSG_HASH(MSG_BRINGING_UP_COMMAND_INTERFACE_ON_PORT,
@@ -1199,3 +1367,141 @@ MSG_HASH(MENU_ENUM_LABEL_BROWSE_START,
       "browse_start")
 MSG_HASH(MENU_ENUM_LABEL_SHADER_PIPELINE_BOKEH,
       "shader_pipeline_bokeh")
+MSG_HASH(MENU_ENUM_LABEL_NETPLAY_USE_MITM_SERVER,
+      "netplay_use_mitm_server")
+MSG_HASH(MENU_ENUM_LABEL_NETPLAY_MITM_SERVER,
+      "netplay_mitm_server")
+MSG_HASH(MENU_ENUM_LABEL_ADD_TO_MIXER,
+      "audio_add_to_mixer")
+MSG_HASH(MENU_ENUM_LABEL_ADD_TO_MIXER_AND_PLAY,
+      "audio_add_to_mixer_and_play")
+MSG_HASH(MENU_ENUM_LABEL_ADD_TO_MIXER_AND_COLLECTION,
+      "audio_add_to_mixer_and_collection")
+MSG_HASH(MENU_ENUM_LABEL_ADD_TO_MIXER_AND_COLLECTION_AND_PLAY,
+      "audio_add_to_mixer_and_collection_and_play")
+MSG_HASH(MENU_ENUM_LABEL_FILTER_BY_CURRENT_CORE,
+      "filter_by_current_Core")
+MSG_HASH(MENU_ENUM_LABEL_AUDIO_MIXER_VOLUME,
+      "audio_mixer_volume")
+MSG_HASH(MENU_ENUM_LABEL_AUDIO_MIXER_MUTE,
+      "audio_mixer_mute_enable")
+MSG_HASH(MENU_ENUM_LABEL_MENU_SHOW_ONLINE_UPDATER,
+      "menu_show_online_updater")
+MSG_HASH(MENU_ENUM_LABEL_MENU_SHOW_CORE_UPDATER,
+      "menu_show_core_updater")
+MSG_HASH(MENU_ENUM_LABEL_CORE_DELETE,
+      "core_delete")
+MSG_HASH(MENU_ENUM_LABEL_PLAYLIST_ENTRY_RENAME,
+      "playlist_entry_rename")
+MSG_HASH(MENU_ENUM_LABEL_MENU_FRAMEBUFFER_OPACITY,
+      "menu_framebuffer_opacity")
+MSG_HASH(MENU_ENUM_LABEL_GOTO_FAVORITES,
+      "goto_favorites")
+MSG_HASH(MENU_ENUM_LABEL_GOTO_MUSIC,
+      "goto_music")
+MSG_HASH(MENU_ENUM_LABEL_GOTO_IMAGES,
+      "goto_images")
+MSG_HASH(MENU_ENUM_LABEL_GOTO_VIDEO,
+      "goto_video")
+MSG_HASH(MENU_ENUM_LABEL_MATERIALUI_ICONS_ENABLE,
+      "materialui_icons_enable")
+MSG_HASH(MENU_ENUM_LABEL_RENAME_ENTRY,
+      "rename_entry")
+MSG_HASH(MENU_ENUM_LABEL_MENU_SHOW_LOAD_CORE,
+      "menu_show_load_core")
+MSG_HASH(MENU_ENUM_LABEL_MENU_SHOW_LOAD_CONTENT,
+      "menu_show_load_content")
+MSG_HASH(MENU_ENUM_LABEL_MENU_SHOW_INFORMATION,
+      "menu_show_information")
+MSG_HASH(MENU_ENUM_LABEL_MENU_SHOW_CONFIGURATIONS,
+      "menu_show_configurations")
+MSG_HASH(MENU_ENUM_LABEL_MENU_SHOW_HELP,
+      "menu_show_help")
+MSG_HASH(MENU_ENUM_LABEL_MENU_SHOW_QUIT_RETROARCH,
+      "menu_show_quit_retroarch")
+MSG_HASH(MENU_ENUM_LABEL_MENU_SHOW_REBOOT,
+      "menu_show_reboot")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_TAKE_SCREENSHOT,
+      "quick_menu_show_take_screenshot")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_SAVE_LOAD_STATE,
+      "quick_menu_show_save_load_state")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_UNDO_SAVE_LOAD_STATE,
+      "quick_menu_show_undo_save_load_state")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_ADD_TO_FAVORITES,
+      "quick_menu_show_add_to_favorites")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_OPTIONS,
+      "quick_menu_show_options")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_CONTROLS,
+      "quick_menu_show_controls")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_CHEATS,
+      "quick_menu_show_cheats")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_SHADERS,
+      "quick_menu_show_shaders")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_SAVE_CORE_OVERRIDES,
+      "quick_menu_show_save_core_overrides")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_SAVE_GAME_OVERRIDES,
+      "quick_menu_show_save_game_overrides")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_SHOW_INFORMATION,
+      "quick_menu_show_information")
+MSG_HASH(MENU_ENUM_LABEL_MENU_ENABLE_KIOSK_MODE,
+      "menu_enable_kiosk_mode")
+MSG_HASH(MENU_ENUM_LABEL_MENU_DISABLE_KIOSK_MODE,
+      "menu_disable_kiosk_mode")
+MSG_HASH(MENU_ENUM_LABEL_MENU_KIOSK_MODE_PASSWORD,
+      "menu_disable_kiosk_mode_password")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_MESSAGE_BGCOLOR_ENABLE,
+      "video_msg_bgcolor_enable")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_MESSAGE_BGCOLOR_RED,
+      "video_msg_bgcolor_red")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_MESSAGE_BGCOLOR_GREEN,
+      "video_msg_bgcolor_green")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_MESSAGE_BGCOLOR_BLUE,
+      "video_msg_bgcolor_blue")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_MESSAGE_BGCOLOR_OPACITY,
+      "video_msg_bgcolor_opacity")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_MESSAGE_COLOR_RED,
+      "video_msg_color_red")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_MESSAGE_COLOR_GREEN,
+      "video_msg_color_green")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_MESSAGE_COLOR_BLUE,
+      "video_msg_color_blue")
+MSG_HASH(MENU_ENUM_LABEL_FRAMECOUNT_SHOW,
+      "framecount_show")
+MSG_HASH(MENU_ENUM_LABEL_AUTOMATICALLY_ADD_CONTENT_TO_PLAYLIST,
+      "automatically_add_content_to_playlist")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_WINDOW_OPACITY,
+      "video_window_opacity")
+MSG_HASH(MENU_ENUM_LABEL_AUDIO_RESAMPLER_QUALITY,
+      "audio_resampler_quality")
+MSG_HASH(MENU_ENUM_LABEL_AUDIO_RESAMPLER_DRIVER_SINC,
+      "sinc")
+MSG_HASH(MENU_ENUM_LABEL_AUDIO_RESAMPLER_DRIVER_CC,
+      "cc")
+MSG_HASH(MENU_ENUM_LABEL_INPUT_DRIVER_UDEV,
+      "udev")
+MSG_HASH(MENU_ENUM_LABEL_INPUT_DRIVER_LINUXRAW,
+      "linuxraw")
+MSG_HASH(MENU_ENUM_LABEL_VIDEO_WINDOW_SHOW_DECORATIONS,
+      "video_window_show_decorations")
+MSG_HASH(MENU_ENUM_LABEL_MENU_RGUI_BORDER_FILLER_ENABLE,
+      "menu_rgui_border_filler_enable")
+MSG_HASH(MENU_ENUM_LABEL_MENU_RGUI_BORDER_FILLER_THICKNESS_ENABLE,
+      "menu_rgui_border_filler_thickness_enable")
+MSG_HASH(MENU_ENUM_LABEL_MENU_RGUI_BACKGROUND_FILLER_THICKNESS_ENABLE,
+      "menu_rgui_background_filler_thickness_enable")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_REWIND,
+      "menu_show_rewind_settings")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_LATENCY,
+      "menu_show_latency_settings")
+MSG_HASH(MENU_ENUM_LABEL_CONTENT_SHOW_OVERLAYS,
+      "menu_show_overlay_settings")
+MSG_HASH(MENU_ENUM_LABEL_AUDIO_ENABLE_MENU,
+      "audio_enable_menu")
+MSG_HASH(MENU_ENUM_LABEL_AUDIO_MIXER_SETTINGS,
+      "audio_mixer_settings")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_MIXER_STREAM_SETTINGS_LIST,
+      "deferred_mixer_stream_settings_list")
+MSG_HASH(MENU_ENUM_LABEL_QUICK_MENU_OVERRIDE_OPTIONS,
+      "quick_menu_override_options")
+MSG_HASH(MENU_ENUM_LABEL_DEFERRED_QUICK_MENU_OVERRIDE_OPTIONS,
+      "deferred_quick_menu_override_options")
