@@ -14,6 +14,7 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <stdio.h>
 #include <stdint.h>
 #include <stddef.h>
 
@@ -21,197 +22,204 @@
 #include <string/stdstring.h>
 
 #include "../msg_hash.h"
-#include "../configuration.h"
 #include "../verbosity.h"
 
-int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
-    uint32_t driver_hash = 0;
+#ifdef RARCH_INTERNAL
+#include "../configuration.h"
+
+int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len)
+{
     settings_t *settings = config_get_ptr();
 
+    if (msg == MENU_ENUM_LABEL_CONNECT_NETPLAY_ROOM)
+    {
+       snprintf(s, len,
+             "TODO/FIXME - Fill in message here."
+             );
+       return 0;
+    }
     if (msg <= MENU_ENUM_LABEL_INPUT_HOTKEY_BIND_END &&
-        msg >= MENU_ENUM_LABEL_INPUT_HOTKEY_BIND_BEGIN) {
-        unsigned idx = msg - MENU_ENUM_LABEL_INPUT_HOTKEY_BIND_BEGIN;
+        msg >= MENU_ENUM_LABEL_INPUT_HOTKEY_BIND_BEGIN)
+    {
+       unsigned idx = msg - MENU_ENUM_LABEL_INPUT_HOTKEY_BIND_BEGIN;
 
-        switch (idx) {
-            case RARCH_FAST_FORWARD_KEY:
-                snprintf(s, len,
-                         "Toggles between fast-forwarding and \n"
-                                 "normal speed."
-                );
-                break;
-            case RARCH_FAST_FORWARD_HOLD_KEY:
-                snprintf(s, len,
-                         "Hold for fast-forward. \n"
-                                 " \n"
-                                 "Releasing button disables fast-forward."
-                );
-                break;
-            case RARCH_PAUSE_TOGGLE:
-                snprintf(s, len,
-                         "Toggle between paused and non-paused state.");
-                break;
-            case RARCH_FRAMEADVANCE:
-                snprintf(s, len,
-                         "Frame advance when content is paused.");
-                break;
-            case RARCH_SHADER_NEXT:
-                snprintf(s, len,
-                         "Applies next shader in directory.");
-                break;
-            case RARCH_SHADER_PREV:
-                snprintf(s, len,
-                         "Applies previous shader in directory.");
-                break;
-            case RARCH_CHEAT_INDEX_PLUS:
-            case RARCH_CHEAT_INDEX_MINUS:
-            case RARCH_CHEAT_TOGGLE:
-                snprintf(s, len,
-                         "Cheats.");
-                break;
-            case RARCH_RESET:
-                snprintf(s, len,
-                         "Reset the content.");
-                break;
-            case RARCH_SCREENSHOT:
-                snprintf(s, len,
-                         "Take screenshot.");
-                break;
-            case RARCH_MUTE:
-                snprintf(s, len,
-                         "Mute/unmute audio.");
-                break;
-            case RARCH_OSK:
-                snprintf(s, len,
-                         "Toggles onscreen keyboard.");
-                break;
-            case RARCH_NETPLAY_FLIP:
-                snprintf(s, len,
-                         "Netplay flip users.");
-                break;
-            case RARCH_NETPLAY_GAME_WATCH:
-                snprintf(s, len,
-                         "Netplay toggle play/spectate mode.");
-                break;
-            case RARCH_SLOWMOTION:
-                snprintf(s, len,
-                         "Hold for slowmotion.");
-                break;
-            case RARCH_ENABLE_HOTKEY:
-                snprintf(s, len,
-                         "Enable other hotkeys. \n"
-                                 " \n"
-                                 "If this hotkey is bound to either\n"
-                                 "a keyboard, joybutton or joyaxis, \n"
-                                 "all other hotkeys will be enabled only \n"
-                                 "if this one is held at the same time. \n"
-                                 " \n"
-                                 "This is useful for RETRO_KEYBOARD centric \n"
-                                 "implementations which query a large area of \n"
-                                 "the keyboard, where it is not desirable that \n"
-                                 "hotkeys get in the way. \n"
-                                 " \n"
-                                 "Alternatively, all hotkeys for keyboard \n"
-                                 "could be disabled by the user.");
-                break;
-            case RARCH_VOLUME_UP:
-                snprintf(s, len,
-                         "Increases audio volume.");
-                break;
-            case RARCH_VOLUME_DOWN:
-                snprintf(s, len,
-                         "Decreases audio volume.");
-                break;
-            case RARCH_OVERLAY_NEXT:
-                snprintf(s, len,
-                         "Switches to next overlay. Wraps around.");
-                break;
-            case RARCH_DISK_EJECT_TOGGLE:
-                snprintf(s, len,
-                         "Toggles eject for disks. \n"
-                                 " \n"
-                                 "Used for multiple-disk content. ");
-                break;
-            case RARCH_DISK_NEXT:
-            case RARCH_DISK_PREV:
-                snprintf(s, len,
-                         "Cycles through disk images. Use after ejecting. \n"
-                                 " \n"
-                                 "Complete by toggling eject again.");
-                break;
-            case RARCH_GRAB_MOUSE_TOGGLE:
-                snprintf(s, len,
-                         "Toggles mouse grab. \n"
-                                 " \n"
-                                 "When mouse is grabbed, RetroArch hides the \n"
-                                 "mouse, and keeps the mouse pointer inside \n"
-                                 "the window to allow relative mouse input to \n"
-                                 "work better.");
-                break;
-            case RARCH_GAME_FOCUS_TOGGLE:
-                snprintf(s, len,
-                         "Toggles game focus.\n"
-                                 " \n"
-                                 "When a game has focus, RetroArch will both disable \n"
-                                 "hotkeys and keep/warp the mouse pointer inside the window.");
-                break;
-            case RARCH_MENU_TOGGLE:
-                snprintf(s, len, "Toggles menu.");
-                break;
-            case RARCH_LOAD_STATE_KEY:
-                snprintf(s, len,
-                         "Loads state.");
-                break;
-            case RARCH_FULLSCREEN_TOGGLE_KEY:
-                snprintf(s, len,
-                         "Toggles fullscreen.");
-                break;
-            case RARCH_QUIT_KEY:
-                snprintf(s, len,
-                         "Key to exit RetroArch cleanly. \n"
-                                " \n"
-                                "Killing it in any hard way (SIGKILL, etc.) will \n"
-                                "terminate RetroArch without saving RAM, etc."
+       switch (idx)
+       {
+          case RARCH_FAST_FORWARD_KEY:
+             snprintf(s, len,
+                   "Toggles between fast-forwarding and \n"
+                   "normal speed."
+                   );
+             break;
+          case RARCH_FAST_FORWARD_HOLD_KEY:
+             snprintf(s, len,
+                   "Hold for fast-forward. \n"
+                   " \n"
+                   "Releasing button disables fast-forward."
+                   );
+             break;
+          case RARCH_SLOWMOTION_KEY:
+             snprintf(s, len,
+                   "Toggles slowmotion.");
+             break;
+          case RARCH_SLOWMOTION_HOLD_KEY:
+             snprintf(s, len,
+                   "Hold for slowmotion.");
+             break;
+          case RARCH_PAUSE_TOGGLE:
+             snprintf(s, len,
+                   "Toggle between paused and non-paused state.");
+             break;
+          case RARCH_FRAMEADVANCE:
+             snprintf(s, len,
+                   "Frame advance when content is paused.");
+             break;
+          case RARCH_SHADER_NEXT:
+             snprintf(s, len,
+                   "Applies next shader in directory.");
+             break;
+          case RARCH_SHADER_PREV:
+             snprintf(s, len,
+                   "Applies previous shader in directory.");
+             break;
+          case RARCH_CHEAT_INDEX_PLUS:
+          case RARCH_CHEAT_INDEX_MINUS:
+          case RARCH_CHEAT_TOGGLE:
+             snprintf(s, len,
+                   "Cheats.");
+             break;
+          case RARCH_RESET:
+             snprintf(s, len,
+                   "Reset the content.");
+             break;
+          case RARCH_SCREENSHOT:
+             snprintf(s, len,
+                   "Take screenshot.");
+             break;
+          case RARCH_MUTE:
+             snprintf(s, len,
+                   "Mute/unmute audio.");
+             break;
+          case RARCH_OSK:
+             snprintf(s, len,
+                   "Toggles onscreen keyboard.");
+             break;
+          case RARCH_NETPLAY_GAME_WATCH:
+             snprintf(s, len,
+                   "Netplay toggle play/spectate mode.");
+             break;
+          case RARCH_ENABLE_HOTKEY:
+             snprintf(s, len,
+                   "Enable other hotkeys. \n"
+                   " \n"
+                   "If this hotkey is bound to either\n"
+                   "a keyboard, joybutton or joyaxis, \n"
+                   "all other hotkeys will be enabled only \n"
+                   "if this one is held at the same time. \n"
+                   " \n"
+                   "Alternatively, all hotkeys for keyboard \n"
+                   "could be disabled by the user.");
+             break;
+          case RARCH_VOLUME_UP:
+             snprintf(s, len,
+                   "Increases audio volume.");
+             break;
+          case RARCH_VOLUME_DOWN:
+             snprintf(s, len,
+                   "Decreases audio volume.");
+             break;
+          case RARCH_OVERLAY_NEXT:
+             snprintf(s, len,
+                   "Switches to next overlay. Wraps around.");
+             break;
+          case RARCH_DISK_EJECT_TOGGLE:
+             snprintf(s, len,
+                   "Toggles eject for disks. \n"
+                   " \n"
+                   "Used for multiple-disk content. ");
+             break;
+          case RARCH_DISK_NEXT:
+          case RARCH_DISK_PREV:
+             snprintf(s, len,
+                   "Cycles through disk images. Use after ejecting. \n"
+                   " \n"
+                   "Complete by toggling eject again.");
+             break;
+          case RARCH_GRAB_MOUSE_TOGGLE:
+             snprintf(s, len,
+                   "Toggles mouse grab. \n"
+                   " \n"
+                   "When mouse is grabbed, RetroArch hides the \n"
+                   "mouse, and keeps the mouse pointer inside \n"
+                   "the window to allow relative mouse input to \n"
+                   "work better.");
+             break;
+          case RARCH_GAME_FOCUS_TOGGLE:
+             snprintf(s, len,
+                   "Toggles game focus.\n"
+                   " \n"
+                   "When a game has focus, RetroArch will both disable \n"
+                   "hotkeys and keep/warp the mouse pointer inside the window.");
+             break;
+          case RARCH_MENU_TOGGLE:
+             snprintf(s, len, "Toggles menu.");
+             break;
+          case RARCH_LOAD_STATE_KEY:
+             snprintf(s, len,
+                   "Loads state.");
+             break;
+          case RARCH_FULLSCREEN_TOGGLE_KEY:
+             snprintf(s, len,
+                   "Toggles fullscreen.");
+             break;
+          case RARCH_QUIT_KEY:
+             snprintf(s, len,
+                   "Key to exit RetroArch cleanly. \n"
+                   " \n"
+                   "Killing it in any hard way (SIGKILL, etc.) will \n"
+                   "terminate RetroArch without saving RAM, etc."
 #ifdef __unix__
-                        "\nOn Unix-likes, SIGINT/SIGTERM allows a clean \n"
-                        "deinitialization."
+                   "\nOn Unix-likes, SIGINT/SIGTERM allows a clean \n"
+                   "deinitialization."
 #endif
-                                "");
-                break;
-            case RARCH_STATE_SLOT_PLUS:
-            case RARCH_STATE_SLOT_MINUS:
-                snprintf(s, len,
-                         "State slots. \n"
-                                 " \n"
-                                 "With slot set to 0, save state name is \n"
-                                 "*.state (or whatever defined on commandline). \n"
-                                 " \n"
-                                 "When slot is not 0, path will be <path><d>, \n"
-                                 "where <d> is slot number.");
-                break;
-            case RARCH_SAVE_STATE_KEY:
-                snprintf(s, len,
-                         "Saves state.");
-                break;
-            case RARCH_REWIND:
-                snprintf(s, len,
-                         "Hold button down to rewind. \n"
-                                 " \n"
-                                 "Rewinding must be enabled.");
-                break;
-            case RARCH_MOVIE_RECORD_TOGGLE:
-                snprintf(s, len,
-                         "Toggle between recording and not.");
-                break;
-            default:
-                if (string_is_empty(s))
-                    strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NO_INFORMATION_AVAILABLE), len);
-                break;
-        }
+                   "");
+             break;
+          case RARCH_STATE_SLOT_PLUS:
+          case RARCH_STATE_SLOT_MINUS:
+             snprintf(s, len,
+                   "State slots. \n"
+                   " \n"
+                   "With slot set to 0, save state name is \n"
+                   "*.state (or whatever defined on commandline). \n"
+                   " \n"
+                   "When slot is not 0, path will be <path><d>, \n"
+                   "where <d> is slot number.");
+             break;
+          case RARCH_SAVE_STATE_KEY:
+             snprintf(s, len,
+                   "Saves state.");
+             break;
+          case RARCH_REWIND:
+             snprintf(s, len,
+                   "Hold button down to rewind. \n"
+                   " \n"
+                   "Rewinding must be enabled.");
+             break;
+          case RARCH_MOVIE_RECORD_TOGGLE:
+             snprintf(s, len,
+                   "Toggle between recording and not.");
+             break;
+          default:
+             if (string_is_empty(s))
+                strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NO_INFORMATION_AVAILABLE), len);
+             break;
+       }
 
-        return 0;
+       return 0;
     }
 
-    switch (msg) {
+    switch (msg)
+    {
         case MENU_ENUM_LABEL_ACCOUNTS_RETRO_ACHIEVEMENTS:
             snprintf(s, len, "Login details for your \n"
                     "Retro Achievements account. \n"
@@ -289,6 +297,9 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                     "The screenshot will be stored inside the \n"
                     "Screenshot Directory.");
             break;
+        case MENU_ENUM_LABEL_ADD_TO_FAVORITES:
+            snprintf(s, len, "Add the entry to your Favorites.");
+            break;
         case MENU_ENUM_LABEL_RUN:
             snprintf(s, len, "Start the content.");
             break;
@@ -369,8 +380,7 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                              " \n"
                              "If this option is disabled, \n"
                              "it will try to load even if such \n"
-                             "firmware is missing. \n"
-                             "down. \n");
+                             "firmware is missing. \n");
             break;
         case MENU_ENUM_LABEL_PARENT_DIRECTORY:
             snprintf(s, len,
@@ -486,9 +496,7 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                              "Setting it to 'Early' or 'Late' can result \n"
                              "in less latency, \n"
                              "depending on your configuration.\n\n"
-                             "When netplay is enabled, the default polling \n"
-                             "behavior (Normal) will be used, \n"
-                             "regardless of the value set here."
+                             "Will be ignored when using netplay."
             );
             break;
         case MENU_ENUM_LABEL_INPUT_DESCRIPTOR_HIDE_UNBOUND:
@@ -578,6 +586,16 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                      "Hide the current overlay from appearing \n"
                              "inside the menu.");
             break;
+        case MENU_ENUM_LABEL_INPUT_OVERLAY_SHOW_PHYSICAL_INPUTS:
+            snprintf(s, len,
+                      "Show keyboard/controller button presses on \n"
+                            "the onscreen overlay.");
+            break;
+        case MENU_ENUM_LABEL_INPUT_OVERLAY_SHOW_PHYSICAL_INPUTS_PORT:
+            snprintf(s, len,
+                      "Select the port to listen for controller input \n"
+                            "to display on the onscreen overlay.");
+            break;
         case MENU_ENUM_LABEL_OVERLAY_PRESET:
             snprintf(s, len,
                      "Path to input overlay.");
@@ -698,48 +716,42 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
             );
             break;
         case MENU_ENUM_LABEL_INPUT_DRIVER:
-            if (settings)
-                driver_hash = msg_hash_calculate(settings->input.driver);
+            {
+               const char *lbl = settings ? settings->arrays.input_driver : NULL;
 
-            switch (driver_hash) {
-                case MENU_LABEL_INPUT_DRIVER_UDEV:
-                    snprintf(s, len,
-                             "udev Input driver. \n"
-                                     " \n"
-                                     "This driver can run without X. \n"
-                                     " \n"
-                                     "It uses the recent evdev joypad API \n"
-                                     "for joystick support. It supports \n"
-                                     "hotplugging and force feedback (if \n"
-                                     "supported by device). \n"
-                                     " \n"
-                                     "The driver reads evdev events for keyboard \n"
-                                     "support. It also supports keyboard callback, \n"
-                                     "mice and touchpads. \n"
-                                     " \n"
-                                     "By default in most distros, /dev/input nodes \n"
-                                     "are root-only (mode 600). You can set up a udev \n"
-                                     "rule which makes these accessible to non-root."
-                    );
-                    break;
-                case MENU_LABEL_INPUT_DRIVER_LINUXRAW:
-                    snprintf(s, len,
-                             "linuxraw Input driver. \n"
-                                     " \n"
-                                     "This driver requires an active TTY. Keyboard \n"
-                                     "events are read directly from the TTY which \n"
-                                     "makes it simpler, but not as flexible as udev. \n" "Mice, etc, are not supported at all. \n"
-                                     " \n"
-                                     "This driver uses the older joystick API \n"
-                                     "(/dev/input/js*).");
-                    break;
-                default:
-                    snprintf(s, len,
-                             "Input driver.\n"
-                                     " \n"
-                                     "Depending on video driver, it might \n"
-                                     "force a different input driver.");
-                    break;
+               if (string_is_equal(lbl, msg_hash_to_str(MENU_ENUM_LABEL_INPUT_DRIVER_UDEV)))
+                     snprintf(s, len,
+                           "udev Input driver. \n"
+                           " \n"
+                           "It uses the recent evdev joypad API \n"
+                           "for joystick support. It supports \n"
+                           "hotplugging and force feedback. \n"
+                           " \n"
+                           "The driver reads evdev events for keyboard \n"
+                           "support. It also supports keyboard callback, \n"
+                           "mice and touchpads. \n"
+                           " \n"
+                           "By default in most distros, /dev/input nodes \n"
+                           "are root-only (mode 600). You can set up a udev \n"
+                           "rule which makes these accessible to non-root."
+                           );
+               else if (string_is_equal(lbl,
+                        msg_hash_to_str(MENU_ENUM_LABEL_INPUT_DRIVER_LINUXRAW)))
+                     snprintf(s, len,
+                           "linuxraw Input driver. \n"
+                           " \n"
+                           "This driver requires an active TTY. Keyboard \n"
+                           "events are read directly from the TTY which \n"
+                           "makes it simpler, but not as flexible as udev. \n" "Mice, etc, are not supported at all. \n"
+                           " \n"
+                           "This driver uses the older joystick API \n"
+                           "(/dev/input/js*).");
+               else
+                     snprintf(s, len,
+                           "Input driver.\n"
+                           " \n"
+                           "Depending on video driver, it might \n"
+                           "force a different input driver.");
             }
             break;
         case MENU_ENUM_LABEL_LOAD_CONTENT_LIST:
@@ -779,7 +791,8 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
             snprintf(s, len,
                      "Current Video driver.");
 
-            if (string_is_equal(settings->video.driver, "gl")) {
+            if (string_is_equal(settings->arrays.video_driver, "gl"))
+            {
                 snprintf(s, len,
                          "OpenGL Video driver. \n"
                                  " \n"
@@ -791,7 +804,9 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                                  "libretro GL core implementations is \n"
                                  "dependent on your graphics card's \n"
                                  "underlying GL driver).");
-            } else if (string_is_equal(settings->video.driver, "sdl2")) {
+            }
+            else if (string_is_equal(settings->arrays.video_driver, "sdl2"))
+            {
                 snprintf(s, len,
                          "SDL 2 Video driver.\n"
                                  " \n"
@@ -801,7 +816,9 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                                  "Performance for software-rendered libretro \n"
                                  "core implementations is dependent \n"
                                  "on your platform SDL implementation.");
-            } else if (string_is_equal(settings->video.driver, "sdl1")) {
+            }
+            else if (string_is_equal(settings->arrays.video_driver, "sdl1"))
+            {
                 snprintf(s, len,
                          "SDL Video driver.\n"
                                  " \n"
@@ -810,14 +827,18 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                                  " \n"
                                  "Performance is considered to be suboptimal. \n"
                                  "Consider using it only as a last resort.");
-            } else if (string_is_equal(settings->video.driver, "d3d")) {
+            }
+            else if (string_is_equal(settings->arrays.video_driver, "d3d"))
+            {
                 snprintf(s, len,
                          "Direct3D Video driver. \n"
                                  " \n"
                                  "Performance for software-rendered cores \n"
                                  "is dependent on your graphic card's \n"
                                  "underlying D3D driver).");
-            } else if (string_is_equal(settings->video.driver, "exynos")) {
+            }
+            else if (string_is_equal(settings->arrays.video_driver, "exynos"))
+            {
                 snprintf(s, len,
                          "Exynos-G2D Video Driver. \n"
                                  " \n"
@@ -827,14 +848,18 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                                  " \n"
                                  "Performance for software rendered cores \n"
                                  "should be optimal.");
-            } else if (string_is_equal(settings->video.driver, "drm")) {
+            }
+            else if (string_is_equal(settings->arrays.video_driver, "drm"))
+            {
                 snprintf(s, len,
                          "Plain DRM Video Driver. \n"
                                  " \n"
                                  "This is a low-level video driver using. \n"
                                  "libdrm for hardware scaling using \n"
                                  "GPU overlays.");
-            } else if (string_is_equal(settings->video.driver, "sunxi")) {
+            }
+            else if (string_is_equal(settings->arrays.video_driver, "sunxi"))
+            {
                 snprintf(s, len,
                          "Sunxi-G2D Video Driver. \n"
                                  " \n"
@@ -850,24 +875,28 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
             );
             break;
         case MENU_ENUM_LABEL_AUDIO_RESAMPLER_DRIVER:
-            if (settings)
-                driver_hash = msg_hash_calculate(settings->audio.resampler);
+            {
+               const char *lbl = settings ? settings->arrays.audio_resampler : NULL;
 
-            switch (driver_hash) {
-                case MENU_LABEL_AUDIO_RESAMPLER_DRIVER_SINC:
-                    snprintf(s, len,
-                             "Windowed SINC implementation.");
-                    break;
-                case MENU_LABEL_AUDIO_RESAMPLER_DRIVER_CC:
-                    snprintf(s, len,
-                             "Convoluted Cosine implementation.");
-                    break;
-                default:
-                    if (string_is_empty(s))
-                        strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NO_INFORMATION_AVAILABLE), len);
-                    break;
+               if (string_is_equal(lbl, msg_hash_to_str(
+                           MENU_ENUM_LABEL_AUDIO_RESAMPLER_DRIVER_SINC)))
+                  strlcpy(s,
+                        "Windowed SINC implementation.", len);
+               else if (string_is_equal(lbl, msg_hash_to_str(
+                           MENU_ENUM_LABEL_AUDIO_RESAMPLER_DRIVER_CC)))
+                  strlcpy(s,
+                        "Convoluted Cosine implementation.", len);
+               else if (string_is_empty(s))
+                  strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NO_INFORMATION_AVAILABLE), len);
             }
             break;
+			
+		case MENU_ENUM_LABEL_CRT_SWITCH_RESOLUTION: snprintf(s, len, "SET CRT"); 
+			break;
+			
+		case MENU_ENUM_LABEL_CRT_SWITCH_RESOLUTION_SUPER: snprintf(s, len, "SET CRT SUPER"); 
+			break;
+			
         case MENU_ENUM_LABEL_VIDEO_SHADER_PRESET:
             snprintf(s, len,
                      "Load Shader Preset. \n"
@@ -1048,6 +1077,15 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                              "saved to Shader Directory."
             );
             break;
+        case MENU_ENUM_LABEL_SHADER_WATCH_FOR_CHANGES:
+            snprintf(s, len,
+                     "Watch shader files for new changes. \n"
+                     " \n"
+                     "After saving changes to a shader on disk, \n"
+                     "it will automatically be recompiled \n"
+                     "and applied to the running content."
+            );
+            break;
         case MENU_ENUM_LABEL_MENU_TOGGLE:
             snprintf(s, len,
                      "Toggles menu.");
@@ -1165,6 +1203,13 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                              "not run at 60Hz, or something close to it, \n"
                              "disable VSync, and leave this at its default.");
             break;
+        case MENU_ENUM_LABEL_VIDEO_REFRESH_RATE_POLLED:
+            snprintf(s, len,
+                     "Set Polled Refresh Rate\n"
+                             " \n"
+                            "Sets the refresh rate to the actual value\n"
+                            "polled from the display driver.");
+            break;
         case MENU_ENUM_LABEL_VIDEO_ROTATION:
             snprintf(s, len,
                      "Forces a certain rotation \n"
@@ -1535,6 +1580,36 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                              "Increasing this value will increase \n"
                              "performance, but introduce more latency.");
             break;
+        case MENU_ENUM_LABEL_NETPLAY_PUBLIC_ANNOUNCE:
+            snprintf(s, len,
+                     "Whether to announce netplay games publicly. \n"
+                             " \n"
+                             "If set to false, clients must manually connect \n"
+                             "rather than using the public lobby.");
+            break;
+        case MENU_ENUM_LABEL_NETPLAY_START_AS_SPECTATOR:
+            snprintf(s, len,
+                     "Whether to start netplay in spectator mode. \n"
+                             " \n"
+                             "If set to true, netplay will be in spectator mode \n"
+                             "on start. It's always possible to change mode \n"
+                             "later.");
+            break;
+        case MENU_ENUM_LABEL_NETPLAY_ALLOW_SLAVES:
+            snprintf(s, len,
+                     "Whether to allow connections in slave mode. \n"
+                             " \n"
+                             "Slave-mode clients require very little processing \n"
+                             "power on either side, but will suffer \n"
+                             "significantly from network latency.");
+            break;
+        case MENU_ENUM_LABEL_NETPLAY_REQUIRE_SLAVES:
+            snprintf(s, len,
+                     "Whether to disallow connections not in slave mode. \n"
+                             " \n"
+                             "Not recommended except for very fast networks \n"
+                             "with very weak machines. \n");
+            break;
         case MENU_ENUM_LABEL_NETPLAY_STATELESS_MODE:
             snprintf(s, len,
                      "Whether to run netplay in a mode not requiring\n"
@@ -1591,6 +1666,18 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                              "connections from the public internet, using\n"
                              "UPnP or similar technologies to escape LANs. \n");
             break;
+        case MENU_ENUM_LABEL_NETPLAY_USE_MITM_SERVER:
+            snprintf(s, len,
+                     "When hosting a netplay session, relay connection through a \n"
+                             "man-in-the-middle server \n"
+                             "to get around firewalls or NAT/UPnP issues. \n");
+            break;
+        case MENU_ENUM_LABEL_NETPLAY_MITM_SERVER:
+            snprintf(s, len,
+                     "Specifies the man-in-the-middle server \n"
+                             "to use for netplay. A server that is \n"
+                             "located closer to you may have less latency. \n");
+            break;
         case MENU_ENUM_LABEL_VIDEO_MAX_SWAPCHAIN_IMAGES:
             snprintf(s, len,
                      "Maximum amount of swapchain images. This \n"
@@ -1690,11 +1777,6 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                      "The username of the person running RetroArch. \n"
                              "This will be used for playing online games.");
             break;
-        case MENU_ENUM_LABEL_NETPLAY_CLIENT_SWAP_INPUT:
-            snprintf(s, len,
-                     "When being client over netplay, use \n"
-                             "keybinds for player 1.");
-            break;
         case MENU_ENUM_LABEL_NETPLAY_TCP_UDP_PORT:
             snprintf(s, len,
                      "The port of the host IP address. \n"
@@ -1810,10 +1892,6 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
             snprintf(s, len,
                      "Saves state.");
             break;
-        case MENU_ENUM_LABEL_NETPLAY_FLIP_PLAYERS:
-            snprintf(s, len,
-                     "Netplay flip users.");
-            break;
         case MENU_ENUM_LABEL_NETPLAY_GAME_WATCH:
             snprintf(s, len,
                      "Netplay toggle play/spectate mode.");
@@ -1851,7 +1929,7 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                      "Hold for fast-forward. Releasing button \n"
                              "disables fast-forward.");
             break;
-        case MENU_ENUM_LABEL_SLOWMOTION:
+        case MENU_ENUM_LABEL_SLOWMOTION_HOLD:
             snprintf(s, len,
                      "Hold for slowmotion.");
             break;
@@ -1927,6 +2005,38 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
                      msg_hash_to_str(MENU_ENUM_LABEL_VALUE_INPUT_OVERLAY_HIDE_IN_MENU)
             );
             break;
+        case MENU_ENUM_LABEL_VALUE_VIDEO_MESSAGE_BGCOLOR_ENABLE:
+            snprintf(s, len,
+                     "Enables a background color for the OSD.");
+            break;
+        case MENU_ENUM_LABEL_VALUE_VIDEO_MESSAGE_BGCOLOR_RED:
+            snprintf(s, len,
+                     "Sets the red value of the OSD background color. Valid values are between 0 and 255.");
+            break;
+        case MENU_ENUM_LABEL_VALUE_VIDEO_MESSAGE_BGCOLOR_GREEN:
+            snprintf(s, len,
+                     "Sets the green value of the OSD background color. Valid values are between 0 and 255.");
+            break;
+        case MENU_ENUM_LABEL_VALUE_VIDEO_MESSAGE_BGCOLOR_BLUE:
+            snprintf(s, len,
+                     "Sets the blue value of the OSD background color. Valid values are between 0 and 255.");
+            break;
+        case MENU_ENUM_LABEL_VALUE_VIDEO_MESSAGE_BGCOLOR_OPACITY:
+            snprintf(s, len,
+                     "Sets the opacity of the OSD background color. Valid values are between 0.0 and 1.0.");
+            break;
+        case MENU_ENUM_LABEL_VALUE_VIDEO_MESSAGE_COLOR_RED:
+            snprintf(s, len,
+                     "Sets the red value of the OSD text color. Valid values are between 0 and 255.");
+            break;
+        case MENU_ENUM_LABEL_VALUE_VIDEO_MESSAGE_COLOR_GREEN:
+            snprintf(s, len,
+                     "Sets the green value of the OSD text color. Valid values are between 0 and 255.");
+            break;
+        case MENU_ENUM_LABEL_VALUE_VIDEO_MESSAGE_COLOR_BLUE:
+            snprintf(s, len,
+                     "Sets the blue value of the OSD text color. Valid values are between 0 and 255.");
+            break;
         default:
             if (string_is_empty(s))
                 strlcpy(s, msg_hash_to_str(MENU_ENUM_LABEL_VALUE_NO_INFORMATION_AVAILABLE), len);
@@ -1935,6 +2045,7 @@ int menu_hash_get_help_us_enum(enum msg_hash_enums msg, char *s, size_t len) {
 
     return 0;
 }
+#endif
 
 #ifdef HAVE_MENU
 static const char *menu_hash_to_str_us_label_enum(enum msg_hash_enums msg)
