@@ -2,7 +2,7 @@
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
  *  Copyright (C) 2016-2017 - Gregor Richards
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -29,10 +29,19 @@
 
 typedef struct netplay netplay_t;
 
+typedef struct mitm_server {
+   const char *name;
+   const char *description;
+} mitm_server_t;
+
+static const mitm_server_t netplay_mitm_server_list[] = {
+   { "nyc", "New York City, USA" },
+   { "madrid", "Madrid, Spain" },
+};
+
 enum rarch_netplay_ctl_state
 {
    RARCH_NETPLAY_CTL_NONE = 0,
-   RARCH_NETPLAY_CTL_FLIP_PLAYERS,
    RARCH_NETPLAY_CTL_GAME_WATCH,
    RARCH_NETPLAY_CTL_POST_FRAME,
    RARCH_NETPLAY_CTL_PRE_FRAME,
@@ -40,11 +49,38 @@ enum rarch_netplay_ctl_state
    RARCH_NETPLAY_CTL_ENABLE_CLIENT,
    RARCH_NETPLAY_CTL_DISABLE,
    RARCH_NETPLAY_CTL_IS_ENABLED,
+   RARCH_NETPLAY_CTL_IS_SERVER,
+   RARCH_NETPLAY_CTL_IS_CONNECTED,
    RARCH_NETPLAY_CTL_IS_DATA_INITED,
    RARCH_NETPLAY_CTL_PAUSE,
    RARCH_NETPLAY_CTL_UNPAUSE,
    RARCH_NETPLAY_CTL_LOAD_SAVESTATE,
-   RARCH_NETPLAY_CTL_DISCONNECT
+   RARCH_NETPLAY_CTL_RESET,
+   RARCH_NETPLAY_CTL_DISCONNECT,
+   RARCH_NETPLAY_CTL_FINISHED_NAT_TRAVERSAL,
+   RARCH_NETPLAY_CTL_DESYNC_PUSH,
+   RARCH_NETPLAY_CTL_DESYNC_POP
+};
+
+/* Preferences for sharing digital devices */
+enum rarch_netplay_share_digital_preference
+{
+   RARCH_NETPLAY_SHARE_DIGITAL_NO_SHARING,
+   RARCH_NETPLAY_SHARE_DIGITAL_NO_PREFERENCE,
+   RARCH_NETPLAY_SHARE_DIGITAL_OR,
+   RARCH_NETPLAY_SHARE_DIGITAL_XOR,
+   RARCH_NETPLAY_SHARE_DIGITAL_VOTE,
+   RARCH_NETPLAY_SHARE_DIGITAL_LAST
+};
+
+/* Preferences for sharing analog devices */
+enum rarch_netplay_share_analog_preference
+{
+   RARCH_NETPLAY_SHARE_ANALOG_NO_SHARING,
+   RARCH_NETPLAY_SHARE_ANALOG_NO_PREFERENCE,
+   RARCH_NETPLAY_SHARE_ANALOG_MAX,
+   RARCH_NETPLAY_SHARE_ANALOG_AVERAGE,
+   RARCH_NETPLAY_SHARE_ANALOG_LAST
 };
 
 int16_t input_state_net(unsigned port, unsigned device,
@@ -77,4 +113,14 @@ void deinit_netplay(void);
 
 bool netplay_driver_ctl(enum rarch_netplay_ctl_state state, void *data);
 
+int netplay_rooms_parse(const char *buf);
+
+struct netplay_room* netplay_room_get(int index);
+
+int netplay_rooms_get_count();
+
+void netplay_rooms_free();
+
+void netplay_get_architecture(char *frontend_architecture, size_t size);
+
 #endif
