@@ -22,21 +22,33 @@
 #include <boolean.h>
 #include <compat/strl.h>
 #include <rhash.h>
+#include <retro_timers.h>
 
 #include "netplay_private.h"
 
+#ifdef HAVE_CONFIG_H
+#include "../../config.h"
+#endif
+
 #include "../../autosave.h"
 #include "../../configuration.h"
 #include "../../content.h"
 #include "../../retroarch.h"
-#include "../../runloop.h"
 #include "../../version.h"
-#include "../../menu/widgets/menu_input_dialog.h"
 
-#ifndef HAVE_SOCKET_LEGACY
+#ifdef HAVE_MENU
+#include "../../menu/widgets/menu_input_dialog.h"
+#endif
+
+const uint32_t netplay_magic = 0x52414E50; /* RANP */
+
+/* TODO/FIXME - replace netplay_log_connection with calls
+ * to inet_ntop_compat and move runloop message queue pushing
+ * outside */
+#if !defined(HAVE_SOCKET_LEGACY) && !defined(WIIU)
 /* Custom inet_ntop. Win32 doesn't seem to support this ... */
 void netplay_log_connection(const struct sockaddr_storage *their_addr,
-      unsigned slot, const char *nick)
+      unsigned slot, const char *nick, char *s, size_t len)
 {
    union
    {
@@ -47,11 +59,8 @@ void netplay_log_connection(const struct sockaddr_storage *their_addr,
    const char *str               = NULL;
    char buf_v4[INET_ADDRSTRLEN]  = {0};
    char buf_v6[INET6_ADDRSTRLEN] = {0};
-   char msg[512];
 
-   msg[0] = '\0';
-
-   u.storage = their_addr;
+   u.storage                     = their_addr;
 
    switch (their_addr->ss_family)
    {
@@ -89,36 +98,23 @@ void netplay_log_connection(const struct sockaddr_storage *their_addr,
 
    if (str)
    {
-      snprintf(msg, sizeof(msg), msg_hash_to_str(MSG_GOT_CONNECTION_FROM_NAME),
+      snprintf(s, len, msg_hash_to_str(MSG_GOT_CONNECTION_FROM_NAME),
             nick, str);
-      runloop_msg_queue_push(msg, 1, 180, false);
-      RARCH_LOG("%s\n", msg);
    }
    else
    {
-      snprintf(msg, sizeof(msg), msg_hash_to_str(MSG_GOT_CONNECTION_FROM),
+      snprintf(s, len, msg_hash_to_str(MSG_GOT_CONNECTION_FROM),
             nick);
-      runloop_msg_queue_push(msg, 1, 180, false);
-      RARCH_LOG("%s\n", msg);
    }
-   RARCH_LOG("%s %u\n", msg_hash_to_str(MSG_CONNECTION_SLOT),
-         slot);
 }
 
 #else
 void netplay_log_connection(const struct sockaddr_storage *their_addr,
-      unsigned slot, const char *nick)
+      unsigned slot, const char *nick, char *s, size_t len)
 {
-   char msg[512];
-
-   msg[0] = '\0';
-
-   snprintf(msg, sizeof(msg), msg_hash_to_str(MSG_GOT_CONNECTION_FROM),
+   /* Stub code - will need to be implemented */
+   snprintf(s, len, msg_hash_to_str(MSG_GOT_CONNECTION_FROM),
          nick);
-   runloop_msg_queue_push(msg, 1, 180, false);
-   RARCH_LOG("%s\n", msg);
-   RARCH_LOG("%s %u\n",
-         msg_hash_to_str(MSG_CONNECTION_SLOT), slot);
 }
 
 #endif
@@ -131,11 +127,11 @@ void netplay_log_connection(const struct sockaddr_storage *their_addr,
  */
 uint32_t netplay_impl_magic(void)
 {
-   size_t i, len;
+   size_t i;
    uint32_t res                        = 0;
    const char *ver                     = PACKAGE_VERSION;
+   size_t len                          = strlen(ver);
 
-   len = strlen(ver);
    for (i = 0; i < len; i++)
       res ^= ver[i] << (i & 0xf);
 
@@ -178,17 +174,18 @@ static bool netplay_endian_mismatch(uint32_t pma, uint32_t pmb)
 
 static unsigned long simple_rand_next = 1;
 
-static int simple_rand()
+static int simple_rand(void)
 {
    simple_rand_next = simple_rand_next * 1103515245 + 12345;
    return((unsigned)(simple_rand_next/65536) % 32768);
 }
 
-static void simple_srand(unsigned int seed) {
+static void simple_srand(unsigned int seed)
+{
    simple_rand_next = seed;
 }
 
-static uint32_t simple_rand_uint32()
+static uint32_t simple_rand_uint32(void)
 {
    uint32_t parts[3];
    parts[0] = simple_rand();
@@ -207,14 +204,19 @@ static uint32_t simple_rand_uint32()
 bool netplay_handshake_init_send(netplay_t *netplay,
    struct netplay_connection *connection)
 {
-   uint32_t header[4] = {0};
+   uint32_t header[6];
    settings_t *settings = config_get_ptr();
 
-   header[0] = htonl(netplay_impl_magic());
+   header[0] = htonl(netplay_magic);
    header[1] = htonl(netplay_platform_magic());
    header[2] = htonl(NETPLAY_COMPRESSION_SUPPORTED);
+   header[3] = 0;
+   header[4] = htonl(NETPLAY_PROTOCOL_VERSION);
+   header[5] = htonl(netplay_impl_magic());
+
    if (netplay->is_server &&
-       (settings->netplay.password[0] || settings->netplay.spectate_password[0]))
+       (settings->paths.netplay_password[0] ||
+        settings->paths.netplay_spectate_password[0]))
    {
       /* Demand a password */
       if (simple_rand_next == 1)
@@ -265,8 +267,9 @@ struct info_buf_s
    } \
    else if (recvd < 0)
 
-static netplay_t *handshake_password_netplay;
+static netplay_t *handshake_password_netplay = NULL;
 
+#ifdef HAVE_MENU
 static void handshake_password(void *ignore, const char *line)
 {
    struct password_buf_s password_buf;
@@ -285,9 +288,12 @@ static void handshake_password(void *ignore, const char *line)
    if (netplay_send(&connection->send_packet_buffer, connection->fd, &password_buf, sizeof(password_buf)))
       netplay_send_flush(&connection->send_packet_buffer, connection->fd, false);
 
+#ifdef HAVE_MENU
    menu_input_dialog_end();
-   rarch_ctl(RARCH_CTL_MENU_RUNNING_FINISHED, NULL);
+   rarch_menu_running_finished();
+#endif
 }
+#endif
 
 /**
  * netplay_handshake_init
@@ -298,31 +304,55 @@ static void handshake_password(void *ignore, const char *line)
 bool netplay_handshake_init(netplay_t *netplay,
    struct netplay_connection *connection, bool *had_input)
 {
-   uint32_t header[4] = {0};
    ssize_t recvd;
-   const char *dmsg;
    struct nick_buf_s nick_buf;
-   uint32_t local_pmagic, remote_pmagic;
-   uint32_t compression;
-   struct compression_transcoder *ctrans;
+   uint32_t header[6];
+   uint32_t local_pmagic                 = 0;
+   uint32_t remote_pmagic                = 0;
+   uint32_t remote_version               = 0;
+   uint32_t compression                  = 0;
+   struct compression_transcoder *ctrans = NULL;
+   const char *dmsg                      = NULL;
 
-   dmsg = NULL;
+   memset(header, 0, sizeof(header));
 
-   RECV(header, sizeof(header))
+   RECV(header, sizeof(uint32_t))
    {
       dmsg = msg_hash_to_str(MSG_FAILED_TO_RECEIVE_HEADER_FROM_CLIENT);
       goto error;
    }
 
-   if (netplay_impl_magic() != ntohl(header[0]))
+   if (ntohl(header[0]) != netplay_magic)
    {
-      dmsg = msg_hash_to_str(MSG_NETPLAY_IMPLEMENTATIONS_DIFFER);
+      dmsg = msg_hash_to_str(MSG_NETPLAY_NOT_RETROARCH);
       goto error;
    }
 
+   RECV(header + 1, sizeof(header) - sizeof(uint32_t))
+   {
+      dmsg = msg_hash_to_str(MSG_FAILED_TO_RECEIVE_HEADER_FROM_CLIENT);
+      goto error;
+   }
+
+   remote_version = ntohl(header[4]);
+   if (remote_version < NETPLAY_PROTOCOL_VERSION)
+   {
+      dmsg = msg_hash_to_str(MSG_NETPLAY_OUT_OF_DATE);
+      goto error;
+   }
+
+   if (ntohl(header[5]) != netplay_impl_magic())
+   {
+      /* We allow the connection but warn that this could cause issues. */
+      dmsg = msg_hash_to_str(MSG_NETPLAY_DIFFERENT_VERSIONS);
+      RARCH_WARN("%s\n", dmsg);
+      runloop_msg_queue_push(dmsg, 1, 180, false);
+   }
+
    /* We only care about platform magic if our core is quirky */
-   local_pmagic = netplay_platform_magic();
+   local_pmagic  = netplay_platform_magic();
    remote_pmagic = ntohl(header[1]);
+
    if ((netplay->quirks & NETPLAY_QUIRK_ENDIAN_DEPENDENT) &&
        netplay_endian_mismatch(local_pmagic, remote_pmagic))
    {
@@ -339,8 +369,9 @@ bool netplay_handshake_init(netplay_t *netplay,
    }
 
    /* Check what compression is supported */
-   compression = ntohl(header[2]);
+   compression  = ntohl(header[2]);
    compression &= NETPLAY_COMPRESSION_SUPPORTED;
+
    if (compression & NETPLAY_COMPRESSION_ZLIB)
    {
       ctrans = &netplay->compress_zlib;
@@ -363,13 +394,14 @@ bool netplay_handshake_init(netplay_t *netplay,
       }
       connection->compression_supported = 0;
    }
+
    if (!ctrans->decompression_backend)
       ctrans->decompression_backend = ctrans->compression_backend->reverse;
 
    /* Allocate our compression stream */
    if (!ctrans->compression_stream)
    {
-      ctrans->compression_stream = ctrans->compression_backend->stream_new();
+      ctrans->compression_stream   = ctrans->compression_backend->stream_new();
       ctrans->decompression_stream = ctrans->decompression_backend->stream_new();
    }
    if (!ctrans->compression_stream || !ctrans->decompression_stream)
@@ -381,15 +413,21 @@ bool netplay_handshake_init(netplay_t *netplay,
    /* If a password is demanded, ask for it */
    if (!netplay->is_server && (connection->salt = ntohl(header[3])))
    {
+#ifdef HAVE_MENU
       menu_input_ctx_line_t line;
-      rarch_ctl(RARCH_CTL_MENU_RUNNING, NULL);
-      memset(&line, 0, sizeof(line));
+      rarch_menu_running();
+#endif
+
       handshake_password_netplay = netplay;
-      line.label = msg_hash_to_str(MSG_NETPLAY_ENTER_PASSWORD);
+
+#ifdef HAVE_MENU
+      memset(&line, 0, sizeof(line));
+      line.label         = msg_hash_to_str(MSG_NETPLAY_ENTER_PASSWORD);
       line.label_setting = "no_setting";
-      line.cb = handshake_password;
+      line.cb            = handshake_password;
       if (!menu_input_dialog_start(&line))
          return false;
+#endif
    }
 
    /* Send our nick */
@@ -417,29 +455,37 @@ error:
    return false;
 }
 
-static void netplay_handshake_ready(netplay_t *netplay, struct netplay_connection *connection)
+static void netplay_handshake_ready(netplay_t *netplay,
+      struct netplay_connection *connection)
 {
    char msg[512];
+   msg[0] = '\0';
 
    if (netplay->is_server)
    {
-      netplay_log_connection(&connection->addr, connection - netplay->connections, connection->nick);
+      unsigned slot = (unsigned)(connection - netplay->connections);
+
+      netplay_log_connection(&connection->addr,
+            slot, connection->nick, msg, sizeof(msg));
+
+      RARCH_LOG("%s %u\n", msg_hash_to_str(MSG_CONNECTION_SLOT), slot);
 
       /* Send them the savestate */
-      if (!(netplay->quirks & (NETPLAY_QUIRK_NO_SAVESTATES|NETPLAY_QUIRK_NO_TRANSMISSION)))
-      {
+      if (!(netplay->quirks &
+               (NETPLAY_QUIRK_NO_SAVESTATES|NETPLAY_QUIRK_NO_TRANSMISSION)))
          netplay->force_send_savestate = true;
-      }
    }
    else
    {
+      netplay->is_connected = true;
       snprintf(msg, sizeof(msg), "%s: \"%s\"",
             msg_hash_to_str(MSG_CONNECTED_TO),
             connection->nick);
-      RARCH_LOG("%s\n", msg);
-      runloop_msg_queue_push(msg, 1, 180, false);
    }
 
+   RARCH_LOG("%s\n", msg);
+   runloop_msg_queue_push(msg, 1, 180, false);
+
    /* Unstall if we were waiting for this */
    if (netplay->stall == NETPLAY_STALL_NO_CONNECTION)
        netplay->stall = NETPLAY_STALL_NONE;
@@ -450,35 +496,38 @@ static void netplay_handshake_ready(netplay_t *netplay, struct netplay_connectio
  *
  * Send an INFO command.
  */
-bool netplay_handshake_info(netplay_t *netplay, struct netplay_connection *connection)
+bool netplay_handshake_info(netplay_t *netplay,
+      struct netplay_connection *connection)
 {
    struct info_buf_s info_buf;
-   rarch_system_info_t *core_info;
-   uint32_t *content_crc_ptr;
+   uint32_t      content_crc      = 0;
+   rarch_system_info_t *core_info = runloop_get_system_info();
 
    memset(&info_buf, 0, sizeof(info_buf));
    info_buf.cmd[0] = htonl(NETPLAY_CMD_INFO);
    info_buf.cmd[1] = htonl(sizeof(info_buf) - 2*sizeof(uint32_t));
 
    /* Get our core info */
-   core_info = NULL;
-   runloop_ctl(RUNLOOP_CTL_SYSTEM_INFO_GET, &core_info);
    if (core_info)
    {
-      strlcpy(info_buf.core_name, core_info->info.library_name, sizeof(info_buf.core_name));
-      strlcpy(info_buf.core_version, core_info->info.library_version, sizeof(info_buf.core_version));
+      strlcpy(info_buf.core_name,
+            core_info->info.library_name, sizeof(info_buf.core_name));
+      strlcpy(info_buf.core_version,
+            core_info->info.library_version, sizeof(info_buf.core_version));
    }
    else
    {
-      strlcpy(info_buf.core_name, "UNKNOWN", sizeof(info_buf.core_name));
-      strlcpy(info_buf.core_version, "UNKNOWN", sizeof(info_buf.core_version));
+      strlcpy(info_buf.core_name,
+            "UNKNOWN", sizeof(info_buf.core_name));
+      strlcpy(info_buf.core_version,
+            "UNKNOWN", sizeof(info_buf.core_version));
    }
 
    /* Get our content CRC */
-   content_crc_ptr = NULL;
-   content_get_crc(&content_crc_ptr);
-   if (content_crc_ptr)
-      info_buf.content_crc = htonl(*content_crc_ptr);
+   content_crc = content_get_crc();
+
+   if (content_crc != 0)
+      info_buf.content_crc = htonl(content_crc);
 
    /* Send it off and wait for info back */
    if (!netplay_send(&connection->send_packet_buffer, connection->fd,
@@ -496,18 +545,18 @@ bool netplay_handshake_info(netplay_t *netplay, struct netplay_connection *conne
  *
  * Send a SYNC command.
  */
-bool netplay_handshake_sync(netplay_t *netplay, struct netplay_connection *connection)
+bool netplay_handshake_sync(netplay_t *netplay,
+      struct netplay_connection *connection)
 {
    /* If we're the server, now we send sync info */
-   uint32_t cmd[5];
-   uint32_t connected_players;
-   settings_t *settings = config_get_ptr();
    size_t i;
-   uint32_t device;
-   retro_ctx_memory_info_t mem_info;
-   size_t nicklen, nickmangle;
    int matchct;
-   bool nick_matched;
+   uint32_t cmd[4];
+   retro_ctx_memory_info_t mem_info;
+   uint32_t client_num        = 0;
+   uint32_t device            = 0;
+   size_t nicklen, nickmangle = 0;
+   bool nick_matched          = false;
 
    autosave_lock();
    mem_info.id = RETRO_MEMORY_SAVE_RAM;
@@ -515,31 +564,54 @@ bool netplay_handshake_sync(netplay_t *netplay, struct netplay_connection *conne
    autosave_unlock();
 
    /* Send basic sync info */
-   cmd[0] = htonl(NETPLAY_CMD_SYNC);
-   cmd[1] = htonl(3*sizeof(uint32_t) + MAX_USERS*sizeof(uint32_t) +
-      NETPLAY_NICK_LEN + mem_info.size);
-   cmd[2] = htonl(netplay->self_frame_count);
-   connected_players = netplay->connected_players;
-   if (netplay->self_mode == NETPLAY_CONNECTION_PLAYING)
-      connected_players |= 1<<netplay->self_player;
+   cmd[0]     = htonl(NETPLAY_CMD_SYNC);
+   cmd[1]     = htonl(2*sizeof(uint32_t)
+         /* Controller devices */
+         + MAX_INPUT_DEVICES*sizeof(uint32_t)
+
+         /* Share modes */
+         + MAX_INPUT_DEVICES*sizeof(uint8_t)
+
+         /* Device-client mapping */
+         + MAX_INPUT_DEVICES*sizeof(uint32_t)
+
+         /* Client nick */
+         + NETPLAY_NICK_LEN
+
+         /* And finally, sram */
+         + mem_info.size);
+   cmd[2]     = htonl(netplay->self_frame_count);
+   client_num = (uint32_t)(connection - netplay->connections + 1);
+    
    if (netplay->local_paused || netplay->remote_paused)
-      connected_players |= NETPLAY_CMD_SYNC_BIT_PAUSED;
-   cmd[3] = htonl(connected_players);
-   if (netplay->flip)
-      cmd[4] = htonl(netplay->flip_frame);
-   else
-      cmd[4] = htonl(0);
+      client_num |= NETPLAY_CMD_SYNC_BIT_PAUSED;
+    
+   cmd[3]     = htonl(client_num);
 
    if (!netplay_send(&connection->send_packet_buffer, connection->fd, cmd,
             sizeof(cmd)))
       return false;
 
    /* Now send the device info */
-   for (i = 0; i < MAX_USERS; i++)
+   for (i = 0; i < MAX_INPUT_DEVICES; i++)
    {
-      device = htonl(settings->input.libretro_device[i]);
+      device = htonl(netplay->config_devices[i]);
       if (!netplay_send(&connection->send_packet_buffer, connection->fd,
-               &device, sizeof(device)))
+            &device, sizeof(device)))
+         return false;
+   }
+
+   /* Then the share mode */
+   if (!netplay_send(&connection->send_packet_buffer, connection->fd,
+         netplay->device_share_modes, sizeof(netplay->device_share_modes)))
+      return false;
+
+   /* Then the device-client mapping */
+   for (i = 0; i < MAX_INPUT_DEVICES; i++)
+   {
+      device = htonl(netplay->device_clients[i]);
+      if (!netplay_send(&connection->send_packet_buffer, connection->fd,
+            &device, sizeof(device)))
          return false;
    }
 
@@ -570,7 +642,8 @@ bool netplay_handshake_sync(netplay_t *netplay, struct netplay_connection *conne
       if (nick_matched)
       {
          /* Somebody has this nick, make a new one! */
-         snprintf(connection->nick + nickmangle, NETPLAY_NICK_LEN - nickmangle, " (%d)", ++matchct);
+         snprintf(connection->nick + nickmangle,
+               NETPLAY_NICK_LEN - nickmangle, " (%d)", ++matchct);
          connection->nick[NETPLAY_NICK_LEN - 1] = '\0';
       }
    } while (nick_matched);
@@ -611,7 +684,6 @@ bool netplay_handshake_pre_nick(netplay_t *netplay,
    struct nick_buf_s nick_buf;
    ssize_t recvd;
    char msg[512];
-   settings_t *settings = config_get_ptr();
 
    msg[0] = '\0';
 
@@ -640,7 +712,10 @@ bool netplay_handshake_pre_nick(netplay_t *netplay,
 
    if (netplay->is_server)
    {
-      if (settings->netplay.password[0] || settings->netplay.spectate_password[0])
+      settings_t *settings = config_get_ptr();
+
+      if (  settings->paths.netplay_password[0] ||
+            settings->paths.netplay_spectate_password[0])
       {
          /* There's a password, so just put them in PRE_PASSWORD mode */
          connection->mode = NETPLAY_CONNECTION_PRE_PASSWORD;
@@ -679,7 +754,7 @@ bool netplay_handshake_pre_password(netplay_t *netplay,
    char password[8+NETPLAY_PASS_LEN]; /* 8 for salt */
    ssize_t recvd;
    char msg[512];
-   bool correct;
+   bool correct         = false;
    settings_t *settings = config_get_ptr();
 
    msg[0] = '\0';
@@ -706,21 +781,32 @@ bool netplay_handshake_pre_password(netplay_t *netplay,
    /* Calculate the correct password hash(es) and compare */
    correct = false;
    snprintf(password, sizeof(password), "%08X", connection->salt);
-   if (settings->netplay.password[0])
+
+   if (settings->paths.netplay_password[0])
    {
-      strlcpy(password + 8, settings->netplay.password, sizeof(password)-8);
-      sha256_hash(corr_password_buf.password, (uint8_t *) password, strlen(password));
-      if (!memcmp(password_buf.password, corr_password_buf.password, sizeof(password_buf.password)))
+      strlcpy(password + 8,
+            settings->paths.netplay_password, sizeof(password)-8);
+
+      sha256_hash(corr_password_buf.password,
+            (uint8_t *) password, strlen(password));
+
+      if (!memcmp(password_buf.password,
+               corr_password_buf.password, sizeof(password_buf.password)))
       {
          correct = true;
          connection->can_play = true;
       }
    }
-   if (settings->netplay.spectate_password[0])
+   if (settings->paths.netplay_spectate_password[0])
    {
-      strlcpy(password + 8, settings->netplay.spectate_password, sizeof(password)-8);
-      sha256_hash(corr_password_buf.password, (uint8_t *) password, strlen(password));
-      if (!memcmp(password_buf.password, corr_password_buf.password, sizeof(password_buf.password)))
+      strlcpy(password + 8,
+            settings->paths.netplay_spectate_password, sizeof(password)-8);
+
+      sha256_hash(corr_password_buf.password,
+            (uint8_t *) password, strlen(password));
+
+      if (!memcmp(password_buf.password,
+               corr_password_buf.password, sizeof(password_buf.password)))
          correct = true;
    }
 
@@ -750,11 +836,11 @@ bool netplay_handshake_pre_info(netplay_t *netplay,
    struct info_buf_s info_buf;
    uint32_t cmd_size;
    ssize_t recvd;
-   rarch_system_info_t *core_info;
-   uint32_t *content_crc_ptr;
-   const char *dmsg = NULL;
+   uint32_t content_crc           = 0;
+   const char *dmsg               = NULL;
+   rarch_system_info_t *core_info = runloop_get_system_info();
 
-   RECV(&info_buf, sizeof(info_buf)) {}
+   RECV(&info_buf, sizeof(info_buf.cmd)) {}
 
    if (recvd < 0 ||
        ntohl(info_buf.cmd[0]) != NETPLAY_CMD_INFO)
@@ -783,28 +869,45 @@ bool netplay_handshake_pre_info(netplay_t *netplay,
       return true;
    }
 
+   RECV(&info_buf.core_name, cmd_size)
+   {
+      RARCH_ERR("Failed to receive netplay info payload.\n");
+      return false;
+   }
+
    /* Check the core info */
-   core_info = NULL;
-   runloop_ctl(RUNLOOP_CTL_SYSTEM_INFO_GET, &core_info);
+
    if (core_info)
    {
-      if (strncmp(info_buf.core_name, core_info->info.library_name, sizeof(info_buf.core_name)) ||
-          strncmp(info_buf.core_version, core_info->info.library_version, sizeof(info_buf.core_version)))
+      if (strncmp(info_buf.core_name,
+               core_info->info.library_name, sizeof(info_buf.core_name)))
       {
-         dmsg = msg_hash_to_str(MSG_NETPLAY_IMPLEMENTATIONS_DIFFER);
-         goto error;
+         /* Wrong core! */
+         dmsg = msg_hash_to_str(MSG_NETPLAY_DIFFERENT_CORES);
+         RARCH_ERR("%s\n", dmsg);
+         runloop_msg_queue_push(dmsg, 1, 180, false);
+         /* FIXME: Should still send INFO, so the other side knows what's what */
+         return false;
+      }
+      if (strncmp(info_buf.core_version,
+             core_info->info.library_version, sizeof(info_buf.core_version)))
+      {
+         dmsg = msg_hash_to_str(MSG_NETPLAY_DIFFERENT_CORE_VERSIONS);
+         RARCH_WARN("%s\n", dmsg);
+         runloop_msg_queue_push(dmsg, 1, 180, false);
       }
    }
 
    /* Check the content CRC */
-   content_crc_ptr = NULL;
-   content_get_crc(&content_crc_ptr);
-   if (content_crc_ptr)
+   content_crc = content_get_crc();
+
+   if (content_crc != 0)
    {
-      if (ntohl(info_buf.content_crc) != *content_crc_ptr)
+      if (ntohl(info_buf.content_crc) != content_crc)
       {
          dmsg = msg_hash_to_str(MSG_CONTENT_CRC32S_DIFFER);
-         goto error;
+         RARCH_WARN("%s\n", dmsg);
+         runloop_msg_queue_push(dmsg, 1, 180, false);
       }
    }
 
@@ -825,24 +928,8 @@ bool netplay_handshake_pre_info(netplay_t *netplay,
    *had_input = true;
    netplay_recv_flush(&connection->recv_packet_buffer);
    return true;
-
-error:
-   if (dmsg)
-   {
-      RARCH_ERR("%s\n", dmsg);
-      runloop_msg_queue_push(dmsg, 1, 180, false);
-   }
-
-   if (!netplay->is_server)
-   {
-      /* Counter-intuitively, we still want to send our info. This is simply so
-       * that the server knows why we disconnected. */
-      if (!netplay_handshake_info(netplay, connection))
-         return false;
-   }
-
-   return false;
 }
+
 /**
  * netplay_handshake_pre_sync
  *
@@ -853,14 +940,15 @@ bool netplay_handshake_pre_sync(netplay_t *netplay,
    struct netplay_connection *connection, bool *had_input)
 {
    uint32_t cmd[2];
-   uint32_t new_frame_count, connected_players, flip_frame;
+   uint32_t new_frame_count, client_num;
    uint32_t device;
    uint32_t local_sram_size, remote_sram_size;
-   size_t i;
+   size_t i, j;
    ssize_t recvd;
    retro_ctx_controller_info_t pad;
    char new_nick[NETPLAY_NICK_LEN];
    retro_ctx_memory_info_t mem_info;
+   settings_t *settings = config_get_ptr();
 
    RECV(cmd, sizeof(cmd))
    {
@@ -872,7 +960,7 @@ bool netplay_handshake_pre_sync(netplay_t *netplay,
 
    /* Only expecting a sync command */
    if (ntohl(cmd[0]) != NETPLAY_CMD_SYNC ||
-       ntohl(cmd[1]) < 3*sizeof(uint32_t) + MAX_USERS*sizeof(uint32_t) +
+         ntohl(cmd[1]) < (2+2*MAX_INPUT_DEVICES)*sizeof(uint32_t) + (MAX_INPUT_DEVICES)*sizeof(uint8_t) +
          NETPLAY_NICK_LEN)
    {
       RARCH_ERR("%s\n",
@@ -885,72 +973,101 @@ bool netplay_handshake_pre_sync(netplay_t *netplay,
       return false;
    new_frame_count = ntohl(new_frame_count);
 
-   /* Get the connected players and pause mode */
-   RECV(&connected_players, sizeof(connected_players))
+   /* Get our client number and pause mode */
+   RECV(&client_num, sizeof(client_num))
       return false;
-   connected_players = ntohl(connected_players);
-   if (connected_players & NETPLAY_CMD_SYNC_BIT_PAUSED)
+   client_num = ntohl(client_num);
+   if (client_num & NETPLAY_CMD_SYNC_BIT_PAUSED)
    {
       netplay->remote_paused = true;
-      connected_players ^= NETPLAY_CMD_SYNC_BIT_PAUSED;
+      client_num ^= NETPLAY_CMD_SYNC_BIT_PAUSED;
    }
-   netplay->connected_players = connected_players;
-
-   /* And the flip state */
-   RECV(&flip_frame, sizeof(flip_frame))
-      return false;
-   flip_frame = ntohl(flip_frame);
-   netplay->flip = !!flip_frame;
-   netplay->flip_frame = flip_frame;
+   netplay->self_client_num = client_num;
 
    /* Set our frame counters as requested */
-   netplay->self_frame_count = netplay->run_frame_count =
-      netplay->other_frame_count = netplay->unread_frame_count =
+   netplay->self_frame_count      = netplay->run_frame_count    =
+      netplay->other_frame_count  = netplay->unread_frame_count =
       netplay->server_frame_count = new_frame_count;
+
+   /* And clear out the framebuffer */
    for (i = 0; i < netplay->buffer_size; i++)
    {
       struct delta_frame *ptr = &netplay->buffer[i];
-      ptr->used = false;
+
+      ptr->used               = false;
 
       if (i == netplay->self_ptr)
       {
          /* Clear out any current data but still use this frame */
          if (!netplay_delta_frame_ready(netplay, ptr, 0))
             return false;
-         ptr->frame = new_frame_count;
-         ptr->have_local = true;
+
+         ptr->frame       = new_frame_count;
+         ptr->have_local  = true;
          netplay->run_ptr = netplay->other_ptr = netplay->unread_ptr =
             netplay->server_ptr = i;
 
       }
    }
-   for (i = 0; i < MAX_USERS; i++)
+   for (i = 0; i < MAX_CLIENTS; i++)
    {
-      if (connected_players & (1<<i))
-      {
-         netplay->read_ptr[i] = netplay->self_ptr;
-         netplay->read_frame_count[i] = netplay->self_frame_count;
-      }
+      netplay->read_ptr[i]         = netplay->self_ptr;
+      netplay->read_frame_count[i] = netplay->self_frame_count;
    }
 
-   /* Get and set each pad */
-   for (i = 0; i < MAX_USERS; i++)
+   /* Get and set each input device */
+   for (i = 0; i < MAX_INPUT_DEVICES; i++)
    {
       RECV(&device, sizeof(device))
          return false;
-      pad.port = i;
+
+      pad.port   = (unsigned)i;
       pad.device = ntohl(device);
+      netplay->config_devices[i] = pad.device;
+      if ((pad.device&RETRO_DEVICE_MASK) == RETRO_DEVICE_KEYBOARD)
+      {
+         netplay->have_updown_device = true;
+         netplay_key_hton_init();
+      }
+
       core_set_controller_port_device(&pad);
    }
 
+   /* Get the share modes */
+   RECV(netplay->device_share_modes, sizeof(netplay->device_share_modes))
+      return false;
+
+   /* Get the client-controller mapping */
+   netplay->connected_players =
+         netplay->connected_slaves =
+         netplay->self_devices = 0;
+   for (i = 0; i < MAX_CLIENTS; i++)
+      netplay->client_devices[i] = 0;
+   for (i = 0; i < MAX_INPUT_DEVICES; i++)
+   {
+      RECV(&device, sizeof(device))
+         return false;
+      device = ntohl(device);
+
+      netplay->device_clients[i] = device;
+      netplay->connected_players |= device;
+      for (j = 0; j < MAX_CLIENTS; j++)
+      {
+         if (device & (1<<j))
+            netplay->client_devices[j] |= 1<<i;
+      }
+   }
+
    /* Get our nick */
    RECV(new_nick, NETPLAY_NICK_LEN)
       return false;
+
    if (strncmp(netplay->nick, new_nick, NETPLAY_NICK_LEN))
    {
       char msg[512];
       strlcpy(netplay->nick, new_nick, NETPLAY_NICK_LEN);
-      snprintf(msg, sizeof(msg), msg_hash_to_str(MSG_NETPLAY_CHANGED_NICK), netplay->nick);
+      snprintf(msg, sizeof(msg),
+            msg_hash_to_str(MSG_NETPLAY_CHANGED_NICK), netplay->nick);
       RARCH_LOG("%s\n", msg);
       runloop_msg_queue_push(msg, 1, 180, false);
    }
@@ -960,9 +1077,9 @@ bool netplay_handshake_pre_sync(netplay_t *netplay,
    mem_info.id = RETRO_MEMORY_SAVE_RAM;
    core_get_memory(&mem_info);
 
-   local_sram_size = mem_info.size;
-   remote_sram_size = ntohl(cmd[1]) - 3*sizeof(uint32_t) -
-      MAX_USERS*sizeof(uint32_t) - NETPLAY_NICK_LEN;
+   local_sram_size  = (unsigned)mem_info.size;
+   remote_sram_size = ntohl(cmd[1]) -
+         (2+2*MAX_INPUT_DEVICES)*sizeof(uint32_t) - (MAX_INPUT_DEVICES)*sizeof(uint8_t) - NETPLAY_NICK_LEN;
 
    if (local_sram_size != 0 && local_sram_size == remote_sram_size)
    {
@@ -1004,8 +1121,11 @@ bool netplay_handshake_pre_sync(netplay_t *netplay,
    netplay_handshake_ready(netplay, connection);
    netplay_recv_flush(&connection->recv_packet_buffer);
 
-   /* Ask to go to player mode */
-   return netplay_cmd_mode(netplay, connection, NETPLAY_CONNECTION_PLAYING);
+   /* Ask to switch to playing mode if we should */
+   if (!settings->bools.netplay_start_as_spectator)
+      return netplay_cmd_mode(netplay, NETPLAY_CONNECTION_PLAYING);
+
+   return true;
 }
 
 /**
@@ -1016,13 +1136,10 @@ bool netplay_handshake_pre_sync(netplay_t *netplay,
 bool netplay_handshake(netplay_t *netplay,
    struct netplay_connection *connection, bool *had_input)
 {
-   bool ret;
+   bool ret = false;
 
    switch (connection->mode)
    {
-      case NETPLAY_CONNECTION_NONE:
-         /* Huh?! */
-         return false;
       case NETPLAY_CONNECTION_INIT:
          ret = netplay_handshake_init(netplay, connection, had_input);
          break;
@@ -1038,6 +1155,7 @@ bool netplay_handshake(netplay_t *netplay,
       case NETPLAY_CONNECTION_PRE_SYNC:
          ret = netplay_handshake_pre_sync(netplay, connection, had_input);
          break;
+      case NETPLAY_CONNECTION_NONE:
       default:
          return false;
    }
