@@ -19,6 +19,7 @@
 #include <file/file_path.h>
 #include <compat/strl.h>
 #include <string/stdstring.h>
+#include <retro_math.h>
 
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
@@ -29,8 +30,8 @@
 #include "../command.h"
 #include "../configuration.h"
 #include "../driver.h"
+#include "../gfx/video_driver.h"
 #include "../retroarch.h"
-#include "../runloop.h"
 #include "../verbosity.h"
 #include "../msg_hash.h"
 #include "../list_special.h"
@@ -103,24 +104,27 @@ void find_record_driver(void)
    settings_t *settings = config_get_ptr();
 
    drv.label = "record_driver";
-   drv.s     = settings->record.driver;
+   drv.s     = settings->arrays.record_driver;
 
    driver_ctl(RARCH_DRIVER_CTL_FIND_INDEX, &drv);
 
-   i = drv.len;
+   i = (int)drv.len;
 
    if (i >= 0)
       recording_driver = (const record_driver_t*)record_driver_find_handle(i);
    else
    {
-      unsigned d;
+      if (verbosity_is_enabled())
+      {
+         unsigned d;
 
-      RARCH_ERR("Couldn't find any record driver named \"%s\"\n",
-            settings->audio.driver);
-      RARCH_LOG_OUTPUT("Available record drivers are:\n");
-      for (d = 0; record_driver_find_handle(d); d++)
-         RARCH_LOG_OUTPUT("\t%s\n", record_driver_find_ident(d));
-      RARCH_WARN("Going to default to first record driver...\n");
+         RARCH_ERR("Couldn't find any record driver named \"%s\"\n",
+               settings->arrays.record_driver);
+         RARCH_LOG_OUTPUT("Available record drivers are:\n");
+         for (d = 0; record_driver_find_handle(d); d++)
+            RARCH_LOG_OUTPUT("\t%s\n", record_driver_find_ident(d));
+         RARCH_WARN("Going to default to first record driver...\n");
+      }
 
       recording_driver = (const record_driver_t*)record_driver_find_handle(0);
 
@@ -187,20 +191,27 @@ void recording_dump_frame(const void *data, unsigned width,
 {
    bool has_gpu_record = false;
    uint8_t *gpu_buf    = NULL;
-   struct ffemu_video_data 
+   struct ffemu_video_data
       ffemu_data       = {0};
 
    video_driver_get_record_status(&has_gpu_record,
          &gpu_buf);
 
-   ffemu_data.pitch    = pitch;
+   ffemu_data.pitch    = (int)pitch;
    ffemu_data.width    = width;
    ffemu_data.height   = height;
    ffemu_data.data     = data;
 
    if (has_gpu_record)
    {
-      struct video_viewport vp = {0};
+      struct video_viewport vp;
+
+      vp.x                        = 0;
+      vp.y                        = 0;
+      vp.width                    = 0;
+      vp.height                   = 0;
+      vp.full_width               = 0;
+      vp.full_height              = 0;
 
       video_driver_get_viewport_info(&vp);
 
@@ -236,9 +247,9 @@ void recording_dump_frame(const void *data, unsigned width,
       if (!video_driver_read_viewport(gpu_buf, is_idle))
          return;
 
-      ffemu_data.pitch  = recording_gpu_width * 3;
-      ffemu_data.width  = recording_gpu_width;
-      ffemu_data.height = recording_gpu_height;
+      ffemu_data.pitch  = (int)(recording_gpu_width * 3);
+      ffemu_data.width  = (unsigned)recording_gpu_width;
+      ffemu_data.height = (unsigned)recording_gpu_height;
       ffemu_data.data   = gpu_buf + (ffemu_data.height - 1) * ffemu_data.pitch;
 
       ffemu_data.pitch  = -ffemu_data.pitch;
@@ -319,7 +330,7 @@ bool recording_init(void)
       return false;
    }
 
-   if (!settings->video.gpu_record 
+   if (!settings->bools.video_gpu_record
          && video_driver_is_hw_context())
    {
       RARCH_WARN("%s.\n",
@@ -350,14 +361,21 @@ bool recording_init(void)
    params.pix_fmt    = (video_driver_get_pixel_format() == RETRO_PIXEL_FORMAT_XRGB8888) ?
       FFEMU_PIX_ARGB8888 : FFEMU_PIX_RGB565;
    params.config     = NULL;
-   
+
    if (!string_is_empty(global->record.config))
       params.config = global->record.config;
 
    if (video_driver_supports_recording())
    {
       unsigned gpu_size;
-      struct video_viewport vp = {0};
+      struct video_viewport vp;
+
+      vp.x                        = 0;
+      vp.y                        = 0;
+      vp.width                    = 0;
+      vp.height                   = 0;
+      vp.full_width               = 0;
+      vp.full_height              = 0;
 
       video_driver_get_viewport_info(&vp);
 
@@ -373,7 +391,7 @@ bool recording_init(void)
       params.fb_width   = next_pow2(vp.width);
       params.fb_height  = next_pow2(vp.height);
 
-      if (settings->video.force_aspect &&
+      if (settings->bools.video_force_aspect &&
             (video_driver_get_aspect_ratio() > 0.0f))
          params.aspect_ratio  = video_driver_get_aspect_ratio();
       else
@@ -398,18 +416,18 @@ bool recording_init(void)
          params.out_height = recording_height;
       }
 
-      if (settings->video.force_aspect &&
+      if (settings->bools.video_force_aspect &&
             (video_driver_get_aspect_ratio() > 0.0f))
          params.aspect_ratio = video_driver_get_aspect_ratio();
       else
          params.aspect_ratio = (float)params.out_width / params.out_height;
 
-      if (settings->video.post_filter_record 
+      if (settings->bools.video_post_filter_record
             && video_driver_frame_filter_alive())
       {
          unsigned max_width  = 0;
          unsigned max_height = 0;
-         
+
          params.pix_fmt    = FFEMU_PIX_RGB565;
 
          if (video_driver_frame_filter_is_32bit())
