@@ -1,5 +1,5 @@
 /*  RetroArch - A frontend for libretro.
- *  Copyright (C) 2011-2016 - Daniel De Matteis
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
  *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
@@ -23,6 +23,9 @@
 #include "config.h"
 #endif
 
+#include "input/input_driver.h"
+#include "config.def.keybinds.h"
+
 #ifdef HAVE_MENU
 #include "menu/menu_driver.h"
 #include "menu/widgets/menu_input_dialog.h"
@@ -31,59 +34,8 @@
 
 #include "configuration.h"
 #include "config.def.h"
-#include "input/input_config.h"
 #include "setting_list.h"
 
-rarch_setting_t setting_terminator_setting(void)
-{
-   rarch_setting_t result;
-
-   result.enum_idx                  = MSG_UNKNOWN;
-   result.type                      = ST_NONE;
-  
-   result.size                      = 0;
-
-   result.name                      = NULL;
-   result.name_hash                 = 0;
-   result.short_description         = NULL;
-   result.group                     = NULL;
-   result.subgroup                  = NULL;
-   result.parent_group              = NULL;
-   result.values                    = NULL;
-
-   result.index                     = 0;
-   result.index_offset              = 0;
-
-   result.min                       = 0.0;
-   result.max                       = 0.0;
-
-   result.flags                     = 0;
-   result.free_flags                = 0;
-
-   result.change_handler            = NULL;
-   result.read_handler              = NULL;
-   result.action_start              = NULL;
-   result.action_left               = NULL;
-   result.action_right              = NULL;
-   result.action_up                 = NULL;
-   result.action_down               = NULL;
-   result.action_cancel             = NULL;
-   result.action_ok                 = NULL;
-   result.action_select             = NULL;
-   result.get_string_representation = NULL;
-
-   result.bind_type                 = 0;
-   result.browser_selection_type    = ST_NONE;
-   result.step                      = 0.0f;
-   result.rounding_fraction         = NULL;
-   result.enforce_minrange          = false;
-   result.enforce_maxrange          = false;
-
-   result.dont_use_enum_idx_representation = false;
-
-   return result;
-}
-
 bool settings_list_append(rarch_setting_t **list,
       rarch_setting_info_t *list_info)
 {
@@ -124,13 +76,11 @@ static int setting_bind_action_ok(void *data, bool wraparound)
 static int setting_int_action_right_default(void *data, bool wraparound)
 {
    rarch_setting_t *setting = (rarch_setting_t*)data;
-   double               min = 0.0f;
    double               max = 0.0f;
-   
+
    if (!setting)
       return -1;
 
-   min = setting->min;
    max = setting->max;
 
    (void)wraparound; /* TODO/FIXME - handle this */
@@ -143,9 +93,10 @@ static int setting_int_action_right_default(void *data, bool wraparound)
       if (*setting->value.target.integer > max)
       {
          settings_t *settings = config_get_ptr();
-
 #ifdef HAVE_MENU
-         if (settings && settings->menu.navigation.wraparound.enable)
+         double          min  = setting->min;
+
+         if (settings && settings->bools.menu_navigation_wraparound_enable)
             *setting->value.target.integer = min;
          else
 #endif
@@ -180,6 +131,8 @@ static int setting_bind_action_start(void *data)
    bind_type    = setting_get_bind_type(setting);
    keybind->key = def_binds[bind_type - MENU_SETTINGS_BIND_BEGIN].key;
 
+   keybind->mbutton = NO_BTN;
+
    return 0;
 }
 #endif
@@ -206,7 +159,8 @@ static int setting_uint_action_left_default(void *data, bool wraparound)
 {
    rarch_setting_t *setting = (rarch_setting_t*)data;
    double               min = 0.0f;
-   
+   bool                 overflowed = false;
+
    if (!setting)
       return -1;
 
@@ -214,16 +168,27 @@ static int setting_uint_action_left_default(void *data, bool wraparound)
 
    (void)wraparound; /* TODO/FIXME - handle this */
 
-   if (*setting->value.target.unsigned_integer != min)
-      *setting->value.target.unsigned_integer =
-         *setting->value.target.unsigned_integer - setting->step;
+   overflowed = setting->step > *setting->value.target.unsigned_integer;
+
+   if (!overflowed)
+      *setting->value.target.unsigned_integer = *setting->value.target.unsigned_integer - setting->step;
 
    if (setting->enforce_minrange)
    {
-      if (*setting->value.target.unsigned_integer < min)
-         *setting->value.target.unsigned_integer = min;
-   }
+      if (overflowed || *setting->value.target.unsigned_integer < min)
+      {
+         settings_t *settings = config_get_ptr();
 
+#ifdef HAVE_MENU
+      double           max = setting->max;
+
+         if (settings && settings->bools.menu_navigation_wraparound_enable)
+            *setting->value.target.unsigned_integer = max;
+         else
+#endif
+            *setting->value.target.unsigned_integer = min;
+      }
+   }
 
    return 0;
 }
@@ -231,18 +196,15 @@ static int setting_uint_action_left_default(void *data, bool wraparound)
 static int setting_uint_action_right_default(void *data, bool wraparound)
 {
    rarch_setting_t *setting = (rarch_setting_t*)data;
-   double               min = 0.0f;
    double               max = 0.0f;
-   
+
    if (!setting)
       return -1;
 
-   min = setting->min;
    max = setting->max;
 
    (void)wraparound; /* TODO/FIXME - handle this */
 
-
    *setting->value.target.unsigned_integer =
       *setting->value.target.unsigned_integer + setting->step;
 
@@ -253,7 +215,9 @@ static int setting_uint_action_right_default(void *data, bool wraparound)
          settings_t *settings = config_get_ptr();
 
 #ifdef HAVE_MENU
-         if (settings && settings->menu.navigation.wraparound.enable)
+         double           min = setting->min;
+
+         if (settings && settings->bools.menu_navigation_wraparound_enable)
             *setting->value.target.unsigned_integer = min;
          else
 #endif
@@ -318,10 +282,9 @@ int setting_set_with_string_representation(rarch_setting_t* setting,
                *setting->value.target.integer = min;
             if (setting->enforce_maxrange && *setting->value.target.integer > max)
             {
-               settings_t *settings = config_get_ptr();
-
 #ifdef HAVE_MENU
-               if (settings && settings->menu.navigation.wraparound.enable)
+               settings_t *settings = config_get_ptr();
+               if (settings && settings->bools.menu_navigation_wraparound_enable)
                   *setting->value.target.integer = min;
                else
 #endif
@@ -337,17 +300,16 @@ int setting_set_with_string_representation(rarch_setting_t* setting,
                *setting->value.target.unsigned_integer = min;
             if (setting->enforce_maxrange && *setting->value.target.unsigned_integer > max)
             {
-               settings_t *settings = config_get_ptr();
-
 #ifdef HAVE_MENU
-               if (settings && settings->menu.navigation.wraparound.enable)
+               settings_t *settings = config_get_ptr();
+               if (settings && settings->bools.menu_navigation_wraparound_enable)
                   *setting->value.target.unsigned_integer = min;
                else
 #endif
                   *setting->value.target.unsigned_integer = max;
             }
          }
-         break;      
+         break;
       case ST_FLOAT:
          sscanf(value, "%f", setting->value.target.fraction);
          if (flags & SD_FLAG_HAS_RANGE)
@@ -356,10 +318,9 @@ int setting_set_with_string_representation(rarch_setting_t* setting,
                *setting->value.target.fraction = min;
             if (setting->enforce_maxrange && *setting->value.target.fraction > max)
             {
-               settings_t *settings = config_get_ptr();
-
 #ifdef HAVE_MENU
-               if (settings && settings->menu.navigation.wraparound.enable)
+               settings_t *settings = config_get_ptr();
+               if (settings && settings->bools.menu_navigation_wraparound_enable)
                   *setting->value.target.fraction = min;
                else
 #endif
@@ -395,7 +356,7 @@ static int setting_fraction_action_left_default(
 {
    rarch_setting_t *setting = (rarch_setting_t*)data;
    double               min = 0.0f;
-   
+
    if (!setting)
       return -1;
 
@@ -403,13 +364,23 @@ static int setting_fraction_action_left_default(
 
    (void)wraparound; /* TODO/FIXME - handle this */
 
-   *setting->value.target.fraction =
-      *setting->value.target.fraction - setting->step;
+   *setting->value.target.fraction = *setting->value.target.fraction - setting->step;
 
    if (setting->enforce_minrange)
    {
       if (*setting->value.target.fraction < min)
-         *setting->value.target.fraction = min;
+      {
+         settings_t *settings = config_get_ptr();
+
+#ifdef HAVE_MENU
+      double           max = setting->max;
+
+         if (settings && settings->bools.menu_navigation_wraparound_enable)
+            *setting->value.target.fraction = max;
+         else
+#endif
+            *setting->value.target.fraction = min;
+      }
    }
 
    return 0;
@@ -419,28 +390,27 @@ static int setting_fraction_action_right_default(
       void *data, bool wraparound)
 {
    rarch_setting_t *setting = (rarch_setting_t*)data;
-   double               min = 0.0f;
    double               max = 0.0f;
 
    if (!setting)
       return -1;
 
-   min = setting->min;
    max = setting->max;
 
    (void)wraparound; /* TODO/FIXME - handle this */
 
-   *setting->value.target.fraction = 
+   *setting->value.target.fraction =
       *setting->value.target.fraction + setting->step;
 
    if (setting->enforce_maxrange)
    {
       if (*setting->value.target.fraction > max)
       {
-         settings_t *settings = config_get_ptr();
-
 #ifdef HAVE_MENU
-         if (settings && settings->menu.navigation.wraparound.enable)
+         settings_t *settings = config_get_ptr();
+         double          min  = setting->min;
+
+         if (settings && settings->bools.menu_navigation_wraparound_enable)
             *setting->value.target.fraction = min;
          else
 #endif
@@ -644,11 +614,10 @@ static rarch_setting_t setting_action_setting(const char* name,
 
    result.enum_idx                  = MSG_UNKNOWN;
    result.type                      = ST_ACTION;
-  
+
    result.size                      = 0;
 
    result.name                      = name;
-   result.name_hash                 = 0;
    result.short_description         = short_description;
    result.group                     = group;
    result.subgroup                  = subgroup;
@@ -683,6 +652,9 @@ static rarch_setting_t setting_action_setting(const char* name,
    result.enforce_minrange          = false;
    result.enforce_maxrange          = false;
 
+   result.cmd_trigger.idx           = CMD_EVENT_NONE;
+   result.cmd_trigger.triggered     = false;
+
    result.dont_use_enum_idx_representation = dont_use_enum_idx;
 
    return result;
@@ -704,11 +676,10 @@ static rarch_setting_t setting_group_setting(enum setting_type type, const char*
 
    result.enum_idx                  = MSG_UNKNOWN;
    result.type                      = type;
-  
+
    result.size                      = 0;
 
    result.name                      = name;
-   result.name_hash                 = 0;
    result.short_description         = name;
    result.group                     = NULL;
    result.subgroup                  = NULL;
@@ -743,6 +714,9 @@ static rarch_setting_t setting_group_setting(enum setting_type type, const char*
    result.enforce_minrange          = false;
    result.enforce_maxrange          = false;
 
+   result.cmd_trigger.idx           = CMD_EVENT_NONE;
+   result.cmd_trigger.triggered     = false;
+
    result.dont_use_enum_idx_representation = false;
 
    return result;
@@ -775,11 +749,10 @@ static rarch_setting_t setting_float_setting(const char* name,
 
    result.enum_idx                  = MSG_UNKNOWN;
    result.type                      = ST_FLOAT;
-  
+
    result.size                      = sizeof(float);
 
    result.name                      = name;
-   result.name_hash                 = 0;
    result.short_description         = short_description;
    result.group                     = group;
    result.subgroup                  = subgroup;
@@ -818,6 +791,9 @@ static rarch_setting_t setting_float_setting(const char* name,
    result.original_value.fraction   = *target;
    result.default_value.fraction    = default_value;
 
+   result.cmd_trigger.idx           = CMD_EVENT_NONE;
+   result.cmd_trigger.triggered     = false;
+
    result.dont_use_enum_idx_representation = dont_use_enum_idx;
 
    return result;
@@ -834,7 +810,7 @@ static rarch_setting_t setting_float_setting(const char* name,
  * @change_handler     : Function callback for change handler function pointer.
  * @read_handler       : Function callback for read handler function pointer.
  *
- * Initializes a setting of type ST_UINT. 
+ * Initializes a setting of type ST_UINT.
  *
  * Returns: setting of type ST_UINT.
  **/
@@ -849,11 +825,10 @@ static rarch_setting_t setting_uint_setting(const char* name,
 
    result.enum_idx                  = MSG_UNKNOWN;
    result.type                      = ST_UINT;
-  
+
    result.size                      = sizeof(unsigned int);
 
    result.name                      = name;
-   result.name_hash                 = 0;
    result.short_description         = short_description;
    result.group                     = group;
    result.subgroup                  = subgroup;
@@ -892,6 +867,9 @@ static rarch_setting_t setting_uint_setting(const char* name,
    result.original_value.unsigned_integer = *target;
    result.default_value.unsigned_integer  = default_value;
 
+   result.cmd_trigger.idx           = CMD_EVENT_NONE;
+   result.cmd_trigger.triggered     = false;
+
    result.dont_use_enum_idx_representation = dont_use_enum_idx;
 
    return result;
@@ -923,11 +901,10 @@ static rarch_setting_t setting_hex_setting(const char* name,
 
    result.enum_idx                  = MSG_UNKNOWN;
    result.type                      = ST_HEX;
-  
+
    result.size                      = sizeof(unsigned int);
 
    result.name                      = name;
-   result.name_hash                 = 0;
    result.short_description         = short_description;
    result.group                     = group;
    result.subgroup                  = subgroup;
@@ -966,6 +943,9 @@ static rarch_setting_t setting_hex_setting(const char* name,
    result.original_value.unsigned_integer = *target;
    result.default_value.unsigned_integer  = default_value;
 
+   result.cmd_trigger.idx           = CMD_EVENT_NONE;
+   result.cmd_trigger.triggered     = false;
+
    result.dont_use_enum_idx_representation = dont_use_enum_idx;
 
    return result;
@@ -982,7 +962,7 @@ static rarch_setting_t setting_hex_setting(const char* name,
  * @group              : Group that the setting belongs to.
  * @subgroup           : Subgroup that the setting belongs to.
  *
- * Initializes a setting of type ST_BIND. 
+ * Initializes a setting of type ST_BIND.
  *
  * Returns: setting of type ST_BIND.
  **/
@@ -998,11 +978,10 @@ static rarch_setting_t setting_bind_setting(const char* name,
 
    result.enum_idx                  = MSG_UNKNOWN;
    result.type                      = ST_BIND;
-  
+
    result.size                      = 0;
 
    result.name                      = name;
-   result.name_hash                 = 0;
    result.short_description         = short_description;
    result.group                     = group;
    result.subgroup                  = subgroup;
@@ -1023,7 +1002,7 @@ static rarch_setting_t setting_bind_setting(const char* name,
 #ifdef HAVE_MENU
    result.action_start              = setting_bind_action_start;
 #else
-   result.action_start              = NULL; 
+   result.action_start              = NULL;
 #endif
    result.action_left               = NULL;
    result.action_right              = NULL;
@@ -1044,6 +1023,9 @@ static rarch_setting_t setting_bind_setting(const char* name,
    result.value.target.keybind      = target;
    result.default_value.keybind     = default_value;
 
+   result.cmd_trigger.idx           = CMD_EVENT_NONE;
+   result.cmd_trigger.triggered     = false;
+
    result.dont_use_enum_idx_representation = dont_use_enum_idx;
 
    return result;
@@ -1051,26 +1033,34 @@ static rarch_setting_t setting_bind_setting(const char* name,
 
 static int setting_int_action_left_default(void *data, bool wraparound)
 {
-   double min               = 0.0f;
    rarch_setting_t *setting = (rarch_setting_t*)data;
+   double               min = 0.0f;
 
    if (!setting)
       return -1;
 
-   min               = setting->min;
+   min = setting->min;
 
    (void)wraparound; /* TODO/FIXME - handle this */
 
-   if (*setting->value.target.integer != min)
-      *setting->value.target.integer =
-         *setting->value.target.integer - setting->step;
+   *setting->value.target.integer = *setting->value.target.integer - setting->step;
 
    if (setting->enforce_minrange)
    {
       if (*setting->value.target.integer < min)
-         *setting->value.target.integer = min;
-   }
+      {
+         settings_t *settings = config_get_ptr();
 
+#ifdef HAVE_MENU
+      double           max = setting->max;
+
+         if (settings && settings->bools.menu_navigation_wraparound_enable)
+            *setting->value.target.integer = max;
+         else
+#endif
+            *setting->value.target.integer = min;
+      }
+   }
 
    return 0;
 }
@@ -1131,7 +1121,7 @@ int setting_string_action_start_generic(void *data)
  * @change_handler     : Function callback for change handler function pointer.
  * @read_handler       : Function callback for read handler function pointer.
  *
- * Initializes a string setting (of type @type). 
+ * Initializes a string setting (of type @type).
  *
  * Returns: String setting of type @type.
  **/
@@ -1147,11 +1137,10 @@ static rarch_setting_t setting_string_setting(enum setting_type type,
 
    result.enum_idx                  = MSG_UNKNOWN;
    result.type                      = type;
-  
+
    result.size                      = size;
 
    result.name                      = name;
-   result.name_hash                 = 0;
    result.short_description         = short_description;
    result.group                     = group;
    result.subgroup                  = subgroup;
@@ -1190,6 +1179,9 @@ static rarch_setting_t setting_string_setting(enum setting_type type,
    result.value.target.string       = target;
    result.default_value.string      = default_value;
 
+   result.cmd_trigger.idx           = CMD_EVENT_NONE;
+   result.cmd_trigger.triggered     = false;
+
    switch (type)
    {
       case ST_DIR:
@@ -1226,7 +1218,7 @@ static rarch_setting_t setting_string_setting(enum setting_type type,
  * @change_handler     : Function callback for change handler function pointer.
  * @read_handler       : Function callback for read handler function pointer.
  *
- * Initializes a string options list setting. 
+ * Initializes a string options list setting.
  *
  * Returns: string option list setting.
  **/
@@ -1266,11 +1258,10 @@ static rarch_setting_t setting_subgroup_setting(enum setting_type type,
 
    result.enum_idx                  = MSG_UNKNOWN;
    result.type                      = type;
-  
+
    result.size                      = 0;
 
    result.name                      = name;
-   result.name_hash                 = 0;
    result.short_description         = name;
    result.group                     = parent_name;
    result.parent_group              = parent_group;
@@ -1305,6 +1296,9 @@ static rarch_setting_t setting_subgroup_setting(enum setting_type type,
    result.enforce_minrange          = false;
    result.enforce_maxrange          = false;
 
+   result.cmd_trigger.idx           = CMD_EVENT_NONE;
+   result.cmd_trigger.triggered     = false;
+
    result.dont_use_enum_idx_representation = dont_use_enum_idx;
 
    return result;
@@ -1338,11 +1332,10 @@ static rarch_setting_t setting_bool_setting(const char* name,
 
    result.enum_idx                  = MSG_UNKNOWN;
    result.type                      = ST_BOOL;
-  
+
    result.size                      = sizeof(bool);
 
    result.name                      = name;
-   result.name_hash                 = name ? msg_hash_calculate(name) : 0;
    result.short_description         = short_description;
    result.group                     = group;
    result.subgroup                  = subgroup;
@@ -1383,6 +1376,9 @@ static rarch_setting_t setting_bool_setting(const char* name,
    result.boolean.off_label         = off;
    result.boolean.on_label          = on;
 
+   result.cmd_trigger.idx           = CMD_EVENT_NONE;
+   result.cmd_trigger.triggered     = false;
+
    result.dont_use_enum_idx_representation = dont_use_enum_idx;
 
    return result;
@@ -1399,7 +1395,7 @@ static rarch_setting_t setting_bool_setting(const char* name,
  * @change_handler     : Function callback for change handler function pointer.
  * @read_handler       : Function callback for read handler function pointer.
  *
- * Initializes a setting of type ST_INT. 
+ * Initializes a setting of type ST_INT.
  *
  * Returns: setting of type ST_INT.
  **/
@@ -1414,11 +1410,10 @@ static rarch_setting_t setting_int_setting(const char* name,
 
    result.enum_idx                  = MSG_UNKNOWN;
    result.type                      = ST_INT;
-  
+
    result.size                      = sizeof(int);
 
    result.name                      = name;
-   result.name_hash                 = name ? msg_hash_calculate(name) : 0;
    result.short_description         = short_description;
    result.group                     = group;
    result.subgroup                  = subgroup;
@@ -1457,6 +1452,9 @@ static rarch_setting_t setting_int_setting(const char* name,
    result.original_value.integer    = *target;
    result.default_value.integer     = default_value;
 
+   result.cmd_trigger.idx           = CMD_EVENT_NONE;
+   result.cmd_trigger.triggered     = false;
+
    result.dont_use_enum_idx_representation = dont_use_enum_idx;
 
    return result;
@@ -1485,8 +1483,6 @@ bool CONFIG_BOOL_ALT(
 
    if (!settings_list_append(list, list_info))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    if (flags != SD_FLAG_NONE)
       settings_data_list_current_add_flags(list, list_info, flags);
@@ -1521,13 +1517,15 @@ bool CONFIG_BOOL(
 
    if (!settings_list_append(list, list_info))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    if (flags != SD_FLAG_NONE)
       settings_data_list_current_add_flags(list, list_info, flags);
+
+#ifdef HAVE_MENU
    menu_settings_list_current_add_enum_idx(list, list_info, name_enum_idx);
    menu_settings_list_current_add_enum_value_idx(list, list_info, SHORT_enum_idx);
+#endif
+
    return true;
 }
 
@@ -1552,11 +1550,13 @@ bool CONFIG_INT(
          false);
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
+
+#ifdef HAVE_MENU
    menu_settings_list_current_add_enum_idx(list, list_info, name_enum_idx);
    menu_settings_list_current_add_enum_value_idx(list, list_info, SHORT_enum_idx);
+#endif
+
    return true;
 }
 
@@ -1577,8 +1577,6 @@ bool CONFIG_UINT_ALT(
                   true);
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    return true;
 }
@@ -1605,11 +1603,13 @@ bool CONFIG_UINT(
          false);
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
+
+#ifdef HAVE_MENU
    menu_settings_list_current_add_enum_idx(list, list_info, name_enum_idx);
    menu_settings_list_current_add_enum_value_idx(list, list_info, SHORT_enum_idx);
+#endif
+
    return true;
 }
 
@@ -1631,11 +1631,13 @@ bool CONFIG_FLOAT(
          false);
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
+
+#ifdef HAVE_MENU
    menu_settings_list_current_add_enum_idx(list, list_info, name_enum_idx);
    menu_settings_list_current_add_enum_value_idx(list, list_info, SHORT_enum_idx);
+#endif
+
    return true;
 }
 
@@ -1654,18 +1656,18 @@ bool CONFIG_PATH(
    rarch_setting_t value = setting_string_setting(ST_PATH,
          msg_hash_to_str(name_enum_idx),
          msg_hash_to_str(SHORT_enum_idx),
-         target, len, default_value, "",
+         target, (unsigned)len, default_value, "",
          group_info->name, subgroup_info->name, parent_group,
          change_handler, read_handler,
          false);
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    settings_data_list_current_add_flags(list, list_info, SD_FLAG_ALLOW_EMPTY);
+#ifdef HAVE_MENU
    menu_settings_list_current_add_enum_idx(list, list_info, name_enum_idx);
    menu_settings_list_current_add_enum_value_idx(list, list_info, SHORT_enum_idx);
+#endif
    return true;
 }
 
@@ -1682,26 +1684,26 @@ bool CONFIG_DIR(
       const char *parent_group,
       change_handler_t change_handler, change_handler_t read_handler)
 {
-   rarch_setting_t value = setting_string_setting(ST_DIR, 
+   rarch_setting_t value = setting_string_setting(ST_DIR,
          msg_hash_to_str(name_enum_idx),
          msg_hash_to_str(SHORT_enum_idx),
-         target, len, default_value,
+         target, (unsigned)len, default_value,
          msg_hash_to_str(empty_enum_idx),
          group_info->name, subgroup_info->name, parent_group,
          change_handler, read_handler,
          false);
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    settings_data_list_current_add_flags(
          list,
          list_info,
          SD_FLAG_ALLOW_EMPTY | SD_FLAG_PATH_DIR | SD_FLAG_BROWSER_ACTION);
 
+#ifdef HAVE_MENU
    menu_settings_list_current_add_enum_idx(list, list_info, name_enum_idx);
    menu_settings_list_current_add_enum_value_idx(list, list_info, SHORT_enum_idx);
+#endif
    return true;
 }
 
@@ -1720,16 +1722,16 @@ bool CONFIG_STRING(
    rarch_setting_t value = setting_string_setting(ST_STRING,
          msg_hash_to_str(name_enum_idx),
          msg_hash_to_str(SHORT_enum_idx),
-         target, len, default_value, "",
+         target, (unsigned)len, default_value, "",
          group_info->name, subgroup_info->name, parent_group,
          change_handler, read_handler, false);
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
+#ifdef HAVE_MENU
    menu_settings_list_current_add_enum_idx(list, list_info, name_enum_idx);
    menu_settings_list_current_add_enum_value_idx(list, list_info, SHORT_enum_idx);
+#endif
    return true;
 }
 
@@ -1748,17 +1750,18 @@ bool CONFIG_STRING_OPTIONS(
    rarch_setting_t value = setting_string_setting_options(ST_STRING_OPTIONS,
          msg_hash_to_str(name_enum_idx),
          msg_hash_to_str(SHORT_enum_idx),
-         target, len, default_value, "", values,
+         target, (unsigned)len, default_value, "", values,
          group_info->name, subgroup_info->name, parent_group,
          change_handler, read_handler, false);
    if (!(settings_list_append(list, list_info)))
       return false;
 
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
+
+#ifdef HAVE_MENU
    menu_settings_list_current_add_enum_idx(list, list_info, name_enum_idx);
    menu_settings_list_current_add_enum_value_idx(list, list_info, SHORT_enum_idx);
+#endif
    /* Request values to be freed later */
    settings_data_list_current_add_free_flags(list, list_info, SD_FREE_FLAG_VALUES);
 
@@ -1771,7 +1774,7 @@ bool CONFIG_HEX(
       unsigned int *target,
       enum msg_hash_enums name_enum_idx,
       enum msg_hash_enums SHORT_enum_idx,
-      unsigned int default_value, 
+      unsigned int default_value,
       rarch_setting_group_info_t *group_info,
       rarch_setting_group_info_t *subgroup_info,
       const char *parent_group,
@@ -1785,11 +1788,13 @@ bool CONFIG_HEX(
          change_handler, read_handler, false);
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
+
+#ifdef HAVE_MENU
    menu_settings_list_current_add_enum_idx(list, list_info, name_enum_idx);
    menu_settings_list_current_add_enum_value_idx(list, list_info, SHORT_enum_idx);
+#endif
+
    return true;
 }
 
@@ -1812,8 +1817,6 @@ bool CONFIG_BIND(
    if (!(settings_list_append(list, list_info)))
       return false;
 
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    /* Request name and short description to be freed later */
    settings_data_list_current_add_free_flags(list, list_info, SD_FREE_FLAG_NAME | SD_FREE_FLAG_SHORT);
@@ -1839,8 +1842,6 @@ bool CONFIG_BIND_ALT(
    if (!(settings_list_append(list, list_info)))
       return false;
 
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    /* Request name and short description to be freed later */
    settings_data_list_current_add_free_flags(list, list_info, SD_FREE_FLAG_NAME | SD_FREE_FLAG_SHORT);
@@ -1862,8 +1863,6 @@ bool CONFIG_ACTION_ALT(
 
    if (!settings_list_append(list, list_info))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    return true;
 }
@@ -1886,11 +1885,13 @@ bool CONFIG_ACTION(
 
    if (!settings_list_append(list, list_info))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
+
+#ifdef HAVE_MENU
    menu_settings_list_current_add_enum_idx(list, list_info, name_enum_idx);
    menu_settings_list_current_add_enum_value_idx(list, list_info, SHORT_enum_idx);
+#endif
+
    return true;
 }
 
@@ -1903,8 +1904,6 @@ bool START_GROUP(rarch_setting_t **list, rarch_setting_info_t *list_info,
    if (!(settings_list_append(list, list_info)))
       return false;
 
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    return true;
 }
@@ -1915,8 +1914,6 @@ bool END_GROUP(rarch_setting_t **list, rarch_setting_info_t *list_info,
    rarch_setting_t value = setting_group_setting (ST_END_GROUP, 0, parent_group);
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    return true;
 }
@@ -1934,8 +1931,6 @@ bool START_SUB_GROUP(rarch_setting_t **list,
 
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    return true;
 }
@@ -1948,8 +1943,6 @@ bool END_SUB_GROUP(
    rarch_setting_t value = setting_group_setting (ST_END_SUB_GROUP, 0, parent_group);
    if (!(settings_list_append(list, list_info)))
       return false;
-   if (value.name)
-      value.name_hash = msg_hash_calculate(value.name);
    (*list)[list_info->index++] = value;
    return true;
 }
@@ -2001,7 +1994,7 @@ static void menu_input_st_hex_cb(void *userdata, const char *str)
          if (str[0] == '#')
             str++;
          if (ptr)
-            *ptr = strtoul(str, NULL, 16);
+            *ptr = (unsigned)strtoul(str, NULL, 16);
       }
    }
 
