@@ -31,6 +31,10 @@
 #include "../verbosity.h"
 #include "../audio/audio_driver.h"
 
+#ifdef HAVE_NETWORKING
+#include "../network/netplay/netplay.h"
+#endif
+
 /* This makes Valgrind throw errors if a core overflows its savestate size. */
 /* Keep it off unless you're chasing a core bug, it slows things down. */
 #define STRICT_BUF_SIZE 0
@@ -63,7 +67,7 @@ static size_t find_change(const uint16_t *a, const uint16_t *b)
 #if __SSE2__
    const __m128i *a128 = (const __m128i*)a;
    const __m128i *b128 = (const __m128i*)b;
-   
+
    for (;;)
    {
       __m128i v0    = _mm_loadu_si128(a128);
@@ -94,7 +98,7 @@ static size_t find_change(const uint16_t *a, const uint16_t *b)
    {
       const size_t *a_big = (const size_t*)a;
       const size_t *b_big = (const size_t*)b;
-      
+
       while (*a_big == *b_big)
       {
          a_big++;
@@ -102,7 +106,7 @@ static size_t find_change(const uint16_t *a, const uint16_t *b)
       }
       a = (const uint16_t*)a_big;
       b = (const uint16_t*)b_big;
-      
+
       while (*a == *b)
       {
          a++;
@@ -128,14 +132,14 @@ static size_t find_same(const uint16_t *a, const uint16_t *b)
       /* With this, it's random whether two consecutive identical
        * words are caught.
        *
-       * Luckily, compression rate is the same for both cases, and 
+       * Luckily, compression rate is the same for both cases, and
        * three is always caught.
        *
-       * (We prefer to miss two-word blocks, anyways; fewer iterations 
+       * (We prefer to miss two-word blocks, anyways; fewer iterations
        * of the outer loop, as well as in the decompressor.) */
       const uint32_t *a_big = (const uint32_t*)a;
       const uint32_t *b_big = (const uint32_t*)b;
-      
+
       while (*a_big != *b_big)
       {
          a_big++;
@@ -143,7 +147,7 @@ static size_t find_same(const uint16_t *a, const uint16_t *b)
       }
       a = (const uint16_t*)a_big;
       b = (const uint16_t*)b_big;
-      
+
       if (a != a_org && a[-1] == b[-1])
       {
          a--;
@@ -168,7 +172,7 @@ struct state_manager
    /* This one is rounded up from reset::blocksize. */
    size_t blocksize;
 
-   /* size_t + (blocksize + 131071) / 131072 * 
+   /* size_t + (blocksize + 131071) / 131072 *
     * (blocksize + u16 + u16) + u16 + u32 + size_t
     * (yes, the math is a bit ugly). */
    size_t maxcompsize;
@@ -211,7 +215,7 @@ struct state_manager_rewind_state
 static struct state_manager_rewind_state rewind_state;
 static bool frame_is_reversed                         = false;
 
-/* Returns the maximum compressed size of a savestate. 
+/* Returns the maximum compressed size of a savestate.
  * It is very likely to compress to far less. */
 static size_t state_manager_raw_maxsize(size_t uncomp)
 {
@@ -234,16 +238,16 @@ static void *state_manager_raw_alloc(size_t len, uint16_t uniq)
    size_t  len16 = (len + sizeof(uint16_t) - 1) & -sizeof(uint16_t);
    uint16_t *ret = (uint16_t*)calloc(len16 + sizeof(uint16_t) * 4 + 16, 1);
 
-   /* Force in a different byte at the end, so we don't need to check 
+   /* Force in a different byte at the end, so we don't need to check
     * bounds in the innermost loop (it's expensive).
     *
-    * There is also a large amount of data that's the same, to stop 
+    * There is also a large amount of data that's the same, to stop
     * the other scan.
     *
-    * There is also some padding at the end. This is so we don't 
+    * There is also some padding at the end. This is so we don't
     * read outside the buffer end if we're reading in large blocks;
     *
-    * It doesn't make any difference to us, but sacrificing 16 bytes to get 
+    * It doesn't make any difference to us, but sacrificing 16 bytes to get
     * Valgrind happy is worth it. */
    ret[len16/sizeof(uint16_t) + 3] = uniq;
 
@@ -252,7 +256,7 @@ static void *state_manager_raw_alloc(size_t len, uint16_t uniq)
 
 /*
  * Takes two savestates and creates a patch that turns 'src' into 'dst'.
- * Both 'src' and 'dst' must be returned from state_manager_raw_alloc(), 
+ * Both 'src' and 'dst' must be returned from state_manager_raw_alloc(),
  * with the same 'len', and different 'uniq'.
  *
  * 'patch' must be size 'state_manager_raw_maxsize(len)' or more.
@@ -264,26 +268,26 @@ static size_t state_manager_raw_compress(const void *src,
    const uint16_t  *old16 = (const uint16_t*)src;
    const uint16_t  *new16 = (const uint16_t*)dst;
    uint16_t *compressed16 = (uint16_t*)patch;
-   size_t          num16s = (len + sizeof(uint16_t) - 1) 
+   size_t          num16s = (len + sizeof(uint16_t) - 1)
       / sizeof(uint16_t);
-   
+
    while (num16s)
    {
       size_t i, changed;
       size_t skip = find_change(old16, new16);
-   
+
       if (skip >= num16s)
          break;
-   
+
       old16  += skip;
       new16  += skip;
       num16s -= skip;
-   
+
       if (skip > UINT16_MAX)
       {
          if (skip > UINT32_MAX)
          {
-            /* This will make it scan the entire thing again, 
+            /* This will make it scan the entire thing again,
              * but it only hits on 8GB unchanged data anyways,
              * and if you're doing that, you've got bigger problems. */
             skip = UINT32_MAX;
@@ -293,36 +297,36 @@ static size_t state_manager_raw_compress(const void *src,
          *compressed16++ = skip >> 16;
          continue;
       }
-   
+
       changed = find_same(old16, new16);
       if (changed > UINT16_MAX)
          changed = UINT16_MAX;
-   
+
       *compressed16++ = changed;
       *compressed16++ = skip;
-   
+
       for (i = 0; i < changed; i++)
          compressed16[i] = old16[i];
-   
+
       old16 += changed;
       new16 += changed;
       num16s -= changed;
       compressed16 += changed;
    }
-   
+
    compressed16[0] = 0;
    compressed16[1] = 0;
    compressed16[2] = 0;
-   
+
    return (uint8_t*)(compressed16+3) - (uint8_t*)patch;
 }
 
 /*
- * Takes 'patch' from a previous call to 'state_manager_raw_compress' 
- * and applies it to 'data' ('src' from that call), 
+ * Takes 'patch' from a previous call to 'state_manager_raw_compress'
+ * and applies it to 'data' ('src' from that call),
  * yielding 'dst' in that call.
  *
- * If the given arguments do not match a previous call to 
+ * If the given arguments do not match a previous call to
  * state_manager_raw_compress(), anything at all can happen.
  */
 static void state_manager_raw_decompress(const void *patch,
@@ -330,10 +334,10 @@ static void state_manager_raw_decompress(const void *patch,
 {
    uint16_t         *out16 = (uint16_t*)data;
    const uint16_t *patch16 = (const uint16_t*)patch;
-   
+
    (void)patchlen;
    (void)datalen;
-   
+
    for (;;)
    {
       uint16_t numchanged = *(patch16++);
@@ -344,7 +348,7 @@ static void state_manager_raw_decompress(const void *patch,
 
          out16 += *patch16++;
 
-         /* We could do memcpy, but it seems that memcpy has a 
+         /* We could do memcpy, but it seems that memcpy has a
           * constant-per-call overhead that actually shows up.
           *
           * Our average size in here seems to be 8 or something.
@@ -368,27 +372,27 @@ static void state_manager_raw_decompress(const void *patch,
 }
 
 /* The start offsets point to 'nextstart' of any given compressed frame.
- * Each uint16 is stored native endian; anything that claims any other 
+ * Each uint16 is stored native endian; anything that claims any other
  * endianness refers to the endianness of this specific item.
  * The uint32 is stored little endian.
  *
- * Each size value is stored native endian if alignment is not enforced; 
+ * Each size value is stored native endian if alignment is not enforced;
  * if it is, they're little endian.
  *
- * The start of the buffer contains a size pointing to the end of the 
+ * The start of the buffer contains a size pointing to the end of the
  * buffer; the end points to its start.
  *
- * Wrapping is handled by returning to the start of the buffer if the 
+ * Wrapping is handled by returning to the start of the buffer if the
  * compressed data could potentially hit the edge;
  *
- * if the compressed data could potentially overwrite the tail pointer, 
+ * if the compressed data could potentially overwrite the tail pointer,
  * the tail retreats until it can no longer collide.
  *
- * This means that on average, ~2 * maxcompsize is 
+ * This means that on average, ~2 * maxcompsize is
  * unused at any given moment. */
 
 
-/* These are called very few constant times per frame, 
+/* These are called very few constant times per frame,
  * keep it as simple as possible. */
 static INLINE void write_size_t(void *ptr, size_t val)
 {
@@ -512,10 +516,10 @@ static bool state_manager_pop(state_manager_t *state, const void **data)
 static void state_manager_push_where(state_manager_t *state, void **data)
 {
    /* We need to ensure we have an uncompressed copy of the last
-    * pushed state, or we could end up applying a 'patch' to wrong 
+    * pushed state, or we could end up applying a 'patch' to wrong
     * savestate, and that'd blow up rather quickly. */
 
-   if (!state->thisblock_valid) 
+   if (!state->thisblock_valid)
    {
       const void *ignored;
       if (state_manager_pop(state, &ignored))
@@ -524,7 +528,7 @@ static void state_manager_push_where(state_manager_t *state, void **data)
          state->entries++;
       }
    }
-   
+
    *data = state->nextblock;
 #if STRICT_BUF_SIZE
    *data = state->debugblock;
@@ -659,11 +663,6 @@ bool state_manager_frame_is_reversed(void)
    return frame_is_reversed;
 }
 
-static void state_manager_set_frame_is_reversed(bool value)
-{
-   frame_is_reversed = value;
-}
-
 void state_manager_event_deinit(void)
 {
    if (rewind_state.state)
@@ -687,11 +686,17 @@ bool state_manager_check_rewind(bool pressed,
 {
    bool ret             = false;
    static bool first    = true;
+#ifdef HAVE_NETWORKING
+   bool was_reversed    = false;
+#endif
 
-   if (state_manager_frame_is_reversed())
+   if (frame_is_reversed)
    {
+#ifdef HAVE_NETWORKING
+      was_reversed = true;
+#endif
       audio_driver_frame_is_reverse();
-      state_manager_set_frame_is_reversed(false);
+      frame_is_reversed = false;
    }
 
    if (first)
@@ -711,7 +716,13 @@ bool state_manager_check_rewind(bool pressed,
       {
          retro_ctx_serialize_info_t serial_info;
 
-         state_manager_set_frame_is_reversed(true);
+#ifdef HAVE_NETWORKING
+         /* Make sure netplay isn't confused */
+         if (!was_reversed)
+            netplay_driver_ctl(RARCH_NETPLAY_CTL_DESYNC_PUSH, NULL);
+#endif
+
+         frame_is_reversed = true;
 
          audio_driver_setup_rewind();
 
@@ -735,10 +746,16 @@ bool state_manager_check_rewind(bool pressed,
          serial_info.size       = rewind_state.size;
          core_unserialize(&serial_info);
 
-         strlcpy(s, 
+#ifdef HAVE_NETWORKING
+         /* Tell netplay we're done */
+         if (was_reversed)
+            netplay_driver_ctl(RARCH_NETPLAY_CTL_DESYNC_POP, NULL);
+#endif
+
+         strlcpy(s,
                msg_hash_to_str(MSG_REWIND_REACHED_END),
                len);
-         
+
          *time = 30;
          ret   = true;
       }
@@ -747,6 +764,12 @@ bool state_manager_check_rewind(bool pressed,
    {
       static unsigned cnt      = 0;
 
+#ifdef HAVE_NETWORKING
+      /* Tell netplay we're done */
+      if (was_reversed)
+         netplay_driver_ctl(RARCH_NETPLAY_CTL_DESYNC_POP, NULL);
+#endif
+
       cnt = (cnt + 1) % (rewind_granularity ?
             rewind_granularity : 1); /* Avoid possible SIGFPE. */
 
