@@ -16,26 +16,20 @@
 #include <string.h>
 #include <errno.h>
 #include <file/nbio.h>
-#include <formats/image.h>
 #include <compat/strl.h>
-#include <retro_assert.h>
 #include <retro_miscellaneous.h>
-#include <lists/string_list.h>
-#include <rhash.h>
 
 #include <string/stdstring.h>
 
 #include "tasks_internal.h"
-#include "../file_path_special.h"
 #include "../verbosity.h"
 
+bool task_image_load_handler(retro_task_t *task);
+
 static int task_file_transfer_iterate_transfer(nbio_handle_t *nbio)
 {
    size_t i;
 
-   if (!nbio)
-      return -1;
-   
    nbio->pos_increment = 5;
 
    if (nbio->is_finished)
@@ -52,9 +46,6 @@ static int task_file_transfer_iterate_transfer(nbio_handle_t *nbio)
 
 static int task_file_transfer_iterate_parse(nbio_handle_t *nbio)
 {
-   if (!nbio)
-      return -1;
-
    if (nbio->cb)
    {
       int len = 0;
@@ -76,24 +67,13 @@ void task_file_load_handler(retro_task_t *task)
          case NBIO_STATUS_INIT:
             if (nbio && !string_is_empty(nbio->path))
             {
-               const char *fullpath  = nbio->path;
-               struct nbio_t *handle = nbio_open(fullpath, NBIO_READ);
+               struct nbio_t *handle = (struct nbio_t*)nbio_open(nbio->path, NBIO_READ);
 
                if (handle)
                {
                   nbio->handle       = handle;
                   nbio->status       = NBIO_STATUS_TRANSFER;
 
-                  if (strstr(fullpath, file_path_str(FILE_PATH_PNG_EXTENSION)))
-                     nbio->image_type = IMAGE_TYPE_PNG;
-                  else if (strstr(fullpath, file_path_str(FILE_PATH_JPEG_EXTENSION)) 
-                        || strstr(fullpath, file_path_str(FILE_PATH_JPG_EXTENSION)))
-                     nbio->image_type = IMAGE_TYPE_JPEG;
-                  else if (strstr(fullpath, file_path_str(FILE_PATH_BMP_EXTENSION)))
-                     nbio->image_type = IMAGE_TYPE_BMP;
-                  else if (strstr(fullpath, file_path_str(FILE_PATH_TGA_EXTENSION)))
-                     nbio->image_type = IMAGE_TYPE_TGA;
-
                   nbio_begin_read(handle);
                   return;
                }
@@ -102,30 +82,37 @@ void task_file_load_handler(retro_task_t *task)
             }
             break;
          case NBIO_STATUS_TRANSFER_PARSE:
-            if (task_file_transfer_iterate_parse(nbio) == -1)
+            if (!nbio || task_file_transfer_iterate_parse(nbio) == -1)
                task_set_cancelled(task, true);
-            nbio->status = NBIO_STATUS_TRANSFER_PARSE_FREE;
+            nbio->status = NBIO_STATUS_TRANSFER_FINISHED;
             break;
          case NBIO_STATUS_TRANSFER:
-            if (task_file_transfer_iterate_transfer(nbio) == -1)
+            if (!nbio || task_file_transfer_iterate_transfer(nbio) == -1)
                nbio->status = NBIO_STATUS_TRANSFER_PARSE;
             break;
-         case NBIO_STATUS_TRANSFER_PARSE_FREE:
-         case NBIO_STATUS_POLL:
-         default:
+         case NBIO_STATUS_TRANSFER_FINISHED:
             break;
       }
 
-      switch (nbio->image_type)
+      switch (nbio->type)
       {
-         case IMAGE_TYPE_PNG:
-         case IMAGE_TYPE_JPEG:
-         case IMAGE_TYPE_TGA:
-         case IMAGE_TYPE_BMP:
+         case NBIO_TYPE_PNG:
+         case NBIO_TYPE_JPEG:
+         case NBIO_TYPE_TGA:
+         case NBIO_TYPE_BMP:
             if (!task_image_load_handler(task))
                task_set_finished(task, true);
             break;
-         case 0:
+         case NBIO_TYPE_MP3:
+         case NBIO_TYPE_FLAC:
+         case NBIO_TYPE_OGG:
+         case NBIO_TYPE_MOD:
+         case NBIO_TYPE_WAV:
+            if (!task_audio_mixer_load_handler(task))
+               task_set_finished(task, true);
+            break;
+         case NBIO_TYPE_NONE:
+         default:
             if (nbio->is_finished)
                task_set_finished(task, true);
             break;
