@@ -13,37 +13,35 @@
  *  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <stdlib.h>
+
 #include <net/net_http.h>
-#include <queues/message_queue.h>
-#include <lists/string_list.h>
 #include <string/stdstring.h>
 #include <compat/strl.h>
 #include <file/file_path.h>
-#include <file/archive_file.h>
 #include <net/net_compat.h>
-#include <retro_stat.h>
+#include <retro_timers.h>
 
-#include "../msg_hash.h"
 #include "../verbosity.h"
+#include "../gfx/video_display_server.h"
 #include "tasks_internal.h"
 
 enum http_status_enum
 {
-   HTTP_STATUS_POLL = 0,
-   HTTP_STATUS_CONNECTION_TRANSFER,
+   HTTP_STATUS_CONNECTION_TRANSFER = 0,
    HTTP_STATUS_CONNECTION_TRANSFER_PARSE,
    HTTP_STATUS_TRANSFER,
    HTTP_STATUS_TRANSFER_PARSE,
    HTTP_STATUS_TRANSFER_PARSE_FREE
 };
 
-typedef struct http_transfer_info
+struct http_transfer_info
 {
    char url[255];
    int progress;
-} http_transfer_info_t;
+};
 
-typedef struct http_handle
+struct http_handle
 {
    struct
    {
@@ -56,7 +54,10 @@ typedef struct http_handle
    transfer_cb_t  cb;
    unsigned status;
    bool error;
-} http_handle_t;
+};
+
+typedef struct http_transfer_info http_transfer_info_t;
+typedef struct http_handle http_handle_t;
 
 static int task_http_con_iterate_transfer(http_handle_t *http)
 {
@@ -73,7 +74,7 @@ static int task_http_conn_iterate_transfer_parse(
       if (http->connection.handle && http->connection.cb)
          http->connection.cb(http, 0);
    }
-   
+
    net_http_connection_free(http->connection.handle);
 
    http->connection.handle = NULL;
@@ -154,7 +155,6 @@ static void task_http_transfer_handler(retro_task_t *task)
             goto task_finished;
          break;
       case HTTP_STATUS_TRANSFER_PARSE:
-      case HTTP_STATUS_POLL:
          goto task_finished;
       default:
          break;
@@ -235,12 +235,21 @@ static bool task_http_retriever(retro_task_t *task, void *data)
    return true;
 }
 
-void *task_push_http_transfer(const char *url, bool mute, const char *type,
+static void http_transfer_progress_cb(retro_task_t *task)
+{
+   if (!task)
+      return;
+   video_display_server_set_window_progress(task->progress, task->finished);
+}
+
+static void* task_push_http_transfer_generic(
+      struct http_connection_t *conn,
+      const char *url, bool mute, const char *type,
       retro_task_callback_t cb, void *user_data)
 {
    task_finder_data_t find_data;
    char tmp[255];
-   struct http_connection_t *conn = NULL;
+   const char* s = NULL;
    retro_task_t  *t               = NULL;
    http_handle_t *http            = NULL;
 
@@ -253,14 +262,12 @@ void *task_push_http_transfer(const char *url, bool mute, const char *type,
    find_data.userdata = (void*)url;
 
    /* Concurrent download of the same file is not allowed */
-   if (task_queue_ctl(TASK_QUEUE_CTL_FIND, &find_data))
+   if (task_queue_find(&find_data))
    {
       RARCH_LOG("[http] '%s'' is already being downloaded.\n", url);
       return NULL;
    }
 
-   conn = net_http_connection_new(url);
-
    if (!conn)
       return NULL;
 
@@ -287,15 +294,27 @@ void *task_push_http_transfer(const char *url, bool mute, const char *type,
    t->state                = http;
    t->mute                 = mute;
    t->callback             = cb;
+   t->progress_cb          = http_transfer_progress_cb;
    t->user_data            = user_data;
    t->progress             = -1;
 
+   if (user_data != NULL)
+      s = ((file_transfer_t*)user_data)->path;
+   else
+      s = url;
+
+   if (strstr(s, ".index"))
+   {
+      snprintf(tmp, sizeof(tmp), "%s %s",
+            msg_hash_to_str(MSG_DOWNLOADING), msg_hash_to_str(MSG_INDEX_FILE));
+   }
+   else
    snprintf(tmp, sizeof(tmp), "%s '%s'",
-         msg_hash_to_str(MSG_DOWNLOADING), path_basename(url));
+         msg_hash_to_str(MSG_DOWNLOADING), s);
 
    t->title                = strdup(tmp);
 
-   task_queue_ctl(TASK_QUEUE_CTL_PUSH, t);
+   task_queue_push(t);
 
    return t;
 
@@ -308,6 +327,29 @@ error:
    return NULL;
 }
 
+void* task_push_http_transfer(const char *url, bool mute,
+      const char *type,
+      retro_task_callback_t cb, void *user_data)
+{
+   struct http_connection_t *conn;
+
+   conn = net_http_connection_new(url, "GET", NULL);
+
+   return task_push_http_transfer_generic(conn, url, mute, type, cb, user_data);
+}
+
+void* task_push_http_post_transfer(const char *url,
+      const char *post_data, bool mute,
+      const char *type, retro_task_callback_t cb, void *user_data)
+{
+   struct http_connection_t *conn;
+
+   conn = net_http_connection_new(url, "POST", post_data);
+
+   return task_push_http_transfer_generic(conn,
+         url, mute, type, cb, user_data);
+}
+
 task_retriever_info_t *http_task_get_transfer_list(void)
 {
    task_retriever_data_t retrieve_data;
@@ -318,6 +360,7 @@ task_retriever_info_t *http_task_get_transfer_list(void)
    retrieve_data.func         = task_http_retriever;
 
    /* Build list of current HTTP transfers and return it */
-   task_queue_ctl(TASK_QUEUE_CTL_RETRIEVE, &retrieve_data);
+   task_queue_retrieve(&retrieve_data);
+
    return retrieve_data.list;
 }
