@@ -27,32 +27,38 @@
 #include "tasks_internal.h"
 
 #include "../file_path_special.h"
-#include "../input/input_config.h"
+#include "../gfx/video_driver.h"
+#include "../input/input_driver.h"
 #include "../input/input_overlay.h"
 #include "../configuration.h"
 #include "../verbosity.h"
 
-typedef struct
+typedef struct overlay_loader overlay_loader_t;
+
+struct overlay_loader
 {
    enum overlay_status state;
    enum overlay_image_transfer_status loading_status;
    config_file_t *conf;
    char *overlay_path;
+   struct overlay *overlays;
+   struct overlay *active;
+   bool overlay_enable;
+   bool overlay_hide_in_menu;
+   size_t resolve_pos;
    unsigned size;
    unsigned pos;
    unsigned pos_increment;
-   struct overlay *overlays;
-   struct overlay *active;
-   size_t resolve_pos;
-
-} overlay_loader_t;
+   float overlay_opacity;
+   float overlay_scale;
+};
 
 static void task_overlay_image_done(struct overlay *overlay)
 {
    overlay->pos           = 0;
    /* Divide iteration steps by half of total descs if size is even,
     * otherwise default to 8 (arbitrary value for now to speed things up). */
-   overlay->pos_increment = (overlay->size / 2) ? (overlay->size / 2) : 8;
+   overlay->pos_increment = (overlay->size / 2) ? ((unsigned)(overlay->size / 2)) : 8;
 }
 
 static void task_overlay_load_desc_image(
@@ -104,12 +110,11 @@ static bool task_overlay_load_desc(
       bool normalized, float alpha_mod, float range_mod)
 {
    float width_mod, height_mod;
-   float tmp_float;
-   uint32_t box_hash, key_hash;
    char overlay_desc_key[64];
    char conf_key[64];
    char overlay_desc_normalized_key[64];
    char overlay[256];
+   float tmp_float                      = 0.0f;
    bool tmp_bool                        = false;
    bool ret                             = true;
    bool by_pixel                        = false;
@@ -120,7 +125,7 @@ static bool task_overlay_load_desc(
    const char *box                      = NULL;
    config_file_t *conf                  = loader->conf;
 
-   overlay_desc_key[0] = conf_key[0] = 
+   overlay_desc_key[0] = conf_key[0] =
       overlay_desc_normalized_key[0] = overlay[0] = '\0';
 
    snprintf(overlay_desc_key, sizeof(overlay_desc_key),
@@ -168,50 +173,40 @@ static bool task_overlay_load_desc(
    y              = list->elems[2].data;
    box            = list->elems[3].data;
 
-   box_hash       = djb2_calculate(box);
-   key_hash       = djb2_calculate(key);
+   desc->retro_key_idx = 0;
+   BIT256_CLEAR_ALL(desc->button_mask);
 
-   desc->key_mask = 0;
-
-   switch (key_hash)
+   if (string_is_equal(key, "analog_left"))
+      desc->type          = OVERLAY_TYPE_ANALOG_LEFT;
+   else if (string_is_equal(key, "analog_right"))
+      desc->type          = OVERLAY_TYPE_ANALOG_RIGHT;
+   else if (strstr(key, "retrok_") == key)
    {
-      case KEY_ANALOG_LEFT:
-         desc->type = OVERLAY_TYPE_ANALOG_LEFT;
-         break;
-      case KEY_ANALOG_RIGHT:
-         desc->type = OVERLAY_TYPE_ANALOG_RIGHT;
-         break;
-      default:
-         if (strstr(key, "retrok_") == key)
-         {
-            desc->type     = OVERLAY_TYPE_KEYBOARD;
-            desc->key_mask = input_config_translate_str_to_rk(key + 7);
-         }
-         else
-         {
-            char      *save = NULL;
-            const char *tmp = strtok_r(key, "|", &save);
+      desc->type          = OVERLAY_TYPE_KEYBOARD;
+      desc->retro_key_idx = input_config_translate_str_to_rk(key + 7);
+   }
+   else
+   {
+      char      *save = NULL;
+      const char *tmp = strtok_r(key, "|", &save);
 
-            desc->type = OVERLAY_TYPE_BUTTONS;
+      desc->type = OVERLAY_TYPE_BUTTONS;
 
-            for (; tmp; tmp = strtok_r(NULL, "|", &save))
-            {
-               if (!string_is_equal(tmp, file_path_str(FILE_PATH_NUL)))
-                  desc->key_mask |= UINT64_C(1) 
-                     << input_config_translate_str_to_bind_id(tmp);
-            }
+      for (; tmp; tmp = strtok_r(NULL, "|", &save))
+      {
+         if (!string_is_equal(tmp, file_path_str(FILE_PATH_NUL)))
+            BIT256_SET(desc->button_mask, input_config_translate_str_to_bind_id(tmp));
+      }
 
-            if (desc->key_mask & (UINT64_C(1) << RARCH_OVERLAY_NEXT))
-            {
-               char overlay_target_key[64];
+      if (BIT256_GET(desc->button_mask, RARCH_OVERLAY_NEXT))
+      {
+         char overlay_target_key[64];
 
-               snprintf(overlay_target_key, sizeof(overlay_target_key),
-                     "overlay%u_desc%u_next_target", ol_idx, desc_idx);
-               config_get_array(conf, overlay_target_key,
-                     desc->next_index_name, sizeof(desc->next_index_name));
-            }
-         }
-         break;
+         snprintf(overlay_target_key, sizeof(overlay_target_key),
+               "overlay%u_desc%u_next_target", ol_idx, desc_idx);
+         config_get_array(conf, overlay_target_key,
+               desc->next_index_name, sizeof(desc->next_index_name));
+      }
    }
 
    width_mod  = 1.0f;
@@ -226,18 +221,15 @@ static bool task_overlay_load_desc(
    desc->x = (float)strtod(x, NULL) * width_mod;
    desc->y = (float)strtod(y, NULL) * height_mod;
 
-   switch (box_hash)
+   if (string_is_equal(box, "radial"))
+      desc->hitbox = OVERLAY_HITBOX_RADIAL;
+   else if (string_is_equal(box, "rect"))
+      desc->hitbox = OVERLAY_HITBOX_RECT;
+   else
    {
-      case BOX_RADIAL:
-         desc->hitbox = OVERLAY_HITBOX_RADIAL;
-         break;
-      case BOX_RECT:
-         desc->hitbox = OVERLAY_HITBOX_RECT;
-         break;
-      default:
-         RARCH_ERR("[Overlay]: Hitbox type (%s) is invalid. Use \"radial\" or \"rect\".\n", box);
-         ret = false;
-         goto end;
+      RARCH_ERR("[Overlay]: Hitbox type (%s) is invalid. Use \"radial\" or \"rect\".\n", box);
+      ret = false;
+      goto end;
    }
 
    switch (desc->type)
@@ -337,8 +329,9 @@ static bool task_overlay_resolve_targets(struct overlay *ol,
 
    for (i = 0; i < current->size; i++)
    {
-      ssize_t next_idx  = 0;
-      const char *next = current->descs[i].next_index_name;
+      struct overlay_desc *desc = (struct overlay_desc*)&current->descs[i];
+      const char *next          = desc->next_index_name;
+      ssize_t         next_idx  = (idx + 1) & size;
 
       if (!string_is_empty(next))
       {
@@ -351,10 +344,8 @@ static bool task_overlay_resolve_targets(struct overlay *ol,
             return false;
          }
       }
-      else
-         next_idx = (idx + 1) & size;
 
-      current->descs[i].next_index = next_idx;
+      desc->next_index = (unsigned)next_idx;
    }
 
    return true;
@@ -427,7 +418,7 @@ static void task_overlay_deferred_loading(retro_task_t *task)
             {
                task_overlay_load_desc_image(loader,
                      &overlay->descs[overlay->pos], overlay,
-                     loader->pos, overlay->pos);
+                     loader->pos, (unsigned)overlay->pos);
             }
             else
             {
@@ -445,7 +436,7 @@ static void task_overlay_deferred_loading(retro_task_t *task)
             {
                if (!task_overlay_load_desc(loader,
                         &overlay->descs[overlay->pos], overlay,
-                        loader->pos, overlay->pos,
+                        loader->pos, (unsigned)overlay->pos,
                         overlay->image.width, overlay->image.height,
                         overlay->config.normalized,
                         overlay->config.alpha_mod, overlay->config.range_mod))
@@ -703,17 +694,16 @@ static void task_overlay_handler(retro_task_t *task)
 
    if (task_get_finished(task) && !task_get_cancelled(task))
    {
-      settings_t *settings      = config_get_ptr();
       overlay_task_data_t *data = (overlay_task_data_t*)
          calloc(1, sizeof(*data));
 
       data->overlays        = loader->overlays;
       data->size            = loader->size;
       data->active          = loader->active;
-      data->hide_in_menu    = settings->input.overlay_hide_in_menu;
-      data->overlay_enable  = settings->input.overlay_enable;
-      data->overlay_opacity = settings->input.overlay_opacity;
-      data->overlay_scale   = settings->input.overlay_scale;
+      data->hide_in_menu    = loader->overlay_hide_in_menu;
+      data->overlay_enable  = loader->overlay_enable;
+      data->overlay_opacity = loader->overlay_opacity;
+      data->overlay_scale   = loader->overlay_scale;
 
       task_set_data(task, data);
    }
@@ -736,21 +726,27 @@ static bool task_overlay_finder(retro_task_t *task, void *user_data)
    return string_is_equal(loader->overlay_path, (const char*)user_data);
 }
 
-static bool task_push_overlay_load(const char *overlay_path,
+bool task_push_overlay_load_default(
       retro_task_callback_t cb, void *user_data)
 {
    task_finder_data_t find_data;
+   settings_t *settings     = config_get_ptr();
+   const char *overlay_path = settings->paths.path_overlay;
    retro_task_t *t          = NULL;
    config_file_t *conf      = NULL;
    overlay_loader_t *loader = (overlay_loader_t*)calloc(1, sizeof(*loader));
 
    if (!loader)
+      return false;
+
+   if (string_is_empty(overlay_path))
       goto error;
 
    /* Prevent overlay from being loaded if it already is being loaded */
    find_data.func     = task_overlay_finder;
    find_data.userdata = (void*)overlay_path;
-   if (task_queue_ctl(TASK_QUEUE_CTL_FIND, &find_data))
+
+   if (task_queue_find(&find_data))
       goto error;
 
    conf = config_file_new(overlay_path);
@@ -770,12 +766,16 @@ static bool task_push_overlay_load(const char *overlay_path,
    if (!loader->overlays)
       goto error;
 
-   loader->overlay_path     = strdup(overlay_path);
-   loader->conf             = conf;
-   loader->state            = OVERLAY_STATUS_DEFERRED_LOAD;
-   loader->pos_increment    = (loader->size / 4) ? (loader->size / 4) : 4;
+   loader->overlay_hide_in_menu = settings->bools.input_overlay_hide_in_menu;
+   loader->overlay_enable       = settings->bools.input_overlay_enable;
+   loader->overlay_opacity      = settings->floats.input_overlay_opacity;
+   loader->overlay_scale        = settings->floats.input_overlay_scale;
+   loader->overlay_path         = strdup(overlay_path);
+   loader->conf                 = conf;
+   loader->state                = OVERLAY_STATUS_DEFERRED_LOAD;
+   loader->pos_increment        = (loader->size / 4) ? (loader->size / 4) : 4;
 
-   t                        = (retro_task_t*)calloc(1, sizeof(*t));
+   t                            = (retro_task_t*)calloc(1, sizeof(*t));
 
    if (!t)
       goto error;
@@ -786,9 +786,10 @@ static bool task_push_overlay_load(const char *overlay_path,
    t->callback              = cb;
    t->user_data             = user_data;
 
-   task_queue_ctl(TASK_QUEUE_CTL_PUSH, t);
+   task_queue_push(t);
 
    return true;
+
 error:
    if (conf)
       config_file_free(conf);
@@ -804,15 +805,3 @@ error:
 
    return false;
 }
-
-bool task_push_overlay_load_default(
-      retro_task_callback_t cb, void *user_data)
-{
-   settings_t *settings = config_get_ptr();
-   const char *path     = settings->path.overlay;
-
-   if (string_is_empty(path))
-      return false;
-
-   return task_push_overlay_load(path, cb, user_data);
-}
