@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2016 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -27,13 +27,11 @@
 #include <compat/msvc.h>
 #include <file/file_path.h>
 #include <streams/file_stream.h>
-#include <retro_stat.h>
 #include <string/stdstring.h>
 
 #include <encodings/crc32.h>
 
 #include "../msg_hash.h"
-#include "../retroarch.h"
 #include "../verbosity.h"
 
 enum bps_mode
@@ -62,30 +60,45 @@ enum patch_error
 
 struct bps_data
 {
-   const uint8_t *modify_data, *source_data; 
+   const uint8_t *modify_data;
+   const uint8_t *source_data;
    uint8_t *target_data;
-   size_t modify_length, source_length, target_length;
-   size_t modify_offset, source_offset, target_offset;
-   uint32_t modify_checksum, source_checksum, target_checksum;
-
-   size_t source_relative_offset, target_relative_offset, output_offset;
+   size_t modify_length;
+   size_t source_length;
+   size_t target_length;
+   size_t modify_offset;
+   size_t source_offset;
+   size_t target_offset;
+   uint32_t modify_checksum;
+   uint32_t source_checksum;
+   uint32_t target_checksum;
+   size_t source_relative_offset;
+   size_t target_relative_offset;
+   size_t output_offset;
 };
 
 struct ups_data
 {
-   const uint8_t *patch_data, *source_data; 
+   const uint8_t *patch_data;
+   const uint8_t *source_data;
    uint8_t *target_data;
-   unsigned patch_length, source_length, target_length;
-   unsigned patch_offset, source_offset, target_offset;
-   unsigned patch_checksum, source_checksum, target_checksum;
+   unsigned patch_length;
+   unsigned source_length;
+   unsigned target_length;
+   unsigned patch_offset;
+   unsigned source_offset;
+   unsigned target_offset;
+   unsigned patch_checksum;
+   unsigned source_checksum;
+   unsigned target_checksum;
 };
 
-typedef enum patch_error (*patch_func_t)(const uint8_t*, size_t,
-      const uint8_t*, size_t, uint8_t*, size_t*);
+typedef enum patch_error (*patch_func_t)(const uint8_t*, uint64_t,
+      const uint8_t*, uint64_t, uint8_t*, uint64_t*);
 
 static uint8_t bps_read(struct bps_data *bps)
 {
-   uint8_t data = bps->modify_data[bps->modify_offset++];
+   uint8_t data         = bps->modify_data[bps->modify_offset++];
    bps->modify_checksum = ~(encoding_crc32(
          ~bps->modify_checksum, &data, 1));
    return data;
@@ -110,24 +123,21 @@ static uint64_t bps_decode(struct bps_data *bps)
 
 static void bps_write(struct bps_data *bps, uint8_t data)
 {
-   if (!bps)
-      return;
-
    bps->target_data[bps->output_offset++] = data;
    bps->target_checksum = ~(encoding_crc32(~bps->target_checksum, &data, 1));
 }
 
 static enum patch_error bps_apply_patch(
-      const uint8_t *modify_data, size_t modify_length,
-      const uint8_t *source_data, size_t source_length,
-      uint8_t *target_data, size_t *target_length)
+      const uint8_t *modify_data, uint64_t modify_length,
+      const uint8_t *source_data, uint64_t source_length,
+      uint8_t *target_data, uint64_t *target_length)
 {
    size_t i;
    uint32_t checksum;
    size_t modify_source_size;
    size_t modify_target_size;
    size_t modify_markup_size;
-   struct bps_data bps = {0};
+   struct bps_data bps;
    uint32_t modify_source_checksum = 0;
    uint32_t modify_target_checksum = 0;
    uint32_t modify_modify_checksum = 0;
@@ -135,22 +145,32 @@ static enum patch_error bps_apply_patch(
    if (modify_length < 19)
       return PATCH_PATCH_TOO_SMALL;
 
-   bps.modify_data     = modify_data;
-   bps.modify_length   = modify_length;
-   bps.target_data     = target_data;
-   bps.target_length   = *target_length;
-   bps.source_data     = source_data;
-   bps.source_length   = source_length;
-   bps.modify_checksum = ~0;
-   bps.target_checksum = ~0;
+   bps.modify_data            = modify_data;
+   bps.source_data            = source_data;
+   bps.target_data            = target_data;
+   bps.modify_length          = modify_length;
+   bps.source_length          = source_length;
+   bps.target_length          = *target_length;
+   bps.modify_offset          = 0;
+   bps.source_offset          = 0;
+   bps.target_offset          = 0;
+   bps.modify_checksum        = ~0;
+   bps.source_checksum        = 0;
+   bps.target_checksum        = ~0;
+   bps.source_relative_offset = 0;
+   bps.target_relative_offset = 0;
+   bps.output_offset          = 0;
 
-   if ((bps_read(&bps) != 'B') || (bps_read(&bps) != 'P') ||
-         (bps_read(&bps) != 'S') || (bps_read(&bps) != '1'))
+   if (  (bps_read(&bps) != 'B') ||
+         (bps_read(&bps) != 'P') ||
+         (bps_read(&bps) != 'S') ||
+         (bps_read(&bps) != '1'))
       return PATCH_PATCH_INVALID_HEADER;
 
    modify_source_size  = bps_decode(&bps);
    modify_target_size  = bps_decode(&bps);
    modify_markup_size  = bps_decode(&bps);
+
    for (i = 0; i < modify_markup_size; i++)
       bps_read(&bps);
 
@@ -181,7 +201,7 @@ static enum patch_error bps_apply_patch(
          case SOURCE_COPY:
          case TARGET_COPY:
          {
-            int    offset = bps_decode(&bps);
+            int    offset = (int)bps_decode(&bps);
             bool negative = offset & 1;
 
             offset >>= 1;
@@ -234,37 +254,37 @@ static enum patch_error bps_apply_patch(
    return PATCH_SUCCESS;
 }
 
-static uint8_t ups_patch_read(struct ups_data *data) 
+static uint8_t ups_patch_read(struct ups_data *data)
 {
-   if (data && data->patch_offset < data->patch_length) 
+   if (data && data->patch_offset < data->patch_length)
    {
       uint8_t n = data->patch_data[data->patch_offset++];
-      data->patch_checksum = 
+      data->patch_checksum =
          ~(encoding_crc32(~data->patch_checksum, &n, 1));
       return n;
    }
    return 0x00;
 }
 
-static uint8_t ups_source_read(struct ups_data *data) 
+static uint8_t ups_source_read(struct ups_data *data)
 {
-   if (data && data->source_offset < data->source_length) 
+   if (data && data->source_offset < data->source_length)
    {
       uint8_t n = data->source_data[data->source_offset++];
-      data->source_checksum =  
+      data->source_checksum =
          ~(encoding_crc32(~data->source_checksum, &n, 1));
       return n;
    }
    return 0x00;
 }
 
-static void ups_target_write(struct ups_data *data, uint8_t n) 
+static void ups_target_write(struct ups_data *data, uint8_t n)
 {
 
-   if (data && data->target_offset < data->target_length) 
+   if (data && data->target_offset < data->target_length)
    {
       data->target_data[data->target_offset] = n;
-      data->target_checksum = 
+      data->target_checksum =
          ~(encoding_crc32(~data->target_checksum, &n, 1));
    }
 
@@ -272,15 +292,15 @@ static void ups_target_write(struct ups_data *data, uint8_t n)
       data->target_offset++;
 }
 
-static uint64_t ups_decode(struct ups_data *data) 
+static uint64_t ups_decode(struct ups_data *data)
 {
    uint64_t offset = 0, shift = 1;
-   while (true) 
+   while (true)
    {
       uint8_t x = ups_patch_read(data);
       offset   += (x & 0x7f) * shift;
 
-      if (x & 0x80) 
+      if (x & 0x80)
          break;
       shift <<= 7;
       offset += shift;
@@ -289,58 +309,64 @@ static uint64_t ups_decode(struct ups_data *data)
 }
 
 static enum patch_error ups_apply_patch(
-      const uint8_t *patchdata, size_t patchlength,
-      const uint8_t *sourcedata, size_t sourcelength,
-      uint8_t *targetdata, size_t *targetlength)
+      const uint8_t *patchdata, uint64_t patchlength,
+      const uint8_t *sourcedata, uint64_t sourcelength,
+      uint8_t *targetdata, uint64_t *targetlength)
 {
    size_t i;
+   struct ups_data data;
    unsigned source_read_length;
    unsigned target_read_length;
    uint32_t patch_result_checksum;
    uint32_t patch_read_checksum  = 0;
    uint32_t source_read_checksum = 0;
    uint32_t target_read_checksum = 0;
-   struct ups_data data          = {0};
 
    data.patch_data      = patchdata;
    data.source_data     = sourcedata;
    data.target_data     = targetdata;
-   data.patch_length    = patchlength;
-   data.source_length   = sourcelength;
-   data.target_length   = *targetlength;
+   data.patch_length    = (unsigned)patchlength;
+   data.source_length   = (unsigned)sourcelength;
+   data.target_length   = (unsigned)*targetlength;
+   data.patch_offset    = 0;
+   data.source_offset   = 0;
+   data.target_offset   = 0;
    data.patch_checksum  = ~0;
    data.source_checksum = ~0;
    data.target_checksum = ~0;
 
-   if (data.patch_length < 18) 
-      return PATCH_PATCH_INVALID;
-   if (ups_patch_read(&data) != 'U') 
-      return PATCH_PATCH_INVALID;
-   if (ups_patch_read(&data) != 'P') 
-      return PATCH_PATCH_INVALID;
-   if (ups_patch_read(&data) != 'S') 
-      return PATCH_PATCH_INVALID;
-   if (ups_patch_read(&data) != '1') 
+   if (data.patch_length < 18)
       return PATCH_PATCH_INVALID;
 
-   source_read_length = ups_decode(&data);
-   target_read_length = ups_decode(&data);
+   if (
+         (ups_patch_read(&data) != 'U') ||
+         (ups_patch_read(&data) != 'P') ||
+         (ups_patch_read(&data) != 'S') ||
+         (ups_patch_read(&data) != '1')
+      )
+      return PATCH_PATCH_INVALID;
 
-   if (data.source_length != source_read_length
-         && data.source_length != target_read_length) 
+   source_read_length = (unsigned)ups_decode(&data);
+   target_read_length = (unsigned)ups_decode(&data);
+
+   if (     (data.source_length != source_read_length)
+         && (data.source_length != target_read_length))
       return PATCH_SOURCE_INVALID;
+
    *targetlength = (data.source_length == source_read_length ?
          target_read_length : source_read_length);
-   if (data.target_length < *targetlength) 
-      return PATCH_TARGET_TOO_SMALL;
-   data.target_length = *targetlength;
 
-   while (data.patch_offset < data.patch_length - 12) 
+   if (data.target_length < *targetlength)
+      return PATCH_TARGET_TOO_SMALL;
+
+   data.target_length = (unsigned)*targetlength;
+
+   while (data.patch_offset < data.patch_length - 12)
    {
-      unsigned length = ups_decode(&data);
-      while (length--) 
+      unsigned length = (unsigned)ups_decode(&data);
+      while (length--)
          ups_target_write(&data, ups_source_read(&data));
-      while (true) 
+      while (true)
       {
          uint8_t patch_xor = ups_patch_read(&data);
          ups_target_write(&data, patch_xor ^ ups_source_read(&data));
@@ -349,51 +375,51 @@ static enum patch_error ups_apply_patch(
       }
    }
 
-   while (data.source_offset < data.source_length) 
+   while (data.source_offset < data.source_length)
       ups_target_write(&data, ups_source_read(&data));
-   while (data.target_offset < data.target_length) 
+   while (data.target_offset < data.target_length)
       ups_target_write(&data, ups_source_read(&data));
 
 
-   for (i = 0; i < 4; i++) 
+   for (i = 0; i < 4; i++)
       source_read_checksum |= ups_patch_read(&data) << (i * 8);
-   for (i = 0; i < 4; i++) 
+   for (i = 0; i < 4; i++)
       target_read_checksum |= ups_patch_read(&data) << (i * 8);
 
    patch_result_checksum = ~data.patch_checksum;
    data.source_checksum  = ~data.source_checksum;
    data.target_checksum  = ~data.target_checksum;
 
-   for (i = 0; i < 4; i++) 
+   for (i = 0; i < 4; i++)
       patch_read_checksum |= ups_patch_read(&data) << (i * 8);
 
-   if (patch_result_checksum != patch_read_checksum) 
+   if (patch_result_checksum != patch_read_checksum)
       return PATCH_PATCH_INVALID;
 
    if (data.source_checksum == source_read_checksum
-         && data.source_length == source_read_length) 
+         && data.source_length == source_read_length)
    {
       if (data.target_checksum == target_read_checksum
-            && data.target_length == target_read_length) 
+            && data.target_length == target_read_length)
          return PATCH_SUCCESS;
       return PATCH_TARGET_INVALID;
-   } 
+   }
    else if (data.source_checksum == target_read_checksum
-         && data.source_length == target_read_length) 
+         && data.source_length == target_read_length)
    {
       if (data.target_checksum == source_read_checksum
-            && data.target_length == source_read_length) 
+            && data.target_length == source_read_length)
          return PATCH_SUCCESS;
       return PATCH_TARGET_INVALID;
-   } 
+   }
 
    return PATCH_SOURCE_INVALID;
 }
 
 static enum patch_error ips_apply_patch(
-      const uint8_t *patchdata, size_t patchlen,
-      const uint8_t *sourcedata, size_t sourcelength,
-      uint8_t *targetdata, size_t *targetlength)
+      const uint8_t *patchdata, uint64_t patchlen,
+      const uint8_t *sourcedata, uint64_t sourcelength,
+      uint8_t *targetdata, uint64_t *targetlength)
 {
    uint32_t offset = 5;
 
@@ -405,7 +431,7 @@ static enum patch_error ips_apply_patch(
          patchdata[4] != 'H')
       return PATCH_PATCH_INVALID;
 
-   memcpy(targetdata, sourcedata, sourcelength);
+   memcpy(targetdata, sourcedata, (size_t)sourcelength);
 
    *targetlength = sourcelength;
 
@@ -475,47 +501,26 @@ static enum patch_error ips_apply_patch(
 
 static bool apply_patch_content(uint8_t **buf,
       ssize_t *size, const char *patch_desc, const char *patch_path,
-      patch_func_t func)
+      patch_func_t func, void *patch_data, int64_t patch_size)
 {
-   size_t target_size;
-   ssize_t patch_size;
-   void *patch_data         = NULL;
    enum patch_error err     = PATCH_UNKNOWN;
-   bool success             = false;
-   uint8_t *patched_content = NULL;
    ssize_t ret_size         = *size;
    uint8_t *ret_buf         = *buf;
-   
-   if (!path_is_valid(patch_path))
-      return false;
-
-   if (!filestream_read_file(patch_path, &patch_data, &patch_size))
-      return false;
-
-   if (patch_size < 0)
-   {
-      free(patch_data);
-      return false;
-   }
-
-   if (!path_file_exists(patch_path))
-   {
-      free(patch_data);
-      return false;
-   }
+   uint64_t target_size     = ret_size * 4; /* Just to be sure. */
+   uint8_t *patched_content = (uint8_t*)malloc((size_t)target_size);
 
    RARCH_LOG("Found %s file in \"%s\", attempting to patch ...\n",
          patch_desc, patch_path);
 
-   target_size = ret_size * 4; /* Just to be sure. */
-
-   patched_content = (uint8_t*)malloc(target_size);
-
    if (!patched_content)
    {
       RARCH_ERR("%s\n",
             msg_hash_to_str(MSG_FAILED_TO_ALLOCATE_MEMORY_FOR_PATCHED_CONTENT));
-      goto error;
+
+      *buf  = ret_buf;
+      *size = ret_size;
+
+      return false;
    }
 
    err = func((const uint8_t*)patch_data, patch_size, ret_buf,
@@ -526,7 +531,9 @@ static bool apply_patch_content(uint8_t **buf,
       RARCH_LOG("%s (%s).\n",
             msg_hash_to_str(MSG_FATAL_ERROR_RECEIVED_IN),
             patch_desc);
-      success = true;
+      free(ret_buf);
+      *buf  = patched_content;
+      *size = target_size;
    }
    else
       RARCH_ERR("%s %s: %s #%u\n",
@@ -535,30 +542,33 @@ static bool apply_patch_content(uint8_t **buf,
             msg_hash_to_str(MSG_ERROR),
             (unsigned)err);
 
-   if (success)
-   {
-      free(ret_buf);
-      *buf = patched_content;
-      *size = target_size;
-   }
-
-   free(patch_data);
    return true;
-
-error:
-   *buf = ret_buf;
-   *size = ret_size;
-   free(patch_data);
-
-   return false;
 }
 
 static bool try_bps_patch(bool allow_bps, const char *name_bps,
       uint8_t **buf, ssize_t *size)
 {
    if (allow_bps && !string_is_empty(name_bps))
-      return apply_patch_content(buf, size, "BPS", name_bps,
-            bps_apply_patch);
+      if (path_is_valid(name_bps) && filestream_exists(name_bps))
+      {
+         int64_t patch_size;
+         bool ret                 = false;
+         void *patch_data         = NULL;
+
+         if (!filestream_read_file(name_bps, &patch_data, &patch_size))
+            return false;
+
+         if (patch_size >= 0)
+         {
+            ret                      = apply_patch_content(
+                  buf, size, "BPS", name_bps,
+                  bps_apply_patch, patch_data, patch_size);
+         }
+
+         if (patch_data)
+            free(patch_data);
+         return ret;
+      }
    return false;
 }
 
@@ -566,8 +576,26 @@ static bool try_ups_patch(bool allow_ups, const char *name_ups,
       uint8_t **buf, ssize_t *size)
 {
    if (allow_ups && !string_is_empty(name_ups))
-      return apply_patch_content(buf, size, "UPS", name_ups,
-            ups_apply_patch);
+      if (path_is_valid(name_ups) && filestream_exists(name_ups))
+      {
+         int64_t patch_size;
+         bool ret                 = false;
+         void *patch_data         = NULL;
+
+         if (!filestream_read_file(name_ups, &patch_data, &patch_size))
+            return false;
+
+         if (patch_size >= 0)
+         {
+            ret                      = apply_patch_content(
+                  buf, size, "UPS", name_ups,
+                  ups_apply_patch, patch_data, patch_size);
+         }
+
+         if (patch_data)
+            free(patch_data);
+         return ret;
+      }
    return false;
 }
 
@@ -575,7 +603,26 @@ static bool try_ips_patch(bool allow_ips,
       const char *name_ips, uint8_t **buf, ssize_t *size)
 {
    if (allow_ips && !string_is_empty(name_ips))
-      return apply_patch_content(buf, size, "IPS", name_ips, ips_apply_patch);
+      if (path_is_valid(name_ips) && filestream_exists(name_ips))
+      {
+         int64_t patch_size;
+         bool ret                 = false;
+         void *patch_data         = NULL;
+
+         if (!filestream_read_file(name_ips, &patch_data, &patch_size))
+            return false;
+
+         if (patch_size >= 0)
+         {
+            ret                      = apply_patch_content(
+                  buf, size, "IPS", name_ips,
+                  ips_apply_patch, patch_data, patch_size);
+         }
+
+         if (patch_data)
+            free(patch_data);
+         return ret;
+      }
    return false;
 }
 
@@ -588,6 +635,9 @@ static bool try_ips_patch(bool allow_ips,
  *
  **/
 static void patch_content(
+      bool is_ips_pref,
+      bool is_bps_pref,
+      bool is_ups_pref,
       const char *name_ips,
       const char *name_bps,
       const char *name_ups,
@@ -595,21 +645,21 @@ static void patch_content(
       void *data)
 {
    ssize_t *size    = (ssize_t*)data;
-   bool allow_ups   = !rarch_ctl(RARCH_CTL_IS_BPS_PREF, NULL) && !rarch_ctl(RARCH_CTL_IS_IPS_PREF, NULL);
-   bool allow_ips   = !rarch_ctl(RARCH_CTL_IS_UPS_PREF, NULL) && !rarch_ctl(RARCH_CTL_IS_BPS_PREF, NULL);
-   bool allow_bps   = !rarch_ctl(RARCH_CTL_IS_UPS_PREF, NULL) && !rarch_ctl(RARCH_CTL_IS_IPS_PREF, NULL);
+   bool allow_ups   = !is_bps_pref && !is_ips_pref;
+   bool allow_ips   = !is_ups_pref && !is_bps_pref;
+   bool allow_bps   = !is_ups_pref && !is_ips_pref;
 
-   if (    (unsigned)rarch_ctl(RARCH_CTL_IS_IPS_PREF, NULL) 
-         + (unsigned)rarch_ctl(RARCH_CTL_IS_BPS_PREF, NULL) 
-         + (unsigned)rarch_ctl(RARCH_CTL_IS_UPS_PREF, NULL) > 1)
+   if (    (unsigned)is_ips_pref
+         + (unsigned)is_bps_pref
+         + (unsigned)is_ups_pref > 1)
    {
       RARCH_WARN("%s\n",
             msg_hash_to_str(MSG_SEVERAL_PATCHES_ARE_EXPLICITLY_DEFINED));
       return;
    }
 
-   if (     !try_ips_patch(allow_ips, name_ips, buf, size) 
-         && !try_bps_patch(allow_bps, name_bps, buf, size) 
+   if (     !try_ips_patch(allow_ips, name_ips, buf, size)
+         && !try_bps_patch(allow_bps, name_bps, buf, size)
          && !try_ups_patch(allow_ups, name_ups, buf, size))
    {
       RARCH_LOG("%s\n",
