@@ -13,7 +13,6 @@
  *  You should have received a copy of the GNU General Public License along with RetroArch.
  *  If not, see <http://www.gnu.org/licenses/>.
  */
-#include <stdio.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -30,10 +29,12 @@
 #include <compat/strl.h>
 #include <retro_assert.h>
 #include <lists/string_list.h>
+#include <streams/interface_stream.h>
 #include <streams/file_stream.h>
 #include <rthreads/rthreads.h>
 #include <file/file_path.h>
 #include <retro_miscellaneous.h>
+#include <string/stdstring.h>
 
 #ifdef HAVE_CONFIG_H
 #include "../core.h"
@@ -46,9 +47,9 @@
 #include "../core.h"
 #include "../file_path_special.h"
 #include "../configuration.h"
+#include "../gfx/video_driver.h"
 #include "../msg_hash.h"
 #include "../retroarch.h"
-#include "../runloop.h"
 #include "../verbosity.h"
 #include "tasks_internal.h"
 
@@ -78,7 +79,7 @@ struct sram_block
 
 typedef struct
 {
-   RFILE *file;
+   intfstream_t *file;
    char path[PATH_MAX_LENGTH];
    void *data;
    void *undo_data;
@@ -93,6 +94,7 @@ typedef struct
    bool mute;
    int state_slot;
    bool thumbnail_enable;
+   bool has_valid_framebuffer;
 } save_task_state_t;
 
 typedef save_task_state_t load_task_data_t;
@@ -118,17 +120,15 @@ struct autosave_st
 struct autosave
 {
    volatile bool quit;
-   slock_t *lock;
-
-   slock_t *cond_lock;
-   scond_t *cond;
-   sthread_t *thread;
-
+   size_t bufsize;
+   unsigned interval;
    void *buffer;
    const void *retro_buffer;
    const char *path;
-   size_t bufsize;
-   unsigned interval;
+   slock_t *lock;
+   slock_t *cond_lock;
+   scond_t *cond;
+   sthread_t *thread;
 };
 
 static struct autosave_st autosave_state;
@@ -149,8 +149,8 @@ static void autosave_thread(void *data)
       bool differ;
 
       slock_lock(save->lock);
-      differ = memcmp(save->buffer, save->retro_buffer,
-            save->bufsize) != 0;
+      differ = string_is_not_equal_fast(save->buffer, save->retro_buffer,
+            save->bufsize);
       if (differ)
          memcpy(save->buffer, save->retro_buffer, save->bufsize);
       slock_unlock(save->lock);
@@ -158,7 +158,8 @@ static void autosave_thread(void *data)
       if (differ)
       {
          /* Should probably deal with this more elegantly. */
-         FILE *file = fopen(save->path, "wb");
+         intfstream_t *file = intfstream_open_file(save->path,
+               RETRO_VFS_FILE_ACCESS_WRITE, RETRO_VFS_FILE_ACCESS_HINT_NONE);
 
          if (file)
          {
@@ -174,10 +175,10 @@ static void autosave_thread(void *data)
             else
                RARCH_LOG("SRAM changed ... autosaving ...\n");
 
-            failed |= fwrite(save->buffer, 1, save->bufsize, file)
-               != save->bufsize;
-            failed |= fflush(file) != 0;
-            failed |= fclose(file) != 0;
+            failed |= ((size_t)intfstream_write(file, save->buffer, save->bufsize) != save->bufsize);
+            failed |= (intfstream_flush(file) != 0);
+            failed |= (intfstream_close(file) != 0);
+            free(file);
             if (failed)
                RARCH_WARN("Failed to autosave SRAM. Disk might be full.\n");
          }
@@ -186,8 +187,11 @@ static void autosave_thread(void *data)
       slock_lock(save->cond_lock);
 
       if (!save->quit)
-         scond_wait_timeout(save->cond, save->cond_lock,
-               save->interval * 1000000LL);
+#if defined(_MSC_VER) && _MSC_VER <= 1200
+         scond_wait_timeout(save->cond, save->cond_lock, save->interval * 1000000);
+#else
+         scond_wait_timeout(save->cond, save->cond_lock, save->interval * 1000000LL);
+#endif
 
       slock_unlock(save->cond_lock);
    }
@@ -209,26 +213,26 @@ static autosave_t *autosave_new(const char *path,
       const void *data, size_t size,
       unsigned interval)
 {
-   autosave_t *handle   = (autosave_t*)calloc(1, sizeof(*handle));
+   autosave_t *handle            = (autosave_t*)malloc(sizeof(*handle));
    if (!handle)
       goto error;
 
-   handle->bufsize      = size;
-   handle->interval     = interval;
-   handle->path         = path;
-   handle->buffer       = malloc(size);
-   handle->retro_buffer = data;
+   handle->quit                  = false;
+   handle->bufsize               = size;
+   handle->interval              = interval;
+   handle->buffer                = malloc(size);
+   handle->retro_buffer          = data;
+   handle->path                  = path;
 
    if (!handle->buffer)
       goto error;
 
    memcpy(handle->buffer, handle->retro_buffer, handle->bufsize);
 
-   handle->lock         = slock_new();
-   handle->cond_lock    = slock_new();
-   handle->cond         = scond_new();
-
-   handle->thread       = sthread_create(autosave_thread, handle);
+   handle->lock                  = slock_new();
+   handle->cond_lock             = slock_new();
+   handle->cond                  = scond_new();
+   handle->thread                = sthread_create(autosave_thread, handle);
 
    return handle;
 
@@ -262,44 +266,55 @@ static void autosave_free(autosave_t *handle)
 }
 
 
-void autosave_init(void)
+bool autosave_init(void)
 {
    unsigned i;
-   autosave_t **list    = NULL;
-   settings_t *settings = config_get_ptr();
+   autosave_t **list          = NULL;
+   settings_t *settings       = config_get_ptr();
+   unsigned autosave_interval = settings->uints.autosave_interval;
 
-   if (settings->autosave_interval < 1 || !task_save_files)
-      return;
+   if (autosave_interval < 1 || !task_save_files)
+      return false;
+
+   list                       = (autosave_t**)
+      calloc(task_save_files->size,
+            sizeof(*autosave_state.list));
 
-   list = (autosave_t**)calloc(task_save_files->size,
-               sizeof(*autosave_state.list));
    if (!list)
-      return;
+      return false;
 
    autosave_state.list = list;
-   autosave_state.num  = task_save_files->size;
+   autosave_state.num  = (unsigned)task_save_files->size;
 
    for (i = 0; i < task_save_files->size; i++)
    {
       retro_ctx_memory_info_t mem_info;
-      const char *path = task_save_files->elems[i].data;
-      unsigned    type = task_save_files->elems[i].attr.i;
+      autosave_t *auto_st = NULL;
+      const char *path    = task_save_files->elems[i].data;
+      unsigned    type    = task_save_files->elems[i].attr.i;
 
-      mem_info.id = type;
+      mem_info.id         = type;
 
       core_get_memory(&mem_info);
 
       if (mem_info.size <= 0)
          continue;
 
-      autosave_state.list[i] = autosave_new(path,
+      auto_st             = autosave_new(path,
             mem_info.data,
             mem_info.size,
-            settings->autosave_interval);
+            autosave_interval);
 
-      if (!autosave_state.list[i])
+      if (!auto_st)
+      {
          RARCH_WARN("%s\n", msg_hash_to_str(MSG_AUTOSAVE_FAILED));
+         continue;
+      }
+
+      autosave_state.list[i] = auto_st;
    }
+
+   return true;
 }
 
 void autosave_deinit(void)
@@ -388,11 +403,11 @@ bool content_undo_load_state(void)
          undo_load_buf.size,
          msg_hash_to_str(MSG_BYTES));
 
-   /* TODO/FIXME - This checking of SRAM overwrite, 
+   /* TODO/FIXME - This checking of SRAM overwrite,
     * the backing up of it and
-    * its flushing could all be in their 
+    * its flushing could all be in their
     * own functions... */
-   if (settings->block_sram_overwrite && task_save_files
+   if (settings->bools.block_sram_overwrite && task_save_files
          && task_save_files->size)
    {
       RARCH_LOG("%s.\n",
@@ -402,7 +417,7 @@ bool content_undo_load_state(void)
 
       if (blocks)
       {
-         num_blocks = task_save_files->size;
+         num_blocks = (unsigned)task_save_files->size;
          for (i = 0; i < num_blocks; i++)
             blocks[i].type = task_save_files->elems[i].attr.i;
       }
@@ -517,7 +532,8 @@ static void task_save_handler_finished(retro_task_t *task,
 
    task_set_finished(task, true);
 
-   filestream_close(state->file);
+   intfstream_close(state->file);
+   free(state->file);
 
    if (!task_get_error(task) && task_get_cancelled(task))
       task_set_error(task, strdup("Task canceled"));
@@ -552,14 +568,15 @@ static void task_save_handler(retro_task_t *task)
 
    if (!state->file)
    {
-      state->file = filestream_open(state->path, RFILE_MODE_WRITE, -1);
+      state->file = intfstream_open_file(state->path, RETRO_VFS_FILE_ACCESS_WRITE,
+            RETRO_VFS_FILE_ACCESS_HINT_NONE);
 
       if (!state->file)
          return;
    }
 
    remaining       = MIN(state->size - state->written, SAVE_STATE_CHUNK);
-   written         = filestream_write(state->file,
+   written         = (int)intfstream_write(state->file,
          (uint8_t*)state->data + state->written, remaining);
 
    state->written += written;
@@ -568,7 +585,7 @@ static void task_save_handler(retro_task_t *task)
 
    if (task_get_cancelled(task) || written != remaining)
    {
-      char err[PATH_MAX_LENGTH];
+      char err[256];
 
       err[0] = '\0';
 
@@ -611,10 +628,16 @@ static void task_save_handler(retro_task_t *task)
       }
 
       if (!task_get_mute(task) && msg)
+      {
          task_set_title(task, msg);
+         msg = NULL;
+      }
 
       task_save_handler_finished(task, state);
 
+      if (!string_is_empty(msg))
+         free(msg);
+
       return;
    }
 }
@@ -637,18 +660,19 @@ static bool task_push_undo_save_state(const char *path, void *data, size_t size)
       goto error;
 
    strlcpy(state->path, path, sizeof(state->path));
-   state->data       = data;
-   state->size       = size;
-   state->undo_save  = true;
-   state->state_slot = settings->state_slot;
+   state->data                   = data;
+   state->size                   = size;
+   state->undo_save              = true;
+   state->state_slot             = settings->ints.state_slot;
+   state->has_valid_framebuffer  = video_driver_cached_frame_has_valid_framebuffer();
 
-   task->type        = TASK_TYPE_BLOCKING;
-   task->state       = state;
-   task->handler     = task_save_handler;
-   task->callback    = undo_save_state_cb;
-   task->title       = strdup(msg_hash_to_str(MSG_UNDOING_SAVE_STATE));
+   task->type                    = TASK_TYPE_BLOCKING;
+   task->state                   = state;
+   task->handler                 = task_save_handler;
+   task->callback                = undo_save_state_cb;
+   task->title                   = strdup(msg_hash_to_str(MSG_UNDOING_SAVE_STATE));
 
-   task_queue_ctl(TASK_QUEUE_CTL_PUSH, task);
+   task_queue_push(task);
 
    return true;
 
@@ -691,7 +715,10 @@ static void task_load_handler_finished(retro_task_t *task,
    task_set_finished(task, true);
 
    if (state->file)
-      filestream_close(state->file);
+   {
+      intfstream_close(state->file);
+      free(state->file);
+   }
 
    if (!task_get_error(task) && task_get_cancelled(task))
       task_set_error(task, strdup("Task canceled"));
@@ -717,20 +744,22 @@ static void task_load_handler(retro_task_t *task)
 
    if (!state->file)
    {
-      state->file = filestream_open(state->path, RFILE_MODE_READ, -1);
+      state->file = intfstream_open_file(state->path,
+            RETRO_VFS_FILE_ACCESS_READ,
+            RETRO_VFS_FILE_ACCESS_HINT_NONE);
 
       if (!state->file)
          goto error;
 
-      if (filestream_seek(state->file, 0, SEEK_END) != 0)
+      if (intfstream_seek(state->file, 0, SEEK_END) != 0)
          goto error;
 
-      state->size = filestream_tell(state->file);
+      state->size = intfstream_tell(state->file);
 
       if (state->size < 0)
          goto error;
 
-      filestream_rewind(state->file);
+      intfstream_rewind(state->file);
 
       state->data = malloc(state->size + 1);
 
@@ -739,7 +768,7 @@ static void task_load_handler(retro_task_t *task)
    }
 
    remaining          = MIN(state->size - state->bytes_read, SAVE_STATE_CHUNK);
-   bytes_read         = filestream_read(state->file,
+   bytes_read         = intfstream_read(state->file,
          (uint8_t*)state->data + state->bytes_read, remaining);
    state->bytes_read += bytes_read;
 
@@ -750,15 +779,18 @@ static void task_load_handler(retro_task_t *task)
    {
       if (state->autoload)
       {
-         char msg[1024];
+         char *msg = (char*)malloc(1024 * sizeof(char));
 
          msg[0] = '\0';
 
-         snprintf(msg, sizeof(msg), "%s \"%s\" %s.",
+         snprintf(msg,
+               1024 * sizeof(char),
+               "%s \"%s\" %s.",
                msg_hash_to_str(MSG_AUTOLOADING_SAVESTATE_FROM),
                state->path,
                msg_hash_to_str(MSG_FAILED));
          task_set_error(task, strdup(msg));
+         free(msg);
       }
       else
          task_set_error(task, strdup(msg_hash_to_str(MSG_FAILED_TO_LOAD_STATE)));
@@ -771,7 +803,7 @@ static void task_load_handler(retro_task_t *task)
 
    if (state->bytes_read == state->size)
    {
-      char msg[1024];
+      char *msg = (char*)malloc(1024 * sizeof(char));
 
       msg[0] = '\0';
 
@@ -779,7 +811,9 @@ static void task_load_handler(retro_task_t *task)
 
       if (state->autoload)
       {
-         snprintf(msg, sizeof(msg), "%s \"%s\" %s.",
+         snprintf(msg,
+               1024 * sizeof(char),
+               "%s \"%s\" %s.",
                msg_hash_to_str(MSG_AUTOLOADING_SAVESTATE_FROM),
                state->path,
                msg_hash_to_str(MSG_SUCCEEDED));
@@ -787,9 +821,13 @@ static void task_load_handler(retro_task_t *task)
       else
       {
          if (state->state_slot < 0)
-            strlcpy(msg, msg_hash_to_str(MSG_LOADED_STATE_FROM_SLOT_AUTO), sizeof(msg));
+            strlcpy(msg, msg_hash_to_str(MSG_LOADED_STATE_FROM_SLOT_AUTO),
+                  1024 * sizeof(char)
+                  );
          else
-            snprintf(msg, sizeof(msg), msg_hash_to_str(MSG_LOADED_STATE_FROM_SLOT),
+            snprintf(msg,
+                  1024 * sizeof(char),
+                  msg_hash_to_str(MSG_LOADED_STATE_FROM_SLOT),
                   state->state_slot);
 
       }
@@ -797,6 +835,7 @@ static void task_load_handler(retro_task_t *task)
       if (!task_get_mute(task))
          task_set_title(task, strdup(msg));
 
+      free(msg);
       task_load_handler_finished(task, state);
 
       return;
@@ -863,7 +902,7 @@ static void content_load_state_cb(void *task_data,
       return;
    }
 
-   if (settings->block_sram_overwrite && task_save_files
+   if (settings->bools.block_sram_overwrite && task_save_files
          && task_save_files->size)
    {
       RARCH_LOG("%s.\n",
@@ -873,7 +912,7 @@ static void content_load_state_cb(void *task_data,
 
       if (blocks)
       {
-         num_blocks = task_save_files->size;
+         num_blocks = (unsigned)task_save_files->size;
          for (i = 0; i < num_blocks; i++)
             blocks[i].type = task_save_files->elems[i].attr.i;
       }
@@ -970,7 +1009,7 @@ static void save_state_cb(void *task_data,
    char               *path = strdup(state->path);
 
    if (state->thumbnail_enable)
-      take_screenshot(path, true);
+      take_screenshot(path, true, state->has_valid_framebuffer);
 
    free(path);
 }
@@ -997,7 +1036,9 @@ static void task_push_save_state(const char *path, void *data, size_t size, bool
    state->size             = size;
    state->autosave         = autosave;
    state->mute             = autosave; /* don't show OSD messages if we are auto-saving */
-   state->thumbnail_enable = settings->savestate_thumbnail_enable;
+   state->thumbnail_enable = settings->bools.savestate_thumbnail_enable;
+   state->state_slot       = settings->ints.state_slot;
+   state->has_valid_framebuffer  = video_driver_cached_frame_has_valid_framebuffer();
 
    task->type              = TASK_TYPE_BLOCKING;
    task->state             = state;
@@ -1006,7 +1047,7 @@ static void task_push_save_state(const char *path, void *data, size_t size, bool
    task->title             = strdup(msg_hash_to_str(MSG_SAVING_STATE));
    task->mute              = state->mute;
 
-   task_queue_ctl(TASK_QUEUE_CTL_PUSH, task);
+   task_queue_push(task);
 
    return;
 
@@ -1054,27 +1095,32 @@ static void content_load_and_save_state_cb(void *task_data,
 static void task_push_load_and_save_state(const char *path, void *data,
       size_t size, bool load_to_backup_buffer, bool autosave)
 {
-   retro_task_t       *task = (retro_task_t*)calloc(1, sizeof(*task));
-   save_task_state_t *state = (save_task_state_t*)calloc(1, sizeof(*state));
+   retro_task_t      *task     = (retro_task_t*)calloc(1, sizeof(*task));
+   save_task_state_t *state    = (save_task_state_t*)calloc(1, sizeof(*state));
+   settings_t        *settings = config_get_ptr();
 
    if (!task || !state)
       goto error;
 
    strlcpy(state->path, path, sizeof(state->path));
    state->load_to_backup_buffer = load_to_backup_buffer;
-   state->undo_size = size;
-   state->undo_data = data;
-   state->autosave  = autosave;
-   state->mute      = autosave; /* don't show OSD messages if we are auto-saving */
+   state->undo_size  = size;
+   state->undo_data  = data;
+   state->autosave   = autosave;
+   state->mute       = autosave; /* don't show OSD messages if we are auto-saving */
+   if(load_to_backup_buffer)
+      state->mute       = true;
+   state->state_slot = settings->ints.state_slot;
+   state->has_valid_framebuffer  = video_driver_cached_frame_has_valid_framebuffer();
 
-   task->state      = state;
-   task->type       = TASK_TYPE_BLOCKING;
-   task->handler    = task_load_handler;
-   task->callback   = content_load_and_save_state_cb;
-   task->title      = strdup(msg_hash_to_str(MSG_LOADING_STATE));
-   task->mute       = state->mute;
+   task->state       = state;
+   task->type        = TASK_TYPE_BLOCKING;
+   task->handler     = task_load_handler;
+   task->callback    = content_load_and_save_state_cb;
+   task->title       = strdup(msg_hash_to_str(MSG_LOADING_STATE));
+   task->mute        = state->mute;
 
-   task_queue_ctl(TASK_QUEUE_CTL_PUSH, task);
+   task_queue_push(task);
 
    return;
 
@@ -1129,7 +1175,7 @@ bool content_save_state(const char *path, bool save_to_disk, bool autosave)
    {
       if (save_to_disk)
       {
-         if (path_file_exists(path))
+         if (filestream_exists(path) && !autosave)
          {
             /* Before overwritting the savestate file, load it into a buffer
             to allow undo_save_state() to work */
@@ -1191,8 +1237,9 @@ bool content_save_state(const char *path, bool save_to_disk, bool autosave)
 bool content_load_state(const char *path,
       bool load_to_backup_buffer, bool autoload)
 {
-   retro_task_t       *task = (retro_task_t*)calloc(1, sizeof(*task));
-   save_task_state_t *state = (save_task_state_t*)calloc(1, sizeof(*state));
+   retro_task_t       *task     = (retro_task_t*)calloc(1, sizeof(*task));
+   save_task_state_t *state     = (save_task_state_t*)calloc(1, sizeof(*state));
+   settings_t *settings         = config_get_ptr();
 
    if (!task || !state)
       goto error;
@@ -1200,14 +1247,16 @@ bool content_load_state(const char *path,
    strlcpy(state->path, path, sizeof(state->path));
    state->load_to_backup_buffer = load_to_backup_buffer;
    state->autoload              = autoload;
+   state->state_slot            = settings->ints.state_slot;
+   state->has_valid_framebuffer  = video_driver_cached_frame_has_valid_framebuffer();
 
-   task->type     = TASK_TYPE_BLOCKING;
-   task->state    = state;
-   task->handler  = task_load_handler;
-   task->callback = content_load_state_cb;
-   task->title    = strdup(msg_hash_to_str(MSG_LOADING_STATE));
+   task->type                   = TASK_TYPE_BLOCKING;
+   task->state                  = state;
+   task->handler                = task_load_handler;
+   task->callback               = content_load_state_cb;
+   task->title                  = strdup(msg_hash_to_str(MSG_LOADING_STATE));
 
-   task_queue_ctl(TASK_QUEUE_CTL_PUSH, task);
+   task_queue_push(task);
 
    return true;
 
@@ -1223,10 +1272,10 @@ error:
 bool content_rename_state(const char *origin, const char *dest)
 {
    int ret = 0;
-   if (path_file_exists(dest))
-      unlink(dest);
+   if (filestream_exists(dest))
+      filestream_delete(dest);
 
-   ret = rename (origin, dest);
+   ret = filestream_rename(origin, dest);
    if (!ret)
       return true;
 
@@ -1298,7 +1347,7 @@ static bool content_get_memory(retro_ctx_memory_info_t *mem_info,
  */
 bool content_load_ram_file(unsigned slot)
 {
-   ssize_t rc;
+   int64_t rc;
    struct ram_type ram;
    retro_ctx_memory_info_t mem_info;
    void *buf        = NULL;
@@ -1321,7 +1370,7 @@ bool content_load_ram_file(unsigned slot)
                (unsigned)mem_info.size);
          rc = mem_info.size;
       }
-      memcpy(mem_info.data, buf, rc);
+      memcpy(mem_info.data, buf, (size_t)rc);
    }
 
    if (buf)
@@ -1342,29 +1391,44 @@ static bool dump_to_file_desperate(const void *data,
       size_t size, unsigned type)
 {
    time_t time_;
-   char timebuf[256];
-   char application_data[PATH_MAX_LENGTH];
-   char path[PATH_MAX_LENGTH];
+   char *path             = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
+   char *timebuf          = (char*)malloc(256 * sizeof(char));
+   char *application_data = (char*)malloc(PATH_MAX_LENGTH * sizeof(char));
 
    timebuf[0] = application_data[0] = path[0] = '\0';
 
    if (!fill_pathname_application_data(application_data,
-            sizeof(application_data)))
-      return false;
+            PATH_MAX_LENGTH * sizeof(char)))
+      goto error;
 
-   snprintf(path, sizeof(path), "%s/RetroArch-recovery-%u",
-      application_data, type);
+   snprintf(path,
+         PATH_MAX_LENGTH * sizeof(char),
+         "%s/RetroArch-recovery-%u",
+         application_data, type);
 
    time(&time_);
 
-   strftime(timebuf, sizeof(timebuf), "%Y-%m-%d-%H-%M-%S", localtime(&time_));
-   strlcat(path, timebuf, sizeof(path));
+   strftime(timebuf,
+         256 * sizeof(char),
+         "%Y-%m-%d-%H-%M-%S", localtime(&time_));
+   strlcat(path, timebuf,
+         PATH_MAX_LENGTH * sizeof(char)
+         );
 
    if (!filestream_write_file(path, data, size))
-      return false;
+      goto error;
 
+   free(application_data);
+   free(timebuf);
    RARCH_WARN("Succeeded in saving RAM data to \"%s\".\n", path);
+   free(path);
    return true;
+
+error:
+   free(application_data);
+   free(timebuf);
+   free(path);
+   return false;
 }
 
 /**
@@ -1389,7 +1453,8 @@ bool content_save_ram_file(unsigned slot)
          msg_hash_to_str(MSG_TO),
          ram.path);
 
-   if (!filestream_write_file(ram.path, mem_info.data, mem_info.size))
+   if (!filestream_write_file(
+            ram.path, mem_info.data, mem_info.size))
    {
       RARCH_ERR("%s.\n",
             msg_hash_to_str(MSG_FAILED_TO_SAVE_SRAM));
@@ -1398,7 +1463,8 @@ bool content_save_ram_file(unsigned slot)
       /* In case the file could not be written to,
        * the fallback function 'dump_to_file_desperate'
        * will be called. */
-      if (!dump_to_file_desperate(mem_info.data, mem_info.size, ram.type))
+      if (!dump_to_file_desperate(
+               mem_info.data, mem_info.size, ram.type))
       {
          RARCH_WARN("Failed ... Cannot recover save file.\n");
       }
@@ -1416,7 +1482,8 @@ bool event_save_files(void)
 {
    unsigned i;
 
-   if (!task_save_files || !rarch_ctl(RARCH_CTL_IS_SRAM_USED, NULL))
+   if (!task_save_files ||
+         !rarch_ctl(RARCH_CTL_IS_SRAM_USED, NULL))
       return false;
 
    for (i = 0; i < task_save_files->size; i++)
@@ -1429,7 +1496,8 @@ bool event_load_save_files(void)
 {
    unsigned i;
 
-   if (!task_save_files || rarch_ctl(RARCH_CTL_IS_SRAM_LOAD_DISABLED, NULL))
+   if (!task_save_files ||
+         rarch_ctl(RARCH_CTL_IS_SRAM_LOAD_DISABLED, NULL))
       return false;
 
    for (i = 0; i < task_save_files->size; i++)
@@ -1438,24 +1506,25 @@ bool event_load_save_files(void)
    return true;
 }
 
-void path_init_savefile_rtc(void)
+void path_init_savefile_rtc(const char *savefile_path)
 {
    union string_list_elem_attr attr;
-   char savefile_name_rtc[PATH_MAX_LENGTH];
-   global_t                        *global = global_get_ptr();
+   char *savefile_name_rtc = (char*)
+      malloc(PATH_MAX_LENGTH * sizeof(char));
 
    savefile_name_rtc[0] = '\0';
 
    attr.i = RETRO_MEMORY_SAVE_RAM;
-   string_list_append(task_save_files, global->name.savefile, attr);
+   string_list_append(task_save_files, savefile_path, attr);
 
    /* Infer .rtc save path from save ram path. */
    attr.i = RETRO_MEMORY_RTC;
    fill_pathname(savefile_name_rtc,
-         global->name.savefile,
+         savefile_path,
          file_path_str(FILE_PATH_RTC_EXTENSION),
-         sizeof(savefile_name_rtc));
+         PATH_MAX_LENGTH * sizeof(char));
    string_list_append(task_save_files, savefile_name_rtc, attr);
+   free(savefile_name_rtc);
 }
 
 void path_deinit_savefile(void)
