@@ -19,11 +19,6 @@
 #include "../config.h"
 #endif
 
-#ifdef _XBOX1
-#include <xtl.h>
-#include <xgraphics.h>
-#endif
-
 #include <stdio.h>
 #include <stddef.h>
 #include <time.h>
@@ -49,8 +44,9 @@
 #endif
 
 #include "../defaults.h"
+#include "../command.h"
 #include "../configuration.h"
-#include "../runloop.h"
+#include "../retroarch.h"
 #include "../paths.h"
 #include "../msg_hash.h"
 
@@ -58,27 +54,26 @@
 
 #include "tasks_internal.h"
 
-typedef struct
+typedef struct screenshot_task_state screenshot_task_state_t;
+
+struct screenshot_task_state
 {
-#ifdef _XBOX1
-   D3DSurface *surf;
-#endif
-   char filename[PATH_MAX_LENGTH];
-   char shotname[256];
-   uint8_t *out_buffer;
-   struct scaler_ctx scaler;
-   const void *frame;
-   unsigned width;
-   unsigned height;
-   int pitch;
    bool bgr24;
    bool silence;
-   void *userbuf;
    bool is_idle;
    bool is_paused;
    bool history_list_enable;
+   int pitch;
+   unsigned width;
+   unsigned height;
    unsigned pixel_format_type;
-} screenshot_task_state_t;
+   uint8_t *out_buffer;
+   const void *frame;
+   char filename[PATH_MAX_LENGTH];
+   char shotname[256];
+   void *userbuf;
+   struct scaler_ctx scaler;
+};
 
 /**
  * task_screenshot_handler:
@@ -105,16 +100,12 @@ static void task_screenshot_handler(retro_task_t *task)
       free(state);
       return;
    }
-    
+
 #ifdef HAVE_RBMP
     (void)bmp_type;
 #endif
 
-#if defined(_XBOX1)
-   if (XGWriteSurfaceToFile(state->surf, state->filename) == S_OK)
-      ret = true;
-   state->surf->Release();
-#elif defined(HAVE_RPNG)
+#if defined(HAVE_RPNG)
    if (state->bgr24)
       scaler->in_fmt   = SCALER_FMT_BGR24;
    else if (state->pixel_format_type == RETRO_PIXEL_FORMAT_XRGB8888)
@@ -125,7 +116,7 @@ static void task_screenshot_handler(retro_task_t *task)
    video_frame_convert_to_bgr24(
          scaler,
          state->out_buffer,
-         (const uint8_t*)state->frame + ((int)state->height - 1) 
+         (const uint8_t*)state->frame + ((int)state->height - 1)
          * state->pitch,
          state->width, state->height,
          -state->pitch);
@@ -156,23 +147,16 @@ static void task_screenshot_handler(retro_task_t *task)
 #endif
 
 #ifdef HAVE_IMAGEVIEWER
-   if (ret && !state->silence)
-   {
-      if (
-            state->history_list_enable 
-            && g_defaults.image_history 
-            && playlist_push(
-               g_defaults.image_history,
-               state->filename,
-               NULL,
-               "builtin",
-               "imageviewer",
-               NULL,
-               NULL
-               )
+   if (  ret                        &&
+         !state->silence            &&
+         state->history_list_enable
          )
-         playlist_write_file(g_defaults.image_history);
-   }
+      command_playlist_push_write(
+            g_defaults.image_history,
+            state->filename,
+            NULL,
+            "builtin",
+            "imageviewer");
 #endif
 
    task_set_progress(task, 100);
@@ -198,18 +182,15 @@ static bool screenshot_dump(
 {
    char screenshot_path[PATH_MAX_LENGTH];
    uint8_t *buf                   = NULL;
-#ifdef _XBOX1
-   d3d_video_t *d3d               = (d3d_video_t*)video_driver_get_ptr(true);
-#endif
    settings_t *settings           = config_get_ptr();
    retro_task_t *task             = (retro_task_t*)calloc(1, sizeof(*task));
    screenshot_task_state_t *state = (screenshot_task_state_t*)
          calloc(1, sizeof(*state));
-   const char *screenshot_dir     = settings->directory.screenshot;
+   const char *screenshot_dir     = settings->paths.directory_screenshot;
 
    screenshot_path[0]             = '\0';
 
-   if (string_is_empty(screenshot_dir))
+   if (string_is_empty(screenshot_dir) || settings->bools.screenshots_in_content_dir)
    {
       fill_pathname_basedir(screenshot_path, name_base,
             sizeof(screenshot_path));
@@ -225,7 +206,7 @@ static bool screenshot_dump(
    state->frame               = frame;
    state->userbuf             = userbuf;
    state->silence             = savestate;
-   state->history_list_enable = settings->history_list_enable;
+   state->history_list_enable = settings->bools.history_list_enable;
    state->pixel_format_type   = video_driver_get_pixel_format();
 
    if (savestate)
@@ -233,7 +214,7 @@ static bool screenshot_dump(
             sizeof(state->filename), "%s.png", name_base);
    else
    {
-      if (settings->auto_screenshot_filename)
+      if (settings->bools.auto_screenshot_filename)
          fill_str_dated_filename(state->shotname, path_basename(name_base),
                IMG_EXT, sizeof(state->shotname));
       else
@@ -244,9 +225,7 @@ static bool screenshot_dump(
             state->shotname, sizeof(state->filename));
    }
 
-#ifdef _XBOX1
-   d3d->dev->GetBackBuffer(-1, D3DBACKBUFFER_TYPE_MONO, &state->surf);
-#elif defined(HAVE_RPNG)
+#if defined(HAVE_RPNG)
    buf = (uint8_t*)malloc(width * height * 3);
    if (!buf)
    {
@@ -265,7 +244,7 @@ static bool screenshot_dump(
    if (!savestate)
       task->title    = strdup(msg_hash_to_str(MSG_TAKING_SCREENSHOT));
 
-   task_queue_ctl(TASK_QUEUE_CTL_PUSH, task);
+   task_queue_push(task);
 
    return true;
 }
@@ -325,16 +304,16 @@ static bool take_screenshot_raw(const char *name_base, void *userbuf,
    /* Negative pitch is needed as screenshot takes bottom-up,
     * but we use top-down.
     */
-   if (!screenshot_dump(name_base, 
+   if (!screenshot_dump(name_base,
          (const uint8_t*)data + (height - 1) * pitch,
-         width, height, -pitch, false, userbuf, savestate, is_idle, is_paused))
+         width, height, (int)(-pitch), false, userbuf, savestate, is_idle, is_paused))
       return false;
 
    return true;
 }
 
 static bool take_screenshot_choice(const char *name_base, bool savestate,
-      bool is_paused, bool is_idle)
+      bool is_paused, bool is_idle, bool has_valid_framebuffer)
 {
    size_t old_pitch;
    unsigned old_width, old_height;
@@ -342,7 +321,7 @@ static bool take_screenshot_choice(const char *name_base, bool savestate,
    void *frame_data            = NULL;
    const void* old_data        = NULL;
    settings_t *settings        = config_get_ptr();
-   const char *screenshot_dir  = settings->directory.screenshot;
+   const char *screenshot_dir  = settings->paths.directory_screenshot;
 
    /* No way to infer screenshot directory. */
    if (     string_is_empty(screenshot_dir)
@@ -362,7 +341,7 @@ static bool take_screenshot_choice(const char *name_base, bool savestate,
 #endif
    }
 
-   if (!video_driver_cached_frame_has_valid_framebuffer())
+   if (!has_valid_framebuffer)
       return take_screenshot_raw(name_base, NULL, savestate, is_idle, is_paused);
 
    if (!video_driver_supports_read_frame_raw())
@@ -387,7 +366,7 @@ static bool take_screenshot_choice(const char *name_base, bool savestate,
    return ret;
 }
 
-bool take_screenshot(const char *name_base, bool silence)
+bool take_screenshot(const char *name_base, bool silence, bool has_valid_framebuffer)
 {
    bool is_paused         = false;
    bool is_idle           = false;
@@ -397,7 +376,8 @@ bool take_screenshot(const char *name_base, bool silence)
 
    runloop_get_status(&is_paused, &is_idle, &is_slowmotion, &is_perfcnt_enable);
 
-   ret       = take_screenshot_choice(name_base, silence, is_paused, is_idle);
+   ret       = take_screenshot_choice(name_base, silence, is_paused, is_idle,
+         has_valid_framebuffer);
 
    if (is_paused && !is_idle)
          video_driver_cached_frame();
