@@ -26,11 +26,7 @@
 
 #include "ui_companion_driver.h"
 
-
 static const ui_companion_driver_t *ui_companion_drivers[] = {
-#ifdef HAVE_QT_WRAPPER
-   &ui_companion_qt,
-#endif
 #if defined(_WIN32) && !defined(_XBOX)
    &ui_companion_win32,
 #endif
@@ -39,17 +35,19 @@ static const ui_companion_driver_t *ui_companion_drivers[] = {
 #endif
 #ifdef HAVE_COCOATOUCH
    &ui_companion_cocoatouch,
-#endif
-#ifdef HAVE_QT
-   &ui_companion_qt,
 #endif
    &ui_companion_null,
    NULL
 };
 
-static bool main_ui_companion_is_on_foreground;
-static const ui_companion_driver_t *ui_companion;
-static void *ui_companion_data;
+static bool main_ui_companion_is_on_foreground = false;
+static const ui_companion_driver_t *ui_companion = NULL;
+static void *ui_companion_data = NULL;
+
+#ifdef HAVE_QT
+static void *ui_companion_qt_data = NULL;
+static bool qt_is_inited = false;
+#endif
 
 /**
  * ui_companion_find_driver:
@@ -87,16 +85,11 @@ bool ui_companion_is_on_foreground(void)
  *
  * Finds first suitable driver and initialize.
  *
- * Returns: pointer to first suitable driver, otherwise NULL. 
+ * Returns: pointer to first suitable driver, otherwise NULL.
  **/
 const ui_companion_driver_t *ui_companion_init_first(void)
 {
-   unsigned i;
-
-   for (i = 0; ui_companion_drivers[i]; i++)
-      return ui_companion_drivers[i];
-
-   return NULL;
+   return ui_companion_drivers[0];
 }
 
 const ui_companion_driver_t *ui_companion_get_ptr(void)
@@ -115,39 +108,88 @@ void ui_companion_event_command(enum event_command action)
 void ui_companion_driver_deinit(void)
 {
    const ui_companion_driver_t *ui = ui_companion_get_ptr();
+
    if (!ui)
       return;
    if (ui->deinit)
       ui->deinit(ui_companion_data);
+
+#ifdef HAVE_QT
+   if (qt_is_inited)
+   {
+      ui_companion_qt.deinit(ui_companion_qt_data);
+      ui_companion_qt_data = NULL;
+   }
+#endif
    ui_companion_data = NULL;
 }
 
 void ui_companion_driver_init_first(void)
 {
-   settings_t *settings    = config_get_ptr();
+   settings_t *settings = config_get_ptr();
 
    ui_companion = (ui_companion_driver_t*)ui_companion_init_first();
 
-   if (ui_companion && ui_companion->toggle)
+#ifdef HAVE_QT
+   if (settings->bools.desktop_menu_enable && settings->bools.ui_companion_toggle)
    {
-      if (settings->ui.companion_start_on_boot)
-         ui_companion->toggle(ui_companion_data);
+      ui_companion_qt_data = ui_companion_qt.init();
+      qt_is_inited = true;
+   }
+#endif
+
+   if (ui_companion)
+   {
+      if (settings->bools.ui_companion_start_on_boot)
+      {
+         if (ui_companion->init)
+            ui_companion_data = ui_companion->init();
+
+         ui_companion_driver_toggle(false);
+      }
    }
 }
 
-void ui_companion_driver_toggle(void)
+void ui_companion_driver_toggle(bool force)
 {
+#ifdef HAVE_QT
+   settings_t *settings = config_get_ptr();
+#endif
+
    if (ui_companion && ui_companion->toggle)
-      ui_companion->toggle(ui_companion_data);
+      ui_companion->toggle(ui_companion_data, false);
+
+#ifdef HAVE_QT
+   if (settings->bools.desktop_menu_enable)
+   {
+      if ((settings->bools.ui_companion_toggle || force) && !qt_is_inited)
+      {
+         ui_companion_qt_data = ui_companion_qt.init();
+         qt_is_inited = true;
+      }
+
+      if (ui_companion_qt.toggle && qt_is_inited)
+         ui_companion_qt.toggle(ui_companion_qt_data, force);
+   }
+#endif
 }
 
 void ui_companion_driver_notify_refresh(void)
 {
    const ui_companion_driver_t *ui = ui_companion_get_ptr();
+#ifdef HAVE_QT
+   settings_t            *settings = config_get_ptr();
+#endif
+
    if (!ui)
       return;
    if (ui->notify_refresh)
       ui->notify_refresh(ui_companion_data);
+#ifdef HAVE_QT
+   if (settings->bools.desktop_menu_enable)
+      if (ui_companion_qt.notify_refresh && qt_is_inited)
+         ui_companion_qt.notify_refresh(ui_companion_qt_data);
+#endif
 }
 
 void ui_companion_driver_notify_list_loaded(file_list_t *list, file_list_t *menu_list)
@@ -197,6 +239,13 @@ const ui_browser_window_t *ui_companion_driver_get_browser_window_ptr(void)
    return ui->browser_window;
 }
 
+#ifdef HAVE_QT
+const ui_application_t *ui_companion_driver_get_qt_application_ptr(void)
+{
+   return ui_companion_qt.application;
+}
+#endif
+
 const ui_application_t *ui_companion_driver_get_application_ptr(void)
 {
    const ui_companion_driver_t *ui = ui_companion_get_ptr();
@@ -205,6 +254,22 @@ const ui_application_t *ui_companion_driver_get_application_ptr(void)
    return ui->application;
 }
 
+void ui_companion_driver_msg_queue_push(const char *msg, unsigned priority, unsigned duration, bool flush)
+{
+   const ui_companion_driver_t *ui = ui_companion_get_ptr();
+#ifdef HAVE_QT
+   settings_t *settings = config_get_ptr();
+#endif
+
+   if (ui && ui->msg_queue_push)
+      ui->msg_queue_push(ui_companion_data, msg, priority, duration, flush);
+#ifdef HAVE_QT
+   if (settings->bools.desktop_menu_enable)
+      if (ui_companion_qt.msg_queue_push && qt_is_inited)
+         ui_companion_qt.msg_queue_push(ui_companion_qt_data, msg, priority, duration, flush);
+#endif
+}
+
 void *ui_companion_driver_get_main_window(void)
 {
    const ui_companion_driver_t *ui = ui_companion_get_ptr();
@@ -220,3 +285,14 @@ const char *ui_companion_driver_get_ident(void)
       return "null";
    return ui->ident;
 }
+
+void ui_companion_driver_log_msg(const char *msg)
+{
+#ifdef HAVE_QT
+   settings_t *settings = config_get_ptr();
+
+   if (settings->bools.desktop_menu_enable)
+      if (ui_companion_qt_data && qt_is_inited)
+         ui_companion_qt.log_msg(ui_companion_qt_data, msg);
+#endif
+}
