@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
- *  Copyright (C) 2011-2016 - Daniel De Matteis
- * 
+ *  Copyright (C) 2011-2017 - Daniel De Matteis
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -17,8 +17,6 @@
 #ifndef RARCH_VERSION_H__
 #define RARCH_VERSION_H__
 
-#ifndef PACKAGE_VERSION
-#define PACKAGE_VERSION "1.4.1"
-#endif
+#include "version.all"
 
 #endif
