@@ -165,7 +165,7 @@ static bool create_softfilter_graph(rarch_softfilter_t *filt,
 
    userdata.conf = filt->conf;
    /* Index-specific configs take priority over ident-specific. */
-   userdata.prefix[0] = key; 
+   userdata.prefix[0] = key;
    userdata.prefix[1] = filt->impl->short_ident;
 
    /* Simple assumptions. */
@@ -209,7 +209,7 @@ static bool create_softfilter_graph(rarch_softfilter_t *filt,
 
    filt->impl_data = filt->impl->create(
          &softfilter_config, input_fmt, input_fmt, max_width, max_height,
-         threads != RARCH_SOFTFILTER_THREADS_AUTO ? threads : 
+         threads != RARCH_SOFTFILTER_THREADS_AUTO ? threads :
          cpu_features_get_core_amount(), cpu_features,
          &userdata);
    if (!filt->impl_data)
@@ -312,7 +312,7 @@ static bool append_softfilter_plugs(rarch_softfilter_t *filt,
 
       RARCH_LOG("[SoftFilter]: Found plug: %s (%s).\n",
             impl->ident, impl->short_ident);
-      
+
       filt->plugs = new_plugs;
       filt->plugs[filt->num_plugs].lib = lib;
       filt->plugs[filt->num_plugs].impl = impl;
@@ -351,7 +351,7 @@ static bool append_softfilter_plugs(rarch_softfilter_t *filt,
       struct string_list *list)
 {
    unsigned i;
-   softfilter_simd_mask_t mask = cpu_features_get();
+   softfilter_simd_mask_t mask = (softfilter_simd_mask_t)cpu_features_get();
 
    (void)list;
 
@@ -388,7 +388,7 @@ rarch_softfilter_t *rarch_softfilter_new(const char *filter_config,
       enum retro_pixel_format in_pixel_format,
       unsigned max_width, unsigned max_height)
 {
-   softfilter_simd_mask_t cpu_features = cpu_features_get();
+   softfilter_simd_mask_t cpu_features = (softfilter_simd_mask_t)cpu_features_get();
    char basedir[PATH_MAX_LENGTH];
 #ifdef HAVE_DYLIB
    char ext_name[PATH_MAX_LENGTH];
@@ -524,7 +524,7 @@ void rarch_softfilter_process(rarch_softfilter_t *filt,
    if (filt->impl && filt->impl->get_work_packets)
       filt->impl->get_work_packets(filt->impl_data, filt->packets,
             output, output_stride, input, width, height, input_stride);
-   
+
 #ifdef HAVE_THREADS
    /* Fire off workers */
    for (i = 0; i < filt->threads; i++)
