@@ -68,11 +68,10 @@ static void *supereagle_generic_create(const struct softfilter_config *config,
       unsigned max_width, unsigned max_height,
       unsigned threads, softfilter_simd_mask_t simd, void *userdata)
 {
+   struct filter_data *filt = (struct filter_data*)calloc(1, sizeof(*filt));
    (void)simd;
    (void)config;
    (void)userdata;
-
-   struct filter_data *filt = (struct filter_data*)calloc(1, sizeof(*filt));
    if (!filt)
       return NULL;
    filt->workers = (struct softfilter_thread_data*)calloc(threads, sizeof(struct softfilter_thread_data));
@@ -216,7 +215,7 @@ static void supereagle_generic_destroy(void *data)
 #endif
 
 static void supereagle_generic_xrgb8888(unsigned width, unsigned height,
-      int first, int last, uint32_t *src, 
+      int first, int last, uint32_t *src,
       unsigned src_stride, uint32_t *dst, unsigned dst_stride)
 {
    unsigned finish;
@@ -240,7 +239,7 @@ static void supereagle_generic_xrgb8888(unsigned width, unsigned height,
 }
 
 static void supereagle_generic_rgb565(unsigned width, unsigned height,
-      int first, int last, uint16_t *src, 
+      int first, int last, uint16_t *src,
       unsigned src_stride, uint16_t *dst, unsigned dst_stride)
 {
    unsigned finish;
@@ -272,7 +271,10 @@ static void supereagle_work_cb_rgb565(void *data, void *thread_data)
    unsigned height = thr->height;
 
    supereagle_generic_rgb565(width, height,
-         thr->first, thr->last, input, thr->in_pitch / SOFTFILTER_BPP_RGB565, output, thr->out_pitch / SOFTFILTER_BPP_RGB565);
+         thr->first, thr->last, input,
+            (unsigned)(thr->in_pitch / SOFTFILTER_BPP_RGB565),
+            output,
+            (unsigned)(thr->out_pitch / SOFTFILTER_BPP_RGB565));
 }
 
 static void supereagle_work_cb_xrgb8888(void *data, void *thread_data)
@@ -284,7 +286,10 @@ static void supereagle_work_cb_xrgb8888(void *data, void *thread_data)
    unsigned height = thr->height;
 
    supereagle_generic_xrgb8888(width, height,
-         thr->first, thr->last, input, thr->in_pitch / SOFTFILTER_BPP_XRGB8888, output, thr->out_pitch / SOFTFILTER_BPP_XRGB8888);
+         thr->first, thr->last, input,
+        (unsigned)(thr->in_pitch / SOFTFILTER_BPP_XRGB8888),
+        output,
+        (unsigned)(thr->out_pitch / SOFTFILTER_BPP_XRGB8888));
 }
 
 static void supereagle_generic_packets(void *data,
