@@ -1,7 +1,7 @@
 /*  RetroArch - A frontend for libretro.
  *  Copyright (C) 2010-2014 - Hans-Kristian Arntzen
  *  Copyright (C) 2011-2017 - Daniel De Matteis
- * 
+ *
  *  RetroArch is free software: you can redistribute it and/or modify it under the terms
  *  of the GNU General Public License as published by the Free Software Found-
  *  ation, either version 3 of the License, or (at your option) any later version.
@@ -21,13 +21,12 @@
 #include <compat/strl.h>
 #include <features/features_cpu.h>
 #include <rthreads/rthreads.h>
+#include <string/stdstring.h>
 
 #include "video_thread_wrapper.h"
 #include "font_driver.h"
-#include "video_shader_driver.h"
 
-#include "../performance_counters.h"
-#include "../runloop.h"
+#include "../retroarch.h"
 #include "../verbosity.h"
 
 enum thread_cmd
@@ -122,7 +121,7 @@ struct thread_packet
 
       struct
       {
-         char msg[255];
+         char msg[128];
          struct font_params params;
       } osd_message;
 
@@ -167,7 +166,6 @@ struct thread_video
    const input_driver_t **input;
    void **input_data;
 
-#if defined(HAVE_MENU)
    struct
    {
       void *frame;
@@ -180,7 +178,6 @@ struct thread_video
       bool enable;
       bool full_screen;
    } texture;
-#endif
    bool apply_state_changes;
 
    bool alive;
@@ -343,7 +340,7 @@ static bool video_thread_handle_packet(
       case CMD_INIT:
          thr->driver_data = thr->driver->init(&thr->info,
                thr->input, thr->input_data);
-         pkt.data.b = thr->driver_data;
+         pkt.data.b = (thr->driver_data != NULL);
          thr->driver->viewport_info(thr->driver_data, &thr->vp);
          video_thread_reply(thr, &pkt);
          break;
@@ -377,7 +374,7 @@ static bool video_thread_handle_packet(
 
          thr->driver->viewport_info(thr->driver_data, &vp);
 
-         if (memcmp(&vp, &thr->read_vp, sizeof(vp)) == 0)
+         if (string_is_equal_fast(&vp, &thr->read_vp, sizeof(vp)))
          {
             /* We can read safely
              *
@@ -523,16 +520,22 @@ static bool video_thread_handle_packet(
          break;
 
       case CMD_POKE_SET_ASPECT_RATIO:
-         thr->poke->set_aspect_ratio(thr->driver_data,
-               pkt.data.i);
+         if (thr->poke && thr->poke->set_aspect_ratio)
+            thr->poke->set_aspect_ratio(thr->driver_data,
+                  pkt.data.i);
          video_thread_reply(thr, &pkt);
          break;
 
       case CMD_POKE_SET_OSD_MSG:
-         if (thr->poke && thr->poke->set_osd_msg)
-            thr->poke->set_osd_msg(thr->driver_data,
-                  pkt.data.osd_message.msg,
-                  &pkt.data.osd_message.params, NULL);
+         {
+            video_frame_info_t video_info;
+            video_driver_build_info(&video_info);
+            if (thr->poke && thr->poke->set_osd_msg)
+               thr->poke->set_osd_msg(thr->driver_data,
+                     &video_info,
+                     pkt.data.osd_message.msg,
+                     &pkt.data.osd_message.params, NULL);
+         }
          video_thread_reply(thr, &pkt);
          break;
 
@@ -585,7 +588,7 @@ static void video_thread_loop(void *data)
       if (thr->frame.updated)
          updated = true;
 
-      /* To avoid race condition where send_cmd is updated 
+      /* To avoid race condition where send_cmd is updated
        * right after the switch is checked. */
       pkt = thr->cmd_data;
 
@@ -656,7 +659,7 @@ static bool video_thread_alive(void *data)
    bool ret;
    thread_video_t *thr = (thread_video_t*)data;
 
-   if (runloop_ctl(RUNLOOP_CTL_IS_PAUSED, NULL))
+   if (rarch_ctl(RARCH_CTL_IS_PAUSED, NULL))
    {
       thread_packet_t pkt = { CMD_ALIVE };
 
@@ -712,12 +715,11 @@ static bool video_thread_frame(void *data, const void *frame_,
       unsigned pitch, const char *msg, video_frame_info_t *video_info)
 {
    unsigned copy_stride;
-   static struct retro_perf_counter thr_frame = {0};
    const uint8_t *src                  = NULL;
    uint8_t *dst                        = NULL;
    thread_video_t *thr                 = (thread_video_t*)data;
 
-   /* If called from within read_viewport, we're actually in the 
+   /* If called from within read_viewport, we're actually in the
     * driver thread, so just render directly. */
    if (thr->frame.within_thread)
    {
@@ -729,10 +731,7 @@ static bool video_thread_frame(void *data, const void *frame_,
       return false;
    }
 
-   performance_counter_init(thr_frame, "thr_frame");
-   performance_counter_start_plus(video_info->is_perfcnt_enable, thr_frame);
-
-   copy_stride = width * (thr->info.rgb32 
+   copy_stride = width * (thr->info.rgb32
          ? sizeof(uint32_t) : sizeof(uint16_t));
 
    src = (const uint8_t*)frame_;
@@ -761,7 +760,7 @@ static bool video_thread_frame(void *data, const void *frame_,
       }
    }
 
-   /* Drop frame if updated flag is still set, as thread is 
+   /* Drop frame if updated flag is still set, as thread is
     * still working on last frame. */
    if (!thr->frame.updated)
    {
@@ -799,8 +798,6 @@ static bool video_thread_frame(void *data, const void *frame_,
 
    slock_unlock(thr->lock);
 
-   performance_counter_stop_plus(video_info->is_perfcnt_enable, thr_frame);
-
    thr->last_time = cpu_features_get_time_usec();
    return true;
 }
@@ -876,13 +873,13 @@ static void video_thread_set_viewport(void *data, unsigned width,
    thread_video_t *thr = (thread_video_t*)data;
    if (!thr)
       return;
-    
+
    slock_lock(thr->lock);
 
     if (thr->driver && thr->driver->set_viewport)
         thr->driver->set_viewport(thr->driver_data, width, height,
                                   force_full, allow_rotate);
-    
+
    slock_unlock(thr->lock);
 }
 
@@ -899,10 +896,10 @@ static void video_thread_set_rotation(void *data, unsigned rotation)
    video_thread_send_and_wait_user_to_thread(thr, &pkt);
 }
 
-/* This value is set async as stalling on the video driver for 
+/* This value is set async as stalling on the video driver for
  * every query is too slow.
  *
- * This means this value might not be correct, so viewport 
+ * This means this value might not be correct, so viewport
  * reads are not supported for now. */
 static void video_thread_viewport_info(void *data, struct video_viewport *vp)
 {
@@ -984,7 +981,7 @@ static bool thread_overlay_load(void *data,
 {
    thread_video_t *thr = (thread_video_t*)data;
    thread_packet_t pkt = { CMD_OVERLAY_LOAD };
-   const struct texture_image *images = 
+   const struct texture_image *images =
       (const struct texture_image*)image_data;
 
    if (!thr)
@@ -1155,7 +1152,6 @@ static void thread_set_aspect_ratio(void *data, unsigned aspectratio_idx)
    video_thread_send_and_wait_user_to_thread(thr, &pkt);
 }
 
-#if defined(HAVE_MENU)
 static void thread_set_texture_frame(void *data, const void *frame,
       bool rgb32, unsigned width, unsigned height, float alpha)
 {
@@ -1163,7 +1159,7 @@ static void thread_set_texture_frame(void *data, const void *frame,
    thread_video_t *thr = (thread_video_t*)data;
 
    slock_lock(thr->frame.lock);
-   required            = width * height * 
+   required            = width * height *
       (rgb32 ? sizeof(uint32_t) : sizeof(uint16_t));
 
    if (required > thr->texture.frame_cap)
@@ -1196,7 +1192,9 @@ static void thread_set_texture_enable(void *data, bool state, bool full_screen)
    slock_unlock(thr->frame.lock);
 }
 
-static void thread_set_osd_msg(void *data, const char *msg,
+static void thread_set_osd_msg(void *data,
+      video_frame_info_t *video_info,
+      const char *msg,
       const void *params, void *font)
 {
    thread_video_t *thr = (thread_video_t*)data;
@@ -1207,9 +1205,8 @@ static void thread_set_osd_msg(void *data, const char *msg,
    /* TODO : find a way to determine if the calling
     * thread is the driver thread or not. */
    if (thr->poke && thr->poke->set_osd_msg)
-      thr->poke->set_osd_msg(thr->driver_data, msg, params, font);
+      thr->poke->set_osd_msg(thr->driver_data, video_info, msg, params, font);
 }
-#endif
 
 static uintptr_t thread_load_texture(void *video_data, void *data,
       bool threaded, enum texture_filter_type filter_type)
@@ -1245,7 +1242,7 @@ static void thread_apply_state_changes(void *data)
    slock_unlock(thr->frame.lock);
 }
 
-/* This is read-only state which should not 
+/* This is read-only state which should not
  * have any kind of race condition. */
 static struct video_shader *thread_get_current_shader(void *data)
 {
@@ -1255,10 +1252,22 @@ static struct video_shader *thread_get_current_shader(void *data)
    return thr->poke->get_current_shader(thr->driver_data);
 }
 
+static uint32_t thread_get_flags(void *data)
+{
+   thread_video_t *thr = (thread_video_t*)data;
+   if (!thr || !thr->poke || !thr->poke->get_flags)
+      return 0;
+   return thr->poke->get_flags(thr->driver_data);
+}
+
 static const video_poke_interface_t thread_poke = {
+   thread_get_flags,
+   NULL,                            /* set_coords */
+   NULL,                            /* set_mvp */
    thread_load_texture,
    thread_unload_texture,
    thread_set_video_mode,
+   NULL,
    thread_set_filtering,
    thread_get_video_output_size,
    thread_get_video_output_prev,
@@ -1267,19 +1276,16 @@ static const video_poke_interface_t thread_poke = {
    NULL, /* get_proc_address */
    thread_set_aspect_ratio,
    thread_apply_state_changes,
-#if defined(HAVE_MENU)
    thread_set_texture_frame,
    thread_set_texture_enable,
    thread_set_osd_msg,
-#else
-   NULL,
-   NULL,
-#endif
 
    NULL,
    NULL,
 
    thread_get_current_shader,
+   NULL,                      /* get_current_software_framebuffer */
+   NULL                       /* get_hw_render_interface */
 };
 
 static void video_thread_get_poke_interface(
@@ -1349,7 +1355,7 @@ static void video_thread_set_callbacks(
  * @out_driver                : Output video driver
  * @out_data                  : Output video data
  * @input                     : Input input driver
- * @input_data                : Input input data 
+ * @input_data                : Input input data
  * @driver                    : Input Video driver
  * @info                      : Video info handle.
  *
@@ -1378,7 +1384,7 @@ bool video_init_thread(const video_driver_t **out_driver,
  * video_thread_get_ptr:
  * @drv                       : Found driver.
  *
- * Gets the underlying video driver associated with the 
+ * Gets the underlying video driver associated with the
  * threaded video wrapper. Sets @drv to the found
  * video driver.
  *
