@@ -29,7 +29,11 @@ enum filebrowser_enums
    FILEBROWSER_NONE              = 0,
    FILEBROWSER_SELECT_DIR,
    FILEBROWSER_SCAN_DIR,
+   FILEBROWSER_SCAN_FILE,
    FILEBROWSER_SELECT_FILE,
+   FILEBROWSER_SELECT_FILE_SUBSYSTEM,
+   FILEBROWSER_SELECT_IMAGE,
+   FILEBROWSER_SELECT_FONT,
    FILEBROWSER_SELECT_COLLECTION
 };
 
